//-----------------------------------------------------------------------------
// File: GameExample.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "GameExample.h"

#include "../Tools/GUI/ThemeWindows.h"
#include "../Tools/GUI/PlatformWindows.h"
#include "../Tools/GUI/video/video.h"

#ifdef SQUALL_SOUND
#include "../Tools/GUI/sound/squall/SoundSquall.h"
#endif

#ifdef BASS_SOUND
#include "../Tools/GUI/sound/bass/SoundBass.h"
#endif

extern GMemWriter wr2;
extern GMemReader rd2;

GGameExample* theGame=NULL;

unsigned int m_color[8]={0x904000,0x6000,0x9634b6,0x8628f,0x1818c1,0x606000,0xface1,0x808080};

CMagicString session_name="";

GGameExample::GGameExample(GUID_GAME guid, const char* name, const char* version, bool fullscreen_mode) : GGame(guid, name, version, fullscreen_mode)
{
	theGame=this;
	font1=-1;
	show_fps=true;
	net_player=-1;
}

GGameExample::~GGameExample(void)
{
	theGame=NULL;
}

//     
void GGameExample::SysOnRender()
{
	bool is=IsActivated();
	if (is)
	{
		GGame::SysOnRender();

		if (backbuffer)
		{
			backbuffer->ResetChange(true);
			if (show_fps)
			{
				static int fps=0;
				static int fps_counter=0;
				static unsigned long old_time=0;

				static unsigned long last_time=0;
				double rate=0.01;
				unsigned long new_time=GetTickCount();

				unsigned long rtime=new_time-old_time;
				if (rtime>1000)
				{
					// eng: more than 1 second passed
					// rus:   
					double percent=((double)rtime)/10.f;
					fps=(int)(percent*fps_counter/100);
					fps_counter=0;
					old_time=new_time;
				}

				fps_counter++;

				CMagicString fps_str="FPS: ";
				CMagicString d;
				d.IntToString(fps);
				fps_str+=d;
				fps_str+="  ";

				backbuffer->TextOut(10,10, fps_str, font1, 0xffffff, 0);
			}

			if (backbuffer->IsChanged())
			{
				//  ,   
				if (device->IsLost())
				{
					VIDEO_MODE vm;
					device->GetMonitorMode(&vm);
					device->Create(vm.width, vm.height);
				}
				SURFACE_LOCKED locked;
				device->LockBackbuffer(&locked);
				int wi, he;
				device->GetSize(wi, he);
				if (locked.pitch>=wi*4)
					backbuffer->CopyToTexture32(locked.ptr, locked.pitch);
				else
					backbuffer->CopyToTexture(locked.ptr, locked.pitch);

				if (device->IsVisibleCursor())
				{
					GImage* cur=GetCursor();
					if (cur)
					{
						//      
						GSurface* cur_surface=cur->GetSurface();
						backbuffer->SetOffset(0, 0);
						backbuffer->ResetClip();
						backbuffer->SetPtrAndPitch(locked.ptr, locked.pitch);
						backbuffer->Blt(cur_surface, cursor_x, cursor_y, 0, 0, 0, 0, true);
						backbuffer->Unlock();
					}
				}

				backbuffer->ResetChange();

				device->UnlockBackbuffer();
				device->Render();
			}
		}
	}
}

//  
void GGameExample::DrawCursor(GImage* cur)
{
	cur->Blt(cursor_x, cursor_y, backbuffer, true);
}

CMagicString global_server_file_name="ServerExample";
const char* GGameExample::GetServerFileName()
{
	return global_server_file_name;
}


//  
GPlatform* GGameExample::NewPlatform(GGame* owner, const char* game_name)
{
	return new GPlatformWindows(owner, game_name);
}

//  
GSound* GGameExample::NewSound()
{
	#ifdef SQUALL_SOUND
	return new GSoundSquall();
	#endif

	#ifdef BASS_SOUND
	return new GSoundBass();
	#endif
}

//   
GNet* GGameExample::NewNet()
{
	extern GNet* NewNetClientLibUV();
	return NewNetClientLibUV();
}

//   
GNetServer* GGameExample::NewServer()
{
	extern GNet* NewNetServerLibUV();
	return (GNetServer*)NewNetServerLibUV();
}

//    
GNetClient* GGameExample::NewDownloader()
{
	extern GNetClient* NewNetDownloaderLibUV();
	return NewNetDownloaderLibUV();
}

//  ,    
bool GGameExample::CreateGame(void* init, int view_width, int view_height)
{
	if (GGame::CreateGame(init, view_width, view_height))
	{
		SetRndIndex(platform->GetTick());
		for (int i=0;i<16;i++)
			license+=(char)GetRndInt(33, 127);

		platform->SetDirectory(platform->GetPathToGame());

		theme=new GThemeWindows(this);
		theme->Create();

		device->RefreshVideoMode();

		//video_mode_menu.width=1920;
		//video_mode_menu.height=1080;
		//VIDEO_MODE vm;
		//device->GetMonitorMode(&vm);
		//video_mode_menu.width=vm.width;
		//video_mode_menu.height=vm.height;
		video_mode_menu.width=1024;
		video_mode_menu.height=768;
		video_mode_menu.bpp=32;

		device->ChangeMonitorResolution(false);

		if (device->SetVideoMode(&video_mode_menu))
		{
			CMagicString path_cfg=platform->GetPathToConfig();
			CMagicString path_lang=path_cfg+"language.ctl";

			CMagicString lang_all="ENGLISH";
			FILE* file_lang=platform->OpenFile(path_lang,"rb");
			if (file_lang)
			{
				//     
				lang_all.LoadFromStream(file_lang);
				fclose(file_lang);
			}

			Localizer& loc=Localizer::GetInstance();
			loc.Load("*.lng",lang_all.c_str());

			font1=CreateFontex("arial15");
			
			//      
			GScene* scene;
			scene=new GSceneExample();
			AddScene(scene);

			playing_channel=-1;
			sound_sample=-1;

			FILE_DATA fd;
			CMagicString sound_path=platform->GetPathToSound();
			sound_path+="cristmas_song.mp3";

			if (platform->LoadFile(sound_path.c_str(), &fd))
				sound_sample=sound->LoadFromMemory(fd);

			#ifndef MAX_FPS
			SetMainTimer(17); // 60 
			#endif

			return true;
		}
	}
	return false;
}

//     
bool GGameExample::CanStartSession()
{
	bool is=true;
	int count=0;
	//  ,     
	for (int i=1;i<MAX_PLAYER;i++)
	{
		PlayerCfg* cfg=&(players[i]);
		if (cfg->type==PLAYER_MAN || cfg->type==PLAYER_COMPUTER)
		{
			count++;
			if (!cfg->ready)
			{
				//   
				is=false;
				break;
			}
		}
	}

	return (is && count>0);
}

//      
bool GGameExample::CanJoinSession()
{
	//  ,     
	for (int i=0;i<MAX_PLAYER;i++)
	{
		PlayerCfg* cfg=&(players[i]);
		if (cfg->type==PLAYER_OPENED)
		{
			return true;
		}
	}
	return false;
}


//     
//                  
void GGameExample::OnEnumSession(GSessionList* sessions, int count_general_session)
{
	GSceneExample* scene=(GSceneExample*)GetScene();
	scene->EnumSession(sessions, count_general_session);
}

//    
void GGameExample::OnCreateSession(int index_player)
{
	net_player=index_player;

	GSceneExample* scene=(GSceneExample*)GetScene();
	scene->config_session=true;

	GDialogNet* dlg=((GDialogNet*)scene->Find(COMPONENT_NET_DIALOG));
	GListBox* list=(GListBox*)dlg->Find(COMPONENT_NET_LIST);
	list->ClearItems();

	dlg->Find(COMPONENT_NET_CREATE)->Show(false);
	dlg->Find(COMPONENT_NET_READY)->Show(false);
	dlg->Find(COMPONENT_NET_START)->Show(true);
	dlg->Find(COMPONENT_NET_JOIN)->Show(false);
	dlg->Find(COMPONENT_NET_PICTURE)->Show(true);
}

//      (   ,   false)
bool GGameExample::OnJoinSession(int index_player, const char* map_name, unsigned int map_id)
{
	net_player=index_player;

	GSceneExample* scene=(GSceneExample*)GetScene();
	scene->config_session=true;

	GDialogNet* dlg=((GDialogNet*)scene->Find(COMPONENT_NET_DIALOG));
	GListBox* list=(GListBox*)dlg->Find(COMPONENT_NET_LIST);
	list->ClearItems();
	
	dlg->Find(COMPONENT_NET_CREATE)->Show(false);
	dlg->Find(COMPONENT_NET_READY)->Show(true);
	dlg->Find(COMPONENT_NET_START)->Show(false);
	dlg->Find(COMPONENT_NET_JOIN)->Show(false);
	dlg->Find(COMPONENT_NET_PICTURE)->Show(true);

	return true;
}

//  -
void GGameExample::OnChatMessage(CMagicString& sender, CMagicString& receiver, bool privated, bool global, CMagicString& chat, bool printed_message)
{
	GSceneExample* scene=(GSceneExample*)GetScene();
	GListBoxNet* list=scene->GetListSession();
	CMagicString mes;
	CMagicString name=GetUserName();
	if (mes!=name)
	{
		mes=sender;
		mes+=": ";
	}
	mes+=chat;
	list->AddString(chat);
}

//       (5,4,3,2,1...   OnStartNetGame(),    )
void GGameExample::OnStartNetCounter(int counter)
{
	GSceneExample* scene=(GSceneExample*)GetScene();
	GComponent* dlg=scene->Find(COMPONENT_NET_DIALOG);
	dlg->Find(COMPONENT_NET_START)->Enable(false);
	dlg->Find(COMPONENT_NET_READY)->Enable(false);

	CMagicString d;
	d.IntToString(counter);
	d+="...";

	CMagicString sender, receiver;
	OnChatMessage(sender, receiver, false, false, d);
}

//   ,          
bool GGameExample::IsNetGameLoaded()
{
	return true;
}

//     
void GGameExample::OnLaunchNetGame()
{
	GSceneExample* scene=(GSceneExample*)GetScene();
	scene->mbox_launch.Alert(" GMessageBox", "        , ,  ,   ,      ,   .  ,    ,  : '<color=0x8000><url=\"http://onimod.land\"> </url><default>'.", GSMB_OK);
}


//     ,       
bool GGameExample::GetSessionInfoStruct(NET_SESSION_INFO* si)
{
	int i;

	//     
	si->tick=0;
	si->ip="";

	//     
	si->name=session_name;
	si->id_map=1;
	si->is_password=false;

	si->ver=game_digital_version;

	int k_player=0;
	for (i=0;i<MAX_PLAYER;i++)
	{
		PlayerCfg* cfg=&(players[i]);
		if (cfg->type!=PLAYER_DISABLED)
			k_player++;
		else
			break;
	}

	if (si->k_player!=k_player)
	{
		//   ,     
		for (i=0;i<si->k_player;i++)
			delete si->m_player[i];

		if (si->m_player)
		{
			free(si->m_player);
			si->m_player=NULL;
		}

		si->k_player=k_player;
		if (si->k_player)
		{
			si->m_player=(NET_PLAYER**)malloc(si->k_player*sizeof(NET_PLAYER*));
		}

		for (i=0;i<si->k_player;i++)
			si->m_player[i]=new NET_PLAYER(si->net);
	}

	for (i=0;i<si->k_player;i++)
		GetSessionPlayerStruct(i, si->m_player[i]);

	return true;
}

bool GGameExample::GetSessionPlayerStruct(int index, NET_PLAYER* np)
{
	if (index>=0 && index<MAX_PLAYER)
	{
		PlayerCfg* cfg=&(players[index]);
		np->type=cfg->type;
		np->name=cfg->name;
		np->player_id=cfg->player_id;

		wr2.Start();
		cfg->Serialize(wr2);
		MEM_DATA buf;
		wr2.Finish(buf);

		if (np->length_info!=buf.length)
		{
			//  .  ,   
			np->length_info=buf.length;
			if (np->info)
			{
				free(np->info);
				np->info=NULL;
			}
			if (np->length_info)
			{
				np->info=(char*)malloc(np->length_info);
			}
		}

		if (np->length_info)
		{
			char* data=(char*)buf.data;
			memcpy(np->info, data, np->length_info);
		}

		return true;
	}
	return false;
}

//       ,    
void GGameExample::SetSessionInfoStruct(NET_SESSION_INFO* si)
{
	for (int i=0;i<si->k_player;i++)
		SetSessionPlayerStruct(i, si->m_player[i]);
}

void GGameExample::SetSessionPlayerStruct(int index, NET_PLAYER* np)
{
	if (index>=0 && index<MAX_PLAYER)
	{
		PlayerCfg* cfg=&(players[index]);
		rd2.Start(np->info);
		cfg->Serialize(rd2);
		cfg->type=np->type;
		cfg->name=np->name;
		cfg->player_id=np->player_id;

		GSceneExample* scene=(GSceneExample*)GetScene();
		scene->RefreshPlayerInfo(index);
	}
}


GSceneExample::GSceneExample()
{
	k_session=0;
	m_session=NULL;
	config_session=false;
	
	GPlatform* platform=game->platform;
	GDevice* device=game->device;

	int bx=10;
	int by=50;

	int step_y=10;
	int step_y2=step_y/4;
	int step_new_y=step_y+10;

	// GCheckBox
	int y=by;
	GCheckBox* check=new GCheckBox;
	check->Create(this, GS_VISIBLE, bx, y, 1001, " GCheckBox");
	int wi, he;
	check->GetSize(wi, he);

	y=by+he+step_y;
	check=new GCheckBox;
	check->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 1002, " GCheckBox  ");
	check->GetSize(wi, he);

	// GStatic
	y+=he+step_y+step_new_y;
	GStatic* st=new GStatic;
	st->Create(this, GS_VISIBLE, bx, y, 0, 0, 1003, " GStatic");
	st->GetSize(wi, he);

	y+=he+step_y;
	st=new GStatic;
	st->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 0, 0, 1004, " GStatic  ");
	st->GetSize(wi, he);

	y+=he+step_y;
	st=new GStatic;
	st->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 200, 0, 1005, " GStatic  ,     200. , ,    ,    . ");
	st->GetSize(wi, he);

	y+=he+step_y;
	st=new GStatic;
	st->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 200, 75, 1006, " GStatic  ,     200,   75. , ,    ,   .");
	st->GetSize(wi, he);

	y+=he+step_y;
	st=new GStatic;
	st->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 200, 0, 1007, " GStatic  ,     200.    (  <color=0xff><color=0xff00ff><color=0xb000><color=0xff0000><default>  <font=1><default>).");
	st->GetSize(wi, he);

	y+=he+step_y;
	st=new GStatic;
	CMagicString url=" GStatic       ";
	url+="<color=0x8000><url=\"";
	url+="http://astralax.ru/titles";
	url+="\">Magic Particles";
	url+="</url>";
	url+="<default>.";
	st->Create(this, GS_VISIBLE, bx, y, 200, 0, 1008, url);
	st->GetSize(wi, he);


	// GEdit
	y+=he+step_y+step_new_y;
	GEdit* edit=new GEdit;
	edit->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 200, 22, 1008, " GEdit -    ");
	edit->GetSize(wi, he);

	y+=he+step_y;
	edit=new GEdit;
	edit->Create(this, GS_VISIBLE | GS_BORDER | GS_DISABLED, bx, y, 200, 22, 1009, "  GEdit");
	edit->GetSize(wi, he);


	// GPicture
	y+=he+step_y+step_new_y;
	st=new GStatic;
	st->Create(this, GS_VISIBLE, bx, y, 200, 0, 1010, " GPicture     :");
	st->GetSize(wi, he);


	FILE_DATA fd;
	CMagicString image_path=platform->GetPathToImage();
	if (platform->LoadFile(image_path+"picture1.jpg", &fd))
	{
		GSurface* surface=device->AddSurface();
		surface->Load(fd.data, fd.length);

		int pwi, phe;
		surface->GetSize(pwi, phe);

		y+=he+step_y2;
		GPicture* pict=new GPictureExample;
	
		pict->Create(this, GS_VISIBLE, bx+40, y, pwi, phe, 1011);
		pict->AddImage(surface);

		if (platform->LoadFile(image_path+"picture2.jpg", &fd))
		{
			surface=device->AddSurface();
			surface->Load(fd.data, fd.length);
			pict->AddImage(surface);
		}

		if (platform->LoadFile(image_path+"picture3.jpg", &fd))
		{
			surface=device->AddSurface();
			surface->Load(fd.data, fd.length);
			pict->AddImage(surface);
		}
	}

	// *****************
	// *   *
	// *****************
	bx=230;

	// GComboBox
	y=by;

	GComboBox* combobox=new GComboBox;
	combobox->Create(this, GS_VISIBLE | GS_BORDER | GSCB_DROPDOWN, bx, y, 280, 23, 1012, 70);
	combobox->SetWindowText("   GComboBox");
	combobox->AddString("   ");
	combobox->AddString("  ");
	combobox->AddString(",   ...");
	combobox->AddString(" ");
	combobox->AddString(" ");
	combobox->AddString(" ");
	combobox->GetSize(wi, he);

	y+=he+step_y;
	combobox=new GComboBox;
	combobox->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 280, 23, 1013, 120);
	combobox->SetWindowText("  GComboBox");
	combobox->AddString("   ");
	combobox->AddString("  ");
	combobox->AddString(",   ...");
	combobox->AddString(" ");
	combobox->AddString(" ");
	combobox->AddString(" ");
	combobox->GetSize(wi, he);

	// GButton  GButtonImage
	y+=he+step_y+step_new_y;
	GButton* button=new GButton;
	button->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 100, 26, 1014, " GButton");
	button->GetSize(wi, he);

	button=new GButton;
	button->Create(this, GS_VISIBLE | GS_BORDER | GS_DISABLED, bx+wi+10, y, 170, 26, 1015, "  GButton");
	button->GetSize(wi, he);

	if (platform->LoadFile(image_path+"button1.jpg", &fd))
	{
		GSurface* butpict1=device->AddSurface();
		butpict1->Load(fd.data, fd.length);

		int pwi, phe;
		butpict1->GetSize(pwi, phe);

		GSurface* butpict2=NULL;
		if (platform->LoadFile(image_path+"button2.jpg", &fd))
		{
			butpict2=device->AddSurface();
			butpict2->Load(fd.data, fd.length);
		}

		GSurface* butpict3=NULL;
		if (platform->LoadFile(image_path+"button3.jpg", &fd))
		{
			butpict3=device->AddSurface();
			butpict3->Load(fd.data, fd.length);
		}

		GSurface* butpict4=NULL;
		if (platform->LoadFile(image_path+"button4.jpg", &fd))
		{
			butpict4=device->AddSurface();
			butpict4->Load(fd.data, fd.length);
		}

		y+=he+step_y;
		GButtonImage* buttonimage=new GButtonImage;
		buttonimage->Create(this, GS_VISIBLE, bx, y, pwi, phe, 1016, butpict1, butpict2, butpict3, butpict4);
		buttonimage->GetSize(wi, he);

		st=new GStatic;
		st->Create(this, GS_VISIBLE, bx, y, 200, 0, 1017, "-  GButtonImage");
		int swi, she;
		st->GetSize(swi, she);
		st->SetPosition(bx+wi+3, y+(he-she)/2);
	}

	// GListBox
	y+=he+step_y+step_new_y;
	int y2=y;

	st=new GStatic;
	st->Create(this, GS_VISIBLE, bx, y, 200, 0, 1010, " GListBox:");
	st->GetSize(wi, he);

	y+=he+step_y2;
	GListBox* listbox=new GListBox;
	listbox->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 150, 150, 1018, NULL);
	listbox->GetSize(wi, he);
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->GetSize(wi, he);

	int bx2=bx+wi+16;

	st=new GStatic;
	st->Create(this, GS_VISIBLE, bx2, y2, 200, 0, 1010, "  GListBox:");
	st->GetSize(wi, he);

	y2+=he+step_y2;

	if (platform->LoadFile(image_path+"ico1.png", &fd))
	{
		GSurface* ico1=device->AddSurface();
		ico1->Load(fd.data, fd.length);

		int pwi, phe;
		ico1->GetSize(pwi, phe);

		icons.Create(pwi, phe);
		icons.AddImage(ico1);

		if (platform->LoadFile(image_path+"ico2.png", &fd))
		{
			GSurface* ico2=device->AddSurface();
			ico2->Load(fd.data, fd.length);
			icons.AddImage(ico2);
		}

		if (platform->LoadFile(image_path+"ico3.png", &fd))
		{
			GSurface* ico3=device->AddSurface();
			ico3->Load(fd.data, fd.length);
			icons.AddImage(ico3);
		}
	}

	listbox=new GListBox;
	listbox->Create(this, GS_VISIBLE | GS_BORDER, bx2, y2, 150, 150, 1018, &icons);
	listbox->GetSize(wi, he);
	GItemText* item=(GItemText*)listbox->AddString(" ");
	item->SetIconIndex(0);
	item=(GItemText*)listbox->AddString(" ", -1, 0xff);
	item->SetIconIndex(1);
	listbox->AddString(" ");
	item=(GItemText*)listbox->AddString(" ", -1, 0xff0000);
	item->SetIconIndex(0);
	listbox->AddString(" ", 1);
	item=(GItemText*)listbox->AddString(" ", 1, 0x8000);
	item->SetIconIndex(1);
	listbox->AddString(" ");
	listbox->AddString(" ");
	listbox->AddString(" ", 1);
	item=(GItemText*)listbox->AddString(" ");
	item->SetIconIndex(2);
	

	

	// GTree
	y+=he+step_y+step_new_y;
	y2=y;

	st=new GStatic;
	st->Create(this, GS_VISIBLE, bx, y, 200, 0, 1019, " GTree:");
	st->GetSize(wi, he);

	y+=he+step_y2;
	GTree* tree=new GTree;
	tree->Create(this, GS_VISIBLE | GS_BORDER | GSTR_HASBUTTONS | GSTR_HASLINES, bx, y, 150, 250, 1020, &icons);
	tree->GetSize(wi, he);

	GItemTree* parent;
	GItemTree* parent2;
	parent=tree->AddNode(NULL, " ", -1, COLOR_KEY_EMPTY, 0);
	tree->AddNode(parent, "   ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent, "   ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent, "   ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent, "   ", -1, COLOR_KEY_EMPTY, 1);
	parent2=tree->AddNode(parent, "   ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent2, "     ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent2, "     ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent2, "     ", -1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(NULL, " ", -1, COLOR_KEY_EMPTY, 0);
	tree->AddNode(NULL, " ", -1, COLOR_KEY_EMPTY, 0);
	tree->AddNode(NULL, " ", -1, COLOR_KEY_EMPTY, 0);
	parent=tree->AddNode(NULL, " ", 1, COLOR_KEY_EMPTY, 0);
	tree->AddNode(parent, "   ", 1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent, "   ", 1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent, "   ", 1, COLOR_KEY_EMPTY, 1);
	tree->AddNode(parent, "   ", 1, COLOR_KEY_EMPTY, 1);
	

	st=new GStatic;
	st->Create(this, GS_VISIBLE, bx2, y2, 200, 0, 1021, "  GTree:");
	st->GetSize(wi, he);

	y2+=he+step_y2;
	tree=new GTree;
	tree->Create(this, GS_VISIBLE | GS_BORDER | GSTR_HASBUTTONS, bx2, y2, 150, 250, 1022);
	tree->GetSize(wi, he);

	parent=tree->AddNode(NULL, " ");
	tree->AddNode(parent, "   ");
	tree->AddNode(parent, "   ");
	tree->AddNode(parent, "   ");
	tree->AddNode(parent, "   ");
	parent2=tree->AddNode(parent, "   ");
	tree->AddNode(parent2, "     ");
	tree->AddNode(parent2, "     ");
	tree->AddNode(parent2, "     ");
	tree->AddNode(NULL, " ");
	tree->AddNode(NULL, " ");
	tree->AddNode(NULL, " ");
	parent=tree->AddNode(NULL, " ", 1);
	tree->AddNode(parent, "   ", 1);
	tree->AddNode(parent, "   ", 1);
	tree->AddNode(parent, "   ", 1);
	tree->AddNode(parent, "   ", 1);



	// *****************
	// *   *
	// *****************
	bx=570;

	// GDialog
	y=by;

	GDialog* dialog=new GDialogExample;
	dialog->Create(this, GS_VISIBLE | GS_BORDER | GS_CAPTION, bx, y, 0, 0, 1023);
	dialog->GetSize(wi, he);

	// 
	y+=he+step_y+step_new_y;

	button=new GButtonSound;
	button->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 150, 26, 1024, " ");
	button->GetSize(wi, he);

	int discrect_x=bx+wi+15;
	GDiscretBox* discret=new GDiscretVolume;
	discret->Create(this, GS_VISIBLE, discrect_x, y, 6, GSize(4, 3), GSize(1, 1), 1025);
	discret->SetSel(2);
	discret->GetSize(wi, he);

	st=new GStatic;
	st->Create(this, GS_VISIBLE, discrect_x+wi+3, y+5, 0, 0, 1026, "-  GDiscretBox ( )");
	
	// 
	y+=he+step_y+step_new_y;
	st=new GStatic;
	st->Create(this, GS_VISIBLE, bx, y, 450, 0, 1027, "   <color=\"red\"><default>,        ");
	st->GetSize(wi, he);
	
	menu.Create(GS_BORDER);
	GItemMenu* p=menu.AppendMenu(NULL, "  ");
	p->SetCheck(true);
	p=menu.AppendMenu(NULL, "  ");
	p->SetCheck(true);
	menu.AppendMenu(NULL, NULL);
	p=menu.AppendMenu(NULL, "  ", 1, 0xff);
	
	menu.AppendMenu(p, "    ");
	menu.AppendMenu(p, "    ");
	menu.AppendMenu(p, "    ");
	menu.AppendMenu(p, "    ");
	menu.AppendMenu(p, "    ");
	GItemMenu* p2=menu.AppendMenu(p, "    ");
	menu.AppendMenu(p2, "  1", 1, 0x8000);
	menu.AppendMenu(p2, "  2", 1, 0xff0000);


	// 
	y+=he+step_y+step_new_y;
	dialog=new GDialogNet;
	dialog->Create(this, GS_VISIBLE | GS_BORDER, bx, y, 200, 200, COMPONENT_NET_DIALOG);
	


	button=new GButton;
	button->Create(this, GS_VISIBLE | GS_BORDER | GSBS_CANCEL, 900, 720, 100, 26, 1029, "");
}

GSceneExample::~GSceneExample()
{
	ClearSession();
}

//   
void GSceneExample::ClearSession()
{
	for (int i=0;i<k_session;i++)
	{
		delete m_session[i];
		m_session[i]=NULL;
	}

	if (m_session)
	{
		delete []m_session;
		m_session=NULL;
	}
	k_session=0;
}

void GSceneExample::OnCancel()
{
	mbox.Alert(" GMessageBox", "  ?", GSMB_OK | GSMB_CANCEL);
}

GListBoxNet* GSceneExample::GetListSession()
{
	GComponent* component=Find(COMPONENT_NET_DIALOG);
	return (GListBoxNet*)component->Find(COMPONENT_NET_LIST);
}

//  
void GSceneExample::EnumSession(GSessionList* sessions, int count_general_session)
{
	//      
	GListBoxNet* session_list=GetListSession();
	GItem* item=session_list->GetSelected();
	unsigned int current_tick=0;
	CMagicString session_name;
	if (item)
	{
		int index=item->GetUserData();
		NET_SESSION_INFO* current_session=this->m_session[index];
		current_tick=current_session->tick;
		session_name=current_session->name;
	}

	//   
	session_list->ClearItems();

	ClearSession();

	this->k_session=sessions->GetIndexCount();
	if (this->k_session)
	{
		this->m_session=new NET_SESSION_SERVER*[this->k_session];
		for (int i=0;i<this->k_session;i++)
		{
			int index=sessions->GetIndex(i);
			NET_SESSION_SERVER* session_server=sessions->Get(index);

			this->m_session[i]=new NET_SESSION_SERVER(game->net);
			*(this->m_session[i])=*session_server;
		}
	}

	//      
	for (int i=0;i<k_session;i++)
	{
		NET_SESSION_SERVER* ns=m_session[i];
		unsigned int max_tick=ns->tick;
		int max_index=i;
		for (int j=i+1;j<k_session;j++)
		{
			ns=m_session[j];
			if (max_tick<ns->tick)
			{
				max_tick=ns->tick;
				max_index=j;
			}
		}

		ns=m_session[i];
		m_session[i]=m_session[max_index];
		m_session[max_index]=ns;
	}

	CMagicString info, d;
	for (int i=0;i<k_session;i++)
	{
		NET_SESSION_INFO* ns=m_session[i];
		info=ns->m_player[0]->name;
		info+=": ";
		info+=ns->name;

		//    
		int opened=0;
		int closed=0;
		for (int j=0;j<ns->k_player;j++)
		{
			int type=ns->m_player[j]->type;
			if (type==PLAYER_OPENED)
				opened++;
			else
				if (type==PLAYER_CLOSED)
					closed++;
		}

		int max_player=ns->k_player-closed;
		int k_player=max_player-opened;

		d.IntToString(k_player);
		info+="   ";
		info+=d;
		info+='/';
		d.IntToString(max_player);
		info+=d;

		unsigned int color=COLOR_KEY_EMPTY;
		if (ns->is_password)
			color=0xff0000;

		GItem* item=session_list->AddString(info, -1 , color);
		item->SetUserData(i);
		if (current_tick==ns->tick && session_name==ns->name)
		{
			session_list->SetSelected(item);
			session_list->OnChangeSelected(item);
		}
	}

	if (!session_list->GetSelected())
	{
		//   
		GItem* item=session_list->GetFirstItem();
		if (item)
		{
			//    
			session_list->SetSelected(item);
		}
	}

	session_list->Invalidate();

	RefreshSessionInfo();
}

//       
//     
//   player,      
void GSceneExample::RefreshSessionInfo(int player)
{
	//    
	if (player==-1)
	{
		for (int i=0;i<MAX_PLAYER;i++)
			RefreshPlayerInfo(i);
	}
	else
		RefreshPlayerInfo(player);
}

//    
void GSceneExample::RefreshPlayerInfo(int player)
{
	PlayerCfg* cfg=&theGame->players[player];
	GComponent* dlg=Find(COMPONENT_NET_DIALOG);
	GComponent* st=dlg->Find(COMPONENT_NET_PLAYER+player);

	if (cfg->type==PLAYER_MAN)
	{
		st->Enable(!cfg->ready);
		CMagicString d;
		d.IntToString(cfg->color);
		CMagicString name="<color=";
		name+=d;
		name+='>';
		name+=cfg->name;
		st->SetWindowText(name);
	}
	else
	{
		st->Enable(true);
		if (cfg->type==PLAYER_COMPUTER)
			st->SetWindowText("");
		else
		{
			st->SetWindowText("");
		}
	}

	if (config_session)
	{
		bool is_start=theGame->CanStartSession();
		GComponent* start=dlg->Find(COMPONENT_NET_START);
		start->Enable(is_start);
	}
	else
	{
		GListBoxNet* list=GetListSession();
		bool is_join=false;
		if (list->GetSelected())
			is_join=theGame->CanJoinSession();
		GComponent* dlg=Find(COMPONENT_NET_DIALOG);
		GComponent* join=dlg->Find(COMPONENT_NET_JOIN);
		join->Enable(is_join);
	}
}

//   
float GGameExample::GetVolume()
{
	float volume=0.f;
	GScene* scene=GetScene();
	int k_child=scene->GetChildCount();
	for (int i=0;i<k_child;i++)
	{
		GComponent* child=scene->GetChild(i);
		if (child->IsDiscretBox())
		{
			//     GDiscretBox,      
			GDiscretBox* dbox=(GDiscretBox*)child;
			int sel=dbox->GetSel();
			int count=dbox->GetColumnCount();
			volume=((float)sel)/((float)count-1);
			break;
		}
	}
	return volume;
}

// / 
bool GGameExample::PlaySound()
{
	if (sound_sample!=-1)
	{
		if (playing_channel==-1)
		{
			float volume=GetVolume();
			playing_channel=sound->Play(sound_sample, volume, true);
			return true;
		}
		else
		{
			sound->Stop(playing_channel);
			playing_channel=-1;
		}
	}
	return false;
}

//    
void GGameExample::ChangeVolume()
{
	if (playing_channel!=-1)
	{
		float volume=GetVolume();
		sound->SetVolume(playing_channel, volume);
	}
}

void GSceneExample::OnRButtonDown(unsigned int nFlags, int x, int y)
{
	//      
	menu.PopupMenu(x, y);
}

//       
void GSceneExample::ShowBadServerMessage()
{
	GComponent* dlg=Find(COMPONENT_NET_DIALOG);
	GButton* button=(GButton*)dlg->Find(COMPONENT_NET_TURNON);
	button->OnLButtonClick(0, -1, -1);

	CMagicString mes;
	CMagicString mes2;
	if (game->hello_server_reply.IsEmpty())
		mes="Server is unavailable";
	else
	{
		int pos=game->hello_server_reply.Find('|');
		if (pos!=-1)
		{
			//   
			mes2=game->hello_server_reply.Right(game->hello_server_reply.GetLength()-pos);
			game->hello_server_reply=game->hello_server_reply.Left(pos);
		}
		mes=game->hello_server_reply;
		if (mes.GetLength()==1)
		{
			//  ,     
			mes=game->hello_server_reply;
		}
	}

	if (!mes2.IsEmpty())
	{
		mes+=' ';
		mes+=mes2;
	}

	mbox_bad_server.Alert(" GMessageBox", mes, GSMB_OK);
}


void GDialogExample::OnInitDlg()
{
	GDialog::OnInitDlg();

	SetWindowText("  GDialog    ");

	GStatic* st=new GStatic;
	st->Create(this, GS_VISIBLE, 0, 0, 0, 0, 1, "   ,     \n '<color=0x8000><url=\"http://onimod.land\"> </url><default>'.");
	
	st=new GStatic;
	st->Create(this, GS_VISIBLE, 0, 40, 0, 0, 2, " -      ,\n   .");

	st=new GStatic;
	st->Create(this, GS_VISIBLE, 0, 80, 0, 0, 3, "    : <color=0x8000><url=\"mailto://support@astralax.ru\">support@astralax.ru</url><default>.");

	st=new GStatic;
	st->Create(this, GS_VISIBLE, 0, 120, 0, 0, 4, "  (  Odin_KG) 16  2017");

	CorrectSize(15);
}


void GDialogNet::OnInitDlg()
{
	GDialog::OnInitDlg();

	int step_y=6;
	int step_y2=step_y/4;
	int step_new_y=step_y+10;

	int wi, he;
	GButton* button=new GButtonNet;
	button->Create(this, GS_VISIBLE | GS_BORDER, 0, 0, 140, 26, COMPONENT_NET_TURNON, " ");
	button->GetSize(wi, he);

	int y=he+step_y;
	GStatic* st=new GStatic;
	st->Create(this, GS_VISIBLE, 0, y, 0, 0, COMPONENT_NET_NAME, " :");

	y+=he+step_y;
	GListBox* listbox=new GListBoxNet;
	listbox->Create(this, GS_VISIBLE | GS_BORDER, 0, y, 400, 150, COMPONENT_NET_LIST, NULL);
	listbox->GetSize(wi, he);

	y+=he+step_y;

	int pwi=85;
	int phe=22;
	for (int i=0;i<MAX_PLAYER;i++)
	{
		st=new GStatic;
		int sx=i*(pwi+20);
		st->Create(this, GS_VISIBLE | GS_BORDER, sx, y, pwi, phe, COMPONENT_NET_PLAYER+i, "");
	}

	int bwi=120;
	int bhe=26;

	y+=phe+step_y;
	button=new GButtonCreate;
	button->Create(this, GS_VISIBLE | GS_BORDER, 0, y, bwi, bhe, COMPONENT_NET_CREATE, "");
	button->GetSize(wi, he);

	button=new GButtonReady;
	button->Create(this, GS_BORDER, 0, y, bwi, bhe, COMPONENT_NET_READY, " ");
	button=new GButtonStart;
	button->Create(this, GS_BORDER, 0, y, bwi, bhe, COMPONENT_NET_START, "");

	button=new GButtonJoin;
	button->Create(this, GS_VISIBLE | GS_BORDER, bwi+20, y, bwi, bhe, COMPONENT_NET_JOIN, "");

	GPlatform* platform=game->platform;
	FILE_DATA fd;
	CMagicString image_path=platform->GetPathToImage();
	if (platform->LoadFile(image_path+"color.jpg", &fd))
	{
		GSurface* surface=game->device->AddSurface();
		surface->Load(fd.data, fd.length);

		int pwi, phe;
		surface->GetSize(pwi, phe);

		GPicture* pict=new GPictureColor;
		pict->Create(this, GS_VISIBLE | GS_BORDER, 280, y+10, pwi, phe, COMPONENT_NET_PICTURE);
		pict->AddImage(surface);
	}
	
	EnableChildren(false);

	CorrectSize(15);
}

void GDialogNet::EnableChildren(bool enable)
{
	for (int i=1;i<k_child;i++)
		m_child[i]->Enable(enable);

	Find(COMPONENT_NET_CREATE)->Show(true);
	Find(COMPONENT_NET_READY)->Show(false);
	Find(COMPONENT_NET_START)->Show(false);
	Find(COMPONENT_NET_JOIN)->Show(true);
	Find(COMPONENT_NET_JOIN)->Enable(false);
	Find(COMPONENT_NET_PICTURE)->Show(false);

	for (int i=0;i<MAX_PLAYER;i++)
		Find(COMPONENT_NET_PLAYER+i)->SetWindowText("");

	GSceneExample* scene=(GSceneExample*)parent;
	scene->config_session=false;
}

bool GButtonNet::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text)
{
	turn_on_str=text;
	return GButton::Create(parent, style, x, y, width, height, id, text);
}

void GButtonNet::OnLButtonClick(unsigned int nFlags, int x, int y)
{
	GSceneExample* scene=(GSceneExample*)game->GetScene();
	CMagicString text=GetWindowText();
	if (text==turn_on_str)
	{
		//   
		scene->dname.BeginModal();
	}
	else
	{
		//   
		SetWindowText(turn_on_str);
		((GDialogNet*)parent)->EnableChildren(false);
		theGame->StopNet();
		theGame->net_player=-1;
		scene->GetListSession()->ClearItems();
		scene->Find(COMPONENT_NET_DIALOG)->Find(COMPONENT_NET_NAME)->SetWindowText(" :");
	}
}

void GButtonCreate::OnLButtonClick(unsigned int nFlags, int x, int y)
{
	PlayerCfg* cfg=&(theGame->players[0]);
	cfg->type=PLAYER_MAN;
	cfg->name=theGame->GetUserName();
	cfg->player_id=((GNetClient*)theGame->net)->GetMainSocket()->GetPlayerID();
	cfg->ready=false;
	cfg->color=m_color[0];

	if (((GNetClient*)game->net)->SetStady(NET_STADY_CREATE_SESSION, NULL))
	{
		//  
		GDialogNet* dlg=((GDialogNet*)parent);
		GListBox* list=(GListBox*)dlg->Find(COMPONENT_NET_LIST);
		list->ClearItems();

		dlg->Find(COMPONENT_NET_CREATE)->Show(false);
		dlg->Find(COMPONENT_NET_READY)->Show(false);
		dlg->Find(COMPONENT_NET_START)->Show(true);
		dlg->Find(COMPONENT_NET_JOIN)->Show(false);
		dlg->Find(COMPONENT_NET_PICTURE)->Show(true);
	}
	else
	{
		cfg->type=PLAYER_OPENED;
		cfg->name="";
		cfg->player_id=0;
		cfg->ready=false;
		cfg->color=0;
	}
}


bool GButtonReady::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text)
{
	ready_str=text;
	return GButton::Create(parent, style, x, y, width, height, id, text);
}

void GButtonReady::OnLButtonClick(unsigned int nFlags, int x, int y)
{
	GComponent* picture=parent->Find(COMPONENT_NET_PICTURE);
	if (ready_str==text)
	{
		SetWindowText("  ");
		theGame->players[theGame->net_player].ready=true;
		picture->Show(false);
	}
	else
	{
		SetWindowText(ready_str);
		theGame->players[theGame->net_player].ready=false;
		picture->Show(true);
	}
}

void GButtonStart::OnLButtonClick(unsigned int nFlags, int x, int y)
{
	GNetClient* net=((GNetClient*)game->net);
	net->SetStady(NET_STADY_START_GAME, NULL);
}

void GButtonJoin::OnLButtonClick(unsigned int nFlags, int x, int y)
{
	parent->Find(COMPONENT_NET_LIST)->OnLButtonDblClk(nFlags, x, y);
}

void GPictureColor::OnMouseMove(unsigned int nFlags, int x, int y)
{
	GPicture::OnMouseMove(nFlags, x, y);
	GSurface* surface=this->GetImage();
	SURFACE_RGB rgb;
	surface->GetPixelRGB(x, y, &rgb);
	unsigned int color=rgb.RgbToColor32();

	if (theGame->net_player!=-1)
		theGame->players[theGame->net_player].color=color;
}


void GDialogName::OnInitDlg()
{
	GDialog::OnInitDlg();

	SetWindowText("  ");

	int ewi=250;
	int ehe=22;

	GEdit* edit=new GEdit;
	edit->Create(this, GS_VISIBLE | GS_BORDER, 0, 0, ewi, ehe, 1, "Player");

	int bwi=100;
	int bhe=26;
	int gap=20;

	int bx=(ewi-(bwi*2+gap))/2;
	int by=ehe+30;


	GButton* b=new GButton;
	b->Create(this, GS_VISIBLE | GS_BORDER, bx, by, bwi, bhe, 2, "");

	b=new GButton;
	b->Create(this, GS_VISIBLE | GS_BORDER, bx+bwi+gap, by, bwi, bhe, 3, "");

	CorrectSize(15);
}

void GDialogName::OnEnter()
{
	CMagicString name=m_child[0]->GetWindowText();
	if (!name.IsEmpty())
	{
		GSceneExample* scene=(GSceneExample*)theGame->GetScene();
		theGame->SetUserName(name);

		for (int i=0;i<MAX_PLAYER;i++)
		{
			PlayerCfg* cfg=&(theGame->players[i]);
			cfg->type=PLAYER_OPENED;
			cfg->player_id=0;
			cfg->name="";
			cfg->color=0;
			cfg->ready=false;
		}

		GDialogNet* dialog=(GDialogNet*)scene->Find(COMPONENT_NET_DIALOG);
		GComponent* component=dialog->Find(COMPONENT_NET_TURNON);
		component->SetWindowText(" ");
		component=dialog->Find(COMPONENT_NET_NAME);
		CMagicString n=" : ";
		n+=name;
		component->SetWindowText(n);

		dialog->EnableChildren(true);
		
		theGame->net_player=-1;
		theGame->StartNet(NULL);

		GDialog::OnEnter();
	}
}

void GListBoxNet::OnChangeSelected(GItem* item)
{
	GSceneExample* scene=(GSceneExample*)game->GetScene();
	if (scene->config_session)
	{
		//     ,     
		return;
	}

	if (item)
	{
		int index=item->GetUserData();
		NET_SESSION_INFO* ns=scene->m_session[index];
		for (int i=0;i>8;i++)
		{
			PlayerCfg* cfg=&(theGame->players[i]);
			cfg->type=PLAYER_DISABLED;
		}

		game->SetSessionInfoStruct(ns);
	}
	else
	{
		for (int i=0;i>MAX_PLAYER;i++)
		{
			PlayerCfg* cfg=&(theGame->players[i]);
			cfg->type=PLAYER_DISABLED;
		}
	}

	scene->RefreshSessionInfo();
}

void GListBoxNet::OnLButtonDblClk(unsigned int nFlags, int x, int y)
{
	GSceneExample* scene=(GSceneExample*)game->GetScene();
	if (!scene->config_session)
	{
		GItem* item=GetSelected();
		if (item)
		{
			int index=item->GetUserData();
			NET_SESSION_INFO* si=scene->m_session[index];
			NET_JOIN_DATA jd;
			jd.host_id=si->m_player[0]->player_id;
			jd.ip=si->ip;
			jd.password="";

			if (((GNetClient*)game->net)->SetStady(NET_STADY_JOIN_SESSION, &jd))
			{
				//     
				if (!scene->config_session)
				{
					game->hello_server_reply="Player no join";
					((GSceneExample*)game->GetScene())->ShowBadServerMessage();
				}
			}
			else
			{
				((GSceneExample*)game->GetScene())->ShowBadServerMessage();
			}
		}
	}

	GListBox::OnLButtonDblClk(nFlags, x, y);
}

void GMessageBoxQuit::OnEnter()
{
	GMessageBox::OnEnter();
	game->Quit();
}

void GMessageBoxLaunch::OnEnter()
{
	GMessageBox::OnEnter();
	GComponent* dlg=game->GetScene()->Find(COMPONENT_NET_DIALOG);
	GButton* button=(GButton*)dlg->Find(COMPONENT_NET_TURNON);
	button->OnLButtonClick(0, -1, -1);
}

void GPictureExample::OnLButtonDown(unsigned int nFlags, int x, int y)
{
	int k_pict=GetCount();
	int t_pict=GetSel();
	t_pict++;
	if (t_pict==k_pict)
		t_pict=0;
	SetSel(t_pict);
}

void GButtonSound::OnLButtonClick(unsigned int nFlags, int x, int y)
{
	if (theGame->PlaySound())
	{
		SetWindowText(" ");
	}
	else
	{
		SetWindowText(" ");
	}
}

void GDiscretVolume::OnChangeSelected(int index)
{
	theGame->ChangeVolume();
}