//-----------------------------------------------------------------------------
// File: GameExample.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once
#include "..\Tools\Gui\Game.h"


#define COMPONENT_NET_DIALOG	1028

#define COMPONENT_NET_TURNON	2001
#define COMPONENT_NET_NAME		2002
#define COMPONENT_NET_LIST		2003
#define COMPONENT_NET_PLAYER	2004

#define COMPONENT_NET_CREATE	2020
#define COMPONENT_NET_READY		2021
#define COMPONENT_NET_START		2022
#define COMPONENT_NET_JOIN		2023
#define COMPONENT_NET_PICTURE	2024


#define MAX_PLAYER				4

struct PlayerCfg
{
	int type;
	CMagicString name;
	unsigned int player_id;
	unsigned int color;
	bool ready;

	void Serialize(CMagicStream& stream)
	{
		if (stream.IsStoring())
		{
			stream<<type;
			stream<<name;
			stream<<player_id;
			stream<<color;
			stream<<ready;
		}
		else
		{
			stream>>type;
			stream>>name;
			stream>>player_id;
			stream>>color;
			stream>>ready;
		}
	}
};


class GGameExample :
	public GGame
{
protected:
	VIDEO_MODE video_mode_menu;

	int font1;

	bool show_fps;

	int playing_channel;
	int sound_sample;

public:
	int net_player;
	//CMagicString net_player_name;

	PlayerCfg players[MAX_PLAYER];

public:
	GGameExample(GUID_GAME guid, const char* name, const char* version, bool fullscreen_mode=false);
	virtual ~GGameExample(void);

	//  ,    
	virtual bool CreateGame(void* init, int view_width=0, int view_height=0);

	//     
	virtual void SysOnRender();

	//  
	virtual void DrawCursor(GImage* cur);

	virtual const char* GetServerFileName();

	//  
	virtual GPlatform* NewPlatform(GGame* owner, const char* game_name);

	//  
	virtual GSound* NewSound();

	//  
	virtual GNet* NewNet();
	//   
	virtual GNetServer* NewServer();
	//    
	virtual GNetClient* NewDownloader();


	//   
	float GetVolume();

	// / 
	bool PlaySound();

	//    
	void ChangeVolume();

	//     
	bool CanStartSession();

	//      
	bool CanJoinSession();

	//     
	//                  
	virtual void OnEnumSession(GSessionList* sessions, int count_general_session);

	//    
	virtual void OnCreateSession(int index_player);
	//      (   ,   false)
	virtual bool OnJoinSession(int index_player, const char* map_name, unsigned int map_id);

	//  -
	virtual void OnChatMessage(CMagicString& sender, CMagicString& receiver, bool privated, bool global, CMagicString& chat, bool printed_message=true);

	//       (5,4,3,2,1...   OnStartNetGame(),    )
	virtual void OnStartNetCounter(int counter);

	//   ,          
	virtual bool IsNetGameLoaded();
	//     
	virtual void OnLaunchNetGame();

	//     ,       
	virtual bool GetSessionInfoStruct(NET_SESSION_INFO* si);
	virtual bool GetSessionPlayerStruct(int index, NET_PLAYER* np);

	//       ,    
	virtual void SetSessionInfoStruct(NET_SESSION_INFO* si);
	virtual void SetSessionPlayerStruct(int index, NET_PLAYER* np);
};

class GDialogExample : public GDialog
{
public:
	virtual void OnInitDlg();
};

class GDialogNet : public GDialog
{
public:
	virtual void OnInitDlg();

	void EnableChildren(bool enable);
};

class GButtonNet : public GButton
{
protected:
	CMagicString turn_on_str;
public:
	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text);
	virtual void OnLButtonClick(unsigned int nFlags, int x, int y);
};

class GButtonCreate : public GButton
{
public:
	virtual void OnLButtonClick(unsigned int nFlags, int x, int y);
};

class GButtonReady :  public GButton
{
protected:
	CMagicString ready_str;
public:
	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text);
	virtual void OnLButtonClick(unsigned int nFlags, int x, int y);
};

class GButtonStart :  public GButton
{
public:
	virtual void OnLButtonClick(unsigned int nFlags, int x, int y);
};

class GButtonJoin :  public GButton
{
public:
	virtual void OnLButtonClick(unsigned int nFlags, int x, int y);
};

class GPictureColor : public GPicture
{
public:
	virtual void OnMouseMove(unsigned int nFlags, int x, int y);
};

class GDialogName : public GDialog
{
public:
	virtual void OnInitDlg();
	virtual void OnEnter();
};

class GListBoxNet : public GListBox
{
public:
	virtual void OnChangeSelected(GItem* item);

	virtual void OnLButtonDblClk(unsigned int nFlags, int x, int y);
};

class GMessageBoxQuit : public GMessageBox
{
public:
	virtual void OnEnter();
};

class GMessageBoxLaunch : public GMessageBox
{
public:
	virtual void OnEnter();
};

class GSceneExample : public GScene
{
public:
	GMenu menu;
	GMessageBoxQuit mbox;
	GMessageBoxLaunch mbox_launch;
	GMessageBox mbox_bad_server;
	GDialogName dname;

	GImageList icons;

	int k_session;
	NET_SESSION_SERVER** m_session;

	bool config_session;

public:
	GSceneExample();
	virtual ~GSceneExample();

	//   
	void ClearSession();

	virtual void OnRButtonDown(unsigned int nFlags, int x, int y);

	virtual void OnCancel();

	GListBoxNet* GetListSession();

	//  
	void EnumSession(GSessionList* sessions, int count_general_session);

	//       
	//     
	//   player,      
	void RefreshSessionInfo(int player=-1);

	//    
	void RefreshPlayerInfo(int player);

	//       
	void ShowBadServerMessage();
};


class GPictureExample : public GPicture
{
public:
	virtual void OnLButtonDown(unsigned int nFlags, int x, int y);
};

class GButtonSound : public GButton
{
public:
	virtual void OnLButtonClick(unsigned int nFlags, int x, int y);
};

class GDiscretVolume : public GDiscretBox
{
public:
	virtual void OnChangeSelected(int index);
};

