//-----------------------------------------------------------------------------
// File: DeviceWindowsDX9.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "StdAfx.h"
#include "DeviceWindowsDX9.h"

extern HWND hWnd;

//    
float GetUV(int coord, int size)
{
	//float size2=(float)(size-1);
	float size2=(float)size;
	float coord2=(float)coord;
	if (size2<=coord)
		return 1.f;

	float result=coord2/size2;
		
	return result;
}

GDeviceWindowsDX9::GDeviceWindowsDX9(bool fullscreen_mode, void* init) : GDeviceWindows(fullscreen_mode, init)
{
	direct_3d=NULL;
	device_3d=NULL;

	#ifndef BACKBUFFER_SCREEN
	vertex_buffer=NULL;
	#endif
	texture=NULL;
}

GDeviceWindowsDX9::~GDeviceWindowsDX9(void)
{
	Destroy();
}

//  
void GDeviceWindowsDX9::Destroy()
{
	GDeviceWindows::Destroy();

	DestroyDirect3D();
}

void GDeviceWindowsDX9::DestroyTexture()
{
	if (texture)
	{
		texture->Release();
		texture=NULL;
	}

	#ifndef BACKBUFFER_SCREEN

	if (vertex_buffer)
	{
		vertex_buffer->Release();
		vertex_buffer=NULL;
	}

	#endif
}

void GDeviceWindowsDX9::DestroyDirect3D()
{
	DestroyTexture();
	
	if (device_3d)
	{
		device_3d->Release();
		device_3d=NULL;
	}

	if (direct_3d)
	{
		direct_3d->Release();
		direct_3d=NULL;
	}
}

//   
void GDeviceWindowsDX9::GetSceneRect(CMagicRect* rect)
{
	RECT r;
	GetClientRect(hWnd, &r);
	rect->left=r.left;
	rect->top=r.top;
	rect->right=r.right;
	rect->bottom=r.bottom;
}

//    
void GDeviceWindowsDX9::RefreshVideoMode()
{
	if (!k_mode)
	{
		//      
		if (direct_3d)
		{
			unsigned int status=_controlfp(0,0);

			D3DFORMAT format=D3DFMT_X8R8G8B8;
			int k_addapter_mode=direct_3d->GetAdapterModeCount(D3DADAPTER_DEFAULT, format);
			for (int i=0;i<k_addapter_mode;i++)
			{
				D3DDISPLAYMODE mode;
				direct_3d->EnumAdapterModes(D3DADAPTER_DEFAULT, format, i, &mode);

				VIDEO_MODE vm;
				vm.width=mode.Width;
				vm.height=mode.Height;
				vm.bpp=32;
				if (IsVideoMode(&vm))
				{
					int index=-1;
					for (int i=0;i<k_mode;i++)
					{
						if (m_mode[i]==vm)
						{
							index=i;
							break;
						}
					}
					if (index==-1)
					{
						m_mode[k_mode]=vm;
						k_mode++;
					}
				}
			}

			_controlfp(status,_MCW_DN | _MCW_IC | _MCW_RC | _MCW_PC);

			//   
			GDevice::RefreshVideoMode();
		}
	}
}

//   
void GDeviceWindowsDX9::GetMonitorMode(VIDEO_MODE* vm)
{
	if (direct_3d)
	{
		unsigned int status=_controlfp(0,0);

		D3DDISPLAYMODE mode;
		HRESULT hr=direct_3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &mode);

		vm->width=mode.Width;
		vm->height=mode.Height;
		vm->bpp=32;

		_controlfp(status,_MCW_DN | _MCW_IC | _MCW_RC | _MCW_PC);

		if (hr!=D3D_OK)
			GDeviceWindows::GetMonitorMode(vm);
	}
	else
		GDeviceWindows::GetMonitorMode(vm);
}

//    /
void GDeviceWindowsDX9::ChangeResolution(bool monitor, bool scene)
{
	unsigned int status=_controlfp(0,0);

	bool change_monitor=monitor;
	if (windowed)
		change_monitor=false;

	GDeviceWindows::ChangeResolution(change_monitor, scene);

	if (monitor)
	{
		DestroyDirect3D();
		scene=true;

		direct_3d=Direct3DCreate9(D3D_SDK_VERSION);
		if (direct_3d)
		{
			D3DDISPLAYMODE Display;
			HRESULT ok=direct_3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &Display);
			if (ok==D3D_OK)
			{
				D3DPRESENT_PARAMETERS d3d9pp;
				ZeroMemory( &d3d9pp, sizeof(d3d9pp) );
				
				d3d9pp.BackBufferFormat=Display.Format;

				d3d9pp.BackBufferCount=1;
				d3d9pp.PresentationInterval=D3DPRESENT_INTERVAL_IMMEDIATE;
				d3d9pp.SwapEffect=D3DSWAPEFFECT_DISCARD;
				d3d9pp.hDeviceWindow=hWnd;

				if (fullscreen_mode & (!windowed))
				{
					d3d9pp.SwapEffect=D3DSWAPEFFECT_FLIP;
					d3d9pp.PresentationInterval=D3DPRESENT_INTERVAL_ONE;
					#ifdef MAX_FPS
					d3d9pp.PresentationInterval=D3DPRESENT_INTERVAL_IMMEDIATE;
					#endif
					d3d9pp.Windowed=false;
					d3d9pp.FullScreen_RefreshRateInHz=Display.RefreshRate;
				}
				else
				{
					d3d9pp.Windowed=true;
				}
				d3d9pp.BackBufferWidth=monitor_width;
				d3d9pp.BackBufferHeight=monitor_height;


				while (true)
				{
					ok=direct_3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
						D3DCREATE_HARDWARE_VERTEXPROCESSING,
						&d3d9pp, &device_3d );

					if (ok==D3D_OK)
						break;

					ok=direct_3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
						D3DCREATE_SOFTWARE_VERTEXPROCESSING,
						&d3d9pp, &device_3d );

					if (ok==D3D_OK)
					{
						break;
					}

					switch (ok)
					{
					case D3DERR_OUTOFVIDEOMEMORY:
						return;
					case D3DERR_INVALIDCALL:
						//  MS-DOS
						Sleep(300);
						break;
					case D3DERR_NOTAVAILABLE:
						//   
						Sleep(300);
						break;
					default:
						Sleep(300);
						break;
					}
				}

				device_3d->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
				device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);

				device_3d->SetRenderState(D3DRS_LIGHTING, FALSE);
				device_3d->SetRenderState(D3DRS_ZENABLE, FALSE);

				device_3d->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
			}
		}
	}

	if (scene && device_3d)
	{
		DestroyTexture();

		//   
		device_3d->Clear(0, NULL, D3DCLEAR_TARGET, 0x0, 0.0f, 0);
		if (fullscreen_mode & (!windowed))
		{
			HRESULT hr=device_3d->Present(NULL, NULL, NULL, NULL);
			device_3d->Clear(0, NULL, D3DCLEAR_TARGET, 0x0, 0.0f, 0);
		}

		#ifdef BACKBUFFER_SCREEN

		device_3d->CreateTexture(width, height, 1, D3DUSAGE_DYNAMIC, D3DFMT_X8R8G8B8, D3DPOOL_SYSTEMMEM, &texture, NULL);

		#else

		int wi2=1;
		while(wi2<width)
			wi2*=2;

		int he2=1;
		while(he2<height)
			he2*=2;

		device_3d->CreateTexture(width, height, 1, D3DUSAGE_DYNAMIC, D3DFMT_A1R5G5B5, D3DPOOL_DEFAULT, &texture, NULL);
		if (texture)
		{
			wi2=width;
			he2=height;
		}
		else
			device_3d->CreateTexture(wi2, he2, 1, D3DUSAGE_DYNAMIC, D3DFMT_A1R5G5B5, D3DPOOL_DEFAULT, &texture, NULL);
		
		int length=6*sizeof(VERTEX_PARTICLE);
		device_3d->CreateVertexBuffer(length, D3DUSAGE_WRITEONLY,FVF_PARTICLE,D3DPOOL_DEFAULT,&vertex_buffer,NULL);

		bool filter=CalcTexRect();
		float x1=(float)tex_rect.left;
		float y1=(float)tex_rect.top;
		float x2=(float)tex_rect.right;
		float y2=(float)tex_rect.bottom;
		
		float u2=GetUV(width, wi2);
		float v2=GetUV(height, he2);

		VERTEX_PARTICLE* locked_buffer;
		vertex_buffer->Lock(0, length, (void**)(&locked_buffer), 0);

		VERTEX_PARTICLE* b=&(locked_buffer[0]);
		b->x=x1;
		b->y=y1;
		b->z=0.f;
		b->rhw=1.f;
		b->u=0.f;
		b->v=0.f;

		b=&(locked_buffer[1]);
		b->x=x2;
		b->y=y1;
		b->z=0.f;
		b->rhw=1.f;
		b->u=u2;
		b->v=0.f;

		b=&(locked_buffer[2]);
		b->x=x1;
		b->y=y2;
		b->z=0.f;
		b->rhw=1.f;
		b->u=0.f;
		b->v=v2;

		b=&(locked_buffer[3]);
		b->x=x1;
		b->y=y2;
		b->z=0.f;
		b->rhw=1.f;
		b->u=0.f;
		b->v=v2;

		b=&(locked_buffer[4]);
		b->x=x2;
		b->y=y1;
		b->z=0.f;
		b->rhw=1.f;
		b->u=u2;
		b->v=0.f;

		b=&(locked_buffer[5]);
		b->x=x2;
		b->y=y2;
		b->z=0.f;
		b->rhw=1.f;
		b->u=u2;
		b->v=v2;

		vertex_buffer->Unlock();

		#endif

		DWORD filter_mode=D3DTEXF_POINT;
		if (filter)
			filter_mode=D3DTEXF_ANISOTROPIC;

		device_3d->SetSamplerState(0, D3DSAMP_MIPFILTER, filter_mode);
		device_3d->SetSamplerState(0, D3DSAMP_MINFILTER, filter_mode);
		device_3d->SetSamplerState(0, D3DSAMP_MAGFILTER, filter_mode); 


		device_3d->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_CLAMP);
		device_3d->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_CLAMP);

		device_3d->SetTextureStageState (0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		device_3d->SetTextureStageState (0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
		device_3d->SetTextureStageState (0, D3DTSS_COLOROP,   D3DTOP_MODULATE);

		device_3d->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
		device_3d->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );
		device_3d->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );
	}

	_controlfp(status,_MCW_DN | _MCW_IC | _MCW_RC | _MCW_PC);
}

//      (    )
//    monitor_width, monitor_height
//    ,   
bool GDeviceWindowsDX9::SetMonitorResolutionOnly()
{
	if (fullscreen_mode)
	{
		VIDEO_MODE mm;
		GetMonitorMode(&mm);

		if (mm.width!=monitor_width || mm.height!=monitor_height)
			return true;
		return false;
	}
	else
		return GDeviceWindows::SetMonitorResolutionOnly();
}

void GDeviceWindowsDX9::LockBackbuffer(SURFACE_LOCKED* locked)
{
	D3DLOCKED_RECT surface;
	RECT rect;
	rect.left=0;
	rect.top=0;
	rect.right=width;
	rect.bottom=height;

	//#ifdef BACKBUFFER_SCREEN
	//device_3d->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &back_surface);
	//HRESULT hr=back_surface->LockRect(&surface, &rect, 0);
	//#else
	HRESULT hr=texture->LockRect(0, &surface, &rect, 0);
	//#endif

	locked->ptr=surface.pBits;
	locked->pitch=surface.Pitch;
	locked->alpha_pitch=0;
	locked->alpha_ptr=NULL;
}

void GDeviceWindowsDX9::UnlockBackbuffer()
{
	//#ifdef BACKBUFFER_SCREEN
	//back_surface->UnlockRect();
	//back_surface->Release();
	//#else
	texture->UnlockRect(0);
	//#endif
}

//    
void GDeviceWindowsDX9::GetScreenModeDefault(VIDEO_MODE* vm)
{
	vm->width=800;
	vm->height=600;
	vm->bpp=32;
}

//   
bool GDeviceWindowsDX9::IsLost()
{
	return (device_3d->TestCooperativeLevel()!=D3D_OK);
}

// 
void GDeviceWindowsDX9::Render()
{
	HRESULT hr=device_3d->BeginScene();
	
	#ifdef BACKBUFFER_SCREEN
	LPDIRECT3DSURFACE9 backbuf=NULL;
	device_3d->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &backbuf);
	LPDIRECT3DSURFACE9 tex=NULL;
	texture->GetSurfaceLevel(0, &tex); 
	device_3d->UpdateSurface(tex, NULL, backbuf, NULL);
	backbuf->Release();
	tex->Release();

	#else

	hr=device_3d->SetTexture(0, texture);
	
	hr=device_3d->SetFVF(FVF_PARTICLE);
	hr=device_3d->SetStreamSource(0, vertex_buffer, 0,sizeof(VERTEX_PARTICLE));

	hr=device_3d->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);

	#endif

	hr=device_3d->EndScene();

	if (fullscreen_mode & (!windowed))
		hr=device_3d->Present(NULL, NULL, NULL, NULL);
	else
	{
		CMagicRect rect;
		GetSceneRect(&rect);
		RECT from, to;
		from.left=rect.left;
		from.top=rect.top;
		from.right=rect.right;
		from.bottom=rect.bottom;

		to.left=0;
		to.top=0;
		to.right=rect.Width();
		to.bottom=rect.Height();

		hr=device_3d->Present(&from, &to, NULL, NULL);
	}
}

/*
//  
void GDeviceWindowsDX9::Activate()
{
	//CreateDirect3D();
	GDeviceWindows::Activate();
}
*/

//  
void GDeviceWindowsDX9::Inactivate()
{
	GDeviceWindows::Inactivate();
	DestroyDirect3D();
}
