//-----------------------------------------------------------------------------
// File: Game.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once

#include "Graphics.h"
#include "Theme.h"
#include "Net.h"
#include "VirtualDrive.h"

#define DEFAULT_POSITION	9823562

class GGame;
class GListBase;
class GTree;
class GSound;

//  
#define KEY_ESCAPE          0x01
#define KEY_1               0x02
#define KEY_2               0x03
#define KEY_3               0x04
#define KEY_4               0x05
#define KEY_5               0x06
#define KEY_6               0x07
#define KEY_7               0x08
#define KEY_8               0x09
#define KEY_9               0x0A
#define KEY_0               0x0B
#define KEY_MINUS           0x0C    /* - on main keyboard */
#define KEY_EQUALS          0x0D
#define KEY_BACK            0x0E    /* backspace */
#define KEY_TAB             0x0F
#define KEY_Q               0x10
#define KEY_W               0x11
#define KEY_E               0x12
#define KEY_R               0x13
#define KEY_T               0x14
#define KEY_Y               0x15
#define KEY_U               0x16
#define KEY_I               0x17
#define KEY_O               0x18
#define KEY_P               0x19
#define KEY_LBRACKET        0x1A
#define KEY_RBRACKET        0x1B
#define KEY_RETURN          0x1C    /* Enter on main keyboard */
#define KEY_LCONTROL        0x1D
#define KEY_A               0x1E
#define KEY_S               0x1F
#define KEY_D               0x20
#define KEY_F               0x21
#define KEY_G               0x22
#define KEY_H               0x23
#define KEY_J               0x24
#define KEY_K               0x25
#define KEY_L               0x26
#define KEY_SEMICOLON       0x27
#define KEY_APOSTROPHE      0x28
#define KEY_GRAVE           0x29    /* accent grave */
#define KEY_LSHIFT          0x2A
#define KEY_BACKSLASH       0x2B
#define KEY_Z               0x2C
#define KEY_X               0x2D
#define KEY_C               0x2E
#define KEY_V               0x2F
#define KEY_B               0x30
#define KEY_N               0x31
#define KEY_M               0x32
#define KEY_COMMA           0x33
#define KEY_PERIOD          0x34    /* . on main keyboard */
#define KEY_SLASH           0x35    /* / on main keyboard */
#define KEY_RSHIFT          0x36
#define KEY_MULTIPLY        0x37    /* * on numeric keypad */
#define KEY_LALT           0x38    /* left Alt */
#define KEY_SPACE           0x39
#define KEY_CAPITAL         0x3A
#define KEY_F1              0x3B
#define KEY_F2              0x3C
#define KEY_F3              0x3D
#define KEY_F4              0x3E
#define KEY_F5              0x3F
#define KEY_F6              0x40
#define KEY_F7              0x41
#define KEY_F8              0x42
#define KEY_F9              0x43
#define KEY_F10             0x44
#define KEY_NUMLOCK         0x45
#define KEY_SCROLL          0x46    /* Scroll Lock */
#define KEY_NUMPAD7         0x47
#define KEY_NUMPAD8         0x48
#define KEY_NUMPAD9         0x49
#define KEY_SUBTRACT        0x4A    /* - on numeric keypad */
#define KEY_NUMPAD4         0x4B
#define KEY_NUMPAD5         0x4C
#define KEY_NUMPAD6         0x4D
#define KEY_ADD             0x4E    /* + on numeric keypad */
#define KEY_NUMPAD1         0x4F
#define KEY_NUMPAD2         0x50
#define KEY_NUMPAD3         0x51
#define KEY_NUMPAD0         0x52
#define KEY_DECIMAL         0x53    /* . on numeric keypad */
#define KEY_OEM_102         0x56    /* < > | on UK/Germany keyboards */
#define KEY_F11             0x57
#define KEY_F12             0x58
#define KEY_F13             0x64    /*                     (NEC PC98) */
#define KEY_F14             0x65    /*                     (NEC PC98) */
#define KEY_F15             0x66    /*                     (NEC PC98) */
#define KEY_KANA            0x70    /* (Japanese keyboard)            */
#define KEY_ABNT_C1         0x73    /* / ? on Portugese (Brazilian) keyboards */
#define KEY_CONVERT         0x79    /* (Japanese keyboard)            */
#define KEY_NOCONVERT       0x7B    /* (Japanese keyboard)            */
#define KEY_YEN             0x7D    /* (Japanese keyboard)            */
#define KEY_ABNT_C2         0x7E    /* Numpad . on Portugese (Brazilian) keyboards */
#define KEY_NUMPADEQUALS    0x8D    /* = on numeric keypad (NEC PC98) */
#define KEY_PREVTRACK       0x90    /* Previous Track (#define KEY_CIRCUMFLEX on Japanese keyboard) */
#define KEY_AT              0x91    /*                     (NEC PC98) */
#define KEY_COLON           0x92    /*                     (NEC PC98) */
#define KEY_UNDERLINE       0x93    /*                     (NEC PC98) */
#define KEY_KANJI           0x94    /* (Japanese keyboard)            */
#define KEY_STOP            0x95    /*                     (NEC PC98) */
#define KEY_AX              0x96    /*                     (Japan AX) */
#define KEY_UNLABELED       0x97    /*                        (J3100) */
#define KEY_NEXTTRACK       0x99    /* Next Track */
#define KEY_NUMPADENTER     0x9C    /* Enter on numeric keypad */
#define KEY_RCONTROL        0x9D
#define KEY_MUTE            0xA0    /* Mute */
#define KEY_CALCULATOR      0xA1    /* Calculator */
#define KEY_PLAYPAUSE       0xA2    /* Play / Pause */
#define KEY_MEDIASTOP       0xA4    /* Media Stop */
#define KEY_VOLUMEDOWN      0xAE    /* Volume - */
#define KEY_VOLUMEUP        0xB0    /* Volume + */
#define KEY_WEBHOME         0xB2    /* Web home */
#define KEY_NUMPADCOMMA     0xB3    /* , on numeric keypad (NEC PC98) */
#define KEY_DIVIDE          0xB5    /* / on numeric keypad */
#define KEY_SYSRQ           0xB7
#define KEY_RALT           0xB8    /* right Alt */
#define KEY_PAUSE           0xC5    /* Pause */
#define KEY_HOME            0xC7    /* Home on arrow keypad */
#define KEY_UP              0xC8    /* UpArrow on arrow keypad */
#define KEY_PRIOR           0xC9    /* PgUp on arrow keypad */
#define KEY_LEFT            0xCB    /* LeftArrow on arrow keypad */
#define KEY_RIGHT           0xCD    /* RightArrow on arrow keypad */
#define KEY_END             0xCF    /* End on arrow keypad */
#define KEY_DOWN            0xD0    /* DownArrow on arrow keypad */
#define KEY_NEXT            0xD1    /* PgDn on arrow keypad */
#define KEY_INSERT          0xD2    /* Insert on arrow keypad */
#define KEY_DELETE          0xD3    /* Delete on arrow keypad */
#define KEY_LWIN            0xDB    /* Left Windows key */
#define KEY_RWIN            0xDC    /* Right Windows key */
#define KEY_APPS            0xDD    /* AppMenu key */
#define KEY_POWER           0xDE    /* System Power */
#define KEY_SLEEP           0xDF    /* System Sleep */
#define KEY_WAKE            0xE3    /* System Wake */
#define KEY_WEBSEARCH       0xE5    /* Web Search */
#define KEY_WEBFAVORITES    0xE6    /* Web Favorites */
#define KEY_WEBREFRESH      0xE7    /* Web Refresh */
#define KEY_WEBSTOP         0xE8    /* Web Stop */
#define KEY_WEBFORWARD      0xE9    /* Web Forward */
#define KEY_WEBBACK         0xEA    /* Web Back */
#define KEY_MYCOMPUTER      0xEB    /* My Computer */
#define KEY_MAIL            0xEC    /* Mail */
#define KEY_MEDIASELECT     0xED    /* Media Select */

//  
#define KEY_LMOUSE			0xEE
#define KEY_RMOUSE			0xEF
#define KEY_MMOUSE			0xF0

#define KEY_MAX				0xF1

//       
class GMemWriter : public CMagicStreamMemory
{
public:
	GMemWriter() : CMagicStreamMemory()
	{
		Open(NULL, 0, MAGIC_STREAM_WRITE);
	}

	//   
	void Start()
	{
		position=0;
	}

	//   
	void Finish(MEM_DATA& buf)
	{
		buf.data=(unsigned char*)GetBuffer();
		buf.length=GetPosition();
	}

	//   
	void SetPositionIndirect(unsigned int position)
	{
		this->position=position;
	}

	//   
	void SetLengthIndirect(unsigned int length)
	{
		this->length=length;
	}
};

//        
class GMemReader : public CMagicStreamMemory
{
public:
	GMemReader() : CMagicStreamMemory()
	{
		Open((const char*)0x1, 1, MAGIC_STREAM_READ);
	}

	//   
	void Start(void* ptr)
	{
		buffer=(char*)ptr;
		length=10000000;
		max_length=10000000;
		position=0;
	}

	//   
	void Finish(MEM_DATA& buf)
	{
		buf.data=(unsigned char*)GetBuffer();
		buf.length=GetPosition();
	}
};

struct FILE_DATA : public MEM_DATA
{
	FILE_DATA();
	virtual ~FILE_DATA();

	void Clear();
	void Create(int len);
	void DetachData();
};

struct TIMER_DATA
{
	GComponent* owner;
	unsigned int id;
	unsigned int period;
	void* timer;

	TIMER_DATA()
	{
		Clear();
	}

	void Clear()
	{
		owner=NULL;
		id=0;
		period=0;
		timer=NULL;
	}
};

struct FINDER_LIST
{
	int k_finder;
	FINDER_DATA** m_finder;

	FINDER_LIST();
	virtual ~FINDER_LIST();

	void Add(FINDER_DATA* finder);
	void Delete();

	FINDER_DATA* Get();
};

struct PROCESS_DATA
{
	unsigned char process_info[128];
};

// ,    
class GCriticalSection
{
public:
	GCriticalSection(){}
	virtual ~GCriticalSection(){}

	virtual void Lock()=0;
	virtual void Unlock()=0;
};

// ,   
#define MAX_DIRECTORY_PATH 2048
class GPlatform
{
protected:
	GGame* owner;
	CMagicString game_name;
	//CMagicString user;

	CMagicString game_path;
	CMagicString temp_path;
	CMagicString start_path;
	
	CMagicString current_directory;

	CMagicString wchar_t8;

	unsigned char* str_utf8;
	int str_utf8_len;
	unsigned short* str_utf16;
	int str_utf16_len;
	unsigned int* str_utf32;
	int str_utf32_len;

	FINDER_LIST finders;

	int k_drive;
	GVirtualDrive** m_drive;
	GVirtualDrive* current_virtual_drive;
	int current_virtual_drive_index;
	//VIRTUAL_FIND_ITEM fd_drive;

public:
	GPlatform(GGame* owner, const char* game_name);
	virtual ~GPlatform();

	// 
	virtual void Create(){}

	//    
	bool DetectFirstLaunch();

	//   
	//void SetUserName(const char* user_name);
	//   
	//const char* GetUserName(){return user.c_str();}

	//  ,     
	virtual void Play()=0;

	// ,       
	virtual void Quit()=0;

	//   
	virtual void SetTimer(TIMER_DATA* data)=0;
	virtual void KillTimer(TIMER_DATA* data)=0;

	//   
	virtual bool IsActivated()=0;
	//  ,   
	virtual bool IsIconic()=0;

	//  
	virtual void ToIconic()=0;

	//    
	virtual void ToMaximize()=0;

	// eng: Returns time in milliseconds
	// rus:    
	virtual unsigned int GetTick()=0;

	// eng: Returns current time
	// rus:   
	virtual void GetCurrentDate(CMagicTime* date)=0;

	// eng: Returns path to working folder
	// rus:     
	const char* GetPathToStart(){return start_path;}

	// eng: Returns path to folder with game
	// rus:      
	virtual const char* GetPathToGame()=0;

	// eng: Returns path to folder which could be used by wrapper to store temporary files
	// rus:     
	virtual const char* GetPathToTemp()=0;

	// eng: Returns path to folder with objects
	// rus:      
	virtual const char* GetPathToObject();

	// eng: Returns path to folder with images
	// rus:      
	virtual const char* GetPathToImage();

	// eng: Returns path to folder with fonts
	// rus:      
	virtual const char* GetPathToFont();

	// eng: Returns path to folder with users
	// rus:      
	virtual const char* GetPathToUsers();

	// eng: Returns path to folder with user profile
	// rus:       
	virtual const char* GetPathToProfile();

	// eng: Returns path to folder with saved games
	// rus:       
	virtual const char* GetPathToSavedGame();

	// eng: Returns path to folder with music
	// rus:      
	virtual const char* GetPathToMusic();

	// eng: Returns path to folder with sounds
	// rus:      
	virtual const char* GetPathToSound();

	// eng: Returns path to folder with voices
	// rus:      
	virtual const char* GetPathToVoice();

	// eng: Returns path to folder with maps
	// rus:      
	virtual const char* GetPathToMaps();

	// eng: Returns path to folder with downloaded maps
	// rus:       
	const char* GetPathToDownloadedMaps();

	// eng: Returns path to folder with configuration
	// rus:      
	virtual const char* GetPathToConfig();

	// eng: Returns path to folder with initialization data
	// rus:       
	virtual const char* GetPathToInit();

	// eng: Returns path to folder with autosave
	// rus:      autosave
	virtual const char* GetPathToAutosave();

	// eng: Returns path to folder with files of virtual disk
	// rus:        
	virtual const char* GetPathToVirtualDrive();
	
	// eng: Finds the first file in folder
	// rus:     
	virtual const char* GetFirstFile(const char* path, const char* ext);
	virtual const char* GetFirstFile_OS(const char* path, const char* ext)=0;

	// eng: Finds the next file folder
	// rus:     
	virtual const char* GetNextFile();
	virtual const char* GetNextFile_OS()=0;

	// eng: Finds the first child folder in folder
	// rus:      
	virtual const char* GetFirstFolder(const char* path);
	virtual const char* GetFirstFolder_OS(const char* path)=0;

	// eng: Finds the next child folder in folder
	// rus:      
	virtual const char* GetNextFolder();
	virtual const char* GetNextFolder_OS()=0;

	// eng: Converts wchar_t string into utf8
	// rus:    wchar_t    utf8
	virtual const char* wchar_to_utf8(const wchar_t* str);

	// eng: Converts utf8 string into wchar_t
	// rus:    utf8    wchar_t
	virtual const wchar_t* utf8_to_wchar(const char* str);

	// eng: Deleting of file (analogue remove())
	// rus:   ( remove())
	virtual int RemoveFile(const char* file)=0;

	// eng: Returns file length or -1
	// rus:     -1,   
	virtual int GetFileLength(const char* file)=0;

	// eng: Creating of folder
	// rus:  
	bool CreateFolder(const char* folder);
	virtual bool CreateFolder_OS(const char* folder)=0;

	// eng: Deleting of folder
	// rus:  
	bool RemoveFolder(const char* folder);
	virtual bool RemoveFolder_OS(const char* folder)=0;

	// eng: Converting path to absolute path
	// rus:      
	const char* PathToAbsolutePath(const char* path);

	//  
	bool CreatePath(CMagicString& path);

	// eng: Loading of file
	// rus:  
	virtual bool LoadFile(const char* file, FILE_DATA* file_data);

	// eng: Saving of file
	// rus:  
	virtual bool SaveFile(const char* file, FILE_DATA* file_data);

	// eng: Opening of file
	// rus:  
	virtual FILE* OpenFile(const char* file, const char* mode);

	// eng: Closing of file
	// rus:  
	virtual bool CloseFile(FILE* file);

	// eng: Opening of stream
	// rus:  
	CMagicStream* OpenStream(const char* file, int mode=MAGIC_STREAM_READ);

	// eng: Copying of file
	// rus:  
	bool CopyFile(const char* from, const char* to);

	// eng: Copying of folder
	// rus:  
	bool CopyFolder(const char* from, const char* to, bool create_top_folder=true);

	// eng: Current directory
	// rus:  
	bool SetDirectory(const char* path);
	const char* GetDirectory();
	virtual bool SetDirectory_OS(const char* path)=0;
	virtual const char* GetDirectory_OS()=0;

	//   ,    
	virtual bool IsPathAbsolute_OS(CMagicString& path)=0;

	//   
	virtual char GetSlash(){return '/';}
	
	// rus:    
	virtual bool ExecuteProcess(const char* path, const char* param, const char* working_dir, PROCESS_DATA* process_id)=0;
	//   
	virtual bool TerminateProcess(PROCESS_DATA* process_id)=0;

	// rus:    
	virtual unsigned int ExecuteThread(unsigned int (*StartAddress)(void*), void* data)=0;

	// rus: 
	virtual void Sleep(unsigned int time)=0;

	// rus:    
	virtual void OpenLink(const char* link)=0;

	// rus:     
	virtual void SetClipboard(CMagicString& str)=0;
	// rus:     
	virtual void GetClipboard(CMagicString& str)=0;

	//    ,   
	virtual bool IsCopyKeyPressed(unsigned int key);
	virtual bool IsCutKeyPressed(unsigned int key);
	virtual bool IsPastKeyPressed(unsigned int key);

	//  
	virtual void LockThread()=0;
	virtual void UnlockThread()=0;
	virtual GCriticalSection* NewCriticalSection()=0;

	//    
	virtual const char* GetExecutedFileExtension()=0;

	//   
	const char* ConnectVirtualDrive(const char* drive_path, const char* drive_name);

	//           
	CMagicString CreateVirtualDrive(const char* path, const char* name);
	//         
	CMagicString OpenVirtualDrive(const char* path);
	//         
	bool CloseVirtualDrive(const char* path);
	//        
	int GetVirtualDriveIndexByName(const char* file_name);

	//   
	virtual void CaptureMouse(){}
	//   
	virtual void ReleaseMouse(){}

protected:
	//   ,    
	GVirtualDrive* GetVirtualDrive(const char* path);

	// rus:    UTF16    UTF8
	const unsigned char* Magic_UTF16to8(const unsigned short* str);

	// rus:    UTF32    UTF8
	const unsigned char* Magic_UTF32to8(const unsigned int* str);

	// rus:    UTF8    UTF16
	const unsigned short* Magic_UTF8to16(const unsigned char* str);

	// rus:    UTF8    UTF32
	const unsigned int* Magic_UTF8to32(const unsigned char* str);
};

//    
enum COMPONENT_ALIGN_TEXT {ALGN_LEFT, ALGN_CENTER, ALGN_RIGHT};

//  
#define GS_VISIBLE	1
#define GS_DISABLED	2
#define GS_BORDER	4
#define GS_CAPTION	8
#define GS_NOBGND	0x10
#define GS_USER1	0x20
#define GS_USER2	0x40

//  
class GComponent
{
protected:
	GThemeComponent* theme;

	GComponent* parent;

	unsigned int id;
	int x,y;
	int width, height;

	unsigned int style;
	COMPONENT_ALIGN_TEXT align;

	CMagicString text;

	bool fixed;

	int k_child;
	GComponent** m_child;

	bool invalidated;
	CMagicRect invalidate_rect;

	GComponent* previous_modal;
	bool is_added_modal;
	GComponent* previous_focus;

	int user;

	int k_covering;
	int max_covering;
	GComponent** m_covering;

	int font;							//  

	unsigned int bg_color;				//  
	unsigned int bg_color_disabled;		//     disabled
	unsigned int bg_color_highlighted;	//     
	unsigned int bg_color_pressed;		//     

	unsigned int text_color;			//  
	unsigned int text_color_disabled;	//     disabled
	unsigned int text_color_highlighted;//     
	unsigned int text_color_pressed;	//     

	unsigned int border_color;			//  
	unsigned int border_color_disabled;	//     disabled
	unsigned int border_color_highlighted;//     
	unsigned int border_color_pressed;	//     

	unsigned int caption_bg_color;				//   
	unsigned int caption_bg_color_disabled;		//      disabled
	unsigned int caption_bg_color_highlighted;	//      
	unsigned int caption_bg_color_pressed;		//      

	unsigned int caption_text_color;			//   
	unsigned int caption_text_color_disabled;	//      disabled
	unsigned int caption_text_color_highlighted;//      
	unsigned int caption_text_color_pressed;	//      


public:
	static GGame* game;

public:
	GComponent();
	virtual ~GComponent();

	//   
	GThemeComponent* GetThemeComponent(){return theme;}

	//  
	unsigned int GetStyle(){return style;}
	void SetStyle(unsigned int style){this->style=style;}

	//  
	COMPONENT_ALIGN_TEXT GetAlignText(){return align;}
	virtual bool SetAlignText(COMPONENT_ALIGN_TEXT align);

	int GetFont(){return font;}
	bool SetFont(int font);

	//  
	unsigned int GetBgColor(){return bg_color;}
	void SetBgColor(unsigned int color){bg_color=color;}
	unsigned int GetBgColorDisabled(){return bg_color_disabled;}
	void SetBgColorDisabled(unsigned int color){bg_color_disabled=color;}
	unsigned int GetBgColorHighlighted(){return bg_color_highlighted;}
	void SetBgColorHighlighted(unsigned int color){bg_color_highlighted=color;}
	unsigned int GetBgColorPressed(){return bg_color_pressed;}
	void SetBgColorPressed(unsigned int color){bg_color_pressed=color;}
	
	//  
	unsigned int GetTextColor(){return text_color;}
	void SetTextColor(unsigned int color){text_color=color;}
	unsigned int GetTextColorDisabled(){return text_color_disabled;}
	void SetTextColorDisabled(unsigned int color){text_color_disabled=color;}
	unsigned int GetTextColorHighlighted(){return text_color_highlighted;}
	void SetTextColorHighlighted(unsigned int color){text_color_highlighted=color;}
	unsigned int GetTextColorPressed(){return text_color_pressed;}
	void SetTextColorPressed(unsigned int color){text_color_pressed=color;}

	//  
	unsigned int GetBorderColor(){return border_color;}
	void SetBorderColor(unsigned int color){border_color=color;}
	unsigned int GetBorderColorDisabled(){return border_color_disabled;}
	void SetBorderColorDisabled(unsigned int color){border_color_disabled=color;}
	unsigned int GetBorderColorHighlighted(){return border_color_highlighted;}
	void SetBorderColorHighlighted(unsigned int color){border_color_highlighted=color;}
	unsigned int GetBorderColorPressed(){return border_color_pressed;}
	void SetBorderColorPressed(unsigned int color){border_color_pressed=color;}

	//   
	unsigned int GetCaptionBgColor(){return caption_bg_color;}
	void SetCaptionBgColor(unsigned int color){caption_bg_color=color;}
	unsigned int GetCaptionBgColorDisabled(){return caption_bg_color_disabled;}
	void SetCaptionBgColorDisabled(unsigned int color){caption_bg_color_disabled=color;}
	unsigned int GetCaptionBgColorHighlighted(){return caption_bg_color_highlighted;}
	void SetCaptionBgColorHighlighted(unsigned int color){caption_bg_color_highlighted=color;}
	unsigned int GetCaptionBgColorPressed(){return caption_bg_color_pressed;}
	void SetCaptionBgColorPressed(unsigned int color){caption_bg_color_pressed=color;}

	//   
	unsigned int GetCaptionTextColor(){return caption_text_color;}
	void SetCaptionTextColor(unsigned int color){caption_text_color=color;}
	unsigned int GetCaptionTextColorDisabled(){return caption_text_color_disabled;}
	void SetCaptionTextColorDisabled(unsigned int color){caption_text_color_disabled=color;}
	unsigned int GetCaptionTextColorHighlighted(){return caption_text_color_highlighted;}
	void SetCaptionTextColorHighlighted(unsigned int color){caption_text_color_highlighted=color;}
	unsigned int GetCaptionTextColorPressed(){return caption_text_color_pressed;}
	void SetCaptionTextColorPressed(unsigned int color){caption_text_color_pressed=color;}

	//       
	void CopyAttributesTo(GComponent* destination);


	//  
	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id);

	bool IsCreated(){return (id!=0xffffffff);}

	void GetPosition(int& x, int& y){x=this->x; y=this->y;}
	void GetSize(int& wi, int& he){wi=width; he=height;}

	void SetPosition(int x, int y){this->x=x; this->y=y; RecalcLayout();}
	void SetSize(int wi, int he){width=wi; height=he; RecalcLayout();}

	//    ,       
	void SetClientSize(int wi, int he);

	virtual void SetWindowText(const char* text);
	virtual const char* GetWindowText(){return text.c_str();}
	int GetWindowTextLength(){return text.GetLength();}

	//   
	void AddChild(GComponent* child);
	//   
	void DeleteChild(GComponent* child, bool del_child=true);

	bool IsInvalidated(){return invalidated;}

	//      
	virtual void OnUpdateModal(){}

protected:
	//    
	void ClearLinks();

public:
	//  item-   
	int Find(GComponent* child);
	GComponent* Find(unsigned int id);

	//     
	void AlignToCenter();

	//     
	void AlignToCenterMonitor();

	unsigned int GetID(){return id;}
	void SetID(unsigned int id){this->id=id;}

	GComponent* GetParent(){return parent;}

	int GetChildCount(){return k_child;}
	GComponent* GetChild(int index){return m_child[index];}

	void SetUserData(int data){user=data;}
	int GetUserData(){return user;}

	//  
	void SetFocus();

	//       
	virtual void GetInnerRect(CMagicRect* rect);

	//    
	virtual void GetInnerOffset(int& x, int& y);

	//      
	virtual void GetClientOffset(int& x, int& y);

	//    
	virtual void GetClientRect(CMagicRect* rect);

	//    
	void GetAbsoluteWindowPosition(int& x, int& y);

	//    
	virtual void Invalidate();

	// /    
	virtual void InvalidateRect(CMagicRect& rect);

	//       SysOnRender()
	virtual void Draw();

	//     
	GComponent* DetectComponentByCursorRecurse(int& x, int& y, int& place);

	//  ,       
	virtual bool IsOver(int x, int y);

	//  
	virtual bool IsVisibled();
	virtual void Show(bool visibled);

	//  
	virtual bool IsEnabled();
	virtual void Enable(bool enable);

	//   
	virtual bool IsFixed(){return fixed;}
	virtual void Fix(bool fix);

	//  ,     - ,    
	virtual bool IsHighlighted();

	//  ,   
	virtual bool IsPressed();

	//  ,     
	virtual bool IsFocused();

	//  ,    
	virtual bool IsModal();

	//     
	virtual void BeginModal();
	//     
	virtual void EndModal();

	//  
	virtual bool IsButton(){return false;}

	//   
	virtual bool IsStatic(){return false;}

	//   
	virtual bool IsEdit(){return false;}

	//  
	virtual bool IsListBox(){return false;}

	//  
	virtual bool IsTree(){return false;}

	//    
	virtual bool IsDialog(){return false;}

	//  
	virtual bool IsScene(){return false;}

	//  
	virtual bool IsPicture(){return false;}

	//  
	virtual bool IsSinglePicture(){return false;}

	//  GDiscretBox-
	virtual bool IsDiscretBox(){return false;}

	//  
	virtual bool IsCheckBox(){return false;}

	//  
	virtual bool IsMenu(){return false;}

	//      
	virtual GComponent* CorrectPositionClientForCaptured(int& x, int& y);

	//      
	virtual GComponent* CorrectPositionClientForModal(int& x, int& y);

	//    item
	virtual GItem* GetFirstVisibledItem(){return NULL;}
	//    item
	virtual GItem* GetNextVisibledItem(){return NULL;}

	//   
	virtual GImageList* GetIcons(){return NULL;}

	//  
	virtual void RecalcLayout(){}
	//  ,    
	virtual bool IsRecalcLayout(){return false;}

	GComponent* GetPreviousModal(){return previous_modal;}
	void SetPreviousModal(GComponent* modal){previous_modal=modal;}

	//       , , ,    
	void DetectCovering(int& k_component, GComponent** m_component);
	//   
	void ClearCoveringComponents();
	//   
	void AddCoveringComponent(GComponent* component);
	//     
	void RecalcCovering();
	//    ,   
	void InvalidateCoveringComponents();


	//        
	void CorrectSize(CMagicRect* rect);

	//        
	void CorrectSize(int gap);

	//            
	void ChildTo(int index);

	//  item-
	virtual void RenderItem(GItem* item);

	//  
	// -------
	virtual void OnRenderWIN(){}
	virtual bool WinOnRender();

	virtual void OnRenderINNER(){}
	virtual bool InnerOnRender(){OnRenderINNER(); return true;}

	virtual void OnRender(){}
	virtual void ClientOnRender(){OnRender();}

	virtual void OnEraseInner(CMagicRect* rect);
	virtual void OnEraseClient();

	// -------

	virtual void OnLButtonDownWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnLButtonDown(unsigned int nFlags, int x, int y){OnLButtonDownWIN(nFlags, x, y);}

	virtual void OnLButtonDownINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnLButtonDown(unsigned int nFlags, int x, int y){OnLButtonDownINNER(nFlags, x, y);}

	virtual void OnLButtonDown(unsigned int nFlags, int x, int y){}
	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y){OnLButtonDown(nFlags, x, y);}

	// -------

	virtual void OnLButtonUpWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnLButtonUp(unsigned int nFlags, int x, int y){OnLButtonUpWIN(nFlags, x, y);}

	virtual void OnLButtonUpINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnLButtonUp(unsigned int nFlags, int x, int y){OnLButtonUpINNER(nFlags, x, y);}

	virtual void OnLButtonUp(unsigned int nFlags, int x, int y){}
	virtual void ClientOnLButtonUp(unsigned int nFlags, int x, int y){OnLButtonUp(nFlags, x, y);}

	// -------

	virtual void OnLButtonClickWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnLButtonClick(unsigned int nFlags, int x, int y){OnLButtonClickWIN(nFlags, x, y);}

	virtual void OnLButtonClickINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnLButtonClick(unsigned int nFlags, int x, int y){OnLButtonClickINNER(nFlags, x, y);}

	virtual void OnLButtonClick(unsigned int nFlags, int x, int y){}
	virtual void ClientOnLButtonClick(unsigned int nFlags, int x, int y){OnLButtonClick(nFlags, x, y);}

	// -------

	virtual void OnLButtonDblClkWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnLButtonDblClk(unsigned int nFlags, int x, int y){OnLButtonDblClkWIN(nFlags, x, y);}

	virtual void OnLButtonDblClkINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnLButtonDblClk(unsigned int nFlags, int x, int y){OnLButtonDblClkINNER(nFlags, x, y);}

	virtual void OnLButtonDblClk(unsigned int nFlags, int x, int y){}
	virtual void ClientOnLButtonDblClk(unsigned int nFlags, int x, int y){OnLButtonDblClk(nFlags, x, y);}
	
	// -------

	virtual void OnRButtonDownWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnRButtonDown(unsigned int nFlags, int x, int y){OnRButtonDownWIN(nFlags, x, y);}

	virtual void OnRButtonDownINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnRButtonDown(unsigned int nFlags, int x, int y){OnRButtonDownINNER(nFlags, x, y);}

	virtual void OnRButtonDown(unsigned int nFlags, int x, int y){}
	virtual void ClientOnRButtonDown(unsigned int nFlags, int x, int y){OnRButtonDown(nFlags, x, y);}

	// -------

	virtual void OnRButtonUpWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnRButtonUp(unsigned int nFlags, int x, int y){OnRButtonUpWIN(nFlags, x, y);}

	virtual void OnRButtonUpINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnRButtonUp(unsigned int nFlags, int x, int y){OnRButtonUpINNER(nFlags, x, y);}

	virtual void OnRButtonUp(unsigned int nFlags, int x, int y){}
	virtual void ClientOnRButtonUp(unsigned int nFlags, int x, int y){OnRButtonUp(nFlags, x, y);}

	// -------

	virtual void OnRButtonClickWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnRButtonClick(unsigned int nFlags, int x, int y){OnRButtonClickWIN(nFlags, x, y);}

	virtual void OnRButtonClickINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnRButtonClick(unsigned int nFlags, int x, int y){OnRButtonClickINNER(nFlags, x, y);}

	virtual void OnRButtonClick(unsigned int nFlags, int x, int y){}
	virtual void ClientOnRButtonClick(unsigned int nFlags, int x, int y){OnRButtonClick(nFlags, x, y);}

	// -------

	virtual void OnRButtonDblClkWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnRButtonDblClk(unsigned int nFlags, int x, int y){OnRButtonDblClkWIN(nFlags, x, y);}

	virtual void OnRButtonDblClkINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnRButtonDblClk(unsigned int nFlags, int x, int y){OnRButtonDblClkINNER(nFlags, x, y);}

	virtual void OnRButtonDblClk(unsigned int nFlags, int x, int y){}
	virtual void ClientOnRButtonDblClk(unsigned int nFlags, int x, int y){OnRButtonDblClk(nFlags, x, y);}

	// -------

	virtual void OnMButtonDownWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnMButtonDown(unsigned int nFlags, int x, int y){OnMButtonDownWIN(nFlags, x, y);}

	virtual void OnMButtonDownINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnMButtonDown(unsigned int nFlags, int x, int y){OnMButtonDownINNER(nFlags, x, y);}

	virtual void OnMButtonDown(unsigned int nFlags, int x, int y){}
	virtual void ClientOnMButtonDown(unsigned int nFlags, int x, int y){OnMButtonDown(nFlags, x, y);}

	// -------

	virtual void OnMButtonUpWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnMButtonUp(unsigned int nFlags, int x, int y){OnMButtonUpWIN(nFlags, x, y);}

	virtual void OnMButtonUpINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnMButtonUp(unsigned int nFlags, int x, int y){OnMButtonUpINNER(nFlags, x, y);}

	virtual void OnMButtonUp(unsigned int nFlags, int x, int y){}
	virtual void ClientOnMButtonUp(unsigned int nFlags, int x, int y){OnMButtonUp(nFlags, x, y);}

	// -------

	virtual void OnMButtonClickWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnMButtonClick(unsigned int nFlags, int x, int y){OnMButtonClickWIN(nFlags, x, y);}

	virtual void OnMButtonClickINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnMButtonClick(unsigned int nFlags, int x, int y){OnMButtonClickINNER(nFlags, x, y);}

	virtual void OnMButtonClick(unsigned int nFlags, int x, int y){}
	virtual void ClientOnMButtonClick(unsigned int nFlags, int x, int y){OnMButtonClick(nFlags, x, y);}

	// -------

	virtual void OnMButtonDblClkWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnMButtonDblClk(unsigned int nFlags, int x, int y){OnMButtonDblClkWIN(nFlags, x, y);}

	virtual void OnMButtonDblClkINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnMButtonDblClk(unsigned int nFlags, int x, int y){OnMButtonDblClkINNER(nFlags, x, y);}

	virtual void OnMButtonDblClk(unsigned int nFlags, int x, int y){}
	virtual void ClientOnMButtonDblClk(unsigned int nFlags, int x, int y){OnMButtonDblClk(nFlags, x, y);}

	// -------

	virtual void OnMouseWheelWIN(short zDelta){}
	virtual bool WinOnMouseWheel(short zDelta){OnMouseWheelWIN(zDelta); return true;}

	virtual void OnMouseWheelINNER(short zDelta){}
	virtual bool InnerOnMouseWheel(short zDelta){OnMouseWheelINNER(zDelta); return true;}

	virtual void OnMouseWheel(short zDelta){}
	virtual void ClientOnMouseWheel(short zDelta){OnMouseWheel(zDelta);}

	// -------

	virtual void OnMouseMoveWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnMouseMove(unsigned int nFlags, int x, int y){OnMouseMoveWIN(nFlags, x, y);}

	virtual void OnMouseMoveINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnMouseMove(unsigned int nFlags, int x, int y){OnMouseMoveINNER(nFlags, x, y);}

	virtual void OnMouseMove(unsigned int nFlags, int x, int y){}
	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y){OnMouseMove(nFlags, x, y);}

	
	virtual void OnKeyDownWIN(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual bool WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnKeyDownWIN(nChar, nRepCnt, nFlags); return true;}

	virtual void OnKeyDownINNER(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual bool InnerOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnKeyDownINNER(nChar, nRepCnt, nFlags); return true;}

	virtual void OnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual void ClientOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnKeyDown(nChar, nRepCnt, nFlags);}

	// -------

	virtual void OnKeyUpWIN(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual bool WinOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnKeyUpWIN(nChar, nRepCnt, nFlags); return true;}

	virtual void OnKeyUpINNER(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual bool InnerOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnKeyUpINNER(nChar, nRepCnt, nFlags); return true;}

	virtual void OnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual void ClientOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnKeyUp(nChar, nRepCnt, nFlags);}

	// -------

	virtual void OnCharWIN(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual bool WinOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnCharWIN(nChar, nRepCnt, nFlags); return true;}

	virtual void OnCharINNER(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual bool InnerOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnCharINNER(nChar, nRepCnt, nFlags); return true;}

	virtual void OnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){}
	virtual void ClientOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags){OnChar(nChar, nRepCnt, nFlags);}

	// -------

	virtual GImage* OnSetCursorWIN(unsigned int nFlags, int x, int y){return NULL;}
	
	virtual GImage* OnSetCursorINNER(unsigned int nFlags, int x, int y){return NULL;}
	
	virtual GImage* OnSetCursor(unsigned int nFlags, int x, int y){return NULL;}

	// -------

	virtual void OnMouseEnterGLOBAL(unsigned int nFlags, int x, int y){}
	virtual void GlobalOnMouseEnter(unsigned int nFlags, int x, int y){OnMouseEnter(nFlags, x, y);}

	virtual void OnMouseEnterWIN(unsigned int nFlags, int x, int y){}
	virtual void WinOnMouseEnter(unsigned int nFlags, int x, int y){OnMouseEnterWIN(nFlags, x, y);}

	virtual void OnMouseEnterINNER(unsigned int nFlags, int x, int y){}
	virtual void InnerOnMouseEnter(unsigned int nFlags, int x, int y){OnMouseEnterINNER(nFlags, x, y);}

	virtual void OnMouseEnter(unsigned int nFlags, int x, int y){}
	virtual void ClientOnMouseEnter(unsigned int nFlags, int x, int y){OnMouseEnter(nFlags, x, y);}

	// -------

	virtual void OnMouseLeaveGLOBAL(unsigned int nFlags){}
	virtual void GlobalOnMouseLeave(unsigned int nFlags){OnMouseLeave(nFlags);}

	virtual void OnMouseLeaveWIN(unsigned int nFlags){}
	virtual void WinOnMouseLeave(unsigned int nFlags){OnMouseLeaveWIN(nFlags);}

	virtual void OnMouseLeaveINNER(unsigned int nFlags){}
	virtual void InnerOnMouseLeave(unsigned int nFlags){OnMouseLeaveINNER(nFlags);}

	virtual void OnMouseLeave(unsigned int nFlags){}
	virtual void ClientOnMouseLeave(unsigned int nFlags){OnMouseLeave(nFlags);}

	// -------

	virtual void OnSetFocus(){}
	virtual void SysOnSetFocus(){OnSetFocus();}
	
	virtual void OnKillFocus(){}
	virtual void SysOnKillFocus(){OnKillFocus();Invalidate();}
	
	//   
	virtual void OnFocusTimer(unsigned int tick){}
	virtual void SysOnFocusTimer(unsigned int tick){OnFocusTimer(tick);}//Invalidate();}

	//   
	virtual void OnChangeText(){}
	virtual void SysOnChangeText(){OnChangeText();}

	//   
	virtual void OnTimer(unsigned int id){}

	// -------

	virtual void OnCopy(){}
	virtual void SysOnCopy(){OnCopy();}

	virtual void OnCut(){}
	virtual void SysOnCut(){OnCut();}

	virtual void OnPast(){}
	virtual void SysOnPast(){OnPast();}
};

#define GSBS_OK		0x400
#define GSBS_CANCEL	0x800

class GButton : public GComponent
{
public:
	GButton();
	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text);

	//  
	virtual bool IsButton(){return true;}

	virtual bool InnerOnRender();
	virtual void ClientOnRender();

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnLButtonUp(unsigned int nFlags, int x, int y);
	virtual void GlobalOnMouseEnter(unsigned int nFlags, int x, int y);
	virtual void GlobalOnMouseLeave(unsigned int nFlags);

	virtual void ClientOnLButtonClick(unsigned int nFlags, int x, int y);
};

class GButtonImage : public GButton
{
protected:
	GImageList list;
public:
	GButtonImage();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, GSurface* normal, GSurface* pressed=NULL, GSurface* highlight=NULL, GSurface* disabled=NULL);

	GImageList* GetImageList(){return &list;}
};

class GCheckBox : public GComponent
{
protected:
	bool checked;

public:
	GCheckBox();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, unsigned int id, const char* text);

	virtual void SetFont(int font);

	virtual void ClientOnRender();

	//  
	virtual bool IsCheckBox(){return true;}

	virtual void SetWindowText(const char* text);

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void GlobalOnMouseEnter(unsigned int nFlags, int x, int y);
	virtual void GlobalOnMouseLeave(unsigned int nFlags);

	bool IsChecked(){return checked;}
	void SetCheck(bool check);

protected:
	//   
	void RecalcSize();
};

class CCButtonImage : public GComponent
{
protected:
	int font;
	int tx, ty;
	int sx_fixed, sy_fixed;

	GImage* normal_image;
	GImage* fixed_image;
	GImage* disabled_image;
	GImage* over_image;

	bool is_over;

public:
	CCButtonImage();
	bool Create(GComponent* parent, int x, int y, unsigned int id, const char* text, int font, GImage* normal_image, GImage* fixed_image=NULL, GImage* disabled_image=NULL, GImage* over_image=NULL);

	//  
	virtual bool IsButton(){return true;}

	//        
	void SetTextPosition(int tx=DEFAULT_POSITION, int ty=DEFAULT_POSITION, int sx_fixed=DEFAULT_POSITION, int sy_fixed=DEFAULT_POSITION);

	//  ,       
	virtual bool IsOver(int x, int y);

	virtual void ClientOnRender();

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnLButtonUp(unsigned int nFlags, int x, int y);
	virtual void GlobalOnMouseEnter(unsigned int nFlags, int x, int y);
	virtual void GlobalOnMouseLeave(unsigned int nFlags);

protected:
	//   
	GImage* GetCurrentPicture();
};

#define SCROLLER_NO			0	//     
#define SCROLLER_PLACE		1	//    ,      
#define SCROLLER_HBUTTON1	2	//     1   
#define SCROLLER_HBUTTON2	3	//     2   
#define SCROLLER_HSLIDER	4	//       
#define SCROLLER_VBUTTON1	5	//     1   
#define SCROLLER_VBUTTON2	6	//     2   
#define SCROLLER_VSLIDER	7	//       

class GScroller : public GComponent
{
protected:
	int scroll_x, scroll_y;			//  
	int max_scroll_x, max_scroll_y;	//   

	double factor_hscroll, factor_vscroll;	//  
	int hslider_size, vslider_size;	//  ,      

	unsigned int normal_color, pressed_color, over_color, slider_color;

	int captured_scroller;
	int over_scroller;

	int slider_x, slider_y;	//     

	bool is_recalc;			//  ,    RecalcLayout()  

public:
	GScroller();

	void SetScrollBarColor(unsigned int normal, unsigned int pressed, unsigned int over, unsigned int slider);

	//  
	void GetScrollPosition(int& x, int& y){x=scroll_x; y=scroll_y;}

	//    
	virtual void GetClientRect(CMagicRect* rect);

	//   ,     
	virtual void GetScrollSize(int& wi, int& he)=0;

	//        
	int DetectScrollElement(int x, int y);

	int GetCapturedScroller(){return captured_scroller;}
	int GetOverScroller(){return over_scroller;}

	//  
	virtual void RecalcLayout();
	//  ,    
	virtual bool IsRecalcLayout(){return is_recalc;}

	virtual bool InnerOnRender();

	virtual void InnerOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void InnerOnLButtonUp(unsigned int nFlags, int x, int y);

	virtual void InnerOnMouseMove(unsigned int nFlags, int x, int y);
	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);

	virtual void InnerOnMouseLeave(unsigned int nFlags);

	virtual bool WinOnMouseWheel(short zDelta);
	
	//      
	bool IsHScrollBar(){return (hslider_size>0);}
	//      
	bool IsVScrollBar(){return (vslider_size>0);}

	//    
	virtual int GetHScrollBarSize();
	virtual int GetVScrollBarSize();

	//    
	virtual int GetMinSlider(){return 10;}

	//        
	virtual int GetHScrollDelta(){return 15;}

	//        
	virtual int GetVScrollDelta(){return 15;}

	//       
	void GetHScrollButton1Rect(CMagicRect* rect);

	//       
	void GetHScrollButton2Rect(CMagicRect* rect);

	//       
	void GetVScrollButton1Rect(CMagicRect* rect);

	//       
	void GetVScrollButton2Rect(CMagicRect* rect);

	//      
	void GetHScrollSliderRect(CMagicRect* rect);

	//      
	void GetVScrollSliderRect(CMagicRect* rect);

	//       
	void GetScrollSquareRect(CMagicRect* rect);

	//      
	void SetHScrollToMax(){scroll_x=max_scroll_x;}
	void SetVScrollToMax(){scroll_y=max_scroll_y;}

	//     
	int GetHScrollMax()
	{
		if (IsHScrollBar())
			return max_scroll_x;
		return 0;
	}
	int GetVScrollMax()
	{
		if (IsVScrollBar())
			return max_scroll_y;
		return 0;
	}

	//   
	void SetHScroll(int pos);
	void SetVScroll(int pos);
};

//  item-  
class GItem
{
protected:
	GListBase* owner;

	GItem* previous;
	GItem* next;

	int width, height;
	int user;
	void* user_struct;

	bool enabled;

	int y;

	double sort;

public:
	GItem(int width, int height);
	virtual ~GItem();

	virtual GItem* GetParent(){return NULL;}

	virtual GImage* GetIcon();
	virtual int GetIconIndex(){return -1;}

	virtual bool HasChildren(){return false;}
	virtual bool IsOpened(){return false;}

	virtual GItem* GetPreviousSibling(){return previous;}
	virtual GItem* GetNextSibling(){return next;}

	virtual unsigned int GetTextColor(){return 0;}
	virtual int GetTextFont(){return 0;}

	void SetOwner(GListBase* owner){this->owner=owner;}
	GListBase* GetOwner(){return owner;}
	
	void GetItemSize(int& wi, int& he){wi=width; he=height;}
	virtual void GetTextSize(int& wi, int& he){wi=0; he=0;}

	void GetItemPosition(int& x, int& y);
	virtual void GetItemRect(CMagicRect* rect);

	//   item-    ,  
	//    
	//void GetItemThemeRect(CMagicRect* rect);

	//   item-   
	void GetItemThemeSize(int& wi, int& he);

	virtual bool IsEnabled();
	virtual void Enable(bool enable);

	//  Check-
	virtual bool IsChecked(){return false;}

	virtual bool IsOver(int& x, int& y);

	//   
	virtual int GetChildLevel(){return 0;}

	GItem* GetPrevious(){return previous;}
	void SetPrevious(GItem* item){previous=item;}
	GItem* GetNext(){return next;}
	void SetNext(GItem* item){next=item;}
	int GetY(){return y;}
	void SetY(int y){this->y=y;}

	int GetWidth(){return width;}
	void SetWidth(int wi){width=wi;}
	int GetHeight(){return height;}
	void SetHeight(int he){height=he;}

	virtual void OnRender(){};

	//    item-e
	virtual bool SysOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void OnLButtonDown(unsigned int nFlags, int x, int y){}
	virtual bool SysOnLButtonDblClk(unsigned int nFlags, int x, int y);
	virtual void OnLButtonDblClk(unsigned int nFlags, int x, int y){}

	virtual bool SysOnRButtonDown(unsigned int nFlags, int x, int y){OnRButtonDown(nFlags, x, y); return false;}
	virtual void OnRButtonDown(unsigned int nFlags, int x, int y){}
	virtual bool SysOnRButtonDblClk(unsigned int nFlags, int x, int y){OnRButtonDblClk(nFlags, x, y); return false;}
	virtual void OnRButtonDblClk(unsigned int nFlags, int x, int y){}

	//  ,   item 
	bool IsSelected();

	void SetUserData(int data){user=data;}
	int GetUserData(){return user;}

	void SetUserStruct(int length, void* data);
	void* GetUserStruct(){return user_struct;}

	virtual const char* GetText(){return NULL;}

	//     SortByField();
	virtual double GetSort(){return sort;}
	virtual void SetSort(double sort){this->sort=sort;}
};


//  
class GListBase : public GScroller
{
protected:
	GItem* first;
public:
	GItem* selected;

protected:
	GItem* visibled;
	int visibled_scroll_y;
	int visibled_y;

	int item_width, item_height;	//    item-
	int tab; //    

	GItem* found_item;
	int found_scroll_y2;

	GImageList* icons;

	//       
	GItem* pushed_scroll_item;
	int pushed_scroll_x, pushed_scroll_y;

public:
	GListBase();
	virtual ~GListBase();

	//  item-
	void ClearItems();

	//   ,     
	virtual void GetScrollSize(int& wi, int& he){wi=item_width; he=item_height;}

	//  ,    
	virtual bool IsRecalcLayout();

	//  
	virtual void RecalcLayout();

	//  item
	virtual void SetSelected(GItem* item){selected=item;}
	virtual GItem* GetSelected(){return selected;}

	//      
	int GetTab(){return tab;}

	//   item-
	void InsertItem(GItem* item, GItem* previous);
	//  item-
	void DeleteItem(GItem* item);

	//    item
	virtual GItem* GetFirstVisibledItem();
	//    item
	virtual GItem* GetNextVisibledItem();

	//  item-  
	GItem* DetectItemByCursor(int& x, int& y);

	//   item-
	GItem* GetFirstItem(){return first;}
	//   item-
	GItem* GetLatestItem();

	//   item-
	int GetItemCount();

	//   Y ,   item   
	void ItemInClient(GItem* item);

	//   
	virtual GImageList* GetIcons(){return icons;}

	//  item-  
	void SortByName(bool save_scroller_position=true);

	//  item-   
	void SortByField(bool save_scroller_position=true);

	//    item-
	void Exchange(GItem* item1, GItem* item2);

	//   
	void PushScrollPosition();
	//   
	void PopScrollPosition();
	//  item-,     
	void SetPushedScrollItem(GItem* item){pushed_scroll_item=item;}

protected:
	//    item-
	void DetectFirstVisibledItem();

public:
	//       item- (, , )
	void CorrectingForChanging(GItem* item, int old_wi, int old_he);

public:
	virtual void ClientOnRender();

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnLButtonDblClk(unsigned int nFlags, int x, int y);
	
	virtual void ClientOnRButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnRButtonDblClk(unsigned int nFlags, int x, int y);

	virtual void OnChangeSelected(GItem* item){}
};

class GItemText : public GItem
{
protected:
	CMagicString text;
	int font;
	int color;

	int icon;

	int text_width, text_height;

public:
	GItemText(GListBase* owner, const char* text, int font, unsigned int color);

public:
	virtual int GetIconIndex(){return icon;}
	virtual void SetIconIndex(int icon);

	virtual const char* GetText(){return text.c_str();}
	virtual void SetText(const char* text);
	virtual void GetTextSize(int& wi, int& he){wi=text_width; he=text_height;}

	void SetTextColor(unsigned int color){this->color=color;}

	virtual unsigned int GetTextColor(){return color;}
	virtual int GetTextFont(){return font;}
	virtual void SetTextFont(int font);



	//    item-
	//        
	void RefreshItemSize();

	//  
	void RefreshText(const char* text);
};

class GListBox : public GListBase
{
public:
	GListBox();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, GImageList* icons=NULL);

	GItem* AddString(const char* text, int font=-1, unsigned int color=COLOR_KEY_EMPTY);

	//  
	virtual bool IsListBox(){return true;}

	virtual bool WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

protected:
	GItemText* NewItem(const char* text, int font, unsigned int color);
};


class GItemTree : public GItemText
{
protected:
	int k_child;
	GItem** m_child;

	GItem* parent;

	unsigned int node_color;

	bool checked;

public:
	GItemTree(GListBase* owner, const char* text, int font, unsigned int color, int icon, int width, int height);
	virtual ~GItemTree();

	virtual GItem* GetParent(){return parent;}
	void SetParent(GItem* parent){this->parent=parent;}
	
	int GetChildCount(){return k_child;}
	GItem* GetChild(int index){return m_child[index];}

	//   item
	void AddChild(GItemTree* item);

	//   
	virtual int GetChildLevel();

	//   ,   item- 
	virtual bool HasChildren();

	//   ,   item- 
	virtual bool IsOpened();

	//   item-
	void Open();
	
	//   item-
	void Close();

	//  Check-
	virtual bool IsChecked(){return checked;}
	void SetCheck(bool check){checked=check;}

	//   item-   
	virtual GItem* GetPreviousSibling();

	//   item-   
	virtual GItem* GetNextSibling();

	//     
	void SysOnNode();

protected:
	bool ButtonDown(unsigned int nFlags, int x, int y);
	bool ButtonDblClk(unsigned int nFlags, int x, int y);

public:
	virtual bool SysOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual bool SysOnLButtonDblClk(unsigned int nFlags, int x, int y);

	virtual bool SysOnRButtonDown(unsigned int nFlags, int x, int y);
	virtual bool SysOnRButtonDblClk(unsigned int nFlags, int x, int y);
};

#define MAX_TREE_LEVELS	15

#define GSTR_HASLINES	0x100
#define GSTR_HASBUTTONS	0x200
class GTree : public GListBox
{
public:
	GTree();
	virtual ~GTree();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, GImageList* icons=NULL);

	//  
	virtual bool IsTree(){return true;}

	virtual bool IsListBox(){return false;}

	GItemTree* AddNode(GItemTree* parent, const char* text, int font=-1, unsigned int color=COLOR_KEY_EMPTY, int icon=-1);

	bool HasButtons();
	bool HasLines();

	bool WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

protected:
	virtual GItemTree* NewItem(const char* text, int font, unsigned int color, int icon);
};

class GItemMenu : public GItemTree
{
public:
	GItemMenu(GListBase* owner, const char* text, int font, unsigned int color, int icon, int width, int height);

	virtual void GetItemRect(CMagicRect* rect);
};

class GItemMenuSeparator : public GItemMenu
{
public:
	GItemMenuSeparator(GListBase* owner);

	virtual bool IsEnabled(){return false;}
	virtual void Enable(bool enable){}
};

class GMenu : public GTree
{
protected:
	GItemMenu* parent_item;

public:
	GMenu();

	virtual bool Create(unsigned int style, GImageList* icons=NULL);

	//  
	virtual bool IsMenu(){return true;}

	GItemMenu* AppendMenu(GItemTree* parent, const char* text, int font=-1, unsigned int color=COLOR_KEY_EMPTY, int icon=-1);

	//      item-
	void RecalcSize();

	virtual GItemTree* NewItem(const char* text, int font, unsigned int color, int icon);

public:
	//      
	void SelectPosition(int x, int y);

	//    
	virtual void PopupMenu(int x, int y);

	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);
	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnRButtonDown(unsigned int nFlags, int x, int y);
	virtual void WinOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void WinOnRButtonDown(unsigned int nFlags, int x, int y);

	virtual bool WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

	//   
	virtual void OnSelectMenu(GItemMenu* item){}

	//   
	void OpenChildMenu(GItemMenu* parent_item);

	//   
	void CloseChildMenu();

	//   
	GMenu* GetChildMenu();

	//      
	GComponent* CorrectPositionClientForModal(int& x, int& y);

	//     
	virtual void EndModal();

	//  item
	virtual void SetSelected(GItem* item);
	virtual GItem* GetSelected();
};

// ,     
struct COLUMN_DATA
{
	CMagicString name;
	int width;
	bool center;
};

//  
class GGrid : public GTree
{
	int k_column;
	COLUMN_DATA* m_column;

	int k_row;
	GItemTree** m_row;
public:
	GGrid();
	virtual ~GGrid();

	//  
	void InsertColumn(int index, const char* name, int width, bool center);
	//  
	GItemTree* InsertRow(int index);

	bool SetText(int column, int row, const char* text);
	const char* GetText(int column, int row);

	bool SetIcon(int column, int row, int icon);
	int GetIcon(int column, int row);

	int GetColumnCount(){return k_column;}
	COLUMN_DATA* GetColumn(int index){return &(m_column[index]);}

	int GetRowCount(){return k_row;}

	//      (row=-1)
	void GetCellRect(int column, int row, CMagicRect* rect);
};

#define GSES_OWNRETURN	1000	//  GEdit,  ,   Enter   ,  Enter      
//   
class GEdit : public GComponent
{
protected:
	int text_x;
	int sel_pos1;
	int sel_pos2;

	int cursor;
	bool visibled_cursor;
	unsigned int visibled_cursor_tick;

	bool was_setfocus;

public:
	GEdit();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text);

	//   
	virtual bool IsEdit(){return true;}

	//    
	void GetClientRect(CMagicRect* rect);

	//      
	void CorrectTextX();

	//   X 
	int GetCursorX();

	//       .
	//       .
	bool GetSelPos(int& sel1, int& sel2);

	//  
	void ResetSelPos();
	
	//  
	void CorrectSelPos(int previous_cursor, bool selecting);

	//       
	void DeleteText();

	//    
	void CursorToEnd(bool selecting=false);

	//      X    
	int GetCursorByPosition(int x);

	//   
	void VisibledCursor();

	int GetTextX(){return text_x;}
	bool IsCursorVisibled(){return visibled_cursor;}

	//   
	void SysOnFocusTimer(unsigned int tick);
	
	//  
	//virtual bool InnerOnRender();
	virtual void ClientOnRender();

	virtual void SysOnSetFocus();
	virtual void SysOnKillFocus();

	virtual void ClientOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);
	virtual void ClientOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);
	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnLButtonUp(unsigned int nFlags, int x, int y);
	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);
	virtual void SetWindowText(const char* text);
	virtual void OnEnter(){}

	virtual void SysOnCopy();
	virtual void SysOnCut();
	virtual void SysOnPast();
};

#define CCBUTTON_NORMAL		0
#define CCBUTTON_OVER		1
#define CCBUTTON_PRESSED	2

//   
#define GSCB_DROPDOWN	0x80
class GComboBox : public GEdit
{
protected:
	int list_height;

	int button_state;

	int k_string;
	int t_string;
	GItem** m_string;

public:
	GComboBox();
	virtual ~GComboBox();

	bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, int list_height=0);

	GItem* AddString(const char* text, int font=-1);

	int GetStringCount(){return k_string;}
	GItem* GetString(int index);

	//  
	void SetSel(int index);
	int GetSel(){return t_string;}

	int GetListHeight(){return list_height;}

	//  
	bool IsEditable();

protected:
	GItemText* NewItem(const char* text, int font);

public:
	//      
	virtual void GetClientOffset(int& x, int& y);

	//    
	void GetClientRect(CMagicRect* rect);

	//   
	//void GetButtonRect(CMagicRect* rect);

	//  
	virtual bool InnerOnRender();

	virtual void InnerOnMouseEnter(unsigned int nFlags, int x, int y);
	virtual void InnerOnMouseLeave(unsigned int nFlags);
	
	virtual void ClientOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);
	virtual void ClientOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);
	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void InnerOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void InnerOnLButtonUp(unsigned int nFlags, int x, int y);
	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);

	virtual void OnChangeSel(){};
};

class GListBoxForComboBox : public GListBox
{
public:
	GListBoxForComboBox();
	virtual ~GListBoxForComboBox();

	virtual bool Create(GComboBox* cbox);

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnRButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnMButtonDown(unsigned int nFlags, int x, int y);

	virtual void WinOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void WinOnRButtonDown(unsigned int nFlags, int x, int y);
	virtual void WinOnMButtonDown(unsigned int nFlags, int x, int y);

	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);

	virtual void SysOnKillFocus();

	virtual bool WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

protected:
	void CloseList(GItem* item);
};

enum DIGIT_ENUM {DIGIT_BOOL, DIGIT_CHAR, DIGIT_STRING, DIGIT_INT, DIGIT_FLOAT, DIGIT_DOUBLE, DIGIT_ERROR};
enum SIGN_ENUM {SIGN_NO, SIGN_MINUS, SIGN_PLUS};
struct DIGIT
{
	DIGIT_ENUM type;

	//   
	int value1;		//  
	int value2;		//  
	SIGN_ENUM sign;

	// DIGIT_CHAR
	// DIGIT_STRING
	char* str;

	DIGIT()
	{
		str=NULL;
	}

	~DIGIT()
	{
		if (str)
		{
			delete []str;
			str=NULL;
		}
	}

	DIGIT& operator=(DIGIT& from);

	bool FromString(CMagicString* str);

	//   
	bool IsInteger(){return (type==DIGIT_INT || type==DIGIT_BOOL || type==DIGIT_CHAR);}

	CMagicString GetExportString();

	void Serialize(CMagicStream& ar);
};

struct STATIC_TEXT
{
	CMagicString text;
	unsigned int color;
	int font;
	int x, y;
	CMagicString link;
};

class GStatic : public GScroller
{
protected:
	int k_text;
	STATIC_TEXT** m_text;
	int text_width, text_height;
	int source_width, source_height;

	bool has_links;
	int current_link;

public:
	GStatic();
	virtual ~GStatic();

	void ClearText();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text);

	//   
	virtual bool IsStatic(){return true;}

	virtual void SetWindowText(const char* text);

	//  
	virtual bool SetAlignText(COMPONENT_ALIGN_TEXT align);

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);

	virtual void ClientOnMouseLeave(unsigned int nFlags);

protected:
	//     
	CMagicString TestSymbols(const char* text);

	//   
	void AddFormatString(STATIC_TEXT* line);
	//     
	void FormatString(STATIC_TEXT* line, bool& new_line);

	//    
	void FormatText();

	//    
	void FormatText2();

	//        
	void AlignText();

public:
	virtual void SysOnChangeText();

	virtual void ClientOnRender();

	//   ,     
	virtual void GetScrollSize(int& wi, int& he){wi=text_width; he=text_height;}

	int GetStringCount(){return k_text;}
	STATIC_TEXT* GetString(int index){return m_text[index];}

	//   
	int GetCurrentLink();
};

class GPicture : public GComponent
{
protected:
	GImageList list;
	int index;

public:
	GPicture();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id);

	//  
	virtual bool IsPicture(){return true;}

	//    
	void AddImage(GSurface* surface);

	//     
	int GetCount();

	//  
	void SetSel(int index);
	//   
	int GetSel(){return index;}

	//   
	GSurface* GetImage();

	virtual void ClientOnRender();
};

//   
class GSinglePicture : public GComponent
{
protected:
	GSurface* surface;

public:
	GSinglePicture();
	virtual ~GSinglePicture();

	//  
	virtual bool IsSinglePicture(){return true;}

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, GSurface* surface, unsigned int id);

	virtual void ClientOnRender();

	//  ,       
	virtual bool IsOver(int x, int y);
};

// ,       
class GDiscretBox : public GPicture
{
protected:
	int sel;
	int over;

	CMagicRect gap_discret;

	int count; //  
	GSize element_size; //  ,    
	GSize element_between; //   

public:
	GDiscretBox();
	virtual ~GDiscretBox();

	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int count, GSize element_size, GSize element_between, unsigned int id);

	//  
	virtual bool IsPicture(){return false;}
	//  GDiscretBox-
	virtual bool IsDiscretBox(){return true;}

	//   
	int GetColumnCount(){return count;}

	//  
	int GetSel(){return sel;}
	void SetSel(int index);

	//  
	void RefreshImage();

	//   
	void GetDiscretRect(int index, CMagicRect* rect);

	//     
	int DetectColumn(int x, int y);

	virtual void ClientOnLButtonDown(unsigned int nFlags, int x, int y);
	virtual void ClientOnMouseMove(unsigned int nFlags, int x, int y);
	virtual void ClientOnMouseLeave(unsigned int nFlags);

	virtual void OnChangeSelected(int index){}
};

// ,   
class GDialog : public GComponent
{
protected:
	GSurface* background;
	bool is_background_ptr;

public:
	GDialog();

	//  
	virtual bool Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id);

	//    
	virtual bool IsDialog(){return true;}

	//     
	virtual void BeginModal();

	//      
	GButton* GetEnterButton();

	//      
	GButton* GetEscapeButton();

	GSurface* GetBackground(){return background;}
	void SetBackground(const char* file);
	void SetBackground(GSurface* surface);
	//      (  ,   ,    )
	void SetBackgroundPtr(GSurface* surface);


	virtual bool WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);
	virtual bool WinOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

	virtual void OnEnter();
	virtual void OnCancel();

	virtual void OnInitDlg();
};

// ,  MessageBox
#define GSMB_OK		1
#define GSMB_CANCEL	2
class GMessageBox : public GDialog
{
public:
	virtual void Alert(const char* title, const char* message, unsigned int style);

	//    
	virtual void Invalidate();
};

//   
class GScene : public GDialog
{
protected:
	int previous_scene;

	unsigned int scene_period;
	unsigned int scene_tick;

	bool is_covering;

	int k_component;
	GComponent* m_component[1000];

public:
	GScene();
	virtual ~GScene();

	virtual void OnCreate(){}
	virtual void OnDestroy(){}

	virtual void OnStart(){}
	virtual void OnFinish(int next_scene){}

	virtual void OnUpdate();
	virtual void OnRender(){}

	//    
	void HideChildren();

	//       
	virtual void ShowChildrenByDefault(){}

	//  
	virtual bool IsScene(){return true;}

	void SetPreviousScene(int scene){previous_scene=scene;}
	int GetPreviousScene(){return previous_scene;}

	//  ,    OnPeriod()
	void SetScenePeriod(unsigned int tick_period);
	void ResetScenePeriod(){scene_period=0;}

	//     ,    
	void RefreshCovering();
	//     
	void SetCoveringFlag(){is_covering=true;}

	//   
	void RenderAll();

	virtual void OnPeriod(){}
};


//   
#define CONTROL_LEFT	1
#define CONTROL_RIGHT	2
#define CONTROL_MIDDLE	4
#define CONTROL_ALT		8
#define CONTROL_CTRL	16
#define CONTROL_SHIFT	32

//  
#define COMPONENT_PLACE_WIN		0
#define COMPONENT_PLACE_INNER	1
#define COMPONENT_PLACE_CLIENT	2

//   
class GGame
{
public:
	GUID_GAME guid;
	GTheme* theme;

	CMagicStringArray args;

protected:
	CMagicString game_name;
	CMagicString game_version;
	DIGITAL_VERSION game_digital_version;
	CMagicString license;

	CMagicString user;

	GSurface* backbuffer;
	GImage* cursor_default;

	int cursor_x, cursor_y;
	float fcursor_x, fcursor_y;
	float cursor_factor;

	GImage* cursor;
	bool is_cursor;

	TIMER_DATA main_timer;

	unsigned int control;
	bool keys[256];

	int k_scene;
	int t_scene;
	GScene** m_scene;

	GComponent* modal;
	GComponent* captured;
	int captured_place;
	GComponent* focus;
	GComponent* over;
	int over_place;
	GComponent* pressed;
	int pressed_place;
	GComponent* key_down;
	int key_down_count;

	unsigned int repeated_key;
	unsigned int repeated_char;
	unsigned int repeated_tick;
	unsigned int repeated_count;

	bool fullscreen_mode;

	CMagicString object_path;
	CMagicString image_path;
	CMagicString font_path;
	CMagicString users_path;
	CMagicString profile_path;
	CMagicString saved_game_path;
	CMagicString music_path;
	CMagicString sound_path;
	CMagicString voice_path;
	CMagicString maps_path;
	CMagicString downloaded_maps_path;
	CMagicString config_path;
	CMagicString init_path;
	CMagicString autosave_path;

	#ifndef _DEBUG
	unsigned int tick_mouse_capture;
	#endif
	
public:
	GDevice* device;
	GPlatform* platform;
	GNet* net;
	GNetClient* net_downloader;
	GSound* sound;

	CMagicString hello_server_reply; //        

public:
	GGame(GUID_GAME guid, const char* name, const char* version, bool fullscreen_mode=false);
	virtual ~GGame(void);

	//  
	GTheme* GetTheme(){return theme;}

	//  
	void ClearControl();

	//    
	unsigned int GetControl(){return control;}
	
	//  ,    
	virtual bool CreateGame(void* init, int view_width=0, int view_height=0);

	const char* GetName(){return game_name;}
	const char* GetVersion(){return game_version;}
	DIGITAL_VERSION* GetDigitalVersion(){return &game_digital_version;}

	//    RND   
	virtual int GetCryptRnd(){return 0;}

	// 
	virtual const CMagicString* GetLicense();
	//   (Steam/)
	virtual int GetLicenseType();

	//  
	virtual void SaveLicense(const char* license);
	//  
	virtual void LoadLicense();

	//     
	GComponent* DetectComponentByCursor(int& x, int& y, int& place);

	//   
	GComponent* GetFocusComponent(){return focus;}
	void SetFocusComponent(GComponent* focus){this->focus=focus;}

	//    
	GSurface* GetBackbuffer(){return backbuffer;}
	//    
	void SetBackbuffer(GSurface* surface){backbuffer=surface;}

	//   
	GScene* GetScene();
	//   
	GScene* GetScene(int index);
	int GetSceneIndex(){return t_scene;}
	//   
	int GetSceneCount(){return k_scene;}

	// / 
	void ShowCursor(bool show);
	//     
	GImage* GetCursor();
	//       
	virtual GImage* GetCursorDefault(){return cursor_default;}

	//   
	void SetCursorPosition(int x, int y);
	//   
	void GetCursorPosition(int& x, int& y){x=cursor_x; y=cursor_y;}

	//    
	void SetCursorSpeed(float factor){cursor_factor=factor;}

	//    
	bool IsKeyHeld(unsigned int key);
	
	//   
	GComponent* GetCapturedComponent(){return captured;}
	void SetCapturedComponent(GComponent* captured, int place){this->captured=captured; captured_place=place;}
	//   
	GComponent* GetModalComponent(){return modal;}
	void SetModalComponent(GComponent* modal){this->modal=modal;}
	//   
	GComponent* GetPressedComponent(){return pressed;}
	void SetPressedComponent(GComponent* pressed, int place){this->pressed=pressed; pressed_place=place;}
	//    
	GComponent* GetOverComponent(){return over;}
	void SetOverComponent(GComponent* over){this->over=over;}
	//        
	GComponent* GetKeyDownComponent(){return key_down;}
	void SetKeyDownComponent(GComponent* key_down){this->key_down=key_down;}

	//   
	virtual GDevice* NewDevice(void* init)=0;

	//  
	virtual GPlatform* NewPlatform(GGame* owner, const char* game_name)=0;

	//  
	virtual GSound* NewSound()=0;

	//  
	virtual GNet* NewNet(){return NULL;}
	//   
	virtual GNetServer* NewServer(){return NULL;}
	//    
	virtual GNetClient* NewDownloader(){return NULL;}

	//   
	virtual void GetStartedVideoMode(VIDEO_MODE* vm);

	//   
	bool SetScene(int index);

	//  
	void AddScene(GScene* scene);

	//  
	int CreateFontex(const char* name);

	//         
	virtual unsigned int GetAdditionalStyles(GComponent* component){return 0;}

	//      
	//void AddInvalidate(GComponent* component);

	//     
	//void DeleteInvalidate(GComponent* component);

	//  
	void PlatformMoveCursor(int sx, int sy);

	//   ,  
	void PlatformKeyDown(int code, unsigned int nRepCnt);
	//   ,  
	void PlatformKeyUp(int code);

	// 
	void PlatformTimer();

	//  ,    
	bool IsActivated();

	//  ,   
	bool IsIconic();

	//    
	void ToIconic();

	//    
	void ToMaximize();

	//     
	void SetMainTimer(unsigned int period);
	//   
	void KillMainTimer();
	//     
	bool IsMainTimer();
	//   
	unsigned int GetMainTimerPeriod();

	//  
	void SetRepeatedKey(unsigned int rkey);
	void SetRepeatedChar(unsigned int rchar);
	void ResetRepeatedKey();
	void RepeateKey();

	bool IsKeyHold(unsigned char key){return keys[key];}

	//  
	virtual CMagicString SavePrintScreen();

	//   
	virtual bool StartNet(const char* ip);

	//   
	virtual void StopNet();

	//  ,     
	bool IsMigratingHost();

	//  ,    
	virtual bool IsShowNews(){return true;}

	//    
	bool DownloadFile(const char* url_file);

	//    
	void StopNetDownloader();

	//  ,     
	virtual void Play();

	// ,       
	virtual void Quit();

	// ,    
	virtual bool CanQuit(){return true;}

	//  
	virtual void DrawCursor(GImage* cur)=0;

	//  
	virtual void SysOnUpdate();

	//  
	virtual void SysOnRender();

	//   
	virtual void OnFirstLaunch(){}

	//  
	virtual void OnChangeVideoMode(int width, int height);

	virtual void OnActivate(unsigned int nState, bool bMinimized){}
	virtual void SysOnActivate(unsigned int nState, bool bMinimized);

	virtual void SysOnLButtonDown(unsigned int nFlags);
	virtual void SysOnLButtonUp(unsigned int nFlags);
	virtual void SysOnLButtonDblClk();

	virtual void SysOnRButtonDown(unsigned int nFlags);
	virtual void SysOnRButtonUp(unsigned int nFlags);
	virtual void SysOnRButtonDblClk();

	virtual void SysOnMButtonDown(unsigned int nFlags);
	virtual void SysOnMButtonUp(unsigned int nFlags);
	virtual void SysOnMButtonDblClk();

	virtual void SysOnMouseWheel(short zDelta);

	virtual void SysOnMouseMove(unsigned int nFlags);

	virtual void SysOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);
	virtual void SysOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

	virtual void SysOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags);

	//    
	void UpdateNet();

public:
	//    

	//   
	void SetUserName(const char* user_name);
	//   
	const char* GetUserName(){return user.c_str();}

	//     ( )
	virtual const char* GetServerFileName()=0;

	// eng: Returns path to folder with objects
	// rus:      
	virtual const char* GetPathToObject();

	// eng: Returns path to folder with images
	// rus:      
	virtual const char* GetPathToImage();

	// eng: Returns path to folder with fonts
	// rus:      
	virtual const char* GetPathToFont();

	// eng: Returns path to folder with users
	// rus:      
	virtual const char* GetPathToUsers();

	// eng: Returns path to folder with user profile
	// rus:       
	virtual const char* GetPathToProfile();

	// eng: Returns path to folder with saved games
	// rus:       
	virtual const char* GetPathToSavedGame();

	// eng: Returns path to folder with music
	// rus:      
	virtual const char* GetPathToMusic();

	// eng: Returns path to folder with sounds
	// rus:      
	virtual const char* GetPathToSound();

	// eng: Returns path to folder with voices
	// rus:      
	virtual const char* GetPathToVoice();

	// eng: Returns path to folder with maps
	// rus:      
	virtual const char* GetPathToMaps();

	// eng: Returns path to folder with downloaded maps
	// rus:       
	const char* GetPathToDownloadedMaps();

	// eng: Returns path to folder with configuration
	// rus:      
	virtual const char* GetPathToConfig();

	// eng: Returns path to folder with initialization data
	// rus:       
	virtual const char* GetPathToInit();

	// eng: Returns path to folder with autosave
	// rus:      autosave
	virtual const char* GetPathToAutosave();

	// eng: Returns path to folder with files of virtual disk
	// rus:        
	virtual const char* GetPathToVirtualDrive(){return NULL;}

	// ---------------------------------------------------
	//    

	//     
	//                  
	virtual void OnEnumSession(GSessionList* sessions, int count_general_session){}

	//       
	virtual bool IsNetVersionsCompatible(DIGITAL_VERSION* ver1, DIGITAL_VERSION* ver2);

	//    
	virtual void OnCreateSession(int index_player){}
	//      (   ,   false)
	virtual bool OnJoinSession(int index_player, const char* map_name, unsigned int map_id){return true;}
	
	//     ,       
	virtual bool GetSessionInfoStruct(NET_SESSION_INFO* si){return false;}
	virtual bool GetSessionPlayerStruct(int index, NET_PLAYER* np){return false;}
	//       ,    
	virtual void SetSessionInfoStruct(NET_SESSION_INFO* si){}
	virtual void SetSessionPlayerStruct(int index, NET_PLAYER* np){}

	//    (        )
	virtual void OnNewPlayer(int index){}

	//   ,     ,   
	virtual bool GetChatStruct(int k_chat, CMagicString* m_chat){return false;}

	//        
	virtual void GetChatList(NET_CHAT_MESSAGE_LIST* list){}

	//        
	virtual int GetIndexOfConnectedPlayer();

	//       (5,4,3,2,1...   OnStartNetGame(),    )
	virtual void OnStartNetCounter(int counter){}

	//    
	virtual void SetStartRND(int start_rnd){}

	//    
	virtual int GetNetGameSpeed(){return 0;}
	//    
	virtual void SetNetGameSpeed(int net_speed){}

	//   ,     ,      
	virtual void OnStartNetGame(){}

	//   ,          
	virtual bool IsNetGameLoaded(){return true;}

	//     
	virtual void OnLaunchNetGame(){}

	//    ,      
	virtual void OnWaitNetPlayers(int k_waited, CMagicString* m_waited){}

	//        ( )
	virtual void OnPrepareTransferedMap(GMemWriter* writter){}

	//       ,       
	virtual void OnMapWasDownloaded(const MEM_DATA& buf){}

	//   ,     - 
	virtual void OnDownloadingMapPercent(int percent){}

	//  -
	virtual void OnChatMessage(CMagicString& sender, CMagicString& receiver, bool privated, bool global, CMagicString& chat, bool printed_message=true){}

	//    (    )
	virtual void OnCancelSession(){}

	//       
	virtual void OnDeletingFromSession(){}


	//            
	//    ,   false,  true
	virtual void SetPlayerNetMessage(unsigned int sender, MEM_DATA& message){}
	//           
	virtual MEM_DATA GetPlayerNetMessage(){MEM_DATA v; v.data=NULL; v.length=0; return v;}

	//        ( -1,   ,       )
	virtual void ClearPlayerNetMessage(int index){}

	//    
	virtual bool OnNextNetTakt(){return false;}

	//    
	virtual int GetNetTakt(){return 0;}

	//         
	virtual void OnWaitingPlayers(unsigned int dtime, int k_player_id, unsigned int* m_player_id){}

	//       
	virtual void OnLostPlayer(unsigned int player_id){}

	//    
	virtual void OnLostConnection(NET_DISCONNECT state){}

	//    
	virtual void OnStartSynchronization(int count){}
	//    
	virtual void OnFinishSynchronization(){}

	//  save-     ,       ( )
	virtual void OnPrepareSyncData(GMemWriter* writter){}

	//      
	//     ,     
	virtual void OnSyncWasDownloaded(MEM_DATA& sync){}

	//    
	virtual void OnDownloadFile(const char* name, MEM_DATA* buf){}

	//        
	//     ,      
	virtual void OnServerInfo(NET_SERVER_INFO* server_info){delete server_info;}

	//        
	//     ,      
	virtual void OnStatisticInfo(int max_license_type, NET_STATISTICS* statistics);

	//  
	//  : 0 -  , 1 -   
	virtual int VerifyLicense(int license_type, const char* license);

	//       
	virtual int GetServerCompatibleVersion(){return 2;}
	//       
	virtual bool IsGameCompatibleWithServer(int received_version){return (received_version==2);}

	//      
	virtual void OnServerRelpyToManager(){}


	//   ,          MESSAGE_TYPE_PING_REPLY
	//      
	virtual void GetPingServerInfo(GMemWriter* writter){}

	//               MESSAGE_TYPE_PING_REPLY
	virtual void OnPingServerInfo(GMemReader* reader){}

	#ifdef STEAM
	//   id   Steam-
	virtual unsigned int GetGameSteamID()=0;
	#endif
};


