//-----------------------------------------------------------------------------
// File: Net.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "Net.h"
#include "Game.h"

#ifdef STEAM
#include "steam\SteamLicense.h"
#endif

int PORT_SERVER_TCP_ADMIN=42599;
int PORT_SERVER_TCP_GAME=42600;
int PORT_CLIENT_TCP=42597;

// --------------------------------------------------------------------------

bool is_local_server=false;
bool IsLocalServer()
{
	return is_local_server;
}

//       
CMagicString GetPathToMigrationServerConfig()
{
	GPlatform* platform=GNet::platform;
	CMagicString path=platform->GetPathToTemp();
	path+="migsrv.plr";
	return path;
}

// --------------------------------------------------------------------------

NET_BUFFER::NET_BUFFER()
{
	owner=NULL;
	max_length=0;
	length=0;
	position=0;
	data=NULL;
}

NET_BUFFER::~NET_BUFFER()
{
	Clear();
}

void NET_BUFFER::Clear()
{
	if (data)
	{
		delete []data;
		data=NULL;
	}
	length=0;
	max_length=0;
}

void NET_BUFFER::SetMaxSize(int size)
{
	Clear();

	max_length=size;
	data=new unsigned char[max_length];
}

void NET_BUFFER::Add(int length, void* data)
{
	int len=this->length+length;
	if (max_length<len)
	{
		//  
		max_length=len;
		unsigned char* vdata=new unsigned char[max_length];
		memcpy(vdata, this->data, this->length);
		delete []this->data;
		this->data=vdata;
	}

	if (data)
		memcpy(this->data+this->length, data, length);
	this->length=len;
}

//    
void NET_BUFFER::Delete(int length)
{
	int size=this->length-length;
	memcpy(data, data+length, size);
	this->length-=size;
}

//      
void NET_BUFFER::IncreaseSize()
{
	if (data)
	{
		max_length=max_length+max_length/4;
		unsigned char* vdata=new unsigned char[max_length];
		memcpy(vdata, data, length);
		delete []data;
		data=vdata;
	}
	else
	{
		max_length=4096;
		data=new unsigned char[max_length];
	}
}

//       ,    
//      -1,    
int NET_BUFFER::HasMessage(GNetSocket* socket)
{
	int struct_size=sizeof(SEND_MESSAGE);

	unsigned char* guid_char=(unsigned char*)&(socket->net->guid);
	int guid_len=sizeof(GUID_GAME);
	int len=length-position;
	if (len<guid_len)
		guid_len=len;
	//       GUID_GAME
	for (int i=0;i<guid_len;i++)
	{
		if (data[position+i]!=guid_char[i])
		{
			//  ,     
			socket->Error();
			return -1;
		}
	}

	int s=position+struct_size;
	if (s<=length)
	{
		//   ,   
		SEND_MESSAGE* sm=(SEND_MESSAGE*)&(data[position]);
		s+=sm->length;
		if (s<=length)
		{
			//   
			return s;
		}
	}
	return -1;
}

// --------------------------------------------------------------------------

void NET_ADDRESS::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<address;
		stream<<port;
	}
	else
	{
		stream>>address;
		stream>>port;
	}
}

void NET_ADDRESS::FromStringIP(const char* ip)
{
	port=0;
	address=ip;
	int pos=address.Find(":");
	if (pos!=-1)
	{
		//  
		int len=address.GetLength();
		CMagicString port_str=address.Right(len-pos-1);
		port=port_str.StringToInt();
		address=address.Left(pos);
	}
	else
	{
		port=PORT_SERVER_TCP_GAME;
	}
}

CMagicString NET_ADDRESS::ToStringIP()
{
	CMagicString ip=address;
	ip+=':';
	CMagicString d;
	d.IntToString(port);
	ip+=d;
	return ip;
}

// --------------------------------------------------------------------------

NET_SERVER_INFO::NET_SERVER_INFO()
{
	version=0;

	start_time=current_time=0;

	k_accept=0;
	k_hello=0;

	//max_socket=0;
	//k_socket=0;
	//m_socket=NULL;

	//max_session=0;
	//k_session=0;
	//m_session=NULL;

	sessions=new GSessionList;
	sockets=new GSocketList;
}

//  (    )
void NET_SERVER_INFO::Clear()
{
	start_time=current_time=0;

	k_accept=0;
	k_hello=0;

	if (sessions)
	{
		delete sessions;
		sessions=NULL;
	}

	if (sockets)
	{
		delete sockets;
		sockets=NULL;
	}
}


// 
void NET_SERVER_INFO::Serialize(CMagicStream& stream, GNet* net)
{
	if (stream.IsStoring())
	{
		stream<<version;

		stream<<server_version;

		stream<<start_time;
		stream<<current_time;
		stream<<k_accept;
		stream<<k_hello;

		sessions->Serialize(stream, net);
		sockets->Serialize(stream, net);
	}
	else
	{
		Clear();

		stream>>version;

		stream>>server_version;

		stream>>start_time;
		stream>>current_time;
		stream>>k_accept;
		stream>>k_hello;
		
		sessions=new GSessionList();
		sessions->Serialize(stream, net);

		sockets=new GSocketList();
		sockets->Serialize(stream, net);
	}
}

// --------------------------------------------------------------------------

NET_SOCKET_INFO::NET_SOCKET_INFO()
{
	version=0;

	player_id=0;
	session_id=-1;
	port=-1;
	udp_tcp=false;
	is_admin=false;
}

NET_SOCKET_INFO::~NET_SOCKET_INFO()
{
}

void NET_SOCKET_INFO::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<version;

		stream<<player_id;
		stream<<session_id;
		stream<<port;
		stream<<udp_tcp;
		stream<<is_admin;
		stream<<time;
		stream<<name;
		stream<<ip;
	}
	else
	{
		stream>>version;

		stream>>player_id;
		stream>>session_id;
		stream>>port;
		stream>>udp_tcp;
		stream>>is_admin;
		stream>>time;
		stream>>name;
		stream>>ip;
	}
}

// --------------------------------------------------------------------------

GNetSocket::GNetSocket(GNet* net)
{
	this->net=net;
	addr=NULL;

	type_license=-1;
	license=NULL;

	rtick=0;

}

GNetSocket::~GNetSocket()
{
	Destroy();

}

//  UDP  TCP- ,   ,        
//       
//      
bool GNetSocket::Create(bool udp_tcp, int port, bool listen)
{
	this->udp_tcp=udp_tcp;
	this->port=port;
	if (!udp_tcp)
		this->addr=new NET_ADDRESS;

	time=net->platform->GetTick();
	rtick=time;

	return true;
}

//  ,    
bool GNetSocket::IsWaitedAutorization()
{
	if (player_id)
		return false;

	if (name.IsEmpty())
		return false;

	if (IsAdmin())
		return false;

	return true;
}

//  
void GNetSocket::Destroy()
{
	if (net->IsServer())
	{
		if (license && type_license!=-1)
		{
			//        
			GNetServer* server=(GNetServer*)net;

			//  
			server->names.Delete(name);

			//  
			server->statistics[type_license].players.Delete(*license);
			license=NULL;
		}

		if (IsWaitedAutorization())
		{
			//   
			//       
			((GNetServer*)net)->deleted_sockets_during_verification.Add(this);
		}
	}

	if (session_id!=-1)
	{
		//     
		assert(net->IsServer());
		NET_SESSION_SERVER* session_server=((GNetServer*)net)->sessions.Get(session_id);
		session_id=-1;
		session_server->DeletePlayer(this);
	}

	if (addr)
	{
		delete addr;
		addr=NULL;
	}
}

//  IP-   TCP-
// own_or_peer          
bool GNetSocket::GetIP(CMagicString& addr, bool own_or_peer)
{
	addr="";
	return IsTCP();
}

//  
void GNetSocket::SendMessage(NET_BUFFER_INDEX* buf, NET_ADDRESS* addr)
{
	if (buf)
	{
		//  

		if (IsTCP())
		{
			SendTCP(buf);
		}
		else
		{
			if (addr)
				*(this->addr)=*addr;
			SendUDP(buf);
		}
	}
}

bool GNetSocket::ReceiveMessages()
{
	bool result=false;
	int next=rbuffer.HasMessage(this);
	if (next!=-1)
	{
		result=true;
		unsigned int player_id=this->player_id;
		GNet* net=this->net;
		unsigned char* data=rbuffer.GetData();
		do
		{
			int pos=rbuffer.GetPosition();
			rbuffer.SetPosition(next);
			unsigned char* current_data=data+pos;
			SEND_MESSAGE* sm=(SEND_MESSAGE*)current_data;
			net->receiving_socket=this;
			int type=sm->type;
			int sender=sm->sender;
			int length=sm->length;
			bool is=net->ReceiveMessage(type, sender, length, current_data+sizeof(SEND_MESSAGE));
			if (is)
			{
				//     
				rtick=net->platform->GetTick();
			}
			else
			{
				//   -  
				return result;
			}
			next=rbuffer.HasMessage(this);
		}
		while (next!=-1);
		int pos=rbuffer.GetPosition();
		int len=rbuffer.GetLength();
		if (pos==len)
		{
			//    ,    
			rbuffer.SetPosition(0);
			rbuffer.SetLength(0);
		}
	}
	return result;
}

// 
bool GNetSocket::Ban(bool autoban)
{
	if (license && type_license && net->IsServer())
	{
		GNetServer* server=(GNetServer*)net;
		CMagicStringTable* bans=&(server->statistics[type_license].bans);
		if (!bans->Find(*license))
		{
			//      
			NET_BAN ban_info;
			net->platform->GetCurrentDate(&ban_info.time);
			strcpy(ban_info.name, name.c_str());
			if (autoban)
				ban_info.type=1;
			else
				ban_info.type=0;

			MEM_DATA v;
			v.data=(unsigned char*)&ban_info;
			v.length=sizeof(ban_info);
			if (bans->Add(*license, &v))
			{
				//     
				CMagicString path=net->platform->GetPathToConfig();
				path+="bans";
				CMagicString d;
				d.IntToString(type_license);
				if (d.GetLength()==1)
					path+='0';
				path+=d;
				path+=".ctl";
				CMagicStreamFile file;
				if (file.Open(path, MAGIC_STREAM_ADD))
				{
					CMagicStringData* result=bans->Find(*license);
					result->Serialize(file);
					file.Close();
				}

				//    
				server->OnLostConnection(this);
				return true;
			}
		}
	}
	return false;
}

//       
bool GNetSocket::Error()
{
	if (license && type_license!=-1 && net->IsServer())
	{
		GNetServer* server=(GNetServer*)net;
		CMagicStringTable* errors=&(server->statistics[type_license].errors);
		CMagicStringData* er_data=errors->Find(*license);
		if (!er_data)
		{
			//       
			NET_ERROR v;
			MEM_DATA vd;
			vd.length=sizeof(v);
			vd.data=(unsigned char*)&v;
			if (!errors->Add(*license, &vd))
			{
				//  -    
				return false;
			}
			er_data=errors->Find(*license);
		}

		MEM_DATA* md=er_data->GetData();
		NET_ERROR* er=(NET_ERROR*)(md->data);
		strcpy(er->name, name.c_str());
		unsigned int time=server->platform->GetTick();
		unsigned int dtime=time-er->time;
		er->time=time;
		if (dtime>1000*60*60*24*3)
		{
			//   
			//    
			er->count_error=1;
			er->ban_stady=0;
		}
		else
		{
			er->count_error++;
			if (er->count_error>=3)
			{
				//    
				er->ban_stady++;
				if (er->ban_stady>=4)
				{
					//   
					if (Ban())
					{
						//   
						errors->Delete(*license);
						er_data=NULL;
					}
					else
						return false;
				}
				else
				{
					er->count_error=-1;
					server->OnLostConnection(this);
				}
			}
			else
				server->OnLostConnection(this);
		}

		if (er_data)
		{
			CMagicString path=net->platform->GetPathToConfig();
			path+="errors";
			CMagicString d;
			d.IntToString(type_license);
			if (d.GetLength()==1)
				path+='0';
			path+=d;
			path+=".ctl";
			CMagicStreamFile file;
			if (file.Open(path, MAGIC_STREAM_ADD))
			{
				er_data->Serialize(file);
				file.Close();
			}
		}
	}
	else
		net->OnLostConnection(this);

	return true;
}

// --------------------------------------------------------------------------

NET_PLAYER::NET_PLAYER(GNet* net)
{
	version=NET_PLAYER_VERSION;
	this->net=net;
	player_id=0;
	ping=0;
	time=0;
	length_info=0;
	//info=NULL;
	info=NULL;
}

NET_PLAYER::~NET_PLAYER()
{
	Clear();
}

void NET_PLAYER::Clear()
{
	if (info)
	{
		//delete []pinfo;
		free(info);
		info=NULL;
		length_info=0;
	}
}

void NET_PLAYER::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<version;
		stream<<type;
		stream<<player_id;
		stream<<ping;
		stream<<name;
		stream<<ip;
		stream<<time;
		stream<<length_info;
		if (length_info>0)
		{
			stream.Write(info, length_info);
		}
	}
	else
	{
		stream>>version;
		stream>>type;
		stream>>player_id;
		stream>>ping;
		stream>>name;
		stream>>ip;
		stream>>time;
		int vlength_info;
		stream>>vlength_info;
		if (length_info!=vlength_info)
		{
			if (info)
			{
				//delete []info;
				free(info);
				info=NULL;
			}
			length_info=vlength_info;
			if (length_info>0)
			{
				//info=new char[length_info];
				info=(char*)malloc(length_info);
			}
		}

		if (length_info>0)
			stream.Read(info, length_info);
	}
}

NET_PLAYER& NET_PLAYER::operator=(const NET_PLAYER& np)
{
	GMemWriter* wr1=net->wr1;
	GMemReader* rd1=net->rd1;

	wr1->Start();
	((NET_PLAYER&)np).Serialize(*wr1);
	MEM_DATA buf;
	wr1->Finish(buf);
	rd1->Start(buf.data);
	Serialize(*rd1);
	return *this;
}

bool NET_PLAYER::operator==(const NET_PLAYER& np)
{
	GMemWriter* wr1=net->wr1;
	GMemWriter* wr2=net->wr2;

	wr1->Start();
	((NET_PLAYER&)np).Serialize(*wr1);
	MEM_DATA buf1;
	wr1->Finish(buf1);

	wr2->Start();
	Serialize(*wr2);
	MEM_DATA buf2;
	wr2->Finish(buf2);

	return (buf1==buf2);
}

// --------------------------------------------------------------------------

//     
void DIGITAL_VERSION::FromString(const char* ver)
{
	CMagicString v=ver;
	ver1=ver2=ver3=ver4=0;

	int pos=v.Find('.');
	if (pos==-1)
	{
		ver1=v.StringToInt();
	}
	else
	{
		CMagicString vv;
		vv=v.Left(pos);
		ver1=vv.StringToInt();
		v=v.Right(v.GetLength()-(pos+1));
		int pos=v.Find('.');
		if (pos==-1)
			ver2=v.StringToInt();
		else
		{
			vv=v.Left(pos);
			ver2=vv.StringToInt();
			v=v.Right(v.GetLength()-(pos+1));
			int pos=v.Find('.');
			if (pos==-1)
				ver3=v.StringToInt();
			else
			{
				vv=v.Left(pos);
				ver3=vv.StringToInt();
				vv=v.Right(v.GetLength()-(pos+1));
				ver4=vv.StringToInt();
			}
		}
	}
}

//    
CMagicString DIGITAL_VERSION::ToString()
{
	CMagicString v;
	v.IntToString(ver1);
	//if (ver2 | ver3 | ver4)
	{
		CMagicString d;
		v+='.';
		d.IntToString(ver2);
		v+=d;
		if (ver3 | ver4)
		{
			v+='.';
			d.IntToString(ver3);
			v+=d;
			if (ver4)
			{
				v+='.';
				d.IntToString(ver4);
				v+=d;
			}
		}
	}
	return v;
}

void DIGITAL_VERSION::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<ver1;
		stream<<ver2;
		stream<<ver3;
		stream<<ver4;
	}
	else
	{
		stream>>ver1;
		stream>>ver2;
		stream>>ver3;
		stream>>ver4;
	}
}

void DIGITAL_VERSION::Load(FILE* file)
{
	fread(&ver1,sizeof(ver1),1,file);
	fread(&ver2,sizeof(ver2),1,file);
	fread(&ver3,sizeof(ver3),1,file);
	fread(&ver4,sizeof(ver4),1,file);
}

void DIGITAL_VERSION::Save(FILE* file)
{
	fwrite(&ver1,sizeof(ver1),1,file);
	fwrite(&ver2,sizeof(ver2),1,file);
	fwrite(&ver3,sizeof(ver3),1,file);
	fwrite(&ver4,sizeof(ver4),1,file);
}

// --------------------------------------------------------------------------

NET_SESSION_INFO::NET_SESSION_INFO(GNet* net)
{
	version=NET_SESSION_VERSION;
	this->net=net;
	id_map=0;
	k_player=0;
	m_player=NULL;
	tick=0;

	length_info=0;
	info=NULL;
}

NET_SESSION_INFO::~NET_SESSION_INFO()
{
	Clear();
}

void NET_SESSION_INFO::Clear()
{
	for (int i=0;i<k_player;i++)
	{
		delete m_player[i];
		m_player[i]=NULL;
	}

	if (m_player)
	{
		//delete []m_player;
		free(m_player);
		m_player=NULL;
	}

	if (info)
	{
		//delete []info;
		free(info);
		info=NULL;
		length_info=0;
	}
}

void NET_SESSION_INFO::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<version;
		stream<<ip;
		stream<<name;
		stream<<id_map;
		stream<<is_password;
		stream<<tick;
		stream<<k_player;
		for (int i=0;i<k_player;i++)
			m_player[i]->Serialize(stream);
		stream<<length_info;
		if (length_info>0)
			stream.Write(info, length_info);
	}
	else
	{
		Clear();
		stream>>version;
		stream>>ip;
		stream>>name;
		stream>>id_map;
		stream>>is_password;
		stream>>tick;
		stream>>k_player;
		if (k_player)
		{
			//m_player=new NET_PLAYER*[k_player];
			m_player=(NET_PLAYER**)malloc(k_player*sizeof(NET_PLAYER*));
			for (int i=0;i<k_player;i++)
			{
				m_player[i]=new NET_PLAYER(net);
				m_player[i]->Serialize(stream);
			}
		}

		stream>>length_info;
		if (length_info>0)
		{
			//info=new char[length_info];
			info=(char*)malloc(length_info);
			stream.Read(info, length_info);
		}
	}
	ver.Serialize(stream);

	version=NET_SESSION_VERSION;
}

NET_SESSION_INFO& NET_SESSION_INFO::operator=(const NET_SESSION_INFO& si)
{
	GMemWriter* wr1=net->wr1;
	GMemReader* rd1=net->rd1;

	wr1->Start();
	((NET_SESSION_INFO&)si).NET_SESSION_INFO::Serialize(*wr1);
	MEM_DATA buf;
	wr1->Finish(buf);
	rd1->Start(buf.data);
	NET_SESSION_INFO::Serialize(*rd1);
	return *this;
}

bool NET_SESSION_INFO::operator==(const NET_SESSION_INFO& si)
{
	GMemWriter* wr1=net->wr1;
	GMemWriter* wr2=net->wr2;

	wr1->Start();
	((NET_SESSION_INFO&)si).Serialize(*wr1);
	MEM_DATA buf1;
	wr1->Finish(buf1);

	wr2->Start();
	Serialize(*wr2);
	MEM_DATA buf2;
	wr2->Finish(buf2);

	return (buf1==buf2);
}

// --------------------------------------------------------------------------

NET_BUFFER_LIST::NET_BUFFER_LIST() : CArrayBase()
{
	net=NULL;

	k_buffer=0;
	m_buffer=NULL;

	k_buffer=10;
	m_buffer=(NET_BUFFER_INDEX**)malloc(k_buffer*sizeof(NET_BUFFER_INDEX*));
	for (int i=0;i<k_buffer;i++)
		m_buffer[i]=NULL;
	IncreaseDeleted(0,k_buffer-1);
}

NET_BUFFER_LIST::~NET_BUFFER_LIST()
{
	Clear();
}

void NET_BUFFER_LIST::Clear()
{
	CArrayBase::ClearExistingAndDeleted();
	for (int i=0;i<k_buffer;i++)
	{
		if (m_buffer[i])
		{
			delete m_buffer[i];
			m_buffer[i]=NULL;
		}
	}

	if (m_buffer)
	{
		free(m_buffer);
		m_buffer=NULL;
	}

	k_buffer=0;
}

int NET_BUFFER_LIST::AddBuffer(const MEM_DATA& buffer)
{
	int index=FromDeletedToExisting();
	if (index==-1)
	{
		int size=sizeof(NET_BUFFER_INDEX*);
		int k_buffer2=k_buffer+k_buffer/4;
		m_buffer=(NET_BUFFER_INDEX**)realloc(m_buffer, size*(k_buffer2));

		for (int i=k_buffer;i<k_buffer2;i++)
			m_buffer[i]=NULL;
		IncreaseDeleted(k_buffer, k_buffer2-1);
		index=FromDeletedToExisting();
		k_buffer=k_buffer2;
	}

	NET_BUFFER_INDEX* buf=m_buffer[index];
	if (!buf)
	{
		buf=net->NewBuffer(index);
		buf->owner=this;
		m_buffer[index]=buf;
	}

	buf->Reset();
	int max_len=buf->GetMaxLength();
	if (max_len<buffer.length)
	{
		//    
		buf->SetMaxSize(buffer.length);
	}

	unsigned char* dest=buf->GetData();
	if (buffer.data)
		memcpy(dest, buffer.data, buffer.length);
	buf->SetLength(buffer.length);

	return index;
}

void NET_BUFFER_LIST::DeleteBuffer(int index)
{
	if (m_buffer&& index>=0 && index<k_buffer)
	{
		if (m_buffer[index])
		{
			//delete m_buffer[index];
			//m_buffer[index]=NULL;

			FromExistingToDeleted(index);
		}
	}
}

// --------------------------------------------------------------------------

NET_SESSION_SERVER::NET_SESSION_SERVER(GNet* net) : NET_SESSION_INFO(net)
{
	enabled=true;
	//time=0;
	k_player_id=0;
	m_player_id=NULL;
	session_index=-1;
}

NET_SESSION_SERVER::~NET_SESSION_SERVER()
{
	if (k_player_id)
	{
		assert(net->IsServer());
		GNetServer* net_server=(GNetServer*)net;
		for (int i=0;i<k_player_id;i++)
		{
			GNetSocket* socket=net_server->sockets.Get(m_player_id[i]);
			socket->session_id=-1;
			net_server->sockets.DeleteSocket(m_player_id[i]);
		}

		if (m_player_id)
		{
			delete []m_player_id;
			m_player_id=NULL;
		}
		k_player_id=0;
	}
}

void NET_SESSION_SERVER::AddPlayer(GNetSocket* socket)
{
	unsigned int* vm_player_id=new unsigned int[k_player_id+1];

	for (int i=0;i<k_player_id;i++)
		vm_player_id[i]=m_player_id[i];
	delete []m_player_id;
	m_player_id=vm_player_id;

	m_player_id[k_player_id]=socket->player_id;
	socket->session_id=session_index;
	k_player_id++;
}

bool NET_SESSION_SERVER::DeletePlayer(GNetSocket* socket)
{
	for (int j=0;j<k_player_id;j++)
	{
		if (m_player_id[j]==socket->player_id)
		{
			if (k_player_id==1)
			{
				delete []m_player_id;
				m_player_id=NULL;
				k_player_id=0;

				//   ,      
				socket->net->sessions.DeleteSession(session_index);
			}
			else
			{
				unsigned int* vm_player_id=new unsigned int[k_player_id-1];
				int i;
				for (i=0;i<j;i++)
					vm_player_id[i]=m_player_id[i];
				for (i=j+1;i<k_player_id;i++)
					vm_player_id[i-1]=m_player_id[i];

				delete []m_player_id;
				m_player_id=vm_player_id;
				k_player_id--;
			}

			socket->session_id=-1;
			
			return true;
		}
	}
	return false;
}

void NET_SESSION_SERVER::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<enabled;
		//stream<<time;
		stream<<k_player_id;
		for (int i=0;i<k_player_id;i++)
			stream<<m_player_id[i];
		stream<<session_index;
	}
	else
	{
		stream>>enabled;
		//stream>>time;
		int vk_player_id=k_player_id;
		stream>>k_player_id;
		if (vk_player_id!=k_player_id)
		{
			if (m_player_id)
			{
				delete []m_player_id;
				m_player_id=NULL;
			}
			if (k_player_id)
				m_player_id=new unsigned int[k_player_id];
		}

		for (int i=0;i<k_player_id;i++)
			stream>>m_player_id[i];
		stream>>session_index;
	}

	NET_SESSION_INFO::Serialize(stream);
}

// --------------------------------------------------------------------------

GSessionList::GSessionList() : CArrayBase()
{
	k_session=0;
	m_session=NULL;
	ReInit();
}

GSessionList::~GSessionList()
{
	Clear();
}

void GSessionList::ReInit()
{
	Clear();
	k_session=10;
	m_session=(NET_SESSION_SERVER**)malloc(k_session*sizeof(NET_SESSION_SERVER*));
	for (int i=k_session-1;i>=0;i--)
	{
		m_session[i]=NULL;
		IncreaseDeleted(i, i);
	}
}

void GSessionList::Clear()
{
	for (int i=0;i<k_session;i++)
	{
		if (m_session[i])
		{
			delete m_session[i];
			m_session[i]=NULL;
		}
	}

	if (m_session)
	{
		free(m_session);
		m_session=NULL;
	}

	k_session=0;

	ClearExistingAndDeleted();
}

//     
NET_SESSION_SERVER* GSessionList::Get(int index)
{
	if (index>=0 && index<k_session)
		return m_session[index];
	return NULL;
}

//  
int GSessionList::AddSession(NET_SESSION_SERVER* session)
{
	int index=FromDeletedToExisting();

	if (index==-1)
	{
		int size=sizeof(NET_SESSION_SERVER*);
		int k_session2=k_session+k_session/4;
		m_session=(NET_SESSION_SERVER**)realloc(m_session,size*(k_session2));

		for (int i=k_session;i<k_session2;i++)
			m_session[i]=NULL;

		IncreaseDeleted(k_session, k_session2-1);
		index=FromDeletedToExisting();

		k_session=k_session2;
	}

	m_session[index]=session;
	session->session_index=index;

	return index;
}

//    
bool GSessionList::DeleteSession(int index)
{
	if (index>=0 && index<k_session)
	{
		if (m_session[index])
		{
			delete m_session[index];
			m_session[index]=NULL;
			FromExistingToDeleted(index);

			return true;
		}
	}
	return false;
}

/*
//      
NET_SESSION_SERVER** GSessionList::GetIndirectData(int& k_session)
{
	k_session=this->k_session;
	this->k_session=0;
	NET_SESSION_SERVER** v=m_session;
	m_session=NULL;
	return v;
}
*/

void GSessionList::Serialize(CMagicStream& stream, GNet* net)
{
	if (stream.IsStoring())
	{
		CArrayBase::Serialize(stream);

		stream<<k_session;
		for (int i=0;i<k_session;i++)
		{
			if (m_session[i])
			{
				bool is=true;
				stream<<is;
				m_session[i]->Serialize(stream);
			}
			else
			{
				bool is=false;
				stream<<is;
			}
		}
	}
	else
	{
		Clear();

		CArrayBase::Serialize(stream);

		stream>>k_session;
		if (k_session)
		{
			//m_session=new NET_SESSION_SERVER*[k_session];
			m_session=(NET_SESSION_SERVER**)malloc(k_session*sizeof(NET_SESSION_SERVER*));
			for (int i=0;i<k_session;i++)
			{
				bool is;
				stream>>is;

				if (is)
				{
					m_session[i]=new NET_SESSION_SERVER(net);
					m_session[i]->Serialize(stream);
				}
				else
					m_session[i]=NULL;
			}
		}
	}
}

// --------------------------------------------------------------------------

GSocketList::GSocketList() : CArrayBase()
{
	k_socket=10;
	m_socket=(GNetSocket**)malloc(k_socket*sizeof(GNetSocket*));
	for (int i=k_socket-1;i>=0;i--)
	{
		m_socket[i]=NULL;
		IncreaseDeleted(i, i);
	}
}

GSocketList::~GSocketList()
{
	Clear();
}

void GSocketList::Clear()
{
	for (int i=0;i<k_socket;i++)
	{
		if (m_socket[i])
		{
			m_socket[i]->Destroy();
			delete m_socket[i];
			m_socket[i]=NULL;
		}
	}

	if (m_socket)
	{
		free(m_socket);
		m_socket=NULL;
	}

	k_socket=0;

	ClearExistingAndDeleted();
}

//     
GNetSocket* GSocketList::Get(int index)
{
	if (index>=0 && index<k_socket)
		return m_socket[index];
	return NULL;
}

//  
int GSocketList::AddSocket(GNetSocket* socket, unsigned int player_id)
{
	int index;
	if (player_id!=0xffffffff)
	{
		index=FromDeletedToExisting(player_id);
	}
	else
		index=FromDeletedToExisting();

	if (index==-1)
	{
		int size=sizeof(GNetSocket*);
		int k_client2=k_socket+k_socket/4;
		m_socket=(GNetSocket**)realloc(m_socket,size*(k_client2));

		for (int i=k_socket;i<k_client2;i++)
			m_socket[i]=NULL;

		IncreaseDeleted(k_socket, k_client2-1);
		index=FromDeletedToExisting();

		k_socket=k_client2;
	}

	m_socket[index]=socket;

	//if (index==4)
	//	index=index;

	return index;
}

//    
bool GSocketList::DeleteSocket(int index)
{
	if (index>=0 && index<k_socket)
	{
		if (m_socket[index])
		{
			m_socket[index]->Destroy();
			delete m_socket[index];
			m_socket[index]=NULL;
			FromExistingToDeleted(index);

			return true;
		}
	}
	return false;
}

void GSocketList::Serialize(CMagicStream& stream, GNet* net)
{
	if (stream.IsStoring())
	{
		CArrayBase::Serialize(stream);

		stream<<k_socket;
		for (int i=0;i<k_socket;i++)
		{
			if (m_socket[i])
			{
				bool is=true;
				stream<<is;
				m_socket[i]->Serialize(stream);
			}
			else
			{
				bool is=false;
				stream<<is;
			}
		}
	}
	else
	{
		Clear();

		CArrayBase::Serialize(stream);

		stream>>k_socket;
		if (k_socket)
		{
			m_socket=(GNetSocket**)malloc(k_socket*sizeof(GNetSocket*));
			//m_socket=new GNetSocket*[k_socket];
			for (int i=0;i<k_socket;i++)
			{
				bool is;
				stream>>is;

				if (is)
				{
					m_socket[i]=net->NewSocket(net);
					m_socket[i]->Serialize(stream);
				}
				else
					m_socket[i]=NULL;
			}
		}
	}
}

//  
void GSocketList::Expand(int max_size)
{
	if (k_socket<max_size)
	{
		int i;
		GNetSocket** vm_socket=new GNetSocket*[max_size];
		for (i=0;i<k_socket;i++)
			vm_socket[i]=m_socket[i];
		for (i=k_socket;i<max_size;i++)
			vm_socket[i]=NULL;

		delete []m_socket;
		m_socket=vm_socket;

		k_socket=max_size;
		IncreaseDeleted(k_socket, max_size-1);
	}
}

// --------------------------------------------------------------------------

GGame* GNet::game=NULL;
GPlatform* GNet::platform=NULL;
GNet::GNet()
{
	wr1=new GMemWriter;
	wr2=new GMemWriter;
	rd1=new GMemReader;
	rd2=new GMemReader;

	internet=false;

	receiving_socket=NULL;

	send_list.SetOwner(this);
}

GNet::~GNet()
{
	Destroy();
	delete wr1;
	delete wr2;
	delete rd1;
	delete rd2;
}

bool GNet::Create(GUID_GAME guid, bool internet)
{
	this->guid=guid;
	this->internet=internet;

	//is_destroyed=false;

	//sessions.Clear();

	return true;
}

void GNet::Destroy()
{
	ClearEnumSessionInfo();

	guid.Clear();
}

//    
NET_BUFFER_INDEX* GNet::PrepareMessage(unsigned int sender_id, int type, int length, const unsigned char* data)
{
	int struct_size=sizeof(SEND_MESSAGE);
	int len=length+struct_size;

	int len16=len;

	NET_BUFFER_LIST* list=GetSendList();
	MEM_DATA buf;
	buf.data=NULL;
	buf.length=len16;
	int index=list->AddBuffer(buf);
	NET_BUFFER_INDEX* bi=list->Get(index);
	bi->SetLength(len);
	unsigned char* buf_data=bi->GetData();
	SEND_MESSAGE* sm=(SEND_MESSAGE*)buf_data;
	sm->guid=guid;

	sm->sender=sender_id;
	sm->type=type;
	sm->length=length;
	if (length)
		memcpy(&(buf_data[struct_size]), data, length);

	return bi;
}

void GNet::DeleteSessionInfo(int index)
{
	sessions.DeleteSession(index);
}

//    id 
int GNet::FindSessionInfo(unsigned int host_id)
{
	int k_session_info=sessions.GetIndexCount();

	for (int i=0;i<k_session_info;i++)
	{
		int index=sessions.GetIndex(i);
		NET_SESSION_INFO* si=sessions.Get(index);
		if (si->m_player[0]->player_id==host_id)
			return i;
	}
	return -1;
}


void GNet::ClearEnumSessionInfo()
{
	sessions.ReInit();
}

// --------------------------------------------------------------------------

GNetStady::GNetStady(GNetClient* owner)
{
	this->owner=owner;

	stady_period=0;
	stady_tick=0;
}

bool GNetStady::OnStart(NET_STADY previous, void* init)
{
	stady_tick=owner->game->platform->GetTick();
	return true;
}

void GNetStady::OnUpdate()
{
	if (stady_period)
	{
		unsigned int tick=owner->game->platform->GetTick();
		if (tick>=stady_tick+stady_period)
		{
			stady_tick=tick;
			OnPeriod();
		}
	}
}

// --------------------

GNetStadyEnumSession::GNetStadyEnumSession(GNetClient* owner) : GNetStady(owner)
{
	stady_period=500;
}

//   ,    
bool GNetStadyEnumSession::OnStart(NET_STADY previous, void* init)
{
	if (GNetStady::OnStart(previous, init))
	{
		if (init)
		{
			char* server_ip=(char*)init;
			return owner->ConnectToServer(server_ip);
		}
		return true;
	}
	return false;
}

void GNetStadyEnumSession::OnFinish(NET_STADY next)
{
	GNetStady::OnFinish(next);
	owner->ClearEnumSessionInfo();
}

//  
void GNetStadyEnumSession::OnPeriod()
{
	//   
	owner->StopEnumSessionInfo();

	//     
	owner->StartEnumSessionInfo();

	if (owner->GetMainSocket()->IsTCP())
		owner->SendPing();
}

//     
bool GNetStadyEnumSession::IsMessageCorrected(int message_type)
{
	return (message_type==MESSAGE_TYPE_ENUM_SESSION_REPLY || message_type==MESSAGE_TYPE_PING_REPLY || message_type==MESSAGE_TYPE_CHAT_FROM_SERVER);
}

// --------------------

GNetStadyJoinSession::GNetStadyJoinSession(GNetClient* owner) : GNetStady(owner)
{
	copy_session=new NET_SESSION_INFO(owner);
	current_session=new NET_SESSION_INFO(owner);

	stady_period=1000;
	transfer_map=NULL;

	SetConnectionResult(0);
}

GNetStadyJoinSession::~GNetStadyJoinSession()
{
	if (transfer_map)
	{
		delete transfer_map;
		transfer_map=NULL;
	}

	delete copy_session;
	copy_session=NULL;
	delete current_session;
	current_session=NULL;
}

//   ,   ,   
bool GNetStadyJoinSession::OnStart(NET_STADY previous, void* init)
{
	ping_takt=0;
	chat.Clear();
	SetConnectionResult(3);
	if (GNetStady::OnStart(previous, init))
	{
		if (owner->ConnectSession((NET_JOIN_DATA*)init))
		{
			if (!GetConnectionResult())
			{
				owner->game->GetSessionInfoStruct(current_session);
				*copy_session=*current_session;
				copy_session->name=""; //  ,     

				index_player=-1;
				unsigned int player_id=owner->GetMainSocket()->player_id;
				for (int i=0;i<current_session->k_player;i++)
				{
					if (player_id==current_session->m_player[i]->player_id && current_session->m_player[i]->type==PLAYER_MAN)
					{
						index_player=i;
						break;
					}
				}
				if (!owner->game->OnJoinSession(index_player, current_session->name.c_str(), current_session->id_map))
				{
					//   ,     
					transfer_map=new GMemWriter;
					NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_GIVE_MAP, 0, NULL, 1, &(current_session->m_player[0]->player_id));
					owner->GetMainSocket()->SendMessage(result);
				}
			
				return true;
			}
		}
	}
	return false;
}

void GNetStadyJoinSession::OnFinish(NET_STADY next)
{
	GNetStady::OnFinish(next);

	if (next!=NET_STADY_START_GAME)
	{
		//    ,    
		//      
		NET_PLAYER* current_player=current_session->m_player[index_player];
		owner->game->GetSessionPlayerStruct(index_player, current_player);
		current_player->type=PLAYER_OPENED;
		owner->game->SetSessionPlayerStruct(index_player, current_player);
		OnUpdate();
		//   
		owner->DestroySocket();
	}

	if (transfer_map)
	{
		delete transfer_map;
		transfer_map=NULL;
	}
}

void GNetStadyJoinSession::OnUpdate()
{
	GNetStady::OnUpdate();

	int k_chat=chat.GetCount();
	if (k_chat)
	{
		//   Update  ,    
		for (int i=0;i<k_chat;i++)
		{
			NET_CHAT_MESSAGE* c=chat.Get(i);
			if ((!c->sender.IsEmpty()) && (!c->privated))
			{
				CMagicString receiver;
				if (c->k_receiver)
					receiver=c->m_receiver[0];
				owner->game->OnChatMessage(c->sender, receiver, false, false, c->message, false);
			}
		}
		chat.Clear();
	}

	//      
	NET_PLAYER* current_player=current_session->m_player[index_player];
	NET_PLAYER* copy_player=copy_session->m_player[index_player];
	owner->game->GetSessionPlayerStruct(index_player, current_player);
	if (*copy_player!=*current_player)
	{
		//       
		*copy_player=*current_player;

		if (current_player->type==PLAYER_MAN)
		{
			GMemWriter* wr1=owner->wr1;
			wr1->Start();
			(*wr1)<<index_player;
			current_player->Serialize(*wr1);
			MEM_DATA buf;
			wr1->Finish(buf);
			//    ,   
			int k_receiver=owner->RefreshReceiverList();
			NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_PLAYER_INFO, buf.length, buf.data, k_receiver, owner->m_receiver);
			owner->GetMainSocket()->SendMessage(result);
		}
	}
}

// Ping
void GNetStadyJoinSession::OnPeriod()
{
	//if (!ping_takt)
	{
		ping_takt=owner->takt;
		//    ,   
		owner->SendPing();
		//int k_receiver=owner->RefreshReceiverList();
		//NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_PING, 0, NULL, k_receiver, owner->m_receiver);
		//owner->GetMainSocket()->SendMessage(result);
		owner->UpdateNet();
	}
}

//     
bool GNetStadyJoinSession::IsMessageCorrected(int message_type)
{
	return (message_type==MESSAGE_TYPE_PLAYER_INFO || message_type==MESSAGE_TYPE_START_GAME || message_type==MESSAGE_TYPE_DOWNLOAD_MAP || message_type==MESSAGE_TYPE_CHAT_FROM_SERVER || message_type==MESSAGE_TYPE_PING_REPLY);
}

// --------------------

GNetStadyCreateSession::GNetStadyCreateSession(GNetClient* owner) : GNetStadyJoinSession(owner)
{
	server_session=new NET_SESSION_INFO(owner);

	stady_period=500;

	k_transfer=0;
	m_transfer=NULL;
}

GNetStadyCreateSession::~GNetStadyCreateSession()
{
	ClearTransfers();

	delete server_session;
	server_session=NULL;
}

//   ,   ,   
bool GNetStadyCreateSession::OnStart(NET_STADY previous, void* init)
{
	password="";
	ClearTransfers();
	if (GNetStady::OnStart(previous, init))
	{
		if (owner->CreateSession())
		{
			if (init)
			{
				//    
				password=(const char*)init;
			}

			owner->game->GetSessionInfoStruct(current_session);
			*copy_session=*current_session;
			copy_session->name=""; //  ,     
			server_session->name=""; //  ,     
			
			index_player=0;
			owner->game->OnCreateSession(index_player);
			
			//      id 
			current_session->m_player[0]->player_id=owner->GetMainSocket()->player_id;
			current_session->is_password=!password.IsEmpty();
			owner->game->SetSessionInfoStruct(current_session);

			k_transfer=current_session->k_player;
			if (k_transfer)
			{
				m_transfer=new RECEIVER_MAP[k_transfer];
				for (int i=0;i<k_transfer;i++)
					m_transfer[i].player_id=0;
				transfer_map=new GMemWriter;
			}
			
			return true;
		}
	}
	return false;
}

//      
void GNetStadyCreateSession::OnFinish(NET_STADY next)
{
	ClearTransfers();
	if (next==NET_STADY_START_GAME)
	{
		//      
		owner->game->GetSessionInfoStruct(current_session);
		GMemWriter* wr1=owner->wr1;
		wr1->Start();
		current_session->Serialize(*wr1);
		int start_rnd=GetRndInt(0, 3000000);
		//start_rnd=4;
		owner->game->SetStartRND(start_rnd);
		(*wr1)<<start_rnd;
		int net_speed=owner->game->GetNetGameSpeed();
		(*wr1)<<net_speed;
		MEM_DATA buf;
		wr1->Finish(buf);
		GNetSocket* socket=owner->GetMainSocket();
		int k_receiver=owner->RefreshReceiverList();
		NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_START_GAME, buf.length, buf.data, k_receiver, owner->m_receiver);
		socket->SendMessage(result);

		//      
		//      ,   -       
		NET_PLAYER* host_player=current_session->m_player[0];
		int vtype=host_player->type;
		host_player->type=-1;
		wr1->Start();
		current_session->Serialize(*wr1);
		wr1->Finish(buf);
		result=owner->PrepareMessage(socket->player_id, MESSAGE_TYPE_SESSION_INFO, buf.length, buf.data);
		socket->SendMessage(result);
		host_player->type=vtype;
	}
	else
	{
		//    ,   ,    
		owner->DestroySocket();
	}
}

void GNetStadyCreateSession::OnUpdate()
{
	GNetStadyJoinSession::OnUpdate();

	//    ,   
	int k_receiver=owner->RefreshReceiverList();

	//          
	int i;
	int k_player=current_session->k_player;
	for (i=1;i<k_player;i++) //  ,       GNetStadyConnectSession::OnUpdate();
	{
		NET_PLAYER* current_player=current_session->m_player[i];
		NET_PLAYER* copy_player=copy_session->m_player[i];
		owner->game->GetSessionPlayerStruct(i, current_player);
		if (*copy_player!=*current_player)
		{
			//       
			unsigned int closed_player_id=0;
			if (current_player->type!=copy_player->type && copy_player->type==PLAYER_MAN)
			{
				//   ,         
				closed_player_id=copy_player->player_id;
			}

			*copy_player=*current_player;

			GMemWriter* wr1=owner->wr1;
			wr1->Start();
			(*wr1)<<i;
			current_player->Serialize(*wr1);
			MEM_DATA buf;
			wr1->Finish(buf);
			NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_PLAYER_INFO, buf.length, buf.data, k_receiver, owner->m_receiver);
			GNetSocket* socket=owner->GetMainSocket();
			socket->SendMessage(result);

			if (closed_player_id)
			{
				//     
				result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_PLAYER_INFO, buf.length, buf.data, 1, &closed_player_id);
				socket->SendMessage(result);
			}
		}
	}

	//     
	for (int i=0;i<k_transfer;i++)
	{
		RECEIVER_MAP* rm=&(m_transfer[i]);
		unsigned int receiver_id=rm->player_id;
		if (receiver_id)
		{
			//  ,   
			bool is=false;
			for (int j=0;j<k_receiver;j++)
			{
				if (owner->m_receiver[j]==receiver_id)
				{
					//     
					is=true;
					break;
				}
			}

			if (is)
			{
				//   
				char* data=transfer_map->GetBuffer();
				int len=transfer_map->GetLength()-4; //      4 ,     

				int portion=1*1024; //   1 

				if (rm->position+portion>=len)
				{
					//  ,  
					portion=len-rm->position;
					rm->player_id=0;	//    
				}

				//       
				int new_pos=rm->position+portion;
				int* len_pos=(int*)&(data[new_pos]);
				int v=*len_pos;
				*len_pos=len; 

				NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_DOWNLOAD_MAP, portion+4, (unsigned char*)(data+rm->position), 1, &receiver_id);
				owner->GetMainSocket()->SendMessage(result);
				rm->position=new_pos;

				//    ,      
				*len_pos=v;
			}
			else
			{
				//  
				rm->player_id=0;
			}
		}
	}
}

//      
void GNetStadyCreateSession::OnPeriod()
{
	GNetStadyJoinSession::OnPeriod();

	//       
	owner->game->GetSessionInfoStruct(current_session);

	if (!current_session->m_player)
		owner->game->GetSessionInfoStruct(current_session);

	if (*server_session!=*current_session)
	{
		//       
		*server_session=*current_session;

		GMemWriter* wr1=owner->wr1;
		wr1->Start();
		current_session->Serialize(*wr1);
		MEM_DATA buf;
		wr1->Finish(buf);
		GNetSocket* socket=owner->GetMainSocket();
		NET_BUFFER_INDEX* result=owner->PrepareMessage(socket->player_id, MESSAGE_TYPE_SESSION_INFO, buf.length, buf.data);
		socket->SendMessage(result);
	}
}

//     
bool GNetStadyCreateSession::IsMessageCorrected(int message_type)
{
	return (message_type==MESSAGE_TYPE_PLAYER_INFO || message_type==MESSAGE_TYPE_CONNECTING || message_type==MESSAGE_TYPE_GIVE_MAP || message_type==MESSAGE_TYPE_CHAT_FROM_SERVER || message_type==MESSAGE_TYPE_PING_REPLY);
}

//    
void GNetStadyCreateSession::AddTransferMap(unsigned int player_id)
{
	for (int i=0;i<k_transfer;i++)
	{
		RECEIVER_MAP* rm=&(m_transfer[i]);
		if (!rm->player_id)
		{
			rm->player_id=player_id;
			rm->position=0;
			break;
		}
	}
}

//    
void GNetStadyCreateSession::ClearTransfers()
{
	if (transfer_map)
	{
		delete transfer_map;
		transfer_map=NULL;
	}

	if (m_transfer)
	{
		delete []m_transfer;
		m_transfer=NULL;
	}
	k_transfer=0;
}


GNetStadyStartGame::GNetStadyStartGame(GNetClient* owner) : GNetStady(owner)
{
	stady_period=500;

	//    
	k_waited_player=0;
	m_waited_player=NULL;
}

//     
bool GNetStadyStartGame::OnStart(NET_STADY previous, void* init)
{
	//    
	start_counter=5;
	if (m_waited_player)
	{
		delete []m_waited_player;
		m_waited_player=NULL;
	}
	k_waited_player=0;

	if (previous==NET_STADY_CREATE_SESSION || previous==NET_STADY_JOIN_SESSION)
	{
		k_waited_player=owner->RefreshReceiverList();
		if (k_waited_player)
		{
			m_waited_player=new unsigned int[k_waited_player];
			for (int i=0;i<k_waited_player;i++)
				m_waited_player[i]=owner->m_receiver[i];
		}

		return true;
	}
	return false;
}

GNetStadyStartGame::~GNetStadyStartGame()
{
	if (m_waited_player)
	{
		delete []m_waited_player;
		m_waited_player=NULL;
	}
	k_waited_player=0;
}

//      
void GNetStadyStartGame::OnPeriod()
{
	if (start_counter>0)
	{
		//  
		owner->game->OnStartNetCounter(start_counter);
		start_counter--;
	}
	else
	{
		if (start_counter==0)
		{
			//   
			owner->game->OnStartNetGame();
			start_counter--;
		}

		if (start_counter<0 && owner->game->IsNetGameLoaded())
		{
			//     ,      
			int k_receiver=owner->RefreshReceiverList();
			NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_PLAYER_STARTED, 0, NULL, k_receiver, owner->m_receiver);
			owner->GetMainSocket()->SendMessage(result);

			int k_waited_name=0;
			CMagicString* m_waited_name=NULL;
			if (k_waited_player)
			{
				//   ,     
				m_waited_name=new CMagicString[k_waited_player];

				NET_SESSION_INFO si(owner);
				owner->game->GetSessionInfoStruct(&si);

				for (int i=0;i<k_waited_player;i++)
				{
					for (int j=0;j<k_receiver;j++)
					{
						//     , ..     
						if (owner->m_receiver[j]==m_waited_player[i])
						{
							for (int k=0;k<si.k_player;k++)
							{
								if (si.m_player[k]->player_id==m_waited_player[i] && si.m_player[k]->type==PLAYER_MAN)
								{
									m_waited_name[k_waited_name]=si.m_player[k]->name;
									k_waited_name++;
									j=k_receiver;
									break;
								}
							}
						}
					}
				}
			}

			if (k_waited_name)
			{
				//     
				owner->game->OnWaitNetPlayers(k_waited_name, m_waited_name);
				delete[] m_waited_name;
			}
			else
			{
				//   
				owner->SetStady(NET_STADY_GAME, NULL);
			}
		}
	}
}

//     
bool GNetStadyStartGame::IsMessageCorrected(int message_type)
{
	return (message_type==MESSAGE_TYPE_PLAYER_STARTED || message_type==MESSAGE_TYPE_PLAYER_GAME);
}


GNetStadyGame::GNetStadyGame(GNetClient* owner) : GNetStady(owner)
{
	stady_period=10000;

	k_player=0;
	m_player=NULL;
	m_waited_player=NULL;
	sync_data=new GMemWriter;
}

GNetStadyGame::~GNetStadyGame()
{
	ClearPlayerData();
	delete sync_data;
	sync_data=NULL;
}

//   
void GNetStadyGame::ClearPlayerData()
{
	if (m_player)
	{
		delete []m_player;
		m_player=NULL;
	}
	k_player=0;

	if (m_waited_player)
	{
		delete []m_waited_player;
		m_waited_player=NULL;
	}
}

//        
void GNetStadyGame::ReceivePlayerCommands(unsigned int sender, int length, unsigned char* data, unsigned int net_takt)
{
	if (!sync_mode)
	{
		//    
		for (int i=0;i<k_player;i++)
		{
			PLAYER_MESSAGE* pm=&(m_player[i]);
			if (pm->player_id==sender)
			{
				//  
				if (net_takt==takt || net_takt==takt+1)
				{
					if (pm->receive_takt==takt)
					{
						//      
						//     ,        
						pm->next_message.Reset();
						pm->next_message.Add(length, data);
					}
					else
					{
						pm->receive_takt=takt;

						MEM_DATA message;
						message.length=length;
						message.data=data;
						owner->game->SetPlayerNetMessage(sender, message);
						//   
						GetWaitedPlayers();
					}
				}

				break;
			}
		}
	}
}

//    
int GNetStadyGame::GetWaitedPlayers()
{
	int i;
	int k_waited=0;
	int k_receiver=owner->RefreshReceiverList();

	bool is=true;
	for (i=0;i<k_player;i++)
	{
		PLAYER_MESSAGE* pm=&(m_player[i]);
		if (pm->receive_takt!=takt)
		{
			for (int j=0;j<k_receiver;j++)
			{
				if (pm->player_id==owner->m_receiver[j])
				{
					//     
					m_waited_player[k_waited]=pm->player_id;
					k_waited++;
					break;
				}
			}
		}
	}

	if (!k_waited)
	{
		//   
		on_next_net_takt=true;
		if (!owner->game->OnNextNetTakt())
		{
			//  
			attempt_sync--;
			if (attempt_sync>0)
			{
				owner->game->OnStartSynchronization(attempt_sync+1);
				if (owner->IsHost())
					sync_mode=true;
			}
			else
				owner->disconnect_state=NET_DISCONNECT_SYNCHRONIZATION;
		}
	}

	return k_waited;
}

//   
void GNetStadyGame::Initialization()
{
	ClearPlayerData();

	takt=0;
	takt_time=owner->platform->GetTick();
	ping_index=0;
	on_next_net_takt=false;
	sync_mode=false;

	sync_data->SetPositionIndirect(0);

	k_player=owner->RefreshReceiverList();
	if (k_player)
	{
		m_player=new PLAYER_MESSAGE[k_player];

		for (int i=0;i<k_player;i++)
		{
			PLAYER_MESSAGE* pm=&(m_player[i]);
			pm->player_id=owner->m_receiver[i];
			pm->receive_takt=0;
			int len=1;
			pm->next_message.SetMaxSize(len);
			pm->next_message.Reset();
			owner->game->ClearPlayerNetMessage(i);
		}
		owner->game->ClearPlayerNetMessage(-1);

		m_waited_player=new unsigned int[k_player];
	}
}

//   
bool GNetStadyGame::OnStart(NET_STADY previous, void* init)
{
	attempt_sync=5;

	if (previous==NET_STADY_MIGRATION_HOST)
	{
		return true;
	}

	if (previous==NET_STADY_START_GAME)
	{
		Initialization();

		owner->game->OnLaunchNetGame();
		return true;
	}
	ClearPlayerData();

	return false;
}

//        
void GNetStadyGame::OnUpdate()
{
	GNetStady::OnUpdate();
	GPlatform* platform=owner->platform;
	unsigned int time=platform->GetTick();
	unsigned int current_takt=owner->game->GetNetTakt();
	if (current_takt!=takt)
	{
		//    
		takt=current_takt;
		takt_time=time;
		ping_index=0;
		on_next_net_takt=false;

		GPlatform* platform=owner->platform;
		takt_time=platform->GetTick();

		//     ,    
		for (int i=0;i<k_player;i++)
		{
			NET_BUFFER* buf=&(m_player[i].next_message);
			int len=buf->GetLength();
			if (len)
			{
				ReceivePlayerCommands(m_player[i].player_id, len, buf->GetData(), takt);
				buf->Reset();
			}
		}

		MEM_DATA message=owner->game->GetPlayerNetMessage();

		GNetSocket* socket=owner->GetMainSocket();
		owner->game->SetPlayerNetMessage(socket->player_id, message); //     

		int k_receiver=owner->RefreshReceiverList();
		owner->m_receiver[k_receiver]=takt; //      ,    
		NET_BUFFER_INDEX* result=owner->PrepareMessageForPlayers(MESSAGE_TYPE_PLAYER_GAME, message.length, message.data, k_receiver, owner->m_receiver, 1);
		socket->SendMessage(result);

		owner->game->ClearPlayerNetMessage(-1); //   
	}
	else
	{
		if (!sync_mode)
		{
			if (!on_next_net_takt)
			{
				//     
				int k_waited_player=GetWaitedPlayers();
				if (k_waited_player)
				{
					GPlatform* platform=owner->platform;
					unsigned int time=platform->GetTick();

					unsigned int dtime=time-takt_time;
					unsigned int message_time=3*1000;

					int current_ping_index=dtime/1000;
					if (current_ping_index!=ping_index)
					{
						//    ,       
						ping_index=current_ping_index;
						owner->SendPing();
					}

					if (dtime>message_time)
					{
						//      
						unsigned int reset_time=30*1000;
						if (dtime>reset_time)
						{
							//    30 
							//    
							for (int i=0;i<k_waited_player;i++)
							{
								owner->LostPlayer(m_waited_player[i]);
							}
						}
						else
							owner->game->OnWaitingPlayers(reset_time-dtime, k_waited_player, m_waited_player);
					}
				}
				else
				{
					//    ,     
					if (!(takt % 100))
					{
						int k_receiver=owner->RefreshReceiverList();
						if (!k_receiver)
						{
							//   ,      ping,    
							owner->SendPing();
						}
					}
				}
			}
		}
	}
}

//     
bool GNetStadyGame::IsMessageCorrected(int message_type)
{
	return (message_type==MESSAGE_TYPE_PLAYER_GAME || message_type==MESSAGE_TYPE_SYNC_GAME);
}


GNetStadyMigrationHost::GNetStadyMigrationHost(GNetClient* owner) : GNetStady(owner)
{
	stady_period=300;	
}

//   ,      
bool GNetStadyMigrationHost::OnStart(NET_STADY previous, void* init)
{
	GNetSocket* main_socket=owner->GetMainSocket();
	if (main_socket)
	{
		migration_player_id=main_socket->player_id;
		k_migration_id=owner->RefreshReceiverList();
		owner->m_receiver[k_migration_id]=migration_player_id;
		k_migration_id++;
		for (int i=0;i<k_migration_id;i++)
			m_migration_id[i]=owner->m_receiver[i];
	}

	int k_receiver=owner->RefreshReceiverList();

	bool is_host=owner->IsHost();
	owner->DestroySocket();
	if (previous==NET_STADY_GAME)
	{
		if (GNetStady::OnStart(previous, init))
		{
			is_connected=false;
			attempts=20;
			if (is_host)
			{
				if (!IsLocalServer())
				{
					//      
					CMagicString path=GetPathToMigrationServerConfig();
					CMagicStreamFile stream;
					stream.Open(path, MAGIC_STREAM_WRITE);

					//int k_receiver=owner->RefreshReceiverList();
					//owner->m_receiver[k_receiver]=migration_player_id;
					//k_receiver++;

					stream<<k_migration_id;
					for (int i=0;i<k_migration_id;i++)
						stream<<m_migration_id[i];

					NET_SESSION_INFO si(owner);
					owner->game->GetSessionInfoStruct(&si);
					si.Serialize(stream);
					
					stream.Close();
					
					bool is=owner->StartLocalServer();
					//owner->platform->RemoveFile(path);
					return is;
				}
			}
			else
				return true;
		}
	}
	return false;
}

//     
void GNetStadyMigrationHost::OnPeriod()
{
	attempts--;
	if (attempts<0)
	{
		//   
		//    
		int k_receiver=owner->RefreshReceiverList();
		for (int i=0;i<k_receiver;i++)
		{
			owner->LostPlayer(owner->m_receiver[i]);
		}
		//    
		OnStart(NET_STADY_GAME, NULL);
	}
	else
	{
		if (!is_connected)
		{
			//    
			//       ,        
			int i=0;
			NET_PLAYER np(owner);
			do 
			{
				owner->game->GetSessionPlayerStruct(i, &np);
				i++;
			}
			while (np.type!=PLAYER_MAN);

			CMagicString ip=np.ip;
			if (ip.IsEmpty())
				ip="127.0.0.1";
			is_connected=owner->ConnectToServer(ip);
			GNetSocket* socket=owner->GetMainSocket();

			socket->player_id=migration_player_id;
		}

		if (is_connected)
		{
			//    ,     
			owner->SendHello();
		}
	}
}

//     
bool GNetStadyMigrationHost::IsMessageCorrected(int message_type)
{
	return (message_type==MESSAGE_TYPE_MIGRATION_OK);
}


// --------------------------------------------------------------------------

NET_CHAT_MESSAGE::NET_CHAT_MESSAGE()
{
	k_receiver=0;
	m_receiver=NULL;
	privated=false;
}

NET_CHAT_MESSAGE::~NET_CHAT_MESSAGE()
{
	ClearReceivers();
}

//     
void NET_CHAT_MESSAGE::Split(const char* mes, bool is_sender)
{
	CMagicString m=mes;
	sender="";
	ClearReceivers();
	int pos=0;
	if (is_sender)
	{
		//  
		pos=m.FindChar(':');
		if (pos!=-1)
		{
			// ,    
			sender=m.Left(pos);
			if (!sender.IsVariableName())
			{
				sender="";
				pos=0;
			}
			pos++;
		}
		else
			pos=0;
	}

	//  
	int pos2=m.FindChar('[', pos);
	if (pos2!=-1)
	{
		//   
		for (int i=pos;i<pos2;i++)
		{
			if (m[i]!=' ')
			{
				//       [
				pos2=-1;
				break;
			}

		}

		if (pos2!=-1)
		{
			pos2++;
			int pos3=m.FindChar(']', pos2);
			if (pos3!=-1)
			{
				//    
				CMagicString rec=m.Mid(pos2, pos3-pos2);
				rec.Trim();
				if (!rec.IsEmpty())
				{
					int pos4=0;
					while(true)
					{
						int vpos4=pos4;
						pos4=rec.FindChar(',', pos4);
						if (pos4==-1)
						{
							//   
							CMagicString receiver=rec.Right(rec.GetLength()-vpos4);
							receiver.Trim();
							if (receiver.IsVariableName())
							{
								AddReceiver(receiver);
								pos=pos3+1;
							}
							else
							{
								ClearReceivers();
							}
							break;
						}
						else
						{
							//   
							CMagicString receiver=rec.Mid(vpos4, pos4);
							receiver.Trim();
							if (receiver.IsVariableName())
							{
								AddReceiver(receiver);
								pos4++;
							}
							else
							{
								ClearReceivers();
								break;
							}
						}
					}
				}
			}
		}
	}

	//    
	message=m.Right(m.GetLength()-pos);
}

//      
void NET_CHAT_MESSAGE::Assemble(CMagicString& mes, bool is_sender)
{
	if (is_sender && (!sender.IsEmpty()))
	{
		mes=sender;
		mes+=": ";
	}
	else
		mes="";

	if (k_receiver>0)
	{
		mes+='[';
		mes+=m_receiver[0];
		for (int i=1;i<k_receiver;i++)
		{
			mes+=", ";
			mes+=m_receiver[i];
		}
		mes+=']';
	}
	mes+=message;
}

//  
void NET_CHAT_MESSAGE::AddReceiver(CMagicString& receiver)
{
	if (FindReceiver(receiver)==-1)
	{
		if (m_receiver)
		{
			CMagicString* vm_receiver=new CMagicString[k_receiver+1];
			for (int i=0;i<k_receiver;i++)
				vm_receiver[i]=m_receiver[i];
			delete []m_receiver;
			m_receiver=vm_receiver;
		}
		else
		{
			m_receiver=new CMagicString[1];
		}

		m_receiver[k_receiver]=receiver;
		k_receiver++;
	}
}

//  
void NET_CHAT_MESSAGE::DeleteReceiver(CMagicString& receiver)
{
	int index=FindReceiver(receiver);
	if (index!=-1)
	{
		if (k_receiver>1)
		{
			CMagicString* vm_receiver=new CMagicString[k_receiver-1];
			for (int i=0;i<index;i++)
				vm_receiver[i]=m_receiver[i];
			for (int i=index+1;i<k_receiver;i++)
				vm_receiver[i-1]=m_receiver[i];
			delete []m_receiver;
			m_receiver=vm_receiver;
			k_receiver--;
		}
		else
		{
			ClearReceivers();
		}
	}
}

// ,      
int NET_CHAT_MESSAGE::FindReceiver(CMagicString& receiver)
{
	for (int i=0;i<k_receiver;i++)
	{
		if (m_receiver[i]==receiver)
			return i;
	}
	return -1;
}

//  
void NET_CHAT_MESSAGE::ClearReceivers()
{
	if (m_receiver)
	{
		delete []m_receiver;
		m_receiver=NULL;
	}
	k_receiver=0;
}

void NET_CHAT_MESSAGE::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<sender;
		stream<<k_receiver;
		for (int i=0;i<k_receiver;i++)
			stream<<m_receiver[i];
		stream<<privated;
		stream<<message;
	}
	else
	{
		ClearReceivers();

		stream>>sender;
		stream>>k_receiver;
		if (k_receiver>0)
		{
			m_receiver=new CMagicString[k_receiver];
			for (int i=0;i<k_receiver;i++)
				stream>>m_receiver[i];
		}
		stream>>privated;
		stream>>message;
	}
}

// --------------------------------------------------------------------------

GNetClient::GNetClient() : GNet()
{
	CMagicString path=GetPathToMigrationServerConfig();
	platform->RemoveFile(path);

	waited_message_type=-1;
	connect_state=CONNECT_STATE_ERROR;
	net_stady=NET_STADY_NO;

	k_session=0;
	m_session=NULL;

	main_socket=NULL;

	k_net_stady=6;
	m_net_stady=new GNetStady*[k_net_stady];
	m_net_stady[NET_STADY_ENUM_SESSION]=new GNetStadyEnumSession(this);
	m_net_stady[NET_STADY_JOIN_SESSION]=new GNetStadyJoinSession(this);
	m_net_stady[NET_STADY_CREATE_SESSION]=new GNetStadyCreateSession(this);
	m_net_stady[NET_STADY_START_GAME]=new GNetStadyStartGame(this);
	m_net_stady[NET_STADY_GAME]=new GNetStadyGame(this);
	m_net_stady[NET_STADY_MIGRATION_HOST]=new GNetStadyMigrationHost(this);

	is_download_file=false;
	header_length=0;
	k_download_list=0;

	//lost_connection=false;
	disconnect_state=NET_DISCONNECT_OK;

	session_refresh_receiver=new NET_SESSION_INFO(this);
}

GNetClient::~GNetClient()
{
	for (int i=0;i<k_net_stady;i++)
	{
		delete m_net_stady[i];
		m_net_stady[i]=NULL;
	}

	if (m_net_stady)
	{
		delete []m_net_stady;
		m_net_stady=NULL;
	}
	k_net_stady=0;

	if (session_refresh_receiver)
	{
		delete session_refresh_receiver;
		session_refresh_receiver=NULL;
	}
}

bool GNetClient::Create(GUID_GAME guid, const char* server_ip)
{
	//lost_connection=false;
	disconnect_state=NET_DISCONNECT_OK;
	takt=0;
	if (server_ip)
	{
		this->server_ip=server_ip;
		internet=true;
	}
	else
	{
		this->server_ip="";
		internet=false;
	}

	net_stady=NET_STADY_NO;

	downloaded_file="";

	if (!GNet::Create(guid, internet))
		return false;

	return true;
}

void GNetClient::Destroy()
{
	//server_info.Clear();

	DestroySocket();
	GNet::Destroy();

	server_ip="";
	client_ip="";
}

//     
bool GNetClient::CreateSocket(bool udp_tcp)
{
	if ((!main_socket) || main_socket->IsTCP()!=udp_tcp)
	{
		//    
		if (main_socket)
		{
			main_socket->Destroy();
			delete main_socket;
			main_socket=NULL;
		}

		main_socket=NewSocket(this);
		if (udp_tcp)
		{
			if (!main_socket->Create(true, PORT_CLIENT_TCP, false))
				return false;
		}
		else
		{
			if (!main_socket->Create(false, PORT_CLIENT_UDP, true))
				return false;
		}
	}
	return true;
}

//      
void GNetClient::DestroySocket()
{
	StopLocalServer();

	if (main_socket)
	{
		main_socket->Destroy();
		delete main_socket;
		main_socket=NULL;
	}
}

//  
bool GNetClient::SetStady(NET_STADY stady, void* init)
{
	if (stady!=net_stady)
	{
		if (net_stady!=NET_STADY_NO)
		{
			//    
			m_net_stady[net_stady]->OnFinish(stady);
		}
		if (stady!=NET_STADY_NO)
		{
			//    
			if (!m_net_stady[stady]->OnStart(net_stady, init))
				return false;
		}
		net_stady=stady;
	}
	return true;
}

//      
NET_BUFFER_INDEX* GNetClient::PrepareMessageForPlayers(int type, int length, const unsigned char* data, int count_receiver, unsigned int* receiver_id, int receiver_count_minus)
{
	if (count_receiver>0)
	{
		count_receiver+=receiver_count_minus;

		int length2=length+sizeof(count_receiver)+sizeof(unsigned int)*count_receiver;
		int struct_size=sizeof(SEND_MESSAGE);
		int len=length2+struct_size;

		int len16=len;

		NET_BUFFER_LIST* list=GetSendList();
		MEM_DATA buf;
		buf.data=NULL;
		buf.length=len16;
		int index=list->AddBuffer(buf);
		NET_BUFFER_INDEX* bi=list->Get(index);
		bi->SetLength(len);
		unsigned char* buf_data=bi->GetData();
		SEND_MESSAGE* sm=(SEND_MESSAGE*)buf_data;
		sm->guid=guid;

		sm->sender=main_socket->player_id;
		sm->type=type;
		sm->length=length2;
		if (length2)
		{
			int* receiver=(int*)&(buf_data[struct_size]);
			*receiver=count_receiver-receiver_count_minus;
			receiver++;
			for (int i=0;i<count_receiver;i++)
			{
				*receiver=receiver_id[i];
				receiver++;
			}

			memcpy(receiver, data, length);
		}

		return bi;
	}
	return NULL;
}


//   
bool GNetClient::CreateSession()
{
	bool is=false;
	if (!internet)
	{
		//   
		if (StartLocalServer())
		{
			is=ConnectToServer("127.0.0.1");
		}
		else
			is=false;
	}
	else
	{
		is=true;
	}

	return is;
}

//   
void GNetClient::StartEnumSessionInfo()
{
	if (net_stady==NET_STADY_CREATE_SESSION)
		net_stady=net_stady;

	ClearEnumSessionInfo();

	CreateSocket(internet);

	wr1->Start();
	game->GetDigitalVersion()->Serialize(*wr1);
	MEM_DATA buf;
	wr1->Finish(buf);

	NET_BUFFER_INDEX* result=PrepareMessage(main_socket->player_id, MESSAGE_TYPE_ENUM_SESSION, buf.length, buf.data);

	if (internet)
	{
		main_socket->SendMessage(result, NULL);
	}
	else
	{
		NET_ADDRESS addr;
		addr.address="255.255.255.255";
		addr.port=PORT_SERVER_UDP;
		main_socket->SendMessage(result, &addr);
	}
}

//    
//        ,    
void GNetClient::StopEnumSessionInfo()
{
	game->OnEnumSession(&sessions, count_general_session);
	sessions.Clear();
}

//    
bool GNetClient::ConnectSession(NET_JOIN_DATA* join)
{
	count_general_session=0;
	bool is;
	if (!internet)
	{
		//   ,    
		is=ConnectToServer(join->ip);
	}
	else
	{
		is=true;
	}

	if (is)
	{
		//        
		SetWaitedReply(MESSAGE_TYPE_CONNECT_REPLY);
		CMagicString name=game->GetUserName();

		wr1->Start();
		(*wr1)<<join->password;
		(*wr1)<<name;
		(*wr1)<<client_ip;
		MEM_DATA buf;
		wr1->Finish(buf);
		NET_BUFFER_INDEX* result=PrepareMessageForPlayers(MESSAGE_TYPE_CONNECTING, buf.length, buf.data, 1, &join->host_id);//&(si->m_player[0]->player_id));
		main_socket->SendMessage(result);
		is=WaitReply();
	}

	return is;
}

//  
bool GNetClient::LostPlayer(unsigned int player_id)
{
	bool own_player=(main_socket->player_id==player_id);
	bool is_host=IsHost(player_id);
	NET_SESSION_INFO si(this);
	game->GetSessionInfoStruct(&si);
	for (int i=0;i<si.k_player;i++)
	{
		NET_PLAYER* np=si.m_player[i];
		if (np->player_id==player_id && np->type==PLAYER_MAN)
		{
			if (net_stady==NET_STADY_CREATE_SESSION || net_stady==NET_STADY_JOIN_SESSION || net_stady==NET_STADY_START_GAME)
			{
				//   
				if (net_stady==NET_STADY_START_GAME || is_host)
				{
					//    ,    
					SetStady(NET_STADY_ENUM_SESSION, (void*)GetServerIP());
					game->OnCancelSession();
				}
				else
				{
					if (main_socket->player_id==player_id)
					{
						//    
						SetStady(NET_STADY_ENUM_SESSION, (void*)GetServerIP());
						game->OnDeletingFromSession();
					}
				}
			}
			else
			{
				if (net_stady==NET_STADY_GAME || net_stady==NET_STADY_MIGRATION_HOST)
				{
					//     
					game->OnLostPlayer(player_id);
				}
			}

			if (main_socket)
			{
				//  ,        
				NET_SESSION_INFO si_new(this);
				game->GetSessionInfoStruct(&si_new);
				for (int j=0;j<si_new.k_player;j++)
				{
					NET_PLAYER* np_new=si_new.m_player[j];
					if (np_new->player_id==main_socket->player_id && np_new->type==PLAYER_MAN)
					{
						//     
						np->player_id=0;
						np->type=PLAYER_OPENED;
						game->SetSessionInfoStruct(&si);
						break;
					}
				}
			}

			break;
		}
	}
	return own_player;
}

bool GNetClient::ReceiveMessage(int type, unsigned int sender, int length, unsigned char* data)
{
	if (type==MESSAGE_TYPE_LOST_CONNECTION)
	{
		//     (     )
		if (sender==SERVER_ID && length==4)
		{
			unsigned int* player_id_ptr=(unsigned int*)data;
			unsigned int player_id=*player_id_ptr;

			return ReceiveLostPlayer(player_id);
		}
	}
	else
	{
		if (IsMessageCorrected(type))
		{
			//    
			switch (type)
			{
			case MESSAGE_TYPE_HELLO_REPLY:
				//    ,   id
				if (sender==SERVER_ID && length>=4 && main_socket->player_id==0)
				{
					rd1->Start(data);
					(*rd1)>>main_socket->player_id;
					if (!main_socket->player_id)
					{
						//    
						(*rd1)>>game->hello_server_reply;
					}
					else
					{
						//   
					}
				}
				break;

			case MESSAGE_TYPE_ENUM_SESSION_REPLY:
				//      
				if (sender==SERVER_ID)
				{
					//  :
					//  - MESSAGE_TYPE_ENUM_SESSION_REPLY
					// 1) IP- , .. 
					// 2)   
					// 3)   
					// 4)     
					//		 NET_SESSION_INFO,      

					rd2->Start(data);
					(*rd2)>>client_ip;

					(*rd2)>>count_general_session;

					int k_session;
					(*rd2)>>k_session;

					CMagicString ip;
					if (!receiving_socket->IsTCP())
					{
						ip=receiving_socket->addr->ToStringIP();
						//   TCP-,  UDP
						int pos=ip.Find(':');
						ip=ip.Left(pos+1);
						CMagicString d;
						d.IntToString(PORT_SERVER_TCP_GAME);
						ip+=d;
					}

					if (k_session)
					{
						NET_SESSION_INFO si(this);
						int i=0;
						do 
						{
							si.Serialize(*rd2);
							si.ip=ip;
							AddSessionInfo(&si);
							i++;
						}
						while (i<k_session);
					}
				}
				break;

			case MESSAGE_TYPE_CONNECTING:
				//          
				{
					int index=-1;
					if (net_stady==NET_STADY_CREATE_SESSION)
						index=game->GetIndexOfConnectedPlayer();

					//  :
					// int 0 - , 1 -  , 2 -  
					//  0,         NET_SESSION_INFO   
					wr1->Start();

					int rlt=0;
					if (index==-1)
						rlt=2; //  

					if (!rlt)
					{
						//  
						rd1->Start(data);
						CMagicString password;
						(*rd1)>>password;
						
						GNetStadyCreateSession* stady=(GNetStadyCreateSession*)m_net_stady[net_stady];
						CMagicString psw=stady->GetPassword();
						if (!psw.IsEmpty())
						{
							//  
							if (psw!=password)
							{
								//  
								rlt=1;
							}
						}
					}

					(*wr1)<<rlt;
					if (!rlt)
					{
						NET_SESSION_INFO si(this);
						game->GetSessionInfoStruct(&si);
						si.m_player[index]->player_id=sender;
						si.m_player[index]->type=PLAYER_MAN;
						(*rd1)>>si.m_player[index]->name;
						(*rd1)>>si.m_player[index]->ip;
						game->SetSessionInfoStruct(&si);
						game->GetSessionInfoStruct(&si);
						si.Serialize(*wr1);

						//   
						GNetStadyJoinSession* session_join=(GNetStadyJoinSession*)GetStadyPtr(NET_STADY_JOIN_SESSION);
						NET_CHAT_MESSAGE_LIST* chat=&(session_join->chat);
						game->GetChatList(chat);
						int k_chat=chat->GetCount();
						(*wr1)<<k_chat;
						for (int i=0;i<k_chat;i++)
						{
							NET_CHAT_MESSAGE* c=chat->Get(i);
							c->Serialize(*wr1);
						}

						game->OnNewPlayer(index);
					}

					MEM_DATA buf;
					wr1->Finish(buf);
					NET_BUFFER_INDEX* result=PrepareMessageForPlayers(MESSAGE_TYPE_CONNECT_REPLY, buf.length, buf.data, 1, &sender);
					main_socket->SendMessage(result);
				}
				break;
	 
			case MESSAGE_TYPE_CONNECT_REPLY:
				//         
				{
					//  :
					// int 0 - , 1 -  , 2 -  
					//  0,         NET_SESSION_INFO   
					rd1->Start(data);
					int rlt;
					(*rd1)>>rlt;
					GNetStadyJoinSession* stady=((GNetStadyJoinSession*)m_net_stady[NET_STADY_JOIN_SESSION]);
					stady->SetConnectionResult(rlt);
					if (!rlt)
					{
						NET_SESSION_INFO si(this);
						si.Serialize(*rd1);
						game->SetSessionInfoStruct(&si);
				
						//   
						int k_chat;
						(*rd1)>>k_chat;
						if (k_chat)
						{
							stady->chat.Create(k_chat);
							for (int i=0;i<k_chat;i++)
							{
								NET_CHAT_MESSAGE* c=stady->chat.Get(i);
								c->Serialize(*rd1);
							}
						}
					}
					else
					{
						//   
						//  -  false  WaitReply()
						SetWaitedReply(-2);
					}
				}
				break;

			case MESSAGE_TYPE_PLAYER_INFO:
				//      
				{
					//  :
					//  ,    
					//   - NET_PLAYER
					rd1->Start(data);
					int index_player;
					(*rd1)>>index_player;
					NET_SESSION_INFO* copy_session=((GNetStadyJoinSession*)m_net_stady[net_stady])->GetCopySession();
					NET_PLAYER* copy_player=copy_session->m_player[index_player];
					unsigned int player_id=copy_player->player_id;
					int copy_type=copy_player->type;
					unsigned int copy_player_id=copy_player->player_id;
					copy_player->Serialize(*rd1);
					if (copy_type==PLAYER_MAN && copy_player->type!=PLAYER_MAN)
					{
						//   
						//    ,       LostPlayer()
						return ReceiveLostPlayer(player_id);
					}
					else
						game->SetSessionPlayerStruct(index_player, copy_player);
				}
				break;

			case MESSAGE_TYPE_START_GAME:
				//    ,   
				{
					//         NET_STADY_START_GAME
					rd1->Start(data);
					NET_SESSION_INFO* copy_session=((GNetStadyJoinSession*)m_net_stady[net_stady])->GetCopySession();
					copy_session->Serialize(*rd1);
					game->SetSessionInfoStruct(copy_session);
					int start_rnd;
					(*rd1)>>start_rnd;
					int net_speed;
					(*rd1)>>net_speed;
					game->SetStartRND(start_rnd);
					game->SetNetGameSpeed(net_speed);
					SetStady(NET_STADY_START_GAME, NULL);
				}
				break;

			case MESSAGE_TYPE_PLAYER_STARTED:
				//    ,          
				{
					GNetStadyStartGame* stady_start_game=(GNetStadyStartGame*)m_net_stady[net_stady];
					for (int i=0;i<stady_start_game->k_waited_player;i++)
					{
						if (stady_start_game->m_waited_player[i]==sender)
						{
							//      
							for (int j=i+1;j<stady_start_game->k_waited_player;j++)
								stady_start_game->m_waited_player[j-1]=stady_start_game->m_waited_player[j];
							stady_start_game->k_waited_player--;
							break;
						}
					}
				}
				break;

			case MESSAGE_TYPE_GIVE_MAP:
				//       
				{
					GNetStadyCreateSession* stady_create=(GNetStadyCreateSession*)m_net_stady[net_stady];
					GMemWriter* writter=stady_create->GetWritterTransferMap();
					if (!writter->GetPosition())
					{
						//      
						game->OnPrepareTransferedMap(writter);
						int v=0;
						*writter<<v;	//   ,    
					}
					stady_create->AddTransferMap(sender);
				}
				break;
			
			case MESSAGE_TYPE_DOWNLOAD_MAP:
				//    
				{
					GNetStadyJoinSession* stady_join=(GNetStadyJoinSession*)m_net_stady[net_stady];
					GMemWriter* writter=stady_join->GetWritterTransferMap();

					int len=length-4; //     
					writter->Write(data, len);

					int* len_pos=(int*)&data[len];

					int position=writter->GetPosition();
					int percent=position*100/(*len_pos);
					game->OnDownloadingMapPercent(percent);

					if (writter->GetLength()==*len_pos)
					{
						//   
						MEM_DATA buf;
						writter->Finish(buf);
						game->OnMapWasDownloaded(buf);
					}
				}
				break;

			case MESSAGE_TYPE_CHAT_FROM_SERVER:
				{
					//  -
					rd1->Start(data);
					int k;
					(*rd1)>>k;
					
					for (int i=0;i<k;i++)
					{
						CMagicString from;
						(*rd1)>>from;
						CMagicString to;
						(*rd1)>>to;
						bool privated;
						(*rd1)>>privated;
						bool global;
						(*rd1)>>global;
						CMagicString message;
						(*rd1)>>message;
						bool printed_message=(i==0);
						game->OnChatMessage(from, to, privated, global, message, printed_message);
					}
				}
				break;

			case MESSAGE_TYPE_PING_REPLY:
				//    ping
				{
					if (length>0)
					{
						rd1->Start(data);
						game->OnPingServerInfo(rd1);
					}
					GNetStadyJoinSession* stady_join=(GNetStadyJoinSession*)m_net_stady[net_stady];
					unsigned int sent_ping=stady_join->GetPingTakt();
					int index_player=stady_join->GetIndexPlayer();
					NET_PLAYER np(this);
					game->GetSessionPlayerStruct(index_player, &np);
					unsigned int razn=takt-sent_ping;
					if (np.ping!=razn)
					{
						np.ping=razn;
						game->SetSessionPlayerStruct(index_player, &np);
					}
				}
				break;

			case MESSAGE_TYPE_PLAYER_GAME:
				//        
				{
					if (net_stady==NET_STADY_START_GAME)
					{
						//            
						//         ,   ,      
						SetStady(NET_STADY_GAME, NULL);
					}

					GNetStadyGame* stady_game=(GNetStadyGame*)m_net_stady[net_stady];
					unsigned int* idata=(unsigned int*)data;
					unsigned int net_takt=*idata;
					stady_game->ReceivePlayerCommands(sender, length, data+sizeof(unsigned int), net_takt);
				}
				break;

			case MESSAGE_TYPE_SYNC_GAME:
				//    save-   
				{
					GNetStadyGame* stady_game=(GNetStadyGame*)m_net_stady[net_stady];
					GMemWriter* writter=stady_game->GetWritterSyncData();

					writter->Write(data, length);
					MEM_DATA buf;
					writter->Finish(buf);
					game->OnSyncWasDownloaded(buf);

					stady_game->Initialization();

					game->OnFinishSynchronization();
				}
				break;

			case MESSAGE_TYPE_MIGRATION_OK:
				//    ,    
				
				SetStady(NET_STADY_GAME, NULL);
				break;

			
			}
		}
	}

	return true;
}

//   " "
bool GNetClient::ReceiveLostPlayer(unsigned int player_id)
{
	unsigned int created_socket_time=receiving_socket->time;
	bool is=!LostPlayer(player_id);
	if (is)
	{
		//   ,     
		if (main_socket)
			is=(main_socket->time==created_socket_time);
		else
			is=false;
	}
	return is;
}

//   -
bool GNetClient::ConnectToServer(const char* ip)
{
	const CMagicString* license=game->GetLicense();
	if (license || IsDownloader())
	{
		if (CreateSocket(true))
		{
			NET_ADDRESS addr;
			addr.FromStringIP(ip);
			if (IsDownloader())
				addr.port=80;

			connect_state=CONNECT_STATE_WAITING;
			
			bool is_connected=false;
			if (addr.address=="127.0.0.1")
			{
				//     
				for (int i=0;i<25;i++)
				{
					if (main_socket->Connect(&addr))
					{
						if (connect_state==CONNECT_STATE_OK)
						{
							is_connected=true;
							break;
						}
						else
						{
							if (connect_state==CONNECT_STATE_ERROR)
								break;
						}
					}
					platform->Sleep(300);
				}
			}
			else
			{
				if (main_socket->Connect(&addr))
				{
					if (connect_state==CONNECT_STATE_OK)
						is_connected=true;
				}
			}

			if (is_connected)
			{
				//      
				is_connected=main_socket->SetConnectedSocketToReadMode();
				if (is_connected && net_stady!=NET_STADY_MIGRATION_HOST && (!IsDownloader()))
				{
					//   ""
					SetWaitedReply(MESSAGE_TYPE_HELLO_REPLY);
					SendHello();

					is_connected=WaitReply();
					if (is_connected)
					{
						if (!main_socket->player_id)
							is_connected=false;
					}
				}
			}
			return is_connected;
		}
	}

	return false;
}



//  -
bool GNetClient::SendChatMessage(CMagicString& message, int k_name, CMagicString* m_name, bool privated, bool global)
{
	if (net_stady!=NET_STADY_NO)
	{
		if (m_net_stady[net_stady]->IsMessageCorrected(MESSAGE_TYPE_CHAT_FROM_SERVER))
		{
			wr1->Start();
			(*wr1)<<message;
			(*wr1)<<privated;
			(*wr1)<<global;
			(*wr1)<<k_name;
			for (int i=0;i<k_name;i++)
				(*wr1)<<m_name[i];
			MEM_DATA buf;
			wr1->Finish(buf);
			NET_BUFFER_INDEX* result=PrepareMessage(main_socket->player_id, MESSAGE_TYPE_CHAT_FROM_CLIENT, buf.length, buf.data);
			main_socket->SendMessage(result);
			
			return true;
		}
	}
	return false;
}

//   ,    
const char* GNetClient::GetServerIP()
{
	if (server_ip.IsEmpty())
		return NULL;
	return server_ip.c_str();
}

//     
bool GNetClient::IsMessageCorrected(int message_type)
{
	if (waited_message_type!=-1)
	{
		//   
		if (waited_message_type==message_type)
		{
			waited_message_type=-1;
			return true;
		}
		return false;
	}

	if (net_stady!=NET_STADY_NO)
		return m_net_stady[net_stady]->IsMessageCorrected(message_type);

	

	return false;
}

//      
bool GNetClient::WaitReply()
{
	for (int i=0;i<10*3;i++)
	{
		UpdateNet();

		if (main_socket)
		{
			NET_BUFFER* buffer=main_socket->GetReceivingBuffer();
			if (buffer->GetLength())
				main_socket->ReceiveMessages();
		}
		else
		{
			waited_message_type=-1;
			break;
		}

		if (waited_message_type==-1)
		{
			return true;
		}
		else
			if (waited_message_type==-2)
			{
				waited_message_type=-1;
				break;
			}

		game->platform->Sleep(300);
	}
	return false;
}

//        ( id    )
int GNetClient::RefreshReceiverList()
{
	int k_receiver=0;
	if (main_socket)
	{
		if (game->GetSessionInfoStruct(session_refresh_receiver))
		{
			for (int i=0;i<session_refresh_receiver->k_player;i++)
			{
				if (session_refresh_receiver->m_player[i]->type==PLAYER_MAN)
				{
					unsigned int id=session_refresh_receiver->m_player[i]->player_id;
					if (main_socket->player_id!=id)
					{
						m_receiver[k_receiver]=id;
						k_receiver++;
					}
				}
			}
		}
	}
	return k_receiver;
}

//   
void GNetClient::OnUpdate()
{
	if (IsDownloader())
	{
		if (!is_download_file)
		{
			if (k_download_list)
			{
				DownloadFile(m_download_list[k_download_list-1]);
				k_download_list--;
			}
		}
	}
	else
	{
		if (net_stady!=NET_STADY_NO)
			m_net_stady[net_stady]->OnUpdate();
	}
}

//     
void GNetClient::OnConnect(bool connected)
{
	if (connect_state==CONNECT_STATE_WAITING)
	{
		if (connected)
			connect_state=CONNECT_STATE_OK;
		else
			connect_state=CONNECT_STATE_ERROR;
	}
}

//  
void GNetClient::OnLostConnection(GNetSocket* socket)
{
	assert(main_socket==socket);
	if (IsDownloader())
	{
		if (main_socket)
		{
			main_socket->Destroy();
			delete main_socket;
			main_socket=NULL;
		}
	}
	else
	{
		bool is_migration=false;
		unsigned int player_id=main_socket->player_id;

		NET_SESSION_INFO si(this);
		game->GetSessionInfoStruct(&si);

		bool first_player=true;
		for (int i=0;i<si.k_player;i++)
		{
			NET_PLAYER* np=si.m_player[i];
			if (np->type==PLAYER_MAN)
			{
				if (np->player_id==player_id)
				{
					//    
					is_migration=true;
					first_player=false;
				}
				else
				{
					if (internet)
					{
						//   ,                
						LostPlayer(np->player_id);
						first_player=false;
					}
					else
					{
						//    ,    
						if (first_player)
						{
							LostPlayer(np->player_id);
							first_player=false;
						}
					}
				}
			}
		}

		if (is_migration && net_stady==NET_STADY_GAME && (!internet))
			SetStady(NET_STADY_MIGRATION_HOST, NULL);
		else
		{
			//lost_connection=true;
			disconnect_state=NET_DISCONNECT_LOST;
			if (main_socket)
			{
				main_socket->Destroy();
				delete main_socket;
				main_socket=NULL;
			}
		}
	}
}

// ,  
int state_license_verification=0;
unsigned int LicenseVerificationThread(void* this_server)
{
	GNetServer* server=(GNetServer*)this_server;

	int result=0xffffffff;

	#ifdef STEAM
	if (!InitSteam())
		return result;
	ISteamUser* steam_user=SteamUser();
	#endif

	state_license_verification=1;
	result=0;

	while (state_license_verification!=2)
	{
		while (server->licenses_for_verification->GetIndexCount())
		{
			//    
			NET_LICENSE* license=new NET_LICENSE;
			int index=server->licenses_for_verification->GetIndex(0);
			server->licenses_for_verification->Get(index, license);

			if (license->type==0)
			{
				CMagicString license_key=license->GetLicenseKey();
				if (server->game->VerifyLicense(license->type, license_key)!=1)
					license->type=-1;
			}
			#ifdef STEAM
			else
			{
				CSteamID* steam_id=(CSteamID*)license->key;
				if (steam_id->IsValid())
				{
					EUserHasLicenseForAppResult result=steam_user->UserHasLicenseForApp(*steam_id, server->game->GetGameSteamID());
					if (result!=k_EUserHasLicenseResultHasLicense)
						license->type=-1;
				}
				else
					license->type=-1;
			}
			#endif

			server->licenses_for_verification->DeleteLicense(index);

			//       
			server->licenses_result->AddLicense(license);
		}

		server->platform->Sleep(500);
	}

	#ifdef STEAM
	CloseSteam();
	#endif

	state_license_verification=0;

	return result;
}

//  ,  ,    
bool IsLicenseVerificationThread()
{
	return (state_license_verification!=0);
}

//      
unsigned int StartLocalServerThread(void* this_game)
{
	GGame* game=(GGame*)this_game;
	GNetServer* local_server=game->NewServer();

	int result=0xffffffff;
	if (local_server->Create(game->guid, false))
	{
		is_local_server=true;
		local_server->UpdateNet();
		local_server->Destroy();
		result=0;
	}

	delete local_server;

	is_local_server=false;

	return result;
}

//   
bool GNetClient::StartLocalServer()
{
	if (!IsLocalServer())
	{
		//    
		platform->ExecuteThread(StartLocalServerThread, game);
		while (true)
		{
			platform->Sleep(200);
			if (IsLocalServer())
				return true;
		}
	}
	return false;
}

//   
void GNetClient::StopLocalServer()
{
	while (IsLocalServer())
	{
		//   
		NET_BUFFER_INDEX* result=PrepareMessage(main_socket->player_id, MESSAGE_TYPE_STOP_SERVER, 0, NULL);
		main_socket->SendMessage(result);
		platform->Sleep(300);
	}
}

//  
bool GNetClient::IsHost(unsigned int player_id)
{
	int i=0;
	NET_PLAYER np(this);
	do 
	{
		game->GetSessionPlayerStruct(i, &np);
		i++;
	}
	while (np.type!=PLAYER_MAN);

	if (player_id)
		return (np.player_id==player_id);

	//   
	return (np.player_id==main_socket->player_id);
}

//  ,        
bool GNetClient::IsNextNetTakt()
{
	if (net_stady==NET_STADY_GAME)
	{
		GNetStadyGame* stady=(GNetStadyGame*)m_net_stady[NET_STADY_GAME];
		if (stady->IsNextNetTakt())
		{
			//    
			if (stady->IsSyncMode())
			{
				if (IsHost())
				{
					//   ,     
					GMemWriter* sync_data=stady->GetWritterSyncData();
					sync_data->Start();
					game->OnPrepareSyncData(sync_data);
					MEM_DATA data;
					sync_data->Finish(data);
					
					int k_receiver=RefreshReceiverList();
					NET_BUFFER_INDEX* result=PrepareMessageForPlayers(MESSAGE_TYPE_SYNC_GAME, data.length, (unsigned char*)(data.data), k_receiver, m_receiver);
					main_socket->SendMessage(result);

					MEM_DATA empty;
					empty.data=NULL;
					empty.length=0;
					game->OnSyncWasDownloaded(empty);
					stady->Initialization();

					game->OnFinishSynchronization();
				}
				return false;
			}
			else
			{
				stady->ResetNextNetTakt();
				return true;
			}
		}
	}
	return false;
}


//    
bool GNetClient::DownloadFile(const char* url_file)
{
	if (IsDownloader())
	{
		if (is_download_file)
		{
			//   ,      
			m_download_list[k_download_list]=url_file;
			k_download_list++;
		}
		else
		{
			//    
			CMagicString url=url_file;
			bool is_http=false;
			int pos=url.Find("/");
			if (pos!=-1)
			{
				int len=url.GetLength();
				if (len>pos+2)
				{
					pos++;
					if (url[pos]=='/')
					{
						is_http=true;
						pos=url.Find("/", pos+1);
					}
				}
				else
					pos=-1;

				if (pos!=-1)
				{
					CMagicString server_addr=url.Left(pos);
					if (is_http)
					{
						int pos=url.Find("/");
						pos+=2;
						server_addr=server_addr.Right(server_addr.GetLength()-pos);
					}
					CMagicString file_path=url.Right(len-pos);
					CMagicString file_ip=URLtoIP(server_addr);
					if (!file_ip.IsEmpty())
					{
						if (server_ip!=file_ip)
						{
							//   
							if (!guid.IsEmpty())
							{
								//    
								Destroy();
							}

							if (!Create(game->guid, file_ip))
								return false;
						}

						if ((!main_socket) || (!main_socket->IsTCP()))
						{
							//   
							if (!ConnectToServer(file_ip))
								return false;
						}

						//       
						char ret='\n';
						CMagicString mes="GET ";
						mes+=file_path;
						mes+=" HTTP/1.0";
						mes+=ret;
						mes+="Host: ";
						mes+=server_addr;
						mes+=ret;
						mes+="Connection: keep-alive";
						mes+=ret;
						mes+=ret;

						is_download_file=true;
						header_length=0;
						downloaded_file_length=0;
						wr1->Start();

						downloaded_file=file_path.GetFileNameOnly();

						MEM_DATA buf;
						buf.data=(unsigned char*)mes.c_str();
						buf.length=mes.GetLength();

						NET_BUFFER_LIST* list=GetSendList();
						int index=list->AddBuffer(buf);
						NET_BUFFER_INDEX* bi=list->Get(index);

						main_socket->SendMessage(bi);

						return true;
					}
				}
			}
		}
	}
	return false;
}

//     (  ,     )
void GNetClient::ReceiveFile(NET_BUFFER* buffer)
{
	if (is_download_file)
	{
		wr1->Write(buffer->data, buffer->length);

		if (!header_length)
		{
			//  

			//        
			char ret='\n';
			char* data=wr1->GetBuffer();
			int pos=wr1->GetPosition();
			header_length=0;
			for (int i=0;i<pos;i++)
			{
				if (data[i]==ret)
				{
					if (i+1<pos)
					{
						if (data[i+1]=='\r' && i+2<pos)
						{
							i++;
							if (data[i+1]==ret)
							{
								header_length=i+2;
								data[header_length-1]=0;
								CMagicString header=data;
								int pos=header.Find("200");
								CMagicString cl="Content-Length:";
								int pos_len=header.Find(cl, pos);
								if (pos!=-1 && pos_len!=-1)
								{
									//    
									pos_len+=cl.GetLength();

									while (header[pos_len]==' ')
										pos_len++;

									CMagicString len_file;
									while (true)
									{
										char s=header[pos_len];
										if (s>='0' && s<='9')
										{
											len_file+=s;
											pos_len++;
										}
										else
											break;
									}

									downloaded_file_length=len_file.StringToInt();
								}

								if (!downloaded_file_length)
								{
									//     
									MEM_DATA empty;
									empty.data=NULL;
									empty.length=0;
									game->OnDownloadFile(downloaded_file, &empty);
									is_download_file=false;
								}

								break;
							}
						}
					}
				}
			}
		}

		if (downloaded_file_length)
		{
			//     
			int position=wr1->GetPosition();
			int sum=header_length+downloaded_file_length;
			if (sum<=position)
			{
				//  
				MEM_DATA buf;
				wr1->Finish(buf);
				buf.data+=header_length;
				buf.length-=header_length;
				if (buf.length!=downloaded_file_length)
				{
					buf.data=NULL;
					buf.length=0;
				}
				game->OnDownloadFile(downloaded_file, &buf);

				is_download_file=false;
				
				delete wr1;
				wr1=new GMemWriter;
			}
		}
	}
}

//         
int GNetClient::AddSessionInfo(NET_SESSION_INFO* session_info)
{
	int index=-1;
	unsigned int host_id=session_info->m_player[0]->player_id;
	int k_session_info=sessions.GetIndexCount();
	for (int i=0;i<k_session_info;i++)
	{
		int index=sessions.GetIndex(i);
		NET_SESSION_INFO* si=sessions.Get(index);
		if (si->tick==session_info->tick && si->id_map==session_info->id_map && si->m_player[0]->player_id==host_id)
		{
			index=i;
			break;
		}
	}
	
	if (index==-1)
	{
		//   
		NET_SESSION_SERVER* session_server=new NET_SESSION_SERVER(this);
		((NET_SESSION_INFO&)*session_server)=*session_info;
		index=sessions.AddSession(session_server);
	}

	return index;
}

//  
void GNetClient::SendHello()
{
	CMagicString name=game->GetUserName();
	wr1->Start();
	int version=0;
	(*wr1)<<version;
	int client_version=game->GetServerCompatibleVersion();
	(*wr1)<<client_version;
	
	(*wr1)<<name;
	int lic_type=game->GetLicenseType();
	(*wr1)<<lic_type;
	CMagicString lic=*game->GetLicense();
	
	(*wr1)<<lic;
	MEM_DATA buf;
	wr1->Finish(buf);
	NET_BUFFER_INDEX* result=PrepareMessage(main_socket->player_id, MESSAGE_TYPE_HELLO, buf.length, buf.data);
	main_socket->SendMessage(result);
}

//   PING
void GNetClient::SendPing()
{
	int k_receiver=RefreshReceiverList();
	wr1->Start();
	(*wr1)<<k_receiver;
	for (int i=0;i<k_receiver;i++)
		(*wr1)<<m_receiver[i];
	MEM_DATA buf;
	wr1->Finish(buf);
	NET_BUFFER_INDEX* result=PrepareMessage(main_socket->player_id, MESSAGE_TYPE_PING, buf.length, buf.data);
	main_socket->SendMessage(result);
}


// --------------------------------------------------------------------------

NET_LICENSE::NET_LICENSE()
{
	socket=NULL;
	type=-1;
	length=0;
	key=NULL;
}
NET_LICENSE::~NET_LICENSE()
{
	if (key)
	{
		delete []key;
		key=NULL;
	}
}

void NET_LICENSE::Create(GNetSocket* socket, int type, CMagicString* password)
{
	this->socket=socket;
	this->type=type;

	length=password->GetLength();
	if (length>0)
	{
		key=new char[length];
		memcpy(key, password->c_str(), length);
	}
}

NET_LICENSE& NET_LICENSE::operator=(const NET_LICENSE& nl)
{
	socket=nl.socket;
	type=nl.type;
	if (length!=nl.length)
	{
		length=nl.length;
		if (key)
		{
			delete []key;
			key=NULL;
		}
		if (length)
			key=new char[length];
	}
	if (length)
		memcpy(key, nl.key, length);

	return *this;
}

CMagicString NET_LICENSE::GetLicenseKey()
{
	CMagicString lic;
	if (key)
	{
		switch(type)
		{
		case 0:
			{
				lic.SetLength(length);
				memcpy((void*)lic.c_str(), key, length);
			}
			break;

		#ifdef STEAM
		case 1:
			{
				CSteamID* steam_id=(CSteamID*)key;
				unsigned int id=steam_id->GetAccountID();
				lic.Format("%u", id);
			}
			break;
		#endif

		default:
			assert(false);
			break;
		}
	}
	return lic;
}

// --------------------------------------------------------------------------

GLicenseList::GLicenseList(GPlatform* platform) : CArrayBase()
{
	critical_secion=platform->NewCriticalSection();

	k_license=10;
	m_license=(NET_LICENSE**)malloc(k_license*sizeof(NET_LICENSE*));
	for (int i=k_license-1;i>=0;i--)
	{
		m_license[i]=NULL;
		IncreaseDeleted(i, i);
	}
}

GLicenseList::~GLicenseList()
{
	Clear();

	delete critical_secion;
}

void GLicenseList::Clear()
{
	for (int i=0;i<k_license;i++)
	{
		if (m_license[i])
		{
			delete m_license[i];
			m_license[i]=NULL;
		}
	}

	if (m_license)
	{
		free(m_license);
		m_license=NULL;
	}

	k_license=0;

	ClearExistingAndDeleted();
}

//     
bool GLicenseList::Get(int index, NET_LICENSE* to)
{
	bool is;
	critical_secion->Lock();
	if (index>=0 && index<k_license)
	{
		*to=*(m_license[index]);
		is=true;
	}
	else
		is=false;
	critical_secion->Unlock();
	return is;
}

//  
int GLicenseList::AddLicense(NET_LICENSE* license)
{
	critical_secion->Lock();

	int index=FromDeletedToExisting();

	if (index==-1)
	{
		int size=sizeof(NET_LICENSE*);
		int k_license2=k_license+k_license/4;
		m_license=(NET_LICENSE**)realloc(m_license,size*(k_license2));

		for (int i=k_license;i<k_license2;i++)
			m_license[i]=NULL;

		IncreaseDeleted(k_license, k_license2-1);
		index=FromDeletedToExisting();

		k_license=k_license2;
	}

	m_license[index]=license;

	critical_secion->Unlock();

	return index;
}

//    
bool GLicenseList::DeleteLicense(int index)
{
	bool is;

	critical_secion->Lock();

	if (index>=0 && index<k_license)
	{
		if (m_license[index])
		{
			delete m_license[index];
			m_license[index]=NULL;
			FromExistingToDeleted(index);

			is=true;
		}
	}
	else
		is=false;

	critical_secion->Unlock();

	return is;
}

// --------------------------------------------------------------------------

// ,     
GChatList::GChatList()
{
	k_message=0;
	max_message=0;
	m_message=NULL;
}

GChatList::~GChatList()
{
	for (int i=0;i<max_message;i++)
	{
		delete m_message[i];
		m_message[i]=NULL;
	}

	if (m_message)
	{
		delete []m_message;
		m_message=NULL;
	}
	max_message=0;
	k_message=0;
}

void GChatList::Reset()
{
	k_message=0;
}

void GChatList::AddMessage(const char* sender, const char* receiver, const char* message)
{
	if (k_message==max_message)
	{
		//   
		int step=max_message/4;
		if (step<4)
			step=4;
		int max_message2=max_message+step;
		CHAT_GLOBAL** vm_message=new CHAT_GLOBAL*[max_message2];

		int i;
		for (i=0;i<k_message;i++)
			vm_message[i]=m_message[i];

		for (i=k_message;i<max_message2;i++)
			vm_message[i]=new CHAT_GLOBAL;

		if (m_message)
			delete []m_message;
		m_message=vm_message;
		max_message=max_message2;
	}
	
	CHAT_GLOBAL* c=m_message[k_message];
	c->sender=sender;
	c->receiver=receiver;
	c->message=message;
	k_message++;
}

// --------------------------------------------------------------------------

// ,     
GNetServer::GNetServer() : GNet()
{
	shutdown_type=SHUTDOWN_TYPE_NO;

	stop_time=0;
	stop_server=false;

	max_player=400;

	k_migration_player=0;
	m_migration_player=NULL;
	migration_session=NULL;
	migration_ok=false;

	start_time=0;
	count_hello=count_accept=0;

	licenses_for_verification=licenses_result=NULL;
	softly_reset=false;
}

GNetServer::~GNetServer()
{
	Destroy();

	if (m_migration_player)
	{
		delete []m_migration_player;
		m_migration_player=NULL;
	}
	k_migration_player=0;

	if (migration_session)
	{
		delete migration_session;
		migration_session=NULL;
	}
}

bool GNetServer::Create(GUID_GAME guid, bool internet)
{
	if (GNet::Create(guid, internet))
	{
		softly_reset=false;
		//   -  
		CMagicString path=platform->GetPathToConfig();
		for (int i=0;i<MAX_LICENSE_TYPE;i++)
		{
			CMagicString file=path+"bans";
			CMagicString d;
			d.IntToString(i);
			if (d.GetLength()==1)
				file+='0';
			file+=d;
			file+=".ctl";

			CMagicStreamFile f;
			if (f.Open(file))
			{
				CMagicStringTable* bans=&(statistics[i].bans);
				bans->Serialize(f, false);
				f.Close();
			}

			file=path+"errors";
			d.IntToString(i);
			if (d.GetLength()==1)
				file+='0';
			file+=d;
			file+=".ctl";

			if (f.Open(file))
			{
				CMagicStringTable* errors=&(statistics[i].errors);
				errors->Serialize(f, false);
				f.Close();
			}
		}

		GNetSocket* socket=NewSocket(this);
		bool is=socket->Create(true, PORT_SERVER_TCP_GAME, true);
		if (is)
		{
			ConnectSocket(socket);
			assert(socket->player_id==SERVER_ID);

			socket=NewSocket(this);
			if (internet)
			{
				is=socket->Create(true, PORT_SERVER_TCP_ADMIN, true);
				socket->SetAdmin();
			}
			else
				is=socket->Create(false, PORT_SERVER_UDP, true);
			if (is)
			{
				ConnectSocket(socket);
				assert(socket->player_id==SERVER_ID+1);

				CMagicString path=GetPathToMigrationServerConfig();
				CMagicStreamFile stream;
				if (stream.Open(path))
				{
					//      ,     
					int i;
					stream>>k_migration_player;
					m_migration_player=new unsigned int[k_migration_player];
					for (i=0;i<k_migration_player;i++)
						stream>>m_migration_player[i];

					migration_session=new NET_SESSION_INFO(this);
					migration_session->Serialize(stream);

					stream.Close();
					platform->RemoveFile(path);

					//        id 
					unsigned int max_id=0;
					for (i=0;i<k_migration_player;i++)
					{
						if (max_id<m_migration_player[i])
							max_id=m_migration_player[i];
					}
					sockets.Expand(max_id+1);
				}
				else
				{
					//      
					StartLicenseVerificationThread();
				}


				start_time=platform->GetTick();

				migration_ok=false;

				return true;
			}
		}
	}

	return false;
}

void GNetServer::Destroy()
{
	StopLicenseVerificationThread();

	sockets.Clear();

	deleted_sockets_during_verification.Clear();

	if (internet)
	{
		//     
		CMagicString d;
		CMagicString path=platform->GetPathToConfig();
		for (int i=0;i<MAX_LICENSE_TYPE;i++)
		{
			CMagicString file=path+"errors";

			d.IntToString(i);
			if (d.GetLength()==1)
				file+='0';
			file+=d;
			file+=".ctl";

			CMagicStreamFile f;
			if (f.Open(file, MAGIC_STREAM_WRITE))
			{
				CMagicStringTable* errors=&(statistics[i].errors);
				errors->Serialize(f, false);
				f.Close();
			}
		}
	}

	GNet::Destroy();
}

//    
void GNetServer::SetAdminData(const char* login, const char* password)
{
	this->login=login;
	this->password=password;
}

void GNetServer::ConnectSocket(GNetSocket* socket, unsigned int player_id)
{
	int index=sockets.AddSocket(socket, player_id);
	socket->player_id=index;
	socket->SetTime(platform->GetTick());
}


//  tcp- ,   
GNetSocket* GNetServer::GetServerTCPSocket()
{
	int max=sockets.GetMaxCount();
	if (max>0)
		return sockets.Get(0);
	return NULL;
}

//  udp- ,   
GNetSocket* GNetServer::GetServerUDPSocket()
{
	int max=sockets.GetMaxCount();
	if (max>1)
		return sockets.Get(1);
	return NULL;
}

//    
bool GNetServer::ReceiveMessage(int type, unsigned int sender, int length, unsigned char* data)
{
	ServiceLicenses();

	if (receiving_socket->IsAdmin())
	{
		
	}
	else
	{
		if (!sender)
		{
			if (receiving_socket->IsTCP())
			{
				if (type!=MESSAGE_TYPE_HELLO)
				{
					//        - 
					//      
					//receiving_socket->Destroy();
					//delete receiving_socket;
					//receiving_socket=NULL;
					receiving_socket->Error();
					return false;
				}
			}
			else
			{
				if (type!=MESSAGE_TYPE_ENUM_SESSION)
				{
					//  UDP-      
					return false;
				}
			}	
		}

		switch(type)
		{
			case MESSAGE_TYPE_HELLO:
				{
					//       ,     
					int index_nohello=sockets_nohello.FindIndex(receiving_socket);
					if (index_nohello!=-1)
						sockets_nohello.Delete(index_nohello);

					rd1->Start(data);
					int vlen=length;
					if (vlen>4)
					{
						int version;
						(*rd1)>>version;
						vlen-=4;
						if (!version)
						{
							int client_version;
							CMagicString name;
							int license_type;
							CMagicString license;

							bool is=false;
							if (vlen>4)
							{
								(*rd1)>>client_version;
								vlen-=4;

								int crypt_rnd=game->GetCryptRnd();
								unsigned char slen;
								if (vlen>1)
								{
									//  
									(*rd1)>>slen;
									int name_len=slen;
									vlen-=1;
									if (vlen>name_len)
									{
										name.SetLength(name_len);
										rd1->Read((void*)name.c_str(), name_len);
										vlen-=name_len;
										if (vlen>4)
										{
											//   
											(*rd1)>>license_type;
											vlen-=4;
											if (vlen>1)
											{
												//  
												(*rd1)>>slen;
												name_len=slen;
												vlen-=1;
												if (vlen==name_len)
												{
													license.SetLength(name_len);
													rd1->Read((void*)license.c_str(), name_len);

													
													is=true;
												}
											}
										}
									}
								}
							}

							if (is)
							{
								if (IsMigration())
								{
									if (sender && (!migration_ok))
									{
										//    
										bool is=false;
										bool is_host=(m_migration_player[k_migration_player-1]==sender); //     
										bool has_session=(sessions.GetIndexCount()>0);
										if (is_host)
										{
											//  
											if (!has_session)
											{
												//   
												receiving_socket->name=name;
												ConnectSocket(receiving_socket, sender);
												AddSessionInfo(migration_session, receiving_socket);
												delete migration_session;
												migration_session=NULL;

												IsAllPlayersMigrated(&name, &license);
												is=true;
											}
										}
										else
										{
											if (has_session)
											{
												//  ,   
												GNetSocket* migration_player=sockets.Get(sender);
												if (!migration_player)
												{
													receiving_socket->name=name;
													ConnectSocket(receiving_socket, sender);
													int index=sessions.GetIndex(0);
													NET_SESSION_SERVER* ss=sessions.Get(index);
													ss->AddPlayer(receiving_socket);

													IsAllPlayersMigrated(&name, &license);
													is=true;
												}
											}
										}

										if (!is)
										{
											unsigned int current_time=platform->GetTick();
											unsigned int dtime=current_time-start_time;
											if (dtime>7000)
											{
												//      ,  c 
												for (int i=0;i<k_migration_player;i++)
												{
													GNetSocket* socket=sockets.Get(m_migration_player[i]);
													if (socket)
													{
														for (int j=0;j<k_migration_player;j++)
														{
															if (!sockets.Get(m_migration_player[j]))
															{
																//    
																NET_BUFFER_INDEX* result2=PrepareMessage(SERVER_ID, MESSAGE_TYPE_LOST_CONNECTION, 4, (unsigned char*)&(m_migration_player[i]));
																socket->SendMessage(result2);
															}
														}
														SendMigration(m_migration_player[i], &name, &license);
													}
												}
												migration_ok=true;
											}
										}
									}
								}
								else
								{
									//   ,      id
									if (!sender)
									{
										wr1->Start();
										unsigned int not_id=0;

										if (game->IsGameCompatibleWithServer(client_version))
										{
											//     
											int k_player=sockets.GetIndexCount();
											if (k_player+1<=max_player)
											{
												//   
												//    
												if (license_type>=0 && license_type<MAX_LICENSE_TYPE)
												{
													NET_STATISTICS* stat=&(statistics[license_type]);

													//    
													NET_LICENSE* lic=new NET_LICENSE;
													lic->Create(receiving_socket, license_type, &license);
													license=lic->GetLicenseKey();

													if (stat->players.Find(license))
													{
														(*wr1)<<not_id;
														CMagicString mes="This player was already authorized";
														(*wr1)<<mes;
													}
													else
													{
														if (stat->bans.Find(license))
														{
															(*wr1)<<not_id;
															CMagicString mes="This player was banned";
															(*wr1)<<mes;
														}
														else
														{
															if (names.Find(name))
															{
																(*wr1)<<not_id;
																CMagicString mes="This player name is already used";
																(*wr1)<<mes;
															}
															else
															{
																bool no_err=true;
																CMagicStringData* er_data=stat->errors.Find(license);
																if (er_data)
																{
																	//        
																	MEM_DATA* md=er_data->GetData();
																	NET_ERROR* er=(NET_ERROR*)(md->data);
																	if (er->count_error==-1)
																	{
																		//   
																		unsigned int tick=platform->GetTick();
																		unsigned int dtime=tick-er->time;

																		unsigned int ban_time;
																		switch (er->ban_stady)
																		{
																		case 1:
																			//   10 
																			ban_time=1000*60*10;
																			break;

																		case 2:
																			//   1 
																			ban_time=1000*60*60;
																			break;

																		default:
																			//   6 
																			ban_time=1000*60*60*6;
																			break;
																		}

																		if (dtime>ban_time)
																		{
																			//  
																			er->ban_stady=0;
																		}
																		else
																		{
																			//  
																			no_err=false;
																			(*wr1)<<not_id;

																			dtime/=(60*1000); //   
																			int minutes=dtime%60;
																			int hours=dtime/60;

																			CMagicString d;
																			CMagicString result;
																			d.IntToString(hours);
																			if (d.GetLength()==1)
																				result+="0";
																			result+=d;
																			result+=':';

																			d.IntToString(minutes);
																			if (d.GetLength()==1)
																				result+="0";
																			result+=d;

																			CMagicString mes="This player has temporary ban:|";
																			mes+=result;
																			(*wr1)<<mes;
																		}
																	}
																}
															
																if (no_err)
																{
																	if (IsLicenseVerificationThread())
																	{
																		//       
																		receiving_socket->name=name;
																		licenses_for_verification->AddLicense(lic);
																		lic=NULL;
																		return true;
																	}
																	else
																	{
																		int license_result=game->VerifyLicense(license_type, license);
																		if (!license_result)
																		{
																			//   
																			(*wr1)<<not_id;
																			CMagicString mes="Access is forbiden";
																			(*wr1)<<mes;
																		}
																		else
																		{
																			if (license_result==1)
																			{
																				//  
																				count_hello++;
																				receiving_socket->name=name;
																				ConnectSocket(receiving_socket);
																				(*wr1)<<receiving_socket->player_id;
																			}
																			else
																			{
																				//   ,        
																				//     
																			}
																		}
																	}
																}
															}
														}
													}

													if (lic)
													{
														delete lic;
														lic=NULL;
													}
												}
											}
											else
											{
												//      
												(*wr1)<<not_id;
												CMagicString mes="Too many players";
												(*wr1)<<mes;
											}
										}
										else
										{
											//      
											(*wr1)<<not_id;
											CMagicString mes="Incompatible version of game";
											(*wr1)<<mes;
										}


										MEM_DATA buf;
										wr1->Finish(buf);
										if (SendHelloReply(receiving_socket, buf, name, license_type, license))
										{
											return true;
										}
										return false;
									}
								}
							}
						}
					}
				}
				break;

			case MESSAGE_TYPE_ENUM_SESSION:
				//       
				if (!IsMigration())
				{
					int k_session_info=sessions.GetIndexCount();
					if (k_session_info)
					{
						//  ,    
						rd1->Start(data);
						DIGITAL_VERSION ver;
						ver.Serialize(*rd1);

						//  :
						//  - MESSAGE_TYPE_ENUM_SESSION_REPLY
						// 1) IP-  (,    )
						// 2)   
						// 3)   
						// 4)     
						//		 NET_SESSION_INFO,      

						wr1->Start();
						
						CMagicString ip;
						if (!sender)
						{
							//    UDP,       
							ip=GetServerUDPSocket()->addr->address;
						}

						int k_session_info=sessions.GetIndexCount();

						(*wr1)<<ip;
						unsigned int pos_count_session_info=wr1->GetPosition();
						(*wr1)<<k_session_info;
						(*wr1)<<k_session_info;
						int count_session_info=0;
						int count_session_info_enabled=0;
						int i=0;
						do 
						{
							//NET_SESSION_INFO* si=m_session_info[i];
							int index=sessions.GetIndex(i);
							NET_SESSION_SERVER* ss=sessions.Get(index);

							if (ss->enabled)
							{
								if (game->IsNetVersionsCompatible(&(ss->ver), &ver))
								{
									ss->NET_SESSION_INFO::Serialize(*wr1);
									count_session_info_enabled++;
								}
								count_session_info++;
							}
							i++;
						}
						while (i<k_session_info);
						unsigned int pos_end=wr1->GetPosition();
						wr1->SetPositionIndirect(pos_count_session_info);
						(*wr1)<<count_session_info;
						(*wr1)<<count_session_info_enabled;
						wr1->SetPositionIndirect(pos_end);

						MEM_DATA buf;
						wr1->Finish(buf);
						NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_ENUM_SESSION_REPLY, buf.length, buf.data);

						if (sender)
						{
							//      
							sockets.Get(sender)->SendMessage(result, NULL);
						}
						else
						{
							//        IP-
							GetServerUDPSocket()->SendMessage(result);
						}
					}
				}
				break;

			case MESSAGE_TYPE_SESSION_INFO:
				{
					//      
					rd1->Start(data);
					NET_SESSION_INFO* si=new NET_SESSION_INFO(this);
					si->Serialize(*rd1);
					//GNetSocket* socket=m_client[sender]->socket;
					GNetSocket* socket=sockets.Get(sender);
					NET_PLAYER* host_player=si->m_player[0];
					if (host_player->type!=PLAYER_MAN)
					{
						if (socket->session_id!=-1)
						{
							NET_SESSION_SERVER* session_server=sessions.Get(socket->session_id);
							if (host_player->type==-1)
							{
								//      -  
								session_server->enabled=false;
							}
							else
							{
								//  ,      
								DeleteSessionInfo(socket->session_id);
								socket->session_id=-1;
							}
						}
					}
					else
					{
						//  ,     
						if (socket->session_id==-1)
						{
							//         
							int index=AddSessionInfo(si, socket);
						}
						else
						{
							NET_SESSION_SERVER* session_server=sessions.Get(socket->session_id);

							si->tick=session_server->tick;
							si->ip=session_server->ip;

							((NET_SESSION_INFO&)*session_server)=*si;
						}

						if (!si->tick)
						{
							//   ,         
							si->tick=platform->GetTick();
							if (!si->tick)
								si->tick=1;
						}
					}

					delete si;
				}
				break;

			case MESSAGE_TYPE_PING:
				//   
				{
					wr1->Start();
					game->GetPingServerInfo(wr1);
					MEM_DATA info;
					wr1->Finish(info);
					if (!info.length)
						info.data=NULL;

					NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_PING_REPLY, info.length, info.data);
					GNetSocket* socket_sender=sockets.Get(sender);
					socket_sender->SendMessage(result);

					//     
					rd1->Start(data);
					int k_receiver;
					(*rd1)>>k_receiver;
					unsigned int vk_client=(unsigned int)sockets.GetIndexCount();
					unsigned int pos=rd1->GetPosition();
					for (int i=0;i<k_receiver;i++)
					{
						unsigned int receiver;
						(*rd1)>>receiver;

						if (receiver<vk_client)
						{
							GNetSocket* socket_receiver=sockets.Get(receiver);
							
							if ((!socket_receiver) || socket_receiver->session_id!=socket_sender->session_id)
							{
								//    -  
								NET_BUFFER_INDEX* result2=PrepareMessage(SERVER_ID, MESSAGE_TYPE_LOST_CONNECTION, 4, (unsigned char*)&receiver);
								socket_sender->SendMessage(result2);
							}
						}
						else
						{
							//  
						}
					}
				}

				break;

			case MESSAGE_TYPE_STOP_SERVER:
				//      
				{
					unsigned int time=platform->GetTick();
					if (stop_time && stop_time<=time)
					{
						unsigned int dtime=time-stop_time;
						if (dtime>=1500) //     
						{
							if (!stop_server)
							{
								stop_server=true;
								StopServer();
							}
						}
					}
					else
					{
						stop_time=time;
						stop_server=false;
					}
				}
				break;

			case MESSAGE_TYPE_CONNECTING:
				//        
				{
					int* idata=(int*)data;
					int k_receiver=*idata;
					if (k_receiver==1)
					{
						unsigned int receiver_id=*(idata+1);
						GNetSocket* host_socket=sockets.Get(receiver_id);
						GNetSocket* player_socket=sockets.Get(sender);
						int session_index=host_socket->session_id;
						//    ,      
						player_socket->session_id=session_index;
						ResendMessage(type, sender, length, data);
						player_socket->session_id=-1;
					}
				}
				break;

			case MESSAGE_TYPE_CONNECT_REPLY:
				//        
				{
					int* idata=(int*)data;
					int k_receiver=*idata;
					if (k_receiver==1)
					{
						unsigned int receiver_id=*(idata+1);
						GNetSocket* player_socket=sockets.Get(receiver_id);
						GNetSocket* host_socket=sockets.Get(sender);
						int session_index=host_socket->session_id;
						NET_SESSION_SERVER* session_server=sessions.Get(session_index);	
						int ok=*(idata+2);
						if (ok==0)
						{
							//   ,       
							session_server->AddPlayer(player_socket);
							ResendMessage(type, sender, length, data);
						}
						else
						{
							//    ,      
							player_socket->session_id=session_index;
							ResendMessage(type, sender, length, data);
							player_socket->session_id=-1;
						}
					}
				}
				break;

			case MESSAGE_TYPE_CHAT_FROM_CLIENT:
				{
					//      
					rd1->Start(data);
					CMagicString message;
					(*rd1)>>message;
					bool privated;
					(*rd1)>>privated;
					bool global;
					(*rd1)>>global;
					int k_name;
					(*rd1)>>k_name;
					int k=1;
					if (k_name<=0)
					{
						//    ,     
						GNetSocket* socket=sockets.Get(sender);
						//if (socket->session_info==-1)
						if (global)
						{
							//    
							chat.AddMessage(socket->name, "", message);
						}
						else
						{
							//       
							CMagicString to_all;
							NET_SESSION_SERVER* session=sessions.Get(socket->session_id);
							if (session->enabled)
							{
								bool privated=false;
								for (int i=0;i<session->k_player_id;i++)
								{
									unsigned int player_id=session->m_player_id[i];
									//if (player_id!=sender)
									{
										GNetSocket* rec_socket=sockets.Get(player_id);

										wr1->Start();
										(*wr1)<<k;
										(*wr1)<<socket->name;
										(*wr1)<<to_all;
										(*wr1)<<privated;
										(*wr1)<<global;
										(*wr1)<<message;
										MEM_DATA buf;
										wr1->Finish(buf);

										NET_BUFFER_INDEX* result=PrepareMessage(sender, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
										rec_socket->SendMessage(result);
									}
								}
							}
						}
					}
					else
					{
						//    ,     
						GNetSocket* socket=sockets.Get(sender);
						CMagicString to;

						if (privated)
						{
							//   
							for (int i=0;i<k_name;i++)
							{
								(*rd1)>>to;

								GNetSocket* rec_socket=ReceiverVerificationByName(sender, to, global);
								if (rec_socket)
								{
									//   
									wr1->Start();
									(*wr1)<<k;
									(*wr1)<<socket->name;
									(*wr1)<<to;
									(*wr1)<<privated;
									(*wr1)<<global;
									(*wr1)<<message;
									MEM_DATA buf;
									wr1->Finish(buf);

									NET_BUFFER_INDEX* result=PrepareMessage(sender, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
									rec_socket->SendMessage(result);

									if (to!=socket->name)
									{
										//     
										result=PrepareMessage(sender, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
										socket->SendMessage(result);
									}
								}
							}
						}
						else
						{
							//   
							//if (socket->session_info==-1)
							if (global)
							{
								//    
								for (int i=0;i<k_name;i++)
								{
									(*rd1)>>to;
									GNetSocket* rec_socket=ReceiverVerificationByName(sender, to, global);
									if (rec_socket)
										chat.AddMessage(socket->name, rec_socket->name, message);
								}
							}
							else
							{
								//       
								NET_SESSION_SERVER* session=sessions.Get(socket->session_id);
								if (session->enabled)
								{
									bool privated=false;
									for (int i=0;i<k_name;i++)
									{
										(*rd1)>>to;
										GNetSocket* rec_socket=ReceiverVerificationByName(sender, to, global);
										if (rec_socket)
										{
											for (int j=0;j<session->k_player_id;j++)
											{
												unsigned int player_id=session->m_player_id[j];
												GNetSocket* receiving_socket=sockets.Get(player_id);

												wr1->Start();
												(*wr1)<<k;
												(*wr1)<<socket->name;
												(*wr1)<<to;
												(*wr1)<<privated;
												(*wr1)<<global;
												(*wr1)<<message;
												MEM_DATA buf;
												wr1->Finish(buf);

												NET_BUFFER_INDEX* result=PrepareMessage(sender, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
												receiving_socket->SendMessage(result);
											}

											if (rec_socket->session_id!=socket->session_id)
											{
												//     ,      
												NET_SESSION_SERVER* session=sessions.Get(rec_socket->session_id);
												for (int j=0;j<session->k_player_id;j++)
												{
													unsigned int player_id=session->m_player_id[j];
													GNetSocket* receiving_socket=sockets.Get(player_id);

													wr1->Start();
													(*wr1)<<k;
													(*wr1)<<socket->name;
													(*wr1)<<to;
													(*wr1)<<privated;
													(*wr1)<<global;
													(*wr1)<<message;
													MEM_DATA buf;
													wr1->Finish(buf);

													NET_BUFFER_INDEX* result=PrepareMessage(sender, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
													receiving_socket->SendMessage(result);
												}
											}
										}
									}
								}
							}
						}
					}
				}
				break;

				/*
			case MESSAGE_TYPE_CHAT_GLOBAL_CLIENT:
				//       
				if (sender)
				{
					rd1->Start(data);
					bool private_for_session;
					(*rd1)>>private_for_session;
					CMagicString message;
					(*rd1)>>message;
					unsigned int k_receiver;
					(*rd1)>>k_receiver;
					if (!k_receiver)
					{
						//   
						chat.AddMessage(sender, 0, message, private_for_session);
					}
					else
					{
						//   
						for (int i=0;i<k_receiver;i++)
						{
							unsigned int receiver;
							(*rd1)>>receiver;
							GNetSocket* socket=sockets.Get(receiver);
							if (socket)
							{
								if (socket->session_info!=-1)
								{
									//     
									NET_SESSION_SERVER* session=sessions.Get(socket->session_info);
									if (session->enabled)
									{
										//  ,      
										NET_BUFFER_INDEX* result=PrepareMessage(sender, MESSAGE_TYPE_CHAT, buf.length, buf.data);
										receiving_socket->SendMessage(result);
										
										NET_BUFFER_INDEX* result=PrepareMessageForPlayers(MESSAGE_TYPE_CHAT, message.GetLength(), (unsigned char *)message.c_str(), session->k_player, session->m_player_id);
										main_socket->SendMessage(result);
									}
									else
									{
										//   ,     ,   
									}
								}
								else
								{
									//     

								}
							}
							else
							{
								//    , ,    ,   

							}
						}
					}
				}
				break;
				*/

			default:
				//       
				ResendMessage(type, sender, length, data);
				break;
		}
	}
	return true;
}



//     
bool GNetServer::SendHelloReply(GNetSocket* socket, MEM_DATA& buf, const char* name, int type_license, const char* license)
{
	bool is_ok=false;
	if (buf.length)
	{
		NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_HELLO_REPLY, buf.length, buf.data);
		socket->SendMessage(result);
		if (socket->player_id)
		{
			//  
			CMagicString lic=license;
			if (statistics[type_license].players.Add(lic))
			{
				//     -
				MEM_DATA data_player_id;
				data_player_id.length=sizeof(socket->player_id);
				data_player_id.data=(unsigned char*)&(socket->player_id);
				CMagicString sname=name;
				names.Add(sname, &data_player_id);

				socket->type_license=type_license;
				socket->license=statistics[type_license].players.Find(lic);
				is_ok=true;
			}
		}
	}

	if (!is_ok)
	{
		//   ,   
		socket->Destroy();
		delete socket;
		socket=NULL;

		if (!internet)
		{
			//   
			if (sockets.GetIndexCount()==2)
			{
				//    
				//   
				StopServer();
			}
		}
		//OnLostConnection(socket);
	}
	return is_ok;
}

//  
void GNetServer::OnLostConnection(GNetSocket* socket)
{
	unsigned int index=socket->GetPlayerID();
	
	if (socket->session_id!=-1)
	{
		//         
		NET_SESSION_SERVER* ss=sessions.Get(socket->session_id);
		assert(ss);

		bool is_host=(ss->m_player_id[0]==index);
		if (is_host)
		{
			//    ,    
			ss->enabled=false;
		}

		for (int i=0;i<ss->k_player_id;i++)
		{
			unsigned int player_id=ss->m_player_id[i];
			if (player_id!=index)
			{
				GNetSocket* socket_receiver=sockets.Get(player_id);
				NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_LOST_CONNECTION, 4, (unsigned char*)&index);
				socket_receiver->SendMessage(result);
			}
		}
	}

	if (index || sockets.Get(0)==socket)
		sockets.DeleteSocket(index);
	else
	{
		//       , ..     Accept(),     
		int index_nohello=sockets_nohello.FindIndex(socket);
		if (index_nohello!=-1)
		{
			sockets_nohello.Delete(index_nohello);
			socket->Destroy();
			delete socket;
		}
	}
}

//     NET_SERVER_INFO
void GNetServer::FillServerInfo(NET_SERVER_INFO& info)
{
	info.sessions=&sessions;
	info.sockets=&sockets;

	info.server_version=game->GetVersion();

	info.current_time=platform->GetTick();
	info.start_time=start_time;

	info.k_accept=count_accept;
	info.k_hello=count_hello;
}

//        
void GNetServer::SaveShutdownType()
{
	if (!shutdown_file.IsEmpty())
	{
		while (true)
		{
			CMagicStream* stream=platform->OpenStream(shutdown_file, MAGIC_STREAM_WRITE);
			if (stream)
			{
				int v=(int)shutdown_type;
				*stream<<v;
				delete stream;
				break;
			}
			else
			{
				platform->Sleep(200);
			}
		}
	}
}

//   
CMagicString mes_error_forbiden="Access is forbiden";
void GNetServer::ServiceLicenses()
{
	if (licenses_result && licenses_result->GetIndexCount())
	{
		NET_LICENSE license;
		do
		{
			int index=licenses_result->GetIndex(0);
			licenses_result->Get(index, &license);
			licenses_result->DeleteLicense(index);

			GNetSocket* socket=license.socket;
			//     
			bool is=true;
			int k_deleted=deleted_sockets_during_verification.GetIndexCount();
			for (int i=0;i<k_deleted;i++)
			{
				int dindex=deleted_sockets_during_verification.GetIndex(i);
				GNetSocket* deleted_socket=(GNetSocket*)deleted_sockets_during_verification.Get(dindex);
				if (socket==deleted_socket)
				{
					//    ,   
					is=false;
					deleted_sockets_during_verification.Delete(dindex);
					break;
				}
			}

			if (is)
			{
				//   
				CMagicString license_key;
				wr1->Start();
				if (license.type==-1)
				{
					//   
					unsigned int not_id=0;
					(*wr1)<<not_id;
					(*wr1)<<mes_error_forbiden;
				}
				else
				{
					//  
					count_hello++;
					ConnectSocket(socket);
					(*wr1)<<socket->player_id;
					license_key=license.GetLicenseKey();
				}


				MEM_DATA buf;
				wr1->Finish(buf);

				if (SendHelloReply(socket, buf, socket->name, license.type, license_key))
				{
				}
			}
		}
		while (licenses_result->GetIndexCount());
	}
}

//    
bool GNetServer::StartLicenseVerificationThread()
{
	if (!IsLicenseVerificationThread())
	{
		//    
		licenses_for_verification=new GLicenseList(platform);
		licenses_result=new GLicenseList(platform);
		platform->ExecuteThread(LicenseVerificationThread, this);
		while (true)
		{
			platform->Sleep(200);
			if (IsLicenseVerificationThread())
				return true;
		}
	}
	return false;
}

//    
void GNetServer::StopLicenseVerificationThread()
{
	if (IsLicenseVerificationThread())
	{
		state_license_verification=2;
		while (IsLicenseVerificationThread())
		{
			platform->Sleep(300);
		}

		if (licenses_for_verification)
		{
			delete licenses_for_verification;
			licenses_for_verification=NULL;
		}

		if (licenses_result)
		{
			delete licenses_result;
			licenses_result=NULL;
		}
	}
}

//  ,    
void GNetServer::ServiceInactiveSockets()
{
	int i;
	unsigned int max_delay=1000*60;
	unsigned int tick=platform->GetTick();
	int k_socket=sockets.GetIndexCount();
	for (i=0;i<k_socket;i++)
	{
		int index=sockets.GetIndex(i);
		GNetSocket* socket=sockets.Get(index);
		unsigned int rtick=socket->GetRecevingTick();
		unsigned int dtick=tick-rtick;
		if (dtick>max_delay && (!socket->IsAdmin()) && socket->player_id>1)
		{
			//    ,       
			OnLostConnection(socket);
			k_socket=sockets.GetIndexCount();
			i--;
		}
	}

	k_socket=sockets_nohello.GetIndexCount();
	for (i=0;i<k_socket;i++)
	{
		int index=sockets_nohello.GetIndex(i);
		GNetSocket* socket=(GNetSocket*)sockets_nohello.Get(index);

		unsigned int rtick=socket->GetRecevingTick();
		unsigned int dtick=tick-rtick;
		if (dtick>max_delay)
		{
			//    ,       
			OnLostConnection(socket);
			k_socket=sockets_nohello.GetIndexCount();
			i--;
		}
	}
}

//       ,    
void GNetServer::SoftlyResetMode()
{
	StopLicenseVerificationThread();

	//   ,     
	int count=0;

	int k_client=sockets.GetIndexCount();
	for (int i=0;i<k_client;i++)
	{
		int index=sockets.GetIndex(i);
		GNetSocket* socket=sockets.Get(index);
		if (socket->IsAdmin() || index==0)
		{
			socket->Destroy();
			sockets.DeleteSocket(index);

			sockets.DeleteSocket(index);
			i--;
			k_client--;
		}
		else
			count++;
	}

	if (!count)
		StopServer();

	softly_reset=true;
}

//     
void GNetServer::ServiceSoftlyResetMode()
{
	if (softly_reset)
	{
		int k_client=sockets.GetIndexCount();
		if (!k_client)
			StopServer();
	}
}

//     
void GNetServer::SendGlobalChat()
{
	int k=chat.GetCount();
	if (k)
	{
		wr1->Start();
		(*wr1)<<k;
		bool privated=false;
		bool global=true;
		int i=0;
		do 
		{
			CHAT_GLOBAL* c=chat.Get(i);
			(*wr1)<<c->sender;
			(*wr1)<<c->receiver;
			(*wr1)<<privated;
			(*wr1)<<global;
			(*wr1)<<c->message;
			i++;
		}
		while (i<k);

		MEM_DATA buf;
		wr1->Finish(buf);

		NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
		NET_BUFFER_LIST* list=result->owner;
		MEM_DATA buffer;

		int len=result->GetLength();
		unsigned char* data=result->GetData();
		buffer.data=NULL;
		int len16=len;
		buffer.length=len16;

		int count=sockets.GetIndexCount();
		for (int i=0;i<count;i++)
		{
			int index=sockets.GetIndex(i);
			GNetSocket* socket=sockets.Get(index);
			if (socket->session_id==-1)
			{
				if (socket->player_id>1 && (!socket->IsAdmin()))
				{
					int index=list->AddBuffer(buffer);
					NET_BUFFER_INDEX* result_copy=list->Get(index);
					memcpy(result_copy->GetData(), data, len);
					result_copy->SetLength(len);
					socket->SendMessage(result_copy);
				}
			}
			else
			{
				NET_SESSION_SERVER* session=sessions.Get(socket->session_id);
				if (session->enabled)
				{
					int index=list->AddBuffer(buffer);
					NET_BUFFER_INDEX* result_copy=list->Get(index);
					memcpy(result_copy->GetData(), data, len);
					result_copy->SetLength(len);
					socket->SendMessage(result_copy);
				}
			}
		}

		int index=result->GetIndex();
		list->DeleteBuffer(index);

		chat.Reset();
	}
}

//       
void GNetServer::ResendMessage(int type, unsigned int sender, int length, unsigned char* data)
{
	rd1->Start(data);
	int k_receiver;
	(*rd1)>>k_receiver;
	int receiver_len=k_receiver*4+4;
	unsigned char* start_message=data+receiver_len;
	GNetSocket* socket_sender=sockets.Get(sender);
	unsigned int vk_client=(unsigned int)sockets.GetMaxCount();
	int k_lost=0;
	unsigned int pos=rd1->GetPosition();
	for (int i=0;i<k_receiver;i++)
	{
		unsigned int receiver;
		(*rd1)>>receiver;

		if (receiver<vk_client)
		{
			GNetSocket* socket_receiver=sockets.Get(receiver);
			if (socket_receiver && socket_receiver->session_id==socket_sender->session_id && socket_sender->session_id!=-1)
			{
				NET_BUFFER_INDEX* result=PrepareMessage(sender, type, length-receiver_len, start_message);
				socket_receiver->SendMessage(result);
			}
			else
			{
				//    -  
				k_lost++;
			}
		}
		else
		{
			//  
		}
	}

	if (k_lost)
	{
		rd1->SetPosition(pos);
		//  ,     
		for (int i=0;i<k_receiver;i++)
		{
			unsigned int receiver;
			(*rd1)>>receiver;

			if (receiver<vk_client)
			{
				GNetSocket* socket_receiver=sockets.Get(receiver);
				if (!(socket_receiver && socket_receiver->session_id==socket_sender->session_id && socket_sender->session_id!=-1))
				{
					//    -  
					NET_BUFFER_INDEX* result2=PrepareMessage(SERVER_ID, MESSAGE_TYPE_LOST_CONNECTION, 4, (unsigned char*)&receiver);
					socket_sender->SendMessage(result2);
				}
			}
			else
			{
				//  
			}
		}
	}
}

int GNetServer::AddSessionInfo(NET_SESSION_INFO* session_info, GNetSocket* socket)
{
	NET_SESSION_SERVER* session_server=new NET_SESSION_SERVER(this);
	((NET_SESSION_INFO&)*session_server)=*session_info;
	//session_server->time=platform->GetTick();
	session_server->tick=platform->GetTick();
	int index=sessions.AddSession(session_server);
	socket->session_id=index;
	session_server->enabled=true;
	session_server->k_player_id=1;
	session_server->m_player_id=new unsigned int[1];
	session_server->m_player_id[0]=socket->player_id;
	return index;
}

//   
bool GNetServer::Unban(int license_type, const char* license)
{
	bool is=false;
	CMagicString lic=license;

	int i=license_type;
	//for (int i=0;i<MAX_LICENSE_TYPE;i++)
	{
		CMagicStringTable* bans=&(statistics[i].bans);
		if (bans->Find(lic))
		{
			if (bans->Delete(lic))
			{
				//      
				//      
				CMagicString path=platform->GetPathToConfig();
				path+="bans";
				CMagicString d;
				d.IntToString(i);
				if (d.GetLength()==1)
					path+='0';
				path+=d;
				path+=".ctl";
				FILE_DATA fd;
				if (platform->LoadFile(path, &fd))
				{
					rd1->Start(fd.data);
					unsigned int pos=0;
					CMagicString read_license;
					unsigned int length=(unsigned int)fd.length;
					while (pos<length)
					{
						(*rd1)>>read_license;
						if (read_license==lic)
						{
							//    
							//      pos  pos2
							unsigned int pos2=rd1->GetPosition();
							int len=fd.length-pos2;
							memmove(&(fd.data[pos]), &(fd.data[pos2]), len);
							fd.length-=(pos2-pos);
							is=platform->SaveFile(path, &fd);
							break;
						}
						pos=rd1->GetPosition();
					}
				}
			}
		}
	}
	return is;
}

//        
//    ,      MESSAGE_TYPE_MIGRATION_OK
bool GNetServer::IsAllPlayersMigrated(CMagicString* name, CMagicString* license)
{
	bool is=true;
	for (int i=0;i<k_migration_player;i++)
	{
		if (!sockets.Get(m_migration_player[i]))
		{
			is=false;
			break;
		}
	}

	if (is)
	{
		//   ,    
		for (int i=0;i<k_migration_player;i++)
		{
			//NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_MIGRATION_OK, 0, NULL);
			//sockets.Get(m_migration_player[i])->SendMessage(result, NULL);
			SendMigration(m_migration_player[i], name, license);
		}
		migration_ok=true;
	}
	return is;
}

//      
void GNetServer::SendMigration(unsigned int receiver, CMagicString* name, CMagicString* license)
{
	wr1->Start();

	MEM_DATA buf;
	wr1->Finish(buf);
	NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_MIGRATION_OK, buf.length, buf.data);
	GNetSocket* socket=sockets.Get(receiver);
	socket->SendMessage(result);

}

//          -
//         ,     
GNetSocket* GNetServer::ReceiverVerificationByName(unsigned int sender, CMagicString& name, bool global)
{
	CMagicStringData* data=names.Find(name);
	if (data)
	{
		//  
		MEM_DATA* data_player_id=data->GetData();
		unsigned int* receiver_ptr=(unsigned int*)(data_player_id->data);
		unsigned int receiver=*receiver_ptr;

		GNetSocket* rec_socket=sockets.Get(receiver);
		if (rec_socket->session_id!=-1)
		{
			//    
			NET_SESSION_SERVER* session=sessions.Get(rec_socket->session_id);
			if (!session->enabled)
			{
				//        
				CMagicString mes=name;
				mes+=" can not receive your message (the game is already started)";

				GNetSocket* socket=sockets.Get(sender);
				CMagicString from="";

				wr1->Start();
				int k=1;
				(*wr1)<<k;
				(*wr1)<<from;
				(*wr1)<<socket->name;
				bool privated=true;
				(*wr1)<<privated;
				(*wr1)<<global;
				(*wr1)<<mes;
				MEM_DATA buf;
				wr1->Finish(buf);

				NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
				socket->SendMessage(result);
				return NULL;
			}
		}
		else
		{
			//     
			if (!global)
			{
				CMagicString mes=name;
				mes+=" does not participate in the session. Use global chat to send message to this user";

				GNetSocket* socket=sockets.Get(sender);
				CMagicString from="";

				wr1->Start();
				int k=1;
				(*wr1)<<k;
				(*wr1)<<from;
				(*wr1)<<socket->name;
				bool privated=true;
				(*wr1)<<privated;
				(*wr1)<<global;
				(*wr1)<<mes;
				MEM_DATA buf;
				wr1->Finish(buf);

				NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
				socket->SendMessage(result);

				return NULL;
			}
		}


		return rec_socket;
	}

	CMagicString mes=name;
	mes+=" is absent on the server";

	GNetSocket* socket=sockets.Get(sender);
	CMagicString from="";

	wr1->Start();
	int k=1;
	(*wr1)<<k;
	(*wr1)<<from;
	(*wr1)<<socket->name;
	bool privated=true;
	(*wr1)<<privated;
	(*wr1)<<global;
	(*wr1)<<mes;
	MEM_DATA buf;
	wr1->Finish(buf);

	NET_BUFFER_INDEX* result=PrepareMessage(SERVER_ID, MESSAGE_TYPE_CHAT_FROM_SERVER, buf.length, buf.data);
	socket->SendMessage(result);
	return NULL;
}

