//-----------------------------------------------------------------------------
// File: Net.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once

#include "utils.h"


class GGame;
class GPlatform;
class GCriticalSection;
class GNet;
class GNetSession;
class GNetClient;
class GNetSocket;
class GMemWriter;
class GMemReader;
class GSessionList;
class GSocketList;

struct NET_BUFFER_LIST;
struct NET_SESSION_INFO;
struct NET_SOCKET_INFO;

extern int PORT_SERVER_TCP_ADMIN;
extern int PORT_SERVER_TCP_GAME;
#define PORT_SERVER_UDP	42598

extern int PORT_CLIENT_TCP;
#define PORT_CLIENT_UDP	42596

#define SERVER_ID		0

// ,   
#define PLAYER_COMPUTER	0	// 
#define PLAYER_MAN		1	// 
#define PLAYER_OPENED	2	// 
#define PLAYER_CLOSED	3	// 
#define PLAYER_DISABLED	4	// 


#define MESSAGE_TYPE_HELLO				0	//    ,     id
#define MESSAGE_TYPE_HELLO_REPLY		1	//    ,    id
#define MESSAGE_TYPE_ENUM_SESSION		2	//    (   UDP    TCP   )
#define MESSAGE_TYPE_ENUM_SESSION_REPLY	3	//       (     )
#define MESSAGE_TYPE_SESSION_INFO		4	//       
#define MESSAGE_TYPE_CONNECTING			5	//      
#define MESSAGE_TYPE_CONNECT_REPLY		6	//          
#define MESSAGE_TYPE_PLAYER_INFO		7	//   ,   
#define MESSAGE_TYPE_START_GAME			8	//  
#define MESSAGE_TYPE_PLAYER_STARTED		9	//   ,            
#define MESSAGE_TYPE_LOST_CONNECTION	10	//           
#define MESSAGE_TYPE_GIVE_MAP			11	//        
#define MESSAGE_TYPE_DOWNLOAD_MAP		12	//      
//#define MESSAGE_TYPE_CHAT				13	// -
#define MESSAGE_TYPE_PING				14	// ping    
#define MESSAGE_TYPE_PING_REPLY			15	// ping    
#define MESSAGE_TYPE_STOP_SERVER		16	//       (   ,    )
#define MESSAGE_TYPE_PLAYER_GAME		17	//   ,     
#define MESSAGE_TYPE_SYNC_GAME			18	//  save-    
#define MESSAGE_TYPE_MIGRATION_OK		19	//    ,    



#define MESSAGE_TYPE_CHAT_FROM_CLIENT		35	//      
#define MESSAGE_TYPE_CHAT_FROM_SERVER		36	//       


struct GUID_GAME
{
	unsigned int guid1, guid2, guid3, guid4;

	GUID_GAME()
	{
		Clear();
	}

	void Clear()
	{
		guid1=guid2=guid3=guid4=0;
	}

	bool IsEmpty()
	{
		return ((guid1 | guid2 | guid3 | guid4)==0);
	}

	bool operator==(const GUID_GAME& guid)
	{
		return (guid1==guid.guid1 && guid2==guid.guid2 && guid3==guid.guid3 && guid4==guid.guid4);
	}

	bool operator!=(const GUID_GAME& guid)
	{
		return (guid1!=guid.guid1 || guid2!=guid.guid2 || guid3!=guid.guid3 || guid4!=guid.guid4);
	}
};

struct NET_BUFFER
{
	NET_BUFFER_LIST* owner;

	int max_length;
	int length;
	int position;
	unsigned char* data;

	NET_BUFFER();
	virtual ~NET_BUFFER();

	void Clear();

	void SetMaxSize(int size);

	//    
	void Add(int length, void* data);

	//    
	void Delete(int length);

	//      
	void IncreaseSize();

	int GetLength(){return length;}
	void SetLength(int len){length=len;}

	int GetMaxLength(){return max_length;}

	unsigned char* GetData(){return data;}

	int GetPosition(){return position;}
	void SetPosition(int pos){position=pos;}

	void Reset(){position=0; length=0;}

	//       ,    
	//      -1,    
	int HasMessage(GNetSocket* socket);
};

class NET_BUFFER_INDEX : public NET_BUFFER
{
protected:
	int index;	//    

//public:
	//int count;	//  ,      

public:
	NET_BUFFER_INDEX(int index) : NET_BUFFER()
	{
		this->index=index;
		//count=0;
	}

	int GetIndex(){return index;};
};

struct NET_ADDRESS
{
	CMagicString address;
	int port;

	void FromStringIP(const char* ip);
	CMagicString ToStringIP();

	void Serialize(CMagicStream& stream);
};

struct NET_SERVER_INFO
{
	unsigned int version;

	CMagicString server_version;

	unsigned int start_time;
	unsigned int current_time;

	int k_accept;
	int k_hello;

	GSessionList* sessions;
	GSocketList* sockets;

	NET_SERVER_INFO();

	//  (    )
	void Clear();

	// 
	void Serialize(CMagicStream& stream, GNet* net);
};



enum SHUTDOWN_TYPE {SHUTDOWN_TYPE_NO, SHUTDOWN_TYPE_RESTART, SHUTDOWN_TYPE_TURNOFF};

struct SEND_MESSAGE
{
	GUID_GAME guid;
	unsigned int sender;
	int type;
	int length;
};

struct NET_SOCKET_INFO
{
	unsigned int version;

	unsigned int player_id;

	bool udp_tcp;
	int port;
	bool is_admin;

	unsigned int time;

	int session_id;	//      ( )

	CMagicString name;
	CMagicString ip;

	NET_SOCKET_INFO();
	virtual ~NET_SOCKET_INFO();

	void Serialize(CMagicStream& stream);

	unsigned int GetPlayerID(){return player_id;}
	void SetPlayerID(unsigned int player_id){this->player_id=player_id;}

	void SetTime(unsigned int time){this->time=time;}
	unsigned int GetTime(){return time;}
};

class GNetSocket : public NET_SOCKET_INFO
{
public:
	GNet* net;

protected:
	NET_BUFFER rbuffer;
	unsigned int rtick;

public:
	int type_license;
	CMagicString* license;


	NET_ADDRESS* addr;

public:
	GNetSocket(GNet* net);
	virtual ~GNetSocket();

	//  UDP  TCP- ,   listaen=true,    
	virtual bool Create(bool udp_tcp, int port, bool listen);

	//     TCP-   
	virtual bool SetConnectedSocketToReadMode()=0;

	NET_BUFFER* GetReceivingBuffer(){return &rbuffer;}

	//    
	unsigned int GetRecevingTick(){return rtick;}
	void SetReceivingTick(unsigned int tick){rtick=tick;}

	//  
	virtual void Destroy();

	bool IsTCP(){return udp_tcp;}
	//int GetPort(){return port;}
	void SetAdmin(){is_admin=true;}
	bool IsAdmin(){return is_admin;}

	//  ,    
	bool IsWaitedAutorization();

	//  IP-   TCP-
	// own_or_peer          
	virtual bool GetIP(CMagicString& addr, bool own_or_peer);

	//    (  TCP-)
	virtual bool Accept()=0;

	//    (  TCP-)
	virtual bool Connect(NET_ADDRESS* addr)=0;

	//  
	void SendMessage(NET_BUFFER_INDEX* buf, NET_ADDRESS* addr=NULL);

	virtual void SendTCP(NET_BUFFER_INDEX* buf)=0;
	virtual void SendUDP(NET_BUFFER_INDEX* buf)=0;

	virtual void ReceiveTCP()=0;
	virtual void ReceiveUPD()=0;

	bool ReceiveMessages();

	// 
	bool Ban(bool autoban=true);
	//       
	bool Error();
};

/*
struct NET_CLIENT
{
	GNetSocket* socket;			// 
	unsigned int player_id;		// id 
	unsigned int deleting_time;	//       

	bool IsExisting(){return (deleting_time==0);}
};
*/

// ,         
#define NET_PLAYER_VERSION	0
struct NET_PLAYER
{
	int version;
	int type;			//   (, , open, close, disabled)
	CMagicString name;
	CMagicString ip;	// ip- (   ,      )
	unsigned int player_id;
	unsigned int ping;	// ping  
	unsigned int time;	//      (      )
	int length_info;
	char* info;

	GNet* net;

	NET_PLAYER(GNet* net);
	virtual ~NET_PLAYER();

	void Clear();

	void Serialize(CMagicStream& stream);

	NET_PLAYER& operator=(const NET_PLAYER& np);
	bool operator==(const NET_PLAYER& np);
	bool operator!=(const NET_PLAYER& np)
	{
		return !(*this==np);
	}
};

// ,      
struct DIGITAL_VERSION
{
	int ver1, ver2, ver3, ver4;

	//     
	void FromString(const char* ver);
	//    
	CMagicString ToString();

	void Serialize(CMagicStream& stream);

	void Load(FILE* file);
	void Save(FILE* file);
};

// ,      
#define NET_SESSION_VERSION	0
struct NET_SESSION_INFO
{
	int version;
	CMagicString ip;
	unsigned int tick;
	CMagicString name;
	unsigned int id_map;
	bool is_password;
	int k_player;
	NET_PLAYER** m_player;

	int length_info;
	char* info;

	DIGITAL_VERSION ver;

	GNet* net;

	NET_SESSION_INFO(GNet* net);
	virtual ~NET_SESSION_INFO();

	void Clear();

	virtual void Serialize(CMagicStream& stream);

	NET_SESSION_INFO& operator=(const NET_SESSION_INFO& si);
	bool operator==(const NET_SESSION_INFO& si);
	bool operator!=(const NET_SESSION_INFO& si)
	{
		return !(*this==si);
	}
};

//     
struct NET_JOIN_DATA
{
	unsigned int host_id;
	CMagicString ip;
	CMagicString password;
};

struct NET_BUFFER_LIST : public CArrayBase
{
	GNet* net;

	int k_buffer;
	NET_BUFFER_INDEX** m_buffer;

	NET_BUFFER_LIST();
	virtual ~NET_BUFFER_LIST();
	void Clear();
	void SetOwner(GNet* owner){net=owner;}

	int AddBuffer(const MEM_DATA& buffer);
	void DeleteBuffer(int index);

	NET_BUFFER_INDEX* Get(int index){return m_buffer[index];}
};

struct NET_SESSION_SERVER : public NET_SESSION_INFO
{
	bool enabled;
	//unsigned int time;
	int k_player_id;
	unsigned int* m_player_id;
	int session_index;

	NET_SESSION_SERVER(GNet* net);
	virtual ~NET_SESSION_SERVER();

	void AddPlayer(GNetSocket* socket);
	bool DeletePlayer(GNetSocket* socket);

	virtual void Serialize(CMagicStream& stream);
};

// ,     
class GSessionList : public CArrayBase
{
protected:
	int k_session;					//  
	NET_SESSION_SERVER** m_session;	//  

public:
	GSessionList();
	virtual ~GSessionList();

	void ReInit();

	void Clear();

	//     
	NET_SESSION_SERVER* Get(int index);

	//  
	int AddSession(NET_SESSION_SERVER* session);

	//    
	bool DeleteSession(int index);

	void Serialize(CMagicStream& stream, GNet* net);
};

// ,   
class GSocketList : public CArrayBase
{
protected:
	int k_socket;			//  
	GNetSocket** m_socket;	//  

public:
	GSocketList();
	virtual ~GSocketList();

	void Clear();

	//     
	GNetSocket* Get(int index);

	//      
	int GetMaxCount(){return k_socket;}

	//  
	int AddSocket(GNetSocket* client, unsigned int player_id=0xffffffff);

	//    
	bool DeleteSocket(int index);

	void Serialize(CMagicStream& stream, GNet* net);

	//  
	void Expand(int max_size);
};

// ,   
class GNet
{
public:
	GUID_GAME guid;
	bool internet;

	GSessionList sessions;

protected:
	NET_BUFFER_LIST send_list;

	NET_BUFFER recv_buffer;	// ,       

public:
	GNetSocket* receiving_socket; // ,   

	GMemWriter* wr1;
	GMemWriter* wr2;
	GMemReader* rd1;
	GMemReader* rd2;


	static GGame* game;
	static GPlatform* platform;

public:
	GNet();
	virtual ~GNet();

	virtual bool Create(GUID_GAME guid, bool internet);
	virtual void Destroy();

	//      
	NET_BUFFER_LIST* GetSendList(){return &send_list;}

	virtual NET_BUFFER_INDEX* NewBuffer(int index)=0;

	virtual GNetSocket* NewSocket(GNet* net)=0;

	//  
	virtual bool IsServer()=0;
	//   
	virtual bool IsDownloader(){return false;};

	//  ,   
	bool IsInternet(){return internet;}

	//    
	NET_BUFFER_INDEX* PrepareMessage(unsigned int sender_id, int type, int length, const unsigned char* data);

	virtual bool ReceiveMessage(int type, unsigned int sender, int length, unsigned char* data)=0;

	//   
	virtual bool UpdateNet()=0;

	//    (    CNetClient)
	//int AddSessionInfo(NET_SESSION_INFO* session_info);

	void DeleteSessionInfo(int index);
	//    id 
	int FindSessionInfo(unsigned int host_id);
	int GetSessionInfoCount(){return sessions.GetIndexCount();}
	//NET_SESSION_INFO* GetSessionInfo(int index){return m_session_info[index];}
	NET_SESSION_SERVER* GetSessionInfo(int index){return sessions.Get(index);}

	void ClearEnumSessionInfo();

	//  ,    
	//bool IsDestroyed(){return is_destroyed;}

	//   
	void virtual OnUpdate()=0;

	//  
	virtual void OnLostConnection(GNetSocket* socket)=0;

	NET_BUFFER* GetRecvBuffer(){return &recv_buffer;}
};

class GNetSession
{
protected:
	GNetClient* owner;
	CMagicString name;			//  
	unsigned int id_host;	//   
	int index;				//     
	int k_player;			//  ,   
	unsigned int* m_player;	//  
};

enum NET_STADY {NET_STADY_ENUM_SESSION, NET_STADY_JOIN_SESSION, NET_STADY_CREATE_SESSION, NET_STADY_START_GAME, NET_STADY_GAME, NET_STADY_MIGRATION_HOST, NET_STADY_NO=-1};

// ,     
class GNetStady
{
protected:
	GNetClient* owner;
	
	unsigned int stady_period;
	unsigned int stady_tick;

public:
	GNetStady(GNetClient* owner);
	virtual ~GNetStady(){}

	virtual bool OnStart(NET_STADY previous, void* init);
	virtual void OnFinish(NET_STADY next){}

	virtual void OnUpdate(); 

	//     ,    stady_period (    0)
	virtual void OnPeriod(){}

	//     
	virtual bool IsMessageCorrected(int message_type){return false;}
};

//   
class GNetStadyEnumSession : public GNetStady
{
public:
	GNetStadyEnumSession(GNetClient* owner);

	//   ,    
	virtual bool OnStart(NET_STADY previous, void* init);

	virtual void OnFinish(NET_STADY next);

	//  
	virtual void OnPeriod();

	//     
	bool IsMessageCorrected(int message_type);
};

//       
struct NET_CHAT_MESSAGE
{
	CMagicString sender;
	int k_receiver;
	CMagicString* m_receiver;
	bool privated;
	CMagicString message;

	NET_CHAT_MESSAGE();
	virtual ~NET_CHAT_MESSAGE();

	//     
	void Split(const char* mes, bool is_sender);
	//      
	void Assemble(CMagicString& mes, bool is_sender);

	//  
	void AddReceiver(CMagicString& receiver);
	//  
	void DeleteReceiver(CMagicString& receiver);
	// ,      
	int FindReceiver(CMagicString& receiver);
	//  
	void ClearReceivers();

	void Serialize(CMagicStream& stream);
};

struct NET_CHAT_MESSAGE_LIST
{
	int k_chat;
	NET_CHAT_MESSAGE* m_chat;

	NET_CHAT_MESSAGE_LIST()
	{
		k_chat=0;
		m_chat=NULL;
	}

	virtual ~NET_CHAT_MESSAGE_LIST()
	{
		Clear();
	}

	void Clear()
	{
		if (m_chat)
		{
			delete []m_chat;
			m_chat=NULL;
		}
		k_chat=0;
	}

	void Create(int length)
	{
		Clear();
		if (length>0)
		{
			k_chat=length;
			m_chat=new NET_CHAT_MESSAGE[k_chat];
		}
	}

	int GetCount(){return k_chat;}
	NET_CHAT_MESSAGE* Get(int index){return &(m_chat[index]);}
};

//    
class GNetStadyJoinSession : public GNetStady
{
protected:
	int index_player;	//    
	NET_SESSION_INFO* copy_session;
	NET_SESSION_INFO* current_session;

	GMemWriter* transfer_map;

	unsigned int ping_takt; //   ,        ping

	int connection_result;	//     
							// 0 -  
							// 1 -  
							// 2 -   
							// 3 -     

public:
	NET_CHAT_MESSAGE_LIST chat;

public:
	GNetStadyJoinSession(GNetClient* owner);
	virtual ~GNetStadyJoinSession();

	//   ,   ,   
	virtual bool OnStart(NET_STADY previous, void* init);

	virtual void OnFinish(NET_STADY next);

	virtual void OnUpdate();

	// Ping
	virtual void OnPeriod();

	//     
	virtual bool IsMessageCorrected(int message_type);

	//    
	NET_SESSION_INFO* GetCopySession(){return copy_session;}
	NET_SESSION_INFO* GetCurrentSession(){return current_session;}

	//   
	int GetIndexPlayer(){return index_player;}

	//     
	GMemWriter* GetWritterTransferMap(){return transfer_map;}

	//  ,       ping
	unsigned int GetPingTakt(){return ping_takt;}

	//     
	void SetConnectionResult(int result){connection_result=result;}
	int GetConnectionResult(){return connection_result;}
};

struct RECEIVER_MAP
{
	int position;				//  
	unsigned int player_id;		// 
};

//    
class GNetStadyCreateSession : public GNetStadyJoinSession
{
protected:
	CMagicString password;				//     

	int k_transfer;
	RECEIVER_MAP* m_transfer;

	NET_SESSION_INFO* server_session;	//    

public:
	GNetStadyCreateSession(GNetClient* owner);
	virtual ~GNetStadyCreateSession();

	//   ,   ,   
	virtual bool OnStart(NET_STADY previous, void* init);

	//      
	virtual void OnFinish(NET_STADY next);

	virtual void OnUpdate(); 

	//      
	virtual void OnPeriod();

	//     
	virtual bool IsMessageCorrected(int message_type);

	//    
	void AddTransferMap(unsigned int player_id);

	//    
	void ClearTransfers();

	//  
	CMagicString GetPassword(){return password;}
};

//   
class GNetStadyStartGame : public GNetStady
{
protected:
	int start_counter;	//   5,4,3,2,1...

public:
	int k_waited_player;
	unsigned int* m_waited_player;

public:
	GNetStadyStartGame(GNetClient* owner);
	virtual ~GNetStadyStartGame();

	//     
	virtual bool OnStart(NET_STADY previous, void* init);

	//      
	virtual void OnPeriod();

	//     
	virtual bool IsMessageCorrected(int message_type);
};

struct PLAYER_MESSAGE
{
	unsigned int player_id;
	unsigned int receive_takt;
	NET_BUFFER next_message;
	
	PLAYER_MESSAGE()
	{
		player_id=0;
		receive_takt=0;
		next_message.length=0;
		next_message.data=NULL;
	}

	virtual ~PLAYER_MESSAGE()
	{
		if (next_message.data)
		{
			delete []next_message.data;
			next_message.data=NULL;
		}
		next_message.length=0;
	}
};

//  
class GNetStadyGame : public GNetStady
{
protected:
	int takt;	//    
	unsigned int takt_time;	//    
	int ping_index;	//  ,          

	bool on_next_net_takt;	//  ,     OnNextNetTakt()   
public:
	bool sync_mode;	//    

protected:
	int k_player;
	PLAYER_MESSAGE* m_player;

	unsigned int* m_waited_player;	//   

	GMemWriter* sync_data;	//     .  save,       
	int attempt_sync;		//     

public:
	GNetStadyGame(GNetClient* owner);
	virtual ~GNetStadyGame();

	//   
	void ClearPlayerData();

	//        
	void ReceivePlayerCommands(unsigned int sender, int length, unsigned char* data, unsigned int net_takt);

	//    
	int GetWaitedPlayers();

	GMemWriter* GetWritterSyncData(){return sync_data;}

	//   
	void Initialization();

	//   
	virtual bool OnStart(NET_STADY previous, void* init);

	//        
	void OnUpdate();

	//     
	bool IsMessageCorrected(int message_type);

	//      
	bool IsNextNetTakt(){return on_next_net_takt;}
	void ResetNextNetTakt(){on_next_net_takt=false;}

	//    
	bool IsSyncMode(){return sync_mode;}

	//    
	int GetSyncAttempt(){return attempt_sync;}
};

//   
class GNetStadyMigrationHost : public GNetStady
{
protected:
	bool is_connected;
	int attempts;
	unsigned int migration_player_id;

	int k_migration_id;
	unsigned int m_migration_id[20];

public:
	GNetStadyMigrationHost(GNetClient* owner);

	//   ,      
	virtual bool OnStart(NET_STADY previous, void* init);

	//     
	virtual void OnPeriod();

	//     
	bool IsMessageCorrected(int message_type);
};

enum NET_DISCONNECT {NET_DISCONNECT_OK, NET_DISCONNECT_LOST, NET_DISCONNECT_SYNCHRONIZATION};

enum CONNECT_STATE {CONNECT_STATE_WAITING, CONNECT_STATE_OK, CONNECT_STATE_ERROR};

// ,    
class GNetClient : public GNet
{
protected:
	NET_STADY net_stady;
	int k_net_stady;
	GNetStady** m_net_stady;

	int k_session;
	GNetSession** m_session;

	//NET_CLIENT client;
	GNetSocket* main_socket;

	int waited_message_type;	//  

	CONNECT_STATE connect_state;	//    

	int count_general_session; //   ,   

	CMagicString server_ip;
	CMagicString client_ip;

	bool is_download_file;		//  ,     
	int header_length;			//    ,     
	int downloaded_file_length;	//   ,   
	CMagicString downloaded_file;	//     

	//    
	int k_download_list;
	CMagicString m_download_list[25];

	NET_SESSION_INFO* session_refresh_receiver;

public:
	unsigned int takt;
	//bool lost_connection;		//  ,    
	NET_DISCONNECT disconnect_state;	//  ,    

public:
	unsigned int m_receiver[100]; //       

public:
	GNetClient();
	virtual ~GNetClient();

	virtual bool Create(GUID_GAME guid, const char* server_ip);
	virtual void Destroy();

	void IncreaseTakt(){takt++;}

	virtual bool IsServer(){return false;}

	//     
	bool CreateSocket(bool udp_tcp);

	//      
	void DestroySocket();

	//  
	bool SetStady(NET_STADY stady, void* init);
	//  
	NET_STADY GetStady(){return net_stady;}
	GNetStady* GetStadyPtr(NET_STADY stady){return m_net_stady[stady];}

	//     
	NET_BUFFER_INDEX* PrepareMessageForPlayers(int type, int length, const unsigned char* data, int count_receiver, unsigned int* receiver_id, int receiver_count_minus=0);

	//   
	bool CreateSession();

	//   
	void StartEnumSessionInfo();

	//    
	//        ,    
	void StopEnumSessionInfo();

	//    
	bool ConnectSession(NET_JOIN_DATA* join);

	//  
	bool LostPlayer(unsigned int player_id);

	//     
	virtual bool ReceiveMessage(int type, unsigned int sender, int length, unsigned char* data);

protected:
	//   " "
	bool ReceiveLostPlayer(unsigned int player_id);

public:
	//   -
	bool ConnectToServer(const char* ip);

	

	//  -
	bool SendChatMessage(CMagicString& message, int k_name, CMagicString* m_name, bool privated, bool global);

	//   
	GNetSocket* GetMainSocket(){return main_socket;}

	//   ,    
	const char* GetServerIP();

	//     
	bool IsMessageCorrected(int message_type);

	//    -   WaitReply()
	void SetWaitedReply(int message_type){waited_message_type=message_type;}
	//       -    SetWaitedReply()
	bool WaitReply();

	//     
	CONNECT_STATE GetConnectionState(){return connect_state;}

	//        ( id    )
	int RefreshReceiverList();

	//   
	virtual void OnUpdate();

	//     
	void OnConnect(bool connected);

	//  
	virtual void OnLostConnection(GNetSocket* socket);

	//   
	NET_DISCONNECT GetDisconnectionState(){return disconnect_state;}

	//   
	bool StartLocalServer();
	//   
	void StopLocalServer();

	//  
	bool IsHost(unsigned int player_id=0);

	//  ,        
	bool IsNextNetTakt();


	//  URL  IP-
	virtual CMagicString URLtoIP(const char* url)=0;

	//    
	bool DownloadFile(const char* url_file);

	//     (  ,     )
	void ReceiveFile(NET_BUFFER* buffer);

	//     
	//void SendSocketInfo();

	//         
	int AddSessionInfo(NET_SESSION_INFO* session_info);

	//  
	void SendHello();

	//   PING
	void SendPing();
};

struct NET_ERROR
{
	char name[MAX_NAME_LENGTH+1];		//  ,    
	int ban_stady;			//    0  3,   4-    
	int count_error;		//     ( -1,   )
	unsigned int time;		//    

	NET_ERROR()
	{
		name[0]=0;
		ban_stady=0;
		count_error=0;
		time=0;
	}
};

struct NET_BAN
{
	char name[MAX_NAME_LENGTH+1];		//   
	CMagicTime time;					//  
	int type;							//   (, )
};


struct NET_STATISTICS
{
	CMagicStringTable players;
	CMagicStringTable bans;
	CMagicStringTable errors;
};

struct NET_LICENSE
{
	GNetSocket* socket;
	int type;			//  

	int length;
	char* key;

	NET_LICENSE();
	virtual ~NET_LICENSE();

	void Create(GNetSocket* socket, int type, CMagicString* password);

	NET_LICENSE& operator=(const NET_LICENSE& nl);

	CMagicString GetLicenseKey();
};

// ,     
class GLicenseList : public CArrayBase
{
protected:
	int k_license;			//  
	NET_LICENSE** m_license;	//  

	GCriticalSection* critical_secion;

public:
	GLicenseList(GPlatform* platform);
	virtual ~GLicenseList();

	void Clear();

	//     
	bool Get(int index, NET_LICENSE* to);

	//      
	int GetMaxCount(){return k_license;}

	//  
	int AddLicense(NET_LICENSE* license);

	//    
	bool DeleteLicense(int index);
};

// ,    
struct CHAT_GLOBAL
{
	CMagicString sender;
	CMagicString receiver;
	CMagicString message;
};

// ,     
class GChatList
{
protected:
	int k_message;
	int max_message;
	CHAT_GLOBAL** m_message;

public:
	GChatList();
	virtual ~GChatList();

	void Reset();
	void AddMessage(const char* sender, const char* receiver, const char* message);

	int GetCount(){return k_message;}
	CHAT_GLOBAL* Get(int index){return m_message[index];}
};


#define MAX_LICENSE_TYPE 2

// ,     
class GNetServer : public GNet
{
public:
	GSocketList sockets;
protected:
	unsigned int stop_time;	//     
	bool stop_server;		//  ,   

	int k_migration_player;
	unsigned int* m_migration_player;
	NET_SESSION_INFO* migration_session;
	bool migration_ok;

	unsigned int start_time;	//   

	CMagicString login;
	CMagicString password;

	CMagicString shutdown_file;	//   ,     

	bool softly_reset;	//       

public:
	NET_STATISTICS statistics[MAX_LICENSE_TYPE];
	CMagicStringTable names;

	GLicenseList* licenses_for_verification;		//   
	GLicenseList* licenses_result;					//  
	CArrayPtr deleted_sockets_during_verification;	// ,       

	CArrayPtr sockets_nohello;	// ,   ,     HELLO

public:
	int max_player;	//   ,   

	int count_hello;
	int count_accept;

	SHUTDOWN_TYPE shutdown_type; //    

	GChatList chat;

public:
	GNetServer();
	virtual ~GNetServer();

	virtual bool Create(GUID_GAME guid, bool internet);
	virtual void Destroy();

	//    
	void SetAdminData(const char* login, const char* password);

	virtual bool IsServer(){return true;}

	//  ,      
	bool IsMigration(){return (k_migration_player!=0);}

	void ConnectSocket(GNetSocket* socket, unsigned int player_id=0xffffffff);
	
	GNetSocket* GetSocket(int index){return sockets.Get(index);}

	//  tcp- ,   
	GNetSocket* GetServerTCPSocket();
	//  udp- ,   
	GNetSocket* GetServerUDPSocket();

	virtual bool ReceiveMessage(int type, unsigned int sender, int length, unsigned char* data);
	

	//  
	virtual void StopServer()=0;

	//   
	virtual void OnUpdate(){UpdateNet();}

	//     
	bool SendHelloReply(GNetSocket* socket, MEM_DATA& buf, const char* name, int type_license, const char* license);

	//  
	virtual void OnLostConnection(GNetSocket* socket);

	//  ,     
	//void GetAllDataForAdmin(GMemWriter* writter);

	//     NET_SERVER_INFO
	void FillServerInfo(NET_SERVER_INFO& info);

	int AddSessionInfo(NET_SESSION_INFO* session_info, GNetSocket* socket);

	//   
	bool Unban(int license_type, const char* license);

	//         
	void SetShutdownFile(const char* path){shutdown_file=path;}

	//        
	void SaveShutdownType();

	//   
	void ServiceLicenses();

	//    
	bool StartLicenseVerificationThread();
	//    
	void StopLicenseVerificationThread();

	//  ,    
	void ServiceInactiveSockets();

	//     
	void ServiceSoftlyResetMode();

	//     
	void SendGlobalChat();

protected:
	//       ,    
	void SoftlyResetMode();

	//       
	void ResendMessage(int type, unsigned int sender,int length, unsigned char* data);

	//        
	//    ,      MESSAGE_TYPE_MIGRATION_OK
	bool IsAllPlayersMigrated(CMagicString* name, CMagicString* license);

	//      
	void SendMigration(unsigned int receiver, CMagicString* name, CMagicString* license);

	//          -
	//         ,     
	GNetSocket* ReceiverVerificationByName(unsigned int sender, CMagicString& name, bool global);

};