//-----------------------------------------------------------------------------
// File: NetLibUV.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "NetLibUV.h"
#include "Game.h"

#pragma comment(lib, "../Tools/GUI/libuv/libuv.lib")

extern void ErrorLibUV(int error);

GNet* NewNetServerLibUV()
{
	return new GNetServerLibUV;
}

GNet* NewNetClientLibUV()
{
	return new GNetClientLibUV;
}

GNetClient* NewNetDownloaderLibUV()
{
	return new GNetDownloaderLibUV;
}

uv_loop_t* GetLoop(GNet* net)
{
	if (net->IsServer())
	{
		GNetServerLibUV* server=(GNetServerLibUV*)net;
		return &(server->loop);
	}
	else
	{
		GNetClientLibUV* client=(GNetClientLibUV*)net;
		return &(client->loop);
	}
}

uv_tcp_t* GetPtrTCP(void* ptr)
{
	return (uv_tcp_t*)(((char*)ptr)+sizeof(void*));
}

uv_udp_t* GetPtrUDP(void* ptr)
{
	return (uv_udp_t*)(((char*)ptr)+sizeof(void*));
}

GNetSocket* GetPtrSocket(void* ptr)
{
	return *((GNetSocket**)ptr);
}

GNetSocket* GetNetSocketPtr(void* uv_socket)
{
	return GetPtrSocket(((char*)uv_socket)-sizeof(void*));
}

uv_write_t* UV_NET_BUFFER::GetPtrWrite()
{
	return (uv_write_t*)sender_object;
}

uv_udp_send_t* UV_NET_BUFFER::GetPtrSend()
{
	return (uv_udp_send_t*)sender_object;
}

// ----------------------------------------------------------

//   
void OnAccept(uv_stream_t* server_socket, int status)
{
	if (status < 0)
	{
		fprintf(stderr, "New connection error %s\n", uv_strerror(status));
		// error!
		return;
	}

	GNetSocket* socket=GetNetSocketPtr(server_socket);
	socket->Accept();
}

void OnCloseSocket(uv_handle_t* handle)
{
	char* ptr=(char*)handle;
	ptr-=sizeof(void*);
	free(ptr);
}

void OnAllocBuffer(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf)
{
	GNetSocket* socket=GetNetSocketPtr(handle);
	NET_BUFFER* recv_buffer=socket->net->GetRecvBuffer();

	unsigned int max_length=recv_buffer->GetMaxLength();
	if (max_length<suggested_size)
		recv_buffer->SetMaxSize(suggested_size);

	buf->len=suggested_size;
	buf->base=(char*)recv_buffer->GetData();
}

char address_converter[30];
void OnReadUDP(uv_udp_t* handle, ssize_t nread, const uv_buf_t* buf, const struct sockaddr* addr, unsigned flags)
{
	GNetSocket* socket=GetNetSocketPtr(handle);
	NET_BUFFER* recv_buffer=socket->net->GetRecvBuffer();
	assert(buf->base==(char*)recv_buffer->GetData());
	recv_buffer->SetLength(nread);

	int r=uv_ip4_name((sockaddr_in*)addr, address_converter, sizeof(address_converter));
	socket->addr->address=address_converter;
	unsigned char* port_ptr=(unsigned char*)&(((sockaddr_in*)addr)->sin_port);
	socket->addr->port=port_ptr[1];
	socket->addr->port+=port_ptr[0]<<8;

	socket->ReceiveUPD();
}

void OnReadTCP(uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf)
{
	GNetSocket* socket=GetNetSocketPtr(stream);

	if (nread>0)
	{
		NET_BUFFER* recv_buffer=socket->net->GetRecvBuffer();
		assert(buf->base==(char*)recv_buffer->GetData());
		recv_buffer->SetLength(nread);
		socket->ReceiveTCP();
	}
	else
	{
		//  ,   
		socket->net->OnLostConnection(socket);
	}
}

void OnReadFile(uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf)
{
	GNetSocketLibUV* socket=(GNetSocketLibUV*)GetNetSocketPtr(stream);

	if (nread>0)
	{
		NET_BUFFER* recv_buffer=socket->net->GetRecvBuffer();
		assert(buf->base==(char*)recv_buffer->GetData());
		recv_buffer->SetLength(nread);
		((GNetClient*)socket->net)->ReceiveFile(recv_buffer);
	}
	else
	{
		//  ,   
		socket->net->OnLostConnection(socket);
	}
}

void OnConnect(uv_connect_t* req, int status)
{
	GNetClient* net=((UV_CONNECT_DATA*)req)->net;
	if (status==0)
	{
		net->OnConnect(true);
	}
	else
	{
		//ErrorLibUV(status);
		net->OnConnect(false);
	}
}

void OnSendUDP(uv_udp_send_t* req, int status)
{
	int offset=offsetof(UV_NET_BUFFER, sender_object);
	UV_NET_BUFFER* buf=(UV_NET_BUFFER*)(((char*)req)-offset);
	NET_BUFFER_LIST* list=buf->owner;
	int index=buf->GetIndex();
	list->DeleteBuffer(index);
}

void OnSendTCP(uv_write_t* req, int status)
{
	int offset=offsetof(UV_NET_BUFFER, sender_object);
	UV_NET_BUFFER* buf=(UV_NET_BUFFER*)(((char*)req)-offset);
	NET_BUFFER_LIST* list=buf->owner;
	int index=buf->GetIndex();
	//buf->count--;
	//if (buf->count<=0)
		list->DeleteBuffer(index);
}

CMagicString url_to_ip;
void OnURL(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
{
	url_to_ip="";
	if (status==0)
	{
		int r=uv_ip4_name((sockaddr_in*)res->ai_addr, address_converter, sizeof(address_converter));
		if (r==0)
		{
			url_to_ip=address_converter;
		}
	}

	if (res)
		uv_freeaddrinfo(res);
}

// ------------------------------------------------------------------------

GNetSocketLibUV::GNetSocketLibUV(GNet* net) : GNetSocket(net)
{
	sock=NULL;
}

GNetSocketLibUV::~GNetSocketLibUV()
{
	Destroy();
}

//  UDP  TCP- ,   listen=true,    
bool GNetSocketLibUV::Create(bool udp_tcp, int port, bool listen)
{
	GNetSocket::Create(udp_tcp, port, listen);

	uv_loop_t* loop=GetLoop(net);

	if (udp_tcp)
	{
		sock=malloc(sizeof(TCP_SOCKET));
		memset(sock, 0, sizeof(TCP_SOCKET));
		((TCP_SOCKET*)sock)->net_socket=this;
		int r=uv_tcp_init(loop, GetPtrTCP(sock));
		assert(r==0);
		r=uv_tcp_nodelay(GetPtrTCP(sock), true);
		assert(r==0);

		uv_tcp_t* tcp=GetPtrTCP(sock);
		if (listen)
		{
			sockaddr_in address;
			uv_ip4_addr("0.0.0.0", port, &address);

			int r=uv_tcp_bind(tcp, (const struct sockaddr*)&address, 0);
			assert(r==0);
			r=uv_listen((uv_stream_t*)tcp, 1024, OnAccept);
			if (r)
				return false;
		}
	}
	else
	{
		sock=malloc(sizeof(UDP_SOCKET));
		memset(sock, 0, sizeof(UDP_SOCKET));
		uv_udp_t* udp=GetPtrUDP(sock);
		int r=uv_udp_init(loop, udp);
		assert(r==0);

		struct sockaddr_in broadcast_addr;
		uv_ip4_addr("0.0.0.0", port, &broadcast_addr);
		r=uv_udp_bind(udp, (const struct sockaddr *)&broadcast_addr, 0);
		assert(r==0);

		r=uv_udp_set_broadcast(udp, 1);
		assert(r==0);

		if (listen)
		{
			r=uv_udp_recv_start(udp, OnAllocBuffer, OnReadUDP);
			assert(r==0);
		}

		((UDP_SOCKET*)sock)->net_socket=this;
	}

	return true;
}

//     TCP-   
bool GNetSocketLibUV::SetConnectedSocketToReadMode()
{
	if (udp_tcp)
	{
		uv_tcp_t* tcp=GetPtrTCP(sock);
		int r=0;
		if (net->IsDownloader())
			r=uv_read_start((uv_stream_t*)tcp, OnAllocBuffer, OnReadFile);
		else
			r=uv_read_start((uv_stream_t*)tcp, OnAllocBuffer, OnReadTCP);
		return (r==0);
	}
	return false;
}

//  
void GNetSocketLibUV::Destroy()
{
	if (sock)
	{
		if (udp_tcp)
		{
			uv_tcp_t* tcp=GetPtrTCP(sock);
			uv_close((uv_handle_t*)tcp, OnCloseSocket);
			((TCP_SOCKET*)sock)->net_socket=NULL;
		}
		else
		{
			uv_udp_t* udp=GetPtrUDP(sock);
			int r=uv_read_stop((uv_stream_t*)udp);
			assert(r==0);
			uv_close((uv_handle_t*)udp, OnCloseSocket);
			((UDP_SOCKET*)sock)->net_socket=NULL;
		}
		sock=NULL;
	}

	GNetSocket::Destroy();
}

//  IP-   TCP-
// own_or_peer          
bool GNetSocketLibUV::GetIP(CMagicString& addr, bool own_or_peer)
{
	if (GNetSocket::GetIP(addr, own_or_peer))
	{
		uv_tcp_t* socket=GetPtrTCP(sock);
		struct sockaddr sockname;
		memset(&sockname, -1, sizeof sockname);  
		int namelen=sizeof sockname;
		int r;

		if (own_or_peer)
			r=uv_tcp_getpeername(socket, &sockname, &namelen);
		else
			r=uv_tcp_getsockname(socket, &sockname, &namelen); 

		if (r==0)
		{
			r=uv_ip4_name((sockaddr_in*)&sockname, address_converter, sizeof(address_converter));
			if (r==0)
			{
				addr=address_converter;
				unsigned char* port_ptr=(unsigned char*)&(((sockaddr_in*)&sockname)->sin_port);
				int port=port_ptr[1];
				port+=port_ptr[0]<<8;
				addr+=':';
				CMagicString d;
				d.IntToString(port);
				addr+=d;
			}
		}
	}
	return false;
}

bool GNetSocketLibUV::Accept()
{
	GNetSocketLibUV* accept_sock=(GNetSocketLibUV*)net->NewSocket(net);
	accept_sock->Create(true, 0, false);
	if (IsAdmin())
		accept_sock->SetAdmin();

	uv_tcp_t* server=GetPtrTCP(sock);
	uv_tcp_t* client=GetPtrTCP(accept_sock->sock);
	if (uv_accept((uv_stream_t*)server, (uv_stream_t*)client)==0)
	{
		if (uv_read_start((uv_stream_t*)client, OnAllocBuffer, OnReadTCP)==0)
		{
			//accept_sock->time=net->platform->GetTick();
			accept_sock->GetIP(accept_sock->ip, true);

			GNetServer* server=((GNetServer*)net);
			server->count_accept++; //   
			server->sockets_nohello.Add(accept_sock);
			return true;
		}
		else
			return false;
	}
	else
	{
		// 
		return false;
	}
}

//    (  TCP-)
bool GNetSocketLibUV::Connect(NET_ADDRESS* addr)
{
	sockaddr_in dest;
	uv_ip4_addr(addr->address, addr->port, &dest);

	uv_tcp_t* tcp=GetPtrTCP(sock);
	UV_CONNECT_DATA* connect_data=&((GNetClientLibUV*)net)->connect_data;
	connect_data->net=(GNetClient*)net;
	if (!uv_tcp_connect(connect_data, tcp, (const struct sockaddr*)&dest, OnConnect))
	{
		uv_loop_t* loop=GetLoop(net);
		uv_run(loop, UV_RUN_DEFAULT);
		return true;
	}
	return false;
}

void GNetSocketLibUV::SendTCP(NET_BUFFER_INDEX* buf)
{
	if (buf->length>0)
	{
		uv_buf_t buffer;
		buffer.len=buf->length;
		buffer.base=(char*)buf->data;
		int r=uv_write(((UV_NET_BUFFER*)buf)->GetPtrWrite(), (uv_stream_t*)GetPtrTCP(sock), &buffer, 1, OnSendTCP);
		assert(r==0);
	}
}

void GNetSocketLibUV::SendUDP(NET_BUFFER_INDEX* buf)
{
	if (buf->length>0)
	{
		uv_buf_t buffer;
		buffer.len=buf->length;
		buffer.base=(char*)buf->data;
		struct sockaddr_in send_addr;
		uv_ip4_addr(addr->address, addr->port, &send_addr);
		uv_udp_send_t* send=((UV_NET_BUFFER*)buf)->GetPtrSend();
		uv_udp_t* udp=GetPtrUDP(sock);

		int r=uv_udp_send(send, udp, &buffer, 1, (const struct sockaddr*)&send_addr, OnSendUDP);
		if (r!=0)
		{
			ErrorLibUV(r);
			assert(r==0);
		}
	}
}

void GNetSocketLibUV::ReceiveTCP()
{
	NET_BUFFER* recv_buffer=net->GetRecvBuffer();
	int received_bytes=recv_buffer->GetLength();
	rbuffer.Add(received_bytes, recv_buffer->GetData());

	if (net->IsServer())
		ReceiveMessages();
}

void GNetSocketLibUV::ReceiveUPD()
{
	NET_BUFFER* recv_buffer=net->GetRecvBuffer();
	int received_bytes=recv_buffer->GetLength();
	rbuffer.Add(received_bytes, recv_buffer->GetData());

	if (net->IsServer())
		ReceiveMessages();
}

// ------------------------------------------------------------------------

GNetClientLibUV::GNetClientLibUV()
{
	memset(&loop, 0, sizeof(loop));
	memset(&connect_data, 0, sizeof(connect_data));
	memset(&url_req, 0, sizeof(url_req));
	
	int r=uv_loop_init(&loop);
	assert(r==0);
}

/*
bool GNetClientLibUV::Create(GUID_GAME guid, const char* server_ip)
{
	//int r=uv_loop_init(&loop);
	//if (r==0)
	//{
		if (GNetClient::Create(guid, server_ip))
			return true;
	//}
	return false;
}
*/

void GNetClientLibUV::Destroy()
{
	GNetClient::Destroy();
	uv_loop_t* loop=GetLoop(this);
	uv_run(loop, UV_RUN_DEFAULT);
	uv_loop_close(loop);
}

GNetSocket* GNetClientLibUV::NewSocket(GNet* net)
{
	return new GNetSocketLibUV(net);
}

NET_BUFFER_INDEX* GNetClientLibUV::NewBuffer(int index)
{
	return new UV_NET_BUFFER(index);
}

//   
bool GNetClientLibUV::UpdateNet()
{
	uv_loop_t* loop=GetLoop(this);
	uv_run(loop, UV_RUN_NOWAIT);
	return (disconnect_state==NET_DISCONNECT_OK);
}

//  URL  IP-
CMagicString GNetClientLibUV::URLtoIP(const char* url)
{
	int r=uv_getaddrinfo(GetLoop(this), &url_req, OnURL, url, NULL, NULL);
	if (r==0)
	{
		url_to_ip="WAIT";

		while(true)
		{
			UpdateNet();
			if (url_to_ip!="WAIT")
				break;
		}
		return url_to_ip;
	}
	return "";
}

// ------------------------------------------------------------------------

unsigned int timer_count=0;
GNetServerLibUV* uv_server=NULL;
void OnTimer(uv_timer_t* handle)
{
	if (uv_server)
	{
		uv_server->ServiceLicenses();

		timer_count++;
		if (!(timer_count % 60)) //   60 
		{
			//   ,    
			uv_server->ServiceInactiveSockets();
			//       " "
			uv_server->ServiceSoftlyResetMode();
		}

		//if (!(timer_count % 3)) //   3 
		{
			//     
			uv_server->SendGlobalChat();
		}
	}
}

GNetServerLibUV::GNetServerLibUV()
{
	timer_count=0;
	memset(&loop, 0, sizeof(loop));
	memset(&timer, 0, sizeof(timer));

	int r=uv_loop_init(&loop);
	assert(r==0);

	uv_timer_init(&loop, &timer);
	uv_timer_start(&timer, OnTimer, 0, 1000);

	uv_server=this;
}

void GNetServerLibUV::Destroy()
{
	GNetServer::Destroy();
	uv_server=NULL;

	uv_timer_stop(&timer);

	uv_run(&loop, UV_RUN_DEFAULT);
	uv_loop_close(&loop);
}

//  
void GNetServerLibUV::StopServer()
{
	uv_stop(GetLoop(this));
}

GNetSocket* GNetServerLibUV::NewSocket(GNet* net)
{
	return new GNetSocketLibUV(net);
}

NET_BUFFER_INDEX* GNetServerLibUV::NewBuffer(int index)
{
	return new UV_NET_BUFFER(index);
}

//   
bool GNetServerLibUV::UpdateNet()
{
	uv_loop_t* loop=GetLoop(this);

	uv_run(loop, UV_RUN_DEFAULT);
	return true;
}

// ------------------------------------------------------------------------

void ErrorLibUV(int error)
{
	int r=0;
	switch (error)
	{
	case UV_E2BIG:
		// argument list too long
		r=1;
		break;

	case UV_EACCES:
		// permission denied
		r=1;
		break;

	case UV_EADDRINUSE:
		// address already in use
		r=1;
		break;

	case UV_EADDRNOTAVAIL:
		//address not available
		r=1;
		break;

	case UV_EAFNOSUPPORT:
		// address family not supported
		r=1;
		break;

	case UV_EAGAIN:
		// resource temporarily unavailable
		r=1;
		break;

	case UV_EAI_ADDRFAMILY:
		// address family not supported
		r=1;
		break;

	case UV_EAI_AGAIN:
		// temporary failure
		r=1;
		break;

	case UV_EAI_BADFLAGS:
		// bad ai_flags value
		r=1;
		break;

	case UV_EAI_BADHINTS:
		// invalid value for hints
		r=1;
		break;

	case UV_EAI_CANCELED:
		// request canceled
		r=1;
		break;

	case UV_EAI_FAIL:
		// permanent failure
		r=1;
		break;

	case UV_EAI_FAMILY:
		// ai_family not supported
		r=1;
		break;

	case UV_EAI_MEMORY:
		// out of memory
		r=1;
		break;

	case UV_EAI_NODATA:
		// no address
		r=1;
		break;

	case UV_EAI_NONAME:
		// unknown node or service
		r=1;
		break;

	case UV_EAI_OVERFLOW:
		// argument buffer overflow
		r=1;
		break;

	case UV_EAI_PROTOCOL:
		// resolved protocol is unknown
		r=1;
		break;

	case UV_EAI_SERVICE:
		// service not available for socket type
		r=1;
		break;

	case UV_EAI_SOCKTYPE:
		// socket type not supported
		r=1;
		break;

	case UV_EALREADY:
		// connection already in progress
		r=1;
		break;

	case UV_EBADF:
		// bad file descriptor
		r=1;
		break;

	case UV_EBUSY:
		// resource busy or locked
		r=1;
		break;

	case UV_ECANCELED:
		// operation canceled
		r=1;
		break;

	case UV_ECHARSET:
		// invalid Unicode character
		r=1;
		break;

	case UV_ECONNABORTED:
		// software caused connection abort
		r=1;
		break;

	case UV_ECONNREFUSED:
		// connection refused
		r=1;
		break;

	case UV_ECONNRESET:
		// connection reset by peer
		r=1;
		break;

	case UV_EDESTADDRREQ:
		// destination address required
		r=1;
		break;

	case UV_EEXIST:
		// file already exists
		r=1;
		break;

	case UV_EFAULT:
		// bad address in system call argument
		r=1;
		break;

	case UV_EFBIG:
		// file too large
		r=1;
		break;

	case UV_EHOSTUNREACH:
		// host is unreachable
		r=1;
		break;

	case UV_EINTR:
		// interrupted system call
		r=1;
		break;

	case UV_EINVAL:
		// invalid argument
		r=1;
		break;

	case UV_EIO:
		// i/o error
		r=1;
		break;

	case UV_EISCONN:
		// socket is already connected
		r=1;
		break;

	case UV_EISDIR:
		// illegal operation on a directory
		r=1;
		break;

	case UV_ELOOP:
		// too many symbolic links encountered
		r=1;
		break;

	case UV_EMFILE:
		// too many open files
		r=1;
		break;

	case UV_EMSGSIZE:
		// message too long
		r=1;
		break;

	case UV_ENAMETOOLONG:
		// name too long
		r=1;
		break;

	case UV_ENETDOWN:
		// network is down
		r=1;
		break;

	case UV_ENETUNREACH:
		// network is unreachable
		r=1;
		break;

	case UV_ENFILE:
		// file table overflow
		r=1;
		break;

	case UV_ENOBUFS:
		// no buffer space available
		r=1;
		break;

	case UV_ENODEV:
		// no such device
		r=1;
		break;

	case UV_ENOENT:
		// no such file or directory
		r=1;
		break;

	case UV_ENOMEM:
		// not enough memory
		r=1;
		break;

	case UV_ENONET:
		// machine is not on the network
		r=1;
		break;

	case UV_ENOPROTOOPT:
		// protocol not available
		r=1;
		break;

	case UV_ENOSPC:
		// no space left on device
		r=1;
		break;

	case UV_ENOSYS:
		// function not implemented
		r=1;
		break;

	case UV_ENOTCONN:
		// socket is not connected
		r=1;
		break;

	case UV_ENOTDIR:
		// not a directory
		r=1;
		break;

	case UV_ENOTEMPTY:
		// directory not empty
		r=1;
		break;

	case UV_ENOTSOCK:
		// socket operation on non-socket
		r=1;
		break;

	case UV_ENOTSUP:
		// operation not supported on socket
		r=1;
		break;

	case UV_EPERM:
		// operation not permitted
		r=1;
		break;

	case UV_EPIPE:
		// broken pipe
		r=1;
		break;

	case UV_EPROTO:
		// protocol error
		r=1;
		break;

	case UV_EPROTONOSUPPORT:
		// protocol not supported
		r=1;
		break;

	case UV_EPROTOTYPE:
		// protocol wrong type for socket
		r=1;
		break;

	case UV_ERANGE:
		// result too large
		r=1;
		break;

	case UV_EROFS:
		// read-only file system
		r=1;
		break;

	case UV_ESHUTDOWN:
		// cannot send after transport endpoint shutdown
		r=1;
		break;

	case UV_ESPIPE:
		// invalid seek
		r=1;
		break;

	case UV_ESRCH:
		// no such process
		r=1;
		break;

	case UV_ETIMEDOUT:
		// connection timed out
		r=1;
		break;

	case UV_ETXTBSY:
		// text file is busy
		r=1;
		break;

	case UV_EXDEV:
		// cross-device link not permitted
		r=1;
		break;

	case UV_UNKNOWN:
		// unknown error
		r=1;
		break;

	case UV_EOF:
		// end of file
		r=1;
		break;

	case UV_ENXIO:
		// no such device or address
		r=1;
		break;

	case UV_EMLINK:
		// too many links
		r=1;
		break;
	}
}
