//-----------------------------------------------------------------------------
// File: NetLibUV.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once
#include "Net.h"

#include "libuv/include/uv.h"

struct NET_SOCKET_PTR
{
	GNetSocket* net_socket;
};

struct TCP_SOCKET : public NET_SOCKET_PTR, public uv_tcp_t
{
};

struct UDP_SOCKET : public NET_SOCKET_PTR, public uv_udp_t
{
};

//#if sizeof(uv_write_t)>sizeof(uv_udp_send_t)
#define UV_SENDER_SIZE sizeof(uv_write_t)
//#else
//#define UV_SENDER_SIZE sizeof(uv_udp_send_t)
//#endif

struct UV_NET_BUFFER : public NET_BUFFER_INDEX
{
	char sender_object[UV_SENDER_SIZE];

	UV_NET_BUFFER(int index) : NET_BUFFER_INDEX(index){}

	uv_write_t* GetPtrWrite();
	uv_udp_send_t* GetPtrSend();
};

class GNetSocketLibUV : public GNetSocket
{
public:
	void* sock;

public:
	GNetSocketLibUV(GNet* net);
	virtual ~GNetSocketLibUV();

	//  UDP  TCP- ,   listaen=true,    
	virtual bool Create(bool udp_tcp, int port, bool listen);

	//     TCP-   
	virtual bool SetConnectedSocketToReadMode();

	//  
	virtual void Destroy();

	//  IP-   TCP-
	// own_or_peer          
	virtual bool GetIP(CMagicString& addr, bool own_or_peer);

	//    (  TCP-)
	virtual bool Accept();

	//    (  TCP-)
	virtual bool Connect(NET_ADDRESS* addr);

	virtual void SendTCP(NET_BUFFER_INDEX* buf);
	virtual void SendUDP(NET_BUFFER_INDEX* buf);

	virtual void ReceiveTCP();
	virtual void ReceiveUPD();
};

struct UV_CONNECT_DATA : public uv_connect_t
{
	GNetClient* net;
};

class GNetClientLibUV :
	public GNetClient
{
public:
	uv_loop_t loop;

	UV_CONNECT_DATA connect_data;
	uv_getaddrinfo_t url_req;

public:
	GNetClientLibUV();

	//virtual bool Create(GUID_GAME guid, const char* server_ip);
	virtual void Destroy();

	virtual GNetSocket* NewSocket(GNet* net);
	virtual NET_BUFFER_INDEX* NewBuffer(int index);

	//   
	virtual bool UpdateNet();

	//  URL  IP-
	virtual CMagicString URLtoIP(const char* url);
};

class GNetServerLibUV :
	public GNetServer
{
public:
	uv_loop_t loop;

	uv_timer_t timer;

public:
	GNetServerLibUV();

	//virtual bool Create(GUID_GAME guid, bool internet);
	virtual void Destroy();

	//  
	virtual void StopServer();

	virtual GNetSocket* NewSocket(GNet* net);
	virtual NET_BUFFER_INDEX* NewBuffer(int index);

	//   
	virtual bool UpdateNet();
};

class GNetDownloaderLibUV :
	public GNetClientLibUV
{
public:
	virtual bool IsDownloader(){return true;}
};