#include "stdafx.h"
#include "PlatformUnix.h"

#include <sys/time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>


#define MAX_PATH	2048


GCriticalSectionUnix::GCriticalSectionUnix() : GCriticalSection()
{
	pthread_mutex_init(&mutex, NULL);
}

GCriticalSectionUnix::~GCriticalSectionUnix()
{
    pthread_mutex_destroy(&mutex);
}

void GCriticalSectionUnix::Lock()
{
    pthread_mutex_lock(&mutex);
}

void GCriticalSectionUnix::Unlock()
{
    pthread_mutex_unlock(&mutex);
}


GPlatformUnix::GPlatformUnix(GGame* owner, const char* game_name) : GPlatform(owner, game_name)
{
	start_path=GetDirectory();
	start_path+=GetSlash();

	is_continued=true;
	#ifndef MAX_FPS
	//hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
	#endif
}

GPlatformUnix::~GPlatformUnix(void)
{
    /*
	#ifndef MAX_FPS
	if (hEvent)
	{
		CloseHandle(hEvent);
		hEvent=NULL;
	}
	#endif
	*/
}

/*
// Функция игры, после завершения которой игра заканчивается
void GPlatformUnix::Play()
{
	MSG msg;
	#ifdef MAX_FPS
	while (is_continued)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) // если указать hWnd, то для WM_CHAR будет автоматически срабатывать автоповтор, но если hWnd=NULL, то срабатывает переключение языков WM_INPUT_LANGUAGE
		//if (PeekMessage(&msg,hWnd,0,0,PM_REMOVE)) // если указать hWnd, то для WM_CHAR будет автоматически срабатывать автоповтор
		//if (PeekMessage(&msg,0,0,0,PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
			owner->PlatformTimer();
	}
	#else

	if (owner->IsMainTimer())
	{
		while (is_continued)
		{
			if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) // если указать hWnd, то для WM_CHAR будет автоматически срабатывать автоповтор, но если hWnd=NULL, то срабатывает переключение языков WM_INPUT_LANGUAGE
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else
			{
				WaitForSingleObject(hEvent,INFINITE);
				owner->PlatformTimer();
			}
		}
	}
	else
	{
		// таймер отсутсвует
		while (is_continued)
		{
			if (GetMessage(&msg, NULL, 0, 0)>0)
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else
				is_continued=false;
		}
	}

	#endif

	DestroyWindow(hWnd);
}

// Функция, которая позволяет завершить игру в любой момент
void GPlatformUnix::Quit()
{
	is_continued=false;
}

void FAR PASCAL OnTimerShot(UINT wTimerID, UINT msg, DWORD_PTR dwUser, DWORD dw1, DWORD dw2)
{
	#ifndef MAX_FPS
	SetEvent(hEvent);
	#endif
}


// Работа с таймерами
void GPlatformUnix::SetTimer(TIMER_DATA* data)
{
	UINT    wTimerRes;  // разрешение таймера
	TIMECAPS  timecaps;  // требуется для функции timeGetDevCaps

	// получить max & min of системного таймера
	if (timeGetDevCaps( &timecaps, sizeof( TIMECAPS ) ) == TIMERR_NOERROR )
	{
		// получить оптимальное разрешение
		wTimerRes = max( timecaps.wPeriodMin, 2 );

		// установить минимальное разрешение для нашего таймера
		if( timeBeginPeriod( wTimerRes ) == TIMERR_NOERROR )
		{
			data->timer=(void*)timeSetEvent(data->period, wTimerRes, OnTimerShot, (DWORD_PTR)data, TIME_PERIODIC);
		}
	}

	//::SetTimer(hWnd, data->id, data->period, NULL);
}

void GPlatformUnix::KillTimer(TIMER_DATA* data)
{
	if (data->timer)
	{
		timeKillEvent((unsigned int)data->timer);
		data->timer=NULL;
	}
	//::KillTimer(hWnd, data->id);
}

// Признак активности окна
bool GPlatformUnix::IsActivated()
{
	HWND active_hwnd=GetForegroundWindow();//GetActiveWindow();
	return (active_hwnd==hWnd);
	//return true;
}

// Признак того, что окно свернуто
bool GPlatformUnix::IsIconic()
{
	if (::IsIconic(hWnd))
		return true;
	return false;
}

// Сворачивание окна
void GPlatformUnix::ToIconic()
{
	ShowWindow(hWnd, SW_MINIMIZE);
}

// Выполнение операции разворачивания окна
void GPlatformUnix::ToMaximize()
{
	ShowWindow(hWnd, SW_MAXIMIZE);
}
*/

// eng: Returns time in milliseconds
// rus: Возвращает время в милисекундах
unsigned int GPlatformUnix::GetTick()
{
	timeval time_val;
	gettimeofday(&time_val, NULL);

	return time_val.tv_sec*1000+time_val.tv_usec/1000;
}

/*
// eng: Returns current time
// rus: Возвращает текущую дату
void GPlatformUnix::GetCurrentDate(CMagicTime* date)
{
	SYSTEMTIME st;
	GetLocalTime(&st);

	date->year=st.wYear;
	date->month=st.wMonth-1;
	date->day=st.wDay;

	date->hour=st.wHour;
	date->minute=st.wMinute;
	date->second=st.wSecond;
}
*/

// eng: Returns path to folder with game
// rus: Возвращает путь к папке с игрой
const char* GPlatformUnix::GetPathToGame()
{
	if (game_path.IsEmpty())
	{
        char path[MAX_PATH+1];
		ssize_t len=readlink("/proc/self/exe", path, MAX_PATH);
		path[len]=0;
	    game_path=path;
	    game_path=game_path.GetPathOnly();
	    game_path+='/';
	}
	return game_path;
}

// eng: Returns path to folder which could be used by wrapper to store temporary files
// rus: Возвращает путь к временной папке
const char* GPlatformUnix::GetPathToTemp()
{
	if (temp_path.IsEmpty())
	{
        /*
        temp_path="~/.";
        temp_path+=game_name;
        bool ok=CreatePath(temp_path);
        temp_path+='/';
        */
        temp_path="/var/tmp/";
	}
	return temp_path;
}


// eng: Finds the first ptc-file in emitters folder
// rus: Поиск первого ptc-файла в папке с эмиттерами
const char* GPlatformUnix::GetFirstFile_OS(const char* path, const char* ext)
{
	POSIX_FIND_ITEM* finder=new POSIX_FIND_ITEM;
	finder->dir=opendir(path);
	if (finder->dir)
	{
        if (ext)
        {
            CMagicString vext=ext;
            if (vext!="*")
            {
                finder->ext='.';
                finder->ext+=ext;
            }
        }
        finders.Add(finder);
		return GetNextFile_OS();
    }

    delete finder;
    return NULL;
}

// eng: Finds the next ptc-file in emitters folder
// rus: Поиск очередного ptc-файла в папке с эмиттерами
const char* GPlatformUnix::GetNextFile_OS()
{
	FINDER_DATA* f=finders.Get();
	if (f && (!f->IsVirtualDrive()))
	{
		POSIX_FIND_ITEM* finder=(POSIX_FIND_ITEM*)f;
		dirent* result=readdir(finder->dir);
        while(result)
        {
            if (result->d_type!=DT_DIR)
            {
                // it is not folder
                if (finder->ext.IsEmpty())
                    return result->d_name;
                else
                {
                    int len=strlen(result->d_name);
                    if (len>4)
                    {
                        const char* ext=&(result->d_name[len-4]);
                        if (strcmp(ext, finder->ext)==0)
                        {
                            return result->d_name;
                        }
                    }
                }
            }
            result=readdir(finder->dir);
        }
        closedir(finder->dir);
        finders.Delete();
	}
	return NULL;
}

// eng: Finds the first child folder in folder
// rus: Поиск первой дочерней папки в папке
const char* GPlatformUnix::GetFirstFolder_OS(const char* path)
{
	POSIX_FIND_ITEM* finder=new POSIX_FIND_ITEM;
	finder->dir=opendir(path);
	if (finder->dir)
	{
        finders.Add(finder);
		return GetNextFolder_OS();
    }

    delete finder;
    return NULL;
}

// eng: Finds the next child folder in folder
// rus: Поиск очередной дочерней папки в папке
const char* GPlatformUnix::GetNextFolder_OS()
{
	FINDER_DATA* f=finders.Get();
	if (f && (!f->IsVirtualDrive()))
	{
		POSIX_FIND_ITEM* finder=(POSIX_FIND_ITEM*)f;
		dirent* result=readdir(finder->dir);
        while(result)
        {
            if (result->d_type==DT_DIR)
            {
                // it is folder
                CMagicString name=result->d_name;
                if (name!="." && name!="..")
                    return result->d_name;
            }
            result=readdir(finder->dir);
        }
        closedir(finder->dir);
        finders.Delete();
	}
	return NULL;
}

// Deleting of file (analogue remove())
// Удаление файла (аналог remove())
int GPlatformUnix::RemoveFile(const char* file)
{
    return remove(file);
}

// eng: Returns file length or -1
// rus: Возвращает длину файла или -1, если файл отсутствует
int GPlatformUnix::GetFileLength(const char* file)
{
	FILE* f=fopen(file, "rb");

	if (f)
	{
		fseek(f,0,SEEK_END);
		long len=ftell(f);
		fclose(f);
		return (int)len;
	}
	return -1;
}

// Creating of folder
// Создание папки
bool GPlatformUnix::CreateFolder_OS(const char* folder)
{
    int status=mkdir(folder, S_IRWXU | S_IRWXG | S_IRWXO);
	if (status==0)
		return true;
	return false;
}

// Deleting of folder
// Удаление папки
bool GPlatformUnix::RemoveFolder_OS(const char* folder)
{
    int state=rmdir(folder);
	if (state==0)
		return true;
	return false;
}

// eng: Opening of file
// rus: Открытие файла
FILE* GPlatformUnix::OpenFile(const char* file, const char* mode)
{
	FILE* f=fopen(file, mode);
	return f;
}

// Current directory
// Текущий каталог
bool GPlatformUnix::SetDirectory_OS(const char* path)
{
    int state=chdir(path);
    if (state==0)
		return true;
	return false;
}

const char* GPlatformUnix::GetDirectory_OS()
{
    char path[MAX_PATH];
    getcwd(path, MAX_PATH);
    current_directory=path;
	return current_directory;
}

// Возвращает признак того, что это абсолютный путь
bool GPlatformUnix::IsPathAbsolute_OS(CMagicString& path)
{
	int len=path.GetLength();
	if (len>0)
	{
		if (path[0]=='/')// || path[0]=='~')
			return true;
	}
	return false;
}

// Запуск файла на выполŜнение
bool GPlatformUnix::ExecuteProcess(const char* path, const char* param, const char* working_dir, PROCESS_DATA* process_id)
{
    if (GetFileLength(path)==-1)
        return false;

    int status;
    pid_t pid;

    pid=fork();
    if (pid>=0)
    {
        // This is the child process.  Execute the shell command
        if (pid==0)
        {
            // This is a child process
            if (execl(path, path, param, NULL)==-1)
                return false;
        }
        pid_t* vpid=(pid_t*)process_id->process_info;
        *vpid=pid;
        return true;
    }
    else
        return false;
/*
	CMagicString full_path=path;
	full_path+=' ';
	full_path+=param;

	wchar_t* wworking_dir=NULL;
	if (working_dir)
	{
		CMagicString v=working_dir;
		int len=v.GetLength();
		if (v)
		{
			const wchar_t* vv=utf8_to_wchar(working_dir);
			wworking_dir=new wchar_t[len+1];
			for (int i=0;i<len;i++)
				wworking_dir[i]=vv[i];
			wworking_dir[len]=0;
		}
	}

	const wchar_t* wfull_path=utf8_to_wchar(full_path);

	PROCESS_INFORMATION* ProcessInfo=(PROCESS_INFORMATION*)(process_id->process_info);
	STARTUPINFOW StartupInfo = {0};
	StartupInfo.cb = sizeof(STARTUPINFO);
	BOOL is=CreateProcessW(NULL, (LPWSTR)wfull_path, NULL, NULL, FALSE, 0, NULL, wworking_dir, &StartupInfo, ProcessInfo);

	if (wworking_dir)
		delete []wworking_dir;

	if (is)
		return true;
		*/
	return false;
}

// Прерывание выполнения файла
bool GPlatformUnix::TerminateProcess(PROCESS_DATA* process_id)
{
    pid_t* id=(pid_t*)process_id->process_info;
    int state=kill(*id, SIGKILL);
	if (state==0)
		return true;
	return false;
}

// rus: Запуск потока на выполнение
unsigned int GPlatformUnix::ExecuteThread(unsigned int (*StartAddress)(void*), void* data)
{
	pthread_t id_thread;

	pthread_attr_t attr;

	pthread_attr_init(&attr);

	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

	int state=pthread_create(&id_thread, &attr, (void* (*)(void *))StartAddress, data);
	pthread_attr_destroy(&attr);

	//_beginthreadex(NULL, 0, (unsigned int (__stdcall *)(void *))StartAddress, data, 0, &id_thread);

	if (state)
        id_thread=0;

	return id_thread;
}

// Пауза
void GPlatformUnix::Sleep(unsigned int time)
{
	usleep(time*1000);
}

/*
// rus: Открытие ссылки в браузере
void GPlatformUnix::OpenLink(const char* link)
{
	ShellExecute(NULL, _T("open"), link, NULL, NULL, SW_MAXIMIZE);
}

// rus: Копирование строки в буфер обмена
void GPlatformUnix::SetClipboard(CMagicString& str)
{
	if (OpenClipboard(hWnd))
	{
		EmptyClipboard();

		int len=str.GetLength();

		HGLOBAL hGlobal=GlobalAlloc(GMEM_MOVEABLE, len+1);

		char* data=(char*)GlobalLock(hGlobal);
		strcpy(data, str.c_str());
		GlobalUnlock(hGlobal);
		SetClipboardData(CF_TEXT, hGlobal);

		CloseClipboard();
	}
}

// rus: Получение строки из буфера обмена
void GPlatformUnix::GetClipboard(CMagicString& str)
{
	if (OpenClipboard(hWnd))
	{
		if (IsClipboardFormatAvailable(CF_TEXT))
		{
			HGLOBAL hGlobal=GetClipboardData(CF_TEXT);
			char* data=(char*)GlobalLock(hGlobal);
			str=data;
			GlobalUnlock(hGlobal);
		}
		CloseClipboard();
	}
}
*/

// Синхронизация потоков
void GPlatformUnix::LockThread()
{
	critsect.Lock();
}

void GPlatformUnix::UnlockThread()
{
	critsect.Unlock();
}

GCriticalSection* GPlatformUnix::NewCriticalSection()
{
	return new GCriticalSectionUnix;
}

// Возвращает расширение пусковых файлов
CMagicString exe_extension="run";
const char* GPlatformUnix::GetExecutedFileExtension()
{
	return exe_extension;
}
