#pragma once
#include "Game.h"

#include <dirent.h>
#include <pthread.h>

struct POSIX_FIND_ITEM : public FINDER_DATA
{
    DIR* dir;
    CMagicString ext;

    POSIX_FIND_ITEM()
    {
        dir=NULL;
    }
};

class GCriticalSectionUnix : public GCriticalSection
{
protected:
    pthread_mutex_t mutex;

public:
	GCriticalSectionUnix();
	virtual ~GCriticalSectionUnix();

	virtual void Lock();
	virtual void Unlock();
};

class GPlatformUnix :
	public GPlatform
{
protected:
	bool is_continued;

	CMagicString file;

	GCriticalSectionUnix critsect;
public:
	GPlatformUnix(GGame* owner, const char* game_name);
	virtual ~GPlatformUnix(void);

	//  ,     
	virtual void Play(){}

	// ,       
	virtual void Quit(){}

	//   
	virtual void SetTimer(TIMER_DATA* data){}
	virtual void KillTimer(TIMER_DATA* data){}

	//   
	bool IsActivated(){return false;}
	//  ,   
	bool IsIconic(){return false;}

	//  
	void ToIconic(){}

	//    
	void ToMaximize(){}

	// eng: Returns time in milliseconds
	// rus:    
	unsigned int GetTick();

	// eng: Returns current time
	// rus:   
	virtual void GetCurrentDate(CMagicTime* date){}

	// rus:    
	virtual void OpenLink(const char* link){}

	// rus:     
	virtual void SetClipboard(CMagicString& str){}
	// rus:     
	virtual void GetClipboard(CMagicString& str){}



	// eng: Returns path to folder with game
	// rus:      
	virtual const char* GetPathToGame();

	// eng: Returns path to folder which could be used by wrapper to store temporary files
	// rus:     
	virtual const char* GetPathToTemp();

	// eng: Returns path to folder with objects
	// rus:      
	//virtual const char* GetPathToObject();

	// eng: Returns path to folder with images
	// rus:      
	//virtual const char* GetPathToImage();

	// eng: Returns path to folder with autosave
	// rus:      autosave
	//virtual const char* GetPathToAutosave();

	// eng: Finds the first file in folder
	// rus:     
	virtual const char* GetFirstFile_OS(const char* path, const char* ext);

	// eng: Finds the next file folder
	// rus:     
	virtual const char* GetNextFile_OS();

	// eng: Finds the first child folder in folder
	// rus:      
	virtual const char* GetFirstFolder_OS(const char* path);

	// eng: Finds the next child folder in folder
	// rus:      
	virtual const char* GetNextFolder_OS();

	// Deleting of file (analogue remove())
	//   ( remove())
	virtual int RemoveFile(const char* file);

	// eng: Returns file length or -1
	// rus:     -1,   
	int GetFileLength(const char* file);

	// Creating of folder
	//  
	virtual bool CreateFolder_OS(const char* folder);

	// Deleting of folder
	//  
	virtual bool RemoveFolder_OS(const char* folder);

	// eng: Opening of file
	// rus:  
	FILE* OpenFile(const char* file, const char* mode);

	// Current directory
	//  
	virtual bool SetDirectory_OS(const char* path);
	virtual const char* GetDirectory_OS();

	//   ,    
	virtual bool IsPathAbsolute_OS(CMagicString& path);

	//   
	virtual char GetSlash(){return '/';}

	//    
	virtual bool ExecuteProcess(const char* path, const char* param, const char* working_dir, PROCESS_DATA* process_id);
	//   
	virtual bool TerminateProcess(PROCESS_DATA* process_id);

	// rus:    
	unsigned int ExecuteThread(unsigned int (*StartAddress)(void*), void* data);

	// 
	virtual void Sleep(unsigned int time);

	//  
	virtual void LockThread();
	virtual void UnlockThread();

	virtual GCriticalSection* NewCriticalSection();

	//    
	const char* GetExecutedFileExtension();
};
