//-----------------------------------------------------------------------------
// File: PlatformWindows.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "PlatformWindows.h"
#include "process.h"
#include "Mmsystem.h"
#include "Shellapi.h"

#pragma comment(lib, "winmm.lib")

extern HWND hWnd;


#ifndef MAX_FPS
HANDLE hEvent=NULL;
#endif


GCriticalSectionWindows::GCriticalSectionWindows() : GCriticalSection()
{
	InitializeCriticalSection(&cr_sec);
}

GCriticalSectionWindows::~GCriticalSectionWindows()
{
	DeleteCriticalSection(&cr_sec);
}

void GCriticalSectionWindows::Lock()
{
	EnterCriticalSection(&cr_sec);
}

void GCriticalSectionWindows::Unlock()
{
	LeaveCriticalSection(&cr_sec);
}


GPlatformWindows::GPlatformWindows(GGame* owner, const char* game_name) : GPlatform(owner, game_name)
{
	start_path=GetDirectory();
	start_path+=GetSlash();

	is_continued=true;
	#ifndef MAX_FPS
	hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
	#endif

	InitializeCriticalSection(&critsect);
}

GPlatformWindows::~GPlatformWindows(void)
{
	DeleteCriticalSection(&critsect);

	#ifndef MAX_FPS
	if (hEvent)
	{
		CloseHandle(hEvent);
		hEvent=NULL;
	}
	#endif
}

//  ,     
void GPlatformWindows::Play()
{
	MSG msg;
	#ifdef MAX_FPS
	while (is_continued)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) //   hWnd,   WM_CHAR    ,   hWnd=NULL,     WM_INPUT_LANGUAGE
		//if (PeekMessage(&msg,hWnd,0,0,PM_REMOVE)) //   hWnd,   WM_CHAR    
		//if (PeekMessage(&msg,0,0,0,PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
			owner->PlatformTimer();
	}
	#else

	if (owner->IsMainTimer())
	{
		while (is_continued)
		{
			if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) //   hWnd,   WM_CHAR    ,   hWnd=NULL,     WM_INPUT_LANGUAGE
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else
			{
				WaitForSingleObject(hEvent,INFINITE);
				owner->PlatformTimer();
			}
		}
	}
	else
	{
		//  
		while (is_continued)
		{
			if (GetMessage(&msg, NULL, 0, 0)>0)
			{
				TranslateMessage(&msg); 
				DispatchMessage(&msg); 
			}
			else
				is_continued=false;
		}
	}

	#endif

	/*
	if (owner->net)
	{
		owner->net->Destroy();
		delete owner->net;
		owner->net=NULL;
	}
	*/

	DestroyWindow(hWnd);
}

// ,       
void GPlatformWindows::Quit()
{
	is_continued=false;
}

void FAR PASCAL OnTimerShot(UINT /*wTimerID*/, UINT /*msg*/, DWORD_PTR dwUser, DWORD /*dw1*/, DWORD /*dw2*/)
{
	#ifndef MAX_FPS
	SetEvent(hEvent);
	#endif
}

/*
//   
void GPlatformWindows::SetTimer(unsigned int id, unsigned int period)
{
	::SetTimer(hWnd, id, period, NULL);
}

void GPlatformWindows::KillTimer(unsigned int id)
{
	::KillTimer(hWnd, id);
}
*/

//   
void GPlatformWindows::SetTimer(TIMER_DATA* data)
{
	UINT    wTimerRes;  //  
	TIMECAPS  timecaps;  //    timeGetDevCaps

	//  max & min of  
	if (timeGetDevCaps( &timecaps, sizeof( TIMECAPS ) ) == TIMERR_NOERROR )
	{
		//   
		wTimerRes = max( timecaps.wPeriodMin, 2 );

		//      
		if( timeBeginPeriod( wTimerRes ) == TIMERR_NOERROR )
		{
			data->timer=(void*)timeSetEvent(data->period, wTimerRes, OnTimerShot, (DWORD_PTR)data, TIME_PERIODIC);
		}
	}


	//::SetTimer(hWnd, data->id, data->period, NULL);
}

void GPlatformWindows::KillTimer(TIMER_DATA* data)
{
	if (data->timer)
	{
		timeKillEvent((unsigned int)data->timer);
		data->timer=NULL;
	}
	//::KillTimer(hWnd, data->id);
}

//   
bool GPlatformWindows::IsActivated()
{
	HWND active_hwnd=GetForegroundWindow();//GetActiveWindow();
	return (active_hwnd==hWnd);
	//return true;
}

//  ,   
bool GPlatformWindows::IsIconic()
{
	if (::IsIconic(hWnd))
		return true;
	return false;
}

//  
void GPlatformWindows::ToIconic()
{
	ShowWindow(hWnd, SW_MINIMIZE);
}

//    
void GPlatformWindows::ToMaximize()
{
	//int wi, he;
	//GSurface::device->GetMonitorMode();
	//MoveWindow(hWnd, 0, 0, wi, he);
	ShowWindow(hWnd, SW_MAXIMIZE);
	/*
	if (GetActiveWindow()!=hWnd)
	{
		SetActiveWindow(hWnd);
		SetForegroundWindow(hWnd);
		SetFocus(hWnd);
	}
	*/
}

// eng: Returns time in milliseconds
// rus:    
unsigned GPlatformWindows::GetTick()
{
	return GetTickCount();
}

// eng: Returns current time
// rus:   
void GPlatformWindows::GetCurrentDate(CMagicTime* date)
{
	SYSTEMTIME st;
	GetLocalTime(&st);

	date->year=st.wYear;
	date->month=st.wMonth-1;
	date->day=st.wDay;

	date->hour=st.wHour;
	date->minute=st.wMinute;
	date->second=st.wSecond;
}

// eng: Returns path to folder with game
// rus:      
const char* GPlatformWindows::GetPathToGame()
{
	if (game_path.IsEmpty())
	{
		wchar_t path[MAX_PATH];
		GetModuleFileNameW(NULL, path, MAX_PATH);
		game_path=wchar_to_utf8(path);
		game_path=game_path.GetPathOnly();
		game_path+='\\';

		/*
		CMagicString v="C:\\Users\\Supervisor\\Desktop\\ServerOL\\";//"D:\\Games\\ \\";
		wchar_t vv[1000];
		int len=v.GetLength();
		MultiByteToWideChar(CP_ACP, 0, v, -1, vv, 999);
		vv[len]=0;
		game_path=wchar_to_utf8(vv);
		*/
	}
	return (LPCSTR)game_path;
}

// eng: Returns path to folder which could be used by wrapper to store temporary files
// rus:     
const char* GPlatformWindows::GetPathToTemp()
{
	if (temp_path.IsEmpty())
	{
		//           
		CMagicString path=GetPathToGame();
		temp_path=path;
		char slash=path[path.GetLength()-1];
		//       
		//   ,   ,        
		unsigned int tick=GetTick();
		CMagicString d;
		d.IntToString(tick);
		path+=d;

		bool use_external_temp_path=true;
		FILE_DATA fd;
		fd.Create(1);
		if (SaveFile(path, &fd))
		{
			RemoveFile(path);
			//   
			CMagicString folder_path=path+'f';
			if (CreateFolder(folder_path))
			{
				//        
				RemoveFolder(folder_path);
				use_external_temp_path=false;
			}
		}

		if (use_external_temp_path)
		{
			/*
			wchar_t path[MAX_PATH];
			GetTempPathW(MAX_PATH, path);
			temp_path=wchar_to_utf8(path);
			char slash=temp_path[temp_path.GetLength()-1];
			temp_path+=game_name;
			temp_path+=slash;
			*/

			CMagicString path;

			TCHAR szPath[MAX_PATH];
			*szPath=0;

			HKEY hKey;
			unsigned long dwSize;
			if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
			{
				dwSize=MAX_PATH;
				RegQueryValueEx(hKey, "Common AppData", NULL, NULL, (PBYTE) &szPath, &dwSize);
				RegCloseKey (hKey);
			}

			path=szPath;

			if (path.IsEmpty())
			{
				//  Windows   Application Data
				//     Temp
				GetWindowsDirectory(szPath, MAX_PATH);
				path=szPath;
				if (path[path.GetLength()-1]!='\\')
				{
					path+='\\';
				}

				path+="Application Data";
			}

			if (path[path.GetLength()-1]!='\\')
			{
				path+='\\';
			}

			path+=game_name;

			bool ok=CreatePath(path);

			path+="\\";

			temp_path=path;
		}
	}
	return (LPCSTR)temp_path;
}

// eng: Returns path to folder with user profile
// rus:       
//const char* GPlatformWindows::GetPathToProfile()
//{
//	if (profile_path.IsEmpty())
//	{
//		/*
//		TCHAR szPath[MAX_PATH];
//		*szPath=0;
//
//		HKEY hKey;
//		unsigned long dwSize;
//		if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
//		{
//			dwSize=MAX_PATH;
//			RegQueryValueEx(hKey, "Common AppData", NULL, NULL, (PBYTE) &szPath, &dwSize);
//			RegCloseKey (hKey);
//		}
//
//		profile_path=szPath;
//
//		if (profile_path.IsEmpty())
//		{
//			//  Windows   Application Data
//			//     Temp
//			GetWindowsDirectory(szPath, MAX_PATH);
//			profile_path=szPath;
//			if (profile_path[profile_path.GetLength()-1]!='\\')
//			{
//				profile_path+='\\';
//			}
//
//			profile_path+="Application Data";
//		}
//
//		if (profile_path[profile_path.GetLength()-1]!='\\')
//		{
//			profile_path+='\\';
//		}
//
//		profile_path+="Particles";
//
//		bool ok=CreatePath(profile_path);
//
//		//ASSERT(ok);
//
//		profile_path+="\\";
//		*/
//	}
//
//	return profile_path;
//}

// eng: Finds the first ptc-file in emitters folder
// rus:   ptc-    
const char* GPlatformWindows::GetFirstFile_OS(const char* path, const char* ext)
{
	CMagicString p=path;
	if (!p.IsEmpty())
		p+='\\';
	p+="*.";
	p+=ext;
	const wchar_t* mask=utf8_to_wchar((LPCSTR)p);

	bool is=false;
	WIN_FIND_ITEM* finder=new WIN_FIND_ITEM;
	finder->hFindFile=FindFirstFileW(mask,finder);
	if (finder->hFindFile!=INVALID_HANDLE_VALUE)
	{
		do
		{
			file=wchar_to_utf8(finder->cFileName);
			if (!(finder->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				is=true;
				finders.Add(finder);
				return (LPCSTR)file;
			}
		}
		while (FindNextFileW(finder->hFindFile,finder));
	}
	FindClose(finder->hFindFile);
	if (is)
		delete finder;
	return NULL;
}

// eng: Finds the next ptc-file in emitters folder
// rus:   ptc-    
const char* GPlatformWindows::GetNextFile_OS()
{
	FINDER_DATA* f=finders.Get();
	if (f && (!f->IsVirtualDrive()))
	{
		WIN_FIND_ITEM* finder=(WIN_FIND_ITEM*)f;
		while(FindNextFileW(finder->hFindFile, finder))
		{
			file=wchar_to_utf8(finder->cFileName);
			if (!(finder->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
				return (LPCSTR)file;
		}
		FindClose(finder->hFindFile);
		finders.Delete();
	}
	return NULL;
}

// eng: Finds the first child folder in folder
// rus:      
const char* GPlatformWindows::GetFirstFolder_OS(const char* path)
{
	CMagicString p=path;
	p+="\\*";
	const wchar_t* mask=utf8_to_wchar((LPCSTR)p);

	bool is=false;
	WIN_FIND_ITEM* finder=new WIN_FIND_ITEM;
	finder->dwFileAttributes=FILE_ATTRIBUTE_DIRECTORY;
	finder->hFindFile=FindFirstFileW(mask, finder);
	if (finder->hFindFile!=INVALID_HANDLE_VALUE)
	{
		do
		{
			file=wchar_to_utf8(finder->cFileName);
			if ((finder->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) && file!="." && file!="..")
			{
				is=true;
				finders.Add(finder);
				return (LPCSTR)file;
			}
		}
		while (FindNextFileW(finder->hFindFile, finder));
	}
	FindClose(finder->hFindFile);
	if (is)
		delete finder;
	return NULL;
}

// eng: Finds the next child folder in folder
// rus:      
const char* GPlatformWindows::GetNextFolder_OS()
{
	FINDER_DATA* f=finders.Get();
	if (f && (!f->IsVirtualDrive()))
	{
		WIN_FIND_ITEM* finder=(WIN_FIND_ITEM*)f;
		while(FindNextFileW(finder->hFindFile, finder))
		{
			file=wchar_to_utf8(finder->cFileName);
			if ((finder->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) && file!="." && file!="..")
				return (LPCSTR)file;
		}
		FindClose(finder->hFindFile);
		finders.Delete();
	}
	return NULL;
}

// Deleting of file (analogue remove())
//   ( remove())
int GPlatformWindows::RemoveFile(const char* file)
{
	return _wremove(utf8_to_wchar(file));
}

// eng: Returns file length or -1
// rus:     -1,   
int GPlatformWindows::GetFileLength(const char* file)
{
	CMagicString mode="rb";
	const wchar_t* pmode=utf8_to_wchar((LPCSTR)mode);
	wchar_t wmode[3];
	wmode[0]=pmode[0];
	wmode[1]=pmode[1];
	wmode[2]=0;

	const wchar_t* wfile=utf8_to_wchar(file);

	FILE* f=NULL;
	_wfopen_s(&f, wfile, wmode);

	if (f)
	{
		fseek(f,0,SEEK_END);
		long len=ftell(f);
		fclose(f);
		return (int)len;
	}
	return -1;
}

// Creating of folder
//  
bool GPlatformWindows::CreateFolder_OS(const char* folder)
{
	if (CreateDirectoryW(utf8_to_wchar(folder),NULL))
		return true;
	return false;
}

// Deleting of folder
//  
bool GPlatformWindows::RemoveFolder_OS(const char* folder)
{
	if (RemoveDirectoryW(utf8_to_wchar(folder)))
		return true;
	return false;
}

// eng: Opening of file
// rus:  
FILE* GPlatformWindows::OpenFile(const char* file, const char* mode)
{
	const wchar_t* pmode=utf8_to_wchar(mode);

	wchar_t wmode[10];
	int i=0;
	wmode[i]=pmode[i];
	while (pmode[i])
	{
		i++;
		wmode[i]=pmode[i];
	}

	const wchar_t* wfile=utf8_to_wchar(file);

	FILE* f=NULL;
	_wfopen_s(&f, wfile, wmode);

	return f;
}

// Current directory
//  
bool GPlatformWindows::SetDirectory_OS(const char* path)
{
	const wchar_t* wpath=utf8_to_wchar(path);
	if (SetCurrentDirectoryW(wpath))
		return true;
	return false;
}

const char* GPlatformWindows::GetDirectory_OS()
{
	wchar_t wcurrent_directory[MAX_DIRECTORY_PATH];
	GetCurrentDirectoryW(MAX_DIRECTORY_PATH-1, wcurrent_directory);

	current_directory=wchar_to_utf8(wcurrent_directory);

	return current_directory;
}

//   ,    
bool GPlatformWindows::IsPathAbsolute_OS(CMagicString& path)
{
	int len=path.GetLength();
	if (len>1)
	{
		if (path[1]==':')
			return true;

		if ((path[0]=='\\' || path[0]=='/') && (path[1]=='\\' || path[1]=='/'))
		{
			//  
			return true;
		}
	}
	return false;
}

//    
bool GPlatformWindows::ExecuteProcess(const char* path, const char* param, const char* working_dir, PROCESS_DATA* process_id)
{
	CMagicString full_path=path;
	full_path+=' ';
	full_path+=param;

	wchar_t* wworking_dir=NULL;
	if (working_dir)
	{
		CMagicString v=working_dir;
		int len=v.GetLength();
		if (v)
		{
			const wchar_t* vv=utf8_to_wchar(working_dir);
			wworking_dir=new wchar_t[len+1];
			for (int i=0;i<len;i++)
				wworking_dir[i]=vv[i];
			wworking_dir[len]=0;
		}
	}

	const wchar_t* wfull_path=utf8_to_wchar(full_path);

	PROCESS_INFORMATION* ProcessInfo=(PROCESS_INFORMATION*)(process_id->process_info);
	STARTUPINFOW StartupInfo = {0};
	StartupInfo.cb = sizeof(STARTUPINFO);
	BOOL is=CreateProcessW(NULL, (LPWSTR)wfull_path, NULL, NULL, FALSE, 0, NULL, wworking_dir, &StartupInfo, ProcessInfo);

	if (wworking_dir)
		delete []wworking_dir;

	if (is)
		return true;
	return false;
}

//   
bool GPlatformWindows::TerminateProcess(PROCESS_DATA* process_id)
{
	PROCESS_INFORMATION* ProcessInfo=(PROCESS_INFORMATION*)(process_id->process_info);
	if (::TerminateProcess(ProcessInfo->hProcess, NO_ERROR))
		return true;
	return false;
}

// rus:    
unsigned int GPlatformWindows::ExecuteThread(unsigned int (*StartAddress)(void*), void* data)
{
	unsigned int id_thread;
	_beginthreadex(NULL, 0, (unsigned int (__stdcall *)(void *))StartAddress, data, 0, &id_thread);
	return id_thread;
}

// 
void GPlatformWindows::Sleep(unsigned int time)
{
	::Sleep(time);
}

// rus:    
void GPlatformWindows::OpenLink(const char* link)
{
	ShellExecute(NULL, _T("open"), link, NULL, NULL, SW_MAXIMIZE);
}

// rus:     
void GPlatformWindows::SetClipboard(CMagicString& str)
{
	if (OpenClipboard(hWnd))
	{
		EmptyClipboard();

		int len=str.GetLength();

		HGLOBAL hGlobal=GlobalAlloc(GMEM_MOVEABLE, len+1);

		char* data=(char*)GlobalLock(hGlobal);
		strcpy(data, str.c_str());
		GlobalUnlock(hGlobal);
		SetClipboardData(CF_TEXT, hGlobal);

		CloseClipboard();
	}
}

// rus:     
void GPlatformWindows::GetClipboard(CMagicString& str)
{
	if (OpenClipboard(hWnd))
	{
		if (IsClipboardFormatAvailable(CF_TEXT))
		{
			HGLOBAL hGlobal=GetClipboardData(CF_TEXT); 
			char* data=(char*)GlobalLock(hGlobal);
			str=data;
			GlobalUnlock(hGlobal);
		}
		CloseClipboard();
	}
}


//  
void GPlatformWindows::LockThread()
{
	EnterCriticalSection(&critsect);
}

void GPlatformWindows::UnlockThread()
{
	LeaveCriticalSection(&critsect);
}

GCriticalSection* GPlatformWindows::NewCriticalSection()
{
	return new GCriticalSectionWindows;
}

//    
CMagicString exe_extension="run";
const char* GPlatformWindows::GetExecutedFileExtension()
{
	return exe_extension;
}

//   
void GPlatformWindows::CaptureMouse()
{
	if (hWnd)
		SetCapture(hWnd);
}

//   
void GPlatformWindows::ReleaseMouse()
{
	ReleaseCapture();
}

