//-----------------------------------------------------------------------------
// File: Surface16.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "Surface16.h"

//   b1 - b2,       a1 - a2
//  false,    
//     1   
extern bool IntersectInterval(int a1, int a2, int& b1, int& b2);

float convert_component_to_float[32]={0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0};

unsigned int convert_color16_to_color32[0x8000];

unsigned short convert_255_to_31[256];

//     ,    
GSurface16::GSurface16() : GSurface()
{
	if (!convert_component_to_float[31])
	{
		//    
		float max=31.f;
		for (int i=0;i<=31;i++)
			convert_component_to_float[i]=((float)i)/max;

		SURFACE_RGB rgb;
		for (unsigned short c=0;c<0x8000;c++)
		{
			PixelToRgb(&c, &rgb);
			convert_color16_to_color32[c]=rgb.RgbToColor32();
		}

		for (int i=0;i<256;i++)
			convert_255_to_31[i]=31*i/255;
	}
}

//    
void GSurface16::Clear(unsigned int color)
{
	int i;

	SURFACE_LOCKED* to_lock=Lock();

	short* ptr=(short*)to_lock->ptr;
	short color16;
	Color32ToColor(color, &color16);

	int len=width*height;
	short dcolor[4];
	dcolor[0]=color16;
	dcolor[1]=color16;
	dcolor[2]=color16;
	dcolor[3]=color16;
	double color64=*((double*)dcolor);
	int len4=len/4;
	double* dptr=(double*)ptr;
	for (i=0;i<len4;i++)
	{
		*dptr=color64;
		dptr++;
	}

	int ost=len%4;
	ptr+=len-ost;
	for (i=0;i<ost;i++)
	{
		*ptr=color16;
		ptr++;
	}

	Unlock();
}

//    
void GSurface16::FillRect(int x, int y, int width, int height, unsigned int color)
{
	x+=offset_x;
	y+=offset_y;

	int bx=x;
	int by=y;
	if (IntersectRect(bx, by, x, y, width, height))
	{
		int i;

		SURFACE_LOCKED* to_lock=Lock();

		short* ptr=(short*)to_lock->ptr;
		short color16;
		Color32ToColor(color, &color16);

		short dcolor[4];
		dcolor[0]=color16;
		dcolor[1]=color16;
		dcolor[2]=color16;
		dcolor[3]=color16;
		double color64=*((double*)dcolor);

		int pitch=to_lock->pitch/2;
		int len=width;
		int len4=len/4;
		int ost=len%4;

		for (int j=0;j<height;j++)
		{
			double* dptr=(double*)(ptr+x+(y+j)*pitch);
			for (i=0;i<len4;i++)
			{
				*dptr=color64;
				dptr++;
			}

			unsigned short* ptr16=(unsigned short*)dptr;
			for (i=0;i<ost;i++)
			{
				*ptr16=color16;
				ptr16++;
			}
		}
	}
}

//   
void GSurface16::DrawHorizontalLine(int x, int y, int length, unsigned int color)
{
	x+=offset_x;
	y+=offset_y;

	if (length<0)
	{
		length=-length;
		x-=length-1;
	}

	int x2=x+length-1;
	if ((y>=clip_y && y<clip_y+clip_height) && IntersectInterval(clip_x, clip_x+clip_width-1, x, x2))
	{
		int i;

		SURFACE_LOCKED* to_lock=Lock();

		short* ptr=(short*)to_lock->ptr;
		short color16;
		Color32ToColor(color, &color16);

		short dcolor[4];
		dcolor[0]=color16;
		dcolor[1]=color16;
		dcolor[2]=color16;
		dcolor[3]=color16;
		double color64=*((double*)dcolor);

		int pitch=to_lock->pitch/2;
		int len=x2-x+1;
		int len4=len/4;
		int ost=len%4;

		double* dptr=(double*)(ptr+x+y*pitch);
		for (i=0;i<len4;i++)
		{
			*dptr=color64;
			dptr++;
		}

		unsigned short* ptr16=(unsigned short*)dptr;
		for (i=0;i<ost;i++)
		{
			*ptr16=color16;
			ptr16++;
		}
	}
}

//   
void GSurface16::DrawVerticalLine(int x, int y, int length, unsigned int color)
{
	x+=offset_x;
	y+=offset_y;

	if (length<0)
	{
		length=-length;
		y-=length-1;
	}

	int y2=y+length-1;
	if ((x>=clip_x && x<clip_x+clip_width) && IntersectInterval(clip_y, clip_y+clip_height-1, y, y2))
	{
		SURFACE_LOCKED* to_lock=Lock();

		short* ptr=(short*)to_lock->ptr;
		short color16;
		Color32ToColor(color, &color16);

		int pitch=to_lock->pitch/2;
		int len=y2-y+1;

		ptr=ptr+x+y*pitch;
		for (int i=0;i<len;i++)
		{
			*ptr=color16;
			ptr+=pitch;
		}
	}
}

//  
void GSurface16::DrawLine(int x1, int y1, int x2, int y2, unsigned int color)
{
	if (x1==x2)
	{
		int len=y2-y1;
		if (len>0)
			len++;
		else
		{
			if (len<0)
				len--;
			else
				return;
		}
		return DrawVerticalLine(x1, y1, len, color);
	}
	if (y1==y2)
	{
		int len=x2-x1;
		if (len>0)
			len++;
		else
		{
			if (len<0)
				len--;
			else
				return;
		}
		return DrawHorizontalLine(x1, y1, len, color);
	}

	x1+=offset_x;
	y1+=offset_y;
	x2+=offset_x;
	y2+=offset_y;

	int cx1=x1;
	int cy1=y1;
	
	int pwi;
	int cwi=x2-x1;
	if (cwi<0)
	{
		int v=x2;
		x2=x1;
		x1=v;
		pwi=-cwi;
	}
	else
		pwi=cwi;

	int phe;
	int che=y2-y1;
	if (che<0)
	{
		int v=y2;
		y2=y1;
		y1=v;
		phe=-che;
	}
	else
		phe=che;

	int x=x1;
	int y=y1;
	int wi=x2-x1+1;
	int he=y2-y1+1;
	if (IntersectRect(x, y, x1, y1, wi, he))
	{
		//      ,    
		SURFACE_LOCKED* to_lock=Lock();
		short* ptr=(short*)to_lock->ptr;
		short color16;
		Color32ToColor(color, &color16);
		int pitch=to_lock->pitch/2;

		if (pwi>phe)
		{
			//    X
			double k=((double)che)/((double)cwi);
			for (int i=0;i<wi;i++)
			{
				int tx=x1+i;
				int ty=(int)((tx-cx1)*k)+cy1;
				ptr[tx+ty*pitch]=color16;
			}
		}
		else
		{
			//    Y
			double k=((double)cwi)/((double)che);
			for (int i=0;i<he;i++)
			{
				int ty=y1+i;
				int tx=(int)((ty-cy1)*k)+cx1;
				ptr[tx+ty*pitch]=color16;
			}
		}
	}
}


GPointList fill1;
GPointList fill2;
//   
void GSurface16::FloodFill(int x, int y, unsigned int fill_color, unsigned int obstacle_color)
{
	x+=offset_x;
	y+=offset_y;
	int clip_x2=clip_x+clip_width;
	int clip_y2=clip_y+clip_height;
	if (x>=clip_x && y>=clip_y && x<clip_x2 && y<clip_y2)
	{
		SURFACE_LOCKED* to_lock=Lock();

		unsigned short* ptr=(unsigned short*)to_lock->ptr;
		unsigned short fill_color16;
		Color32ToColor(fill_color, &fill_color16);
		unsigned short obstacle_color16;
		Color32ToColor(obstacle_color, &obstacle_color16);
		int pitch=to_lock->pitch/2;

		unsigned short* cptr=&(ptr[x+y*pitch]);
		if ((*cptr)!=obstacle_color16)
		{
			GPointList* f1=&fill1;
			GPointList* f2=&fill2;
			f1->Add(x, y);
			ptr[x+y*pitch]=fill_color16;

			static GPoint fill_offset[4]={0,-1,  0, 1, -1, 0,  1, 0};

			do 
			{
				int k_f1=f1->GetCount();
				for (int i=0;i<k_f1;i++)
				{
					GPoint p=f1->Get(i);

					for (int j=0;j<4;j++)
					{
						int vx=p.x;
						int vy=p.y;
						GPoint* ofs=&(fill_offset[j]);
						vx+=ofs->x;
						vy+=ofs->y;

						if (vx>=clip_x && vy>=clip_y && vx<clip_x2 && vy<clip_y2)
						{
							unsigned short* cptr=&(ptr[vx+vy*pitch]);
							if ((*cptr)!=obstacle_color16 && (*cptr)!=fill_color16)
							{
								*cptr=fill_color16;
								f2->Add(vx, vy);
							}
						}
					}
				}

				//    
				GPointList* v=f1;
				f1=f2;
				f2=v;
				f2->Clear();
			}
			while(f1->GetCount());
		}
	}
}

//      ,    -
void GSurface16::MaskTransparentPixel(int position)
{
	SURFACE_LOCKED* locked=Lock();
	unsigned short* ptr=(unsigned short*)locked->ptr;
	ptr[position]=ptr[position] | 0x8000;
	is_opacity=false;
}

//   RGB 
void GSurface16::GetPixelRGB(int x, int y, SURFACE_RGB* rgb)
{
	SURFACE_LOCKED* locked=Lock();
	unsigned short* ptr=(unsigned short*)locked->ptr;
	int position=x+width*y;
	unsigned short color=ptr[position];
	if (color & 0x8000)
	{
		rgb->r=0.f;
		rgb->g=0.f;
		rgb->b=0.f;
	}
	else
	{
		PixelToRgb(&color, rgb);
	}
}

//   RGB 
void GSurface16::SetPixelRGB(int x, int y, SURFACE_RGB* rgb)
{
	SURFACE_LOCKED* locked=Lock();
	unsigned short* ptr=(unsigned short*)locked->ptr;
	int position=x+width*y;
	unsigned short* color=&(ptr[position]);
	RgbToPixel(rgb, color);
}

//   Alpha 
float GSurface16::GetPixelAlpha(int x, int y)
{
	SURFACE_LOCKED* locked=Lock();
	if (locked->alpha_ptr)
	{
		//  -
		return locked->alpha_ptr[x+width*y];
	}
	else
	{
		// - 
		if (is_opacity)
		{
			//  
			return 1.f;
		}
		unsigned short* ptr=(unsigned short*)locked->ptr;
		int position=x+width*y;
		unsigned short color=ptr[position];
		if (color & 0x8000)
			return 0.f;
		return 1.f;
	}
}

//   
int GSurface16::GetBPP()
{
	return 16;
}

//      
bool GSurface16::IsOver(int x, int y)
{
	if (x>=0 && x<width && y>=0 && y<height)
	{
		if (alpha)
			return (alpha[x+y*width]>0.1f);

		SURFACE_LOCKED* locked=Lock();
		unsigned short* ptr=(unsigned short*)locked->ptr;
		int pitch=locked->pitch/2;
		ptr=ptr+x+pitch*y;

		return !((*ptr) & 0x8000);
	}

	return false;
}

/*
//     SURFACE_LOCKED
void GSurface16::BltOperation(SURFACE_LOCKED* to_lock, int x, int y, int x_from, int y_from, int width, int height, bool transparent)
{
	SURFACE_LOCKED* from_lock=Lock();

	short* to_ptr=(short*)to_lock->ptr;
	int to_pitch=to_lock->pitch/2;
	to_ptr+=x+y*to_pitch;

	short* from_ptr=(short*)from_lock->ptr;
	int from_pitch=from_lock->pitch/2;
	from_ptr+=x_from+y_from*from_pitch;

	if (from_lock->alpha_ptr)
	{
		//  -,     
		float* alpha_ptr=from_lock->alpha_ptr;
		int alpha_pitch=this->width;
		alpha_ptr+=x_from+y_from*alpha_pitch;
		SURFACE_RGB rgb;
		SURFACE_RGB rgb_color;
		for (int j=0;j<height;j++)
		{
			for (int i=0;i<width;i++)
			{
				float a=alpha_ptr[i];
				if (a)
				{
					if (a==1.f)
						to_ptr[i]=from_ptr[i];
					else
					{
						PixelToRgb(&(to_ptr[i]), &rgb);
						PixelToRgb(&(from_ptr[i]), &rgb_color);
						rgb.BlendNormal(&rgb_color, a);
						RgbToPixel(&rgb, &(to_ptr[i]));
					}
				}
			}
			to_ptr+=to_pitch;
			from_ptr+=from_pitch;
			alpha_ptr+=alpha_pitch;
		}
	}
	else
	{
		if (transparent==false)
		{
			//      
			for (int j=0;j<height;j++)
			{
				for (int i=0;i<width;i++)
				{
					to_ptr[i]=from_ptr[i];
				}
				to_ptr+=to_pitch;
				from_ptr+=from_pitch;
			}
		}
		else
		{
			//    
			for (int j=0;j<height;j++)
			{
				for (int i=0;i<width;i++)
				{
					short color=from_ptr[i];
					if (!(color & 0x8000))
						to_ptr[i]=color;
				}
				to_ptr+=to_pitch;
				from_ptr+=from_pitch;
			}
		}
	}
}
*/


/*
//      
bool GSurface16::Blt(GSurface* from, int x, int y, int x_from, int y_from, int width, int height, bool transparent)
{
	x+=offset_x;
	y+=offset_y;

	int wi,he;
	from->GetSize(wi, he);
	if (!width)
		width=wi;
	if (!height)
		height=he;

	if (from->IsOpacity())
		transparent=false;

	if (!GSurface::Blt(from, x, y, x_from, y_from, width, height, transparent))
	{
		if (IntersectRect(x, y, x_from, y_from, width, height))
		{
			is_changed=true;

			SURFACE_LOCKED* to_lock=Lock();
			//((GSurface16*)from)->BltOperation(to_lock, x, y, x_from, y_from, width, height, transparent);
			SURFACE_LOCKED* from_lock=from->Lock();

			short* to_ptr=(short*)to_lock->ptr;
			int to_pitch=to_lock->pitch/2;
			to_ptr+=x+y*to_pitch;

			short* from_ptr=(short*)from_lock->ptr;
			int from_pitch=from_lock->pitch/2;
			from_ptr+=x_from+y_from*from_pitch;

			if (from_lock->alpha_ptr)
			{
				//  -,     
				float* alpha_ptr=from_lock->alpha_ptr;
				int fwi, fhe;
				from->GetSize(fwi, fhe);
				int alpha_pitch=fwi;
				alpha_ptr+=x_from+y_from*alpha_pitch;
				SURFACE_RGB rgb;
				SURFACE_RGB rgb_color;
				for (int j=0;j<height;j++)
				{
					for (int i=0;i<width;i++)
					{
						float a=alpha_ptr[i];
						if (a)
						{
							if (a==1.f)
								to_ptr[i]=from_ptr[i];
							else
							{
								PixelToRgb(&(to_ptr[i]), &rgb);
								PixelToRgb(&(from_ptr[i]), &rgb_color);
								rgb.BlendNormal(&rgb_color, a);
								RgbToPixel(&rgb, &(to_ptr[i]));
							}
						}
					}
					to_ptr+=to_pitch;
					from_ptr+=from_pitch;
					alpha_ptr+=alpha_pitch;
				}
			}
			else
			{
				if (transparent==false)
				{
					//      
					for (int j=0;j<height;j++)
					{
						for (int i=0;i<width;i++)
						{
							to_ptr[i]=from_ptr[i];
						}
						to_ptr+=to_pitch;
						from_ptr+=from_pitch;
					}
				}
				else
				{
					//    
					for (int j=0;j<height;j++)
					{
						for (int i=0;i<width;i++)
						{
							short color=from_ptr[i];
							if (!(color & 0x8000))
								to_ptr[i]=color;
						}
						to_ptr+=to_pitch;
						from_ptr+=from_pitch;
					}
				}
			}
		}
	}
	return true;
}
*/


//     
void GSurface16::BltSymbol(SYMBOL* symbol, int x, int y, unsigned int color, unsigned int color_transparent)
{
	x+=offset_x;
	y+=offset_y;

	int sx=0;
	int sy=0;
	int swidth=symbol->width;
	int sheight=symbol->height;
	if (IntersectRect(x, y, sx, sy, swidth, sheight))
	{
		SURFACE_LOCKED* to_lock=Lock();

		is_changed=true;

		short* to_ptr=(short*)to_lock->ptr;
		int to_pitch=to_lock->pitch/2;
		to_ptr+=x+y*to_pitch;
		float** from_ptr=symbol->symbol;

		short color16;
		Color32ToColor(color, &color16);

		SURFACE_RGB rgb;

		SURFACE_RGB rgb_color;
		PixelToRgb(&color16, &rgb_color);

		if (color_transparent==COLOR_KEY_EMPTY)
		{
			//    
			for (int j=0;j<sheight;j++)
			{
				short* to_ptr2=to_ptr+j*to_pitch;
				for (int i=0;i<swidth;i++)
				{
					float alpha=from_ptr[i+sx][j+sy];
					if (alpha)
					{
						//    0 -  
						if (alpha==1.f)
						{
							//   -      
							to_ptr2[i]=color16;
						}
						else
						{
							//    
							PixelToRgb(&(to_ptr2[i]), &rgb);
							rgb.BlendNormal(&rgb_color, alpha);
							RgbToPixel(&rgb, &(to_ptr2[i]));
						}
					}
				}
			}
		}
		else
		{
			//      
			short color_transparent16;
			Color32ToColor(color_transparent, &color_transparent16);
			PixelToRgb(&color_transparent16, &rgb);

			SURFACE_RGB temp;

			for (int j=0;j<sheight;j++)
			{
				short* to_ptr2=to_ptr+j*to_pitch;
				for (int i=0;i<swidth;i++)
				{
					float alpha=from_ptr[i+sx][j+sy];
					if (alpha)
					{
						//    0 -  
						if (alpha==1.f)
						{
							//   -      
							to_ptr2[i]=color16;
						}
						else
						{
							//    
							temp=rgb;
							temp.BlendNormal(&rgb_color, alpha);
							RgbToPixel(&temp, &(to_ptr2[i]));
						}
					}
					else
					{
						to_ptr2[i]=color_transparent16;
					}
				}
			}
		}
	}
}

//    -    (     )
bool GSurface16::Copy(GSurface* from, int x, int y)
{
	//     
	//        
	Clear(0);
	SURFACE_LOCKED* from_lock=from->Lock();
	if (from->IsOpacity())
	{
		if (alpha)
		{
			delete []alpha;
			alpha=NULL;
		}
	}
	else
	{
		is_opacity=false;
		int len=this->width*this->height;
		if (from_lock->alpha_ptr)
		{
			if (!alpha)
			{
				//    -
				alpha=new float[this->width*this->height];
				for (int i=0;i<len;i++)
					alpha[i]=0;
			}
		}
		else
		{
			// - ,     
			//      
			for (int i=0;i<len;i++)
				MaskTransparentPixel(i);
		}
	}

	//         ,   

	int width, height;
	from->GetSize(width, height);

	int x_from=0;
	int y_from=0;

	if (IntersectRect(x, y, x_from, y_from, width, height))
	{
		SURFACE_LOCKED* to_lock=Lock();

		is_changed=true;

		short* to_ptr=(short*)to_lock->ptr;
		int to_pitch=to_lock->pitch/2;
		to_ptr+=x+y*to_pitch;

		short* from_ptr=(short*)from_lock->ptr;
		int from_pitch=from_lock->pitch/2;
		from_ptr+=x_from+y_from*from_pitch;

		if (from_lock->alpha_ptr)
		{
			//  -,     
			float* alpha_from_ptr=from_lock->alpha_ptr;
			float* alpha_to_ptr=to_lock->alpha_ptr;
			int alpha_from_pitch=width;
			int alpha_to_pitch=this->width;
			alpha_from_ptr+=x_from+y_from*alpha_from_pitch;
			alpha_to_ptr+=x+y*alpha_to_pitch;
			for (int j=0;j<height;j++)
			{
				for (int i=0;i<width;i++)
				{
					alpha_to_ptr[i]=alpha_from_ptr[i];
					to_ptr[i]=from_ptr[i];
				}
				to_ptr+=to_pitch;
				from_ptr+=from_pitch;
				alpha_to_ptr+=alpha_to_pitch;
				alpha_from_ptr+=alpha_from_pitch;
			}
		}
		else
		{
			for (int j=0;j<height;j++)
			{
				for (int i=0;i<width;i++)
				{
					to_ptr[i]=from_ptr[i];
				}
				to_ptr+=to_pitch;
				from_ptr+=from_pitch;
			}
		}
		return true;
	}
	return false;
}

//  
void GSurface16::CopyPixel(void* from, void* to)
{
	unsigned short* f=(unsigned short*)from;
	unsigned short* t=(unsigned short*)to;
	*t=*f;
}

//   32-        (-  )
//     to_x, to_y.     width, height    
void GSurface16::BltVideo(unsigned int* from, int to_x, int to_y, int from_x, int from_y, int width, int height, int pitch)
{
	SURFACE_LOCKED* locked=Lock();
	unsigned short* ptr=(unsigned short*)locked->ptr;
	int pitch2=locked->pitch/2;

	ptr+=to_y*pitch2+to_x;

	if (!pitch)
		pitch=width;

	from=from+from_x+from_y*pitch;
	
	for (int j=0;j<height;j++)
	{
		unsigned short* vptr=ptr;
		unsigned int* vfrom=from;

		for (int i=0;i<width;i++)
		{
			unsigned char* rgba=(unsigned char*)vfrom;
			from++;
			unsigned char r=rgba[0];
			unsigned char g=rgba[1];
			unsigned char b=rgba[2];
			unsigned short cr=convert_255_to_31[r];
			unsigned short cg=convert_255_to_31[g];
			unsigned short cb=convert_255_to_31[b];
			(*vptr)=cr<<10 | cg<<5 | cb;
			vptr++;
			vfrom++;
		}
		ptr+=pitch2;
	}

	Unlock();

	is_changed=true;
}


//       R,G,B
void GSurface16::PixelToRgb(void* color_ptr, SURFACE_RGB* rgb)
{
	unsigned short color=*((unsigned short*)color_ptr);
	color=color & 0x7fff; //    
	unsigned int color32=color;

	unsigned short float_index=color32 & 0x1f;
	rgb->b=convert_component_to_float[float_index];

	float_index=(color32>>5) & 0x1f;
	rgb->g=convert_component_to_float[float_index];

	float_index=color32>>10;
	rgb->r=convert_component_to_float[float_index];
}

//    R,G,B  
void GSurface16::RgbToPixel(SURFACE_RGB* rgb, void* color_ptr)
{
	float component=rgb->b;
	if (component>1.f)
		component=1.f;
	short color16=(short)(31.f*component+0.5f);

	component=rgb->g;
	if (component>1.f)
		component=1.f;
	color16=color16 | ((short)((31.f*component+0.5f))<<5);

	component=rgb->r;
	if (component>1.f)
		component=1.f;
	color16=color16 | ((short)((31.f*component+0.5f))<<10);

	*((short*)color_ptr)=color16;
}

//      
// ,      
void GSurface16::CopyToTexture32(void* texture_ptr, int texture_pitch)
{
	if (IsChanged())
	{
		/*
		SURFACE_LOCKED* from_lock=Lock();

		short* from_ptr=(short*)from_lock->ptr;
		int from_pitch=from_lock->pitch/2;

		texture_pitch/=4;

		for (int j=0;j<height;j++)
		{
			for (int i=0;i<width;i++)
				texture_ptr[i]=convert_color16_to_color32[from_ptr[i]];
			texture_ptr+=texture_pitch;
			from_ptr+=from_pitch;
		}
		*/
	}
}

//         
// ,      
void GSurface16::CopyToTexture(void* texture_ptr, int texture_pitch)
{
	if (IsChanged())
	{
		SURFACE_LOCKED* from_lock=Lock();

		/*
		int* from_ptr=(int*)from_lock->ptr;
		int from_pitch=from_lock->pitch/4;

		texture_pitch/=4;

		int wi2=width/2;
		for (int j=0;j<height;j++)
		{
			for (int i=0;i<wi2;i++)
				texture_ptr[i]=from_ptr[i];
			texture_ptr+=texture_pitch;
			from_ptr+=from_pitch;
		}
		*/


		/*
		double* from_ptr64=(double*)from_lock->ptr;
		int from_pitch=from_lock->pitch/8;

		texture_pitch/=8;

		double* texture_ptr64=(double*)texture_ptr;

		int wi64=width/4;
		for (int j=0;j<height;j++)
		{
			for (int i=0;i<wi64;i++)
				texture_ptr64[i]=from_ptr64[i];

			texture_ptr64+=texture_pitch;
			from_ptr64+=from_pitch;
		}
		*/

		char* from_ptr=(char*)from_lock->ptr;
		int from_pitch=from_lock->pitch;
		if (texture_pitch==width*2)
		{
			//          -   
			int len=width*height*2;
			memcpy(texture_ptr, from_ptr, len);
		}
		else
		{
			int len=width*2;
			char* texture_ptr8=(char*)texture_ptr;
			for (int j=0;j<height;j++)
			{
				memcpy(texture_ptr8, from_ptr, len);
				texture_ptr8+=texture_pitch;
				from_ptr+=from_pitch;
			}
		}
	}
}

//      
void GSurface16::BltHorizontalLine(BLT_LINE_DATA* data)
{
	unsigned short* from_ptr=(unsigned short*)data->from;
	unsigned short* to_ptr=(unsigned short*)data->to;
	int wi=data->width;
	if (data->is_opacity)
	{
		//  
		memcpy(to_ptr, from_ptr, wi*2);
	}
	else
	{
		float alpha_factor=data->alpha_factor;
		if (data->is_alpha)
		{
			float* alpha_ptr=data->alpha;
			SURFACE_RGB rgb;
			SURFACE_RGB rgb_color;
			for (int i=0;i<wi;i++)
			{
				float a=alpha_ptr[i];
				if (a)
				{
					a*=alpha_factor;
					if (a==1.f)
						to_ptr[i]=from_ptr[i];
					else
					{
						PixelToRgb(&(to_ptr[i]), &rgb);
						PixelToRgb(&(from_ptr[i]), &rgb_color);
						rgb.BlendNormal(&rgb_color, a);
						RgbToPixel(&rgb, &(to_ptr[i]));
					}
				}
			}
		}
		else
		{
			if (alpha_factor==1.f)
			{
				//    
				for (int i=0;i<wi;i++)
				{
					short color=from_ptr[i];
					if (!(color & 0x8000))
						to_ptr[i]=color;
				}
			}
			else
			{
				SURFACE_RGB rgb;
				SURFACE_RGB rgb_color;
				for (int i=0;i<wi;i++)
				{
					short color=from_ptr[i];
					if (!(color & 0x8000))
					{
						PixelToRgb(&(to_ptr[i]), &rgb);
						PixelToRgb(&color, &rgb_color);
						rgb.BlendNormal(&rgb_color, alpha_factor);
						RgbToPixel(&rgb, &(to_ptr[i]));
					}
				}
			}
		}
	}
}

//      
void GSurface16::BltHorizontalLineRev(BLT_LINE_DATA* data)
{
	unsigned short* from_ptr=(unsigned short*)data->from;
	unsigned short* to_ptr=(unsigned short*)data->to;
	int wi=data->width;
	int index=0;
	if (data->is_opacity)
	{
		//  
		from_ptr-=(wi-1);
		to_ptr-=(wi-1);
		memmove(to_ptr, from_ptr, wi*2);
	}
	else
	{
		float alpha_factor=data->alpha_factor;
		if (data->is_alpha)
		{
			float* alpha_ptr=data->alpha;
			SURFACE_RGB rgb;
			SURFACE_RGB rgb_color;
			for (int i=0;i<wi;i++)
			{
				float a=alpha_ptr[index];
				if (a)
				{
					a*=alpha_factor;
					if (a==1.f)
						to_ptr[index]=from_ptr[index];
					else
					{
						PixelToRgb(&(to_ptr[index]), &rgb);
						PixelToRgb(&(from_ptr[index]), &rgb_color);
						rgb.BlendNormal(&rgb_color, a);
						RgbToPixel(&rgb, &(to_ptr[index]));
					}
				}
				index--;
			}
		}
		else
		{
			//    
			if (alpha_factor==1.f)
			{
				for (int i=0;i<wi;i++)
				{
					short color=from_ptr[index];
					if (!(color & 0x8000))
						to_ptr[index]=color;
					index--;
				}
			}
			else
			{
				SURFACE_RGB rgb;
				SURFACE_RGB rgb_color;
				for (int i=0;i<wi;i++)
				{
					short color=from_ptr[index];
					if (!(color & 0x8000))
					{
						PixelToRgb(&(to_ptr[index]), &rgb);
						PixelToRgb(&color, &rgb_color);
						rgb.BlendNormal(&rgb_color, alpha_factor);
						RgbToPixel(&rgb, &(to_ptr[index]));
					}
					index--;
				}
			}
		}
	}
}
