//-----------------------------------------------------------------------------
// File: Theme.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "Theme.h"
#include "Game.h"

GSurface* GThemeComponent::GetBackbuffer()
{
	return owner->GetGame()->GetBackbuffer();
}

bool GThemeComponent::RenderWin(GComponent* component)
{
	unsigned int style=component->GetStyle();

	if (style & GS_CAPTION)
		owner->RenderCaption(component);

	if (style & GS_BORDER)
		owner->RenderBorder(component);

	return true;
}

bool GThemeComponent::RenderInner(GComponent* component)
{
	return true;
}

void GThemeComponent::RenderClient(GComponent* component)
{
	GSurface* backbuffer=GetBackbuffer();
	int vx, vy;
	backbuffer->GetOffset(vx, vy);
	GItem* next=component->GetFirstVisibledItem();
	while (next)
	{
		int x, y;
		next->GetItemPosition(x, y);
		backbuffer->SetOffset(vx+x, vy+y);
		component->RenderItem(next);
		next=component->GetNextVisibledItem();
	}
	backbuffer->SetOffset(vx, vy);
}

void GThemeComponent::RenderScroller(GScroller* component)
{
	owner->RenderScroller(component);
}

int GThemeComponent::GetSBSize()
{
	return owner->GetSBSize();
}

void GThemeComponent::GetInnerGap(GComponent* component, CMagicRect* rect)
{
	owner->GetInnerGap(component, rect);
}

void GThemeComponent::GetClientGap(CMagicRect* rect)
{
	rect->left=0;
	rect->top=0;
	rect->right=0;
	rect->bottom=0;
}

bool GThemeComponent::IsOver(GComponent* component, int x, int y)
{
	if ((!component->IsVisibled()) || (!component->IsEnabled()))
		return false;
	int cx, cy;
	component->GetPosition(cx, cy);
	int cwi, che;
	component->GetSize(cwi, che);

	return (x>=cx && x<cx+cwi && y>=cy && y<cy+che);
}

//    (    )
void GThemeComponent::EraseInnerRect(GComponent* component, CMagicRect* rect)
{
	owner->EraseInnerRect(component, rect);
}

//   
void GThemeComponent::EraseClient(GComponent* component)
{
	owner->EraseClient(component);
}

//   
int GThemeComponent::GetBorderSize(GComponent* component)
{
	return owner->GetBorderSize(component);
}

//      ( ,   )
int GThemeComponent::GetBorderSizeOfCaption(GComponent* component)
{
	return owner->GetBorderSizeOfCaption(component);
}

//   
unsigned int GThemeComponent::GetBorderColor(GComponent* component)
{
	return owner->GetBorderColor(component);
}

//  
void GThemeComponent::RenderBorder(GComponent* component)
{
	return owner->RenderBorder(component);
}

//   
int GThemeComponent::GetCaptionHeight(GComponent* component)
{
	return owner->GetCaptionHeight(component);
}

//  
void GThemeComponent::RenderCaption(GComponent* component)
{
	return owner->RenderCaption(component);
}

//   
void GThemeComponent::RenderBgCaption(GComponent* component, CMagicRect* rect, unsigned int color)
{
	owner->RenderBgCaption(component, rect, color);
}

//   
unsigned int GThemeComponent::GetBgColor(GComponent* component)
{
	return owner->GetBgColor(component);
}

//   
unsigned int GThemeComponent::GetTextColor(GComponent* component)
{
	return owner->GetTextColor(component);
}

//   
int GThemeComponent::GetTextFont(GComponent* component)
{
	return owner->GetTextFont(component);
}

//    
unsigned int GThemeComponent::GetSelectedTextColor(GComponent* component)
{
	return owner->GetSelectedTextColor(component);
}

//   
unsigned int GThemeComponent::GetSelectedColor(GComponent* component)
{
	return owner->GetSelectedColor(component);
}

//    
unsigned int GThemeComponent::GetCaptionBgColor(GComponent* component)
{
	return owner->GetCaptionBgColor(component);
}

//    
unsigned int GThemeComponent::GetCaptionTextColor(GComponent* component)
{
	return owner->GetCaptionTextColor(component);
}

//    
int GThemeComponent::GetCaptionTextFont(GComponent* component)
{
	return owner->GetCaptionTextFont(component);
}

//   
bool GThemeComponent::IsAlpha(GComponent* component)
{
	return owner->IsAlpha(component);
}

//  item-
void GThemeComponent::RenderItem(GItem* item)
{
	GSurface* backbuffer=GetBackbuffer();

	GListBase* component=item->GetOwner();

	CMagicRect rect;
	bool is=GetItemTextRect(item, &rect);

	unsigned int color_text;
	if (item->IsSelected() && item->IsEnabled())
	{
		//   
		color_text=GetSelectedTextColor(component);
		unsigned int color=GetSelectedColor(component);
		if (ItemHighlightFull())
		{
			CMagicRect client;
			component->GetClientRect(&client);
			int vwi, vhe;
			item->GetItemSize(vwi, vhe);
			vwi=client.Width();
			int sx, sy;
			component->GetScrollPosition(sx, sy);
			backbuffer->FillRect(sx, 0, vwi, vhe, color);
		}
		else
		{
			backbuffer->FillRect(rect.left, rect.top, rect.Width(), rect.Height(), color);
		}
	}
	else
		color_text=item->GetTextColor();

	if (is)
	{
		//  
		if (!item->IsEnabled())
		{
			unsigned int style=component->GetStyle();
			component->SetStyle(style | GS_DISABLED);
			color_text=GetTextColor(component);
			component->SetStyle(style);
		}
		backbuffer->TextOut(rect.left, rect.top, item->GetText(), item->GetTextFont(), color_text);
	}

	is=GetItemIconRect(item, &rect);
	if (is)
	{
		//  
		GImage* icon=item->GetIcon();
		if (icon)
		{
			//icon->BltAlpha(rect.left, rect.top, backbuffer, 0.4f);
			icon->Blt(rect.left, rect.top, backbuffer, true);
		}
	}

	/*
	is=GetItemNodeRect(item, &rect);
	if (is)
	{
		//  
		unsigned int color_node=GetNodeColor(component);
	}
	*/
}

//     item-
void GThemeComponent::GetItemGapRect(GItem* item, CMagicRect* rect)
{
	rect->left=0;
	rect->top=0;
	rect->right=0;
	rect->bottom=0;
}

// -------------------------------------------------

GTheme::GTheme(GGame* game, const char* name)
{
	this->game=game;
	this->name=name;

	k_font=0;
	m_font=NULL;

	k_picture=0;
	m_picture=NULL;

	k_cursor=0;
	m_cursor=NULL;

	for (int i=0;i<THEME_MAX;i++)
		components[i]=NULL;

	font_text=font_caption=1;

	alpha_surface=NULL;
	alpha_surface_color=0;
	alpha_surface_alpha=1.f;
}

GTheme::~GTheme(void)
{
	for (int i=0;i<THEME_MAX;i++)
	{
		if (components[i])
		{
			delete components[i];
			components[i]=NULL;
		}
	}

	if (m_font)
	{
		delete []m_font;
		m_font=NULL;
	}
	k_font=0;

	if (m_picture)
	{
		delete []m_picture;
		m_picture=NULL;
	}
	k_picture=0;

	if (m_cursor)
	{
		delete []m_cursor;
		m_cursor=NULL;
	}
	k_cursor=0;
}

void GTheme::Create()
{
	//        
	const char* font_name=GetTextNameFont();
	if (font_name)
		font_text=AddFont(font_name);
	font_name=GetCaptionTextNameFont();
	if (font_name)
		font_caption=AddFont(font_name);

	CreateScrollerButton(SCROLLER_ARROW_DIRECTION_LEFT);
	CreateScrollerButton(SCROLLER_ARROW_DIRECTION_TOP);
	CreateScrollerButton(SCROLLER_ARROW_DIRECTION_RIGHT);
	CreateScrollerButton(SCROLLER_ARROW_DIRECTION_BOTTOM);
}

GPlatform* GTheme::GetPlatform()
{
	return game->platform;
}

GDevice* GTheme::GetDevice()
{
	return game->device;
}

//       
const char* GTheme::GetPath()
{
	if (path.IsEmpty())
	{
		path=GetPlatform()->GetPathToGame();
		path+=name;
		path+=GetPlatform()->GetSlash();
	}
	return path.c_str();
}

//  
int GTheme::AddFont(const char* file)
{
	for (int i=0;i<k_font;i++)
	{
		THEME_FONT* tf=&(m_font[i]);
		if (tf->file==file)
			return tf->font;
	}

	//   
	CMagicString path=GetPath();
	path+=file;

	if (k_font)
	{
		THEME_FONT* vm_font=new THEME_FONT[k_font+1];

		for (int i=0;i<k_font;i++)
			vm_font[i]=m_font[i];

		delete []m_font;
		m_font=vm_font;
	}
	else
	{
		m_font=new THEME_FONT[1];
	}

	THEME_FONT* tf=&(m_font[k_font]);
	k_font++;

	tf->font=game->CreateFontex(path);
	tf->file=file;

	return tf->font;
}

//  
GImage* GTheme::AddPicture(const char* file)
{
	for (int i=0;i<k_picture;i++)
	{
		THEME_PICTURE_DATA* tp=&(m_picture[i]);
		if (tp->file==file)
			return tp->picture;
	}

	//   
	CMagicString path=GetPath();
	path+=file;

	if (k_picture)
	{
		THEME_PICTURE_DATA* vm_picture=new THEME_PICTURE_DATA[k_picture+1];

		for (int i=0;i<k_picture;i++)
			vm_picture[i]=m_picture[i];

		delete []m_picture;
		m_picture=vm_picture;
	}
	else
	{
		m_picture=new THEME_PICTURE_DATA[1];
	}

	THEME_PICTURE_DATA* tp=&(m_picture[k_picture]);
	k_picture++;

	FILE_DATA fd;
	GetPlatform()->LoadFile(path.c_str(), &fd);
	GDevice* device=GetDevice();
	GSurface* surface=device->AddSurface();
	surface->Load(fd.data, fd.length);
	tp->picture=device->AddPicture(surface);

	tp->file=file;

	return tp->picture;
}

//  
GImage* GTheme::AddCursor(const char* file)
{
	for (int i=0;i<k_cursor;i++)
	{
		THEME_PICTURE_DATA* tp=&(m_cursor[i]);
		if (tp->file==file)
			return tp->picture;
	}

	//   
	CMagicString path=GetPath();
	path+=file;

	if (k_cursor)
	{
		THEME_PICTURE_DATA* vm_cursor=new THEME_PICTURE_DATA[k_cursor+1];

		for (int i=0;i<k_cursor;i++)
			vm_cursor[i]=m_cursor[i];

		delete []m_cursor;
		m_cursor=vm_cursor;
	}
	else
	{
		m_cursor=new THEME_PICTURE_DATA[1];
	}

	THEME_PICTURE_DATA* tp=&(m_cursor[k_cursor]);
	k_cursor++;

	FILE_DATA fd;
	GetPlatform()->LoadFile(path.c_str(), &fd);
	GDevice* device=GetDevice();
	GSurface* surface=device->AddSurface();
	surface->Load(fd.data, fd.length);
	tp->picture=device->AddPicture(surface);

	tp->file=file;

	return tp->picture;
}

//    (    )
void GTheme::EraseInnerRect(GComponent* component, CMagicRect* rect)
{
	GSurface* backbuffer=game->GetBackbuffer();
	unsigned int color=component->GetThemeComponent()->GetBgColor(component);
	if (color!=COLOR_KEY_EMPTY)
		backbuffer->FillRect(rect->left, rect->top, rect->Width(), rect->Height(), color);
}

//   
void GTheme::EraseClient(GComponent* component)
{
	GSurface* backbuffer=game->GetBackbuffer();
	CMagicRect rect;
	component->GetClientRect(&rect);
	unsigned int color=component->GetThemeComponent()->GetBgColor(component);
	//if (color!=COLOR_KEY_EMPTY)
		backbuffer->FillRect(0, 0, rect.Width(), rect.Height(), color);
}

//  
void GTheme::RenderBorder(GComponent* component)
{
	GSurface* backbuffer=game->GetBackbuffer();
	//int size=border->GetBorderSize();
	GThemeComponent* theme_component=component->GetThemeComponent();
	int size=theme_component->GetBorderSize(component);
	if (size>0)
	{
		unsigned int border_color=theme_component->GetBorderColor(component);
		int wi, he;
		component->GetSize(wi, he);
		backbuffer->DrawRect(0, 0, wi, he, border_color);

		if (size>1)
		{
			//      
			unsigned int bg_color=theme_component->GetCaptionBgColor(component);

			CMagicRect gap;
			GetInnerGap(component, &gap);

			//    (    )
			int vhe=he-gap.top-gap.bottom+2;
			backbuffer->DrawVerticalLine(gap.left-1, gap.top-1, vhe, border_color);
			backbuffer->FillRect(1, gap.top-1, gap.left-2, vhe+1, bg_color);

			//    (    )
			int right=wi-gap.right;
			backbuffer->DrawVerticalLine(right, gap.top-1, vhe, border_color);
			backbuffer->FillRect(right+1, gap.top-1, gap.right-2, vhe, bg_color);

			//   
			int bottom=he-gap.bottom;//gap.top+vhe;
			backbuffer->DrawHorizontalLine(gap.left, bottom, right-gap.left, border_color);
			backbuffer->FillRect(1, bottom+1, wi-2, gap.bottom-2, bg_color);

			unsigned int style=component->GetStyle();
			if (!(style & GS_CAPTION))
			{
				//  ,      
				int bottom=gap.top+vhe;
				backbuffer->DrawHorizontalLine(gap.left+1, gap.top-1, right-(gap.left+1), border_color);
				backbuffer->FillRect(1, 1, wi-2, gap.top-2, bg_color);
			}
		}
	}
}

//  
void GTheme::RenderCaption(GComponent* component)
{
	GSurface* backbuffer=game->GetBackbuffer();
	unsigned int style=component->GetStyle();
	CMagicRect gap;
	GetInnerGap(component, &gap);
	GThemeComponent* theme_component=component->GetThemeComponent();
	unsigned int bg_color=theme_component->GetCaptionBgColor(component);
	int wi, he;
	component->GetSize(wi, he);

	CMagicRect caption_rect;

	if (style & GS_BORDER)
	{
		unsigned int border_color=theme_component->GetBorderColor(component);
		int border=theme_component->GetBorderSize(component);
		int right=wi-gap.right;
		backbuffer->DrawHorizontalLine(gap.left, gap.top-1, right-gap.left, border_color);
		int border_caption=theme_component->GetBorderSizeOfCaption(component);
		caption_rect.left=border_caption;
		caption_rect.top=border_caption;
		caption_rect.right=caption_rect.left+(wi-border_caption*2);
		caption_rect.bottom=caption_rect.top+(gap.top-border_caption-1);
	}
	else
	{
		//  
		caption_rect.left=0;
		caption_rect.top=0;
		caption_rect.right=caption_rect.left+wi;
		caption_rect.bottom=caption_rect.top+gap.top;
		//backbuffer->FillRect(0, 0, wi, gap.top, bg_color);
	}

	theme_component->RenderBgCaption(component, &caption_rect, bg_color);
	//backbuffer->FillRect(caption_rect.left, caption_rect.top, caption_rect.Width(), caption_rect.Height(), bg_color);

	//  
	const char* text=component->GetWindowText();
	GDevice* device=GetDevice();
	int twi, the;
	int font=theme_component->GetCaptionTextFont(component);
	device->GetTextSize(text, font, twi, the);

	int x=caption_rect.left+(caption_rect.Width()-twi)/2;
	int y=caption_rect.top+(caption_rect.Height()-the)/2;

	backbuffer->TextOut(x, y, text, font, theme_component->GetCaptionTextColor(component));
}


//   
void GTheme::RenderBgCaption(GComponent* component, CMagicRect* rect, unsigned int color)
{
	GSurface* backbuffer=game->GetBackbuffer();
	backbuffer->FillRect(rect->left, rect->top, rect->Width(), rect->Height(), color);
}

//   
int GTheme::GetBorderSize(GComponent* component)
{
	unsigned int style=component->GetStyle();
	if (style & GS_BORDER)
	{
		if (style & GS_CAPTION)
			return 5;
		return 1;
	}
	return 0;
}

//   
unsigned int GTheme::GetBorderColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
	{
		if (component->IsPressed())
		{
			color=component->GetBorderColorPressed();
			if (color==COLOR_KEY_EMPTY)
			{
				color=component->GetBorderColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetBorderColor();
			}
		}
		else
		{
			if (component->IsHighlighted())
			{
				color=component->GetBorderColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetBorderColor();
			}
			else
				color=component->GetBorderColor();
		}
		if (color==COLOR_KEY_EMPTY)
			color=GetBorderColorDefault();
	}
	else
	{
		color=component->GetBorderColorDisabled();
		if (color==COLOR_KEY_EMPTY)
			color=GetBorderColorDisabledDefault();
	}
	return color;
}

//   
unsigned int GTheme::GetBgColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
	{
		if (component->IsPressed())
		{
			color=component->GetBgColorPressed();
			if (color==COLOR_KEY_EMPTY)
			{
				color=component->GetBgColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetBgColor();
			}
		}
		else
		{
			if (component->IsHighlighted())
			{
				color=component->GetBgColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetBgColor();
			}
			else
				color=component->GetBgColor();
		}
		if (color==COLOR_KEY_EMPTY)
			color=GetBgColorDefault();
	}
	else
	{
		color=component->GetBgColorDisabled();
		if (color==COLOR_KEY_EMPTY)
			color=GetBgColorDisabledDefault();
	}
	return color;
}

//   
unsigned int GTheme::GetTextColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
	{
		if (component->IsPressed())
		{
			color=component->GetTextColorPressed();
			if (color==COLOR_KEY_EMPTY)
			{
				color=component->GetTextColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetTextColor();
			}
		}
		else
		{
			if (component->IsHighlighted())
			{
				color=component->GetTextColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetTextColor();
			}
			else
				color=component->GetTextColor();
		}
		if (color==COLOR_KEY_EMPTY)
			color=GetTextColorDefault();
	}
	else
	{
		color=component->GetTextColorDisabled();
		if (color==COLOR_KEY_EMPTY)
			color=GetTextColorDisabledDefault();
	}
	return color;
}

//    
unsigned int GTheme::GetCaptionBgColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
	{
		if (component->IsPressed())
		{
			color=component->GetCaptionBgColorPressed();
			if (color==COLOR_KEY_EMPTY)
			{
				color=component->GetCaptionBgColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetCaptionBgColor();
			}
		}
		else
		{
			if (component->IsHighlighted())
			{
				color=component->GetCaptionBgColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetCaptionBgColor();
			}
			else
				color=component->GetCaptionBgColor();
		}
		if (color==COLOR_KEY_EMPTY)
			color=GetCaptionBgColorDefault();
	}
	else
	{
		color=component->GetCaptionBgColorDisabled();
		if (color==COLOR_KEY_EMPTY)
			color=GetCaptionBgColorDisabledDefault();
	}
	return color;
}

//    
unsigned int GTheme::GetCaptionTextColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
	{
		if (component->IsPressed())
		{
			color=component->GetCaptionTextColorPressed();
			if (color==COLOR_KEY_EMPTY)
			{
				color=component->GetCaptionTextColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetCaptionTextColor();
			}
		}
		else
		{
			if (component->IsHighlighted())
			{
				color=component->GetCaptionTextColorHighlighted();
				if (color==COLOR_KEY_EMPTY)
					color=component->GetCaptionTextColor();
			}
			else
				color=component->GetCaptionTextColor();
		}
		if (color==COLOR_KEY_EMPTY)
			color=GetCaptionTextColorDefault();
	}
	else
	{
		color=component->GetCaptionTextColorDisabled();
		if (color==COLOR_KEY_EMPTY)
			color=GetCaptionTextColorDisabledDefault();
	}
	return color;
}


void GTheme::GetInnerGap(GComponent* component, CMagicRect* rect)
{
	unsigned int style=component->GetStyle();

	GThemeComponent* theme_component=component->GetThemeComponent();
	int border_size=theme_component->GetBorderSize(component);
	rect->left=border_size;
	rect->right=border_size;
	rect->bottom=border_size;

	if (style & GS_CAPTION)
		rect->top=theme_component->GetCaptionHeight(component);
	else
		rect->top=border_size;
}

//   
int GTheme::GetTextFont(GComponent* component)
{
	if (component)
	{
		int font=component->GetFont();
		if (font>=0)
			return font;
	}
	return font_text;
};

//   
void GTheme::FillRectAlpha(GSurface* to, int x, int y, int width, int height, unsigned int color, float alpha)
{
	if (!alpha_surface)
	{
		alpha_surface=GSurface::device->AddSurface();
		alpha_surface->Create(width, height);
		alpha_surface->Clear(color);
		alpha_surface->ChangeAlpha(0.f);
		alpha_surface_color=color;
		alpha_surface_alpha=1.f;
	}
	else
	{
		int awi, ahe;
		alpha_surface->GetSize(awi, ahe);
		if (awi<width || ahe<height)
		{
			alpha_surface->Create(width, height);
			alpha_surface->Clear(color);
			alpha_surface->ChangeAlpha(0.f);
			alpha_surface_color=color;
			alpha_surface_alpha=1.f;
		}
	}
	
	if (alpha_surface_color!=color)
	{
		alpha_surface_color=color;
		alpha_surface->Clear(color);
	}

	if (alpha_surface_alpha!=alpha)
	{
		alpha_surface_alpha=alpha;
		SURFACE_LOCKED* locked=alpha_surface->Lock();
		int awi, ahe;
		alpha_surface->GetSize(awi, ahe);
		int len=awi*ahe;
		float* alpha_ptr=locked->alpha_ptr;
		for (int i=0;i<len;i++)
			alpha_ptr[i]=alpha;
		alpha_surface->Unlock();
	}

	to->Blt(alpha_surface, x, y, 0, 0, width, height);
}

//  -   
GSurface* GTheme::CreateScrollerArrow(unsigned int color_arrow, unsigned int color_background, SCROLLER_ARROW_DIRECTION direction, int width, int height)
{
	const int size=7;
	if (!width)
		width=size;
	if (!height)
		height=size;

	GDevice* device=game->device;
	GSurface* s=device->AddSurface();
	s->Create(width, height);

	int i;
	if (color_background==COLOR_KEY_EMPTY)
	{
		int len=width*height;
		for (i=0;i<len;i++)
			s->MaskTransparentPixel(i);
	}
	else
		s->Clear(color_background);

	int offset[]={0, 1, 1, 1, -1, -1, -1};
	int line=3;

	int x=(width-size)/2;
	int y=(height-size)/2;

	if (direction==SCROLLER_ARROW_DIRECTION_TOP || direction==SCROLLER_ARROW_DIRECTION_BOTTOM)
	{
		for (i=0;i<size;i++)
		{
			y+=offset[i];
			s->DrawVerticalLine(x, y, line, color_arrow);
			x++;
		}

		if (direction==SCROLLER_ARROW_DIRECTION_TOP)
			s->FlipVertical();
	}
	else
	{
		for (i=0;i<size;i++)
		{
			x+=offset[i];
			s->DrawHorizontalLine(x, y, line, color_arrow);
			y++;
		}
		if (direction==SCROLLER_ARROW_DIRECTION_LEFT)
			s->FlipHorizontal();
	}
	return s;
}

//     
void GTheme::CreateScrollerButton(SCROLLER_ARROW_DIRECTION direction)
{
	GSurface** sb=NULL;
	switch (direction)
	{
	case SCROLLER_ARROW_DIRECTION_LEFT:
		sb=sb_left;
		break;
	case SCROLLER_ARROW_DIRECTION_TOP:
		sb=sb_top;
		break;
	case SCROLLER_ARROW_DIRECTION_RIGHT:
		sb=sb_right;
		break;
	case SCROLLER_ARROW_DIRECTION_BOTTOM:
		sb=sb_bottom;
		break;
	default:
		return;
	}

	int size=GetSBSize();

	SCROLLER_BUTTON_STATE state[4]={SCROLLER_BUTTON_STATE_NORMAL, SCROLLER_BUTTON_STATE_OVER, SCROLLER_BUTTON_STATE_PRESSED, SCROLLER_BUTTON_STATE_DISABLED};
	
	for (int i=0;i<4;i++)
	{
		unsigned int color_arrow, color_background;
		GetSBColors(state[i], color_arrow, color_background);
		sb[i]=CreateScrollerArrow(color_arrow, color_background, direction, size, size);
	}


	/*
	CMagicString file;
	
	if (hor_vert)
	{
		if (one_two)
		{
			file="sb_bottom.tga";
			sb=sb_bottom;
		}
		else
		{
			file="sb_top.tga";
			sb=sb_top;
		}
	}
	else
	{
		if (one_two)
		{
			file="sb_right.tga";
			sb=sb_right;
		}
		else
		{
			file="sb_left.tga";
			sb=sb_left;
		}
	}

	



	GImage* button=AddPicture(file.c_str());
	GDevice* device=GetDevice();
	int bsize=GetSBSize();
	unsigned int color[4]={0xf0f0f0, 0xdadada, 0x606060, 0xf5f5f5};
	//unsigned int color[4]={0xff, 0x0ff00, 0xff0000, 0xffff};
	for (int i=0;i<4;i++)
	{
		sb[i]=device->AddSurface();
		sb[i]->Create(bsize, bsize);
		sb[i]->Clear(color[i]);

		int sx, sy;
		if (hor_vert)
		{
			sb[i]->DrawVerticalLine(0, 0, bsize, 0xffffff);
			sb[i]->DrawVerticalLine(bsize-1, 0, bsize, 0xf0f0f0);
			sx=5;
			sy=6;
		}
		else
		{
			sb[i]->DrawHorizontalLine(0, 0, bsize, 0xffffff);
			sb[i]->DrawHorizontalLine(0, bsize-1, bsize, 0xf0f0f0);
			sx=5;
			sy=5;
		}

		if (i>=2)
		{
			//      
			GSurface* bsurf=button->GetSurface();
			int bwi, bhe;
			button->GetSize(bwi, bhe);
			SURFACE_RGB rgb;
			if (i==3)
			{
				float grey=0.8f;
				rgb.r=grey;
				rgb.g=grey;
				rgb.b=grey;
			}
			else
			{
				rgb.r=1.f;
				rgb.g=1.f;
				rgb.b=1.f;
			}
			for (int m=0;m<bwi;m++)
			{
				for (int n=0;n<bhe;n++)
				{
					if (bsurf->IsOver(m, n))
					{
						bsurf->SetPixelRGB(m, n, &rgb);
					}
				}
			}
		}
		button->Blt(5, 6, sb[i], true);
	}
	*/
}

//       
void GTheme::GetSBColors(SCROLLER_BUTTON_STATE state, unsigned int& color_arrow, unsigned int& color_background)
{
	switch (state)
	{
	case SCROLLER_BUTTON_STATE_NORMAL:
		color_arrow=0x606060;
		color_background=0xf0f0f0;
		break;

	case SCROLLER_BUTTON_STATE_OVER:
		color_arrow=0x606060;
		color_background=0xdadada;
		break;

	case SCROLLER_BUTTON_STATE_PRESSED:
		color_arrow=0xffffff;
		color_background=0x606060;
		break;

	case SCROLLER_BUTTON_STATE_DISABLED:
		color_arrow=0xcccccc;
		color_background=0xf5f5f5;
		break;
	}
}