//-----------------------------------------------------------------------------
// File: Theme.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once
#include "utils.h"
#include "Graphics.h"

class GTheme;
class GGame;
class GComponent;
class GScroller;
class GItem;

#define THEME_BUTTON		0
#define THEME_LISTBOX		1
#define THEME_COMBOBOX		2
#define THEME_STATIC		3
#define THEME_MENU			4
#define THEME_EDIT			5
#define THEME_TREE			6
#define THEME_PICTURE		7
#define THEME_GRID			8
#define THEME_DIALOG		9
#define THEME_CHECKBOX		10
#define THEME_SCENE			11
#define THEME_BUTTON_IMAGE	12
#define THEME_MAX			13

enum SCROLLER_ARROW_DIRECTION {SCROLLER_ARROW_DIRECTION_LEFT, SCROLLER_ARROW_DIRECTION_TOP, SCROLLER_ARROW_DIRECTION_RIGHT, SCROLLER_ARROW_DIRECTION_BOTTOM};
enum SCROLLER_BUTTON_STATE {SCROLLER_BUTTON_STATE_NORMAL, SCROLLER_BUTTON_STATE_OVER, SCROLLER_BUTTON_STATE_PRESSED, SCROLLER_BUTTON_STATE_DISABLED};

struct THEME_FONT
{
	CMagicString file;
	int font;
};

struct THEME_PICTURE_DATA
{
	CMagicString file;
	GImage* picture;
};

class GThemeComponent
{
protected:
	GTheme* owner;
public:
	GThemeComponent(GTheme* owner){this->owner=owner;}
	virtual ~GThemeComponent(){}

	GTheme* GetOwner(){return owner;}

	GSurface* GetBackbuffer();
	
	virtual bool RenderWin(GComponent* component);
	virtual bool RenderInner(GComponent* component);
	virtual void RenderClient(GComponent* component);

	//   
	virtual void RenderScroller(GScroller* component);
	//     
	virtual int GetSBSize();

	virtual void GetInnerGap(GComponent* component, CMagicRect* rect);
	virtual void GetClientGap(CMagicRect* rect);

	virtual bool IsOver(GComponent* component, int x, int y);

	virtual void GetMinSize(GComponent* component, int& wi, int& he){wi=8; he=8;}
	virtual void GetMaxSize(GComponent* component, int& wi, int& he){wi=1000; he=1000;}

	//    (    )
	virtual void EraseInnerRect(GComponent* component, CMagicRect* rect);
	//   
	virtual void EraseClient(GComponent* component);

	//   
	virtual int GetBorderSize(GComponent* component);
	//      ( ,   )
	virtual int GetBorderSizeOfCaption(GComponent* component);
	//   
	virtual unsigned int GetBorderColor(GComponent* component);

	//  
	virtual void RenderBorder(GComponent* component);

	//   
	virtual int GetCaptionHeight(GComponent* component);

	//  
	virtual void RenderCaption(GComponent* component);
	//   
	void RenderBgCaption(GComponent* component, CMagicRect* rect, unsigned int color);

	//   
	virtual unsigned int GetBgColor(GComponent* component);
	//   
	virtual unsigned int GetTextColor(GComponent* component);
	//    
	virtual unsigned int GetSelectedTextColor(GComponent* component);
	//   
	virtual unsigned int GetSelectedColor(GComponent* component);
	//   
	virtual int GetTextFont(GComponent* component);

	//    
	virtual unsigned int GetCaptionBgColor(GComponent* component);
	//    
	virtual unsigned int GetCaptionTextColor(GComponent* component);
	//    
	virtual int GetCaptionTextFont(GComponent* component);

	//   
	virtual bool IsAlpha(GComponent* component);

	//  item-
	virtual void RenderItem(GItem* item);

	//     item-
	virtual bool GetItemTextRect(GItem* item, CMagicRect* rect){return false;}

	//     item-
	virtual bool GetItemIconRect(GItem* item, CMagicRect* rect){return false;}

	//      item- (          )
	virtual bool GetItemAdditionalRect(GItem* item, CMagicRect* rect){return false;}

	//     item-
	virtual void GetItemGapRect(GItem* item, CMagicRect* rect);

	//       item-
	virtual bool ItemHighlightFull(){return true;}

	//   
	virtual int GetItemSeparatorHeight(){return 1;}

	//    (     )
	virtual int GetAdditionalGap(){return 0;}
};

class GTheme
{
protected:
	GGame* game;
	CMagicString name;
	CMagicString path;

	int k_font;
	THEME_FONT* m_font;

	int k_picture;
	THEME_PICTURE_DATA* m_picture;

	int k_cursor;
	THEME_PICTURE_DATA* m_cursor;

	GThemeComponent* components[THEME_MAX];

	int font_text, font_caption;

	GSurface* alpha_surface;
	unsigned int alpha_surface_color;
	float alpha_surface_alpha;

	GSurface* sb_left[4];
	GSurface* sb_right[4];
	GSurface* sb_top[4];
	GSurface* sb_bottom[4];

public:
	GTheme(GGame* game, const char* name);
	virtual ~GTheme(void);

	virtual void Create();

	GGame* GetGame(){return game;}

	GThemeComponent* GetThemeComponent(int index){return components[index];}

	GPlatform* GetPlatform();
	GDevice* GetDevice();

	//       
	const char* GetPath();

	//  
	int AddFont(const char* file);

	//  
	GImage* AddPicture(const char* file);

	//  
	GImage* AddCursor(const char* file);

	//    (    )
	virtual void EraseInnerRect(GComponent* component, CMagicRect* rect);
	//   
	virtual void EraseClient(GComponent* component);

	//     
	virtual int GetSBSize()=0;

	//   
	virtual void RenderScroller(GScroller* component)=0;

	virtual void GetInnerGap(GComponent* component, CMagicRect* rect);

	//   
	virtual int GetBorderSize(GComponent* component);
	//      ( ,   )
	virtual int GetBorderSizeOfCaption(GComponent* component){return 1;}
	//   
	virtual unsigned int GetBorderColor(GComponent* component);
	virtual unsigned int GetBorderColorDefault(){return 0xffffff;}
	virtual unsigned int GetBorderColorDisabledDefault(){return 0xc3c3c3;}
	
	//  
	virtual void RenderBorder(GComponent* component);

	//   
	virtual int GetCaptionHeight(GComponent* component)=0;

	//  
	virtual void RenderCaption(GComponent* component);
	//   
	virtual void RenderBgCaption(GComponent* component, CMagicRect* rect, unsigned int color);

	//   
	virtual unsigned int GetBgColor(GComponent* component);
	virtual unsigned int GetBgColorDefault(){return 0xffffff;}
	virtual unsigned int GetBgColorDisabledDefault(){return 0xefefef;}

	//   
	virtual unsigned int GetTextColor(GComponent* component);
	virtual unsigned int GetTextColorDefault(){return 0;}
	virtual unsigned int GetTextColorDisabledDefault(){return 0xafafaf;}

	//    
	virtual unsigned int GetCaptionBgColor(GComponent* component);
	virtual unsigned int GetCaptionBgColorDefault(){return 0x947955;}
	virtual unsigned int GetCaptionBgColorDisabledDefault(){return 0x838383;}

	//    
	virtual unsigned int GetCaptionTextColor(GComponent* component);
	virtual unsigned int GetCaptionTextColorDefault(){return 0;}
	virtual unsigned int GetCaptionTextColorDisabledDefault(){return 0xefefef;}

	//    
	virtual unsigned int GetSelectedTextColor(GComponent* component){return 0;}
	//   
	virtual unsigned int GetSelectedColor(GComponent* component){return 0xffffff;}
	//   
	virtual int GetTextFont(GComponent* component);
	//       
	virtual const char* GetTextNameFont()=0;

	//    
	virtual int GetCaptionTextFont(GComponent* component){return font_caption;};
	//       
	virtual const char* GetCaptionTextNameFont()=0;

	//   
	virtual bool IsAlpha(GComponent* component){return false;}

	//   
	void FillRectAlpha(GSurface* to, int x, int y, int width, int height, unsigned int color, float alpha);

	//     MessageBox
	virtual void GetButtonSizeForMessageBox(int& button_width, int& button_height){button_width=115; button_height=20;}

	//  
	virtual const char* Translate(const char* text){return text;}

	//  -   
	virtual GSurface* CreateScrollerArrow(unsigned int color_arrow, unsigned int color_background, SCROLLER_ARROW_DIRECTION direction, int width=0, int height=0);

	//     
	virtual void CreateScrollerButton(SCROLLER_ARROW_DIRECTION direction);

	//       
	virtual void GetSBColors(SCROLLER_BUTTON_STATE state, unsigned int& color_arrow, unsigned int& color_background);
};
