//-----------------------------------------------------------------------------
// File: ThemeWindows.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "StdAfx.h"
#include "ThemeWindows.h"
#include "Game.h"

unsigned int GetBorderColorGeneral(GComponent* component)
{
	unsigned int color=0xacacac;
	if (component->IsEnabled())
	{
		if (component->IsFixed())
		{
			color=0xe59d56;
		}
		else
		{
			if (component->IsFocused())
				color=0xff9933;
			else
				if (component->IsHighlighted())
					color=0xeab47e;
		}
	}
	else
		color=0xd9d9d9;

	return color;
}

//    ComboBox-   DropBox
void RenderButton(GComponent* component, bool button_cbox, GThemeComponent* theme)
{
	GSurface* backbuffer=theme->GetBackbuffer();
	unsigned int color=0xeeeeee;
	unsigned color_font=theme->GetTextColor(component);
	int smx=0;
	int smy=0;
	if (component->IsEnabled())
	{
		if (component->IsFixed())
		{
			color=0xfce1c5;
			smx=1;
			smy=1;
		}
		else
		{
			if (component->IsHighlighted())
				color=0xfcf2e8;
		}
	}
	else
	{
		color=0xefefef;
		color_font=0xafafaf;
	}
	CMagicRect rect;
	component->GetInnerRect(&rect);
	int wi=rect.Width();
	int he=rect.Height();
	backbuffer->FillRect(0, 0, wi, he, color);

	const char* text=component->GetWindowText();

	int x, y;
	int font=theme->GetTextFont(component);
	GDevice* device=theme->GetOwner()->GetDevice();
	if (button_cbox)
	{
		int the=device->GetFontex(font)->data.height;
		x=2;
		y=(he-the)/2;

		//    ,        drop
		int clip_x, clip_y, clip_wi, clip_he; 
		backbuffer->GetClip(clip_x, clip_y, clip_wi, clip_he);
		backbuffer->SetClip(clip_x, clip_y, clip_wi-he, clip_he);

		backbuffer->TextOut(x+smx, y+smy, text, theme->GetTextFont(component), color_font);

		backbuffer->SetClip(clip_x, clip_y, clip_wi, clip_he);
	}
	else
	{
		int twi, the;
		device->GetTextSize(text, font, twi, the);

		x=(wi-twi)/2;
		y=(he-the)/2;

		backbuffer->TextOut(x+smx, y+smy, text, theme->GetTextFont(component), color_font);
	}
}

GThemeWindows::GThemeWindows(GGame* game) : GTheme(game, "windows")
{
	components[THEME_BUTTON]=new GThemeButtonWindows(this);
	components[THEME_STATIC]=new GThemeStaticWindows(this);
	components[THEME_EDIT]=new GThemeEditWindows(this);
	components[THEME_LISTBOX]=new GThemeListBoxWindows(this);
	components[THEME_COMBOBOX]=new GThemeComboBoxWindows(this);
	components[THEME_MENU]=new GThemeMenuWindows(this);
	components[THEME_TREE]=new GThemeTreeWindows(this);
	components[THEME_PICTURE]=new GThemePictureWindows(this);
	components[THEME_GRID]=new GThemeGridWindows(this);
	components[THEME_DIALOG]=new GThemeDialogWindows(this);
	components[THEME_CHECKBOX]=new GThemeCheckBoxWindows(this);
	components[THEME_BUTTON_IMAGE]=new GThemeButtonImageWindows(this);
}

GThemeWindows::~GThemeWindows(void)
{
}

/*
//     
void GThemeWindows::CreateScrollButton(bool hor_vert, bool one_two)
{
	CMagicString file;
	GSurface** sb;
	if (hor_vert)
	{
		if (one_two)
		{
			file="sb_bottom.tga";
			sb=sb_bottom;
		}
		else
		{
			file="sb_top.tga";
			sb=sb_top;
		}
	}
	else
	{
		if (one_two)
		{
			file="sb_right.tga";
			sb=sb_right;
		}
		else
		{
			file="sb_left.tga";
			sb=sb_left;
		}
	}

	GImage* button=AddPicture(file.c_str());
	GDevice* device=GetDevice();
	int bsize=GetSBSize();
	unsigned int color[4]={0xf0f0f0, 0xdadada, 0x606060, 0xf5f5f5};
	//unsigned int color[4]={0xff, 0x0ff00, 0xff0000, 0xffff};
	for (int i=0;i<4;i++)
	{
		sb[i]=device->AddSurface();
		sb[i]->Create(bsize, bsize);
		sb[i]->Clear(color[i]);

		int sx, sy;
		if (hor_vert)
		{
			sb[i]->DrawVerticalLine(0, 0, bsize, 0xffffff);
			sb[i]->DrawVerticalLine(bsize-1, 0, bsize, 0xf0f0f0);
			sx=5;
			sy=6;
		}
		else
		{
			sb[i]->DrawHorizontalLine(0, 0, bsize, 0xffffff);
			sb[i]->DrawHorizontalLine(0, bsize-1, bsize, 0xf0f0f0);
			sx=5;
			sy=5;
		}

		if (i>=2)
		{
			//      
			GSurface* bsurf=button->GetSurface();
			int bwi, bhe;
			button->GetSize(bwi, bhe);
			SURFACE_RGB rgb;
			if (i==3)
			{
				float grey=0.8f;
				rgb.r=grey;
				rgb.g=grey;
				rgb.b=grey;
			}
			else
			{
				rgb.r=1.f;
				rgb.g=1.f;
				rgb.b=1.f;
			}
			for (int m=0;m<bwi;m++)
			{
				for (int n=0;n<bhe;n++)
				{
					if (bsurf->IsOver(m, n))
					{
						bsurf->SetPixelRGB(m, n, &rgb);
					}
				}
			}
		}
		button->Blt(5, 6, sb[i], true);
	}
}
*/

//   
void GThemeWindows::RenderScroller(GScroller* scroller)
{
	unsigned int color_place, color_place_border, color_slider_normal, color_slider_over, color_slider_pressed;
	GetScrollerPlaceColors(color_place, color_place_border, color_slider_normal, color_slider_over, color_slider_pressed);

	GSurface* backbuffer=game->GetBackbuffer();
	int captured_scroller=scroller->GetCapturedScroller();
	int over_scroller=scroller->GetOverScroller();
	if (scroller->IsHScrollBar())
	{
		//     

		//  1
		CMagicRect b1;
		scroller->GetHScrollButton1Rect(&b1);

		GSurface* button=sb_left[0];
		if (scroller->IsEnabled())
		{
			if (captured_scroller==SCROLLER_HBUTTON1)
				button=sb_left[2];
			else
			{
				if (over_scroller==SCROLLER_HBUTTON1)
					button=sb_left[1];
			}
		}
		else
			button=sb_left[3];

		backbuffer->Blt(button, b1.left, b1.top, 0, 0, b1.Width(), b1.Height());

		//  2
		CMagicRect b2;
		scroller->GetHScrollButton2Rect(&b2);

		if (scroller->IsEnabled())
		{
			button=sb_right[0];
			if (captured_scroller==SCROLLER_HBUTTON2)
				button=sb_right[2];
			else
			{
				if (over_scroller==SCROLLER_HBUTTON2)
					button=sb_right[1];
			}
		}
		else
			button=sb_right[3];

		backbuffer->Blt(button, b2.left, b2.top, 0, 0, b1.Width(), b1.Height());

		//  
		CMagicRect place;
		place=b1;
		place.left=place.right;
		place.right=b2.left;
		backbuffer->FillRect(place.left, place.top+1, place.Width(), place.Height()-1, color_place);
		//backbuffer->DrawHorizontalLine(place.left, place.top, place.Width(), color_place_border);
		backbuffer->DrawHorizontalLine(b1.left, place.top, b2.right-b1.left, color_place_border);

		if (scroller->IsEnabled())
		{
			// 
			CMagicRect slider;
			scroller->GetHScrollSliderRect(&slider);
			unsigned int color=color_slider_normal;
			if (captured_scroller==SCROLLER_HSLIDER)
				color=color_slider_pressed;
			else
			{
				if (over_scroller==SCROLLER_HSLIDER)
					color=color_slider_over;
			}
			backbuffer->FillRect(slider.left, slider.top+1, slider.Width(), slider.Height()-1, color);
		}
	}

	if (scroller->IsVScrollBar())
	{
		//     

		//  1
		CMagicRect b1;
		scroller->GetVScrollButton1Rect(&b1);

		GSurface* button=sb_top[0];
		if (scroller->IsEnabled())
		{
			if (captured_scroller==SCROLLER_VBUTTON1)
				button=sb_top[2];
			else
			{
				if (over_scroller==SCROLLER_VBUTTON1)
					button=sb_top[1];
			}
		}
		else
			button=sb_top[3];

		backbuffer->Blt(button, b1.left, b1.top, 0, 0, b1.Width(), b1.Height());

		//  2
		CMagicRect b2;
		scroller->GetVScrollButton2Rect(&b2);

		button=sb_bottom[0];
		if (scroller->IsEnabled())
		{
			if (captured_scroller==SCROLLER_VBUTTON2)
				button=sb_bottom[2];
			else
			{
				if (over_scroller==SCROLLER_VBUTTON2)
					button=sb_bottom[1];
			}
		}
		else
			button=sb_bottom[3];

		backbuffer->Blt(button, b2.left, b2.top, 0, 0, b1.Width(), b1.Height());

		//  
		CMagicRect place;
		place=b1;
		place.top=place.bottom;
		place.bottom=b2.top;
		backbuffer->FillRect(place.left+1, place.top, place.Width()-1, place.Height(), color_place);
		//backbuffer->DrawVerticalLine(place.left, place.top, place.Height(), color_place_border);
		backbuffer->DrawVerticalLine(place.left, b1.top, b2.bottom-b1.top, color_place_border);

		if (scroller->IsEnabled())
		{
			// 
			CMagicRect slider;
			scroller->GetVScrollSliderRect(&slider);
			unsigned int color=color_slider_normal;
			if (captured_scroller==SCROLLER_VSLIDER)
				color=color_slider_pressed;
			else
			{
				if (over_scroller==SCROLLER_VSLIDER)
					color=color_slider_over;
			}
			backbuffer->FillRect(slider.left+1, slider.top, slider.Width()-1, slider.Height(), color);
		}

		if (scroller->IsHScrollBar())
		{
			//      
			CMagicRect square;
			scroller->GetScrollSquareRect(&square);
			backbuffer->FillRect(square.left, square.top, square.Width(), square.Height(), 0xf0f0f0);
		}
	}
}

//        (  )
void GThemeWindows::GetScrollerPlaceColors(unsigned int& color_place, unsigned int& color_place_border, unsigned int& color_slider_normal, unsigned int& color_slider_over, unsigned int& color_slider_pressed)
{
	color_place=0xf0f0f0;
	color_place_border=0xffffff;
	color_slider_normal=0xcdcdcd;
	color_slider_over=0xa6a6a6;
	color_slider_pressed=0x606060;
}

/*
//   
unsigned int GThemeWindows::GetBorderColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
		color=0;
	else
		color=0xd9d9d9;

	return color;
}
*/

//   
int GThemeWindows::GetCaptionHeight(GComponent* component)
{
	return 30;
}

/*
//   
unsigned int GThemeWindows::GetBgColor(GComponent* component)
{
	if (component->IsEnabled())
		return 0xffffff;
	return 0xefefef;
}

//   
unsigned int GThemeWindows::GetTextColor(GComponent* component)
{
	if (component->IsEnabled())
		return 0;
	return 0x838383;
}
*/

// --------------------------------------------------------------------

GThemeButtonWindows::GThemeButtonWindows(GTheme* owner) : GThemeComponent(owner)
{
}

GThemeButtonWindows::~GThemeButtonWindows()
{
}

//   
unsigned int GThemeButtonWindows::GetBorderColor(GComponent* component)
{
	return GetBorderColorGeneral(component);
}

void GThemeButtonWindows::RenderClient(GComponent* component)
{
	GSurface* backbuffer=GetBackbuffer();
	RenderButton(component, false, this);
}

void GThemeButtonWindows::GetClientGap(CMagicRect* rect)
{
	rect->left=0;
	rect->top=0;
	rect->right=0;
	rect->bottom=0;
}

// ---------------------------------------------------------------

GThemeButtonImageWindows::GThemeButtonImageWindows(GTheme* owner) : GThemeComponent(owner)
{
}

GThemeButtonImageWindows::~GThemeButtonImageWindows()
{
}

void GThemeButtonImageWindows::RenderClient(GComponent* component)
{
	GButtonImage* button=(GButtonImage*)component;
	GImageList* list=button->GetImageList();
	GSurface* backbuffer=GetBackbuffer();
	int index=0;
	if (component->IsEnabled())
	{
		if (component->IsFixed())
		{
			index=1;
		}
		else
		{
			if (component->IsHighlighted())
				index=2;
		}
	}
	else
	{
		index=3;
	}

	GSurface* surface=list->GetImage(index)->GetSurface();

	backbuffer->Blt(surface);
}

void GThemeButtonImageWindows::GetClientGap(CMagicRect* rect)
{
	rect->left=0;
	rect->top=0;
	rect->right=0;
	rect->bottom=0;
}

//   
unsigned int GThemeButtonImageWindows::GetBorderColor(GComponent* component)
{
	return GetBorderColorGeneral(component);
}

// ---------------------------------------------------------------

GThemeStaticWindows::GThemeStaticWindows(GTheme* owner) : GThemeComponent(owner)
{
}

GThemeStaticWindows::~GThemeStaticWindows()
{
}

void GThemeStaticWindows::RenderClient(GComponent* component)
{
	GSurface* backbuffer=GetBackbuffer();

	GStatic* st=(GStatic*)component;
	int sx, sy;
	st->GetScrollPosition(sx, sy);
	
	int k_text=st->GetStringCount();

	bool enabled=component->IsEnabled();

	for (int i=0;i<k_text;i++)
	{
		STATIC_TEXT* s=st->GetString(i);

		unsigned int color=s->color;
		if (!enabled)
			color=GetTextColor(component);//0x838383;

		int x=s->x-sx;
		int y=s->y-sy;
		backbuffer->TextOut(x, y, s->text.c_str(), s->font, color);

		if (enabled && (!s->link.IsEmpty()) && st->GetCurrentLink()!=i)
		{
			//  ,   
			int  twi, the;
			backbuffer->device->GetTextSize(s->text.c_str(), s->font, twi, the);
			backbuffer->DrawHorizontalLine(x, y+the-1, twi, color);
		}
	}
}

void GThemeStaticWindows::GetClientGap(CMagicRect* rect)
{
	int gap=3;
	//rect->left=gap;
	//rect->top=gap;
	//rect->right=0;
	//rect->bottom=0;

	rect->left=gap;
	rect->top=1;
	rect->right=gap;
	rect->bottom=gap;
}

void GThemeStaticWindows::GetMinSize(GComponent* component, int& wi, int& he)
{
	wi=he=4;
}

//   
unsigned int GThemeStaticWindows::GetBgColor(GComponent* component)
{
	bool is_nocolor=false;
	unsigned int color;
	if (component->IsEnabled())
	{
		color=component->GetBgColor();
		if (color==COLOR_KEY_EMPTY)
			is_nocolor=true;
	}
	else
	{
		color=component->GetBgColorDisabled();
		if (color==COLOR_KEY_EMPTY)
			is_nocolor=true;
	}

	if (is_nocolor)
	{
		GComponent* parent=component->GetParent();
		if (parent)
			color=parent->GetThemeComponent()->GetBgColor(parent);
		else
			color=0xffffff;
	}
	return color;
}

// ---------------------------------------------------------------

GThemeEditWindows::GThemeEditWindows(GTheme* owner) : GThemeStaticWindows(owner)
{
}

GThemeEditWindows::~GThemeEditWindows()
{
}

//   
unsigned int GThemeEditWindows::GetBorderColor(GComponent* component)
{
	//return GetBorderColorGeneral(component);

	unsigned int color;
	if (component->IsFocused())
		color=0xaa843e;
	else
	{
		 if (component->IsHighlighted())
			 color=0xcaa46e;
		 else
			 color=component->GetThemeComponent()->GThemeComponent::GetBorderColor(component);
	}

	return color;
}

//   
unsigned int GThemeEditWindows::GetBgColor(GComponent* component)
{
	return owner->GetBgColor(component);
}

void GThemeEditWindows::RenderClient(GComponent* component)
{
	GEdit* edit=(GEdit*)component;
	int text_x=edit->GetTextX();
	int font=GetTextFont(edit);
	GDevice* device=owner->GetDevice();
	unsigned int color_text=GetTextColor(edit);

	GSurface* backbuffer=GetBackbuffer();

	int sel1, sel2;
	bool is_sel=edit->GetSelPos(sel1, sel2);
	if (is_sel)
	{
		//    
		char* str=(char*)edit->GetWindowText();
		char s;
		int vx, vy;
		int sx=-text_x;
		if (sel1>0)
		{
			s=str[sel1];
			str[sel1]=0;
			backbuffer->TextOut(sx, 0, str, font, color_text);
			device->GetTextSize(str, font, vx, vy);
			sx+=vx;
			str[sel1]=s;
		}

		//   
		int sel22=sel2+1;
		s=str[sel22];
		str[sel22]=0;
		//unsigned int sel_color=GetBorderColor(component);
		unsigned int sel_color=GetSelectedColor(edit);
		backbuffer->TextOut(sx, 0, &(str[sel1]), font, GetSelectedTextColor(edit), sel_color);
		device->GetTextSize(&(str[sel1]), font, vx, vy);
		sx+=vx;
		str[sel22]=s;

		int len=edit->GetWindowTextLength();
		if (sel2<len-1)
		{
			backbuffer->TextOut(sx, 0, &(str[sel22]), font, color_text);
		}
	}
	else
	{
		//   
		backbuffer->TextOut(-text_x, 0, edit->GetWindowText(), font, color_text);
	}

	if (edit->IsCursorVisibled())
	{
		//   
		int x_cur=edit->GetCursorX();

		//int he=device->GetFontex(font)->data.height;
		CMagicRect rect;
		edit->GetClientRect(&rect);
		int he=rect.Height();

		unsigned int color_cur;
		if (is_sel)
			color_cur=0x66cc;
		else
			color_cur=color_text;
		backbuffer->DrawVerticalLine(x_cur, 0, he, color_cur);
	}
}

void GThemeEditWindows::GetClientGap(CMagicRect* rect)
{
	int gap=2;
	rect->left=gap;
	rect->top=gap;
	rect->right=gap;
	rect->bottom=gap;
}

void GThemeEditWindows::GetMinSize(GComponent* component, int& wi, int& he)
{
	wi=8;
	int font=GetTextFont(component);
	GDevice* device=owner->GetDevice();
	GFontex* fontex=device->GetFontex(font);
	he=fontex->data.height;

	CMagicRect rect;
	GetClientGap(&rect);
	he+=rect.top+rect.bottom+1;

	unsigned int style=component->GetStyle();
	if (style & GS_BORDER)
	{
		he+=GetBorderSize(component)*2;
	}
}

// ---------------------------------------------------------------

GThemeListBoxWindows::GThemeListBoxWindows(GTheme* owner) : GThemeStaticWindows(owner)
{
}

GThemeListBoxWindows::~GThemeListBoxWindows()
{
}

//   
unsigned int GThemeListBoxWindows::GetBgColor(GComponent* component)
{
	return owner->GetBgColor(component);
}

//     item-
bool GThemeListBoxWindows::GetItemTextRect(GItem* item, CMagicRect* rect)
{
	CMagicRect gap;
	GetItemGapRect(item, &gap);

	GListBox* listbox=(GListBox*)item->GetOwner();
	GImageList* icons=listbox->GetIcons();
	
	int icon_offset_x=gap.left;
	//GImage* icon=item->GetIcon();
	//if (icon)
	if (icons)
	{
		int iwi, ihe;
		icons->GetSize(iwi, ihe);
		icon_offset_x+=ihe;
		
		icon_offset_x+=3; //      1 
	}

	int twi, the;
	item->GetTextSize(twi, the);

	int wi, he;
	item->GetItemSize(wi, he);

	rect->left=icon_offset_x;
	rect->right=rect->left+twi;
	rect->top=(he-the)/2+gap.top;
	rect->bottom=rect->top+the;

	return true;
}

//     item-
bool GThemeListBoxWindows::GetItemIconRect(GItem* item, CMagicRect* rect)
{
	GImageList* il=item->GetOwner()->GetIcons();
	if (il)
	{
		//GImage* icon=item->GetIcon();
		CMagicRect gap;
		GetItemGapRect(item, &gap);

		int wi, he;
		item->GetItemSize(wi, he);
		int iwi, ihe;
		//icon->GetSize(iwi, ihe);
		il->GetSize(iwi, ihe);
		rect->left=gap.left;
		rect->right=rect->left+iwi;
		rect->top=(he-ihe)/2;
		rect->bottom=rect->top+ihe;

		return true;
	}
	return false;
}

//     item-
void GThemeListBoxWindows::GetItemGapRect(GItem* item, CMagicRect* rect)
{
	rect->left=2;
	rect->top=0;
	rect->right=0;
	rect->bottom=2;
}

/*
//   
unsigned int GThemeListBoxWindows::GetBorderColor(GComponent* component)
{
	if (component->IsModal())
		return GetBorderColorGeneral(component);
	else
		return GThemeStaticWindows::GetBorderColor(component);
}
*/

// ---------------------------------------------------------------

GThemeComboBoxWindows::GThemeComboBoxWindows(GTheme* owner) : GThemeEditWindows(owner)
{
	//drop=owner->AddPicture("drop.tga");
	drop=owner->CreateScrollerArrow(owner->GetBorderColorDefault(), COLOR_KEY_EMPTY, SCROLLER_ARROW_DIRECTION_BOTTOM);
	drop_disabled=owner->CreateScrollerArrow(owner->GetBorderColorDisabledDefault(), COLOR_KEY_EMPTY, SCROLLER_ARROW_DIRECTION_BOTTOM);
}

GThemeComboBoxWindows::~GThemeComboBoxWindows()
{
}

bool GThemeComboBoxWindows::RenderInner(GComponent* component)
{
	RenderButton(component, true, this);

	GSurface* backbuffer=GetBackbuffer();

	CMagicRect rect;
	GetItemAdditionalRect((GItem*) component, &rect);

	//cbox->GetButtonRect(&rect);

	int wi, he;
	drop->GetSize(wi, he);

	int x=rect.left+(rect.Width()-wi)/2;
	int y=rect.top+(rect.Height()-he)/2;

	//drop->Blt(x, y, backbuffer, true);
	backbuffer->Blt(drop, x, y);

	GComboBox* cbox=(GComboBox*)component;
	return cbox->IsEditable();
}

//  ,   
//  item-     ComboBox-
bool GThemeComboBoxWindows::GetItemAdditionalRect(GItem* item, CMagicRect* rect)
{
	GComponent* component=(GComponent*)item;
	component->GetInnerRect(rect);
	int he=rect->Height();
	rect->left=rect->right-he;
	return true;
}

//   
unsigned int GThemeComboBoxWindows::GetBorderColor(GComponent* component)
{
	unsigned int style=component->GetStyle();
	if (style & GSCB_DROPDOWN)
		return GThemeComponent::GetBorderColor(component);

	return GThemeEditWindows::GetBorderColor(component);
}

// ---------------------------------------------------------------

GThemeMenuWindows::GThemeMenuWindows(GTheme* owner) : GThemeListBoxWindows(owner)
{
	//has_children=owner->AddPicture("has_children.tga");
	has_children=NULL;
}

GThemeMenuWindows::~GThemeMenuWindows()
{
}

void GThemeMenuWindows::GetClientGap(CMagicRect* rect)
{
	int gap=2;
	rect->left=gap;
	rect->top=gap;
	rect->right=gap;
	rect->bottom=gap;
}

//     item-
void GThemeMenuWindows::GetItemGapRect(GItem* item, CMagicRect* rect)
{
	rect->left=0;
	rect->top=2;
	rect->right=0;
	rect->bottom=2;
}

//     item-
bool GThemeMenuWindows::GetItemTextRect(GItem* item, CMagicRect* rect)
{
	bool is=GThemeListBoxWindows::GetItemTextRect(item, rect);
	int wi=rect->Width();
	rect->left+=GetIconFieldWidth(item)+GetWidthBeforeText();
	rect->right=rect->left+wi;
	return is;
}

//      item- (          )
bool GThemeMenuWindows::GetItemAdditionalRect(GItem* item, CMagicRect* rect)
{
	if (item->HasChildren())
	{
		int wi, he;
		item->GetItemSize(wi, he);
		//rect->left=wi-he;
		rect->left=wi-16;
		rect->top=0;
		rect->right=wi;
		rect->bottom=he;
		return true;
	}
	rect->left=0;
	rect->top=0;
	rect->right=0;
	rect->bottom=0;
	return false;
}

/*
//   
unsigned int GThemeMenuWindows::GetBorderColor(GComponent* component)
{
	unsigned int color;
	if (component->IsEnabled())
		color=0xa0a0a0;
	else
		color=0xd9d9d9;
	return color;
}
*/

void GThemeMenuWindows::RenderItem(GItem* item)
{
	GSurface* backbuffer=GetBackbuffer();
	CMagicRect rect;

	//unsigned int line_color1=0xb4a0a0;
	//unsigned int line_color2=0xffffff;
	unsigned int line_color1=GetBorderColor(item->GetOwner());
	SURFACE_RGB rgb;
	rgb.Color32ToRgb(line_color1);
	rgb.Mul(1.2f);
	unsigned int line_color2=rgb.RgbToColor32();

	//  ,    
	int x;

	int icon_field_width=GetIconFieldWidth(item);
	
	int separator_height=GetItemSeparatorHeight();
	CMagicRect gap;
	GetItemGapRect(item, &gap);
	bool is_separator=item->GetHeight()==separator_height+gap.top+gap.bottom;
	if (is_separator)
	{
		//  
		x=icon_field_width+GetWidthBeforeText();;
		item->GetOwner()->GetClientRect(&rect);
		int len=rect.Width()-x;
		backbuffer->DrawHorizontalLine(x, gap.top, len, line_color1);
		backbuffer->DrawHorizontalLine(x, gap.top+1, len, line_color2);
	}
	else
	{
		GThemeComponent::RenderItem(item);
		
		if (GetItemAdditionalRect(item, &rect))
		{
			int wi, he;
			if (!has_children)
			{
				has_children=owner->GetDevice()->AddSurface();
				wi=4;
				he=7;
				has_children->Create(wi, he);
				int len=wi*he;
				for (int i=0;i<len;i++)
					has_children->MaskTransparentPixel(i);

				unsigned int color=owner->GetBorderColorDefault();

				for (int i=0;i<wi;i++)
				{
					int length=he-i*2;
					has_children->DrawVerticalLine(i, i, length, color);
				}
			}

			has_children->GetSize(wi, he);

			int x=rect.left+(rect.Width()-wi)/2;
			int y=rect.top+(rect.Height()-he)/2;

			backbuffer->Blt(has_children, x, y);
			//has_children->Blt(x, y, backbuffer, true);
		}
	}

	x=icon_field_width;
	int item_he=item->GetHeight();
	backbuffer->DrawVerticalLine(x-2, 0, item_he, line_color1);
	backbuffer->DrawVerticalLine(x-1, 0, item_he, line_color2);

	if (item->IsChecked() && (!is_separator))
	{
		unsigned int border_color=GetBorderColor(item->GetOwner());
		int check_size=icon_field_width/4;
		int max_he=item_he-4;
		if (max_he>0)
		{
			x=(icon_field_width-check_size)/2-1;
			int y=(item_he-check_size)/2+1;
			backbuffer->FillRect(x, y, check_size, check_size, border_color);
		}
	}
}

//     
int GThemeMenuWindows::GetIconFieldWidth(GItem* item)
{
	int width=24;
	GImageList* il=item->GetOwner()->GetIcons();
	if (il)
	{
		int wi, he;
		il->GetSize(wi, he);
		if (width<wi+4)
			width=wi+4;
	}
	return width;
}

// ---------------------------------------------------------------

GThemeTreeWindows::GThemeTreeWindows(GTheme* owner) : GThemeListBoxWindows(owner)
{
}

GThemeTreeWindows::~GThemeTreeWindows()
{
}

//     item-
bool GThemeTreeWindows::GetItemTextRect(GItem* item, CMagicRect* rect)
{
	bool is=GThemeListBoxWindows::GetItemTextRect(item, rect);
	if (is)
	{
		CMagicRect irect;
		if (GetItemIconRect(item, &irect))
		{
			int twi=rect->Width();
			rect->left=irect.right+3;
			rect->right=rect->left+twi;
		}
		else
		{
			int wi=rect->Width();

			CMagicRect node;
			GetItemAdditionalRect(item, &node);

			int distance=GetDistanceFromNodeToItem(item->GetOwner());
			rect->left=node.right+distance;
			rect->right=rect->left+wi;
		}
	}
	return is;
}

//     item-
bool GThemeTreeWindows::GetItemIconRect(GItem* item, CMagicRect* rect)
{
	bool is=GThemeListBoxWindows::GetItemIconRect(item, rect);
	if (is)
	{
		int wi=rect->Width();

		CMagicRect node;
		GetItemAdditionalRect(item, &node);

		int distance=GetDistanceFromNodeToItem(item->GetOwner());
		rect->left=node.right+distance;
		rect->right=rect->left+wi;
	}
	return is;
}

//      item- (          )
bool GThemeTreeWindows::GetItemAdditionalRect(GItem* item, CMagicRect* rect)
{
	//CCTree* tree=(CCTree*)item->GetOwner();

	int level=item->GetChildLevel();
	
	int x=GetNodeX(item, level);

	int half=GetNodeHalf();
	int size=half*2+1;
	x-=half;

	int he=item->GetHeight();
	int y=he/2;
	y-=half;
	rect->left=x;
	rect->top=y;
	
	rect->right=rect->left+size;
	rect->bottom=rect->top+size;

	return true;
}

//  X-  
int GThemeTreeWindows::GetNodeX(GItem* item, int level)
{
	GTree* tree=(GTree*)item->GetOwner();

	int distance_level=GetLevelDistance(tree);

	CMagicRect gap;
	GetItemGapRect(item, &gap);
	int x=gap.left;

	/*
	if (!tree->HasButtons())
	{
		//        ,      
		// ..      +   
		int distance=GetDistanceFromNodeToItem(tree);
		x-=distance;
	}
	*/

	x+=level*distance_level;

	int half=GetNodeHalf();
	x+=half;

	return x;
}


//  item-
bool level_has_sibling[100];
void GThemeTreeWindows::RenderItem(GItem* item)
{
	GSurface* backbuffer=GetBackbuffer();

	//    
	GThemeComponent::RenderItem(item);

	//       
	GTree* tree=(GTree*)item->GetOwner();

	CMagicRect rect;
	GetItemAdditionalRect(item, &rect);

	unsigned int node_color=0x919191;//GetTextColor(tree);
	int level=item->GetChildLevel();
	int height=item->GetHeight();

	int node_wi=rect.Width();
	int node_he=rect.Height();
	int node_wi2=node_wi/2;
	int node_he2=node_he/2;

	if (tree->HasButtons())
	{
		//   
		if (item->HasChildren())
		{
			//   item-
			backbuffer->DrawRect(rect.left, rect.top, node_wi, node_he, node_color);
			backbuffer->DrawHorizontalLine(rect.left+2, rect.top+node_he2, node_wi-4, node_color);
			if (!item->IsOpened())
				backbuffer->DrawVerticalLine(rect.left+node_wi2, rect.top+2, node_he-4, node_color);

			if (rect.top>0 && tree->HasLines())
				backbuffer->DrawVerticalLine(rect.left+node_wi2, rect.top-2, -(rect.top-1), node_color);

			if (item->GetNextSibling())
			{
				level_has_sibling[level]=true;
				if (rect.bottom<height-1 && tree->HasLines())
					backbuffer->DrawVerticalLine(rect.left+node_wi2, rect.bottom+1, height-(rect.bottom+1), node_color);
			}
			else
				level_has_sibling[level]=false;

			if (tree->HasLines())
				backbuffer->DrawHorizontalLine(rect.right+1, rect.top+node_he2, GetDistanceFromNodeToItem(tree)-3, node_color);
		}
		else
		{
			if (tree->HasLines())
			{
				int nx=rect.left+rect.Width()/2;
				int ny=rect.top+rect.Height()/2;
				int dist=GetDistanceFromNodeToItem(tree);
				int x1, x2;
				GItem* next=item->GetNext();
				if ((!next) || next->GetParent()!=item->GetParent())
				{
					x1=nx+1;
					x2=dist+3;
				}
				else
				{
					x1=nx+3;
					x2=dist+1;
				}

				backbuffer->DrawHorizontalLine(x1, ny, x2, node_color);

				int he=height;
				if (item->GetNextSibling())
					level_has_sibling[level]=true;
				else
				{
					level_has_sibling[level]=false;
					he=ny+1;
				}
				backbuffer->DrawVerticalLine(nx, 0, he, node_color);
			}
		}
	}
	else
	{
		if (tree->HasLines())
		{
			//backbuffer->DrawHorizontalLine(rect.left, rect.top+rect.Height()/2, node_color);
			//backbuffer->DrawHorizontalLine(rect.left, rect.top+rect.Height()/2, node_color);
		}
	}

	if (tree->HasLines())
	{
		int nx, ny;
		ny=rect.top+node_he2;
		//for (int i=level-1;i>0;i--)
		//for (int i=level;i>0;i--)
		for (int i=level-1;i>=0;i--)
		{
			if (level_has_sibling[i])
			{
				//nx=GetNodeX(item, i-1);
				nx=GetNodeX(item, i);
				backbuffer->DrawVerticalLine(nx, 0, height, node_color);
			}
		}
	}


	/*



	CMagicRect rect;

	unsigned int line_color1=0xb4a0a0;
	unsigned int line_color2=0xffffff;

	//  ,    
	int x;

	int icon_field_width=GetIconFieldWidth(item);

	int separator_height=GetItemSeparatorHeight();
	CMagicRect gap;
	GetItemGapRect(item, &gap);
	if (item->GetHeight()==separator_height+gap.top+gap.bottom)
	{
		//  
		x=icon_field_width+GetWidthBeforeText();;
		item->GetOwner()->GetClientRect(&rect);
		int len=rect.Width()-x;
		backbuffer->DrawHorizontalLine(x, gap.top, len, line_color1);
		backbuffer->DrawHorizontalLine(x, gap.top+1, len, line_color2);
	}
	else
	{
		GThemeComponent::RenderItem(item);

		if (GetItemAdditionalRect(item, &rect))
		{
			int wi, he;
			has_children->GetSize(wi, he);

			int x=rect.left+(rect.Width()-wi)/2;
			int y=rect.top+(rect.Height()-he)/2;

			has_children->Blt(x, y, backbuffer, true);
		}
	}

	x=icon_field_width;
	int item_he=item->GetHeight();
	backbuffer->DrawVerticalLine(x-2, 0, item_he, line_color1);
	backbuffer->DrawVerticalLine(x-1, 0, item_he, line_color2);
	*/
}

//   
int GThemeTreeWindows::GetLevelDistance(GComponent* component)
{
	/*
	int wi=0;
	GTree* tree=(GTree*)component;
	if (tree->HasLines())
	{
		GImageList* il=component->GetIcons();
		if (il)
		{
			int he;
			il->GetSize(wi, he);
		}
		//return 24;
	}
	return wi+3;
	//return 16;
	*/
	return 24;
}

//    X        item-
int GThemeTreeWindows::GetDistanceFromNodeToItem(GComponent* component)
{
	return 6;
	/*
	int distance=GetLevelDistance(component);
	GImageList* il=component->GetIcons();
	if (il)
	{
		int wi, he;
		il->GetSize(wi, he);
		distance-=wi/2;
	}
	return distance;
	*/
}

// ---------------------------------------------------------------

GThemePictureWindows::GThemePictureWindows(GTheme* owner) : GThemeComponent(owner)
{
}

void GThemePictureWindows::RenderClient(GComponent* component)
{
	GPicture* pict=(GPicture*)component;
	GSurface* surface=pict->GetImage();
	if (surface)
		GetBackbuffer()->Blt(surface);
}

// ---------------------------------------------------------------

GThemeGridWindows::GThemeGridWindows(GTheme* owner) : GThemeListBoxWindows(owner)
{
}

//  
void GThemeGridWindows::RenderCaption(GComponent* component)
{
	GThemeListBoxWindows::RenderCaption(component);

	//    
	GGrid* grid=(GGrid*)component;

	/*
	CMagicRect gap;
	GetInnerGap(component, &gap);
	GThemeComponent* theme_component=component->GetThemeComponent();
	unsigned int bg_color=theme_component->GetCaptionBgColor(component);
	int wi, he;
	component->GetSize(wi, he);

	CMagicRect caption_rect;
	int style=component->GetStyle();
	if (style & GS_BORDER)
	{
		caption_rect.left=1;
		caption_rect.top=1;
		caption_rect.right=wi-2;
		caption_rect.bottom=gap.top-2;
	}
	else
	{
		//  
		caption_rect.left=0;
		caption_rect.top=0;
		caption_rect.right=wi;
		caption_rect.bottom=gap.top;
	}
	*/

	GSurface* backbuffer=GetBackbuffer();
	GDevice* device=GetOwner()->GetDevice();
	CMagicRect rect;
	int font=GetTextFont(component);
	unsigned int color=GetTextColor(component);
	int k_column=grid->GetColumnCount();
	for (int i=0;i<k_column;i++)
	{
		COLUMN_DATA* c=grid->GetColumn(i);
		int swi, she;
		device->GetTextSize(c->name, font, swi, she);
		grid->GetCellRect(i, -1, &rect);
		int x=rect.left+(rect.Width()-swi)/2;
		int y=rect.top+(rect.Height()-she)/2;
		backbuffer->TextOut(x, y, c->name, font, color);
	}
}

//  item-
void GThemeGridWindows::RenderItem(GItem* item)
{
	GItemTree* it=(GItemTree*)item;
	int k=it->GetChildCount();
	for (int i=0;i<k;i++)
	{
		GItem* child=it->GetChild(i);


	}
}

// ---------------------------------------------------------------

GThemeDialogWindows::GThemeDialogWindows(GTheme* owner) : GThemeComponent(owner)
{
}

GThemeDialogWindows::~GThemeDialogWindows()
{
}

//   
unsigned int GThemeDialogWindows::GetBgColor(GComponent* component)
{
	return 0xf0f0f0;
}

void GThemeDialogWindows::EraseClient(GComponent* component)
{
	GDialog* dlg=(GDialog*)component;
	GSurface* bg=dlg->GetBackground();
	if (bg)
	{
		int bwi, bhe;
		bg->GetSize(bwi, bhe);
		CMagicRect rect;
		component->GetClientRect(&rect);
		int dwi, dhe;
		dwi=rect.Width();
		dhe=rect.Height();

		if (dwi>bwi || dhe>bhe)
			GThemeComponent::EraseClient(component);

		int cx=(dwi-bwi)/2;
		int cy=(dhe-bhe)/2;

		GetBackbuffer()->Blt(bg, cx, cy);
	}
	else
		GThemeComponent::EraseClient(component);
}

// ---------------------------------------------------------------

GThemeCheckBoxWindows::GThemeCheckBoxWindows(GTheme* owner) : GThemeStaticWindows(owner)
{
}

GThemeCheckBoxWindows::~GThemeCheckBoxWindows()
{
}

void GThemeCheckBoxWindows::RenderClient(GComponent* component)
{
	GSurface* backbuffer=GetBackbuffer();

	int check_wi, check_he;
	GetMinSize(component, check_wi, check_he);

	int wi, he;
	CMagicRect rect;
	component->GetClientRect(&rect);
	wi=rect.Width();
	he=rect.Height();

	//  
	unsigned int color_border;
	unsigned int color_background;
	unsigned int color_check;
	unsigned int color_text;
	GetCheckBoxColors(component, color_border, color_background, color_check, color_text);

	int x=0;
	int y=he-check_he-1;

	backbuffer->DrawRect(x, y, check_wi, check_he, color_border);
	backbuffer->FillRect(x+1, y+1, check_wi-2, check_he-2, color_background);

	//  
	if (((GCheckBox*)component)->IsChecked())
	{
		backbuffer->DrawLine(x+2, y+2, x+check_wi-3, y+check_he-3, color_check);
		backbuffer->DrawLine(x+2, y+check_he-3, x+check_wi-3, y+2, color_check);
	}

	x+=check_wi;
	x+=GetAdditionalGap();

	int font=GetTextFont(component);

	int twi, the;
	backbuffer->device->GetTextSize("0", 0, twi, the);
	y=he-the;

	const char* text=component->GetWindowText();
	backbuffer->TextOut(x, y, text, font, color_text);
}

void GThemeCheckBoxWindows::GetCheckBoxColors(GComponent* component, unsigned int& color_border, unsigned int& color_background, unsigned int& color_check, unsigned int& color_text)
{
	if (component->IsEnabled())
	{
		if (component->IsHighlighted())
		{
			color_border=0x3399ff;
			color_background=0xf3f9ff;
		}
		else
		{
			color_border=0x707070;
			color_background=0xffffff;
		}
		color_check=0;
		color_text=0;
	}
	else
	{
		color_border=0xbcbcbc;
		color_background=0xe6e6e6;
		color_check=0x707070;
		color_text=0x838383;
	}
}

//      
int GThemeCheckBoxWindows::GetAdditionalGap()
{
	return 5;
}

void GThemeCheckBoxWindows::GetClientGap(CMagicRect* rect)
{
	rect->left=1;
	rect->top=1;
	rect->right=1;
	rect->bottom=1;
}

void GThemeCheckBoxWindows::GetMinSize(GComponent* component, int& wi, int& he)
{
	wi=13;
	he=13;
}