//-----------------------------------------------------------------------------
// File: ThemeWindows.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once
#include "theme.h"

class GScroller;

class GThemeButtonWindows : public GThemeComponent
{
public:
	GThemeButtonWindows(GTheme* owner);
	virtual ~GThemeButtonWindows();

	virtual void RenderClient(GComponent* component);
	virtual void EraseInnerRect(GComponent* component, CMagicRect* rect){}
	virtual void EraseClient(GComponent* component){}

	virtual void GetClientGap(CMagicRect* rect);

	//   
	virtual unsigned int GetBorderColor(GComponent* component);
};

class GThemeButtonImageWindows : public GThemeComponent
{
public:
	GThemeButtonImageWindows(GTheme* owner);
	virtual ~GThemeButtonImageWindows();

	virtual void RenderClient(GComponent* component);

	virtual void GetClientGap(CMagicRect* rect);

	//   
	virtual unsigned int GetBorderColor(GComponent* component);
};

class GThemeStaticWindows : public GThemeComponent
{
public:
	GThemeStaticWindows(GTheme* owner);
	virtual ~GThemeStaticWindows();

	virtual void RenderClient(GComponent* component);

	virtual void GetClientGap(CMagicRect* rect);

	virtual void GetMinSize(GComponent* component, int& wi, int& he);

	//   
	unsigned int GetBgColor(GComponent* component);
};

class GThemeEditWindows : public GThemeStaticWindows
{
public:
	GThemeEditWindows(GTheme* owner);
	virtual ~GThemeEditWindows();

	//   
	virtual unsigned int GetBorderColor(GComponent* component);

	//   
	unsigned int GetBgColor(GComponent* component);

	virtual void RenderClient(GComponent* component);

	virtual void GetClientGap(CMagicRect* rect);

	virtual void GetMinSize(GComponent* component, int& wi, int& he);
};

class GThemeListBoxWindows : public GThemeStaticWindows
{
public:
	GThemeListBoxWindows(GTheme* owner);
	virtual ~GThemeListBoxWindows();

	//   
	virtual unsigned int GetBgColor(GComponent* component);

	//     item-
	virtual bool GetItemTextRect(GItem* item, CMagicRect* rect);

	//     item-
	virtual bool GetItemIconRect(GItem* item, CMagicRect* rect);

	virtual void GetClientGap(CMagicRect* rect){rect->left=0; rect->top=0; rect->right=0; rect->bottom=0;}

	//     item-
	void GetItemGapRect(GItem* item, CMagicRect* rect);

	//   
	//virtual unsigned int GetBorderColor(GComponent* component);

	virtual void RenderClient(GComponent* component){GThemeComponent::RenderClient(component);}
};

class GThemeComboBoxWindows : public GThemeEditWindows
{
protected:
	GSurface* drop;
	GSurface* drop_disabled;

public:
	GThemeComboBoxWindows(GTheme* owner);
	virtual ~GThemeComboBoxWindows();

	virtual bool RenderInner(GComponent* component);

	//  ,   
	//  item-     ComboBox-
	virtual bool GetItemAdditionalRect(GItem* item, CMagicRect* rect);

	//   
	virtual unsigned int GetBorderColor(GComponent* component);
};

class GThemeMenuWindows : public GThemeListBoxWindows
{
protected:
	GSurface* has_children;

public:
	GThemeMenuWindows(GTheme* owner);
	virtual ~GThemeMenuWindows();

	virtual void GetClientGap(CMagicRect* rect);

	//     item-
	void GetItemGapRect(GItem* item, CMagicRect* rect);

	//     item-
	virtual bool GetItemTextRect(GItem* item, CMagicRect* rect);

	//      item- (          )
	virtual bool GetItemAdditionalRect(GItem* item, CMagicRect* rect);

	//   
	//virtual unsigned int GetBorderColor(GComponent* component);

	//   
	//virtual unsigned int GetBgColor(GComponent* component){return 0xf0f0f0;}

	virtual void RenderItem(GItem* item);

	//     
	int GetIconFieldWidth(GItem* item);

	//      
	int GetWidthBeforeText(){return 6;}
};

class GThemeTreeWindows : public GThemeListBoxWindows
{
public:
	GThemeTreeWindows(GTheme* owner);
	virtual ~GThemeTreeWindows();

	//     item-
	bool GetItemTextRect(GItem* item, CMagicRect* rect);

	//     item-
	virtual bool GetItemIconRect(GItem* item, CMagicRect* rect);

	//      item- (          )
	bool GetItemAdditionalRect(GItem* item, CMagicRect* rect);

	//  X-  
	int GetNodeX(GItem* item, int level);

	//  item-
	virtual void RenderItem(GItem* item);

	//       item-
	virtual bool ItemHighlightFull(){return false;}

	//    
	virtual int GetNodeHalf(){return 5;}

	//   
	virtual int GetLevelDistance(GComponent* component);

	//    X        item-
	virtual int GetDistanceFromNodeToItem(GComponent* component);
};

class GThemePictureWindows : public GThemeComponent
{
public:
	GThemePictureWindows(GTheme* owner);

	virtual void RenderClient(GComponent* component);
	virtual void EraseInnerRect(GComponent* component, CMagicRect* rect){}
	virtual void EraseClient(GComponent* component){}
};

class GThemeGridWindows : public GThemeListBoxWindows
{
public:
	GThemeGridWindows(GTheme* owner);

	//  
	void RenderCaption(GComponent* component);

	//  item-
	void RenderItem(GItem* item);
};

class GThemeDialogWindows : public GThemeComponent
{
public:
	GThemeDialogWindows(GTheme* owner);
	virtual ~GThemeDialogWindows();

	//   
	unsigned int GetBgColor(GComponent* component);

	virtual void EraseClient(GComponent* component);
};

class GThemeCheckBoxWindows : public GThemeStaticWindows
{
public:
	GThemeCheckBoxWindows(GTheme* owner);
	virtual ~GThemeCheckBoxWindows();

	virtual void RenderClient(GComponent* component);

	virtual void GetCheckBoxColors(GComponent* component, unsigned int& color_border, unsigned int& color_background, unsigned int& color_check, unsigned int& color_text);

	virtual void GetClientGap(CMagicRect* rect);

	virtual void GetMinSize(GComponent* component, int& wi, int& he);

	//      
	virtual int GetAdditionalGap();
};

class GThemeWindows :
	public GTheme
{
public:
	GThemeWindows(GGame* game);
	virtual ~GThemeWindows(void);

	//     
	//void CreateScrollButton(bool hor_vert, bool one_two);

	//     
	virtual int GetSBSize(){return 17;}

	//   
	virtual void RenderScroller(GScroller* component);

	//        (  )
	virtual void GetScrollerPlaceColors(unsigned int& color_place, unsigned int& color_place_border, unsigned int& color_slider_normal, unsigned int& color_slider_over, unsigned int& color_slider_pressed);

	//   
	//virtual unsigned int GetBorderColor(GComponent* component);

	//  
	//virtual void RenderBorder(GComponent* component);

	//  
	//void RenderCaption(GComponent* component);

	//   
	virtual int GetCaptionHeight(GComponent* component);

	//       
	virtual const char* GetTextNameFont(){return "Tahoma16bold";}//"segoe_ui_semilight16bold";}//"arial15";}

	//       
	virtual const char* GetCaptionTextNameFont(){return "Tahoma21";}//"nyala22bold";}//"corbel24b";}

	//   
	//virtual unsigned int GetBgColor(GComponent* component);
	//   
	//virtual unsigned int GetTextColor(GComponent* component);
	//    
	virtual unsigned int GetSelectedTextColor(GComponent* component){return 0xffffff;}
	//   
	virtual unsigned int GetSelectedColor(GComponent* component){return 0xa08000;}

	//   
	virtual unsigned int GetBorderColorDefault(){return 0xa0a0a0;}
	
	//    
	//virtual unsigned int GetCaptionBgColor(GComponent* component){return 0x947955;}
	//    
	//virtual unsigned int GetCaptionTextColor(GComponent* component){return 0;}

	//   
	virtual int GetItemSeparatorHeight(){return 2;}
};
