//-----------------------------------------------------------------------------
// File: VirtualDrive.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "VirtualDrive.h"
#include "Game.h"

const unsigned int virtual_file_data_prefix=0xf984ab4c;
const unsigned int virtual_file_fat_prefix=0x6f9ae7d9;
const unsigned int virtual_file_prefix_size=4;
const unsigned int virtual_claster_size=4*1024;

// -------------------------------------------------------

MIX_CLASTER_LIST::MIX_CLASTER_LIST()
{
	k_claster=0;
	m_claster=NULL;
}

MIX_CLASTER_LIST::~MIX_CLASTER_LIST()
{
	for (int i=0;i<k_claster;i++)
	{
		delete m_claster[i];
		m_claster[i]=NULL;
	}

	if (m_claster)
	{
		delete []m_claster;
		m_claster=NULL;
	}
	k_claster=0;
}

//   
void MIX_CLASTER_LIST::Add(int count)
{
	int i;
	if (m_claster)
	{
		MIX_CLASTER** vm_claster=new MIX_CLASTER*[k_claster+count];
		for (i=0;i<k_claster;i++)
			vm_claster[i]=m_claster[i];
		delete []m_claster;
		m_claster=vm_claster;
	}
	else
		m_claster=new MIX_CLASTER*[count];
	for (i=0;i<count;i++)
		m_claster[i+k_claster]=new MIX_CLASTER;
	k_claster+=count;
}

//  
void MIX_CLASTER_LIST::Mix()
{
	if (k_claster>1)
	{
		int max=k_claster-1;
		int i=0;

		MIX_CLASTER** vm_claster=new MIX_CLASTER*[k_claster];
		//      
		do 
		{
			int to=m_claster[i]->claster;
			vm_claster[to]=m_claster[i];

			//MIX_CLASTER* v=m_claster[i];
			//m_claster[i]=m_claster[to];
			//m_claster[to]=v;
			i++;
		}
		while (i<k_claster);
		delete []m_claster;
		m_claster=vm_claster;

		i=0;
		do 
		{
			int to=GetRndInt(0, max);
			int v=m_claster[i]->claster;
			m_claster[i]->claster=m_claster[to]->claster;
			m_claster[to]->claster=v;
			i++;
		}
		while (i<k_claster);
	}
}

// -------------------------------------------------------


void VIRTUAL_ITEM::SaveToFile(FILE* file)
{
	name.SaveToStream(file);
}

void VIRTUAL_ITEM::LoadFromFile(FILE* file)
{
	name.LoadFromStream(file);
}

// -------------------------------------------------------

VIRTUAL_ITEM_LIST::VIRTUAL_ITEM_LIST()
{
	k_item=0;
	m_item=NULL;
}

VIRTUAL_ITEM_LIST::~VIRTUAL_ITEM_LIST()
{
	Clear();
}

void VIRTUAL_ITEM_LIST::Clear()
{
	for (int i=0;i<k_item;i++)
	{
		delete m_item[i];
		m_item[i]=NULL;
	}

	if (m_item)
	{
		delete []m_item;
		m_item=NULL;
	}
	k_item=0;
}

void VIRTUAL_ITEM_LIST::SaveToFile(FILE* file)
{
	fwrite(&k_item, sizeof(k_item), 1, file);
	for (int i=0;i<k_item;i++)
	{
		int type=0;
		if (m_item[i]->IsFile())
			type=1;
		fwrite(&type, sizeof(type), 1, file);
		m_item[i]->SaveToFile(file);
	}
}

void VIRTUAL_ITEM_LIST::LoadFromFile(FILE* file)
{
	Clear();

	fread(&k_item, sizeof(k_item), 1, file);
	if (k_item)
	{
		m_item=new VIRTUAL_ITEM*[k_item];
		for (int i=0;i<k_item;i++)
		{
			int type;
			fread(&type, sizeof(type), 1, file);
			if (!type)
			{
				//  
				m_item[i]=new VIRTUAL_FOLDER;
			}
			else
			{
				//  
				m_item[i]=new VIRTUAL_FILE;
			}
			m_item[i]->LoadFromFile(file);
		}
	}
}


// -------------------------------------------------------

VIRTUAL_FOLDER::VIRTUAL_FOLDER() : VIRTUAL_ITEM()
{
	version_virtual_folder=VERSION_VIRTUAL_FOLDER;

	k_list=0;
	m_list=NULL;
}

VIRTUAL_FOLDER::~VIRTUAL_FOLDER()
{
	Clear();
}

void VIRTUAL_FOLDER::Clear()
{
	for (int i=0;i<k_list;i++)
	{
		if (m_list[i])
		{
			delete m_list[i];
			m_list[i]=NULL;
		}
	}

	if (m_list)
	{
		delete []m_list;
		m_list=NULL;
	}
	k_list=0;
}

VIRTUAL_ITEM* VIRTUAL_FOLDER::Find(CMagicString& name)
{
	int index=(unsigned char)name[0];
	if (index<k_list)
	{
		VIRTUAL_ITEM_LIST* list=m_list[index];
		if (list)
		{
			for (int i=0;i<list->k_item;i++)
			{
				if (list->m_item[i]->name==name)
					return list->m_item[i];
			}
		}
	}
	return NULL;
}

void VIRTUAL_FOLDER::Add(VIRTUAL_ITEM* item)
{
	int index=(unsigned char)item->name[0];
	if (index>=k_list)
	{
		//   
		int k_list2=index+1;
		VIRTUAL_ITEM_LIST** vm_list=new VIRTUAL_ITEM_LIST*[k_list2];
		int i;
		for (i=0;i<k_list;i++)
			vm_list[i]=m_list[i];
		for (i=k_list;i<k_list2;i++)
			vm_list[i]=NULL;
		if (m_list)
			delete []m_list;
		m_list=vm_list;
		k_list=k_list2;
	}

	VIRTUAL_ITEM_LIST* list=m_list[index];
	if (!list)
	{
		list=new VIRTUAL_ITEM_LIST;
		m_list[index]=list;
		list->k_item=1;
		list->m_item=new VIRTUAL_ITEM*[1];
		list->m_item[0]=item;
	}
	else
	{
		VIRTUAL_ITEM** vm_item=new VIRTUAL_ITEM*[list->k_item+1];
		for (int i=0;i<list->k_item;i++)
			vm_item[i]=list->m_item[i];
		vm_item[list->k_item]=item;
		delete []list->m_item;
		list->m_item=vm_item;
		list->k_item++;
	}

	item->parent=this;
}

void VIRTUAL_FOLDER::Delete(CMagicString& name, GVirtualDrive* drive)
{
	int index=(unsigned char)name[0];
	if (index<k_list)
	{
		VIRTUAL_ITEM_LIST* list=m_list[index];
		if (list)
		{
			for (int i=0;i<list->k_item;i++)
			{
				VIRTUAL_ITEM* item=list->m_item[i];
				if (item->name==name)
				{
					//   item
					if (item->IsFile())
					{
						VIRTUAL_FILE* file=(VIRTUAL_FILE*)item;
						file->ClastersToDrive(drive);
					}
					delete item;

					list->k_item--;
					if (i!=list->k_item)
					{
						list->m_item[i]=list->m_item[list->k_item];
					}
					else
					{
						if (!list->k_item)
						{
							//   
							delete m_list[index];
							m_list[index]=NULL;
							if (index==k_list-1)
							{
								//    
								//   
								int k=k_list-2;
								k_list=0;
								for (int i=k;i>=0;i--)
								{
									if (m_list[i])
									{
										k_list=i+1;
										break;
									}
								}

								if (!k_list)
								{
									delete []m_list;
									m_list=NULL;
								}
							}
						}
					}
					break;
				}
			}
		}
	}
}

void VIRTUAL_FOLDER::SaveToFile(FILE* file)
{
	VIRTUAL_ITEM::SaveToFile(file);

	fwrite(&version_virtual_folder, sizeof(version_virtual_folder), 1, file);
	
	fwrite(&k_list, sizeof(k_list), 1, file);
	if (k_list>0)
	{
		int i=0;
		do 
		{
			if (m_list[i])
			{
				fwrite(&i, sizeof(i), 1, file);
				m_list[i]->SaveToFile(file);
			}
			i++;
		}
		while (i<k_list);
		i=-1;
		fwrite(&i, sizeof(i), 1, file);
	}
}

void VIRTUAL_FOLDER::LoadFromFile(FILE* file)
{
	Clear();

	VIRTUAL_ITEM::LoadFromFile(file);

	fread(&version_virtual_folder, sizeof(version_virtual_folder), 1, file);

	fread(&k_list, sizeof(k_list), 1, file);
	if (k_list>0)
	{
		m_list=new VIRTUAL_ITEM_LIST*[k_list];
		int i=0;
		do 
		{
			m_list[i]=NULL;
			i++;
		}
		while (i<k_list);

		while(true)
		{
			int index;
			fread(&index, sizeof(index), 1, file);
			if (index==-1)
				break;
			m_list[index]=new VIRTUAL_ITEM_LIST;
			m_list[index]->LoadFromFile(file);
		}
	}
}

//  item-
bool VIRTUAL_FOLDER::GetFirstItem(VIRTUAL_FIND_ITEM& fd, bool is_file, const char* ext)
{
	fd.is_file=is_file;
	if (ext && is_file)
	{
		fd.ext=ext;
		if (fd.ext=="*")
			fd.ext="";
	}
	else
		fd.ext="";
	fd.index_sort=0;
	fd.index=0;
	return GetNextItem(fd);
}

bool VIRTUAL_FOLDER::GetNextItem(VIRTUAL_FIND_ITEM& fd)
{
	while (fd.index_sort<k_list)
	{
		VIRTUAL_ITEM_LIST* list=m_list[fd.index_sort];
		if (list)
		{
			if (fd.index<list->k_item)
			{
				VIRTUAL_ITEM* item=list->m_item[fd.index];
				fd.index++;
				if (item->IsFile()==fd.is_file)
				{
					//  item- 
					bool is=true;
					if (!fd.ext.IsEmpty())
					{
						//     
						VIRTUAL_FILE* file=(VIRTUAL_FILE*)item;
						CMagicString ext=file->name.GetExtension();
						is=(ext==fd.ext);
					}

					if (is)
					{
						//  
						fd.name=item->name;
						return true;
					}
				}
			}
			else
			{
				//     
				fd.index=0;
				fd.index_sort++;
			}
		}
		else
			fd.index_sort++;
	}
	fd.name="";
	return false;
}

//       
void VIRTUAL_FOLDER::AddMixedClisters(MIX_CLASTER_LIST* list)
{
	for (int i=0;i<k_list;i++)
	{
		VIRTUAL_ITEM_LIST* item_list=m_list[i];
		if (item_list)
		{
			for (int j=0;j<item_list->k_item;j++)
			{
				VIRTUAL_ITEM* item=item_list->m_item[j];
				if (item->IsFolder())
				{
					VIRTUAL_FOLDER* folder=(VIRTUAL_FOLDER*)item;
					folder->AddMixedClisters(list);
				}
				else
				{
					VIRTUAL_FILE* file=(VIRTUAL_FILE*)item;
					file->AddMixedClisters(list);
				}
			}
		}
	}
}

// -------------------------------------------------

VIRTUAL_FILE::VIRTUAL_FILE() : VIRTUAL_ITEM()
{
	version_virtual_file=VERSION_VIRTUAL_FILE;

	position=0;
	length=0;

	max_claster=k_claster=0;
	m_claster=NULL;
}

VIRTUAL_FILE::~VIRTUAL_FILE()
{
	Clear();
}

void VIRTUAL_FILE::Clear()
{
	if (m_claster)
	{
		delete []m_claster;
		m_claster=NULL;
	}
	max_claster=k_claster=0;
}

/*
void VIRTUAL_FILE::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<version_virtual_file;

		stream<<position;
		stream<<length;
	}
	else
	{
		stream>>version_virtual_file;

		stream>>position;
		stream>>length;
	}
	version_virtual_file=VERSION_VIRTUAL_FILE;
}
*/

void VIRTUAL_FILE::SaveToFile(FILE* file)
{
	VIRTUAL_ITEM::SaveToFile(file);

	fwrite(&version_virtual_file, sizeof(version_virtual_file), 1, file);

	fwrite(&position, sizeof(position), 1, file);
	fwrite(&length, sizeof(length), 1, file);
	fwrite(&k_claster, sizeof(k_claster), 1, file);
	if (k_claster>0)
		fwrite(m_claster, sizeof(int), k_claster, file);
}

void VIRTUAL_FILE::LoadFromFile(FILE* file)
{
	Clear();

	VIRTUAL_ITEM::LoadFromFile(file);

	fread(&version_virtual_file, sizeof(version_virtual_file), 1, file);

	fread(&position, sizeof(position), 1, file);
	fread(&length, sizeof(length), 1, file);
	fread(&k_claster, sizeof(k_claster), 1, file);
	max_claster=k_claster;
	if (k_claster>0)
	{
		m_claster=new int[k_claster];
		fread(m_claster, sizeof(int), k_claster, file);
	}
}

void VIRTUAL_FILE::AddClaster(int claster)
{
	if (max_claster==k_claster)
	{
		//   
		int step=max_claster/4;
		if (step<4)
			step=4;
		int vmax_claster=max_claster+step;
		int* vm_claster=new int[vmax_claster];
		int i;
		for (i=0;i<max_claster;i++)
			vm_claster[i]=m_claster[i];
		if (m_claster)
			delete []m_claster;
		m_claster=vm_claster;
		max_claster=vmax_claster;
	}
	m_claster[k_claster]=claster;
	k_claster++;
}

void VIRTUAL_FILE::ClastersToDrive(GVirtualDrive* drive)
{
	for (int i=0;i<k_claster;i++)
		drive->AddDeletedClaster(m_claster[i]);
}

//       
void VIRTUAL_FILE::AddMixedClisters(MIX_CLASTER_LIST* list)
{
	//if (name=="space.png")
	//	name=name;
	if (k_claster)
	{
		int index=list->GetCount();
		list->Add(k_claster);
		int i=0;
		do 
		{
			MIX_CLASTER* claster=list->GetClaster(i+index);
			claster->claster_ptr=&(m_claster[i]);
			claster->claster=*claster->claster_ptr;

			//if (claster->claster==14496)
			//	i=i;

			i++;
		}
		while (i<k_claster);
	}
}

// -------------------------------------------------

GVirtualDrive::GVirtualDrive(void)
{
	version=VERSION_VIRTUAL_DRIVE;
	is_changed_fat=false;
	file=NULL;
	file_length=0;
	file_position=0xffffffff;
}

GVirtualDrive::~GVirtualDrive(void)
{
	Close();
}

//   
CMagicString GVirtualDrive::Create(const char* path, const char* name)
{
	if (!file)
	{
		GPlatform* platform=GSurface::device->platform;
		CMagicString full_path=platform->PathToAbsolutePath(path);
		full_path+=name;
		this->path=full_path+".vdt";
		if (platform->GetFileLength(this->path)==-1)
		{
			//   
			CMagicString path_vdd=full_path+".vdd";
			name_vdd=path_vdd.GetFileNameOnly();
			is_changed_fat=true;
			if (Flush())
			{
				file=platform->OpenFile(path_vdd, "wb");
				if (file)
				{
					fwrite(&virtual_file_data_prefix, sizeof(virtual_file_data_prefix), 1, file);
					platform->CloseFile(file);
					return this->path;
				}
			}
		}
	}
	return "";
}

//     
bool GVirtualDrive::Open(const char* path)
{
	if (!file)
	{
		file_position=0xffffffff;
		GPlatform* platform=GSurface::device->platform;
		if (file_length!=-1)
		{
			file=platform->OpenFile(path, "rb");
			if (file)
			{
				unsigned int prefix;
				fread(&prefix, sizeof(prefix), 1, file);
				if (virtual_file_fat_prefix==prefix)
				{
					this->path=platform->PathToAbsolutePath(path);
					fread(&claster_size, sizeof(claster_size), 1, file);
					name_vdd.LoadFromStream(file);
					root.LoadFromFile(file);
					del_clasters.LoadFromFile(file);
					platform->CloseFile(file);

					CMagicString path_vdd=path;
					path_vdd=path_vdd.GetPathOnly();
					path_vdd=platform->PathToAbsolutePath(path_vdd);
					path_vdd+=platform->GetSlash();
					path_vdd+=name_vdd;
					file_length=platform->GetFileLength(path_vdd);
					if (file_length!=-1)
					{
						file=platform->OpenFile(path_vdd, "rb+");
						if (!file)
						{
							//         ,       
							file=platform->OpenFile(path_vdd, "rb");
						}
						if (file)
						{
							name_vdt=this->path.GetFileNameOnly();
							int ext_len=name_vdt.GetExtension().GetLength();
							name_vdt=name_vdt.Left(name_vdt.GetLength()-ext_len-1);

							fread(&prefix, sizeof(prefix), 1, file);
							if (virtual_file_data_prefix==prefix)
								return true;
							platform->CloseFile(file);
						}
					}
				}
			}
		}
	}
	file=NULL;
	return false;
}

//   
void GVirtualDrive::Close()
{
	if (file)
	{
		Flush();
		GPlatform* platform=GSurface::device->platform;
		platform->CloseFile(file);
		root.Clear();
		del_clasters.Clear();
		file=NULL;
	}
}

//   
void GVirtualDrive::SetFilePosition(unsigned int position, VIRTUAL_FILE* vf)
{
	if (position>vf->length)
		position=vf->length;
	int index_claster=GetClaster(position);
	if (vf->k_claster>index_claster)
	{
		file_position=position;
		int claster=vf->m_claster[index_claster];
		unsigned int pos=virtual_file_prefix_size+claster_size*claster;
		if (pos!=file_position)
		{
			file_position=pos;
			fseek(file, pos, SEEK_SET);
		}
	}
	else
	{
		//       
		if (file_position!=file_length)
		{
			file_position=file_length;
			fseek(file, 0, SEEK_END);
		}
	}
}

//   ,   
int GVirtualDrive::GetClaster(unsigned int position)
{
	return position/claster_size;
}

//  ,   
unsigned int GVirtualDrive::GetClasterPosition(int claster)
{
	return claster_size*claster;
}

//    
void GVirtualDrive::ReadClaster(char* m_data, int k_data)
{
	fread(m_data, 1, k_data, file);
	file_position+=k_data;
}

//    
void GVirtualDrive::WriteClaster(char* m_data, int k_data, unsigned int position, VIRTUAL_FILE* vf)
{
	if ((position % claster_size)==0)
	{
		//    
		//    
		int claster;
		int count_deleted=del_clasters.GetIndexCount();
		if (count_deleted>0)
		{
			claster=del_clasters.Get(count_deleted-1);
			del_clasters.Delete(count_deleted-1);
			is_changed_fat=true;
		}
		else
		{
			claster=(file_length-virtual_file_prefix_size)/claster_size;
			file_length+=k_data;
		}
		vf->AddClaster(claster);
		vf->length+=k_data;
	}
	fwrite(m_data, 1, k_data, file);
	file_position+=k_data;
}

//  FAT
bool GVirtualDrive::Flush()
{
	if (is_changed_fat)
	{
		is_changed_fat=false;
		GPlatform* platform=GSurface::device->platform;
		FILE* file_vdt=platform->OpenFile(path, "wb");
		if (file_vdt)
		{
			fwrite(&virtual_file_fat_prefix, sizeof(virtual_file_fat_prefix), 1, file_vdt);
			fwrite(&virtual_claster_size, sizeof(virtual_claster_size), 1, file_vdt);
			name_vdd.SaveToStream(file_vdt);
			root.SaveToFile(file_vdt);
			del_clasters.SaveToFile(file_vdt);
			platform->CloseFile(file_vdt);
			return true;
		}
		return false;
	}
	return true;
}

//     
void GVirtualDrive::AddDeletedClaster(int claster)
{
	del_clasters.Add(claster);
}

//       
VIRTUAL_ITEM* GVirtualDrive::GetItem(const char* path)
{
	full_path=path;
	int len=full_path.GetLength();
	if (len>1)
	{
		if (full_path[0]!=VIRTUAL_DRIVE_SYMBOL)
		{
			//   
			//     
			full_path=current_directory+full_path;
			len=full_path.GetLength();
		}

		VIRTUAL_FOLDER* folder=&root;
		for (int i=1;i<len;i++)
		{
			CMagicString name;
			int j;
			for (j=i;j<len;j++)
			{
				char s=full_path[j];
				if (s=='/' || s=='\\')
				{
					//   
					break;
				}
				else
					name+=s;
			}

			i=j;
			if ((!name.IsEmpty()) && name!=".")
			{
				//      item   
				if (name=="..")
				{
					//      
					folder=folder->GetParent();
					if (!folder)
						return NULL;
				}
				else
				{
					VIRTUAL_ITEM* item=folder->Find(name);
					if (item)
					{
						if (item->IsFolder())
						{
							//  
							folder=(VIRTUAL_FOLDER*)item;
						}
						else
						{
							//  
							if (i==len) //     
								return item;
							else
								return NULL;
						}
					}
					else
					{
						// item     
						return NULL;
					}
				}
			}
		}
		return folder;
	}
	else
	{
		if (len==0 || full_path[0]==VIRTUAL_DRIVE_SYMBOL)
			return &root;
	}
	return NULL;
}

//  
bool GVirtualDrive::CreateDirectory(const char* path)
{
	VIRTUAL_ITEM* item=GetItem(path);
	if (!item)
	{
		//   
		CMagicString parent_path=full_path.GetPathOnly();
		CMagicString name=full_path.GetFileNameOnly();

		item=GetItem(parent_path);
		if (item && item->IsFolder())
		{
			//   
			VIRTUAL_FOLDER* folder=(VIRTUAL_FOLDER*)item;
			//if (!folder->Find(name))
			{
				//      
				VIRTUAL_FOLDER* f=new VIRTUAL_FOLDER;
				f->name=name;
				folder->Add(f);
				is_changed_fat=true;
				return true;
			}
		}
	}
	return false;
}

//  
bool GVirtualDrive::RemoveDirectory(const char* path)
{
	VIRTUAL_ITEM* item=GetItem(path);
	if (item && item->IsFolder())
	{
		//   
		VIRTUAL_FOLDER* folder=(VIRTUAL_FOLDER*)item;
		if (!folder->HasChildren())
		{
			CMagicString parent_path=full_path.GetPathOnly();
			CMagicString name=full_path.GetFileNameOnly();
			folder=(VIRTUAL_FOLDER*)GetItem(parent_path);
			folder->Delete(name, this);
			if (!GetDirectory())
				current_directory=VIRTUAL_DRIVE_SYMBOL;
			is_changed_fat=true;
			return true;
		}
	}
	return false;
}

//  
bool GVirtualDrive::CreateFile(const char* path)
{
	VIRTUAL_ITEM* item=GetItem(path);
	if (!item)
	{
		//   
		CMagicString parent_path=full_path.GetPathOnly();
		CMagicString name=full_path.GetFileNameOnly();

		item=GetItem(parent_path);
		if (item && item->IsFolder())
		{
			//   
			VIRTUAL_FOLDER* folder=(VIRTUAL_FOLDER*)item;
			//if (!folder->Find(name))
			{
				//      
				VIRTUAL_FILE* f=new VIRTUAL_FILE;
				f->name=name;
				folder->Add(f);

				//     ,      
				unsigned int len=file_length-virtual_file_prefix_size;
				unsigned int dop=len % claster_size;
				if (dop)
				{
					dop=claster_size-dop;
					fseek(file, 0, SEEK_END);
					char* v=new char[dop];
					fwrite(v, 1, dop, file);
					delete []v;
					file_position=ftell(file);
					file_length=file_position;
				}

				is_changed_fat=true;
				return true;
			}
		}
	}
	return false;
}

//  
bool GVirtualDrive::RemoveFile(const char* path)
{
	VIRTUAL_ITEM* item=GetItem(path);
	if (item && item->IsFile())
	{
		//   
		CMagicString parent_path=full_path.GetPathOnly();
		CMagicString name=full_path.GetFileNameOnly();
		VIRTUAL_FOLDER* folder=(VIRTUAL_FOLDER*)GetItem(parent_path);
		folder->Delete(name, this);
		is_changed_fat=true;
		return true;
	}
	return false;
}

//   
const char* GVirtualDrive::GetDirectory()
{
	return current_directory;
}

//   
bool GVirtualDrive::SetDirectory(const char* path)
{
	VIRTUAL_ITEM* item=GetItem(path);
	if (item && item->IsFolder())
	{
		current_directory=full_path;
		return true;
	}
	return false;
}

//      
CMagicStreamVirtualFile* GVirtualDrive::OpenStream(const char* file, int mode)
{
	CMagicStreamVirtualFile* stream=new CMagicStreamVirtualFile;
	if (!stream->Open(this, file, mode))
	{
		delete stream;
		stream=NULL;
	}
	return stream;
}

//    
void GVirtualDrive::Mix()
{
	//         

	MIX_CLASTER_LIST mix_list;
	root.AddMixedClisters(&mix_list);

	int k_claster=mix_list.GetCount();
	if (k_claster>1)
	{
		//  
		int i;

		GPlatform* platform=GSurface::device->platform;
		SetRndIndex(platform->GetTick());
		
		mix_list.Mix();

		int max_len=virtual_file_prefix_size+k_claster*claster_size;
		char* data=new char[max_len];

		//   
		fseek(file, 0, SEEK_SET);
		for (i=0;i<virtual_file_prefix_size;i++)
			fread(data+i, 1, 1, file);
		
		//    
		for (i=0;i<k_claster;i++)
		{
			MIX_CLASTER* claster=mix_list.GetClaster(i);
			int from=claster->claster;
			unsigned int pos=GetClasterPosition(from);
			pos+=virtual_file_prefix_size;
			fread(data+pos, 1, claster_size, file);
		}

		fseek(file, 0, SEEK_SET);
		fwrite(data, 1, max_len, file);
		delete []data;
		data=NULL;

		//    FAT
		for (i=0;i<k_claster;i++)
		{
			MIX_CLASTER* claster=mix_list.GetClaster(i);
			*(claster->claster_ptr)=claster->claster;
		}

		file_length=max_len;
		file_position=0xffffffff;

		is_changed_fat=true;
		Flush();
	}
}

// ---------------------------------------------------------------------

CMagicStreamVirtualFile::CMagicStreamVirtualFile() : CMagicStream()
{
	drive=NULL;
	file=NULL;
	position=0;
}

CMagicStreamVirtualFile::~CMagicStreamVirtualFile()
{
	Close();
}

//    
bool CMagicStreamVirtualFile::Open(GVirtualDrive* drive, const char* file, int mode)
{
	VIRTUAL_ITEM* item=drive->GetItem(file);
	if (item)
	{
		// item 
		if (item->IsFile())
		{
			// item  
			if (mode==MAGIC_STREAM_WRITE)
			{
				//    
				drive->RemoveFile(file);
				drive->CreateFile(file);
				item=drive->GetItem(file);
			}
			this->file=(VIRTUAL_FILE*)item;
		}
	}
	else
	{
		// item  
		if (mode==MAGIC_STREAM_WRITE || mode==MAGIC_STREAM_ADD)
		{
			//    
			if (drive->CreateFile(file))
			{
				//   
				this->file=(VIRTUAL_FILE*)drive->GetItem(file);
			}
		}
	}

	if (this->file)
	{
		//  
		this->drive=drive;
		this->mode=mode;
		if (mode==MAGIC_STREAM_ADD)
		{
			//      
			unsigned int len=this->file->GetLength();
			drive->SetFilePosition(len, this->file);
		}
		return true;
	}
	return false;
}

// 
void CMagicStreamVirtualFile::Close()
{
	drive=NULL;
	file=NULL;
}

//   
unsigned int CMagicStreamVirtualFile::GetLength()
{
	return file->GetLength();
}

//   
unsigned int CMagicStreamVirtualFile::GetPosition()
{
	return position;
}

//   
void CMagicStreamVirtualFile::SetPosition(unsigned int pos)
{
	unsigned int len=GetLength();
	if (position>len)
		position=len;
}

unsigned int CMagicStreamVirtualFile::Write(void* m_data, unsigned int k_data)
{
	char* m=(char*)m_data;
	unsigned int k_data_copy=k_data;
	
	drive->SetFilePosition(position, file);

	int first_claster=drive->GetClaster(position);
	int last_claster=drive->GetClaster(position+k_data-1);
	if (first_claster!=last_claster)
	{
		//   ,       
		int second_claster=first_claster+1;
		unsigned int position_next=drive->GetClasterPosition(second_claster);
		int first_len=position_next-position;
		//   
		drive->WriteClaster(m, first_len, position, file);
		m+=first_len;
		k_data-=first_len;
		position+=first_len;

		//       
		int claster_len=drive->GetClasterSize();
		for (int i=second_claster;i<last_claster;i++)
		{
			drive->SetFilePosition(position, file);
			drive->WriteClaster(m, claster_len, position, file);
			m+=claster_len;
			k_data-=claster_len;
			position+=claster_len;
		}
	}
	//   
	drive->SetFilePosition(position, file);
	drive->WriteClaster(m, k_data, position, file);
	position+=k_data;
	return k_data_copy;
}

unsigned int CMagicStreamVirtualFile::Read(void* m_data,unsigned int k_data)
{
	char* m=(char*)m_data;

	unsigned int len=file->GetLength();
	unsigned int max_data=len-position;
	if (max_data<k_data)
	{
		k_data=max_data;
		if (!k_data)
			return 0;
	}
	unsigned int k_data_copy=k_data;

	drive->SetFilePosition(position, file);

	int first_claster=drive->GetClaster(position);
	int last_claster=drive->GetClaster(position+k_data-1);
	if (first_claster!=last_claster)
	{
		//   ,       
		int second_claster=first_claster+1;
		unsigned int position_next=drive->GetClasterPosition(second_claster);
		int first_len=position_next-position;
		//   
		drive->ReadClaster(m, first_len);
		m+=first_len;
		k_data-=first_len;
		position+=first_len;

		//       
		int claster_len=drive->GetClasterSize();
		for (int i=second_claster;i<last_claster;i++)
		{
			drive->SetFilePosition(position, file);
			drive->ReadClaster(m, claster_len);
			m+=claster_len;
			k_data-=claster_len;
			position+=claster_len;
		}
	}
	//   
	drive->SetFilePosition(position, file);
	drive->ReadClaster(m, k_data);
	position+=k_data;
	return k_data_copy;
}
