//-----------------------------------------------------------------------------
// File: VirtualDrive.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#pragma once

#include "utils.h"

#define VIRTUAL_DRIVE_SYMBOL	':'

struct VIRTUAL_FOLDER;
class GVirtualDrive;
class CMagicStreamVirtualFile;

//    
struct MIX_CLASTER
{
	int claster;
	int* claster_ptr;

	MIX_CLASTER()
	{
		claster=-1;
		claster_ptr=NULL;
	}
};

// ,    
struct MIX_CLASTER_LIST
{
	int k_claster;
	MIX_CLASTER** m_claster;

	MIX_CLASTER_LIST();
	virtual ~MIX_CLASTER_LIST();

	//   
	void Add(int count);

	int GetCount(){return k_claster;}

	//  
	MIX_CLASTER* GetClaster(int index){return m_claster[index];}

	//  
	void Mix();
};

//      
struct FINDER_DATA
{
	bool is_virtual_drive;

	FINDER_DATA()
	{
		is_virtual_drive=false;
	}

	virtual ~FINDER_DATA(){}

	bool IsVirtualDrive(){return is_virtual_drive;}
};

//   item-
struct VIRTUAL_FIND_ITEM : public FINDER_DATA
{
	VIRTUAL_FOLDER* folder;
	CMagicString name;
	int index_sort;
	int index;
	bool is_file;
	CMagicString ext;

	VIRTUAL_FIND_ITEM()
	{
		is_virtual_drive=true;
		folder=NULL;
		index_sort=index=-1;
		is_file=false;
	}

	virtual ~VIRTUAL_FIND_ITEM()
	{
	}
};

struct VIRTUAL_ITEM
{
	CMagicString name;
	VIRTUAL_FOLDER* parent;

	virtual ~VIRTUAL_ITEM(){}

	const char* GetName(){return name.c_str();}

	VIRTUAL_FOLDER* GetParent(){return parent;}

	virtual bool IsFolder(){return false;}
	virtual bool IsFile(){return false;}

	//virtual void Serialize(CMagicStream& stream)=0;
	virtual void SaveToFile(FILE* file);
	virtual void LoadFromFile(FILE* file);
};

struct VIRTUAL_ITEM_LIST
{
	int k_item;
	VIRTUAL_ITEM** m_item;

	VIRTUAL_ITEM_LIST();
	virtual ~VIRTUAL_ITEM_LIST();

	void Clear();

	void SaveToFile(FILE* file);
	void LoadFromFile(FILE* file);
};

#define VERSION_VIRTUAL_FOLDER	0
struct VIRTUAL_FOLDER : public VIRTUAL_ITEM
{
	unsigned int version_virtual_folder;

	int k_list;
	VIRTUAL_ITEM_LIST** m_list;

	VIRTUAL_FOLDER();
	virtual ~VIRTUAL_FOLDER();

	void Clear();

	virtual bool IsFolder(){return true;}

	VIRTUAL_ITEM* Find(CMagicString& name);

	void Add(VIRTUAL_ITEM* item);
	void Delete(CMagicString& name, GVirtualDrive* drive);

	bool HasChildren(){return (k_list!=0);}

	//virtual void Serialize(CMagicStream& stream);

	virtual void SaveToFile(FILE* file);
	virtual void LoadFromFile(FILE* file);

	//     
	//void SaveFiles(CMagicStream& stream);

	//  item-
	bool GetFirstItem(VIRTUAL_FIND_ITEM& fd, bool is_file, const char* ext);
	bool GetNextItem(VIRTUAL_FIND_ITEM& fd);

	//       
	void AddMixedClisters(MIX_CLASTER_LIST* list);
};

#define VERSION_VIRTUAL_FILE	0
struct VIRTUAL_FILE : public VIRTUAL_ITEM
{
	unsigned int version_virtual_file;

	unsigned int position;	//   
	unsigned int length;	// 

	int max_claster;
	int k_claster;
	int* m_claster;

	VIRTUAL_FILE();
	virtual ~VIRTUAL_FILE();

	void Clear();

	virtual bool IsFile(){return true;}

	unsigned int GetLength(){return length;}

	//virtual void Serialize(CMagicStream& stream);

	virtual void SaveToFile(FILE* file);
	virtual void LoadFromFile(FILE* file);

	void AddClaster(int claster);

	void ClastersToDrive(GVirtualDrive* drive);

	//       
	void AddMixedClisters(MIX_CLASTER_LIST* list);
};

#define VERSION_VIRTUAL_DRIVE	0
class GVirtualDrive
{
protected:
	unsigned int version;

	CMagicString path;
	CMagicString name_vdt;
	CMagicString name_vdd;

	FILE* file;
	unsigned int file_position;
	unsigned int file_length;

	VIRTUAL_FOLDER root;

	int claster_size;
	CArrayInt del_clasters;

	bool is_changed_fat;

	CMagicString current_directory;
	CMagicString full_path;

public:
	GVirtualDrive(void);
	virtual ~GVirtualDrive(void);

	const char* GetPath(){return path.c_str();}

	//   
	CMagicString Create(const char* path, const char* name);

	//     
	bool Open(const char* path);

	//   
	void Close();

	//   
	int GetClasterSize(){return claster_size;}

	//   
	void SetFilePosition(unsigned int position, VIRTUAL_FILE* vf);

	//   ,   
	int GetClaster(unsigned int position);

	//  ,   
	unsigned int GetClasterPosition(int claster);

	//    
	void ReadClaster(char* m_data, int k_data);

	//    
	void WriteClaster(char* m_data, int k_data, unsigned int position, VIRTUAL_FILE* vf);

	//  FAT
	bool Flush();

	//     
	void AddDeletedClaster(int claster);

	//       
	VIRTUAL_ITEM* GetItem(const char* path);

	//  
	bool CreateDirectory(const char* path);
	//  
	bool RemoveDirectory(const char* path);

	//  
	bool CreateFile(const char* path);
	//  
	bool RemoveFile(const char* path);

	//   
	const char* GetDirectory();
	//   
	bool SetDirectory(const char* path);

	//      
	CMagicStreamVirtualFile* OpenStream(const char* file, int mode);

	//        
	bool IsNameEqual(CMagicString& file_name){return (file_name==name_vdt);}

	//    
	void Mix();
};

class CMagicStreamVirtualFile : public CMagicStream
{
protected:
	GVirtualDrive* drive;
	VIRTUAL_FILE* file;

	unsigned int position;

public:
	CMagicStreamVirtualFile();
	virtual ~CMagicStreamVirtualFile();

	//    
	bool Open(GVirtualDrive* drive, const char* file, int mode=MAGIC_STREAM_READ);

	// 
	virtual void Close();

	//   
	virtual unsigned int GetLength();

	//   
	virtual unsigned int GetPosition();
	//   
	virtual void SetPosition(unsigned int pos);

	virtual unsigned int Write(void* m_data,unsigned int k_data);
	virtual unsigned int Read(void* m_data,unsigned int k_data);

	//   
	virtual void ChangeMode(int mode){}
};

