#include "..\StdAfx.h"
#include "Sound.h"

GSound* GSoundSample::sound=NULL;
GSound* GSoundChannel::sound=NULL;

GSound::GSound(void)
{
	GSoundSample::sound=this;
	GSoundChannel::sound=this;

	k_sample=0;
	m_sample=NULL;

	k_channel=0;
	m_channel=NULL;
}

GSound::~GSound(void)
{
	GSoundSample::sound=NULL;
	GSoundChannel::sound=NULL;

	for (int i=0;i<k_channel;i++)
	{
		if (m_channel[i])
		{
			m_channel[i]->Stop();
			delete m_channel[i];
			m_channel[i]=NULL;
		}
	}

	if (m_channel)
	{
		delete []m_channel;
		m_channel=NULL;
	}


	for (int i=0;i<k_sample;i++)
	{
		if (m_sample[i])
		{
			m_sample[i]->Unload();
			delete m_sample[i];
			m_sample[i]=NULL;
		}
	}

	if (m_sample)
	{
		delete []m_sample;
		m_sample=NULL;
	}
}

int GSound::LoadFromMemory(FILE_DATA& fd)
{
	GSoundSample* sample=NewSample();
	if (sample->LoadFromMemory(fd))
	{
		return AddSample(sample);
	}
	delete sample;
	return -1;
}

bool GSound::Unload(int sample)
{
	if (GetSample(sample))
	{
		m_sample[sample]->Unload();
		delete m_sample[sample];
		m_sample[sample]=NULL;
	}
	return false;
}

int GSound::Play(int sample, float volume, bool loop)
{
	GSoundSample* s=GetSample(sample);
	if (s)
	{
		GSoundChannel* channel=s->Play(loop, CalcVolume(volume));
		return AddChannel(channel);
	}
	return -1;
}

int GSound::Play(int sample, int volume, bool loop)
{
	GSoundSample* s=GetSample(sample);
	if (s)
	{
		GSoundChannel* channel=s->Play(loop, volume);
		return AddChannel(channel);
	}
	return -1;
}

// ---------------------------------------------------------------

bool GSound::Stop(int channel)
{
	GSoundChannel* s=GetChannel(channel);
	if (s)
	{
		s->Stop();
		delete m_channel[channel];
		m_channel[channel]=NULL;
		return true;
	}
	return false;
}

bool GSound::SetVolume(int channel, float volume)
{
	GSoundChannel* s=GetChannel(channel);
	if (s)
	{
		s->SetVolume(volume);
		return true;
	}
	return false;
}

bool GSound::SetVolume(int channel, int volume)
{
	GSoundChannel* s=GetChannel(channel);
	if (s)
	{
		s->SetVolume(volume);
		return true;
	}
	return false;
}

bool GSound::SetMaxPriority(int channel)
{
	GSoundChannel* s=GetChannel(channel);
	if (s)
	{
		s->SetMaxPriority();
		return true;
	}
	return false;
}

int GSound::GetState(int channel)
{
	GSoundChannel* s=GetChannel(channel);
	if (s)
	{
		return s->GetState();
	}
	return SOUND_NO;
}

int GSound::GetPosition(int channel)
{
	GSoundChannel* s=GetChannel(channel);
	if (s)
	{
		return s->GetPosition();
	}
	return SOUND_NO;
}

int GSound::AddSample(GSoundSample* sample)
{
	for (int i=0;i<k_sample;i++)
	{
		if (!m_sample[i])
		{
			m_sample[i]=sample;
			return i;
		}
	}

	if (k_sample)
	{
		GSoundSample** vm_sample=new GSoundSample*[k_sample+1];
		for (int i=0;i<k_sample;i++)
			vm_sample[i]=m_sample[i];
		delete []m_sample;
		m_sample=vm_sample;
	}
	else
		m_sample=new GSoundSample*[1];

	m_sample[k_sample]=sample;
	k_sample++;
	return k_sample-1;
}

int GSound::AddChannel(GSoundChannel* channel)
{
	for (int i=0;i<k_channel;i++)
	{
		if (!m_channel[i])
		{
			m_channel[i]=channel;
			return i;
		}
	}

	if (k_channel)
	{
		GSoundChannel** vm_channel=new GSoundChannel*[k_channel+1];
		for (int i=0;i<k_channel;i++)
			vm_channel[i]=m_channel[i];
		delete []m_channel;
		m_channel=vm_channel;
	}
	else
		m_channel=new GSoundChannel*[1];

	m_channel[k_channel]=channel;
	k_channel++;
	return k_channel-1;
}

GSoundSample* GSound::GetSample(int sample)
{
	if (sample>=0 && sample<k_sample)
	{
		return m_sample[sample];
	}
	return NULL;
}

GSoundChannel* GSound::GetChannel(int channel)
{
	if (channel>=0 && channel<k_channel)
	{
		return m_channel[channel];
	}
	return NULL;
}
