#ifdef BASS_SOUND

#include "SoundBass.h"
#include <math.h>

#pragma comment(lib, "bass.lib")

bool GSoundSampleBass::LoadFromMemory(FILE_DATA& fd)
{
	if (fd.length==0 || fd.data==NULL)
		return false;
	
	bass_sample=BASS_SampleLoad(true, fd.data, 0, fd.length, 5, 0);
	return (bass_sample!=0);
}

void GSoundSampleBass::Unload()
{
	BASS_SampleFree(bass_sample);
}

GSoundChannel* GSoundSampleBass::Play(bool loop, int volume)
{
	HCHANNEL channel=BASS_SampleGetChannel(bass_sample, false);
	if (channel>0)
		return new GSoundChannelBass(channel, loop, volume);
	return NULL;
}

// -------------------------------------------------------------------------------

GSoundChannelBass::GSoundChannelBass(HCHANNEL channel, bool loop, int volume) : GSoundChannel()
{
	bass_channel=channel;
	SetVolume(volume);

	if (BASS_ChannelFlags(channel, 0, 0) & BASS_SAMPLE_LOOP)
	{
		//  
		if (!loop)
		{
			//   
			BASS_ChannelFlags(channel, 0, BASS_SAMPLE_LOOP);
		}
	}
	else
	{
		//  
		if (loop)
		{
			//   
			BASS_ChannelFlags(channel, BASS_SAMPLE_LOOP, BASS_SAMPLE_LOOP);
		}
	}

	BASS_ChannelPlay(channel, true);
}

void GSoundChannelBass::Stop()
{
	BASS_ChannelStop(bass_channel);
}

void GSoundChannelBass::SetVolume(float volume)
{
	float vol=pow(volume, 3.5f);
	if (vol>1.f)
		vol=1.f;
	BASS_ChannelSetAttribute(bass_channel,BASS_ATTRIB_VOL,vol);
}

void GSoundChannelBass::SetVolume(int volume)
{
	float fvolume=((float)volume)/100.f;
	if (fvolume>1.f)
		fvolume=1.f;
	SetVolume(fvolume);
}

void GSoundChannelBass::SetMaxPriority()
{
}

int GSoundChannelBass::GetState()
{
	switch (BASS_ChannelIsActive(bass_channel))
	{
	//case BASS_ACTIVE_STOPPED:
	//	return SOUND_PREPARED;
	case BASS_ACTIVE_PLAYING:
	case BASS_ACTIVE_STALLED:
		return SOUND_PLAY;
	case BASS_ACTIVE_PAUSED:
		return SOUND_PAUSE;
	}
	return SOUND_NO;
}

int GSoundChannelBass::GetPosition()
{
	QWORD pos=BASS_ChannelGetPosition(bass_channel, BASS_POS_BYTE);
	if (pos==-1)
		return -1;
	return (int)pos;
}

// ---------------------------------------------

extern HWND hWnd;
GSoundBass::GSoundBass(void)
{
	BASS_Init(-1, 44100, 0, hWnd, NULL);
}

GSoundBass::~GSoundBass(void)
{
	BASS_Free();
}

GSoundSample* GSoundBass::NewSample()
{
	return new GSoundSampleBass();
}

//       0  1     
int GSoundBass::CalcVolume(float volume)
{
	int ivolume=(int)(volume*100.f);
	if (ivolume>100)
		ivolume=100;
	else
		if (ivolume<0)
			ivolume=0;

	return ivolume;
}

#endif