#ifdef SQUALL_SOUND

#include "SoundSquall.h"

#pragma comment(lib, "squall.lib")


bool GSoundSampleSquall::LoadFromMemory(FILE_DATA& fd)
{
	if (fd.length==0 || fd.data==NULL)
		return false;
	
	squall_sample=SQUALL_Sample_LoadFromMemory(fd.data, fd.length, 1, NULL);
	return (squall_sample>=0);
}

void GSoundSampleSquall::Unload()
{
	SQUALL_Sample_Unload(squall_sample);
}

GSoundChannel* GSoundSampleSquall::Play(bool loop, int volume)
{
	int channel=SQUALL_Sample_PlayEx(squall_sample, loop, 0, true, 0, volume, 0, 50);
	if (channel>=0)
	{
		return new GSoundChannelSquall(channel);
	}
	return NULL;
}

// -------------------------------------------------------------------------------

void GSoundChannelSquall::Stop()
{
	SQUALL_Channel_Stop(squall_channel);
}

void GSoundChannelSquall::SetVolume(float volume)
{
	SetVolume(sound->CalcVolume(volume));
}

void GSoundChannelSquall::SetVolume(int volume)
{
	SQUALL_Channel_SetVolume(squall_channel, volume);
}

void GSoundChannelSquall::SetMaxPriority()
{
}

int GSoundChannelSquall::GetState()
{
	int status=SQUALL_Channel_Status(squall_channel);

	switch(status)
	{
		case SQUALL_CHANNEL_STATUS_PLAY:
		case SQUALL_CHANNEL_STATUS_PREPARED:
			return SOUND_PLAY;

		case SQUALL_CHANNEL_STATUS_PAUSE:
			return SOUND_PAUSE;

		//case SQUALL_CHANNEL_STATUS_PREPARED:
		//	return SOUND_PREPARED;
	}

	return SOUND_NO;
}

int GSoundChannelSquall::GetPosition()
{
	return SQUALL_Channel_GetPlayPosition(squall_channel);
}

// ---------------------------------------------

GSoundSquall::GSoundSquall(void)
{
	SQUALL_Init(NULL);
}

GSoundSquall::~GSoundSquall(void)
{
	SQUALL_Sample_UnloadAll();
	SQUALL_Free();
}

GSoundSample* GSoundSquall::NewSample()
{
	return new GSoundSampleSquall();
}

//       0  1     
const int k_squal_volumes=5;
float m_squal_volumes[k_squal_volumes]={0.f, 40.f, 60.f, 80.f, 100.f};
int GSoundSquall::CalcVolume(float volume)
{
	if (volume<=0.f)
		return (int)m_squal_volumes[0];
	if (volume>=1.f)
		return (int)m_squal_volumes[k_squal_volumes-1];

	float step=1.f/k_squal_volumes;

	int index=-1;
	for (int i=1;i<k_squal_volumes;i++)
	{
		float level=step*i;
		if (volume<level)
		{
			index=i-1;
			break;
		}
	}

	if (index==-1)
		return (int)m_squal_volumes[k_squal_volumes-1];

	float base1=m_squal_volumes[index];
	float base2=m_squal_volumes[index+1];
	float dbase=base2-base1;
	float delta=volume-(step*(index-1));
	float factor=delta/step;

	int result=(int)(base1+dbase*factor);
	return result;
}

#endif
