//-----------------------------------------------------------------------------
// File: utils.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "utils.h"
#include "utf.h"

#include "Game.h"


#pragma warning(disable: 4996)

#ifdef __BIG_ENDIAN__
#define INVERT_BYTES(VAR) do{InvBytes((void*)&VAR,sizeof(VAR));} while(0)
#else
#define INVERT_BYTES(VAR)
#endif

const unsigned char* utf16to8(const unsigned short* str_utf16)
{
	unsigned char* str8=NULL;
	if (str_utf16)
	{
		//    , ,     0
		int length=0;
		while (true)
		{
			unsigned short s=str_utf16[length];
			length++;
			if (!s)
				break;
		}

		int factor=5;

		int end=length*factor;
		str8=new unsigned char[end];

		//      ,      
		const UTF16* vstr_utf16=str_utf16;
		const UTF16** source=(const UTF16**)&vstr_utf16;
		unsigned char* vstr8=str8;
		UTF8** target=&vstr8;

		ConversionResult result=__ConvertUTF16toUTF8 (
			source, (const UTF16*)(&(str_utf16[length])), 
			target, &(str8[end]), lenientConversion);

		if (result!=conversionOK)
		{
			delete []str8;
			str8=NULL;
		}
	}

	return str8;
}

//    utf8  utf16
const unsigned short* utf8to16(const unsigned char* str_utf8)
{
	unsigned short* str16=NULL;
	if (str_utf8)
	{
		//    , ,     0
		int length=0;
		while (true)
		{
			short s=str_utf8[length];
			length++;
			if (!s)
				break;
		}

		int factor=5;

		int end=length*factor;
		str16=new unsigned short[end];

		//      ,      
		const UTF8* vstr_utf8=str_utf8;
		const UTF8** source=&vstr_utf8;
		UTF16* vstr16=str16;
		UTF16** target=&vstr16;

		ConversionResult result=__ConvertUTF8toUTF16 (
			source, (const UTF8*)(&(str_utf8[length])), 
			target, &(str16[end]), lenientConversion);

		if (result!=conversionOK)
		{
			delete []str16;
			str16=NULL;
		}
	}

	return str16;
}

bool disable_game_log=false;
int count_error=0;
//      
void FileError(const char* error)
{
	if (disable_game_log)
		return;

	GPlatform* platform=GSurface::device->platform;

	platform->LockThread();

	CMagicString path=platform->GetPathToTemp();
	path=path+"error.log";

	FILE* log_file;

	if (count_error>0)
		log_file=platform->OpenFile(path.c_str(), "ab");
	else
		log_file=platform->OpenFile(path.c_str(), "wb");

	count_error++;
	CMagicString er=error;
	fwrite(er.c_str(),sizeof(char),er.length(),log_file);
	unsigned short enter=0xa0d;
	fwrite(&enter,sizeof(unsigned short),1,log_file);
	platform->CloseFile(log_file);

	platform->UnlockThread();
}

// ************************************************
// *   
// ************************************************

unsigned int n = 0;
inline double random_MagicParticles() //    [0,1]
{
	static const unsigned int A = 1686629717;
	static const unsigned int C = 907633385;

	return ((double)(n = A*n+C))/0xFFFFFFFF;
}

//    
void SetRndIndex(unsigned int value)
{
	n=value;
}

//    
unsigned int GetRndIndex()
{
	return n;
}

float GetRndFloat(float i1,float i2)
{
	float range=i2-i1;
	double rnd=random_MagicParticles();
	rnd=rnd*range+i1;;
	return (float)rnd;
}

double GetRndDouble(double i1,double i2)
{
	double range=i2-i1;
	double rnd=random_MagicParticles();
	rnd=rnd*range+i1;;
	return rnd;
}

//      
int GetRndInt(int min,int max)
{
	int range=max-min+1;
	double rnd=random_MagicParticles();
	rnd=rnd*range+min;
	int irnd=(int)rnd;
	if (irnd>max)
		irnd=max;

	return irnd;
}

//      
unsigned int GetRndDword(unsigned int min,unsigned int max)
{
	unsigned int range=max-min+1;
	double rnd=random_MagicParticles();
	rnd=rnd*range+min;
	unsigned int irnd=(unsigned int)rnd;
	if (irnd>max)
		irnd=max;

	return irnd;
}

// ----------------------------------------------------

//     fopen
FILE* fopen_magic(const char* file, const char* mode)
{
	FILE* stream=NULL;
	#ifdef WIN32
	//  Windows    utf8  utf16
	wchar_t* wfile=(wchar_t*)utf8to16((unsigned const char*)file);
	wchar_t* wm=(wchar_t*)utf8to16((unsigned const char*)mode);
	stream=_wfopen(wfile, wm);
	delete []wfile;
	delete []wm;
	#else
	stream=fopen(file,mode);
	#endif
	return stream;
}


// 
CMagicString::CMagicString()
{
	theString=new char[1];
	*theString=0;
	stringLen=0;
}

CMagicString::CMagicString(int n)
{
	theString=new char[n+1];
    *theString=0;
	theString[n]=0;
    stringLen=n;
}

CMagicString::CMagicString(char s)
{
	theString=new char[2];
	theString[0]=s;
	theString[1]='\0';
	stringLen=1;
}

CMagicString::CMagicString(const char* s)
{
	int len=strlen(s);
	theString=new char [len+1];
	strcpy(theString,s);
	stringLen=len;
	theString[len]=0;
}

CMagicString::CMagicString(const CMagicString& so)
{
	theString=new char[so.stringLen+1];
	strcpy(theString,so.theString);
	theString[so.stringLen]=0;
	stringLen=so.stringLen;
}

// 
CMagicString::~CMagicString(void)
{
	delete []theString;
}

void CMagicString::SetLength(int length)
{
	if (stringLen!=length)
	{
		if (theString)
			delete []theString;

		theString=new char[length+1];
		stringLen=length;
		theString[length]=0;
	}
	theString[0]=0;
}

//    
void CMagicString::LoadFromStream(FILE* stream)
{
	int dln;
    fread((void*)&dln,sizeof(dln),1,stream); //  
	if (dln!=stringLen)
	{
		delete []theString;
		theString=new char[dln+1];
		stringLen=dln;
	}
	if (dln)
		(void)fread((void*)theString,sizeof(char),dln,stream);
	theString[dln]=0;
}

//    
void CMagicString::SaveToStream(FILE* stream)
{
	fwrite((void*)&stringLen,sizeof(stringLen),1,stream); //  
    if (stringLen)
		fwrite((void*)theString,sizeof(char),stringLen,stream);
}

//    ,   String
void CMagicString::LoadFromStreamForClassString(FILE* stream)
{
	short dln;
	(void )fread((void*)&dln,sizeof(short),1,stream); //  
	if (dln!=stringLen)
	{
		delete []theString;
		theString=new char[dln+1];
		stringLen=dln;
	}
	if (dln)
		(void)fread((void*)theString,sizeof(char),dln,stream);
	theString[dln]=0;
}


/*
//       
void CMagicString::SaveToStreamByText(FILE* stream)
{
	(void)fwrite((void*)theString,sizeof(char),stringLen,stream);
}
*/

//  
CMagicString& CMagicString::operator=(const CMagicString& so)
{
	if (this!=&so)
   	{
		delete []theString;
		theString=new char[so.stringLen+1];
		strcpy(theString,so.theString);
		stringLen=so.stringLen;
	}
	return *this;
}

//  
CMagicString& CMagicString::operator=(const char* s)
{
	int len=strlen(s);
	delete []theString;
	theString=new char[len+1];
	strcpy(theString,s);
	stringLen=len;
	return *this;
}

//  
bool CMagicString::operator==(const CMagicString& so)
{
	int dl1=this->stringLen;
	int dl2=so.stringLen;
	if (dl1!=dl2)
   		return false;
	char* s1=this->theString;
	char* s2=so.theString;
	for (int i=0;i<dl1;i++)
   		if (s1[i]!=s2[i])
      		return false;
	return true;
}

bool CMagicString::operator==(const char* s)
{
	if (strcmp(this->theString,s)==0)
   		return true;
	return false;
}

bool CMagicString::operator!=(const CMagicString& so)
{
	if (strcmp(this->theString,so.theString)==0)
   		return false;
	return true;
}

bool CMagicString::operator!=(const char* s)
{
	if (strcmp(this->theString,s)!=0)
   		return true;
	return false;
}

bool CMagicString::operator>(const CMagicString& so)
{
	if (strcmp(this->theString,so.theString)>0)
   		return true;
	return false;
}

bool CMagicString::operator<(const CMagicString& so)
{
	if (strcmp(this->theString,so.theString)<0)
   		return true;
	return false;
}

// 
char& CMagicString::operator[](int n) const
{
	if (n<0)
   		return theString[0];
   	else
		if (n>stringLen)
         	return theString[stringLen-1];
        else
           	return theString[n];
}

#ifdef _WINDOWS
// 
CMagicString& CMagicString::operator~(void)
{
	strrev(theString);
	return *this;
}
#endif

// 
CMagicString CMagicString::operator+(const CMagicString& so)
{
	int len=stringLen+so.stringLen;
	CMagicString temp(len);
	strcpy(temp.theString,this->theString);
	strcat(temp.theString,so.theString);
	temp.stringLen=len;
	//temp.currentLen=len;
	return temp;
}

CMagicString CMagicString::operator+(const char c)
{
	int len=stringLen+1;
	CMagicString temp(len);
	strcpy(temp.theString,this->theString);
	*(temp.theString+len-1)=c;
	*(temp.theString+len)='\0';
	temp.stringLen=len;
	return temp;
}

// 
CMagicString CMagicString::operator+=(const CMagicString& so)
{
	*this=*this+so;
	return *this;
}

//    
CMagicString& CMagicString::toUpper(void)
{
	#ifdef _WINDOWS
	strupr(theString);
	#else
	for (int i=0;i<stringLen;i++)
		theString[i]=toupper(theString[i]);
	#endif
	return *this;
}

//    
CMagicString& CMagicString::toLower(void)
{
	#ifdef _WINDOWS
	strlwr(theString);
	#else
	for (int i=0;i<stringLen;i++)
		theString[i]=tolower(theString[i]);
	#endif
	return *this;
}

//    
int CMagicString::FindChar(char c, int pos)
{
	if (pos<0 || pos>=stringLen)
		return -1;

	char* v=&(theString[pos]);
	char* s=strchr(v,c);
	if (s==NULL)
   		return -1;
    else
      	return (int)(s-theString);
}

//    
int CMagicString::Find(const char* subs, int pos) const
{
	if (pos<0 || pos>=stringLen)
		return -1;

	char* v=&(theString[pos]);
	char* s=strstr(v,subs);
	if (s==NULL)
   		return -1;
    else
      	return (int)(s-theString);
}

int CMagicString::Find(const CMagicString& so, int pos) const
{
	if (pos<0 || pos>=stringLen)
		return -1;

	char* v=&(theString[pos]);
	char* s=strstr(v,so.theString);
	if (s==NULL)
   		return -1;
    else
      	return (int)(s-theString);
}

//   
const char* CMagicString::c_str()
{
	return (const char*)theString;
}

//          
//  
CMagicString CMagicString::substr(int pos,int plen) const
{
	CMagicString temp(plen);
	for (int i=0;i<plen;i++)
   		temp.theString[i]=theString[pos+i];
	*(temp.theString+plen)='\0';
	return temp;
}


//           (   2- )
void CMagicString::ToSize(int size)
{
	if (stringLen<size)
	{
		int i;
		int razn=size-stringLen;
		int d1=razn/2;
		int d2=razn/2+(razn & 1);
		CMagicString v="";
		for (i=0;i<d1;i++)
			v=v+' ';

		v=v+(*this);
		for (i=0;i<d2;i++)
			v=v+' ';
		(*this)=v;
	}
}

//        
unsigned int CMagicString::TimeOfReading()
{
	//    
	int len=0;
	for (int i=0;i<stringLen;i++)
	{
		if (theString[i]!=' ')
			len++;
	}

	// 600    - 
	//unsigned int reading=(int)(((double)len)*1000*60/600)+1000;
	unsigned int reading=(int)(((double)len)*1000*60/1200)+1000;
	return reading;
}

//    
void CMagicString::IntToString(int v)
{
	#ifdef _WINDOWS
	char s[20];
	itoa(v,s,10);
	*this=s;
	#else
	Format("%d", v);
	#endif
}

//    
int CMagicString::StringToInt()
{
	return atoi(c_str());
}

#ifdef _WINDOWS
//    
void CMagicString::DoubleToString(double v)
{
	char s[50];
	_gcvt(v,10,s);
	*this=s;
}
#endif

//    
double CMagicString::StringToDouble()
{
	return atof(c_str());
}

//      
void CMagicString::StringToHexCodes()
{
	if (!IsEmpty())
	{
		int vlen=stringLen*2;
		char* v=new char[vlen+1];
		int p[2];
		for (int i=0;i<stringLen;i++)
		{
			unsigned int s=(unsigned char)theString[i];
			p[0]=s/16;
			p[1]=s%16;
			int index=i*2;
			for (int j=0;j<2;j++)
			{
				int pp=p[j];
				if (pp>=10)
				{
					v[index]=pp-10+'A';
				}
				else
				{
					v[index]=pp+'0';
				}
				index++;
			}
		}
		v[vlen]=0;

		delete []theString;
		theString=v;
		stringLen=vlen;
	}
}

//       
void CMagicString::Dump(int length, unsigned char* data, int per_line)
{
	if (length)
	{
		if (theString)
		{
			delete []theString;
			theString=NULL;
		}
		stringLen=0;

		CMagicString ret=GetReturn();
		int ret_len=ret.GetLength();
		int len=length*3+1;
		if (per_line>0)
			len+=(length/per_line)*ret_len;
		theString=new char[len];
		int p[2];
		for (int i=0;i<length;i++)
		{
			unsigned int s=(unsigned char)data[i];
			p[0]=s/16;
			p[1]=s%16;
			for (int j=0;j<2;j++)
			{
				int pp=p[j];
				if (pp>=10)
				{
					theString[stringLen]=pp-10+'A';
				}
				else
				{
					theString[stringLen]=pp+'0';
				}
				stringLen++;
			}

			theString[stringLen]=' ';
			stringLen++;

			if (per_line>0)
			{
				if ((i+1) % per_line==0)
				{
					//     
					for (int j=0;j<ret_len;j++)
					{
						theString[stringLen]=ret[j];
						stringLen++;
					}
				}
			}
		}
		theString[stringLen]=0;
	}
	else
		*this="";
}

// 
void CMagicString::Format(const char* type, int data)
{
	char s[30];
	int len=sprintf(s,type,data);
	s[len]=0;
	*this=s;
}

void CMagicString::Format(const char* type, float data)
{
	char s[30];
	int len=sprintf(s,type,data);
	s[len]=0;
	*this=s;
}

void CMagicString::Format(const char* type, double data)
{
	char s[30];
	int len=sprintf(s,type,data);
	s[len]=0;
	*this=s;
}

void CMagicString::Format(const char* type, char data)
{
	char s[30];
	int len=sprintf(s,type,data);
	s[len]=0;
	*this=s;
}

void CMagicString::Format(const char* type, unsigned int data)
{
	char s[30];
	int len=sprintf(s,type,data);
	s[len]=0;
	*this=s;
}


//   
CMagicString CMagicString::Left(int len) const
{
	return substr(0,len);
}

//   
CMagicString CMagicString::Right(int len) const
{
	return substr(length()-len,len);
}

//  
CMagicString CMagicString::Mid(int pos, int count) const
{
	return substr(pos, count);
}

//          (  )
CMagicString CMagicString::GetPathOnly()
{
	int len=length();
	for (int i=len-1;i>=0;i--)
	{
		char s=theString[i];
		if (s=='/' || s=='\\')
		{
			//    
			return Left(i);
		}
	}
	return "";
}

//       
CMagicString CMagicString::GetFileNameOnly()
{
	int i;
	for (i=GetLength()-1;i>=0 && (theString[i]!='\\' && theString[i]!='/' && theString[i]!=VIRTUAL_DRIVE_SYMBOL);i--);
	return Right(GetLength()-i-1);
}

//        
void CMagicString::Trim()
{
	int i;
	int beginning=0;
	for (i=0;i<stringLen;i++)
	{
		char s=theString[i];
		if (s!=' ' && s!='\t')
			break;
		beginning++;
	}

	int end=stringLen-1;
	for (i=stringLen-1;i>=0;i--)
	{
		char s=theString[i];
		if (s!=' ' && s!='\t')
			break;
		end--;
	}

	if (beginning>0 || end<stringLen-1)
	{
		//   
		*this=Mid(beginning, end-beginning+1);
	}
}

//      (    )
int CMagicString::GetParamCount()
{
	for (int i=0;true;i++)
	{
		CMagicString param=GetParam(i);
		if (param.IsEmpty())
			return i;
	}
}

//      (    )
CMagicString CMagicString::GetParam(int index)
{
	int count=0;
	CMagicString t=*this;
	t.Trim();
	int len=t.GetLength();
	bool is_param=true;
	bool is_quote=false;
	CMagicString param;
	for (int i=0;i<len;i++)
	{
		char s=t[i];
		if (s==' ' || s=='\t')
		{
			//  
			if (is_quote)
			{
				//    
				if (count==index)
					param+=s;
			}
			else
				is_param=false;
		}
		else
		{
			//  
			if (!is_param)
			{
				is_param=true;
				count++;
				if (s=='"')
				{
					//    
					is_quote=true;
					continue;
				}
			}
			else
			{
				if (s=='"')
				{
					is_quote=false;
					continue;
				}
			}

			//  
			if (count==index)
				param+=s;
		}
	}

	return param;
}

//    2    
bool CMagicString::Divide(const CMagicString& divider, CMagicString& str1, CMagicString& str2)
{
	int pos=Find(divider);
	if (pos!=-1)
	{
		//  
		str1=Mid(0, pos);

		int len=divider.GetLength();
		int start=pos+len;
		int len2=GetLength();
		str2=Mid(start, len2-start);

		return true;
	}
	return false;
}

//   
CMagicString CMagicString::GetExtension()
{
	int len=GetLength();
	CMagicString ext="";
	for (int i=len-1;i>=0;i--)
	{
		char simbol=theString[i];
		if (simbol=='.')
		{
			//    c 
			int len2=ext.GetLength();
			CMagicString vext="";
			for (int j=len2-1;j>=0;j--)
				vext=vext+ext[j];
			return vext;
		}
		ext=ext+simbol;
	}
	return "";
}

//       ::
bool CMagicString::DeleteNamespace()
{
	for (int i=stringLen-1;i>=0;i--)
	{
		if (theString[i]==':')
		{
			if (i>0)
			{
				if (theString[i-1]==':')
				{
					*this=Right(stringLen-(i+1));
					return true;
				}
				else
					return false;
			}
		}
	}
	return false;
}

//       
void CMagicString::SaveText(FILE* file)
{
	fwrite(theString, 1, stringLen, file);
}

//   
CMagicString return_str="\r\n";
CMagicString CMagicString::GetReturn()
{
	return return_str;
}

//   
void CMagicString::SetReturn(const char* s)
{
	return_str=s;
}

//      nIndex   nCount
int CMagicString::Delete(int nIndex, int nCount)
{
	int i;
	int len=stringLen-nCount;
	char* v=new char[len+1];
	for (i=0;i<nIndex;i++)
		v[i]=theString[i];
	for (i=nIndex;i<len;i++)
		v[i]=theString[i+nCount];
	delete []theString;
	theString=v;
	stringLen=len;
	theString[len]=0;
	return len;
}



//     ,   
void CMagicString::DeleteLastSlash()
{
	while(stringLen)
	{
		int index=stringLen-1;
		 char s=theString[index];
		 if (s=='\\' || s=='/')
			 *this=Left(index);
		 else
			 break;
	}
}

//   ,      
bool CMagicString::IsVariableName()
{
	if (stringLen>0)
	{
		for (int i=0;i<stringLen;i++)
		{
			char ss=theString[i];
			if (!((ss>='0' && ss<='9') || (ss>='a' && ss<='z') || (ss>='A' && ss<='Z') || ss=='_'))
				return false;
		}
		return true;
	}
	return false;
}

// ------------------------------------------------------------------------

//     

CMagicStream::CMagicStream()
{
	bytes=0;
	mode=MAGIC_STREAM_READ;
	name="";
}

CMagicStream::~CMagicStream()
{
}

bool CMagicStream::IsStoring()
{
	return (mode!=MAGIC_STREAM_READ);
}

bool CMagicStream::IsLoading()
{
	return (mode==MAGIC_STREAM_READ);
}

//  
void CMagicStream::operator<<(bool& value)
{
	char* v=(char*)(&value);
	Write(v,1);
}

void CMagicStream::operator<<(char& value)
{
	char* v=(char*)(&value);
	Write(v,1);
}

void CMagicStream::operator<<(unsigned char& value)
{
	char* v=(char*)(&value);
	Write(v,1);
}

void CMagicStream::operator<<(CMagicString& value)
{
	int len=value.GetLength();
	if (len>=0xff)
	{
		unsigned char vlen=0xff;
		Write((char*)&vlen,1);
		unsigned short vlen2=len;
		(*this)<<vlen2;
	}
	else
	{
		unsigned char vlen=(unsigned char)len;
		Write((char*)&vlen,1);
	}

	if (len)
	{
		char* v=(char*)(const char*)value;
		Write(v,len);
	}
}

/*
void CMagicStream::operator<<(long& value)
{
	long v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);
}
*/

void CMagicStream::operator<<(int& value)
{
	int v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);
}

void CMagicStream::operator<<(short& value)
{
	short v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,2);
}

void CMagicStream::operator<<(unsigned short& value)
{
	short v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,2);
}

void CMagicStream::operator<<(float& value)
{
	float v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);
}

void CMagicStream::operator<<(double& value)
{
	double v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,8);
}

void CMagicStream::operator<<(unsigned int& value)
{
	unsigned int v2=value;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);
}

void CMagicStream::operator<<(CMagicSize& value)
{
	int v2=value.cx;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);

	v2=value.cy;
	INVERT_BYTES(v2);
	v=(char*)(&v2);
	Write(v,4);
}

void CMagicStream::operator<<(CMagicPoint& value)
{
	int v2=value.x;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);

	v2=value.y;
	INVERT_BYTES(v2);
	v=(char*)(&v2);
	Write(v,4);
}

void CMagicStream::operator<<(CMagicRect& value)
{
	int v2=value.left;
	INVERT_BYTES(v2);
	char* v=(char*)(&v2);
	Write(v,4);

	v2=value.top;
	INVERT_BYTES(v2);
	v=(char*)(&v2);
	Write(v,4);

	v2=value.right;
	INVERT_BYTES(v2);
	v=(char*)(&v2);
	Write(v,4);

	v2=value.bottom;
	INVERT_BYTES(v2);
	v=(char*)(&v2);
	Write(v,4);
}


//  
void CMagicStream::operator>>(bool& value)
{
	char* v=(char*)(&value);
	Read(v,1);
}

void CMagicStream::operator>>(char& value)
{
	char* v=(char*)(&value);
	Read(v,1);
}

void CMagicStream::operator>>(unsigned char& value)
{
	char* v=(char*)(&value);
	Read(v,1);
}

void CMagicStream::operator>>(CMagicString& value)
{
	int len;
	unsigned char vlen;
	Read((char*)&vlen,1);
	if (vlen==0xff)
	{
		unsigned short vlen2;
		(*this)>>vlen2;
		len=vlen2;
	}
	else
		len=vlen;

	if (len)
	{
		value.SetLength(len);
		char* m=(char*)value.c_str();
		Read(m,len);
		m[len]=0;
	}
	else
		value="";
}

/*
void CMagicStream::operator>>(long& value)
{
	char* v=(char*)(&value);
	Read(v,4);
	INVERT_BYTES(value);
}
*/

void CMagicStream::operator>>(int& value)
{
	char* v=(char*)(&value);
	Read(v,4);
	INVERT_BYTES(value);
}

void CMagicStream::operator>>(short& value)
{
	char* v=(char*)(&value);
	Read(v,2);
	INVERT_BYTES(value);
}

void CMagicStream::operator>>(unsigned short& value)
{
	char* v=(char*)(&value);
	Read(v,2);
	INVERT_BYTES(value);
}

void CMagicStream::operator>>(float& value)
{
	char* v=(char*)(&value);
	Read(v,4);
	INVERT_BYTES(value);
}

void CMagicStream::operator>>(double& value)
{
	char* v=(char*)(&value);
	Read(v,8);
	INVERT_BYTES(value);
}

void CMagicStream::operator>>(unsigned int& value)
{
	char* v=(char*)(&value);
	Read(v,4);
	INVERT_BYTES(value);
}

void CMagicStream::operator>>(CMagicSize& value)
{
	char* v=(char*)(&value.cx);
	Read(v,4);
	INVERT_BYTES(value.cx);
	v=(char*)(&value.cy);
	Read(v,4);
	INVERT_BYTES(value.cy);
}

void CMagicStream::operator>>(CMagicPoint& value)
{
	char* v=(char*)(&value.x);
	Read(v,4);
	INVERT_BYTES(value.x);
	v=(char*)(&value.y);
	Read(v,4);
	INVERT_BYTES(value.y);
}

void CMagicStream::operator>>(CMagicRect& value)
{
	char* v=(char*)(&value.left);
	Read(v,4);
	INVERT_BYTES(value.left);
	v=(char*)(&value.top);
	Read(v,4);
	INVERT_BYTES(value.top);
	v=(char*)(&value.right);
	Read(v,4);
	INVERT_BYTES(value.right);
	v=(char*)(&value.bottom);
	Read(v,4);
	INVERT_BYTES(value.bottom);
}

//   
void CMagicStream::SetMode(int mode)
{
	if (mode!=this->mode && mode>=MAGIC_STREAM_READ && mode<=MAGIC_STREAM_ADD)
		ChangeMode(mode);
}


// ---------------------------------------------------------

CMagicStreamFile::CMagicStreamFile() : CMagicStream()
{
	stream=NULL;
}

CMagicStreamFile::~CMagicStreamFile()
{
	if (stream)
	{
		fclose(stream);
		stream=NULL;
	}
}


void CMagicStreamFile::Close()
{
	if (stream)
	{
		fclose(stream);
		stream=NULL;
		name="";
	}
}

//    
bool CMagicStreamFile::Open(const char* file, int mode)
{
	name=file;
	
	ChangeMode(mode);

	if (!stream)
	{
		this->mode=MAGIC_STREAM_READ;
		name="";
	}

	return (stream!=NULL);
}

//   
unsigned int CMagicStreamFile::GetLength()
{
	unsigned int len=0;
	if (stream)
	{
		unsigned int pos=ftell(stream);
		fseek(stream,0,SEEK_END);
		len=ftell(stream);
		fseek(stream,pos,SEEK_SET);
	}
	return len;
}

//   
unsigned int CMagicStreamFile::GetPosition()
{
	unsigned int pos=0;
	if (stream)
		pos=ftell(stream);
	return pos;
}

//   
void CMagicStreamFile::SetPosition(unsigned int pos)
{
	if (stream && IsLoading())
	{
		fseek(stream,pos,SEEK_SET);
	}
}

unsigned int CMagicStreamFile::Write(void* m_data,unsigned int k_data)
{
	bytes=0;
	if (stream && IsStoring())
		bytes=fwrite(m_data,1,k_data,stream);

	return bytes;
}

unsigned int CMagicStreamFile::Read(void* m_data,unsigned int k_data)
{
	bytes=0;
	if (stream && IsLoading())
		bytes=fread(m_data,1,k_data,stream);

	return bytes;
}

//   
void CMagicStreamFile::ChangeMode(int mode)
{
	CMagicString vname=name;
	Close();
	name=vname;

	CMagicString smode;
	switch (mode)
	{
	case MAGIC_STREAM_READ:
		smode="rb";
		break;
	case MAGIC_STREAM_WRITE:
		smode="wb";
		break;

	case MAGIC_STREAM_ADD:
		smode="ab";
		break;

	default:
		return;
	}

	this->mode=mode;

	if (GSurface::device && GSurface::device->platform)
		stream=GSurface::device->platform->OpenFile(name, smode);
	else
		stream=fopen_magic(name, smode.c_str());
}


// -----------------------------------------------------------

CMagicStreamMemory::CMagicStreamMemory() : CMagicStream()
{
	buffer=NULL;
	position=0;
	length=0;
	max_length=0;
	is_created_buffer=false;
}

CMagicStreamMemory::~CMagicStreamMemory()
{
	if (buffer)
	{
		if (is_created_buffer)
			free(buffer);
		buffer=NULL;
		position=0;
		length=0;
		max_length=0;
	}
}

void CMagicStreamMemory::Close()
{
	if (buffer)
	{
		if (is_created_buffer)
			free(buffer);
		buffer=NULL;
		position=0;
		length=0;

		max_length=MAGIC_STREAM_MEMORY_START_SIZE;
		buffer=(char*)malloc(max_length);
		is_created_buffer=true;
	}
}

//    
bool CMagicStreamMemory::Open(const char* buffer, unsigned int len, int mode)
{
	Close();

	if (mode==MAGIC_STREAM_ADD && ((!buffer) || len<=0))
		mode=MAGIC_STREAM_WRITE;

	this->mode=mode;

	switch (mode)
	{
	case MAGIC_STREAM_READ:
		this->buffer=(char*)buffer;
		length=len;
		max_length=len;
		is_created_buffer=false;
		break;

	case MAGIC_STREAM_WRITE:
		if (buffer || len>0)
			return false;
		max_length=MAGIC_STREAM_MEMORY_START_SIZE;
		this->buffer=(char*)malloc(max_length);
		is_created_buffer=true;
		break;

	case MAGIC_STREAM_ADD:
		if ((!buffer) || len<=0)
		{
			//   
			max_length=MAGIC_STREAM_MEMORY_START_SIZE;
			this->buffer=(char*)malloc(max_length);
		}
		else
		{
			//     
			max_length=MAGIC_STREAM_MEMORY_START_SIZE+len;
			this->buffer=(char*)malloc(max_length);
			for (unsigned int i=0;i<len;i++)
				this->buffer[i]=buffer[i];
			length=len;
			position=len;
		}
		is_created_buffer=true;
		break;

	default:
		return false;
	}

	return true;
}


//   
unsigned int CMagicStreamMemory::GetLength()
{
	if (buffer)
		return length;
	return 0;
}

//   
unsigned int CMagicStreamMemory::GetPosition()
{
	if (buffer)
		return position;
	return 0;
}

//   
void CMagicStreamMemory::SetPosition(unsigned int pos)
{
	if (buffer && IsLoading())
	{
		if (length && pos>length)
			pos=length;
		position=pos;
	}
}

unsigned int CMagicStreamMemory::Write(void* m_data,unsigned int k_data)
{
	bytes=0;
	if (buffer && IsStoring())
	{
		unsigned int new_position=position+k_data;
		if (new_position>max_length)
		{
			//  
			max_length=new_position+new_position/4;
			buffer=(char*)realloc(buffer, max_length);
		}

		char* d=(char*)m_data;

		for (unsigned int i=0;i<k_data;i++)
		{
			buffer[position]=d[i];
			position++;
		}

		if (position>length)
			length=position;
	}

	return bytes;
}

unsigned int CMagicStreamMemory::Read(void* m_data,unsigned int k_data)
{
	bytes=0;
	if (buffer && IsLoading())
	{
		unsigned int new_position=position+k_data;
		bytes=k_data;
		if (length && new_position>length)
			bytes=length-position;

		char* d=(char*)m_data;

		for (unsigned int i=0;i<bytes;i++)
		{
			d[i]=buffer[position];
			position++;
		}
	}

	return bytes;
}

//   
void CMagicStreamMemory::ChangeMode(int mode)
{
	switch (mode)
	{
	case MAGIC_STREAM_READ:
		position=0;
		break;

	case MAGIC_STREAM_WRITE:
		Close();
		max_length=MAGIC_STREAM_MEMORY_START_SIZE;
		buffer=(char*)malloc(max_length);
		is_created_buffer=true;
		break;

	case MAGIC_STREAM_ADD:
		position=length;
		break;
	}

	this->mode=mode;
}



CText::CText()
{
	data=NULL;
	length=0;
	pos=0;
}

CText::~CText()
{
	Clear();
}

void CText::Clear()
{
	if (data)
	{
		delete []data;
		data=NULL;
	}
	length=0;
	pos=0;
}

void CText::Create(int len)
{
	Clear();

	if (len>0)
	{
		data=new char[len];
		length=len;
	}
}

//     
bool CText::ReadString(CMagicString& str)
{
	if (!str.IsEmpty())
		str="";
	MissEmptyStrings();
	if (pos<length)
	{
		for (;pos<length;pos++)
		{
			char s=data[pos];
			if (s=='\r')
			{
				//  
				pos++;
				if (pos<length)
				{
					s=data[pos];
					if (s=='\n')
					{
						pos++;
					}
				}
				break;
			}
			else
			{
				if (s=='\n')
				{
					pos++;
					break;
				}
			}
			str+=s;
		}

		return true;
	}
	return false;
}

//    
char CText::ReadOne()
{
	char sym=0;
	if (pos<length)
	{
		sym=data[pos];
		pos++;
	}
	return sym;
}

//     C++
bool CText::ReadWordC(CMagicString* str)
{
	if (pos<length)
	{
		bool is=false;
		char s;
		do 
		{
			s=data[pos];
			pos++;
			if (s!=' ' && s!='\t' && s!='\r' && s!='\n')
			{
				is=true;
				break;
			}
		} 
		while (pos<length);

		if (is)
		{
			//   
			*str=s;
			if ((s>='0' && s<='9') || s=='-' || s=='+')
			{
				//  
				for (;pos<length;pos++)
				{
					s=data[pos];
					if ((s>='a' && s<='z') || (s>='A' && s<='Z') || (s>='0' && s<='9') || s=='_' || s=='.')
						(*str)+=s;
					else
						break;
				}
				return true;
			}
			else
			{
				if (s=='"' || s=='\'')
				{
					//     
					//      
					char quote=s;

					for (;pos<length;pos++)
					{
						s=data[pos];
						(*str)+=s;
						if (s==quote)
						{
							pos++;
							break;
						}
					}
					return true;
				}
				else
				{
					if ((s>='a' && s<='z') || (s>='A' && s<='Z') || s=='_' || s=='#')
					{
						//  
						for (;pos<length;pos++)
						{
							s=data[pos];
							if ((s>='a' && s<='z') || (s>='A' && s<='Z') || (s>='0' && s<='9') || s=='_')
								(*str)+=s;
							else
							{
								if (s==':')
								{
									if (pos+1<length)
									{
										s=data[pos+1];
										if (s==':')
										{
											(*str)+="::";
											pos++;
											continue;
										}
									}
									break;
								}
								else
									break;
							}
						}
						return true;
					}
					else
					{
						//  - .  
						return true;
					}
				}
			}
		}
		else
		{
			if (!str->IsEmpty())
				*str="";
		}
	}
	else
	{
		if (!str->IsEmpty())
			*str="";
	}
	return false;
}

//   
void CText::MissEmptyStrings()
{
	if (pos<length)
	{
		for (;pos<length;pos++)
		{
			char s=data[pos];
			switch(s)
			{
				case '\r':
				case '\n':
					//pos++;
					break;
				default:
					return;
			}
		}
	}
}

//  
void CText::RemoveRemarks()
{
	if (length)
	{
		char* vdata=new char[length];
		int vlen=0;

		bool is_rem=false;

		for (int i=0;i<length;i++)
		{
			char s=data[i];

			if (s=='/')
			{
				if (i<length-1)
				{
					s=data[i+1];
					if (s=='/')
					{
						//   
						//    
						is_rem=true;
						for (i=i+2;i<length;i++)
						{
							char s=data[i];
							if (s=='\r')
							{
								if (i<length-1)
								{
									s=data[i+1];
									if (s=='\n')
										i++;
								}
								break;
							}
						}
					}
					else
					{
						if (s=='*')
						{
							//   
							is_rem=true;
							for (i=i+2;i<length;i++)
							{
								char s=data[i];
								if (s=='*')
								{
									if (i<length-1)
									{
										s=data[i+1];
										if (s=='/')
										{
											i++;
											break;
										}
									}
								}
							}
						}
					}
				}
			}

			if (is_rem)
				is_rem=false;
			else
			{
				vdata[vlen]=s;
				vlen++;
			}
		}

		length=vlen;
		delete []data;
		data=vdata;
	}
}

//     #if, #ifdef, #ifndef
bool CText::ReadIF(CMagicString* str)
{
	if (ReadWordC(str))
	{
		if ((*str)=="#if")
		{
			ReadString(*str);
			str->Trim();
			return true;
		}
		else
		{
			if ((*str)=="#ifdef" || (*str)=="#ifndef")
			{
				CMagicString s;
				ReadString(s);
				str->Trim();
				if (str->Find(' ')==-1)
				{
					if (str->Find('\t')==-1)
					{
						if ((*str)=="#ifdef")
							(*str)="defined(";
						else
							(*str)="!defined(";
						(*str)+=s;
						(*str)+=')';
						return true;
					}
				}
			}
		}
	}
	return false;
}

//  
void CText::FromString(CMagicString* str)
{
	Clear();
	if (!str->IsEmpty())
	{
		length=str->GetLength();
		data=new char[length];
		for (int i=0;i<length;i++)
			data[i]=(*str)[i];
	}
}

//       /*......*/
void CText::ToRemark(int start, int length)
{
	if (length>=4)
	{
		data[start]='/';
		data[start+1]='*';
		data[start+length-2]='*';
		data[start+length-1]='/';
	}
}

// --------------------------------

CMagicStringArray::CMagicStringArray()
{
	k_str=0;
	max_str=0;
	m_str=NULL;
}

CMagicStringArray::~CMagicStringArray()
{
	Clear();
}

void CMagicStringArray::Clear()
{
	for (int i=0;i<k_str;i++)
	{
		delete m_str[i];
		m_str[i]=NULL;
	}

	if (m_str)
	{
		delete []m_str;
		m_str=NULL;
	}

	k_str=max_str=0;
}

void CMagicStringArray::Serialize(CMagicStream& ar)
{
	if (ar.IsStoring())
	{
		ar<<max_str;
		ar<<k_str;
		for (int i=0;i<k_str;i++)
			ar<<(*(m_str[i]));
	}
	else
	{
		Clear();

		ar>>max_str;
		if (max_str)
		{
			m_str=new CMagicString*[max_str];
			ar>>k_str;
			for (int i=0;i<k_str;i++)
			{
				m_str[i]=new CMagicString;
				ar>>(*(m_str[i]));
			}
		}
	}
}

int CMagicStringArray::Add(CMagicString* str)
{
	if (k_str==max_str)
	{
		//    
		int step=max_str/4;
		if (step<10)
			step=10;

		int max=max_str+step;
		CMagicString** vm_str=new CMagicString*[max];
		for (int i=0;i<max_str;i++)
			vm_str[i]=m_str[i];
		max_str=max;
		if (m_str)
			delete []m_str;
		m_str=vm_str;
	}

	m_str[k_str]=new CMagicString;
	*(m_str[k_str])=*str;
	k_str++;
	return k_str-1;
}

CMagicString* CMagicStringArray::Get(int index)
{
	if (index>=0 && index<k_str)
		return m_str[index];
	return NULL;
}

//          
void CMagicStringArray::FromString(const char* str)
{
	Clear();

	CMagicString from=str;
	from.Trim();

	if (!from.IsEmpty())
	{
		CMagicString ss;
		int len=from.GetLength();
		for (int i=0;i<len;i++)
		{
			char s=from[i];
			if (s==' ' || s=='\t')
			{
				if (!ss.IsEmpty())
				{
					Add(&ss);
					ss="";
				}
			}
			else
				ss+=s;
		}
		if (!ss.IsEmpty())
			Add(&ss);
	}
}

// ----------------------------------------------------------------------

// ,      
CRC::CRC()
{
	length=0;
	for (int i=0;i<MAX_CRC;i++)
		crc[i]=0;
}

//      
void CRC::Create(unsigned char* m_data, unsigned int k_data)
{
	length=k_data;
	crc[0]=0;
	for (unsigned int j=0;j<k_data;j++)
	{
		unsigned int data=m_data[j];
		crc[0]+=data;
	}

	for (int i=1;i<MAX_CRC;i++)
		CreateCRCWithStep(m_data,k_data,i);
}

//     int
int CRC::ToInt()
{
	int result=length;
	for (int i=0;i<MAX_CRC;i++)
	{
		result+=crc[i]*(i+2)-crc[i]*(i+3);
	}
	return result;
}

//      
void CRC::CreateCRCWithStep(unsigned char* m_data, int unsigned k_data, int step)
{
	unsigned int* crc_ptr=&(crc[step]);
	*crc_ptr=0;
	unsigned int step2=step*2;

	unsigned int discret=k_data/step2;
	unsigned int ost=k_data%step2;
	if (ost)
		discret++;
	unsigned int vk_data=discret*step2;
	unsigned char* vm_data=new unsigned char[vk_data];

	for (unsigned int t=0;t<vk_data;t++)
		vm_data[t]=0;
	memcpy(vm_data,m_data,k_data);

	for (unsigned int i=0;i<vk_data;i+=step2)
	{
		unsigned char* vvm_data=&(vm_data[i]);
		int k;
		for (k=0;k<step;k++)
		{
			unsigned int data=vvm_data[k];
			(*crc_ptr)+=data;
		}

		vvm_data=&(vm_data[i+step]);
		for (k=0;k<step;k++)
		{
			unsigned int data=vvm_data[k];
			(*crc_ptr)-=data;
		}
	}

	delete []vm_data;
}

// 
void CRC::Serialize(CMagicStream& ar)
{
	if (ar.IsLoading())
	{
		int i;
		length=0;
		for (i=0;i<MAX_CRC;i++)
			crc[i]=0;

		int max=0;
		ar>>max;
		ar>>length;
		for (i=0;i<MAX_CRC;i++)
			ar>>crc[i];

		int ostatok=max-MAX_CRC;
		unsigned int v=0;
		for (i=0;i<ostatok;i++)
			ar>>v;
	}
	else
	{
		int max=MAX_CRC;
		ar<<max;
		ar<<length;
		for (int i=0;i<MAX_CRC;i++)
			ar<<crc[i];
	}
}

//  
bool CRC::operator==(const CRC& crc)
{
	bool ok=false;
	if (length==crc.length)
	{
		ok=true;
		for (int i=0;i<MAX_CRC;i++)
		{
			if (this->crc[i]!=crc.crc[i])
			{
				ok=false;
				break;
			}
		}
	}
	return ok;
}

//  
bool CRC::operator!=(const CRC& crc)
{
	return (!((*this)==crc));
}

//  
CRC& CRC::operator=(const CRC& crc)
{
	length=crc.length;
	for (int i=0;i<MAX_CRC;i++)
		this->crc[i]=crc.crc[i];
	return *this;
}

// ------------------------------------------------------

CArrayBase::CArrayBase()
{
	max_existing=0;
	k_existing=0;
	m_existing=NULL;

	max_deleted=0;
	k_deleted=0;
	m_deleted=NULL;

	max_indexed=0;
	m_indexed=NULL;
}

CArrayBase::~CArrayBase()
{
	ClearExistingAndDeleted();
}

void CArrayBase::ClearExistingAndDeleted()
{
	max_existing=k_existing=0;
	if (m_existing)
	{
		delete []m_existing;
		m_existing=NULL;
	}

	max_deleted=k_deleted=0;
	if (m_deleted)
	{
		delete []m_deleted;
		m_deleted=NULL;
	}

	max_indexed=0;
	if (m_indexed)
	{
		delete []m_indexed;
		m_indexed=0;
	}
}

void CArrayBase::IncreaseDeleted(int from, int to)
{
	int count=to-from+1;
	for (int i=0;i<count;i++)
		AddToDeleted(from+i);
}

void CArrayBase::AddToExisting(int index)
{
	int i;

	if (max_indexed<=index)
	{
		//    
		int index1=index+1;
		int max_indexed2=index1+index1/4;
		int* vm_indexed=new int[max_indexed2];
		for (i=0;i<max_indexed;i++)
			vm_indexed[i]=m_indexed[i];
		for (i=max_indexed;i<max_indexed2;i++)
			vm_indexed[i]=-1;
		if (m_indexed)
			delete []m_indexed;
		m_indexed=vm_indexed;
		max_indexed=max_indexed2;
	}
	m_indexed[index]=k_existing;

	if (max_existing==k_existing)
	{
		//     
		int step=max_existing/4;
		if (step<4)
			step=4;
		int vmax_existing=max_existing+step;
		int* vm_existing=new int[vmax_existing];
		for (i=0;i<max_existing;i++)
			vm_existing[i]=m_existing[i];
		if (m_existing)
			delete []m_existing;
		m_existing=vm_existing;
		max_existing=vmax_existing;
	}
	m_existing[k_existing]=index;
	k_existing++;
}

void CArrayBase::AddToDeleted(int index)
{
	if (max_indexed>index)
	{
		//   
		m_indexed[index]=-1;
	}

	if (max_deleted==k_deleted)
	{
		//   
		int step=max_deleted/4;
		if (step<4)
			step=4;
		int vmax_deleted=max_deleted+step;
		int* vm_deleted=new int[vmax_deleted];
		int i;
		for (i=0;i<max_deleted;i++)
			vm_deleted[i]=m_deleted[i];
		if (m_deleted)
			delete []m_deleted;
		m_deleted=vm_deleted;
		max_deleted=vmax_deleted;
	}
	m_deleted[k_deleted]=index;
	k_deleted++;
}

void CArrayBase::FromExistingToDeleted(int index)
{
	assert(max_indexed>index);
	int from=m_indexed[index];
	assert(from>=0);
	m_indexed[index]=-1;
	k_existing--;
	if (from!=k_existing)
	{
		m_existing[from]=m_existing[k_existing];
		m_indexed[k_existing]=from;
	}

	/*
	int i;
	int from=-1;
	for (i=0;i<k_existing;i++)
	{
		if (m_existing[i]==index)
		{
			from=i;
			k_existing--;
			m_existing[from]=m_existing[k_existing];
			break;
		}
	}
	*/
	AddToDeleted(index);
}

int CArrayBase::FromDeletedToExisting()
{
	int index=-1;
	if (k_deleted)
	{
		k_deleted--;
		index=m_deleted[k_deleted];
		AddToExisting(index);
	}
	return index;
}

int CArrayBase::FromDeletedToExisting(int deleted_index)
{
	int index=-1;
	for (int i=0;i<k_deleted;i++)
	{
		if (m_deleted[i]==deleted_index)
		{
			m_deleted[i]=m_deleted[k_deleted-1];
			k_deleted--;
			AddToExisting(deleted_index);
			index=deleted_index;
			break;
		}
	}
	return index;
}

void CArrayBase::Serialize(CMagicStream& stream)
{
	//int i;
	if (stream.IsStoring())
	{
		stream<<max_existing;
		stream<<k_existing;
		if (k_existing)
			stream.Write(m_existing, k_existing*sizeof(int));
		//for (i=0;i<k_existing;i++)
		//	stream<<m_existing[i];

		stream<<max_deleted;
		stream<<k_deleted;
		if (k_deleted)
			stream.Write(m_deleted, k_deleted*sizeof(int));
		//for (i=0;i<k_deleted;i++)
		//	stream<<m_deleted[i];

		stream<<max_indexed;
		if (max_indexed)
			stream.Write(m_indexed, max_indexed*sizeof(int));
		//for (i=0;i<max_indexed;i++)
		//	stream<<m_indexed[i];
	}
	else
	{
		ClearExistingAndDeleted();

		stream>>max_existing;
		stream>>k_existing;
		if (max_existing)
		{
			m_existing=new int[max_existing];
			if (k_existing)
				stream.Read(m_existing, k_existing*sizeof(int));
			//for (i=0;i<k_existing;i++)
			//	stream>>m_existing[i];
		}

		stream>>max_deleted;
		stream>>k_deleted;
		if (max_deleted)
		{
			m_deleted=new int[max_deleted];
			if (k_deleted)
				stream.Read(m_deleted, k_deleted*sizeof(int));
			//for (i=0;i<k_deleted;i++)
			//	stream>>m_deleted[i];
		}

		stream>>max_indexed;
		if (max_indexed)
		{
			m_indexed=new int[max_indexed];
			stream.Read(m_indexed, max_indexed*sizeof(int));
			//for (i=0;i<max_indexed;i++)
			//	stream>>m_indexed[i];
		}
	}
}

void CArrayBase::SaveToFile(FILE* file)
{
	fwrite(&max_existing, sizeof(max_existing), 1, file);
	fwrite(&k_existing, sizeof(k_existing), 1, file);

	if (k_existing>0)
		fwrite(m_existing, sizeof(int), k_existing, file);

	fwrite(&max_deleted, sizeof(max_deleted), 1, file);
	fwrite(&k_deleted, sizeof(k_deleted), 1, file);

	if (k_deleted>0)
		fwrite(m_deleted, sizeof(int), k_deleted, file);

	fwrite(&max_indexed, sizeof(max_indexed), 1, file);
	if (max_indexed)
		fwrite(m_indexed, sizeof(int), max_indexed, file);
}

void CArrayBase::LoadFromFile(FILE* file)
{
	fread(&max_existing, sizeof(max_existing), 1, file);
	fread(&k_existing, sizeof(k_existing), 1, file);

	if (max_existing>0)
	{
		m_existing=new int[max_existing];
		fread(m_existing, sizeof(int), k_existing, file);
	}

	fread(&max_deleted, sizeof(max_deleted), 1, file);
	fread(&k_deleted, sizeof(k_deleted), 1, file);

	if (max_deleted>0)
	{
		m_deleted=new int[max_deleted];
		fread(m_deleted, sizeof(int), k_deleted, file);
	}

	fread(&max_indexed, sizeof(max_indexed), 1, file);
	if (max_indexed>0)
	{
		m_indexed=new int[max_indexed];
		fread(m_indexed, sizeof(int), max_indexed, file);
	}
}

// --------------------------------------------------------------------------

CArrayInt::CArrayInt() : CArrayBase()
{
	k_int=0;
	m_int=NULL;
}

CArrayInt::~CArrayInt()
{
	Clear();
}

void CArrayInt::Clear()
{
	if (m_int)
	{
		free(m_int);
		m_int=NULL;
	}

	k_int=0;

	ClearExistingAndDeleted();
}

//     
int CArrayInt::Get(int index)
{
	if (index>=0 && index<k_int)
		return m_int[index];
	return NULL;
}

//  
int CArrayInt::Add(int data)
{
	int index=FromDeletedToExisting();

	if (index==-1)
	{
		int size=sizeof(int);
		int step=k_int/4;
		if (step<4)
			step=4;
		int k_int2=k_int+step;
		m_int=(int*)realloc(m_int,size*(k_int2));

		for (int i=k_int;i<k_int2;i++)
			m_int[i]=-1;

		IncreaseDeleted(k_int, k_int2-1);
		index=FromDeletedToExisting();

		k_int=k_int2;
	}

	m_int[index]=data;

	return index;
}

//    
bool CArrayInt::Delete(int index)
{
	if (index>=0 && index<k_int)
	{
		if (m_int[index])
		{
			m_int[index]=-1;
			FromExistingToDeleted(index);

			return true;
		}
	}
	return false;
}

void CArrayInt::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		CArrayBase::Serialize(stream);

		stream<<k_int;
		if (k_int>0)
			stream.Write(m_int, sizeof(int)*k_int);
	}
	else
	{
		Clear();

		CArrayBase::Serialize(stream);

		stream>>k_int;
		if (k_int>0)
		{
			m_int=(int*)malloc(k_int*sizeof(int));
			stream.Read(m_int, sizeof(int)*k_int);
		}
	}
}

void CArrayInt::SaveToFile(FILE* file)
{
	CArrayBase::SaveToFile(file);
	fwrite(&k_int, sizeof(k_int), 1, file);
	if (k_int>0)
		fwrite(m_int, sizeof(int), k_int, file);
}

void CArrayInt::LoadFromFile(FILE* file)
{
	Clear();

	CArrayBase::LoadFromFile(file);

	fread(&k_int, sizeof(k_int), 1, file);
	if (k_int>0)
	{
		m_int=(int*)malloc(k_int*sizeof(int));
		fread(m_int, sizeof(int), k_int, file);
	}
}

// --------------------------------------------------------------------------

CArrayPtr::CArrayPtr() : CArrayBase()
{
	k_ptr=0;
	m_ptr=NULL;
}

CArrayPtr::~CArrayPtr()
{
	Clear();
}

void CArrayPtr::Clear()
{
	if (m_ptr)
	{
		free(m_ptr);
		m_ptr=NULL;
	}

	k_ptr=0;

	ClearExistingAndDeleted();
}

//     
void* CArrayPtr::Get(int index)
{
	if (index>=0 && index<k_ptr)
		return m_ptr[index];
	return NULL;
}

//  
int CArrayPtr::Add(void* data)
{
	int index=FromDeletedToExisting();

	if (index==-1)
	{
		int size=sizeof(void*);
		int step=k_ptr/4;
		if (step<4)
			step=4;
		int k_ptr2=k_ptr+step;
		m_ptr=(void**)realloc(m_ptr,size*(k_ptr2));

		for (int i=k_ptr;i<k_ptr2;i++)
			m_ptr[i]=NULL;

		IncreaseDeleted(k_ptr, k_ptr2-1);
		index=FromDeletedToExisting();

		k_ptr=k_ptr2;
	}

	m_ptr[index]=data;

	return index;
}

//    
bool CArrayPtr::Delete(int index)
{
	if (index>=0 && index<k_ptr)
	{
		if (m_ptr[index])
		{
			m_ptr[index]=NULL;
			FromExistingToDeleted(index);

			return true;
		}
	}
	return false;
}

//    
int CArrayPtr::FindIndex(void* ptr)
{
	int index=-1;

	for (int i=0;i<k_existing;i++)
	{
		int ind=m_existing[i];
		if (m_ptr[ind]==ptr)
		{
			index=ind;
			break;
		}
	}

	return index;
}

void CArrayPtr::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		CArrayBase::Serialize(stream);

		stream<<k_ptr;
		if (k_ptr>0)
			stream.Write(m_ptr, sizeof(void*)*k_ptr);
	}
	else
	{
		Clear();

		CArrayBase::Serialize(stream);

		stream>>k_ptr;
		if (k_ptr>0)
		{
			m_ptr=(void**)malloc(k_ptr*sizeof(void*));
			stream.Read(m_ptr, sizeof(void*)*k_ptr);
		}
	}
}

void CArrayPtr::SaveToFile(FILE* file)
{
	CArrayBase::SaveToFile(file);
	fwrite(&k_ptr, sizeof(k_ptr), 1, file);
	if (k_ptr>0)
		fwrite(m_ptr, sizeof(void*), k_ptr, file);
}

void CArrayPtr::LoadFromFile(FILE* file)
{
	Clear();

	CArrayBase::LoadFromFile(file);

	fread(&k_ptr, sizeof(k_ptr), 1, file);
	if (k_ptr>0)
	{
		m_ptr=(void**)malloc(k_ptr*sizeof(void*));
		fread(m_ptr, sizeof(void*), k_ptr, file);
	}
}


//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

CMagicTime::CMagicTime()
{
	version=0;

	year=1975;
	month=4;
	day=14;
	hour=13;
	minute=40;
	second=0;
}

void CMagicTime::Serialize(CMagicStream& stream)
{
	if (stream.IsStoring())
	{
		stream<<version;

		stream<<year;
		stream<<month;
		stream<<day;
		stream<<hour;
		stream<<minute;
		stream<<second;
	}
	else
	{
		stream>>version;

		stream>>year;
		stream>>month;
		stream>>day;
		stream>>hour;
		stream>>minute;
		stream>>second;
	}

	version=0;
}

CMagicString month_time[12]={"January",  "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
int order_month_day_time=0;	//    

//     
void CMagicTime::Format(MAGIC_TIME_ENUM flags, CMagicString& result)
{
	if (!order_month_day_time)
	{
		//   
		Localizer& loc=Localizer::GetInstance();
		for (int i=0;i<12;i++)
		{
			CMagicString v=loc.Translate(month_time[i]);
			if (v.GetLength()>1)
				month_time[i]=v;
		}

		order_month_day_time=1; //  
		CMagicString order=loc.Translate("First Month than Day");
		if (order=="No" || order=="no")
		{
			//  
			order_month_day_time=2;
		}
	}

	result="";
	CMagicString str0='0';
	CMagicString d;
	if (flags & MAGIC_TIME_DAY)
	{
		//    
		d.IntToString(day);
		if (d.GetLength()==1)
			d=str0+d;

		if (order_month_day_time)
		{
			//  
			result+=d;
			result+=' ';
			result+=month_time[month];
		}
		else
		{
			//  
			result+=month_time[month];
			result+=' ';
			result+=d;
		}
	}

	if (flags & MAGIC_TIME_YEAR)
	{
		if (!result.IsEmpty())
			result+=' ';
		//  
		d.IntToString(year);
		result+=d;
	}

	if (flags & MAGIC_TIME_MINUTE)
	{
		if (!result.IsEmpty())
			result+=" ,";
		//  
		d.IntToString(hour);
		if (d.GetLength()==1)
			d=str0+d;
		result+=d;
		result+=':';
		//  
		d.IntToString(minute);
		if (d.GetLength()==1)
			d=str0+d;
		result+=d;
	}

	if (flags & MAGIC_TIME_SECOND)
	{
		if (!result.IsEmpty())
			result+=" ,";
		//  
		d.IntToString(second);
		if (d.GetLength()==1)
			d=str0+d;
		result+=d;
	}
}

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

CMagicStringData::CMagicStringData() : CMagicString()
{
	data.length=0;
	data.data=NULL;
	index=-1;
}

CMagicStringData::~CMagicStringData()
{
	if (data.data)
	{
		delete []data.data;
		data.data=NULL;
		data.length=0;
	}
}

void CMagicStringData::SetData(int length, unsigned char* data)
{
	if (this->data.length!=length)
	{
		if (this->data.data)
		{
			delete []this->data.data;
			this->data.data=NULL;
			this->data.length=0;
		}

		if (length)
		{
			this->data.length=length;
			this->data.data=new unsigned char[length];
		}
	}
	memcpy(this->data.data, data, length);
}

void CMagicStringData::Serialize(CMagicStream& stream)
{
	CMagicString* ms=this;
	if (stream.IsStoring())
	{
		stream<<*ms;
		stream<<data.length;
		if (data.length>0)
			stream.Write(data.data, data.length);
	}
	else
	{
		if (data.data)
		{
			delete []data.data;
			data.data=NULL;
			data.length=0;
		}

		stream>>*ms;

		stream>>data.length;
		if (data.length>0)
		{
			data.data=new unsigned char[data.length];
			stream.Read(data.data, data.length);
		}
	}
}

// --------------------------------

CMagicStringDataArray::CMagicStringDataArray()
{
	k_str=0;
	max_str=0;
	m_str=NULL;
}

CMagicStringDataArray::~CMagicStringDataArray()
{
	Clear();
}

void CMagicStringDataArray::Clear()
{
	for (int i=0;i<k_str;i++)
	{
		delete m_str[i];
		m_str[i]=NULL;
	}

	if (m_str)
	{
		delete []m_str;
		m_str=NULL;
	}

	k_str=max_str=0;
}

void CMagicStringDataArray::Serialize(CMagicStream& ar)
{
	if (ar.IsStoring())
	{
		ar<<max_str;
		ar<<k_str;
		for (int i=0;i<k_str;i++)
			m_str[i]->Serialize(ar);
	}
	else
	{
		Clear();

		ar>>max_str;
		if (max_str)
		{
			m_str=new CMagicStringData*[max_str];
			ar>>k_str;
			for (int i=0;i<k_str;i++)
			{
				m_str[i]=new CMagicStringData;
				m_str[i]->Serialize(ar);
			}
		}
	}
}

int CMagicStringDataArray::Add(CMagicString* str, MEM_DATA* data)
{
	if (k_str==max_str)
	{
		//    
		int step=max_str/4;
		if (step<10)
			step=10;

		int max=max_str+step;
		CMagicStringData** vm_str=new CMagicStringData*[max];
		for (int i=0;i<max_str;i++)
			vm_str[i]=m_str[i];
		max_str=max;
		if (m_str)
			delete []m_str;
		m_str=vm_str;
	}

	CMagicStringData* md=new CMagicStringData;
	m_str[k_str]=md;

	CMagicString* vmd=md;

	*vmd=*str;
	if (data)
		md->SetData(data->length, data->data);

	k_str++;
	return k_str-1;
}

int CMagicStringDataArray::Delete(int index)
{
	if (index>=0 && index<k_str)
	{
		int index_fast=m_str[index]->GetIndex();

		delete m_str[index];

		for (int i=index+1;i<k_str;i++)
			m_str[i-1]=m_str[i];

		k_str--;
		m_str[k_str]=NULL;

		return index_fast;
	}
	return -1;
}

CMagicStringData* CMagicStringDataArray::Get(int index)
{
	if (index>=0 && index<k_str)
		return m_str[index];
	return NULL;
}

// --------------------------------

__STRING_TABLE_3::__STRING_TABLE_3()
{
	k_data=0;
	m_data=NULL;
}

__STRING_TABLE_3::~__STRING_TABLE_3()
{
	Clear();
}

void __STRING_TABLE_3::Clear()
{
	for (int i=0;i<k_data;i++)
	{
		if (m_data[i])
		{
			delete m_data[i];
			m_data[i]=NULL;
		}
	}

	if (m_data)
	{
		delete []m_data;
		m_data=NULL;
		k_data=0;
	}
}

void __STRING_TABLE_3::SetLength(int len)
{
	if (k_data<len)
	{
		CMagicStringDataArray** vm_data=new CMagicStringDataArray*[len];
		for (int i=0;i<k_data;i++)
			vm_data[i]=m_data[i];
		for (int i=k_data;i<len;i++)
			vm_data[i]=NULL;
		delete []m_data;
		m_data=vm_data;
		k_data=len;
	}
}

// -----------

__STRING_TABLE_2::__STRING_TABLE_2()
{
	k_data=0;
	m_data=NULL;
}

__STRING_TABLE_2::~__STRING_TABLE_2()
{
	Clear();
}

void __STRING_TABLE_2::Clear()
{
	for (int i=0;i<k_data;i++)
	{
		if (m_data[i])
		{
			delete m_data[i];
			m_data[i]=NULL;
		}
	}

	if (m_data)
	{
		delete []m_data;
		m_data=NULL;
		k_data=0;
	}
}

void __STRING_TABLE_2::SetLength(int len)
{
	if (k_data<len)
	{
		__STRING_TABLE_3** vm_data=new __STRING_TABLE_3*[len];
		for (int i=0;i<k_data;i++)
			vm_data[i]=m_data[i];
		for (int i=k_data;i<len;i++)
			vm_data[i]=NULL;
		delete []m_data;
		m_data=vm_data;
		k_data=len;
	}
}

// -----------

__STRING_TABLE_1::__STRING_TABLE_1()
{
	k_data=0;
	m_data=NULL;
}

__STRING_TABLE_1::~__STRING_TABLE_1()
{
	Clear();
}

void __STRING_TABLE_1::Clear()
{
	for (int i=0;i<k_data;i++)
	{
		if (m_data[i])
		{
			delete m_data[i];
			m_data[i]=NULL;
		}
	}

	if (m_data)
	{
		delete []m_data;
		m_data=NULL;
		k_data=0;
	}
}

void __STRING_TABLE_1::SetLength(int len)
{
	if (k_data<len)
	{
		__STRING_TABLE_2** vm_data=new __STRING_TABLE_2*[len];
		for (int i=0;i<k_data;i++)
			vm_data[i]=m_data[i];
		for (int i=k_data;i<len;i++)
			vm_data[i]=NULL;
		delete []m_data;
		m_data=vm_data;
		k_data=len;
	}
}

// --------------------------------

CMagicStringTable::CMagicStringTable()
{
	max_ptr=0;
	k_ptr=0;
	m_ptr=NULL;
}

CMagicStringTable::~CMagicStringTable()
{
	Clear();
}

void CMagicStringTable::Clear()
{
	table.Clear();
	if (m_ptr)
	{
		delete []m_ptr;
		m_ptr=NULL;
	}
	max_ptr=0;
	k_ptr=0;
}

//   
CMagicStringData* CMagicStringTable::Find(CMagicString& name)
{
	int first, middle, last;
	if (GetSymbols(name, first, middle, last))
	{
		if (table.k_data>first)
		{
			__STRING_TABLE_2* table2=table.m_data[first];
			if (table2)
			{
				if (table2->k_data>last)
				{
					__STRING_TABLE_3* table3=table2->m_data[last];
					if (table3)
					{
						if (table3->k_data>middle)
						{
							CMagicStringDataArray* str_array=table3->m_data[middle];
							if (str_array)
							{
								int k=str_array->GetCount();
								for (int i=0;i<k;i++)
								{
									CMagicStringData* str=str_array->Get(i);
									if (name==*str)
										return str;
								}
							}
						}
					}
				}
			}
		}
	}
	return NULL;
}

//  
bool CMagicStringTable::Add(CMagicString& name, MEM_DATA* data)
{
	if (!Find(name))
	{
		int first, middle, last;
		if (GetSymbols(name, first, middle, last))
		{
			if (table.k_data<=first)
				table.SetLength(first+1);
			__STRING_TABLE_2* table2=table.m_data[first];
			if (!table2)
			{
				table.m_data[first]=new __STRING_TABLE_2;
				table2=table.m_data[first];
			}
			if (table2->k_data<=last)
				table2->SetLength(last+1);
			__STRING_TABLE_3* table3=table2->m_data[last];
			if (!table3)
			{
				table2->m_data[last]=new __STRING_TABLE_3;
				table3=table2->m_data[last];
			}
			if (table3->k_data<=middle)
				table3->SetLength(middle+1);
			CMagicStringDataArray* str_array=table3->m_data[middle];
			if (!str_array)
			{
				str_array=new CMagicStringDataArray;
				table3->m_data[middle]=str_array;
			}
			int k=str_array->GetCount();
			for (int i=0;i<k;i++)
			{
				CMagicString* str=str_array->Get(i);
				if (name==*str)
				{
					//    
					return false;
				}
			}
			int index=str_array->Add(&name, data);
			CMagicStringData* added_str=str_array->Get(index);

			if (max_ptr==k_ptr)
			{
				//   
				int step=max_ptr/4;
				if (step<4)
					step=4;
				max_ptr+=step;
				CMagicStringData** vm_ptr=new CMagicStringData*[max_ptr];
				if (m_ptr)
				{
					for (int i=0;i<k_ptr;i++)
						vm_ptr[i]=m_ptr[i];

					delete []m_ptr;
				}
				m_ptr=vm_ptr;
			}
			m_ptr[k_ptr]=added_str;
			added_str->SetIndex(k_ptr);
			k_ptr++;
			return true;
		}
	}
	return false;
}

//  
bool CMagicStringTable::Delete(CMagicString& name)
{
	if (Find(name))
	{
		int first, middle, last;
		if (GetSymbols(name, first, middle, last))
		{
			__STRING_TABLE_2* table2=table.m_data[first];
			__STRING_TABLE_3* table3=table2->m_data[last];
			CMagicStringDataArray* str_array=table3->m_data[middle];
			int k=str_array->GetCount();
			for (int i=0;i<k;i++)
			{
				CMagicStringData* str=str_array->Get(i);
				if (name==*str)
				{
					int index=str_array->Delete(i);
					k_ptr--;
					if (index!=k_ptr)
					{
						m_ptr[index]=m_ptr[k_ptr];
						m_ptr[index]->SetIndex(index);
					}
					break;
				}
			}
		}

		return true;
	}
	return false;
}

bool CMagicStringTable::GetSymbols(CMagicString& name, int& first, int& middle, int& last)
{
	int len=name.GetLength();
	if (len>0)
	{
		unsigned char v=name[0];
		first=v;
		v=name[len/2];
		middle=v;
		v=name[len-1];
		last=v;
		return true;
	}
	first=middle=last=-1;
	return false;
}

void CMagicStringTable::Serialize(CMagicStream& stream, bool count_yes)
{
	if (stream.IsStoring())
	{
		if (count_yes)
			stream<<k_ptr;
		for (int i=0;i<k_ptr;i++)
			m_ptr[i]->Serialize(stream);
	}
	else
	{
		Clear();

		CMagicStringData d;
		if (count_yes)
		{
			stream>>k_ptr;
			for (int i=0;i<k_ptr;i++)
			{
				d.Serialize(stream);
				Add(d, d.GetData());
			}
		}
		else
		{
			//     
			unsigned int length=stream.GetLength();
			unsigned int pos=0;
			while (pos<length)
			{
				d.Serialize(stream);
				Add(d, d.GetData());
				pos=stream.GetPosition();
			}
		}
	}
}

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

//   "ConfigFile"

// 
ConfigFile::ConfigFile()
{
	s=new char[3000];
	k_file=0; 		//   
	m_file=new char*[MAX_FILE];
	dl_file=new int[MAX_FILE];
	pos_file=new int[MAX_FILE];
	for (int i=0;i<MAX_FILE;i++)
   	{
		m_file[i]=NULL;
		dl_file[i]=0;
		pos_file[i]=0;
    }
}

// 
ConfigFile::~ConfigFile()
{
	delete []s;
	if (m_file)
   	{
		for (int i=0;i<k_file;i++)
			if (m_file[i])
      			delete []m_file[i];
		delete []m_file;
    }

	if (dl_file)
	{
		delete []dl_file;
		dl_file=NULL;
	}

	if (pos_file)
	{
		delete []pos_file;
		pos_file=NULL;
	}
}

//   
void ConfigFile::OpenFile(CMagicString name)
{
	//  
	FILE* file=GSurface::device->platform->OpenFile(name.c_str(), "rb");
	if (!file) return;
	//    
	int block;
	do
   	{
		block=fread (s, sizeof(char), 32, file);
		dl_file[k_file]=dl_file[k_file]+block; //  
    }
	while (block==32);

	m_file[k_file]=new char[dl_file[k_file]];
	fseek(file,0,SEEK_SET);
	fread(m_file[k_file],sizeof(char),dl_file[k_file],file); //  
	k_file++;
	fclose(file);
}

//     
void ConfigFile::OpenFileDir(CMagicString mask)
{
	GPlatform* platform=GSurface::device->platform;
	CMagicString ext=mask.Right(3);
	mask=mask.GetPathOnly();
	char slash=platform->GetSlash();
	const char* file=platform->GetFirstFile(mask, ext);
	while (file)
	{
		if (mask.IsEmpty())
		{
			OpenFile(file);
		}
		else
		{
			CMagicString path=mask+slash+file;
			OpenFile(path);
		}
		file=platform->GetNextFile();
	}

	/*
	WIN32_FIND_DATA fd;
	HANDLE hFindFile=FindFirstFile(mask.c_str(),&fd);
	mask=mask.GetPathOnly();
	if (hFindFile!=INVALID_HANDLE_VALUE)
		do
        	OpenFile(mask+fd.cFileName);
		while (FindNextFile(hFindFile,&fd));
	FindClose(hFindFile);
	*/
}

//    
void ConfigFile::OpenMemory(char* data,int len)
{
	dl_file[k_file]=len;
	m_file[k_file]=new char[len];
	for (int i=0;i<len;i++)
	{
		m_file[k_file][i]=data[i];
	}

	k_file++;
}

//     
bool ConfigFile::ReadChar(int nom_file,char& data)
{
	if (dl_file[nom_file]<=pos_file[nom_file]) return false; //  
	data=m_file[nom_file][pos_file[nom_file]];
	pos_file[nom_file]++; //  
	if (data==';')
   	{
		//  ,  
		do
      	{
			if (dl_file[nom_file]<=pos_file[nom_file]) return false; //  
			data=m_file[nom_file][pos_file[nom_file]];
   			pos_file[nom_file]++; //  
        }
		while (data!=0xd);
    }
	return true;
}

//       ,    
int ConfigFile::FindString(int nom_file,char* text,int pos)
{
	pos_file[nom_file]=pos;
    int len=strlen(text);
    int nom=0;
    do
    {
		char data;
		if (!ReadChar (nom_file, data)) //  
        	return -1; //   
        if (text[nom]==data)
        {
			nom++;
            if (nom==len) return pos_file[nom_file];
        }
        else nom=0;
	}
    while (true);
}

//            -
//   '['
int ConfigFile::FindStringToBracket(int nom_file,char* text,int pos)
{
	pos_file[nom_file]=pos;
    int len=strlen(text);
    int nom=0;
    do
    {
		char data;
		if (!ReadChar (nom_file, data)) //  
         	return -1; //   
        if (data=='[')
         	return -1; //   
        if (text[nom]==data)
        {
			nom++;
            if (nom==len)
            	return pos_file[nom_file];
        }
        else nom=0;
	}
    while (true);
}

//         
// ( text   )
int ConfigFile::FindLabel(int nom_file,char* text,int pos)
{
	char data;
	pos_file[nom_file]=pos;
	text[0]=0;
	//    '['
	do
		if (!ReadChar(nom_file,data))
			return -1; //  
	while (data!='[');

	int nom=0;
	do
   	{
		if (!ReadChar(nom_file,data))
   		{
			text[0]=0; // 
			return -1; //   
        }
		if (data==']')
      	{
			text[nom]=0;
			return pos_file[nom_file]; //  
        }
		text[nom]=data;
		nom++;
		if (nom>250)
      		nom=0;
	}
	while (true);
}

//           
//      
bool ConfigFile::ReadToEnter(int nom_file, char* text)
{
	char data;
	text[0]=0;
	//     
	do
   		if (!ReadChar(nom_file,data))
			return false; //  
	while (data==' ' || data==9);
	pos_file[nom_file]--; //    ,  

	int pos=0;
	bool fl=true;
	do
   	{
		if (!ReadChar(nom_file,data))//  
   			fl=false; //  
        else
         	if (data==0xd)
            	fl=false; //  
            else
           	{
			    text[pos]=data;
   				pos++;
            }
   	}
	while (fl);

	text[pos]=0;

	//     
	if (pos)
   	{
   		for (pos--;text[pos]==' ' || text[pos]==9;pos--);
			text[pos+1]=0;
    }
    else return false;
	return true;
}

//         
CMagicString ConfigFile::FindParametrFile(int nom_file,CMagicString param)
{
	s[0]=0;
	int pos=FindString (nom_file,(char*)param.c_str(),0);
	if (pos!=-1)
		ReadToEnter(nom_file,s);
	return s;
}

//            
//  '['    
CMagicString ConfigFile::FindParametrToLabel(int nom_file,CMagicString param,int pos)
{
	s[0]=0;
	int pos2=FindStringToBracket (nom_file,(char*)param.c_str(),pos);
	if (pos2!=-1)
		ReadToEnter(nom_file,s);
	return s;
}

//        
CMagicString ConfigFile::FindParametrDir(CMagicString param)
{
	CMagicString value;
	for (int i=0;i<k_file;i++)
   	{
   		value=FindParametrFile(i,param);
		if (value!="") return value;
    }
	return value;
}

//    
int ConfigFile::GetCountFile()
{
	return k_file;
}

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

//   "Config"

// 
Config::Config()
{
	s=new char[256];
	k_file=0; 		//   
	m_file=new char*[MAX_FILE];
	dl_file=new int[MAX_FILE];
	pos_file=new int[MAX_FILE];
	for (int i=0;i<MAX_FILE;i++)
   	{
		m_file[i]=NULL;
		dl_file[i]=0;
		pos_file[i]=0;
    }
}

// 
Config::~Config()
{
	delete []s;
	if (m_file)
   	{
		for (int i=0;i<k_file;i++)
			if (m_file[i])
      			delete []m_file[i];
		delete []m_file;
    }
	if (dl_file) delete []dl_file;
	if (pos_file) delete []pos_file;
}

//   
void Config::OpenFile(const char* name)
{
	GPlatform* platform=GSurface::device->platform;
	CMagicStream* stream=platform->OpenStream(name);
	if (stream)
	{
		ReadStream(*stream);
		delete stream;
	}
}

//   
void Config::ReadStream(CMagicStream& stream)
{
	int pos=stream.GetPosition();
	int len=stream.GetLength();
	int length=len-pos;
	m_file[k_file]=new char[length];
	dl_file[k_file]=length;
	stream.Read(m_file[k_file], length);
	k_file++;
}

/*
//     
void Config::OpenFileDir(CMagicString mask)
{
	WIN32_FIND_DATA fd;
	HANDLE hFindFile=FindFirstFile(mask.c_str(),&fd);
	if (((int)hFindFile)!=-1)
		do
        	OpenFile(fd.cFileName);
		while (FindNextFile(hFindFile,&fd));
	FindClose(hFindFile);
}
*/

//     
bool Config::ReadChar(int nom_file, char& data)
{
	if (dl_file[nom_file]<=pos_file[nom_file]) return false; //  
	data=m_file[nom_file][pos_file[nom_file]];
	pos_file[nom_file]++; //  
	if (data==';')
   	{
		//  ,  
		do
      	{
			if (dl_file[nom_file]<=pos_file[nom_file]) return false; //  
			data=m_file[nom_file][pos_file[nom_file]];
   			pos_file[nom_file]++; //  
        }
		while (data!=0xd);
    }
	return true;
}

//       ,    
int Config::FindString(int nom_file, const char* text,int pos)
{
	pos_file[nom_file]=pos;
    int len=strlen(text);
    int nom=0;
    char data;
    do
    {
		if (!ReadChar (nom_file, data)) //  
        	return -1; //   
        if (text[nom]==data)
        {
			nom++;
            if (nom==len) return pos_file[nom_file];
        }
        else nom=0;
	}
    while (true);
}

//            -
//   '['
int Config::FindStringToBracket(int nom_file, const char* text,int pos)
{
	pos_file[nom_file]=pos;
	int len=strlen(text);
	int nom=0;
	char data;
	do
	{
		if (!ReadChar (nom_file, data)) //  
			return -1; //   
		if (data=='[')
			return -1; //   
		if (text[nom]==data)
		{
			nom++;
            if (nom==len)
            	return pos_file[nom_file];
            }
         	else nom=0;
		}
	while (true);
}

//         
// ( text   )
int Config::FindLabel(int nom_file, CMagicString& text, int pos)
{
	char data;
	pos_file[nom_file]=pos;
	text[0]=0;
	//    '['
	do
		if (!ReadChar(nom_file,data))
			return -1; //  
	while (data!='[');

	int nom=0;
	do
   	{
		if (!ReadChar(nom_file,data))
		{
			text[0]=0; // 
			return -1; //   
		}
		if (data==']')
      	{
			text[nom]=0;
			return pos_file[nom_file]; //  
		}
		text[nom]=data;
		nom++;
		if (nom>250)
			nom=0;
	}
	while (true);
}

//           
//      
bool Config::ReadToEnter(int nom_file, char* text)
{
	char data;
	text[0]=0;
	//     
	do
   		if (!ReadChar(nom_file,data))
			return false; //  
	while (data==' ' || data==9);
	pos_file[nom_file]--; //    ,  

	int pos=0;
	bool fl=true;
	do
   	{
		if (!ReadChar(nom_file,data))//  
   			fl=false; //  
        else
         	if (data!='\r')
			{
				if (data=='\n')
					fl=false; //  
				else
				{
					text[pos]=data;
   						pos++;
				}
			}
	}
	while (fl);

	text[pos]=0;

	//     
	if (pos)
   	{
		for (pos--;text[pos]==' ' || text[pos]==9; pos--);
			text[pos+1]=0;
    }
    else
		return false;
	return true;
}

//         
CMagicString Config::FindParametrFile(int nom_file, CMagicString param)
{
	s[0]=0;
	int pos=FindString (nom_file,(char*)param.c_str(),0);
	if (pos!=-1)
		ReadToEnter(nom_file, s);

	//return s;
	return TranslateParam();
}

//        
CMagicString Config::TranslateParam()
{
	CMagicString param=s;
	int i;
	char* str=(char*)param.c_str();
	if (param!="" && str[0]=='=')
	{
		//    
		for (i=1;str[i]==' ' || str[i]=='\t';i++);
			param=param.substr(i,param.length()-i);
	}
	return param;
}

//            
//  '['    
CMagicString Config::FindParametrToLabel(int nom_file, CMagicString param, int pos)
{
	s[0]=0;
	int pos2=FindStringToBracket (nom_file,(char*)param.c_str(),pos);
	if (pos2!=-1)
		ReadToEnter(nom_file,s);
	return s;
}

//        
CMagicString Config::FindParametrDir(CMagicString param)
{
	CMagicString value;
	for (int i=0;i<k_file;i++)
   	{
		value=FindParametrFile(i,param);
		if (value!="") return value;
	}
	return value;
}

//    
int Config::GetCountFile()
{
	return k_file;
}

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

//  "LangAccelerator3"

LangAccelerator3::LangAccelerator3(char simbol_base)
{
	simbol=simbol_base;
	m_label=NULL;//new Met[1];
	k_label=0;
}

LangAccelerator3::~LangAccelerator3()
{
	if (m_label)
	{
		delete []m_label;
		m_label=NULL;
	}
}

//   
void LangAccelerator3::AddLabel(CMagicString label,CMagicString value)
{
	int index=FindLabel(label);
	if (index=-1)
	{
		//  
		Met* vm_label=new Met[k_label+1];
		for (int i=0;i<k_label;i++)
			vm_label[i]=m_label[i];
		if (m_label)
		{
			delete []m_label;
			m_label=NULL;
		}
		m_label=vm_label;

		m_label[k_label].label=label;
		m_label[k_label].phrase=value;

		k_label++;
	}
}

//    
const char* LangAccelerator3::GetLabel(CMagicString& label)
{
	int index=FindLabel(label);
	if (index!=-1)
	{
		return m_label[index].phrase.c_str();
	}
	else
	{
		//   
		return NULL;
	}
}

//  
int LangAccelerator3::FindLabel(CMagicString& label)
{
	for (int i=0;i<k_label;i++)
	{
		if (label==m_label[i].label)
			return i;
	}
	return -1;
}

//  "LangAccelerator2"
LangAccelerator2::LangAccelerator2(char simbol_base)
{
	simbol=simbol_base;
	k_index=0;
	m_index=NULL;
}

LangAccelerator2::~LangAccelerator2()
{
	if (m_index)
	{
		for (int i=0;i<k_index;i++)
			delete m_index[i];
		delete []m_index;
	}
}

//   
void LangAccelerator2::AddLabel(CMagicString label,CMagicString value)
{
	int index=-1;
	unsigned char search_simbol=(unsigned char)label[label.length()/2];
	for (int i=0;i<k_index;i++)
	{
		if (search_simbol==m_index[i]->GetSimbol())
		{
			index=i;
			break;
		}
	}

	if (index==-1)
	{
		//     
		LangAccelerator3** vm_index=new LangAccelerator3*[k_index+1];
		for (int i=0;i<k_index;i++)
			vm_index[i]=m_index[i];
		if (m_index)
		{
			delete []m_index;
			m_index=NULL;
		}
		m_index=vm_index;

		m_index[k_index]=new LangAccelerator3(search_simbol);
		m_index[k_index]->AddLabel(label,value);
		
		k_index++;

	}
	else
	{
		//      
		m_index[index]->AddLabel(label,value);
	}
}

//    
const char* LangAccelerator2::GetLabel(CMagicString& label)
{
	int index=-1;
	unsigned char search_simbol=(unsigned char)label[label.length()/2];
	for (int i=0;i<k_index;i++)
	{
		if (search_simbol==m_index[i]->GetSimbol())
		{
			index=i;
			break;
		}
	}

	if (index!=-1)
	{
		return m_index[index]->GetLabel(label);
	}
	else
	{
		return NULL;
	}
}


//  "LangAccelerator1"
LangAccelerator1::LangAccelerator1()
{
	k_index=0;
	m_index=0;
}

LangAccelerator1::~LangAccelerator1()
{
	if (m_index)
	{
		for (int i=0;i<k_index;i++)
			delete m_index[i];
		delete []m_index;
	}
}

//   
void LangAccelerator1::AddLabel(CMagicString label,CMagicString value)
{
	int index=-1;
	unsigned char search_simbol=(unsigned char)label[label.length()-1];
	for (int i=0;i<k_index;i++)
	{
		if (search_simbol==m_index[i]->GetSimbol())
		{
			index=i;
			break;
		}
	}

	if (index==-1)
	{
		//     
		LangAccelerator2** vm_index=new LangAccelerator2*[k_index+1];
		for (int i=0;i<k_index;i++)
			vm_index[i]=m_index[i];
		if (m_index)
		{
			delete []m_index;
			m_index=NULL;
		}
		m_index=vm_index;

		m_index[k_index]=new LangAccelerator2(search_simbol);
		m_index[k_index]->AddLabel(label,value);
		
		k_index++;

	}
	else
	{
		//      
		m_index[index]->AddLabel(label,value);
	}
}

//    
const char* LangAccelerator1::GetLabel(CMagicString& label)
{
	int index=-1;
	unsigned char search_simbol=(unsigned char)label[label.length()-1];
	for (int i=0;i<k_index;i++)
	{
		if (search_simbol==m_index[i]->GetSimbol())
		{
			index=i;
			break;
		}
	}

	if (index!=-1)
	{
		return m_index[index]->GetLabel(label);
	}
	else
	{
		return NULL;
	}
}


//   "Language"

// 
Language::Language(CMagicString file,CMagicString lang)
{
	Init1();

	if (file.Find('*')!=-1)
		conf->OpenFileDir(file);
	else
		conf->OpenFile(file);
	Init2(lang);
}

// 
Language::Language(char* data,int length,CMagicString lang)
{
	Init1();
	conf->OpenMemory(data,length);
	Init2(lang);
}

//  
void Language::Init1()
{
	int i;
	s=new char[256];
	k_lang=0; //   
	m_lang=new LangProperty[MAX_LANGUAGE]; //    
	conf=NULL;

	//    
	language.name="ENGLISH"; //  
	language.install_name="English"; //    
	language.font="Times New Roman"; //   
	language.italic=false; //   
	language.weight=300; //    0  900   100
	language.width=6;
	language.height=14;
	language.color=0xffffff; //  
	language.label="ru"; //    

	for (i=0;i<256;i++)
		data[i]=NULL; //    ,   

	//  
	conf=new ConfigFile;
}

//  
void Language::Init2(CMagicString lang)
{
	int i;
	//   
	GetLanguageDir();

	//    
	default_lang=conf->FindParametrDir("LANG_DEFAULT=");
	if (lang=="")
		lang=default_lang;

	//    
	default_phrase=conf->FindParametrDir("TEXT_DEFAULT=");
	if (default_phrase=="")
   		default_phrase="?";
	
	//   
	GetLabelDir(lang);

	//     
	for (i=0;i<k_lang;i++)
   	{
		if (lang==m_lang[i].name)
      	{
			language.name=m_lang[i].name;
			language.install_name=m_lang[i].install_name;
   			language.font=m_lang[i].font;
   			language.italic=m_lang[i].italic;
   			language.weight=m_lang[i].weight;
			language.width=m_lang[i].width;
			language.height=m_lang[i].height;
   			language.color=m_lang[i].color;
   			language.label=m_lang[i].label;
        }
	}

	delete conf;
	conf=NULL;

	delete []s;
	s=NULL;
}


// 
Language::~Language()
{
	if (s)
	{
		delete []s;
		s=NULL;
	}
	if (m_lang)
	{
		delete []m_lang;
		m_lang=NULL;
	}
	
	if (conf)
	{
		delete conf;
		conf=NULL;
	}

	for (int i=0;i<256;i++)
		if (data[i])
		{
			delete data[i];
			data[i]=NULL;
		}
}

//  
const char* Language::Label(const char* label,bool indexed)
{
	const char* result=NULL;
	unsigned char simbol_search=(unsigned char)label[0];
	if (data[simbol_search])
	{
		//     
		CMagicString str=label;
		result=data[simbol_search]->GetLabel(str);
	}

	if ((!result) && (!indexed))
	{
		//     
		result=default_phrase.c_str();
	}
	return result;
}

//       
void Language::FindLanguageStream(int nom_file)
{
	int pos=0;
	do
   	{
   		pos=conf->FindString(nom_file,"[LANGUAGE]",pos);
		if (pos==-1) return;
		//  
		m_lang[k_lang].name=conf->FindParametrToLabel(nom_file,"NAME=",pos);
		if (m_lang[k_lang].name!="")
      	{
			//  
			m_lang[k_lang].install_name=conf->FindParametrToLabel(nom_file,"INSTALL_NAME=",pos);
      		if (m_lang[k_lang].install_name=="")
         		m_lang[k_lang].install_name=m_lang[k_lang].name;

			// 
			m_lang[k_lang].font=conf->FindParametrToLabel(nom_file,"FONT=",pos);
			if (m_lang[k_lang].font=="")
         		m_lang[k_lang].font="Times New Roman Cyr";

			//  
			CMagicString v=conf->FindParametrToLabel(nom_file,"WIDTH=",pos);
			if (v=="")
         		m_lang[k_lang].width=6;
            else
            {
				int v2;
				v2=atoi(v.c_str());
				m_lang[k_lang].width=v2;
            }

			//  
			v=conf->FindParametrToLabel(nom_file,"HEIGHT=",pos);
			if (v=="")
         		m_lang[k_lang].height=16;
            else
            {
				int v2;
				v2=atoi(v.c_str());
				m_lang[k_lang].height=v2;
            }

			//  
			v=conf->FindParametrToLabel(nom_file,"ITALIC=",pos);
			if (v=="YES")
         		m_lang[k_lang].italic=true;
            else
            	m_lang[k_lang].italic=false;

			//  
			v=conf->FindParametrToLabel(nom_file,"WEIGHT=",pos);
			if (v=="")
         		m_lang[k_lang].weight=0;
            else
            {
				int v2;
				v2=atoi(v.c_str());
				v2=v2/100;
				v2=v2*100;
				m_lang[k_lang].weight=v2;
            }

			//  
			v=conf->FindParametrToLabel(nom_file,"COLOR=",pos);
			if (v=="")
         		m_lang[k_lang].color=0xffffff;
            else
            {
				int v2=atoi(v.c_str());
				if (v2==0)
               		v2=0xffffff;
				m_lang[k_lang].color=v2;
            }

			// 
      		m_lang[k_lang].label=conf->FindParametrToLabel(nom_file,"LABEL=",pos);
			if (m_lang[k_lang].label!="")
         	{
				m_lang[k_lang].label=m_lang[k_lang].label+":";
         		k_lang++;
            }
		}
	}
	while (true);
}

//       *.lng
void Language::GetLanguageDir()
{
	for (int i=0;i<conf->GetCountFile();i++)
   	{
		FindLanguageStream(i);
    }
}

//           ,  -
//       
void Language::GetLabelFromStream(int nom_file,CMagicString pref,CMagicString prefd)
{
	int pos=0;
	do
   	{
		pos=conf->FindLabel(nom_file,s,pos);
		if (pos==-1) return;
		CMagicString met(s);
		//   ,  ,    
		CMagicString phrase=conf->FindParametrToLabel(nom_file,pref,pos);
		if (phrase=="")
		{
			//       
			phrase=conf->FindParametrToLabel(nom_file,prefd,pos);
		}

		if (phrase!="")
		{
			//    
			unsigned char simbol_search=(unsigned char) met[0];
			if (!(data[simbol_search]))
				data[simbol_search]=new LangAccelerator1;

			data[simbol_search]->AddLabel(met,phrase);
		}
	}
	while (true);
}

//        *.lng     
//   
void Language::GetLabelDir(CMagicString lang)
{
	CMagicString pref=GetNameLabel(lang);
	CMagicString prefd=GetNameLabel(default_lang);
	for (int i=0;i<conf->GetCountFile();i++)
		GetLabelFromStream(i,pref,prefd);
}

//      
CMagicString Language::GetNameLabel(CMagicString lang)
{
	for (int i=0;i<k_lang;i++)
   	{
		if (lang==m_lang[i].name) return m_lang[i].label;
    }
	return "NOT_LABEL";
}

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

//  Localizer

// 
Localizer::Localizer()
{
	name=new CMagicString;
	*name="";
	lang=NULL;
	result=new CMagicString;
	*result="";
}

// 
Localizer::~Localizer()
{
	if (lang)
	{
		delete lang;
		lang=NULL;
	}
	delete name;
	delete result;
}

Localizer& Localizer::GetInstance()
{
	static Localizer localizer;
	return localizer;
}

//   
void Localizer::Load(const char* file,const char* lang)
{
	if (this->lang)
	{
		// -   
		if (*name!=file)
		{
			//    
			if (this->lang!=NULL)
			{
				delete this->lang;
				this->lang=NULL;
			}
			*name="";
		}
		else
		{
			//    
			return;
		}
	}

	//    
	*name=file;
	this->lang=new Language(file,lang);
	if (!this->lang->k_lang)
	{
		//   
		*name="";
		delete this->lang;
		this->lang=NULL;
	}
}

//     
void Localizer::Load(char* data,int length,char* lang)
{
	CMagicString file="MEMORY";
	if (this->lang)
	{
		// -   
		if (*name!=file)
		{
			//    
			if (this->lang!=NULL)
			{
				delete this->lang;
				this->lang=NULL;
			}
			*name="";
		}
		else
		{
			//    
			return;
		}
	}

	//    
	*name=file;
	this->lang=new Language(data,length,lang);
	if (!this->lang->k_lang)
	{
		//   
		*name="";
		delete this->lang;
		this->lang=NULL;
	}
}


//  
const char* Localizer::Translate(const char* label)
{
	if (!lang)
	{
		//   
		*result="Error of the resource file.";
	}
	else
	{
		*result=lang->Label(label,false);
	}

	return result->c_str();
}

//      (  []),       
const char* Localizer::GetPtr(const char* label)
{
	const char* index=NULL;
	if (lang)
	{
		index=lang->Label(label,true);
	}
	return index;
}

//       
int Localizer::GetLanguageCount()
{
	int count=0;
	if (lang)
	{
		count=lang->k_lang;
	}
	return count;
}

//       
const char* Localizer::GetLanguageName(int index)
{
	*result="";
	if (lang)
	{
		if (index>=0 && index<lang->k_lang)
			*result=lang->m_lang[index].name;

	}
	return result->c_str();
}

//    
const char* Localizer::GetLanguageName()
{
	*result="";
	if (lang)
		*result=lang->language.name;
	return result->c_str();
}

//   
//           ,        
void Localizer::SetLanguageName(const char* name)
{
	lang->language.name=name;
}

//    
const char* Localizer::GetLanguageLabel()
{
	if (lang)
		return lang->language.label.c_str();
	return NULL;
}

//     
const char* Localizer::GetFontName()
{
	*result="";
	if (lang)
		*result=lang->language.font;
	return result->c_str();
}

//     
int Localizer::GetFontWidth()
{
	int data=0;
	if (lang)
		data=lang->language.width;
	return data;
}

//     
int Localizer::GetFontHeight()
{
	int data=0;
	if (lang)
		data=lang->language.height;
	return data;
}

//      
bool Localizer::GetFontItalic()
{
	bool data=0;
	if (lang)
		data=lang->language.italic;
	return data;
}

//   ( )   
int Localizer::GetFontWeight()
{
	int data=0;
	if (lang)
		data=lang->language.weight;
	return data;
}

//     
int Localizer::GetFontColor()
{
	int data=0;
	if (lang)
		data=lang->language.color;
	return data;
}