//-----------------------------------------------------------------------------
// File: utils.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#ifndef UTILS
#define UTILS

#include "stdafx.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define MAX_NAME_LENGTH	24 //    

//      
extern void FileError(const char* error);

//    
void SetRndIndex(unsigned int value);
//    
unsigned int GetRndIndex();

//      
float GetRndFloat(float i1,float i2);
double GetRndDouble(double i1,double i2);
int GetRndInt(int min,int max);
unsigned int GetRndDword(unsigned int min,unsigned int max);

enum MAGIC_STREAM_ENUM {MAGIC_STREAM_READ, MAGIC_STREAM_WRITE, MAGIC_STREAM_ADD, MAGIC_STREAM__MAX, MAGIC_STREAM__ERROR=-2};

class CMagicStream;

struct CMagicPoint
{
	int x,y;

	CMagicPoint(){x=y=0;}
	CMagicPoint(int x,int y){this->x=x;this->y=y;}
};

struct CMagicRect
{
	int left,top,right,bottom;

	bool PtInRect(CMagicPoint point)
	{
		return (point.x>=left && point.x<right && point.y>=top && point.y<bottom);
	}

	bool PtInRect(int x, int y)
	{
		return (x>=left && x<right && y>=top && y<bottom);
	}

	int Width()
	{
		return right-left;
	}

	int Height()
	{
		return bottom-top;
	}

	bool operator==(const CMagicRect& r)
	{
		return (left==r.left && top==r.top && right==r.right && bottom==r.bottom);
	}

	bool Intersect(CMagicRect& r1, CMagicRect& r2)
	{
		if (r2.left>=r1.right)
			return false;

		if (r2.right<=r1.left)
			return false;

		if (r2.top>=r1.bottom)
			return false;

		if (r2.bottom<=r1.top)
			return false;


		if (r2.left>=r1.left)
		{
			left=r2.left;
			if (r2.right>r1.right)
				right=r1.right;
			else
				right=r2.right;
		}
		else
		{
			left=r1.left;
			if (r2.right>r1.right)
				right=r1.right;
			else
				right=r2.right;
		}


		if (r2.top>=r1.top)
		{
			top=r2.top;
			if (r2.bottom>r1.bottom)
				bottom=r1.bottom;
			else
				bottom=r2.bottom;
		}
		else
		{
			top=r1.top;
			if (r2.bottom>r1.bottom)
				bottom=r1.bottom;
			else
				bottom=r2.bottom;
		}

		return true;
	}

	void Add(CMagicRect& rect)
	{
		if (rect.left<left)
			left=rect.left;
		if (rect.top<top)
			top=rect.top;
		if (rect.right>right)
			right=rect.right;
		if (rect.bottom>bottom)
			bottom=rect.bottom;
	}

	#ifdef _WINDOWS
	CMagicRect& operator=(const RECT& r)
	{
		left=r.left;
		top=r.top;
		right=r.right;
		bottom=r.bottom;
		return *this;
	}
	#endif
};

struct CMagicRectFloat
{
	float left,top,right,bottom;

	void Serialize(CMagicStream& ar);
};

struct CMagicSize
{
	int cx,cy;

	CMagicSize(){cx=cy=0;}
	CMagicSize(int x,int y){cx=x;cy=y;}
};

class CMagicString
{
protected:
	char* theString;		//   
	int stringLen;			//  

public:
	// 
	CMagicString();
	CMagicString(int n);
	CMagicString(char s);
	CMagicString (const char* s);
	CMagicString(const CMagicString& so);

	// 
	virtual ~CMagicString(void);

	operator const char*()
	{
		return c_str();
	}

	void SetLength(int length);

	int GetLength() const
	{
		return stringLen;
	}

	//   
	int length(void) const
	{
		return stringLen;
	}

	//  
	CMagicString& operator=(const CMagicString&);
	CMagicString& operator=(const char* s);

	//    
	void LoadFromStream(FILE* stream);

	//    
	void SaveToStream(FILE* stream);

	//    ,   String
	void LoadFromStreamForClassString(FILE* stream);

	//       
	//void SaveToStreamByText(FILE* stream);

	//  
	bool operator==(const CMagicString&);
	bool operator==(const char* s);

	bool operator!=(const CMagicString&);
	bool operator!=(const char* s);

	bool operator>(const CMagicString&);

	bool operator<(const CMagicString&);

	//    
	bool IsEmpty() const {return (stringLen==0);}

	// 
	char& operator[](int n) const;

	#ifdef _WINDOWS
	// 
	CMagicString& operator~(void);
	#endif

	// 
	CMagicString operator+(const CMagicString& so);

	CMagicString operator+(const char c);

	// 
	CMagicString operator+=(const CMagicString& so);

	//    
	CMagicString& toUpper(void);

	//    
	CMagicString& toLower(void);

	//    
	int FindChar(char c, int pos=0);

	//    
	int Find(const char* subs, int pos=0) const;

	int Find(const CMagicString& so, int pos=0) const;

	//   
	const char* c_str();

	//          
	//  
	CMagicString substr(int pos,int plen) const;

	//           (   2- )
	void ToSize(int size);

	//      
	unsigned int TimeOfReading();

	//    
	void IntToString(int v);

	//    
	int StringToInt();

	#ifdef _WINDOWS
	//    
	void DoubleToString(double v);
	#endif

	//    
	double StringToDouble();

	//      
	void StringToHexCodes();

	//       
	void Dump(int length, unsigned char* data, int per_line);

	// 
	void Format(const char* type, int data);
	void Format(const char* type, float data);
	void Format(const char* type, double data);
	void Format(const char* type, char data);
	void Format(const char* type, unsigned int data);

	//   
	CMagicString Left(int len) const;

	//   
	CMagicString Right(int len) const;

	//  
	CMagicString Mid(int pos, int count) const;

	//          (  )
	CMagicString GetPathOnly();

	//       
	CMagicString GetFileNameOnly();

	//        
	void Trim();

	//      (    )
	int GetParamCount();

	//      (    )
	CMagicString GetParam(int index);

	//    2    
	bool Divide(const CMagicString& divider, CMagicString& str1, CMagicString& str2);

	//   
	CMagicString GetExtension();

	//       ::
	bool DeleteNamespace();

	//       
	void SaveText(FILE* file);

	//   
	static CMagicString GetReturn();

	//   
	static void SetReturn(const char* s);

	//      nIndex   nCount
	int Delete(int nIndex, int nCount = 1);



	//     ,   
	void DeleteLastSlash();

	//   ,      
	bool IsVariableName();
};

//     

#define MAGIC_STREAM_MEMORY_START_SIZE 4096

class CMagicFile;

class CMagicStream
{
protected:

	int mode; // read/write/add

	unsigned int bytes;	//   

	CMagicString name;

public:

	CMagicStream();

	virtual ~CMagicStream();

	// 
	virtual void Close()=0;

	//   
	virtual unsigned int GetLength()=0;

	//   
	virtual unsigned int GetPosition()=0;
	//   
	virtual void SetPosition(unsigned int pos)=0;
	void Seek(unsigned int pos){SetPosition(pos);}

	virtual unsigned int Write(void* m_data,unsigned int k_data)=0;
	virtual unsigned int Read(void* m_data,unsigned int k_data)=0;

	//   
	virtual void ChangeMode(int mode)=0;

	bool IsStoring();
	bool IsLoading();

	const char* GetName(){return name.c_str();}

	//  
	void SetMode(int mode);
	int GetMode(){return mode;}

	//  
	void operator<<(bool& value);
	void operator<<(char& value);
	void operator<<(unsigned char& value);
	void operator<<(CMagicString& value);
	//void operator<<(long& value);
	void operator<<(int& value);
	void operator<<(short& value);
	void operator<<(unsigned short& value);
	void operator<<(float& value);
	void operator<<(double& value);
	void operator<<(unsigned int& value);
	void operator<<(CMagicSize& value);
	void operator<<(CMagicPoint& value);
	void operator<<(CMagicRect& value);

	//  
	void operator>>(bool& value);
	void operator>>(char& value);
	void operator>>(unsigned char& value);
	void operator>>(CMagicString& value);
	//void operator>>(long& value);
	void operator>>(int& value);
	void operator>>(short& value);
	void operator>>(unsigned short& value);
	void operator>>(float& value);
	void operator>>(double& value);
	void operator>>(unsigned int& value);
	void operator>>(CMagicSize& value);
	void operator>>(CMagicPoint& value);
	void operator>>(CMagicRect& value);
};

class CMagicStreamFile : public CMagicStream
{
protected:
	FILE* stream;

public:
	CMagicStreamFile();
	virtual ~CMagicStreamFile();

	//    
	bool Open(const char* file, int mode=MAGIC_STREAM_READ);

	// 
	virtual void Close();

	//   
	virtual unsigned int GetLength();

	//   
	virtual unsigned int GetPosition();
	//   
	virtual void SetPosition(unsigned int pos);

	virtual unsigned int Write(void* m_data,unsigned int k_data);
	virtual unsigned int Read(void* m_data,unsigned int k_data);

	//   
	virtual void ChangeMode(int mode);
};

class CMagicStreamMemory : public CMagicStream
{
protected:
	char* buffer;
	unsigned int position;
	unsigned int length;
	unsigned int max_length;
	bool is_created_buffer;

public:
	CMagicStreamMemory();
	virtual ~CMagicStreamMemory();

	//    
	bool Open(const char* buffer, unsigned int length, int mode);

	// 
	virtual void Close();

	//   
	virtual unsigned int GetLength();

	//   
	virtual unsigned int GetPosition();
	//   
	virtual void SetPosition(unsigned int pos);

	virtual unsigned int Write(void* m_data,unsigned int k_data);
	virtual unsigned int Read(void* m_data,unsigned int k_data);

	//   
	virtual void ChangeMode(int mode);

	char* GetBuffer(){return buffer;}
};

class CText
{
protected:
	char* data;
	int length;
	int pos;

public:
	CText();
	virtual ~CText();

	void Clear();
	void Create(int len);

	char* GetData(){return data;}
	int GetLength(){return length;}

	//     
	bool ReadString(CMagicString& str);

	//    
	char ReadOne();

	//     C++
	bool ReadWordC(CMagicString* str);

	//   
	void MissEmptyStrings();

	int GetPosition(){return pos;}
	void SetPosition(int pos){this->pos=pos;}

	//  
	void RemoveRemarks();

	//     #if, #ifdef, #ifndef
	bool ReadIF(CMagicString* str);

	//  
	void FromString(CMagicString* str);

	//       /*......*/
	void ToRemark(int start, int length);
};

class CMagicStringArray
{
protected:
	int k_str;
	int max_str;
	CMagicString** m_str;

public:
	CMagicStringArray();
	virtual ~CMagicStringArray();

	void Clear();

	void Serialize(CMagicStream& ar);

	int Add(CMagicString* str);

	CMagicString* Get(int index);

	int GetCount(){return k_str;}

	//          
	void FromString(const char* str);
};

// ,      
#define MAX_CRC	9 //  
class CRC
{
protected:
	unsigned int length;
	unsigned int crc[MAX_CRC];

public:
	CRC();

	//      
	void Create(unsigned char* m_data, unsigned int k_data);

	//     int
	int ToInt();

	// 
	void Serialize(CMagicStream& ar);

	//  
	bool operator==(const CRC& crc);
	bool operator!=(const CRC& crc);

	//  
	CRC& operator=(const CRC& crc);

	unsigned int* GetCrcArray(){return crc;}

protected:
	//      
	void CreateCRCWithStep(unsigned char* m_data, int unsigned k_data, int step);
};

// ,      ,       
class CArrayBase
{
protected:
	int max_existing;
	int k_existing;
	int* m_existing;

	int max_deleted;
	int k_deleted;
	int* m_deleted;

	int max_indexed;
	int* m_indexed;

public:
	CArrayBase();
	virtual ~CArrayBase();

	void ClearExistingAndDeleted();

	void IncreaseDeleted(int from, int to);

	void AddToExisting(int index);
	void AddToDeleted(int index);

	void FromExistingToDeleted(int index);
	int FromDeletedToExisting();
	int FromDeletedToExisting(int deleted_index);

	int GetIndexCount(){return k_existing;}
	int GetIndex(int index){return m_existing[index];}

	virtual void Serialize(CMagicStream& stream);
	virtual void SaveToFile(FILE* file);
	virtual void LoadFromFile(FILE* file);
};

// ,     int
class CArrayInt : public CArrayBase
{
protected:
	int k_int;
	int* m_int;

public:
	CArrayInt();
	virtual ~CArrayInt();

	void Clear();

	//     
	int Get(int index);

	// 
	int Add(int data);

	// 
	bool Delete(int index);

	void Serialize(CMagicStream& stream);
	virtual void SaveToFile(FILE* file);
	virtual void LoadFromFile(FILE* file);
};

// ,     void*
class CArrayPtr : public CArrayBase
{
protected:
	int k_ptr;
	void** m_ptr;

public:
	CArrayPtr();
	virtual ~CArrayPtr();

	void Clear();

	//     
	void* Get(int index);

	// 
	int Add(void* data);

	// 
	bool Delete(int index);

	//    
	int FindIndex(void* ptr);

	void Serialize(CMagicStream& stream);
	virtual void SaveToFile(FILE* file);
	virtual void LoadFromFile(FILE* file);
};


//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

enum MAGIC_TIME_ENUM {MAGIC_TIME_YEAR=1, MAGIC_TIME_DAY=2, MAGIC_TIME_MINUTE=4, MAGIC_TIME_SECOND=8};
// MAGIC_TIME_YEAR		 
// MAGIC_TIME_DAY		   
// MAGIC_TIME_MINUTE	   
// MAGIC_TIME_SECOND	 

class CMagicTime
{
public:
	unsigned int version;

	unsigned int year;		//   
	unsigned int month;		//  ;  = 0,  = 1,   
	unsigned int day;		//   .
	unsigned int hour;		// .
	unsigned int minute;	// .
	unsigned int second;	// .

	CMagicTime();

	void Serialize(CMagicStream& stream);

	//     
	void Format(MAGIC_TIME_ENUM flags, CMagicString& result);
};

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

struct MEM_DATA
{
	unsigned char* data;
	int length;

	bool operator==(const MEM_DATA& d)
	{
		if (length==d.length)
		{
			for (int i=0;i<length;i++)
				if (data[i]!=d.data[i])
					return false;
			return true;
		}
		return false;
	}
};

//    
class CMagicStringData : public CMagicString
{
protected:
	MEM_DATA data;
	int index;

public:
	CMagicStringData();
	virtual ~CMagicStringData();

	MEM_DATA* GetData(){return &data;}
	void SetData(int length, unsigned char* data);

	void SetIndex(int index){this->index=index;}
	int GetIndex(){return index;}

	void Serialize(CMagicStream& stream);
};

class CMagicStringDataArray
{
protected:
	int k_str;
	int max_str;
	CMagicStringData** m_str;

public:
	CMagicStringDataArray();
	virtual ~CMagicStringDataArray();

	void Clear();

	void Serialize(CMagicStream& ar);

	int Add(CMagicString* str, MEM_DATA* data=NULL);
	int Delete(int index);

	CMagicStringData* Get(int index);

	int GetCount(){return k_str;}
};

struct __STRING_TABLE_3
{
	int k_data;
	CMagicStringDataArray** m_data;

	__STRING_TABLE_3();
	~__STRING_TABLE_3();

	void Clear();

	void SetLength(int len);
};

struct __STRING_TABLE_2
{
	int k_data;
	__STRING_TABLE_3** m_data;

	__STRING_TABLE_2();
	~__STRING_TABLE_2();

	void Clear();

	void SetLength(int len);
};

struct __STRING_TABLE_1
{
	int k_data;
	__STRING_TABLE_2** m_data;

	__STRING_TABLE_1();
	~__STRING_TABLE_1();

	void Clear();
	
	void SetLength(int len);
};

// - 
class CMagicStringTable
{
	__STRING_TABLE_1 table;

	int max_ptr;
	int k_ptr;
	CMagicStringData** m_ptr;

public:
	CMagicStringTable();
	virtual ~CMagicStringTable();

	void Clear();

	//   
	CMagicStringData* Find(CMagicString& name);

	//  
	bool Add(CMagicString& name, MEM_DATA* data=NULL);

	//  
	bool Delete(CMagicString& name);

	// 
	//  count_yes ,       
	void Serialize(CMagicStream& stream, bool count_yes=true);

	int GetCount(){return k_ptr;}
	CMagicStringData* Get(int index){return m_ptr[index];}


protected:
	bool GetSymbols(CMagicString& name, int& first, int& middle, int& last);
};

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

//   "Config".
//        
//         , ,  -
//   .

#define MAX_FILE 25 //    

class Config
{
	char* s;
	int k_file; 	//   
	char** m_file; //   
	int* dl_file; 	//    
	int* pos_file;	//      

public:
   
	// 
	Config();

	// 
	virtual ~Config();
   
	//   
	void OpenFile(const char* name);

	//   
	void ReadStream(CMagicStream& stream);

	//     
	//void OpenFileDir(CMagicString mask);


	//     
	bool ReadChar(int nom_file, char& data);

	//       ,    
	int FindString(int nom_file, const char* text, int pos);

	//            -
	//   '['
	int FindStringToBracket(int nom_file, const char* text,int pos);

	//         
	// ( text   )
	int FindLabel(int nom_file, CMagicString& text,int pos);

	//           
	//      
	bool ReadToEnter(int nom_file, char* text);

	//         
	CMagicString FindParametrFile(int nom_file, CMagicString param);

	//        
	CMagicString TranslateParam();

	//            
	//  '['    
	CMagicString FindParametrToLabel(int nom_file, CMagicString param, int pos);

	//        
	CMagicString FindParametrDir(CMagicString param);

	//    
	int GetCountFile();
};


//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

//   "ConfigFile".
//        
//         , ,  -
//   .

#define MAX_FILE 25 //    

class ConfigFile
{
	char* s;
	int k_file; 	//   
	char** m_file;	//   
	int* dl_file; 	//    
	int* pos_file;	//      

	public:
   
	// 
	ConfigFile();

	// 
	~ConfigFile();
   
	//   
	void OpenFile(CMagicString name);

	//     
	void OpenFileDir(CMagicString mask);

	//    
	void OpenMemory(char* data,int len);

	//     
	bool ReadChar(int nom_file,char& data);

	//       ,    
	int FindString(int nom_file,char* text,int pos);

	//            -
	//   '['
	int FindStringToBracket(int nom_file,char* text,int pos);

	//         
	// ( text   )
	int FindLabel(int nom_file,char* text,int pos);

	//           
	//      
	bool ReadToEnter(int nom_file, char* text);

	//         
	CMagicString FindParametrFile(int nom_file,CMagicString param);

	//            
	//  '['    
	CMagicString FindParametrToLabel(int nom_file,CMagicString param,int pos);

	//        
	CMagicString FindParametrDir(CMagicString param);

	//    
	int GetCountFile();
};

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

// ,      
struct Met
{
	CMagicString label; // 
	CMagicString phrase; // 
};

//  "LangAccelerator3"
class LangAccelerator3
{
	unsigned char simbol;
	int k_label;
	Met* m_label;

public:
	LangAccelerator3(char simbol_base);
	~LangAccelerator3();

	//   
	void AddLabel(CMagicString lael, CMagicString value);
	//    
	const char* GetLabel(CMagicString& label);

	//  ,     
	unsigned char GetSimbol(){return simbol;};

private:
	//  
	int FindLabel(CMagicString& label);
};

//  "LangAccelerator2"
class LangAccelerator2
{
	unsigned char simbol;
	int k_index;
	LangAccelerator3** m_index;

public:
	LangAccelerator2(char simbol_base);
	~LangAccelerator2();

	//   
	void AddLabel(CMagicString label, CMagicString value);

	//    
	const char* GetLabel(CMagicString& label);

	//  ,     
	unsigned char GetSimbol(){return simbol;};
};

//  "LangAccelerator1"
class LangAccelerator1
{
	int k_index;
	LangAccelerator2** m_index;

public:
	LangAccelerator1();
	~LangAccelerator1();

	//   
	void AddLabel(CMagicString label, CMagicString value);

	//    
	const char* GetLabel(CMagicString& label);
};

//    
struct LangProperty
{
	CMagicString name;			//  
	CMagicString install_name;	//    
	CMagicString font;			//   
	bool italic;				//   
	int weight;					//    0  900   100
	int width;					//  
	int height;					//  
	int color;					//  
	CMagicString label;			//    
};

//   "Language"
//     ,   .   -
//         
// .       "*.lng"  
//        ,      
//         .   ,      
// .     .

#define MAX_LANGUAGE 25 //    

class Language
{
public:
   
	char* s; //     
	ConfigFile* conf;	// ,    
	CMagicString default_lang; //   
	CMagicString default_phrase;	//   
	int k_lang; //   
	LangProperty* m_lang; //    

	char** m_phrase; //    
	LangProperty language; //  

	LangAccelerator1* data[256]; //    ,   

public:

	// ,   
	Language(CMagicString file,CMagicString lang);

	// 
	Language(char* data,int length,CMagicString lang);

	// 
	~Language();

	//  
	const char* Label(const char* label,bool indexed);

protected:

	//       
	void FindLanguageStream(int nom_file);

	//       *.lng
	void GetLanguageDir();

	//           ,  -
	//       
	void GetLabelFromStream(int nom_file,CMagicString pref,CMagicString prefd);

	//        *.lng     
	//   
	void GetLabelDir(CMagicString lang);

	//      
	CMagicString GetNameLabel(CMagicString lang);

	//      
	int AddPhrase(CMagicString phrase);

	//        
	void AddPhraseAfterLoad(CMagicString phrase,int nom);

	//  
	void Init1();

	//  
	void Init2(CMagicString lang);
};

//  Localizer
class Localizer
{
	CMagicString* name;
	Language* lang;
	CMagicString* result;

public:
	// 
	Localizer();

	// 
	~Localizer();

	static Localizer& GetInstance();

	//   
	// file -   (   *.lng, ..        lng)
	// language -  ,    ( language="",      LANG_DEFAULT)
	void Load(const char* file,const char* language);

	//     
	// data -  
	// length -   
	void Load(char* data,int length,char* lang);

	//  
	//      (  []),       .
	const char* Translate(const char* label);
	//      (  []),       
	const char* GetPtr(const char* label);

	//       
	int GetLanguageCount();

	//       
	const char* GetLanguageName(int index);

	//    
	const char* GetLanguageName();

	//   
	//           ,        
	void SetLanguageName(const char* name);

	//    
	const char* GetLanguageLabel();

	//     
	const char* GetFontName();

	//     
	int GetFontWidth();

	//     
	int GetFontHeight();

	//      
	bool GetFontItalic();

	//   ( )   
	int GetFontWeight();

	//     
	int GetFontColor();
};

#endif

