//-----------------------------------------------------------------------------
// File: Game.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "Game.h"
#include "sound/Sound.h"
#include "utf.h"


#ifdef STEAM
#include "steam/SteamLicense.h"
#endif

//       
GMemWriter wr1, wr2;
GMemReader rd1, rd2;


FILE_DATA::FILE_DATA()
{
	DetachData();
}

FILE_DATA::~FILE_DATA()
{
	Clear();
}

void FILE_DATA::Clear()
{
	if (data)
	{
		delete []data;
		data=NULL;
	}
	length=0;
}

void FILE_DATA::Create(int len)
{
	Clear();
	data=new unsigned char[len];
	length=len;
}

void FILE_DATA::DetachData()
{
	data=NULL;
	length=0;
}

// ------------------------------------------------------------------------------------

FINDER_LIST::FINDER_LIST()
{
	k_finder=0;
	m_finder=NULL;
}

FINDER_LIST::~FINDER_LIST()
{
	for (int i=0;i<k_finder;i++)
	{
		delete m_finder[i];
		m_finder[i]=NULL;
	}

	if (m_finder)
	{
		delete []m_finder;
		m_finder=NULL;
	}
	k_finder=0;
}

void FINDER_LIST::Add(FINDER_DATA* finder)
{
	if (m_finder)
	{
		FINDER_DATA** vm_finder=new FINDER_DATA*[k_finder+1];
		for (int i=0;i<k_finder;i++)
			vm_finder[i]=m_finder[i];
		delete []m_finder;
		m_finder=vm_finder;
	}
	else
	{
		m_finder=new FINDER_DATA*[1];
	}
	m_finder[k_finder]=finder;
	k_finder++;
}

void FINDER_LIST::Delete()
{
	if (k_finder)
	{
		k_finder--;
		delete m_finder[k_finder];
		if (!k_finder)
		{
			delete []m_finder;
			m_finder=NULL;
		}
	}
}
FINDER_DATA* FINDER_LIST::Get()
{
	if (k_finder>0)
		return m_finder[k_finder-1];
	return NULL;
}

// ------------------------------------------------------------------------------------

GPlatform::GPlatform(GGame* owner, const char* game_name)
{
	GDevice::platform=this;
	GNet::platform=this;

	this->owner=owner;
	this->game_name=game_name;
	//user="";

	str_utf8=NULL;
	str_utf8_len=0;
	str_utf16=NULL;
	str_utf16_len=0;
	str_utf32=NULL;
	str_utf32_len=0;

	k_drive=0;
	m_drive=NULL;
	current_virtual_drive=NULL;
	//fd_drive.folder=NULL;
}

GPlatform::~GPlatform()
{
	for (int i=0;i<k_drive;i++)
	{
		if (m_drive[i])
		{
			delete m_drive[i];
			m_drive[i]=NULL;
		}
	}
	if (m_drive)
	{
		delete []m_drive;
		m_drive=NULL;
	}

	GDevice::platform=NULL;
	GNet::platform=NULL;

	if (str_utf8)
	{
		delete str_utf8;
		str_utf8=NULL;
	}

	if (str_utf16)
	{
		delete str_utf16;
		str_utf16=NULL;
	}

	if (str_utf32)
	{
		delete str_utf32;
		str_utf32=NULL;
	}
}

//    
bool GPlatform::DetectFirstLaunch()
{
	CMagicString path=GetPathToConfig();
	path+="nofirst.ctl";

	bool is=false;
	GGame* game=GSurface::device->game;
	if (game)
	{
		CMagicString version=game->GetVersion();

		CMagicStream* stream=OpenStream(path, MAGIC_STREAM_READ);
		if (stream)
		{
			CMagicString version2;
			(*stream)>>version2;
			delete stream;
			if (version==version2)
			{
				//  
				is=true;
			}
		}

		if (!is)
		{
			//    
			stream=OpenStream(path, MAGIC_STREAM_WRITE);
			if (stream)
			{
				(*stream)<<version;
				delete stream;
				game->OnFirstLaunch();
			}
		}
	}
	else
		is=true;

	return !is;

	/*
	FILE_DATA fd;
	if (!LoadFile(path, &fd))
	{
		bool result=SaveFile(path, &fd);
		if (GSurface::device->game)
			GSurface::device->game->OnFirstLaunch();
		return true;
	}
	return false;
	*/
}

// eng: Returns path to folder with objects
// rus:      
const char* GPlatform::GetPathToObject()
{
	return owner->GetPathToObject();
}

// eng: Returns path to folder with images
// rus:      
const char* GPlatform::GetPathToImage()
{
	return owner->GetPathToImage();
}

// eng: Returns path to folder with fonts
// rus:      
const char* GPlatform::GetPathToFont()
{
	return owner->GetPathToFont();
}

// eng: Returns path to folder with users
// rus:      
const char* GPlatform::GetPathToUsers()
{
	return owner->GetPathToUsers();
}

// eng: Returns path to folder with user profile
// rus:       
const char* GPlatform::GetPathToProfile()
{
	return owner->GetPathToProfile();
}

// eng: Returns path to folder with saved games
// rus:       
const char* GPlatform::GetPathToSavedGame()
{
	return owner->GetPathToSavedGame();
}

// eng: Returns path to folder with music
// rus:      
const char* GPlatform::GetPathToMusic()
{
	return owner->GetPathToMusic();
}

// eng: Returns path to folder with sounds
// rus:      
const char* GPlatform::GetPathToSound()
{
	return owner->GetPathToSound();
}

// eng: Returns path to folder with voices
// rus:      
const char* GPlatform::GetPathToVoice()
{
	return owner->GetPathToVoice();
}

// eng: Returns path to folder with maps
// rus:      
const char* GPlatform::GetPathToMaps()
{
	return owner->GetPathToMaps();
}

// eng: Returns path to folder with downloaded maps
// rus:       
const char* GPlatform::GetPathToDownloadedMaps()
{
	return owner->GetPathToDownloadedMaps();
}

// eng: Returns path to folder with configuration
// rus:      
const char* GPlatform::GetPathToConfig()
{
	if (owner)
		return owner->GetPathToConfig();
	return GetPathToTemp();
}

// eng: Returns path to folder with initialization data
// rus:       
const char* GPlatform::GetPathToInit()
{
	return owner->GetPathToInit();
}

// eng: Returns path to folder with autosave
// rus:      autosave
const char* GPlatform::GetPathToAutosave()
{
	return owner->GetPathToAutosave();
}

// eng: Returns path to folder with files of virtual disk
// rus:        
const char* GPlatform::GetPathToVirtualDrive()
{
	return owner->GetPathToVirtualDrive();
}

// eng: Finds the first file in folder
// rus:     
const char* GPlatform::GetFirstFile(const char* path, const char* ext)
{
	GVirtualDrive* drive=GetVirtualDrive(path);
	if (drive)
	{
		VIRTUAL_ITEM* item=drive->GetItem(current_directory);
		if (item && item->IsFolder())
		{
			VIRTUAL_FIND_ITEM* finder=new VIRTUAL_FIND_ITEM;
			finder->folder=(VIRTUAL_FOLDER*)item;
			if (finder->folder->GetFirstItem(*finder, true, ext))
			{
				finders.Add(finder);
				return finder->name;
			}
			else
				delete finder;
		}
		return NULL;
	}
	//fd_drive.folder=NULL;
	return GetFirstFile_OS(path, ext);
}

// eng: Finds the next file folder
// rus:     
const char* GPlatform::GetNextFile()
{
	FINDER_DATA* finder=finders.Get();
	if (finder)
	{
		if (finder->IsVirtualDrive())
		{
			VIRTUAL_FIND_ITEM* virtual_finder=(VIRTUAL_FIND_ITEM*)finder;
			if (virtual_finder->folder->GetNextItem(*virtual_finder))
				return virtual_finder->name;
			finders.Delete();
			return NULL;
		}
		return GetNextFile_OS();
	}
	return NULL;
}

// eng: Finds the first child folder in folder
// rus:      
const char* GPlatform::GetFirstFolder(const char* path)
{
	GVirtualDrive* drive=GetVirtualDrive(path);
	if (drive)
	{
		VIRTUAL_ITEM* item=drive->GetItem(current_directory);
		if (item && item->IsFolder())
		{
			VIRTUAL_FIND_ITEM* finder=new VIRTUAL_FIND_ITEM;
			finder->folder=(VIRTUAL_FOLDER*)item;
			if (finder->folder->GetFirstItem(*finder, false, NULL))
			{
				finders.Add(finder);
				return finder->name;
			}
			else
				delete finder;
		}
		return NULL;
	}
	//fd_drive.folder=NULL;
	return GetFirstFolder_OS(path);
}

// eng: Finds the next child folder in folder
// rus:      
const char* GPlatform::GetNextFolder()
{
	FINDER_DATA* finder=finders.Get();
	if (finder)
	{
		if (finder->IsVirtualDrive())
		{
			VIRTUAL_FIND_ITEM* virtual_finder=(VIRTUAL_FIND_ITEM*)finder;
			if (virtual_finder->folder->GetNextItem(*virtual_finder))
				return virtual_finder->name;
			finders.Delete();
			return NULL;
		}
		return GetNextFolder_OS();
	}
	return NULL;

	//if (fd_drive.folder)
	//{
	//	if (fd_drive.folder->GetNextItem(fd_drive))
	//		return fd_drive.name;
	//	return NULL;
	//}
	//return GetNextFolder_OS();
}

// eng: Creating of folder
// rus:  
bool GPlatform::CreateFolder(const char* folder)
{
	CMagicString path=folder;
	path.DeleteLastSlash();
	GVirtualDrive* drive=GetVirtualDrive(path);
	if (drive)
		return drive->CreateDirectory(current_directory);
	return CreateFolder_OS(path);
}
// eng: Deleting of folder
// rus:  
bool GPlatform::RemoveFolder(const char* folder)
{
	CMagicString path=folder;
	path.DeleteLastSlash();
	GVirtualDrive* drive=GetVirtualDrive(path);
	if (drive)
		return drive->RemoveDirectory(current_directory);
	return RemoveFolder_OS(path);
}

// eng: Converting path to absolute path
// rus:      
const char* GPlatform::PathToAbsolutePath(const char* path)
{
	CMagicString current_path=GetDirectory();
	if (SetDirectory(path))
	{
		CMagicString vpath=path;
		int len=vpath.GetLength();
		char s=vpath[len-1];
		vpath=GetDirectory(); //    current_directory
		SetDirectory(current_path);
		if (s=='/' || s=='\\')
			vpath+=s;
		current_directory=vpath;
		return current_directory;
	}

	//   ,       
	CMagicString file_path=path;
	CMagicString path_only=file_path.GetPathOnly();
	if (SetDirectory(path_only))
	{
		CMagicString cpath=GetDirectory(); //    current_directory
		SetDirectory(current_path);
		int len_file=file_path.GetLength();
		int len_path=path_only.GetLength();
		if (len_file>len_path)
		{
			CMagicString file_only=file_path.Right(len_file-len_path);
			cpath+=file_only;

			current_directory=cpath;
			//memcpy(current_directory, cpath.c_str(), cpath.GetLength()+1);
		}

		return current_directory;
	}
	
	return NULL;
}

//  
bool GPlatform::CreatePath(CMagicString& path)
{
	int k_dirs=0;
	CMagicString m_dirs[50];
	int len=path.GetLength();
	CMagicString dir="";
	const char* s=path;

	int i=0;

	bool net=false;

	if (len>1 && (!IsPathAbsolute_OS(path)))
	{
		//     
		if ((s[0]=='\\' || s[0]=='/') && (s[1]=='\\' || s[1]=='/'))
		{
			dir=s[0];
			dir+=s[0];
			i=2;
			net=true;
		}
	}

	for (;i<len;i++)
	{
		char simbol=s[i];
		if (simbol=='\\' || simbol=='/')
		{
			//dir+=simbol;
			if (net)
				net=false;
			else
			{
				if (!dir.IsEmpty())
				{
					m_dirs[k_dirs]=dir;
					k_dirs++;
					dir="";
				}
			}
		}
		else
		{
			dir+=simbol;
		}
	}

	if (!dir.IsEmpty())
	{
		m_dirs[k_dirs]=dir;
		k_dirs++;
	}

	CMagicString old_path;

	old_path=GetDirectory();

	bool ok=false;
	int create=10000;
	int t=-1;

	if (k_dirs>0)
	{
		int i=0;
		if (m_dirs[0].GetLength()==2 && m_dirs[0][1]==':')
		{
			//      Windows
			m_dirs[0]+=GetSlash();
		}
		do 
		{
			dir=m_dirs[i];
			ok=SetDirectory(dir);
			if (!ok)
			{
				ok=CreateFolder(dir);
				if (ok)
					ok=SetDirectory(dir);

				if (!ok)
				{
					//  
					break;
				}
				else
				{
					if (create==10000)
						create=i;
				}
			}
			t=i;
			i++;
		}
		while (i<k_dirs);
	}

	if (!ok)
	{
		//    
		for (i=t;i>=create;i--)
		{
			dir=m_dirs[i];
			if (SetDirectory(".."))
			{
				if (!RemoveFolder(dir))
					break;
			}
			else
				break;
		}
	}

	SetDirectory(old_path);

	if (ok)
		return true;
	else
		return false;
}

// Loading of file
//  
bool GPlatform::LoadFile(const char* file, FILE_DATA* file_data)
{
	CMagicStream* stream=OpenStream(file, MAGIC_STREAM_READ);
	if (stream)
	{
		int len=stream->GetLength();
		file_data->Create(len);
		stream->Read(file_data->data, len);
		delete stream;
		return true;
	}
	return false;
}

// Saving of file
//  
bool GPlatform::SaveFile(const char* file, FILE_DATA* file_data)
{
	CMagicString path=file;
	path=path.GetPathOnly();
	if (!path.IsEmpty())
	{
		//  ,     
		CreatePath(path);
	}

	CMagicStream* stream=OpenStream(file, MAGIC_STREAM_WRITE);
	if (stream)
	{
		stream->Write(file_data->data, file_data->length);
		delete stream;
		return true;
	}
	return false;
	/*
	FILE* f=OpenFile(file, "wb");

	if (f)
	{
		fwrite(file_data->data, 1, file_data->length, f);
		CloseFile(f);

		return  true;
	}

	return false;
	*/
}


// eng: Opening of file
// rus:  
FILE* GPlatform::OpenFile(const char* file, const char* mode)
{
	return fopen(file, mode);
}

// eng: Closing of file
// rus:  
bool GPlatform::CloseFile(FILE* file)
{
	return (fclose(file)==0);
}

// eng: Opening of stream
// rus:  
CMagicStream* GPlatform::OpenStream(const char* file, int mode)
{
	GVirtualDrive* drive=GetVirtualDrive(file);
	if (drive)
		return drive->OpenStream(current_directory, mode);

	CMagicStreamFile* stream=new CMagicStreamFile;
	if (!stream->Open(file, mode))
	{
		delete stream;
		stream=NULL;
	}
	return stream;
}

// eng: Copying of file
// rus:  
bool GPlatform::CopyFile(const char* from, const char* to)
{
	FILE_DATA fd;
	if (LoadFile(from, &fd))
	{
		if (SaveFile(to, &fd))
			return true;
	}
	return false;
}

// eng: Copying of folder
// rus:  
bool GPlatform::CopyFolder(const char* from, const char* to, bool create_top_folder)
{
	bool is=true;
	CMagicString path_from=PathToAbsolutePath(from);
	path_from+=GetSlash();
	CMagicString path_to=PathToAbsolutePath(to);
	path_to+=GetSlash();

	if (create_top_folder)
	{
		CMagicString vpath=GetDirectory();
		if (SetDirectory(path_from+".."))
		{
			CMagicString path1=GetDirectory();
			if (SetDirectory(from))
			{
				CMagicString path2=GetDirectory();
				int rlen=path2.GetLength()-path1.GetLength();
				CMagicString top_folder=path_to+path2.Right(rlen);
				if (CreateFolder(top_folder))
				{
					path_to+=top_folder;
					path_to+=GetSlash();
				}
				else
					is=false;
			}
		}
		SetDirectory(vpath);
	}

	if (is)
	{
		const char* folder=GetFirstFolder(from);
		while(folder)
		{
			CMagicString folder_path=path_to+folder;
			if (CreateFolder(folder_path))
			{
				CMagicString new_from=path_from;
				new_from+=folder;
				if (!CopyFolder(new_from, folder_path, false))
					is=false;
			}
			else
			{
				is=false;
			}
			folder=GetNextFolder();
		}

		const char* file=GetFirstFile(from, "*");
		while(file)
		{
			if (!CopyFile(path_from+file, path_to+file))
				is=false;
			file=GetNextFile();
		}
	}
	return is;
}

// eng: Current directory
// rus:  
bool GPlatform::SetDirectory(const char* path)
{
	GVirtualDrive* drive=GetVirtualDrive(path);
	if (drive)
	{
		if (drive->SetDirectory(current_directory))
		{
			current_virtual_drive=drive;
			return true;
		}
		return false;
	}
	
	if (SetDirectory_OS(path))
	{
		current_virtual_drive=NULL;
		return true;
	}
	return false;
}
const char* GPlatform::GetDirectory()
{
	if (current_virtual_drive)
	{
		for (int i=0;i<k_drive;i++)
		{
			if (m_drive[i]==current_virtual_drive)
			{
				//   
				CMagicString d;
				d.IntToString(i);
				current_directory=VIRTUAL_DRIVE_SYMBOL;
				current_directory+=d;
				d=current_virtual_drive->GetDirectory();
				d=d.Right(d.GetLength()-1);
				if (!d.IsEmpty())
					current_directory+=GetSlash();
				current_directory+=d;
				return current_directory;
			}
		}
	}
	return GetDirectory_OS();
}

//    ,   
bool GPlatform::IsCopyKeyPressed(unsigned int key)
{
	if (key==KEY_C)
	{
		unsigned int control=owner->GetControl();
		if (control & CONTROL_CTRL)
			return true;
	}
	return false;
}
bool GPlatform::IsCutKeyPressed(unsigned int key)
{
	if (key==KEY_X)
	{
		unsigned int control=owner->GetControl();
		if (control & CONTROL_CTRL)
			return true;
	}
	return false;
}
bool GPlatform::IsPastKeyPressed(unsigned int key)
{
	if (key==KEY_V)
	{
		unsigned int control=owner->GetControl();
		if (control & CONTROL_CTRL)
			return true;
	}
	return false;
}

//           
CMagicString GPlatform::CreateVirtualDrive(const char* path, const char* name)
{
	GVirtualDrive* drive=new GVirtualDrive;
	CMagicString full_path=drive->Create(path, name);
	delete drive;
	if (!full_path.IsEmpty())
	{
		full_path=OpenVirtualDrive(full_path);
		return full_path;
	}
	return "";
}

//         
CMagicString GPlatform::OpenVirtualDrive(const char* path)
{
	GVirtualDrive* drive=new GVirtualDrive;
	if (drive->Open(path))
	{
		//    
		int i;
		int index=-1;
		for (i=0;i<k_drive;i++)
		{
			if (!m_drive[i])
			{
				index=i;
				break;
			}
		}

		if (index==-1)
		{
			//   
			if (m_drive)
			{
				GVirtualDrive** vm_drive=new GVirtualDrive*[k_drive+1];
				for (i=0;i<k_drive;i++)
					vm_drive[i]=m_drive[i];
				delete []m_drive;
				m_drive=vm_drive;
			}
			else
			{
				m_drive=new GVirtualDrive*[1];
			}
			index=k_drive;
			k_drive++;
		}
		m_drive[index]=drive;

		CMagicString name=VIRTUAL_DRIVE_SYMBOL;
		CMagicString d;
		d.IntToString(index);
		name+=d;
		name+=GetSlash();
		return name;
	}
	delete drive;
	return "";
}

//         
bool GPlatform::CloseVirtualDrive(const char* path)
{
	GVirtualDrive* drive=GetVirtualDrive(path);
	if (drive)
	{
		for (int i=0;i<k_drive;i++)
		{
			if (m_drive[i]==drive)
			{
				//   
				if (drive==current_virtual_drive)
					current_virtual_drive=NULL;
				m_drive[i]->Close();
				delete m_drive[i];
				m_drive[i]=NULL;
				return true;
			}
		}
	}
	return false;
}

//        
int GPlatform::GetVirtualDriveIndexByName(const char* file_name)
{
	CMagicString file=file_name;
	for (int i=0;i<k_drive;i++)
	{
		GVirtualDrive* drive=m_drive[i];
		if (drive && drive->IsNameEqual(file))
			return i;
	}
	return -1;
}

//   ,    
//         current_directory
GVirtualDrive* GPlatform::GetVirtualDrive(const char* path)
{
	current_directory=path;
	int len=current_directory.GetLength();
	if (len>0)
	{
		if (current_directory[0]==VIRTUAL_DRIVE_SYMBOL)
		{
			//      
			//    
			CMagicString index_str;
			for (int i=1;i<len;i++)
			{
				char s=current_directory[i];
				if (s>='0' && s<='9')
					index_str+=s;
				else
					break;
			}
			if (!index_str.IsEmpty())
			{
				int index=index_str.StringToInt();
				if (index<k_drive)
				{
					current_directory=path;

					int index_len=index_str.GetLength();
					current_directory=current_directory.Right(len-index_len-1);
					if (current_directory.IsEmpty())
						current_directory=VIRTUAL_DRIVE_SYMBOL;
					else
					{
						if (current_directory[0]!='/' && current_directory[0]!='\\')
							return NULL;
						current_directory[0]=VIRTUAL_DRIVE_SYMBOL;
					}
					return m_drive[index];
				}
			}
		}
		else
		{
			if (IsPathAbsolute_OS(current_directory))
			{
				//        
				return NULL;
			}
		}
	}
	return current_virtual_drive;
}

//   
const char* GPlatform::ConnectVirtualDrive(const char* drive_path, const char* drive_name)
{
	CMagicString temp_str;
	CMagicString virtual_drive=drive_name;

	int index=GetVirtualDriveIndexByName(drive_name);
	if (index==-1)
	{
		//     
		CMagicString image_path=drive_path;
		temp_str=image_path;
		CMagicString full_path=temp_str+virtual_drive;
		full_path+=".vdt";
		temp_str=OpenVirtualDrive(full_path);
		if (temp_str.IsEmpty())
		{
			//    ,    
			CMagicString drive_path=CreateVirtualDrive(image_path, virtual_drive);
			CMagicString vdd_path=GetPathToGame();
			vdd_path+="vdd";
			vdd_path+=GetSlash();
			bool is=CopyFolder(vdd_path+drive_name, drive_path, false);
			CloseVirtualDrive(drive_path); //   ,      
			if (is)
			{
				temp_str=OpenVirtualDrive(full_path);
				GVirtualDrive* drive=GetVirtualDrive(drive_path);
				drive->Mix();
				index=GetVirtualDriveIndexByName(virtual_drive);
			}
			else
				return NULL;
		}
		else
		{
			index=GetVirtualDriveIndexByName(virtual_drive);
		}
	}

	virtual_drive.IntToString(index);
	temp_str=VIRTUAL_DRIVE_SYMBOL;
	temp_str+=virtual_drive;
	temp_str+=GetSlash();
	current_directory=temp_str;
	return current_directory.c_str();
}


// rus:    UTF16    UTF8
const unsigned char* GPlatform::Magic_UTF16to8(const unsigned short* str)
{
	if (str)
	{
		int length=0;
		while (true)
		{
			unsigned short s=str[length];
			length++;
			if (!s)
				break;
		}

		int end=length*5;
		if (end>str_utf8_len)
		{
			delete []str_utf8;
			str_utf8=new UTF8[end];
			str_utf8_len=end;
		}

		//      ,      
		const UTF16* vstr_utf16=str;
		const UTF16** source=&vstr_utf16;
		UTF8* vstr8=str_utf8;
		UTF8** target=&vstr8;

		ConversionResult result=__ConvertUTF16toUTF8 (
			source, &(str[length]),
			target, &(str_utf8[str_utf8_len]), lenientConversion);

		if (result==conversionOK)
			return str_utf8;
	}
	return NULL;
}

// rus:    UTF32    UTF8
const unsigned char* GPlatform::Magic_UTF32to8(const unsigned int* str)
{
	if (str)
	{
		int length=0;
		while (true)
		{
			unsigned int s=str[length];
			length++;
			if (!s)
				break;
		}

		int end=length*5;
		if (end>str_utf8_len)
		{
			delete []str_utf8;
			str_utf8_len=end;
			str_utf8=new UTF8[str_utf8_len];
		}

		//      ,      
		const UTF32* vstr_utf32=str;
		const UTF32** source=&vstr_utf32;
		UTF8* vstr8=str_utf8;
		UTF8** target=&vstr8;

		ConversionResult result=__ConvertUTF32toUTF8 (
			source, &(str[length]),
			target, &(str_utf8[str_utf8_len]), lenientConversion);

		if (result==conversionOK)
			return str_utf8;
	}
	return NULL;
}

// rus:    UTF8    UTF16
const unsigned short* GPlatform::Magic_UTF8to16(const unsigned char* str)
{
	if (str)
	{
		int length=0;
		while (true)
		{
			unsigned int s=str[length];
			length++;
			if (!s)
				break;
		}

		if (length>str_utf16_len)
		{
			delete []str_utf16;
			str_utf16_len=length;
			str_utf16=new UTF16[str_utf16_len];
		}

		//      ,      
		const UTF8* vstr_utf8=str;
		const UTF8** source=&vstr_utf8;
		UTF16* vstr16=str_utf16;
		UTF16** target=&vstr16;

		ConversionResult result=__ConvertUTF8toUTF16 (
			source, &(str[length]),
			target, &(str_utf16[str_utf16_len]), lenientConversion);

		if (result==conversionOK)
			return str_utf16;
	}
	return NULL;
}

// rus:    UTF8    UTF32
const unsigned int* GPlatform::Magic_UTF8to32(const unsigned char* str)
{
	if (str)
	{
		int length=0;
		while (true)
		{
			char s=str[length];
			length++;
			if (!s)
				break;
		}

		if (length>str_utf32_len)
		{
			delete []str_utf32;
			str_utf32=new UTF32[length];
			str_utf32_len=length;
		}

		//      ,      
		const UTF8* vstr_utf8=str;
		const UTF8** source=&vstr_utf8;
		UTF32* vstr32=str_utf32;
		UTF32** target=&vstr32;

		ConversionResult result=__ConvertUTF8toUTF32 (
			source, &(str[length]),
			target, &(str_utf32[str_utf32_len]), lenientConversion);

		if (result==conversionOK)
			return str_utf32;
	}
	return NULL;
}

// ru:  wchar_t  utf8
// en: Conversion of wchar_t in utf8
const char* GPlatform::wchar_to_utf8(const wchar_t* str)
{
	const char* str8=NULL;

	int size=sizeof(wchar_t);
	switch (size)
	{
	case 1:
		wchar_t8=(const char*)str;
		str8=(const char*)wchar_t8;
		break;

	case 2:
		str8=(const char*)Magic_UTF16to8((const unsigned short*)str);
		break;

	case 4:
		str8=(const char*)Magic_UTF32to8((const unsigned int*)str);
		break;
	}
	return str8;
}

// ru:  utf8  wchar_t
// en: Conversion of utf8 in wchar_t
const wchar_t* GPlatform::utf8_to_wchar(const char* str)
{
	wchar_t* strw=NULL;

	int size=sizeof(wchar_t);
	switch (size)
	{
	case 1:
		wchar_t8=str;
		strw=(wchar_t*)((const char*)wchar_t8);
		break;

	case 2:
		strw=(wchar_t*)Magic_UTF8to16((const unsigned char*)str);
		break;

	case 4:
		strw=(wchar_t*)Magic_UTF8to32((const unsigned char*)str);
		break;
	}

	return strw;
}

// ------------------------------------------------------------------------------------

GGame* GComponent::game=NULL;

GComponent::GComponent()
{
	theme=NULL;
	parent=NULL;

	invalidated=false;

	id=0xffffffff;
	x=y=0;
	width=height=0;

	style=0;
	align=ALGN_LEFT;

	fixed=false;

	k_child=0;
	m_child=NULL;

	previous_modal=NULL;
	is_added_modal=false;
	previous_focus=NULL;

	user=0;

	k_covering=max_covering=0;
	m_covering=NULL;

	font=-1;
	bg_color=bg_color_disabled=bg_color_highlighted=bg_color_pressed=COLOR_KEY_EMPTY;
	text_color=text_color_disabled=text_color_highlighted=text_color_pressed=COLOR_KEY_EMPTY;
	border_color=border_color_disabled=border_color_highlighted=border_color_pressed=COLOR_KEY_EMPTY;
	caption_bg_color=caption_bg_color_disabled=caption_bg_color_highlighted=caption_bg_color_pressed=COLOR_KEY_EMPTY;
	caption_text_color=caption_text_color_disabled=caption_text_color_highlighted=caption_text_color_pressed=COLOR_KEY_EMPTY;
}

GComponent::~GComponent()
{
	

	for (int i=0;i<k_child;i++)
	{
		delete m_child[i];
		m_child[i]=NULL;
	}

	if (m_child)
	{
		delete []m_child;
		m_child=NULL;
	}
	k_child=0;


	if (m_covering)
	{
		delete []m_covering;
		m_covering=NULL;
	}
	k_covering=max_covering=0;


	if (game)
	{
		if (game->GetCapturedComponent()==this)
			game->SetCapturedComponent(NULL, COMPONENT_PLACE_WIN);

		if (game->GetFocusComponent()==this)
			game->SetFocusComponent(NULL);

		if (game->GetModalComponent()==this)
			game->SetModalComponent(NULL);
	}
}

//  
bool GComponent::SetAlignText(COMPONENT_ALIGN_TEXT align)
{
	if (this->align!=align)
	{
		this->align=align;
		Invalidate();
		return true;
	}
	return false;
}

bool GComponent::SetFont(int font)
{
	if (this->font!=font)
	{
		this->font=font;
		Invalidate();
		return true;
	}
	return false;
}

//       
void GComponent::CopyAttributesTo(GComponent* destination)
{
	destination->font=font;

	destination->bg_color=bg_color;
	destination->bg_color_disabled=bg_color_disabled;
	destination->bg_color_highlighted=bg_color_highlighted;
	destination->bg_color_pressed=bg_color_pressed;

	destination->text_color=text_color;
	destination->text_color_disabled=text_color_disabled;
	destination->text_color_highlighted=text_color_highlighted;
	destination->text_color_pressed=text_color_pressed;

	destination->border_color=border_color;
	destination->border_color_disabled=border_color_disabled;
	destination->border_color_highlighted=border_color_highlighted;
	destination->border_color_pressed=border_color_pressed;

	destination->caption_bg_color=caption_bg_color;
	destination->caption_bg_color_disabled=caption_bg_color_disabled;
	destination->caption_bg_color_highlighted=caption_bg_color_highlighted;
	destination->caption_bg_color_pressed=caption_bg_color_pressed;

	destination->caption_text_color=caption_text_color;
	destination->caption_text_color_disabled=caption_text_color_disabled;
	destination->caption_text_color_highlighted=caption_text_color_highlighted;
	destination->caption_text_color_pressed=caption_text_color_pressed;
}

//  
bool GComponent::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id)
{
	if (IsCreated())
		return false;
	if (theme)
	{
		int wi, he;
		theme->GetMinSize(this, wi, he);
		if (width<wi)
			width=wi;
		if (height<he)
			height=he;

		theme->GetMaxSize(this, wi, he);
		if (width>wi)
			width=wi;
		if (height>he)
			height=he;
	}

	unsigned int additional_style=game->GetAdditionalStyles(this);

	this->style=style | additional_style;

	this->id=id;

	this->x=x;
	this->y=y;

	this->width=width;
	this->height=height;

	if (parent)
		parent->AddChild(this);

	RecalcCovering();

	return true;
}

//    ,       
void GComponent::SetClientSize(int wi, int he)
{
	int copy_width=width;
	int copy_height=height;

	width=wi;
	height=he;
	CMagicRect rect;
	GetClientRect(&rect);
	int client_wi=rect.Width();
	int client_he=rect.Height();

	int rx=wi-client_wi;
	int ry=he-client_he;

	//   
	width+=rx;
	height+=ry;

	if (copy_width!=width || copy_height!=height)
	{
		RecalcLayout();
		Invalidate();
	}
}


void GComponent::SetWindowText(const char* text)
{
	if (strcmp(text, this->text.c_str()))
	{
		this->text=text;
		SysOnChangeText();
		Invalidate();
	}
}

//   
void GComponent::AddChild(GComponent* child)
{
	if (this)
	{
		if (k_child)
		{
			GComponent** vm_child=new GComponent*[k_child+1];

			for (int i=0;i<k_child;i++)
				vm_child[i]=m_child[i];

			delete []m_child;
			m_child=vm_child;
		}
		else
		{
			m_child=new GComponent*[1];
		}

		child->parent=this;

		m_child[k_child]=child;
		k_child++;
	}
}

//   
void GComponent::DeleteChild(GComponent* child, bool del_child)
{
	int index=Find(child);
	if (index!=-1)
	{
		RecalcCovering();

		if (k_child>1)
		{
			GComponent** vm_child=new GComponent*[k_child-1];
			for (int i=0;i<index;i++)
				vm_child[i]=m_child[i];

			for (int i=index+1;i<k_child;i++)
				vm_child[i-1]=m_child[i];

			delete []m_child;
			m_child=vm_child;
		}
		else
		{
			delete []m_child;
			m_child=NULL;
		}

		k_child--;

		if (game)
		{
			child->ClearLinks();

			/*
			if (child->invalidated)
			{
				//      
				game->DeleteInvalidate(child);
				child->invalidated=false;
			}
			*/
		}

		if (del_child)
			delete child;
	}
}

//    
void GComponent::ClearLinks()
{
	if (game)
	{
		if (game->GetCapturedComponent()==this)
			game->SetCapturedComponent(NULL, COMPONENT_PLACE_WIN);
		if (game->GetFocusComponent()==this)
			game->SetFocusComponent(NULL);
		if (game->GetModalComponent()==this)
			game->SetModalComponent(NULL);
		if (game->GetPressedComponent()==this)
			game->SetPressedComponent(NULL, COMPONENT_PLACE_WIN);
		if (game->GetOverComponent()==this)
			game->SetOverComponent(NULL);
		if (game->GetKeyDownComponent()==this)
			game->SetKeyDownComponent(NULL);
	}
}

//  item-   
int GComponent::Find(GComponent* child)
{
	for (int i=0;i<k_child;i++)
	{
		if (m_child[i]==child)
			return i;
	}
	return -1;
}

GComponent* GComponent::Find(unsigned int id)
{
	for (int i=0;i<k_child;i++)
	{
		if (m_child[i]->GetID()==id)
			return m_child[i];
	}
	return NULL;
}

//     
void GComponent::AlignToCenter()
{
	if (parent)
	{
		int  wi, he;
		parent->GetSize(wi, he);

		int nx=(wi-width)/2;
		int ny=(he-height)/2;

		if (nx!=x || ny!=y)
		{
			x=nx;
			y=ny;
			parent->Invalidate();
		}
	}
}

//     
void GComponent::AlignToCenterMonitor()
{
	GSurface* backbuffer=game->GetBackbuffer();
	int wi, he;
	backbuffer->GetSize(wi, he);
	int sx=(wi-width)/2;
	int sy=(he-height)/2;
	int ax, ay;
	GetAbsoluteWindowPosition(ax, ay);
	if (ax!=sx || ay!=sy)
	{
		x+=sx-ax;
		y+=sy-ay;
		if (parent)
			parent->Invalidate();
	}
}

//  
void GComponent::SetFocus()
{
	GComponent* focus=game->GetFocusComponent();
	if (focus!=this)
	{
		if (focus)
			focus->SysOnKillFocus();
		game->SetFocusComponent(this);
		SysOnSetFocus();
	}
}

//       
void GComponent::GetInnerRect(CMagicRect* rect)
{
	rect->left=0;
	rect->top=0;
	rect->right=width;
	rect->bottom=height;

	CMagicRect gap;
	theme->GetInnerGap(this, &gap);
	rect->right-=(gap.left+gap.right);
	rect->bottom-=(gap.top+gap.bottom);
}

//    
void GComponent::GetInnerOffset(int& x, int& y)
{
	CMagicRect gap;
	theme->GetInnerGap(this, &gap);
	x=gap.left;
	y=gap.top;
}

//      
void GComponent::GetClientOffset(int& x, int& y)
{
	if (theme)
	{
		CMagicRect gap;
		theme->GetClientGap(&gap);
		x=gap.left;
		y=gap.top;
	}
	else
	{
		x=0;
		y=0;
	}
}

//    
void GComponent::GetClientRect(CMagicRect* rect)
{
	GetInnerRect(rect);
	if (theme)
	{
		CMagicRect gap;
		theme->GetClientGap(&gap);
		rect->right-=(gap.left+gap.right);
		rect->bottom-=(gap.top+gap.bottom);
	}
}

//    
void GComponent::GetAbsoluteWindowPosition(int& x, int& y)
{
	if (parent)
	{
		parent->GetAbsoluteWindowPosition(x, y);
		int ix, iy;
		parent->GetInnerOffset(ix, iy);
		int cx, cy;
		parent->GetClientOffset(cx, cy);
		x+=this->x+ix+cx;
		y+=this->y+iy+cy;
	}
	else
	{
		x=this->x;
		y=this->y;
	}
}

//    
void GComponent::Invalidate()
{
	CMagicRect rect;
	rect.left=0;
	rect.top=0;
	rect.right=width;
	rect.bottom=height;
	InvalidateRect(rect);

	/*
	//if (invalidated!=invalidate)
	{
		invalidated=invalidate;
		if (invalidate)
		{
			game->AddInvalidate(this);

			//if (game->IsKeyHold(KEY_SPACE))
   			//	invalidated=invalidated;

		}
	}
	*/
}

// /    
void GComponent::InvalidateRect(CMagicRect& rect)
{
	if (IsVisibled())
	{
		if (invalidated)
		{
			//    , , ,     
			invalidate_rect.Add(rect);
		}
		else
		{
			invalidated=true;
			invalidate_rect=rect;
		}

		//if (invalidate_rect.top>invalidate_rect.bottom)
		//	invalidate_rect.top=invalidate_rect.top;

		if (theme->IsAlpha(this))
		{
			//    
			if (parent)
			{
				int ax, ay;
				GetAbsoluteWindowPosition(ax, ay);
				CMagicRect r1;
				r1.left=rect.left+ax;
				r1.top=rect.top+ay;
				r1.right=rect.right+ax;
				r1.bottom=rect.bottom+ay;

				int pax, pay;
				parent->GetAbsoluteWindowPosition(pax, pay);
				int pwi, phe;
				parent->GetSize(pwi, phe);
				CMagicRect r2;
				r2.left=pax;
				r2.top=pay;
				r2.right=r2.left+pwi;
				r2.bottom=r2.top+phe;

				CMagicRect r;
				if (r.Intersect(r1, r2) && r==r1)
				{
					r.left-=pax;
					r.top-=pay;
					r.right-=pax;
					r.bottom-=pay;

					parent->InvalidateRect(r);
				}
				else
				{
					//       
					//       
					game->GetScene()->InvalidateRect(r1);
				}
			}
		}
	}
}

//  
void GComponent::Draw()
{
	if (invalidated)
	{
		invalidated=false;

		if (IsVisibled())
		{
			if (IsRecalcLayout())
			{
				//   
				RecalcLayout();
			}

			//    
			int bx=x;
			int by=y;
			GetAbsoluteWindowPosition(bx, by);

			CMagicRect drawn_rect;
			drawn_rect.left=invalidate_rect.left+bx;
			drawn_rect.top=invalidate_rect.top+by;
			drawn_rect.right=invalidate_rect.right+bx;
			drawn_rect.bottom=invalidate_rect.bottom+by;

			GSurface* backbuffer=game->GetBackbuffer();
			CMagicRect clip_rect, vclip;
			clip_rect.left=bx;
			clip_rect.top=by;
			clip_rect.right=bx+width;
			clip_rect.bottom=by+height;
			
			if (vclip.Intersect(drawn_rect, clip_rect))
			{
				backbuffer->SetOffset(bx, by);
				//backbuffer->SetClip(bx, by, width, height);
				backbuffer->SetClip(vclip.left, vclip.top, vclip.Width(), vclip.Height());
				if (WinOnRender() || (!(style & (GS_BORDER | GS_CAPTION))))
				{
					//    
					CMagicRect inner;
					GetInnerRect(&inner);
					int cx, cy;
					GetInnerOffset(cx, cy);
					cx+=bx;
					cy+=by;
					backbuffer->SetOffset(cx, cy);
					int iwi=inner.Width();
					int ihe=inner.Height();
					//backbuffer->SetClip(cx, cy, iwi, ihe);
					clip_rect.left=cx;
					clip_rect.top=cy;
					clip_rect.right=cx+iwi;
					clip_rect.bottom=cy+ihe;
					if (vclip.Intersect(drawn_rect, clip_rect))
					{
						backbuffer->SetClip(vclip.left, vclip.top, vclip.Width(), vclip.Height());
					
						CMagicRect client;
						GetClientRect(&client);
						int dx, dy;
						GetClientOffset(dx, dy);
						int cwi=client.Width();
						int che=client.Height();

						if (!(style & GS_NOBGND))
						{
							if (iwi>cwi || ihe>che)
							{
								//   ,  
								//    
								CMagicRect r;
								if (dx)
								{
									r.left=0;
									r.top=0;
									r.right=dx;
									r.bottom=ihe;
									OnEraseInner(&r);
								}
								
								int dx2=dx+cwi;
								if (dx2<iwi)
								{
									r.left=dx2;
									r.top=0;
									r.right=iwi;
									r.bottom=ihe;
									OnEraseInner(&r);
								}

								if (dy)
								{
									r.left=dx;
									r.top=0;
									r.right=dx2;
									r.bottom=dy;
									OnEraseInner(&r);
								}

								int dy2=dy+che;
								if (dy2<ihe)
								{
									r.left=dx;
									r.top=dy2;
									r.right=dx2;
									r.bottom=ihe;
									OnEraseInner(&r);
								}
							}
						}

						if (InnerOnRender())
						{
							//    
							dx+=cx;
							dy+=cy;
							
							//backbuffer->SetClip(dx, dy, cwi, che);
							clip_rect.left=dx;
							clip_rect.top=dy;
							clip_rect.right=dx+cwi;
							clip_rect.bottom=dy+che;
							if (vclip.Intersect(drawn_rect, clip_rect))
							{
								backbuffer->SetOffset(dx, dy);
								backbuffer->SetClip(vclip.left, vclip.top, vclip.Width(), vclip.Height());
								if (!(style & GS_NOBGND))
									OnEraseClient();
								ClientOnRender();
							}
						}
					}
				}
			
				/*
				for (int i=0;i<k_child;i++)
				{
					if (m_child[i]->IsVisibled())
					{
						m_child[i]->invalidated=true;
						m_child[i]->Draw();
					}
				}
				*/

				backbuffer->SetOffset(0, 0);
				backbuffer->ResetClip();
			}
		}
	}
}

/*
//     
GComponent* GComponent::DetectComponentByCursorRecurse(int& x, int& y, int& place)
{
	if (IsOver(x,y))
	{
		//   
		place=COMPONENT_PLACE_WIN;

		x-=this->x;
		y-=this->y;

		//     
		int vx, vy;
		GetInnerOffset(vx, vy);
		x-=vx;
		y-=vy;

		CMagicRect rect;
		GetInnerRect(&rect);
		if (rect.PtInRect(x, y))
		{
			place=COMPONENT_PLACE_INNER;
			int bx, by;
			GetClientOffset(bx, by);
			x-=bx;
			y-=by;
			GetClientRect(&rect);
			if (rect.PtInRect(x, y))
			{
				place=COMPONENT_PLACE_CLIENT;
				for (int i=k_child-1;i>=0;i--)
				{
					//     
					GComponent* component=m_child[i]->DetectComponentByCursorRecurse(x, y, place);
					if (component)
						return component;
				}
			}
			x+=bx;
			y+=by;
		}
		x+=vx;
		y+=vy;
		return this;
	}
	return NULL;
}
*/

//     
GComponent* GComponent::DetectComponentByCursorRecurse(int& x, int& y, int& place)
{
	if (IsOver(x,y))
	{
		//   
		place=COMPONENT_PLACE_WIN;

		x-=this->x;
		y-=this->y;

		//     
		int vx, vy;
		GetInnerOffset(vx, vy);
		x-=vx;
		y-=vy;

		CMagicRect rect;
		GetInnerRect(&rect);
		if (rect.PtInRect(x, y))
		{
			place=COMPONENT_PLACE_INNER;
			int bx, by;
			GetClientOffset(bx, by);
			x-=bx;
			y-=by;
			GetClientRect(&rect);
			if (rect.PtInRect(x, y))
			{
				place=COMPONENT_PLACE_CLIENT;
				for (int i=k_child-1;i>=0;i--)
				{
					//     
					GComponent* component=m_child[i]->DetectComponentByCursorRecurse(x, y, place);
					if (component)
						return component;
				}

				return this;
			}
			x+=bx;
			y+=by;
		}
		else
		{
			x+=vx;
			y+=vy;
		}
		return this;
	}
	return NULL;
}

//  ,       
bool GComponent::IsOver(int x, int y)
{
	if (!IsVisibled() || (!IsEnabled()))
		return false;
	return (x>=this->x && x<this->x+width && y>=this->y && y<this->y+height);
}

//  
bool GComponent::IsVisibled()
{
	if (style & GS_VISIBLE)
	{
		if (parent)
			return parent->IsVisibled();
		else
			return true;
	}
	return false;
}

void GComponent::Show(bool visibled)
{
	bool is=(IsVisibled()!=visibled);
	if (visibled)
	{
		style=style | GS_VISIBLE;
		if (is)
		{
			RecalcCovering();
			Invalidate();
		}
	}
	else
	{
		ClearLinks();
		style=style & (~GS_VISIBLE);
		if (is)
		{
			RecalcCovering();
			if (parent)
				parent->Invalidate();
		}
	}
}

//  
bool GComponent::IsEnabled()
{
	if (style & GS_DISABLED)
		return false;

	if (parent)
		return parent->IsEnabled();

	return true;
}

void GComponent::Enable(bool enable)
{
	bool is=(IsEnabled()!=enable);

	if (enable)
		style=style & (~GS_DISABLED);
	else
	{
		ClearLinks();
		style=style | GS_DISABLED;
	}

	if (is)
		Invalidate();
}

void GComponent::Fix(bool fix)
{
	if (fixed!=fix)
	{
		fixed=fix;
		Invalidate();
	}
}

//  ,     - ,    
bool GComponent::IsHighlighted()
{
	return (game->GetOverComponent()==this);
}

//  ,   
bool GComponent::IsPressed()
{
	return (game->GetPressedComponent()==this);
}

//  ,     
bool GComponent::IsFocused()
{
	return (game->GetFocusComponent()==this);
}

//  ,    
bool GComponent::IsModal()
{
	return (game->GetModalComponent()==this);
}

//     
void GComponent::BeginModal()
{
	previous_modal=game->GetModalComponent();
	if (!parent)
	{
		is_added_modal=true;
		game->GetScene()->AddChild(this);
	}
	else
		is_added_modal=false;

	game->SetModalComponent(this);
	Show(true);

	int k_child=parent->GetChildCount();
	ChildTo(k_child-1); //     
	RecalcCovering();

	previous_focus=game->GetFocusComponent();
	if (previous_focus!=this)
	{
		if (previous_focus)
		{
			previous_focus->SysOnKillFocus();
			previous_focus->Invalidate();

		}
		game->SetFocusComponent(this);
		SysOnSetFocus();
		Invalidate();
	}
}

//     
void GComponent::EndModal()
{
	game->SetModalComponent(previous_modal);
	previous_modal=NULL;
	Show(false);
	//GScene* scene=game->GetScene();
	//scene->Invalidate();

	if (is_added_modal)
	{
		is_added_modal=false;
		parent->DeleteChild(this, false);
		parent=NULL;
	}

	if (previous_focus!=this)
	{
		game->SetFocusComponent(previous_focus);
		if (previous_focus)
			previous_focus->SysOnSetFocus();
	}
}

//       , , ,    
void GComponent::DetectCovering(int& k_component, GComponent** m_component)
{
	//if (IsVisibled())
	{
		int i;
		int start_component=k_component;
		for (i=0;i<k_child;i++)
		{
			if (m_child[i]->IsVisibled() && (!m_child[i]->IsModal()))
			{
				m_component[k_component]=m_child[i];
				k_component++;
			}
		}

		int k=k_component;
		for (i=start_component;i<k;i++)
			m_component[i]->DetectCovering(k_component, m_component);
	}
}

//   
void GComponent::ClearCoveringComponents()
{
	k_covering=0;
}

//   
void GComponent::AddCoveringComponent(GComponent* component)
{
	if (k_covering==max_covering)
	{
		if (m_covering)
		{
			GComponent** vm_covering=new GComponent*[max_covering+1];
			for (int i=0;i<max_covering;i++)
				vm_covering[i]=m_covering[i];
			delete []m_covering;
			m_covering=	vm_covering;
		}
		else
			m_covering=new GComponent*[1];
		max_covering++;
	}

	m_covering[k_covering]=component;
	k_covering++;
}

//     
void GComponent::RecalcCovering()
{
	GComponent* scene=NULL;

	GComponent* next=parent;
	while(next)
	{
		scene=next;
		next=next->GetParent();
	}

	if (scene && scene->IsScene())
		((GScene*)scene)->SetCoveringFlag();
}

//    ,   
void GComponent::InvalidateCoveringComponents()
{
	if (k_covering)
	{
		int ax, ay;
		GetAbsoluteWindowPosition(ax, ay);
		CMagicRect r=invalidate_rect;
		r.left+=ax;
		r.top+=ay;
		r.right+=ax;
		r.bottom+=ay;

		int i=0;
		do 
		{
			GComponent* c=m_covering[i];
			int cx, cy;
			c->GetAbsoluteWindowPosition(cx, cy);
			int wi, he;
			c->GetSize(wi, he);
			CMagicRect r2;
			r2.left=cx;
			r2.top=cy;
			r2.right=r2.left+wi;
			r2.bottom=r2.top+he;

			CMagicRect rr;
			bool is=rr.Intersect(r, r2);
			//assert(is);

			if (is)
			{
				rr.left-=cx;
				rr.top-=cy;
				rr.right-=cx;
				rr.bottom-=cy;

				c->InvalidateRect(rr);
			}

			i++;
		}
		while (i<k_covering);
	}
}

//        
void GComponent::CorrectSize(CMagicRect* rect)
{
	if (k_child>0)
	{
		int min_x=10000000;
		int min_y=10000000;
		int max_x=-10000000;
		int max_y=-10000000;

		for (int i=0;i<k_child;i++)
		{
			GComponent* child=GetChild(i);

			int x, y;
			child->GetPosition(x, y);
			if (min_x>x)
				min_x=x;
			if (min_y>y)
				min_y=y;

			int wi, he;
			child->GetSize(wi, he);

			int x2=x+wi;
			int y2=y+he;
			if (max_x<x2)
				max_x=x2;
			if (max_y<y2)
				max_y=y2;
		}

		CMagicRect client;
		GetClientRect(&client);
		int cwi=client.Width();
		int che=client.Height();

		int wi=max_x-min_x;
		int he=max_y-min_y;
		wi+=rect->left+rect->right;
		he+=rect->top+rect->bottom;

		int rx=wi-cwi;
		int ry=he-che;

		width+=rx;
		height+=ry;

		for (int i=0;i<k_child;i++)
		{
			GComponent* child=GetChild(i);
			int x, y;
			child->GetPosition(x, y);
			x+=rect->left-min_x;
			y+=rect->top-min_y;
			child->SetPosition(x, y);
		}
		Invalidate();
	}
}

//        
void GComponent::CorrectSize(int gap)
{
	CMagicRect rect;
	rect.left=gap;
	rect.top=gap;
	rect.right=gap;
	rect.bottom=gap;
	CorrectSize(&rect);
}

//            
void GComponent::ChildTo(int index)
{
	if (parent)
	{
		int k=parent->GetChildCount();
		if (index>=0 && index<k)
		{
			int pos=parent->Find(this);
			if (pos!=-1)
			{
				if (index>pos)
				{
					//   
					for (int i=pos+1;i<index;i++)
						parent->m_child[i-1]=parent->m_child[i];
				}
				else
				{
					if (index<pos)
					{
						for (int i=index-1;i>=pos;i--)
							parent->m_child[i+1]=parent->m_child[i];
					}
				}

				parent->m_child[index]=this;

				//GComponent* v=parent->m_child[index];
				//parent->m_child[index]=parent->m_child[pos];
				//parent->m_child[pos]=v;
			}
		}
	}
}

//  item-
void GComponent::RenderItem(GItem* item)
{
	theme->RenderItem(item);
}

//      
GComponent* GComponent::CorrectPositionClientForCaptured(int& x, int& y)
{
	int bx, by;
	GetAbsoluteWindowPosition(bx, by);
	x-=bx;
	y-=by;

	int vx, vy;
	GetInnerOffset(vx, vy);
	x-=vx;
	y-=vy;

	return this;
}

//      
GComponent* GComponent::CorrectPositionClientForModal(int& x, int& y)
{
	//return CorrectPositionClientForCaptured(x, y);
	return this;
}

bool GComponent::WinOnRender()
{
	bool is=theme->RenderWin(this);
	OnRenderWIN();
	return is;
}

void GComponent::OnEraseInner(CMagicRect* rect)
{
	if (theme)
		theme->EraseInnerRect(this, rect);
}

void GComponent::OnEraseClient()
{
	if (theme)
		theme->EraseClient(this);
}

// ------------------------------------------------------------------------------------

GButton::GButton() : GComponent()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_BUTTON);
}

bool GButton::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text)
{
	if (GComponent::Create(parent, style, x, y, width, height, id))
	{
		if (text)
			this->text=text;
		return true;
	}
	return false;
}

bool GButton::InnerOnRender()
{
	return theme->RenderInner(this);
}

void GButton::ClientOnRender()
{
	theme->RenderClient(this);
	OnRender();
}

void GButton::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	Fix(true);
	OnLButtonDown(nFlags, x, y);
}

void GButton::ClientOnLButtonUp(unsigned int nFlags, int x, int y)
{
	Fix(false);
	OnLButtonUp(nFlags, x, y);

	//ClientOnBtnClk();
}

void GButton::ClientOnLButtonClick(unsigned int nFlags, int x, int y)
{
	if (parent && parent->IsDialog())
	{
		GDialog* dlg=(GDialog*)parent;
		if (dlg->GetEnterButton()==this)
		{
			dlg->OnEnter();
		}
		else
			if (dlg->GetEscapeButton()==this)
			{
				dlg->OnCancel();
			}
	}
	OnLButtonClick(nFlags, x, y);
}

void GButton::GlobalOnMouseEnter(unsigned int nFlags, int x, int y)
{
	OnMouseEnter(nFlags, x,y);
	Invalidate();
}

void GButton::GlobalOnMouseLeave(unsigned int nFlags)
{
	OnMouseLeave(nFlags);
	Invalidate();
}

// ------------------------------------------------------------------------------------

GButtonImage::GButtonImage()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_BUTTON_IMAGE);
}

bool GButtonImage::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, GSurface* normal, GSurface* pressed, GSurface* highlight, GSurface* disabled)
{
	if (GComponent::Create(parent, style, x, y, width, height, id))
	{
		int border_size=theme->GetBorderSize(this);

		int wi=width-border_size*2;
		int he=height-border_size*2;

		list.Create(wi, he);
		list.AddImage(normal);

		int nwi, nhe;
		normal->GetSize(nwi, nhe);

		if (pressed)
		{
			border_color_pressed=theme->GetBorderColor(this);
			list.AddImage(pressed);
		}
		else
			list.AddImage(normal);

		if (highlight)
		{
			border_color_highlighted=theme->GetBorderColor(this);
			list.AddImage(highlight);
		}
		else
			list.AddImage(normal);

		if (disabled)
		{
			border_color_disabled=theme->GetBorderColor(this);
			list.AddImage(disabled);
		}
		else
		{
			list.AddImage(normal);
		}

		return true;
	}
	return false;
}

// ------------------------------------------------------------------------------------

GCheckBox::GCheckBox() : GComponent()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_CHECKBOX);
	checked=false;
}

bool GCheckBox::Create(GComponent* parent, unsigned int style, int x, int y, unsigned int id, const char* text)
{
	int width, height;
	theme->GetMinSize(this, width, height);
	if (GComponent::Create(parent, style, x, y, width, height, id))
	{
		if (text)
			this->text=text;

		RecalcSize();

		return true;
	}
	return false;
}

void GCheckBox::SetFont(int font)
{
	if (GComponent::SetFont(font))
		RecalcSize();
}

void GCheckBox::ClientOnRender()
{
	theme->RenderClient(this);
	OnRender();
}


void GCheckBox::SetWindowText(const char* text)
{
	if (strcmp(text, this->text.c_str()))
	{
		this->text=text;
		RecalcSize();
		SysOnChangeText();
		Invalidate();
	}
}

void GCheckBox::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	checked=!checked;

	OnLButtonDown(nFlags, x, y);
	Invalidate();
}

void GCheckBox::GlobalOnMouseEnter(unsigned int nFlags, int x, int y)
{
	OnMouseEnter(nFlags, x,y);
	Invalidate();
}

void GCheckBox::GlobalOnMouseLeave(unsigned int nFlags)
{
	OnMouseLeave(nFlags);
	Invalidate();
}

void GCheckBox::SetCheck(bool check)
{
	if (checked!=check)
	{
		checked=check;
		Invalidate();
	}
}

//   
void GCheckBox::RecalcSize()
{
	int check_wi, check_he;
	theme->GetMinSize(this, check_wi, check_he);

	int gap=theme->GetAdditionalGap();

	int font=theme->GetTextFont(this);

	int twi, the;
	GSurface::device->GetTextSize(text, font, twi, the);

	int wi=check_wi+gap+twi;
	int he;
	if (check_he<the)
		he=the;
	else
		he=check_he;

	width=wi;
	height=he;

	CMagicRect rect;
	GetClientRect(&rect);

	int rx=width-rect.Width();
	int ry=height-rect.Height();

	width+=rx;
	height+=ry;
}

// ------------------------------------------------------------------------------------

CCButtonImage::CCButtonImage() : GComponent()
{
	font=-1;
	tx=ty=DEFAULT_POSITION;
	sx_fixed=sy_fixed=1;
	normal_image=fixed_image=disabled_image=over_image=NULL;

	is_over=false;
}

bool CCButtonImage::Create(GComponent* parent, int x, int y, unsigned int id, const char* text, int font, GImage* normal_image, GImage* fixed_image, GImage* disabled_image, GImage* over_image)
{
	if (normal_image)
	{
		this->id=id;

		if (text)
		{
			this->text=text;
			this->font=font;
		}

		this->x=x;
		this->y=y;

		if (!fixed_image)
			fixed_image=normal_image;
		if (!disabled_image)
			disabled_image=normal_image;

		this->normal_image=normal_image;
		this->fixed_image=fixed_image;
		this->disabled_image=disabled_image;
		this->over_image=over_image;

		normal_image->GetSize(width, height);

		parent->AddChild(this);

		return true;
	}
	return false;
}

//        
void CCButtonImage::SetTextPosition(int tx, int ty, int sx_fixed, int sy_fixed)
{
	this->tx=tx;
	this->ty=ty;
	this->sx_fixed=sx_fixed;
	this->sy_fixed=sy_fixed;
	if (this->sx_fixed==DEFAULT_POSITION)
		this->sx_fixed=1;
	if (this->sy_fixed==DEFAULT_POSITION)
		this->sy_fixed=1;
}

//  ,       
bool CCButtonImage::IsOver(int x, int y)
{
	if (IsVisibled() && IsEnabled())
	{
		GImage* pict=GetCurrentPicture();
		x-=this->x;
		y-=this->y;
		return pict->IsOver(x, y);
	}
	return false;
}

void CCButtonImage::ClientOnRender()
{
	GImage* pict=GetCurrentPicture();

	int bx=0;
	int by=0;

	GSurface* backbuffer=game->GetBackbuffer();

	pict->Blt(bx, by, backbuffer);

	if (!text.IsEmpty())
	{
		if (tx==DEFAULT_POSITION || ty==DEFAULT_POSITION)
		{
			int twi, the;
			game->device->GetTextSize(text, font, twi, the);

			int pwi, phe;
			pict->GetSize(pwi, phe);
			if (tx==DEFAULT_POSITION)
				tx=(pwi-twi)/2;
			if (ty==DEFAULT_POSITION)
				ty=(phe-the)/2;
		}

		int vx=bx+tx;
		int vy=by+ty;
		if (IsFixed())
		{
			vx+=sx_fixed;
			vy+=sy_fixed;
		}

		int pivot_x, pivot_y;
		pict->GetPivot(pivot_x, pivot_y);
		vx-=pivot_x;
		vy-=pivot_y;

		backbuffer->TextOut(vx, vy, GetWindowText(), font, 0xffffff);
	}

	OnRender();
}

void CCButtonImage::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	Fix(true);
	OnLButtonDown(nFlags, x, y);
}

void CCButtonImage::ClientOnLButtonUp(unsigned int nFlags, int x, int y)
{
	Fix(false);
	OnLButtonUp(nFlags, x, y);
}

void CCButtonImage::GlobalOnMouseEnter(unsigned int nFlags, int x, int y)
{
	if (over_image)
	{
		is_over=true;
		OnMouseEnter(nFlags, x,y);
		Invalidate();
	}
}

void CCButtonImage::GlobalOnMouseLeave(unsigned int nFlags)
{
	if (over_image)
	{
		is_over=false;
		OnMouseLeave(nFlags);
		Invalidate();
	}
}

//   
GImage* CCButtonImage::GetCurrentPicture()
{
	if (IsEnabled())
	{
		if (IsFixed())
			return fixed_image;

		if (is_over)
			return over_image;

		return normal_image;
	}
	return disabled_image;
}

// ------------------------------------------------------------------------------------

GScroller::GScroller() : GComponent()
{
	scroll_x=scroll_y=0;
	max_scroll_x=max_scroll_y=0;

	factor_hscroll=factor_vscroll=1;
	hslider_size=vslider_size=0;

	normal_color=0xd0d0d0;
	pressed_color=0x606060;
	over_color=0x808080;
	slider_color=0xa0a0a0;

	captured_scroller=SCROLLER_NO;
	over_scroller=SCROLLER_NO;

	slider_x=slider_y=-1;

	is_recalc=true;
}

void GScroller::SetScrollBarColor(unsigned int normal, unsigned int pressed, unsigned int over, unsigned int slider)
{
	normal_color=normal;
	pressed_color=pressed;
	over_color=over;
	slider_color=slider;
}

//    
void GScroller::GetClientRect(CMagicRect* rect)
{
	GetInnerRect(rect);

	if (theme)
	{
		CMagicRect gap;
		theme->GetClientGap(&gap);
		rect->right-=(gap.left+gap.right);
		rect->bottom-=(gap.top+gap.bottom);
	}

	if (IsHScrollBar())
		rect->bottom-=GetHScrollBarSize();

	if (IsVScrollBar())
		rect->right-=GetVScrollBarSize();
}

//        
int GScroller::DetectScrollElement(int x, int y)
{
	CMagicPoint pt(x, y);
	CMagicRect rect;
	GetClientRect(&rect);
	if (rect.PtInRect(pt))
		return SCROLLER_NO;

	GetInnerRect(&rect);
	if (rect.PtInRect(pt))
	{
		if (IsHScrollBar())
		{
			GetHScrollButton1Rect(&rect);
			if (rect.PtInRect(pt))
				return SCROLLER_HBUTTON1;

			GetHScrollButton2Rect(&rect);
			if (rect.PtInRect(pt))
				return SCROLLER_HBUTTON2;

			GetHScrollSliderRect(&rect);
			if (rect.PtInRect(pt))
				return SCROLLER_HSLIDER;
		}

		if (IsVScrollBar())
		{
			GetVScrollButton1Rect(&rect);
			if (rect.PtInRect(pt))
				return SCROLLER_VBUTTON1;

			GetVScrollButton2Rect(&rect);
			if (rect.PtInRect(pt))
				return SCROLLER_VBUTTON2;

			GetVScrollSliderRect(&rect);
			if (rect.PtInRect(pt))
				return SCROLLER_VSLIDER;
		}

		if (IsHScrollBar() || (IsVScrollBar()))
			return SCROLLER_PLACE;
	}

	return SCROLLER_NO;
}

//  
void GScroller::RecalcLayout()
{
	is_recalc=false;

	hslider_size=vslider_size=0;
	
	int wi_full, he_full;
	GetScrollSize(wi_full, he_full);

	CMagicRect rect;
	GetClientRect(&rect);
	int wi=rect.Width();
	int he=rect.Height();
	int rwi=wi-wi_full;
	hslider_size=(rwi<0);
	int rhe=he-he_full;
	vslider_size=(rhe<0);

	//            
	GetClientRect(&rect);
	wi=rect.Width();
	he=rect.Height();
	rwi=wi-wi_full;
	hslider_size=(rwi<0);
	rhe=he-he_full;
	vslider_size=(rhe<0);

	//            
	GetClientRect(&rect);
	wi=rect.Width();
	he=rect.Height();
	rwi=wi-wi_full;
	hslider_size=(rwi<0);
	rhe=he-he_full;
	vslider_size=(rhe<0);
	
	/*
	if (IsVScrollBar())
	{
		wi-=GetVScrollBarSize();
		rwi=wi-wi_full;
		hslider_size=(rwi<0);
	}

	if (IsHScrollBar())
	{
		he-=GetHScrollBarSize();
		rhe=he-he_full;
		vslider_size=(rhe<0);
	}
	*/



	//  rwi, rhe   ,    
	//    ,     
	if (IsHScrollBar())
	{
		//    
		CMagicRect b1, b2;
		GetHScrollButton1Rect(&b1);
		GetHScrollButton2Rect(&b2);
		int full_free_place=b2.left-b1.right;

		hslider_size=(int)(((double)full_free_place)*((double)wi)/((double)wi_full));
		int min_slider=GetMinSlider();
		if (hslider_size<min_slider)
			hslider_size=min_slider;

		int free_place=full_free_place-hslider_size;

		max_scroll_x=-rwi;
		factor_hscroll=((double)max_scroll_x)/((double)free_place);

		if (scroll_x>max_scroll_x)
			scroll_x=max_scroll_x;
	}
	else
	{
		scroll_x=max_scroll_x=0;
	}

	if (IsVScrollBar())
	{
		//    
		CMagicRect b1, b2;
		GetVScrollButton1Rect(&b1);
		GetVScrollButton2Rect(&b2);
		int full_free_place=b2.top-b1.bottom;

		vslider_size=(int)(((double)full_free_place)*((double)he)/((double)he_full));
		int min_slider=GetMinSlider();
		if (vslider_size<min_slider)
			vslider_size=min_slider;

		int free_place=full_free_place-vslider_size;

		max_scroll_y=-rhe;
		factor_vscroll=((double)max_scroll_y)/((double)free_place);

		if (scroll_y>max_scroll_y)
			scroll_y=max_scroll_y;
	}
	else
		scroll_y=max_scroll_y=0;


	if (IsHScrollBar())
	{
		CMagicRect b1, b2, slider;
		GetHScrollButton1Rect(&b1);
		GetHScrollButton2Rect(&b2);
		GetHScrollSliderRect(&slider);

		bool disable=false;
		if (b1.right>slider.left)
			disable=true;
		else
			if (slider.right>b2.left)
				disable=true;
		if (disable)
		{
			//     
			hslider_size=0;
			scroll_x=max_scroll_x=0;
		}
		//else
		//	max_scroll_y+=b1.Height();
	}

	if (IsVScrollBar())
	{
		CMagicRect b1, b2, slider;
		GetVScrollButton1Rect(&b1);
		GetVScrollButton2Rect(&b2);
		GetVScrollSliderRect(&slider);

		bool disable=false;
		if (b1.bottom>slider.top)
			disable=true;
		else
			if (slider.bottom>b2.top)
				disable=true;
		if (disable)
		{
			//     
			vslider_size=0;
			scroll_y=max_scroll_y=0;
		}
		//else
		//	max_scroll_x+=b1.Width();
	}
}

bool GScroller::InnerOnRender()
{
	theme->RenderScroller(this);
	return true;
}

void GScroller::InnerOnLButtonDown(unsigned int nFlags, int x, int y)
{
	captured_scroller=DetectScrollElement(x, y);
	over_scroller=SCROLLER_NO;

	switch (captured_scroller)
	{
	case SCROLLER_HBUTTON1:
		//  
		scroll_x-=GetHScrollDelta();
		if (scroll_x<0)
			scroll_x=0;
		Invalidate();
		game->SetRepeatedKey(KEY_LMOUSE);
		break;

	case SCROLLER_HBUTTON2:
		//  
		scroll_x+=GetHScrollDelta();
		if (scroll_x>max_scroll_x)
			scroll_x=max_scroll_x;
		Invalidate();
		game->SetRepeatedKey(KEY_LMOUSE);
		break;

	case SCROLLER_HSLIDER:
		//  
		{
			CMagicRect rect;
			GetHScrollSliderRect(&rect);
			slider_x=x-rect.left;
			slider_y=y-rect.top;
			Invalidate();
		}
		break;

	case SCROLLER_VBUTTON1:
		//  
		scroll_y-=GetVScrollDelta();
		if (scroll_y<0)
			scroll_y=0;
		Invalidate();
		game->SetRepeatedKey(KEY_LMOUSE);
		break;

	case SCROLLER_VBUTTON2:
		//  
		scroll_y+=GetVScrollDelta();
		if (scroll_y>max_scroll_y)
			scroll_y=max_scroll_y;
		Invalidate();
		game->SetRepeatedKey(KEY_LMOUSE);
		break;

	case SCROLLER_VSLIDER:
		//  
		{
			CMagicRect rect;
			GetVScrollSliderRect(&rect);
			slider_x=x-rect.left;
			slider_y=y-rect.top;
			Invalidate();
		}
		break;
	}
}

void GScroller::InnerOnLButtonUp(unsigned int nFlags, int x, int y)
{
	captured_scroller=SCROLLER_NO;
	over_scroller=DetectScrollElement(x, y);
	Invalidate();
}

void GScroller::InnerOnMouseMove(unsigned int nFlags, int x, int y)
{
	if (captured_scroller==SCROLLER_HSLIDER)
	{
		CMagicRect slider;
		GetHScrollSliderRect(&slider);
		int r=x-slider.left;
		int rr=r-slider_x;
		CMagicRect b1;
		GetHScrollButton1Rect(&b1);
		int pos=slider.left+rr-b1.right;
		int scroll_x2=(int)(pos*factor_hscroll+0.5);
		if (scroll_x2<0)
			scroll_x2=0;
		else
			if (scroll_x2>max_scroll_x)
				scroll_x2=max_scroll_x;
		if (scroll_x!=scroll_x2)
		{
			scroll_x=scroll_x2;
			Invalidate();
		}
	}
	else
	{
		if (captured_scroller==SCROLLER_VSLIDER)
		{
			CMagicRect slider;
			GetVScrollSliderRect(&slider);
			int r=y-slider.top;
			int rr=r-slider_y;
			CMagicRect b1;
			GetVScrollButton1Rect(&b1);
			int pos=slider.top+rr-b1.bottom;
			int scroll_y2=(int)(pos*factor_vscroll+0.5);
			if (scroll_y2<0)
				scroll_y2=0;
			else
				if (scroll_y2>max_scroll_y)
					scroll_y2=max_scroll_y;
			if (scroll_y!=scroll_y2)
			{
				scroll_y=scroll_y2;
				Invalidate();
			}
		}
		else
		{
			if (game->GetCapturedComponent()==NULL)
			{
				//   
				//     ,    
				int over_scroller_new=DetectScrollElement(x, y);
				if (over_scroller!=over_scroller_new)
				{
					over_scroller=over_scroller_new;
					Invalidate();
				}
			}
		}
	}
}

void GScroller::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	if (game->GetCapturedComponent()==NULL)
	{
		//  
		if (over_scroller!=SCROLLER_NO)
		{
			over_scroller=SCROLLER_NO;
			Invalidate();
		}
	}
}

void GScroller::InnerOnMouseLeave(unsigned int nFlags)
{
	if (over_scroller!=SCROLLER_NO)
	{
		over_scroller=SCROLLER_NO;
		Invalidate();
	}
}

bool GScroller::WinOnMouseWheel(short zDelta)
{
	if (IsVScrollBar())
	{
		int scroll_y2=scroll_y-zDelta/3;
		if (scroll_y2<0)
			scroll_y2=0;
		else
			if (scroll_y2>max_scroll_y)
				scroll_y2=max_scroll_y;
		if (scroll_y!=scroll_y2)
		{
			scroll_y=scroll_y2;
			Invalidate();
		}
	}
	return false;
}

//    
int GScroller::GetHScrollBarSize()
{
	if (theme)
		return theme->GetSBSize();
	return 18;
}
int GScroller::GetVScrollBarSize()
{
	if (theme)
		return theme->GetSBSize();
	return 18;
}

//       
void GScroller::GetHScrollButton1Rect(CMagicRect* rect)
{
	if (IsHScrollBar())
	{
		int bsize=GetHScrollBarSize();
		GetInnerRect(rect);
		rect->right=rect->left+bsize;
		rect->top=rect->bottom-bsize;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//       
void GScroller::GetHScrollButton2Rect(CMagicRect* rect)
{
	if (IsHScrollBar())
	{
		int bsize=GetHScrollBarSize();
		GetInnerRect(rect);
		if (IsVScrollBar())
			rect->right-=bsize;
		rect->left=rect->right-bsize;
		rect->top=rect->bottom-bsize;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//       
void GScroller::GetVScrollButton1Rect(CMagicRect* rect)
{
	if (IsVScrollBar())
	{
		int bsize=GetVScrollBarSize();
		GetInnerRect(rect);
		rect->left=rect->right-bsize;
		rect->bottom=rect->top+bsize;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//       
void GScroller::GetVScrollButton2Rect(CMagicRect* rect)
{
	if (IsVScrollBar())
	{
		int bsize=GetVScrollBarSize();
		GetInnerRect(rect);
		if (IsHScrollBar())
			rect->bottom-=bsize;
		rect->left=rect->right-bsize;
		rect->top=rect->bottom-bsize;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//      
void GScroller::GetHScrollSliderRect(CMagicRect* rect)
{
	if (IsHScrollBar())
	{
		int pos=(int)(((double)scroll_x)/factor_hscroll+0.5);
		GetHScrollButton1Rect(rect);
		rect->left=rect->right+pos;
		rect->right=rect->left+hslider_size;
		rect->bottom--;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//      
void GScroller::GetVScrollSliderRect(CMagicRect* rect)
{
	if (IsVScrollBar())
	{
		int pos=(int)(((double)scroll_y)/factor_vscroll+0.5);
		GetVScrollButton1Rect(rect);
		rect->top=rect->bottom+pos;
		rect->bottom=rect->top+vslider_size;
		rect->right--;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//       
void GScroller::GetScrollSquareRect(CMagicRect* rect)
{
	if (IsHScrollBar() && IsVScrollBar())
	{
		GetHScrollButton2Rect(rect);
		rect->left=rect->right;
		rect->right=rect->left+GetVScrollBarSize();
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//   
void GScroller::SetHScroll(int pos)
{
	if (pos<0)
		scroll_x=0;
	else
		if (pos>max_scroll_x)
			scroll_x=max_scroll_x;
		else
			scroll_x=pos;
}

void GScroller::SetVScroll(int pos)
{
	if (pos<0)
		scroll_y=0;
	else
		if (pos>max_scroll_y)
			scroll_y=max_scroll_y;
		else
			scroll_y=pos;
}

// ------------------------------------------------------------------------------------

GItem::GItem(int width, int height)
{
	owner=NULL;
	previous=NULL;
	next=NULL;

	this->width=width;
	this->height=height;

	user=0;
	user_struct=NULL;

	y=0;

	enabled=true;

	sort=0;
}

GItem::~GItem()
{
	if (user_struct)
	{
		free(user_struct);
		user_struct=NULL;
	}

	if (next && (!next->GetParent()))
	{
		delete next;
		next=NULL;
	}
}

void GItem::GetItemPosition(int& x, int& y)
{
	int scroll_x, scroll_y;
	owner->GetScrollPosition(scroll_x, scroll_y);
	x=-scroll_x;
	y=this->y-scroll_y;
}

void GItem::GetItemRect(CMagicRect* rect)
{
	GetItemPosition(rect->left, rect->top);
	rect->right=rect->left+width;
	rect->bottom=rect->top+height;
}

/*
//   item-    ,  
//    
void GItem::GetItemThemeRect(CMagicRect* rect)
{
	GThemeComponent* theme=owner->GetThemeComponent();

	bool is=theme->GetItemTextRect(this, rect);
	int icon=GetIconIndex();
	if (icon!=-1)
	{
		CMagicRect irect;
		if (theme->GetItemIconRect(this, &irect))
		{
			if (rect->left>irect.left)
				rect->left=irect.left;
			if (rect->top>irect.top)
				rect->top=irect.top;
			if (rect->right<irect.right)
				rect->right=irect.right;
			if (rect->bottom<irect.bottom)
				rect->bottom=irect.bottom;
		}
	}

	rect->bottom=rect->top+GetHeight();
	//CMagicRect gap;
	//theme->GetItemGapRect(this, &gap);
	//rect->bottom+=gap.top+gap.bottom;
}
*/

//   item-   
void GItem::GetItemThemeSize(int& wi, int& he)
{
	GThemeComponent* theme=owner->GetThemeComponent();

	CMagicRect rect;
	theme->GetItemTextRect(this, &rect);
	wi=rect.right;
	he=GetHeight();
}

void GItem::Enable(bool enable)
{
	if (enable!=enabled)
	{
		if (IsEnabled()!=enable)
		{
			if (owner)
				owner->Invalidate();
		}
		enabled=enable;
	}
}

GImage* GItem::GetIcon()
{
	GImageList* il=owner->GetIcons();
	if (il)
	{
		int index=GetIconIndex();
		return il->GetImage(index);
	}
	return NULL;
}

bool GItem::IsEnabled()
{
	if (enabled)
	{
		if (owner)
			return owner->IsEnabled();
		return true;
	}
	return false;
}

bool GItem::IsOver(int& x, int& y)
{
	if (IsEnabled())
	{
		CMagicRect rect;
		GetItemRect(&rect);
		GThemeComponent* theme=owner->GetThemeComponent();

		bool is=(y>=rect.top && y<rect.bottom);
		
		if (is)
		{
			x-=rect.left;
			y-=rect.top;
			return true;
		}
	}
	return false;
}

bool GItem::SysOnLButtonDown(unsigned int nFlags, int x, int y)
{
	if (this!=owner->GetSelected())
	{
		owner->SetSelected(this);
		owner->OnChangeSelected(this);
		owner->Invalidate();
	}
	OnLButtonDown(nFlags, x, y);
	return false;
}

bool GItem::SysOnLButtonDblClk(unsigned int nFlags, int x, int y)
{
	OnLButtonDblClk(nFlags, x, y);
	return false;
}

//  ,   item 
bool GItem::IsSelected()
{
	//return (owner->GetSelected()==this);
	return (owner->selected==this);
}

void GItem::SetUserStruct(int length, void* data)
{
	if (user_struct)
	{
		free(user_struct);
		user_struct=NULL;
	}

	if (data && length>0)
	{
		user_struct=malloc(length);
		memcpy(user_struct, data, length);
	}
}

// ------------------------------------------------------------------------------------

GListBase::GListBase() : GScroller()
{
	icons=NULL;

	first=NULL;
	selected=NULL;
	visibled=NULL;
	visibled_scroll_y=-1;
	visibled_y=-1;

	item_width=item_height=0;
	tab=3;

	//       
	pushed_scroll_item=NULL;
	pushed_scroll_x=pushed_scroll_y=NULL;
}

GListBase::~GListBase()
{
	ClearItems();
}

//  item-
void GListBase::ClearItems()
{
	if (first)
	{
		delete first;
		first=NULL;
	}
	selected=NULL;
	visibled=NULL;
	item_width=item_height=0;
	scroll_x=scroll_y=0;
	is_recalc=true;

	pushed_scroll_item=NULL;
}

//  ,    
bool GListBase::IsRecalcLayout()
{
	if (GScroller::IsRecalcLayout())
		return true;

	//       item-
	if (scroll_y!=visibled_scroll_y || (!visibled) || visibled->GetY()!=visibled_y)
		DetectFirstVisibledItem();

	return false;
}

//  
void GListBase::RecalcLayout()
{
	GScroller::RecalcLayout();
	DetectFirstVisibledItem();
}

//   item-
void GListBase::InsertItem(GItem* item, GItem* previous)
{
	is_recalc=true;

	if (!item->GetOwner())
	{
		item->SetOwner(this);
		((GItemText*)item)->RefreshItemSize();
	}

	GItem* next;
	if (previous)
	{
		//    
		next=previous->GetNext();
		int y=previous->GetY();
		int he=previous->GetHeight();
		item->SetY(y+he);
		previous->SetNext(item);
	}
	else
	{
		//   
		next=first;
		first=item;
		item->SetY(0);
	}

	if (next)
		next->SetPrevious(item);

	item->SetPrevious(previous);
	item->SetNext(next);

	int wi, he;
	item->GetItemThemeSize(wi, he);
	//int wi=item->GetWidth();
	//int he=item->GetHeight();

	while(next)
	{
		int y=next->GetY();
		y+=he;
		next->SetY(y);
		next=next->GetNext();
	}

	item_height+=he;
	if (item_width<wi)
		item_width=wi;

	Invalidate();
}

//  item-
void GListBase::DeleteItem(GItem* item)
{
	is_recalc=true;

	bool is_first=false;
	GItem* previous=item->GetPrevious();
	GItem* next=item->GetNext();
	if (previous)
	{
		//    item  
		previous->SetNext(next);
		if (next)
			next->SetPrevious(previous);
	}
	else
	{
		//   item
		first=next;
		is_first=true;
	}

	int wi, he;
	item->GetItemThemeSize(wi, he);
	//int wi=item->GetWidth();
	//int he=item->GetHeight();

	while(next)
	{
		int y=next->GetY();
		y-=he;
		next->SetY(y);
		next=next->GetNext();
	}

	if (selected==item)
	{
		GItem* parent=item->GetParent();
		if (parent)
		{
			SetSelected(parent);
		}
		else
		{
			selected=item->GetNext();
			if (!selected)
				selected=item->GetPrevious();
		}
		OnChangeSelected(selected);
	}

	if (visibled==item)
		visibled=first;

	item_height-=he;
	if (item_width==wi)
	{
		//  item   
		//    item,    
		item_width=0;
		next=first;
		while(next)
		{
			wi=next->GetWidth();
			if (item_width<wi)
				item_width=wi;
			next=next->GetNext();
		}
	}

	item->SetPrevious(NULL);
	item->SetNext(NULL);

	if (is_first && first)
		first->SetPrevious(NULL);

	if (game)
		Invalidate();
}

//    item-
void GListBase::DetectFirstVisibledItem()
{
	if (scroll_y>0)
	{
		if (!visibled)
			visibled=first;

		int y=visibled->GetY();
		if (y+scroll_y<0)
		{
			//  visibled   ,   
			GItem* item=visibled->GetNext();
			while(item)
			{
				y=item->GetY();
				if (y+scroll_y>0)
				{
					//  item     ,    
					break;
				}

				visibled=item;
				item=visibled->GetNext();
			}
		}
		else
		{
			//  visibled   ,   
			GItem* item=visibled->GetPrevious();
			while(item)
			{
				y=item->GetY();
				if (y+scroll_y<0)
				{
					//  item     ,    
					break;
				}

				visibled=item;
				item=visibled->GetPrevious();
			}
		}
	}
	else
		visibled=first;

	visibled_scroll_y=scroll_y;
	if (visibled)
		visibled_y=visibled->GetY();
	else
		visibled_y=-1;
}

//       item- (, , )
void GListBase::CorrectingForChanging(GItem* item, int old_wi, int old_he)
{
	if (item->GetPrevious() || item->GetNext() || item==first) //   ,   item ,      item-    
	{
		int wi, he;
		item->GetItemThemeSize(wi, he);

		int rx=wi-old_wi;
		int ry=he-old_he;

		if (rx || ry)
		{
			if (item_width<wi)
				item_width=wi;
			else
			{
				if (item_width==wi)
				{
					//    item-,  ,    
					item_width=0;
					GItem* next=first;
					while (next)
					{
						int wi, he;
						item->GetItemThemeSize(wi, he);
						if (item_width<wi)
							item_width=wi;
						next=next->GetNext();
					}
				}
			}

			item_height+=ry;
			GItem* next=item->GetNext();
			while(next)
			{
				int y=next->GetY();
				y+=ry;
				next->SetY(y);
				next=next->GetNext();
			}

			RecalcLayout();
		}
	}
}

//    item
GItem* GListBase::GetFirstVisibledItem()
{
	CMagicRect rect;
	GetClientRect(&rect);
	found_scroll_y2=scroll_y+rect.Height()-1;
	found_item=visibled;
	return found_item;
}

//    item
GItem* GListBase::GetNextVisibledItem()
{
	found_item=found_item->GetNext();
	if (found_item)
	{
		int y=found_item->GetY();
		if (found_scroll_y2<y)
			found_item=NULL;
	}
	return found_item;
}

//  item-  
GItem* GListBase::DetectItemByCursor(int& x, int& y)
{
	GItem* item=NULL;
	GItem* next=GetFirstVisibledItem();
	while (next)
	{
		if (next->IsOver(x, y))
		{
			item=next;
			break;
		}
		next=GetNextVisibledItem();
	}
	return item;
}

//   item-
GItem* GListBase::GetLatestItem()
{
	GItem* next=first;
	GItem* latest=next;

	while (next)
	{
		latest=next;
		next=next->GetNext();
	}

	return latest;
}

//   item-
int GListBase::GetItemCount()
{
	int count=0;
	GItem* next=first;
	
	while (next)
	{
		count++;
		next=next->GetNext();
	}

	return count;
}

//   Y ,   item   
void GListBase::ItemInClient(GItem* item)
{
	if (IsVScrollBar())
	{
		int y=item->GetY()-scroll_y;
		int y2=y+item->GetHeight();
		if (y2<=0)
		{
			// item   ,    
			scroll_y+=y;
			DetectFirstVisibledItem();
			Invalidate();
		}
		else
		{
			CMagicRect rect;
			GetClientRect(&rect);
			int he=rect.Height();
			if (he<=y)
			{
				// item   ,    
				scroll_y-=(he-y2);
				DetectFirstVisibledItem();
				Invalidate();
			}
		}
	}
}

//  item-  
void GListBase::SortByName(bool save_scroller_position)
{
	if (save_scroller_position)
		PushScrollPosition();

	GItem* next=first;
	while (next)
	{
		CMagicString sname=next->GetText();
		sname=sname.toUpper();
		GItem* item=next;

		GItem* next2=next->GetNextSibling();
		while (next2)
		{
			CMagicString sname2=next2->GetText();
			sname2=sname2.toUpper();

			if (strcmp(sname, sname2)>0)
			{
				sname=sname2;
				item=next2;
			}

			next2=next2->GetNextSibling();
		}

		Exchange(next, item);
		next=item->GetNextSibling();
		/*
		if (item!=next)
		{
			//   item- 
			GItem* previous1=next->GetPrevious();
			GItem* next1=next->GetNext();

			GItem* previous2=item->GetPrevious();
			GItem* next2=item->GetNext();

			if (previous1)
			{
				previous1->SetNext(item);
				item->SetPrevious(previous1);
			}
			else
			{
				first=item;
				item->SetPrevious(NULL);
			}

			if (next1)
			{
				next1->SetPrevious(item);
				item->SetNext(next1);
			}
			else
			{
				item->SetNext(NULL);
			}


			if (previous2)
			{
				previous2->SetNext(next);
				next->SetPrevious(previous2);
			}
			else
			{
				first=next;
				next->SetPrevious(NULL);
			}

			if (next2)
			{
				next2->SetPrevious(next);
				next->SetNext(next2);
			}
			else
			{
				next->SetNext(NULL);
			}
		}

		next=next->GetNextSibling();
		*/
	}

	int y=0;
	next=first;
	while (next)
	{
		//    
		next->SetY(y);

		int he=next->GetHeight();
		y+=he;

		next=next->GetNext();
	}
	
	RecalcLayout();
	Invalidate();

	if (save_scroller_position)
		PopScrollPosition();
}

//  item-   
void GListBase::SortByField(bool save_scroller_position)
{
	if (save_scroller_position)
		PushScrollPosition();

	GItem* next=first;
	while (next)
	{
		double sort=next->GetSort();
		GItem* item=next;

		GItem* next2=next->GetNextSibling();
		while (next2)
		{
			double sort2=next2->GetSort();
			
			if (sort>sort2)
			{
				sort=sort2;
				item=next2;
			}

			next2=next2->GetNextSibling();
		}

		Exchange(next, item);
		next=item->GetNextSibling();
		/*
		if (item!=next)
		{
			//   item- 
			GItem* previous1=next->GetPrevious();
			GItem* next1=next->GetNext();

			GItem* previous2=item->GetPrevious();
			GItem* next2=item->GetNext();

			if (previous1)
			{
				previous1->SetNext(item);
				item->SetPrevious(previous1);
			}
			else
			{
				first=item;
				item->SetPrevious(NULL);
			}

			if (next1)
			{
				next1->SetPrevious(item);
				item->SetNext(next1);
			}
			else
			{
				item->SetNext(NULL);
			}


			if (previous2)
			{
				previous2->SetNext(next);
				next->SetPrevious(previous2);
			}
			else
			{
				first=next;
				next->SetPrevious(NULL);
			}

			if (next2)
			{
				next2->SetPrevious(next);
				next->SetNext(next2);
			}
			else
			{
				next->SetNext(NULL);
			}
		}

		next=next->GetNextSibling();
		*/
	}

	int y=0;
	next=first;
	while (next)
	{
		//    
		next->SetY(y);

		int he=next->GetHeight();
		y+=he;

		next=next->GetNext();
	}

	RecalcLayout();
	Invalidate();

	if (save_scroller_position)
		PopScrollPosition();
}

//    item-
void GListBase::Exchange(GItem* item1, GItem* item2)
{
	if (item1!=item2 && item1->GetParent()==item2->GetParent())
	{
		GItem* for_previous1=item1;
		GItem* for_previous2=item2;

		GItem* for_next1=item1->GetNextSibling();
		if (!for_next1)
		{
			for_next1=item1;
			GItem* next=item1->GetNext();
			while (next)
			{
				for_next1=next;
				next=next->GetNext();
			}
		}
		else
			for_next1=for_next1->GetPrevious();

		GItem* for_next2=item2->GetNextSibling();
		if (!for_next2)
		{
			for_next2=item2;
			GItem* next=item2->GetNext();
			while (next)
			{
				for_next2=next;
				next=next->GetNext();
			}
		}
		else
			for_next2=for_next2->GetPrevious();

		GItem* previous1=for_previous1->GetPrevious();
		GItem* next1=for_next1->GetNext();
		GItem* previous2=for_previous2->GetPrevious();
		GItem* next2=for_next2->GetNext();

		if (next1==for_previous2)
		{
			// item-  
			for_previous1->SetPrevious(for_next2);
			for_next2->SetNext(for_previous1);

			for_next1->SetNext(next2);
			if (next2)
				next2->SetPrevious(for_next1);

			for_previous2->SetPrevious(previous1);
			if (previous1)
				previous1->SetNext(for_previous2);
			else
				first=for_previous2;
		}
		else
		{
			for_previous1->SetPrevious(previous2);
			if (previous2)
				previous2->SetNext(for_previous1);
			else
				first=for_previous1;

			for_previous2->SetPrevious(previous1);
			if (previous1)
				previous1->SetNext(for_previous2);
			else
				first=for_previous2;

			for_next1->SetNext(next2);
			if (next2)
				next2->SetPrevious(for_next1);
			
			for_next2->SetNext(next1);
			if (next1)
				next1->SetPrevious(for_next2);
		}

		/*
		if (first==for_previous1)
			first=for_previous2;
		else
		{
			if (first==for_previous2)
				first=for_previous1;
		}
		*/
	}
}

//   
void GListBase::PushScrollPosition()
{
	pushed_scroll_item=selected;
	if (!pushed_scroll_item)
		pushed_scroll_item=GetFirstVisibledItem();

	if (pushed_scroll_item)
	{
		pushed_scroll_item->GetItemPosition(pushed_scroll_x, pushed_scroll_y);
	}
	else
	{
		pushed_scroll_x=pushed_scroll_y=0;
	}
}

//   
void GListBase::PopScrollPosition()
{
	if (pushed_scroll_item)
	{
		int cx, cy;
		pushed_scroll_item->GetItemPosition(cx, cy);

		int rx=pushed_scroll_x-cx;
		int ry=pushed_scroll_y-cy;

		int vx=scroll_x;
		int vy=scroll_y;
		
		SetHScroll(vx+rx);
		SetVScroll(vy+ry);

		if (scroll_x!=vx || scroll_y!=vy)
			Invalidate();
	}
}


void GListBase::ClientOnRender()
{
	theme->RenderClient(this);
	OnRender();
}

void GListBase::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item)
		item->SysOnLButtonDown(nFlags, x, y);
	OnLButtonDown(nFlags, x, y);
}

void GListBase::ClientOnLButtonDblClk(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item)
		item->SysOnLButtonDblClk(nFlags, x, y);
	OnLButtonDblClk(nFlags, x, y);
}

void GListBase::ClientOnRButtonDown(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item)
		item->SysOnRButtonDown(nFlags, x, y);
	OnRButtonDown(nFlags, x, y);
}

void GListBase::ClientOnRButtonDblClk(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item)
		item->SysOnRButtonDblClk(nFlags, x, y);
	OnRButtonDblClk(nFlags, x, y);
}

// ------------------------------------------------------------------------------------

GItemText::GItemText(GListBase* owner, const char* text, int font, unsigned int color) : GItem(0, 0)
{
	this->owner=owner;
	this->font=font;
	this->color=color;
	icon=-1;

	RefreshText(text);

	/*
	if (text)
	{
		this->text=text;
		GSurface::device->GetTextSize(text, font, text_width, text_height);
	}
	else
	{
		text_width=3;
		text_height=GSurface::device->GetFontex(font)->data.height;
	}

	
	CMagicRect gap;
	theme->GetItemGapRect(this, &gap);
	SetWidth(text_width+gap.left+gap.right);
	SetHeight(text_height+gap.top+gap.bottom);
	*/
}

//    item-
//        
void GItemText::RefreshItemSize()
{
	if (owner)
	{
		GThemeComponent* theme=owner->GetThemeComponent();
		CMagicRect gap;
		theme->GetItemGapRect(this, &gap);
		int wi=text_width;
		int he=text_height;

		int icon=GetIconIndex();
		if (icon>=0)
		{
			GImageList* il=owner->GetIcons();
			if (il)
			{
				int iwi, ihe;
				il->GetSize(iwi, ihe);
				if (he<ihe)
					he=ihe;
			}
		}

		SetWidth(wi+gap.left+gap.right);
		SetHeight(he+gap.top+gap.bottom);
	}
}

//  
void GItemText::RefreshText(const char* text)
{
	if (text)
	{
		this->text=text;
		GSurface::device->GetTextSize(text, font, text_width, text_height);
	}
	else
	{
		text_width=3;
		text_height=GSurface::device->GetFontex(font)->data.height;
	}

	RefreshItemSize();
}

void GItemText::SetIconIndex(int icon)
{
	if ((this->icon<0 && icon>=0) || (this->icon>=0 && icon<0))
	{
		int old_wi, old_he;
		GetItemThemeSize(old_wi, old_he);
		this->icon=icon;
		RefreshItemSize();
		((GListBase*)owner)->CorrectingForChanging(this, old_wi, old_he);
	}
	else
		this->icon=icon;
}

void GItemText::SetText(const char* text)
{
	if (this->text!=text)
	{
		int old_wi, old_he;
		GetItemThemeSize(old_wi, old_he);
		//this->text=text;
		RefreshText(text);
		//GSurface::device->GetTextSize(text, font, text_width, text_height);

		((GListBase*)owner)->CorrectingForChanging(this, old_wi, old_he);
	}
}

void GItemText::SetTextFont(int font)
{
	if (this->font!=font)
	{
		int old_wi, old_he;
		GetItemThemeSize(old_wi, old_he);
		this->font=font;
		RefreshText(text);
		//GSurface::device->GetTextSize(text, font, text_width, text_height);

		((GListBase*)owner)->CorrectingForChanging(this, old_wi, old_he);
	}
}

// ------------------------------------------------------------------------------------

GListBox::GListBox() : GListBase()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_LISTBOX);
}

bool GListBox::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, GImageList* icons)
{
	if (GListBase::Create(parent, style, x, y, width, height, id))
	{
		this->icons=icons;
		return true;
	}
	return false;
}

GItem* GListBox::AddString(const char* text, int font, unsigned int color)
{
	GItemText* item=NewItem(text, font, color);

	//    item
	GItem* latest=GetLatestItem();
	InsertItem(item, latest);

	return item;
}

GItemText* GListBox::NewItem(const char* text, int font, unsigned int color)
{
	if (font<0)
		font=theme->GetTextFont(this);

	if (color==COLOR_KEY_EMPTY)
		color=theme->GetTextColor(this);

	//int wi, he;
	//game->device->GetTextSize(text, font, wi, he);

	return new GItemText(this, text, font, color);
}

bool GListBox::WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (nChar==KEY_DOWN)
	{
		GItem* sel=GetSelected();
		if (sel)
			sel=sel->GetNext();
		else
			sel=first;

		if (sel)
		{
			SetSelected(sel);
			ItemInClient(sel);
			OnChangeSelected(sel);
			Invalidate();
			//return false;
		}
		return false;
	}
	else
	{
		if (nChar==KEY_UP)
		{
			GItem* sel=GetSelected();
			if (sel)
				sel=sel->GetPrevious();
			else
				sel=first;

			if (sel)
			{
				SetSelected(sel);
				ItemInClient(sel);
				OnChangeSelected(sel);
				Invalidate();
				//return false;
			}
			return false;
		}
		else
		{
			if (nChar==KEY_HOME)
			{
				GItem* sel=GetSelected();
				if (sel!=first)
				{
					sel=first;
					SetSelected(sel);
					ItemInClient(sel);
					OnChangeSelected(sel);
					Invalidate();
					//return false;
				}
				return false;
			}
			else
			{
				if (nChar==KEY_END)
				{
					GItem* sel=GetSelected();
					GItem* latest=GetLatestItem();
					if (sel!=latest)
					{
						sel=latest;
						SetSelected(sel);
						ItemInClient(sel);
						OnChangeSelected(sel);
						Invalidate();
						//return false;
					}
					return false;
				}
			}
		}
	}

	return true;
}

// ------------------------------------------------------------------------------------

GItemTree::GItemTree(GListBase* owner, const char* text, int font, unsigned int color, int icon, int width, int height) : GItemText(owner, text, font, color)
{
	k_child=0;
	m_child=NULL;
	parent=NULL;
	this->icon=icon;

	node_color=0xffffff;

	checked=false;
}

GItemTree::~GItemTree()
{
	if (IsOpened())
	{
		next=GetNextSibling();
		if (next)
			next->SetPrevious(this);
	}

	for (int i=0;i<k_child;i++)
	{
		m_child[i]->SetPrevious(NULL);
		m_child[i]->SetNext(NULL);
		delete m_child[i];
		m_child[i]=NULL;
	}

	if (m_child)
	{
		delete []m_child;
		m_child=NULL;
	}
	k_child=0;
}

//   item
void GItemTree::AddChild(GItemTree* item)
{
	if (k_child)
	{
		GItem** vm_child=new GItem*[k_child+1];

		for (int i=0;i<k_child;i++)
			vm_child[i]=m_child[i];

		delete []m_child;
		m_child=vm_child;
	}
	else
	{
		m_child=new GItem*[1];
	}

	item->parent=this;

	m_child[k_child]=item;
	k_child++;
	item->SetPrevious(NULL);
	item->SetNext(NULL);
}

//   
int GItemTree::GetChildLevel()
{
	int level=0;
	GItem* next=parent;
	while (next)
	{
		next=next->GetParent();
		level++;
	}
	return level;
}

//   ,   item- 
bool GItemTree::HasChildren()
{
	return (k_child>0);
}

//   ,   item- 
bool GItemTree::IsOpened()
{
	return (HasChildren() && m_child[0]==next);
}

//   item-
void GItemTree::Open()
{
	if (!IsOpened())
	{
		GItem* previous=this;
		for (int i=0;i<k_child;i++)
		{
			owner->InsertItem(m_child[i], previous);
			previous=m_child[i];
		}
	}
}

//   item-
void GItemTree::Close()
{
	if (IsOpened())
	{
		for (int i=0;i<k_child;i++)
		{
			((GItemTree*)m_child[i])->Close();
			owner->DeleteItem(m_child[i]);
		}

		for (int i=0;i<k_child;i++)
		{
			m_child[i]->SetPrevious(NULL);
			m_child[i]->SetNext(NULL);
		}
	}
}

//   item-   
GItem* GItemTree::GetPreviousSibling()
{
	int child_level=GetChildLevel();
	GItemTree* item=(GItemTree*)GetPrevious();
	while (item)
	{
		int level=item->GetChildLevel();
		if (level==child_level)
			break;
		if (level<child_level)
		{
			item=NULL;
			break;
		}
		item=(GItemTree*)item->GetPrevious();
	}
	return item;
}

//   item-   
GItem* GItemTree::GetNextSibling()
{
	GItemTree* item;
	if (IsOpened())
	{
		item=this;
		int child_level=GetChildLevel();
		do
		{
			if (item->IsOpened())
				item=(GItemTree*)(item->m_child[k_child-1]->GetNext());
			else
				item=(GItemTree*)item->GetNext();
			if (item)
			{
				if (item->GetParent()!=parent)
				{
					int child_level2=item->GetChildLevel();
					if (child_level2<=child_level)
						item=NULL;
				}
				else
					// item 
					break;
			}
		}
		while (item);	
	}
	else
	{
		item=(GItemTree*)next;
		if (item)
		{
			if (item->GetParent()!=GetParent())
				return NULL;
		}
	}
	return item;
}

//     
void GItemTree::SysOnNode()
{
	if (HasChildren())
	{
		if (IsOpened())
		{
			//   
			Close();
		}
		else
		{
			//   
			Open();
		}
	}
}

bool GItemTree::ButtonDown(unsigned int nFlags, int x, int y)
{
	GThemeComponent* theme=owner->GetThemeComponent();

	CMagicRect rect;
	theme->GetItemAdditionalRect(this, &rect);
	if (rect.PtInRect(x, y))
	{
		//  
		SysOnNode();
	}
	else
	{
		bool is=false;
		if (theme->GetItemIconRect(this, &rect))
		{
			if (x>=rect.left && x<rect.right)
			{
				//  
				is=true;
			}
		}

		if (!is)
		{
			if (theme->GetItemTextRect(this, &rect))
			{
				if (x>=rect.left && x<rect.right)
				{
					//  
					is=true;
				}
			}
		}

		if (is)
		{
			if (this!=owner->GetSelected())
			{
				owner->SetSelected(this);
				owner->OnChangeSelected(this);
				owner->Invalidate();
			}
			return true;
		}
	}
	return false;
}

bool GItemTree::SysOnLButtonDown(unsigned int nFlags, int x, int y)
{
	return ButtonDown(nFlags, x, y);
}

bool GItemTree::ButtonDblClk(unsigned int nFlags, int x, int y)
{
	GThemeComponent* theme=owner->GetThemeComponent();

	CMagicRect rect;

	bool is=false;
	if (theme->GetItemIconRect(this, &rect))
	{
		if (x>=rect.left && x<rect.right)
		{
			//  
			is=true;
		}
	}

	if (!is)
	{
		if (theme->GetItemTextRect(this, &rect))
		{
			if (x>=rect.left && x<rect.right)
			{
				//  
				is=true;
			}
		}
	}

	if (is)
	{
		SysOnNode();
		OnLButtonDblClk(nFlags, x, y);
		return true;
	}

	return false;
}

bool GItemTree::SysOnLButtonDblClk(unsigned int nFlags, int x, int y)
{
	return ButtonDblClk(nFlags, x, y);
}

bool GItemTree::SysOnRButtonDown(unsigned int nFlags, int x, int y)
{
	return ButtonDown(nFlags, x, y);
}

bool GItemTree::SysOnRButtonDblClk(unsigned int nFlags, int x, int y)
{
	return ButtonDblClk(nFlags, x, y);
}

bool GTree::WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (GListBox::WinOnKeyDown(nChar, nRepCnt, nFlags))
	{
		if (nChar==KEY_RIGHT)
		{
			GItemTree* sel=(GItemTree*)GetSelected();
			if (sel && sel->HasChildren() && (!sel->IsOpened()))
				sel->Open();
			return false;
		}
		else
		{
			if (nChar==KEY_LEFT)
			{
				GItemTree* sel=(GItemTree*)GetSelected();
				if (sel && sel->HasChildren() && sel->IsOpened())
					sel->Close();
				return false;
			}
		}
		return true;
	}
	return false;
}

// ------------------------------------------------------------------------------------

GTree::GTree() : GListBox()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_TREE);
}

GTree::~GTree()
{
	GItem* item=first;
	while (item)
	{
		if (item->IsOpened())
			((GItemTree*)item)->Close();
		item=item->GetNextSibling();
	}
}

bool GTree::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, GImageList* icons)
{
	if (GListBox::Create(parent, style, x, y, width, height, id))
	{
		this->icons=icons;
		return true;
	}
	return false;
}

GItemTree* GTree::AddNode(GItemTree* parent, const char* text, int font, unsigned int color, int icon)
{
	if (font<0)
		font=theme->GetTextFont(this);

	if (color==COLOR_KEY_EMPTY)
		color=theme->GetTextColor(this);

	GItemTree* item=NewItem(text, font, color, icon);

	item->SetOwner(this);
	CMagicRect gap;
	theme->GetItemGapRect(item, &gap);
	item->SetHeight(item->GetHeight()+gap.top+gap.bottom);

	CMagicRect rect;
	if (parent)
	{
		item->SetParent(parent);
		theme->GetItemTextRect(item, &rect);
		item->SetWidth(rect.right+gap.right);

		parent->AddChild(item);

	}
	else
	{
		GItem* latest=GetLatestItem();
		/*
		if (latest)
		{
			GItem* par=latest->GetParent();
			while (par)
			{
				latest=par;
				par=par->GetParent();
			}

			item->SetParent(parent);
		}
		*/
		item->SetParent(NULL);
		
		theme->GetItemTextRect(item, &rect);
		item->SetWidth(rect.right+gap.right);

		InsertItem(item, latest);
	}

	return item;
}

GItemTree* GTree::NewItem(const char* text, int font, unsigned int color, int icon)
{
	if (font<0)
		font=theme->GetTextFont(this);

	if (color==COLOR_KEY_EMPTY)
		color=theme->GetTextColor(this);

	int wi, he;
	game->device->GetTextSize(text, font, wi, he);

	GItemTree* item=new GItemTree(this, text, font, color, icon, wi, he);
	return item;
}

bool GTree::HasButtons()
{
	if (style & GSTR_HASBUTTONS)
		return true;
	return false;
}

bool GTree::HasLines()
{
	if (style & GSTR_HASLINES)
		return true;
	return false;
}

// ------------------------------------------------------------------------------------

GItemMenu::GItemMenu(GListBase* owner, const char* text, int font, unsigned int color, int icon, int width, int height) : GItemTree(owner, text, font, color, icon, width, height)
{
}

void GItemMenu::GetItemRect(CMagicRect* rect)
{
	owner->GetClientRect(rect);
	int wi=rect->Width()+1;
	GetItemPosition(rect->left, rect->top);
	rect->right=rect->left+wi;
	rect->bottom=rect->top+height;
}

// ------------------------------------------------------------------------------------

GItemMenuSeparator::GItemMenuSeparator(GListBase* owner) : GItemMenu(owner, NULL, 0, 0xffffff, NULL, 0, 0)
{
	enabled=false;
	width=0;
	GThemeComponent* theme=owner->GetThemeComponent();
	height=theme->GetItemSeparatorHeight();
	CMagicRect rect;
	theme->GetItemGapRect(this, &rect);
	height+=rect.top+rect.bottom;
}

// ------------------------------------------------------------------------------------

GMenu::GMenu() : GTree()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_MENU);
	previous_modal=NULL;
	parent_item=NULL;
}

bool GMenu::Create(unsigned int style, GImageList* icons)
{
	return GTree::Create(NULL, style, 0, 0, 100, 100, 0xfffffffd, icons);
}

GItemMenu* GMenu::AppendMenu(GItemTree* parent, const char* text, int font, unsigned int color, int icon)
{
	GItemMenu* item;
	if (text || icon!=-1)
	{
		item=(GItemMenu*)AddNode(parent, text, font, color, icon);
	}
	else
	{
		item=new GItemMenuSeparator(this);
		GItem* previous=GetLatestItem();
		InsertItem(item, previous);
	}

	RecalcSize();

	return (GItemMenu*)item;
}

//      item-
void GMenu::RecalcSize()
{
	//    
	int max_wi=0;
	int max_he=0;
	GItem* next=(GItemTree*)first;
	while (next)
	{
		CMagicRect rect;
		theme->GetItemTextRect(next, &rect);
		int wi=rect.right;
		if (next->HasChildren())
		{
			theme->GetItemAdditionalRect(next, &rect);
			wi+=rect.Width();
		}
		max_he+=next->GetHeight();
		if (max_wi<wi)
			max_wi=wi;

		next=next->GetNextSibling();
	}

	SetClientSize(max_wi, max_he);
	
	next=(GItemTree*)first;
	while (next)
	{
		next->SetWidth(max_wi);
		next=next->GetNextSibling();
	}
}

GItemTree* GMenu::NewItem(const char* text, int font, unsigned int color, int icon)
{
	if (font<0)
		font=theme->GetTextFont(this);

	if (color==COLOR_KEY_EMPTY)
		color=theme->GetTextColor(this);

	int wi, he;
	game->device->GetTextSize(text, font, wi, he);

	GItemTree* item=new GItemMenu(this, text, font, color, icon, wi, he);
	return item;
}

//      
void GMenu::SelectPosition(int x, int y)
{
	//  4   
	//     
	int wi, he;
	game->device->GetSize(wi, he);

	int px=x+1;
	int px2=px+width;
	if (px2>=wi)
		px=x-width;
	int py=y+1;
	int py2=py+height;
	if (py2>=he)
		py=y-height;

	this->x=px;
	this->y=py;
}

//    
void GMenu::PopupMenu(int x, int y)
{
	if (!IsCreated())
	{
		Create(GS_BORDER);
		RecalcSize();
	}
	SelectPosition(x, y);
	BeginModal();
}

void GMenu::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	GItemMenu* item=(GItemMenu*)DetectItemByCursor(x, y);
	if (selected!=item)
	{
		GMenu* child=GetChildMenu();
		if (child)
		{
			child->CloseChildMenu();
			game->GetScene()->Invalidate();
		}

		SetSelected(item);
		if (item && item->HasChildren())
		{
			GMenu* child_menu=new GMenu;
			child_menu->Create(GS_VISIBLE | GS_BORDER);
			CopyAttributesTo(child_menu);
			AddChild(child_menu);
			child_menu->OpenChildMenu(item);
		}

		Invalidate();
	}

	OnMouseMove(nFlags, x, y);
}

void GMenu::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item)
		item->SysOnLButtonDown(nFlags, x, y);
	if (item)
	{
		GItemMenu* itemmenu=(GItemMenu*)item;
		if (!itemmenu->HasChildren())
			OnSelectMenu(itemmenu);
	}
}

void GMenu::ClientOnRButtonDown(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item)
		item->SysOnRButtonDown(nFlags, x, y);
	if (item)
	{
		GItemMenu* itemmenu=(GItemMenu*)item;
		if (!itemmenu->HasChildren())
			OnSelectMenu(itemmenu);
	}
}

void GMenu::WinOnLButtonDown(unsigned int nFlags, int x, int y)
{
	EndModal();
}

void GMenu::WinOnRButtonDown(unsigned int nFlags, int x, int y)
{
	EndModal();
}

bool GMenu::WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (nChar==KEY_ESCAPE)
	{
		EndModal();
		return false;
	}
	else
	{
		if (nChar==KEY_RETURN)
		{
			GItem* item=selected;//GetSelected();
			if (item)
			{
				GItemMenu* itemmenu=(GItemMenu*)item;
				if (itemmenu->HasChildren())
					return false;
				OnSelectMenu(itemmenu);
			}
			EndModal();
			return false;
		}
	}
	return true;
}

//   
void GMenu::OpenChildMenu(GItemMenu* parent_item)
{
	this->parent_item=parent_item;

	int k_child=parent_item->GetChildCount();
	GItemTree* previous=NULL;
	for (int i=0;i<k_child;i++)
	{
		GItemTree* child=(GItemTree*)(parent_item->GetChild(i));
		child->SetParent(NULL);
		child->SetOwner(this);
		InsertItem(child, previous);
		previous=child;
	}

	RecalcSize();

	//      
	//       item-   ,       item-
	
	//     item-  
	//      ,         + 
	CMagicRect rect;
	parent->GetClientRect(&rect);
	int wx, wy;
	parent->GetAbsoluteWindowPosition(wx, wy);
	int px=wx;
	int py=wy+parent_item->GetY();
	int px2=px+rect.Width();
	int py2=py;
	parent->GetInnerRect(&rect);
	int vx=rect.Width();
	int vy=rect.Height();
	int bx, by;
	parent->GetInnerOffset(bx, by);
	vx+=bx;
	vy+=by;
	int pwi, phe;
	parent->GetSize(pwi, phe);
	int rx=pwi-vx;
	int ry=phe-vy;

	px2+=rx;
	py2+=ry;

	int wi, he;
	game->device->GetSize(wi, he);

	vx=px2;
	int vx2=vx+width;
	if (vx2>=wi)
		vx=px-(width-1);

	vy=py2-by;

	this->x=vx-wx;
	this->y=vy-wy;

	// ,       
	GetAbsoluteWindowPosition(wx, wy);
	ry=wy+height;
	if (he<=ry)
		this->y-=(ry-he+1);
	
	Show(true);
	RecalcCovering();
	Invalidate();
}

//   
void GMenu::CloseChildMenu()
{
	GMenu* child=GetChildMenu();
	if (child)
	{
		//     
		child->CloseChildMenu();
		DeleteChild(child, false);
	}

	GItemTree* item=(GItemTree*)first;
	while (item)
	{
		item->SetParent(parent_item);
		item=(GItemTree*)item->GetNextSibling();
	}

	first=NULL;

	parent->DeleteChild(this, false);
}

//   
GMenu* GMenu::GetChildMenu()
{
	if (k_child)
		return (GMenu*)m_child[0];
	return NULL;
}

//      
GComponent* GMenu::CorrectPositionClientForModal(int& x, int& y)
{
	//    
	GMenu* child=this;
	while (true) 
	{
		if (child->GetChildMenu())
			child=child->GetChildMenu();
		else
			break;
	}

	while (true) 
	{
		int cx, cy;
		child->GetAbsoluteWindowPosition(cx, cy);
		int wi, he;
		child->GetSize(wi, he);
		if (x>=cx && x<cx+wi && y>=cy && y<cy+he)
			return child;

		if (child==this)
			break;

		child=(GMenu*)(child->GetParent());
	}

	return GComponent::CorrectPositionClientForModal(x, y);
}

//     
void GMenu::EndModal()
{
	//      
	GMenu* menu=this;
	if (parent_item)
	{
		GItem* pitem=parent_item;
		while (pitem)
		{
			menu=(GMenu*)pitem->GetOwner();
			pitem=menu->parent_item;
		}
	}
	
	GMenu* child_menu=menu->GetChildMenu();
	if (child_menu)
		menu->CloseChildMenu();

	menu->GTree::EndModal();
}

//  item
void GMenu::SetSelected(GItem* item)
{
	GItem* citem=GetSelected();
	if (citem!=item)
	{
		//   
		while (citem)
		{
			citem->GetOwner()->GListBase::SetSelected(NULL);
			citem=citem->GetParent();
		}

		//   
		while (item)
		{
			item->GetOwner()->GListBase::SetSelected(item);
			item=item->GetParent();
		}
	}
}

GItem* GMenu::GetSelected()
{
	GMenu* child=this;
	GMenu* latest_child=this;
	while (true)
	{
		child=child->GetChildMenu();
		if (!child)
			break;
		latest_child=child;
	}

	return latest_child->GListBase::GetSelected();
}

// ------------------------------------------------------------------------------------

//  
GGrid::GGrid() : GTree()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_GRID);

	k_column=0;
	m_column=NULL;

	k_row=0;
	m_row=NULL;
}

GGrid::~GGrid()
{
	if (m_column)
	{
		delete []m_column;
		m_column=NULL;
	}

	if (m_row)
	{
		delete []m_row;
		m_row=NULL;
	}
}

//  
void GGrid::InsertColumn(int index, const char* name, int width, bool center)
{
	if (index<0 || index>k_column)
		index=k_column;

	if (k_column)
	{
		COLUMN_DATA* vm_column=new COLUMN_DATA[k_column+1];
		for (int i=0;i<index;i++)
			vm_column[i]=m_column[i];

		for (int i=index;i<k_column;i++)
			vm_column[i+1]=m_column[i];

		delete []m_column;
		m_column=vm_column;
	}
	else
	{
		m_column=new COLUMN_DATA[1];
	}
	k_column++;
	COLUMN_DATA* c=&(m_column[index]);
	c->name=name;
	c->width=width;
	c->center=center;
}

//  
GItemTree* GGrid::InsertRow(int index)
{
	if (index<0 || index>k_row)
		index=k_row;

	if (k_row)
	{
		GItemTree** vm_row=new GItemTree*[k_row+1];
		for (int i=0;i<index;i++)
			vm_row[i]=m_row[i];

		for (int i=index;i<k_row;i++)
			vm_row[i+1]=m_row[i];

		delete []m_row;
		m_row=vm_row;
	}
	else
	{
		m_row=new GItemTree*[1];
	}
	k_row++;


	GItemTree* item=NewItem("Row", -1, COLOR_KEY_EMPTY, NULL);
	for (int i=0;i<k_column;i++)
	{
		GItemTree* it=NewItem("", -1, COLOR_KEY_EMPTY, NULL);
		item->AddChild(it);
	}

	GItem* previous;
	if (index==k_row-1)
		previous=GetLatestItem();
	else
		previous=m_row[index];

	InsertItem(item, previous);
	m_row[index]=item;
	return item;
}

bool GGrid::SetText(int column, int row, const char* text)
{
	if (row>=0 && row<k_row)
	{
		GItemTree* item=m_row[row];
		if (column>=0 && column<k_column)
		{
			GItemTree* child=(GItemTree*)item->GetChild(column);
			child->SetText(text);
		}
	}
	return false;
}

const char* GGrid::GetText(int column, int row)
{
	if (row>=0 && row<k_row)
	{
		GItemTree* item=m_row[row];
		if (column>=0 && column<k_column)
		{
			GItem* child=item->GetChild(column);
			return child->GetText();
		}
	}
	return NULL;
}

bool GGrid::SetIcon(int column, int row, int icon)
{
	if (row>=0 && row<k_row)
	{
		GItemTree* item=m_row[row];
		if (column>=0 && column<k_column)
		{
			GItemTree* child=(GItemTree*)item->GetChild(column);
			child->SetIconIndex(icon);
		}
	}
	return false;
}

int GGrid::GetIcon(int column, int row)
{
	if (row>=0 && row<k_row)
	{
		GItemTree* item=m_row[row];
		if (column>=0 && column<k_column)
		{
			GItem* child=item->GetChild(column);
			return child->GetIconIndex();
		}
	}
	return -1;
}

//      (row=-1)
void GGrid::GetCellRect(int column, int row, CMagicRect* rect)
{
	int x=0;
	for (int i=0;i<column;i++)
		x+=m_column[i].width;

	if (row<0)
	{
		//  
		rect->top=theme->GetBorderSize(this);
		rect->right=rect->top+theme->GetCaptionHeight(this);
	}
	else
	{
		m_row[row]->GetItemPosition(rect->left, rect->top);
		int swi, she;
		m_row[row]->GetItemSize(swi, she);
		rect->bottom=rect->top+swi;
	}

	rect->left+=x;
	rect->right=rect->left+m_column[column].width;

}

// ------------------------------------------------------------------------------------

GEdit::GEdit()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_EDIT);
	text_x=0;
	sel_pos1=sel_pos2=-1;

	cursor=0;
	visibled_cursor=false;
	visibled_cursor_tick=0xffffffff;

	was_setfocus=false;
}

bool GEdit::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text)
{
	if (GComponent::Create(parent, style, x, y, width, height, id))
	{
		if (text)
			this->text=text;
		return true;
	}
	return false;
}

//    
void GEdit::GetClientRect(CMagicRect* rect)
{
	GetInnerRect(rect);

	if (theme)
	{
		CMagicRect gap;
		theme->GetClientGap(&gap);
		rect->right-=(gap.left+gap.right);
		rect->bottom-=(gap.top+gap.bottom);
	}
	else
	{
		rect->right-=2;
		rect->bottom-=2;
	}
}

//      
void GEdit::CorrectTextX()
{
	int font=theme->GetTextFont(this);

	CMagicRect rect;
	GetClientRect(&rect);
	int cwi=rect.Width();

	int wi, he;
	game->device->GetTextSize(GetWindowText(), font, wi, he);
	if (wi<=cwi)
	{
		//    
		if (text_x>0)
		{
			text_x=0;
			VisibledCursor();
			Invalidate();
		}
	}
	else
	{
		//   
		//    
		int x=GetCursorX();
		if (x<0)
		{
			//  ,   
			text_x-=-x;
			VisibledCursor();
			Invalidate();
		}
		else
		{
			if (x>=cwi)
			{
				//  ,   
				text_x+=x-(cwi-1);
				VisibledCursor();
				Invalidate();
			}
		}
	}
}

//   X 
int GEdit::GetCursorX()
{
	int x;
	if (cursor)
	{
		int font=theme->GetTextFont(this);

		int wi, he;
		int len=text.GetLength();
		if (cursor==len)
		{
			game->device->GetTextSize(GetWindowText(), font, wi, he);
		}
		else
		{
			char* str=(char*)GetWindowText();
			//     
			char s=str[cursor];
			str[cursor]=0;
			game->device->GetTextSize(str, font, wi, he);
			str[cursor]=s;
		}
		x=-text_x+wi;
	}
	else
		x=-text_x;
	return x;
}

//       .
//       .
bool GEdit::GetSelPos(int& sel1, int& sel2)
{
	if (sel_pos1>=0 && IsEnabled())
	{
		if (sel_pos1>sel_pos2)
		{
			sel2=sel_pos1-1;
			sel1=sel_pos2;
			return true;
		}
		else
		{
			if (sel_pos1<sel_pos2)
			{
				sel1=sel_pos1;
				sel2=sel_pos2-1;
				return true;
			}
		}
	}
	sel1=sel2=-1;
	return false;
}

//  
void GEdit::ResetSelPos()
{
	if (sel_pos1!=-1)
	{
		sel_pos1=sel_pos2=-1;
		Invalidate();
	}
}

//  
void GEdit::CorrectSelPos(int previous_cursor, bool selecting)
{
	if (selecting)
	{
		if (sel_pos1==-1)
			sel_pos1=previous_cursor;
		sel_pos2=cursor;
		Invalidate();
	}
	else
		ResetSelPos();
	CorrectTextX();
}

//       
void GEdit::DeleteText()
{
	int sel1, sel2;
	if (!GetSelPos(sel1, sel2))
	{
		sel1=cursor;
		sel2=cursor;
	}

	CMagicString t1=text.Left(sel1);
	text=t1+text.Right(text.GetLength()-sel2-1);
	cursor=sel1;

	ResetSelPos();

	CorrectTextX();
	VisibledCursor();
	Invalidate();
}

//    
void GEdit::CursorToEnd(bool selecting)
{
	int len=text.GetLength();
	if (cursor<len)
	{
		int cursor_pos=cursor;
		cursor=len;
		CorrectSelPos(cursor_pos, selecting);
		VisibledCursor();
	}
}

//      X    
int GEdit::GetCursorByPosition(int x)
{
	int x1=-text_x;
	if (x>x1)
	{
		char* str=(char*)GetWindowText();
		int len=text.GetLength();
		char s[2];
		s[1]=0;

		for (int i=0;i<len;i++)
		{
			s[0]=str[i];
			int wi, he;
			int font=theme->GetTextFont(this);
			game->device->GetTextSize(s, font, wi, he);
			int x2=x1+wi;
			if (x<=x2)
			{
				//   
				// x     x1  x2

				/*
				//   ,   
				int half=(x2-x1)/2;
				if (x1+half<x)
				{
					//   
					return i;
				}
				else
				{
					//   
					if (i>0)
						return i-1;
					return i;
				}
				*/

				return i;
			}
			x1=x2;
		}
		return len;
	}
	return 0;
}

//   
void GEdit::VisibledCursor()
{
	visibled_cursor=false;//true;
	visibled_cursor_tick=0xffffffff;
}

//   
void GEdit::SysOnFocusTimer(unsigned int tick)
{
	if (visibled_cursor_tick>tick || visibled_cursor_tick<tick-500)
	{
		visibled_cursor=!visibled_cursor;
		visibled_cursor_tick=tick;
		Invalidate();
		OnFocusTimer(tick);
	}

	/*
	if (visibled_cursor_tick>tick)
	{
		//     
		visibled_cursor_tick=tick;
	}
	else
	{
		if (visibled_cursor_tick<tick-500)
		{
			visibled_cursor=!visibled_cursor;
			visibled_cursor_tick=tick;
			Invalidate();
			OnFocusTimer(tick);
		}
	}
	*/
}

/*
bool CCEdit::InnerOnRender()
{
	GSurface* backbuffer=game->GetBackbuffer();

	//  
	CMagicRect rect;
	GetInnerRect(&rect);
	int wi=rect.Width();
	int he=rect.Height();
	backbuffer->FillRect(0, 0, wi, he, 0);

	OnRenderINNER();

	return true;
}
*/

//  
void GEdit::ClientOnRender()
{
	theme->RenderClient(this);
	OnRender();
}

void GEdit::SysOnSetFocus()
{
	was_setfocus=true;
	if (!text.IsEmpty())
	{
		sel_pos1=0;
		sel_pos2=text.GetLength();
	}

	VisibledCursor();
	Invalidate();
}

void GEdit::SysOnKillFocus()
{
	ResetSelPos();
	visibled_cursor=false;
	Invalidate();
	OnKillFocus();
}

void GEdit::ClientOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	CMagicString text_copy=text;

	int cursor_pos=cursor;
	bool shift=false;
	if (nFlags & CONTROL_SHIFT)
		shift=true;

	switch (nChar)
	{
	case KEY_LEFT:
		cursor--;
		if (cursor<0)
			cursor=0;
		CorrectSelPos(cursor_pos, shift);
		VisibledCursor();
		break;

	case KEY_RIGHT:
		{
			int len=text.GetLength();
			cursor++;
			if (cursor>len)
				cursor=len;
			CorrectSelPos(cursor_pos, shift);
			VisibledCursor();
		}
		break;

	case KEY_HOME:
		cursor=0;
		CorrectSelPos(cursor_pos, shift);
		VisibledCursor();
		break;

	case KEY_END:
		cursor=text.GetLength();
		CorrectSelPos(cursor_pos, shift);
		VisibledCursor();
		break;

	case KEY_DELETE:
		{
			int sel1, sel2;
			if (GetSelPos(sel1, sel2))
				DeleteText();
			else
			{
				int len=text.GetLength();
				if (cursor<len)
					DeleteText();
			}
		}
		break;

	case KEY_BACK:
		{
			int sel1, sel2;
			if (GetSelPos(sel1, sel2))
				DeleteText();
			if (cursor)
			{
				cursor--;
				DeleteText();
			}
		}
		break;

	case KEY_RETURN:
		OnEnter();
		break;

	case KEY_A:
		if (nFlags & CONTROL_CTRL)
		{
			if (shift)
			{
				ResetSelPos();
				VisibledCursor();
			}
			else
			{
				cursor_pos=0;
				cursor=text.GetLength();
				CorrectSelPos(cursor_pos, true);
				VisibledCursor();
			}
			break;
		}

	default:
		OnKeyDown(nChar, nRepCnt, nFlags);
		break;
	}

	if (text_copy!=text)
		SysOnChangeText();
}

void GEdit::ClientOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	CMagicString text_copy=text;

	int vs1, vs2;
	if (GetSelPos(vs1, vs2))
	{
		//  
		cursor=vs1;
		DeleteText();
	}
	CMagicString t1=text.Left(cursor);
	CMagicString t2=text.Right(text.GetLength()-cursor);
	text=t1+nChar;
	text+=t2;
	cursor++;

	CorrectTextX();

	if (text_copy!=text)
		SysOnChangeText();

	Invalidate();
}

void GEdit::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	//if (was_setfocus)
	//	was_setfocus=false;
	//else
	if (!was_setfocus)
	{
		int previous_cursor=cursor;
		bool shift=false;
		if (nFlags & CONTROL_SHIFT)
			shift=true;

		cursor=GetCursorByPosition(x);

		CorrectSelPos(previous_cursor, shift);
	}
	else
		cursor=GetCursorByPosition(x);
	VisibledCursor();
	Invalidate();
}

void GEdit::ClientOnLButtonUp(unsigned int nFlags, int x, int y)
{
	was_setfocus=false;
}

void GEdit::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	if (!was_setfocus)
	{
		if (nFlags & CONTROL_LEFT)
		{
			bool shift=false;
			if (nFlags & CONTROL_SHIFT)
				shift=true;
			int previous_cursor=cursor;
			cursor=GetCursorByPosition(x);
			if (cursor!=previous_cursor)
				CorrectSelPos(previous_cursor, true);
		}
	}
}

void GEdit::SetWindowText(const char* text)
{
	if (strcmp(text, this->text.c_str()))
	{
		this->text=text;

		text_x=0;
		sel_pos1=sel_pos2=-1;

		cursor=0;
		visibled_cursor=false;
		visibled_cursor_tick=0xffffffff;

		SysOnChangeText();
		Invalidate();
	}
}

void GEdit::SysOnCopy()
{
	int vs1, vs2;
	if (GetSelPos(vs1, vs2))
	{
		//  
		CMagicString clipboard;
		for (int i=vs1;i<=vs2;i++)
			clipboard+=text[i];

		game->platform->SetClipboard(clipboard);
	}

	OnCopy();
}

void GEdit::SysOnCut()
{
	int vs1, vs2;
	if (GetSelPos(vs1, vs2))
	{
		//  
		CMagicString clipboard;
		for (int i=vs1;i<=vs2;i++)
			clipboard+=text[i];

		game->platform->SetClipboard(clipboard);

		DeleteText();
	}

	OnCut();
}

void GEdit::SysOnPast()
{
	CMagicString clipboard;
	game->platform->GetClipboard(clipboard);
	if (!clipboard.IsEmpty())
	{
		CMagicString text_copy=text;

		int vs1, vs2;
		if (GetSelPos(vs1, vs2))
		{
			//  
			cursor=vs1;
			DeleteText();
		}
		CMagicString t1=text.Left(cursor);
		CMagicString t2=text.Right(text.GetLength()-cursor);
		text=t1+clipboard;
		text+=t2;
		cursor+=clipboard.GetLength();

		CorrectTextX();

		if (text_copy!=text)
			SysOnChangeText();

		Invalidate();
	}

	OnPast();
}

// ------------------------------------------------------------------------------------

//   
GComboBox::GComboBox() : GEdit()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_COMBOBOX);

	list_height=100;
	button_state=CCBUTTON_NORMAL;

	k_string=0;
	t_string=-1;
	m_string=NULL;
}

GComboBox::~GComboBox()
{
	for (int i=0;i<k_string;i++)
	{
		m_string[i]->SetNext(NULL);
		delete m_string[i];
		m_string[i]=NULL;
	}

	if (m_string)
	{
		delete []m_string;
		m_string=NULL;
	}

	k_string=0;
	t_string=-1;
}

bool GComboBox::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, int list_height)
{
	if (GEdit::Create(parent, style, x, y, width, height, id, NULL))
	{
		if (list_height>0)
			this->list_height=list_height;
		else
			this->list_height=200;
		return true;
	}
	return false;
}

GItem* GComboBox::AddString(const char* text, int font)
{
	GItem* string=NewItem(text, font);

	if (k_string)
	{
		GItem** vm_string=new GItem*[k_string+1];

		for (int i=0;i<k_string;i++)
			vm_string[i]=m_string[i];

		delete []m_string;
		m_string=vm_string;
	}
	else
	{
		m_string=new GItem*[1];
	}

	m_string[k_string]=string;
	k_string++;

	return string;
}

GItem* GComboBox::GetString(int index)
{
	if (index>=0 && index<k_string)
		return m_string[index];
	return NULL;
}

//  
bool GComboBox::IsEditable()
{
	if (style & GSCB_DROPDOWN)
		return false;
	return true;
}

//  
void GComboBox::SetSel(int index)
{
	if (index>=0 && index<k_string)
		t_string=index;
	else
		t_string=-1;

	text="";
	if (t_string!=-1)
	{
		GItem* t=m_string[t_string];
		if (t)
		{
			const char* str=t->GetText();
			if (str)
				SetWindowText(str);
		}
	}
	else
		SetWindowText("");

	Invalidate();
}

GItemText* GComboBox::NewItem(const char* text, int font)
{
	if (font<0)
		font=theme->GetTextFont(this);

	unsigned int color;
	if (theme)
		color=theme->GetTextColor(this);
	else
		color=0xffffff;

	return new GItemText(NULL, text, font, color);
}

//      
void GComboBox::GetClientOffset(int& x, int& y)
{
	if (IsEditable())
		return GEdit::GetClientOffset(x, y);
	x=y=0;
}

//    
void GComboBox::GetClientRect(CMagicRect* rect)
{
	if (IsEditable())
	{
		GEdit::GetClientRect(rect);
		CMagicRect drop;
		theme->GetItemAdditionalRect((GItem*)this, &drop);
		int he=drop.Height();
		rect->right-=he;
	}
	else
	{
		rect->left=0;
		rect->top=0;
		rect->right=0;
		rect->bottom=0;
	}
}

//  
bool GComboBox::InnerOnRender()
{
	return theme->RenderInner(this);
}

void GComboBox::InnerOnMouseEnter(unsigned int nFlags, int x, int y)
{
	if (button_state==CCBUTTON_NORMAL)
	{
		button_state=CCBUTTON_OVER;
		Invalidate();
	}
}

void GComboBox::InnerOnMouseLeave(unsigned int nFlags)
{
	if (button_state==CCBUTTON_OVER)
	{
		button_state=CCBUTTON_NORMAL;
		Invalidate();
	}
}

void GComboBox::ClientOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (IsEditable())
	{
		GEdit::ClientOnKeyDown(nChar, nRepCnt, nFlags);
	}
}

void GComboBox::ClientOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (IsEditable())
	{
		GEdit::ClientOnChar(nChar, nRepCnt, nFlags);
	}
}

void GComboBox::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	if (IsEditable())
	{
		GEdit::ClientOnLButtonDown(nFlags, x, y);
	}
}

void GComboBox::InnerOnLButtonDown(unsigned int nFlags, int x, int y)
{
	GEdit::InnerOnLButtonDown(nFlags, x, y);
	button_state=CCBUTTON_PRESSED;

	if (k_string)
	{
		//    
		GListBoxForComboBox* list=new GListBoxForComboBox();
		list->Create(this);
		CopyAttributesTo(list);
		AddChild(list);
		list->SetPreviousModal(game->GetModalComponent());
		game->SetModalComponent(list);
		game->SetFocusComponent(list);
		visibled_cursor=false;
		list->RecalcCovering();
		list->Invalidate();
	}

	Invalidate();
}

void GComboBox::InnerOnLButtonUp(unsigned int nFlags, int x, int y)
{
	button_state=CCBUTTON_NORMAL;
	Invalidate();
}

void GComboBox::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	if (IsEditable())
	{
		GEdit::ClientOnMouseMove(nFlags, x, y);
	}
}

// ------------------------------------------------------------------------------------

GListBoxForComboBox::GListBoxForComboBox() : GListBox()
{
}

GListBoxForComboBox::~GListBoxForComboBox()
{
	first=NULL;
}

bool GListBoxForComboBox::Create(GComboBox* cbox)
{
	CMagicRect gap;
	theme->GetInnerGap(this, &gap);
	int border2=gap.top+gap.bottom;

	this->style=GS_VISIBLE;
	unsigned int style=cbox->GetStyle();
	if (style & GS_BORDER)
		this->style=this->style | GS_BORDER;

	int wi, he;
	cbox->GetSize(wi, he);
	GItem* previous=NULL;
	int k=cbox->GetStringCount();
	int h=0;
	int t_sel=cbox->GetSel();
	for (int i=0;i<k;i++)
	{
		GItem* item=cbox->GetString(i);
		InsertItem(item, previous);
		h+=item->GetHeight();
		if (i==t_sel)
			SetSelected(item);
		previous=item;
	}

	h+=border2;
	x=-cbox->GetThemeComponent()->GetBorderSize(this);
	unsigned int cb_style=cbox->GetStyle();
	if ((cb_style & GSCB_DROPDOWN)==0)
	{
		//   ComboBox
		CMagicRect cgap;
		cbox->GetThemeComponent()->GetClientGap(&cgap);
		x-=cgap.left;
	}
	y=he;
	width=wi;


	height=h;
	CMagicRect rect;
	GetClientRect(&rect);
	int rhe=rect.Height()-h;
	if (rhe)
		h-=rhe;

	height=cbox->GetListHeight();
	if (height>h)
		height=h;

	int vx, vy;
	cbox->GetAbsoluteWindowPosition(vx, vy);
	vy+=he;
	vy+=height;

	int sx, sy;
	game->device->GetSize(sx, sy);
	if (sy<vy)
	{
		//    
		y=-height;
	}

	RecalcLayout();

	if (selected)
		ItemInClient(selected);

	return true;
}

void GListBoxForComboBox::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	CloseList(item);
}

void GListBoxForComboBox::ClientOnRButtonDown(unsigned int nFlags, int x, int y)
{
	CloseList(NULL);
}

void GListBoxForComboBox::ClientOnMButtonDown(unsigned int nFlags, int x, int y)
{
	CloseList(NULL);
}

void GListBoxForComboBox::WinOnLButtonDown(unsigned int nFlags, int x, int y)
{
	CloseList(NULL);
}

void GListBoxForComboBox::WinOnRButtonDown(unsigned int nFlags, int x, int y)
{
	CloseList(NULL);
}

void GListBoxForComboBox::WinOnMButtonDown(unsigned int nFlags, int x, int y)
{
	CloseList(NULL);
}

void GListBoxForComboBox::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	GItem* item=DetectItemByCursor(x, y);
	if (item && item!=GetSelected())
	{
		SetSelected(item);
		Invalidate();
	}
}

void GListBoxForComboBox::SysOnKillFocus()
{
	CloseList(NULL);
	OnKillFocus();
}

bool GListBoxForComboBox::WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (GListBox::WinOnKeyDown(nChar, nRepCnt, nFlags))
	{
		if (nChar==KEY_ESCAPE)
		{
			CloseList(NULL);
			return false;
		}
		else
		{
			if (nChar==KEY_RETURN)
			{
				GItem* sel=GetSelected();
				CloseList(sel);
				return false;
			}
		}
		return true;
	}
	return false;
}

void GListBoxForComboBox::CloseList(GItem* item)
{
	GComboBox* cbox=(GComboBox*)parent;
	if (item)
	{
		int index=0;
		GItem* next=first;
		do 
		{
			if (next==item)
				break;
			index++;
			next=next->GetNext();
		}
		while (true);
		
		cbox->SetSel(index);
		cbox->OnChangeSel();
	}

	GItem* next=first;
	while (next) 
	{
		next->SetOwner(NULL);
		next=next->GetNext();
	}

	first=NULL;
	GComponent* previous_modal=GetPreviousModal();
	GComponent* p=parent;
	parent->DeleteChild(this);

	p->SysOnSetFocus();
	game->SetFocusComponent(p);
	game->SetModalComponent(previous_modal);
	
	game->GetScene()->Invalidate();
}


// ------------------------------------------------

DIGIT& DIGIT::operator=(DIGIT& from)
{
	if (str)
		delete []str;

	type=from.type;
	value1=from.value1;
	value2=from.value2;
	sign=from.sign;

	if (from.str)
	{
		int len=strlen(from.str);
		str=new char[len+1];
		strcpy(str, from.str);
	}

	return *this;
}

bool DIGIT::FromString(CMagicString* str)
{
	value1=value2=0;
	type=DIGIT_ERROR;
	int len=str->GetLength();
	if (len>0)
	{
		if (*str=="true")
		{
			type=DIGIT_BOOL;
			value1=1;
			return true;
		}

		if (*str=="false")
		{
			type=DIGIT_BOOL;
			return true;
		}

		if (*str=="NULL")
		{
			type=DIGIT_INT;
			return true;
		}

		int pos=0;
		char s=(*str)[pos];
		if (s=='+')
		{
			sign=SIGN_PLUS;
			s=(*str)[pos];
			pos++;
		}
		else
		{
			if (s=='-')
			{
				sign=SIGN_MINUS;
				s=(*str)[pos];
				pos++;
			}
			else
			{
				sign=SIGN_NO;

				if (s=='"' || s=='\'')
				{
					//  
					if ((*str)[len-1]==s)
					{
						//   , ..   
						int str_len=len-2;
						if (s=='\'')
						{
							type=DIGIT_CHAR;
							if (str_len>0 || str_len<=2)
							{
								//      1  2 
								s=(*str)[1];
								if (s=='\\')
								{
									if (str_len==1)
										return false;
								}
								else
									if (str_len==2)
										return false;
							}
							else
								return false;
						}
						else
						{
							type=DIGIT_STRING;
						}

						if (str_len>0)
						{
							this->str=new char[str_len+1];
							for (int i=0;i<str_len;i++)
								this->str[i]=(*str)[i+1];
							this->str[str_len]=0;
						}
						return true;
					}
					else
						return false;
				}
			}
		}

		bool is_hex=false;
		if (len>pos+1)
		{
			if (s=='0')
			{
				// ,    
				s=(*str)[pos+1];
				if (s=='x')
				{
					//   
					is_hex=true;
					pos++;
					s=(*str)[pos];
					pos++;
				}
			}
		}

		//     
		bool is_real=false;
		do 
		{
			s=(*str)[pos];
			pos++;
			if (is_hex)
			{
				if (s>='0' && s<='9')
				{
					value1*=16;
					value1+=s-'0';
				}
				else
				{
					if (s>='a' && s<='f')
					{
						value1*=16;
						value1+=s-'a'+10;
					}
					else
					{
						if (s>='A' && s<='F')
						{
							value1*=16;
							value1+=s-'A'+10;
						}
						else
						{
							//  
							return false;
						}
					}
				}
			}
			else
			{
				if (s>='0' && s<='9')
				{
					value1*=10;
					value1+=s-'0';
				}
				else
				{
					//  
					if (s=='.')
					{
						is_real=true;
						//s=(*str)[pos];
						//pos++;
						break;
					}
					return false;
				}
			}
		}
		while (pos<len);

		if (is_real)
		{
			//     
			type=DIGIT_DOUBLE;
			for (;pos<len;pos++)
			{
				s=(*str)[pos];
				if (s>='0' && s<='9')
				{
					value2*=10;
					value2+=s-'0';
				}
				else
				{
					if (s=='f')
					{
						if (pos==len-1)
						{
							type=DIGIT_FLOAT;
							return true;
						}
						else
						{
							type=DIGIT_ERROR;
							return false;
						}
					}
				}
			}
		}
		else
		{
			if (pos==len)
			{
				type=DIGIT_INT;
				return true;
			}
		}
	}
	return false;
}

CMagicString DIGIT::GetExportString()
{
	CMagicString s;
	if (sign==SIGN_MINUS)
		s='-';

	switch (type)
	{
	case DIGIT_BOOL:
		if (value1)
			s+="true";
		else
			s+="false";
		break;

	case DIGIT_INT:
		{
			CMagicString d;
			d.IntToString(value1);
			s+=d;
		}
		break;

	case DIGIT_FLOAT:
		{
			CMagicString d;
			d.IntToString(value1);
			s+=d;
			s+='.';
			if (value2)
			{
				d.IntToString(value2);
				s+=d;
			}
			s+='f';
		}
		break;

	case DIGIT_DOUBLE:
		{
			CMagicString d;
			d.IntToString(value1);
			s+=d;
			s+='.';
			d.IntToString(value2);
			s+=d;
		}
		break;

	case DIGIT_CHAR:
		s+='\'';
		s+=str;
		s+='\'';
		break;

	case DIGIT_STRING:
		s+='"';
		if (str)
			s+=str;
		s+='"';
	}

	return s;
}

void DIGIT::Serialize(CMagicStream& ar)
{
	if (ar.IsStoring())
	{
		int vtype=(int)type;
		ar<<vtype;
		ar<<value1;
		ar<<value2;
		int vsign=(int)sign;
		if (type==DIGIT_CHAR || type==DIGIT_STRING)
		{
			CMagicString v;
			v=str;
			ar<<v;
		}
	}
	else
	{
		if (str)
		{
			delete []str;
			str=NULL;
		}

		int vtype;
		ar>>vtype;
		type=(DIGIT_ENUM)vtype;
		ar>>value1;
		ar>>value2;
		int vsign;
		ar>>vsign;
		sign=(SIGN_ENUM)vsign;
		if (type==DIGIT_CHAR || type==DIGIT_STRING)
		{
			CMagicString v;
			ar>>v;
			int len=v.GetLength();
			str=new char[len+1];
			for (int i=0;i<len;i++)
				str[i]=v[i];
			str[len]=0;
		}
	}
}

// ------------------------------------------------------------------------------------

GStatic::GStatic() : GScroller()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_STATIC);
	k_text=0;
	m_text=NULL;
	text_width=text_height=0;

	source_width=source_height=0;

	has_links=false;
	current_link=-1;
}

GStatic::~GStatic()
{
	ClearText();
}

void GStatic::ClearText()
{
	for (int i=0;i<k_text;i++)
	{
		delete m_text[i];
		m_text[i]=NULL;
	}

	if (m_text)
	{
		delete []m_text;
		m_text=NULL;
	}
	k_text=0;
	text_width=text_height=0;

	has_links=false;
	current_link=-1;
}

bool GStatic::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id, const char* text)
{
	if (GScroller::Create(parent, style, x, y, 100, 100, id))
	{
		this->text=TestSymbols(text);
		
		this->style=style;

		source_width=width;
		if (width)
			this->width=width;
		source_height=height;
		if (height)
			this->height=height;

		CMagicRect client;
		GetClientRect(&client);
		if (source_width)
			source_width=client.Width();
		if (source_height)
			source_height=client.Height();

		FormatText();

		return true;
	}
	return false;
}

void GStatic::SetWindowText(const char* text)
{
	CMagicString new_text=TestSymbols(text); //   
	GComponent::SetWindowText(new_text);
}

//  
bool GStatic::SetAlignText(COMPONENT_ALIGN_TEXT align)
{
	if (GComponent::SetAlignText(align))
	{
		FormatText();
		return true;
	}
	return false;
}

void GStatic::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	int link=GetCurrentLink();
	if (link!=-1)
	{
		//   
		game->platform->OpenLink(m_text[link]->link);
	}

	OnLButtonDown(nFlags, x, y);
}

void GStatic::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	if (has_links)
	{
		//      
		int index=-1;
		for (int i=0;i<k_text;i++)
		{
			STATIC_TEXT* text=m_text[i];
			if (text->y>y)
				break;
			if (!text->link.IsEmpty())
			{
				if (text->x<=x)
				{
					int twi, the;
					game->device->GetTextSize(text->text, text->font, twi, the);
					if (x>=text->x && x<text->x+twi && y>=text->y && y<text->y+the)
					{
						index=i;
						break;
					}
				}
			}
		}

		if (index!=current_link)
		{
			current_link=index;
			if (style & GS_NOBGND)
				parent->Invalidate();
			else
				Invalidate();
		}
	}

	OnMouseMove(nFlags, x, y);
}

void GStatic::ClientOnMouseLeave(unsigned int nFlags)
{
	int link=GetCurrentLink();
	if (link!=-1)
	{
		current_link=-1;
		if (style & GS_NOBGND)
			parent->Invalidate();
		else
			Invalidate();
	}
}

//     
CMagicString GStatic::TestSymbols(const char* text)
{
	CMagicString str;
	if (text)
	{
		int i=0;
		while (true)
		{
			char s=text[i];
			i++;
			if (s)
			{
				unsigned char ss=s;
				if (ss>=0x20 || ss=='\n')
					str+=s;
			}
			else
				break;
		}
	}
	return str;
}

void GStatic::SysOnChangeText()
{
	int wi=width;
	int he=height;

	FormatText();

	if ((style & GS_NOBGND)==0 && (wi>width || he>height))
	{
		//      ,     
		unsigned int color=theme->GetBgColor(this);
		int x, y;
		GetAbsoluteWindowPosition(x, y);
		GSurface* backbuffer=theme->GetBackbuffer();
		backbuffer->FillRect(x, y, wi, he, color);
	}

	OnChangeText();
}

//   
void GStatic::AddFormatString(STATIC_TEXT* line)
{
	if (k_text)
	{
		STATIC_TEXT** vm_text=new STATIC_TEXT*[k_text+1];

		for (int i=0;i<k_text;i++)
			vm_text[i]=m_text[i];

		delete []m_text;
		m_text=vm_text;
	}
	else
	{
		m_text=new STATIC_TEXT*[1];
	}

	STATIC_TEXT* s=new STATIC_TEXT;
	*s=*line;

	m_text[k_text]=s;
	k_text++;
}

//     
void GStatic::FormatString(STATIC_TEXT* line, bool& new_line)
{
	if (!line->link.IsEmpty())
	{
		//  
		if (line->text.IsEmpty())
		{
			line->text=line->link;
		}
	}

	if (!line->text.IsEmpty())
	{
		if (source_width)
		{
			//    
			if (new_line)
				line->x=0;
			else
			{
				STATIC_TEXT* st=m_text[k_text-1];
				int twi, the;
				game->device->GetTextSize(st->text.c_str(), st->font, twi, the);
				line->x=st->x+twi;
			}
			
			char ss[2];
			ss[1]=0;
			int len=line->text.GetLength();
			int wi=0;
			for (int i=0;i<len;i++)
			{
				ss[0]=line->text[i];
				int swi, she;
				game->device->GetTextSize(ss, line->font, swi, she);
				wi+=swi;
				int x2=line->x+wi;
				if (x2>=source_width)
				{
					//   
					//        
					wi=0;
					int j=i;
					for (;j>=0;j--)
					{
						if (line->text[j]==' ')
						{
							//  
							CMagicString v1=line->text.Left(j);
							CMagicString v2=line->text.Right(len-j-1);
							line->text=v1;
							AddFormatString(line);
							line->text=v2;
							i=-1;
							len=line->text.GetLength();
							line->x=0;
							break;
						}
					}
					if (j<0)
					{
						//    
						if (line->x)
						{
							//       ,      
							line->x=0;
							i=-1;
						}
						else
						{
							//      ,    
							//CMagicString v1=line->text.Left(i-1);
							//int swi, she;
							//game->device->GetTextSize(v1, line->font, swi, she);
							//CMagicString v2=line->text.Right(len-i+1);

							CMagicString v1=line->text.Left(i);
							CMagicString v2=line->text.Right(len-i);

							line->text=v1;
							AddFormatString(line);
							line->text=v2;
							i=-1;
							len=line->text.GetLength();
							line->x=0;
						}
					}
				}
			}

			if (len)
				AddFormatString(line);
		}
		else
		{
			if (!new_line)
			{
				STATIC_TEXT* st=m_text[k_text-1];
				int twi, the;
				game->device->GetTextSize(st->text.c_str(), st->font, twi, the);
				line->x=st->x+twi;
			}
			AddFormatString(line);
		}

		line->text="";
		new_line=false;
	}
}

//    
CMagicString color_text_name[16]={"black", "silver", "gray", "white", "maroon", "red", "purple", "fuchsia", "green", "lime", "olive", "yellow", "navy", "blue", "teal", "aqua"};
unsigned int color_text_value[16]={0, 0xc0c0c0, 0x808080, 0xffffff, 0x800000, 0xff0000, 0x800080, 0xff00ff, 0x008000, 0xff00, 0x808000, 0xffff00, 0x80, 0xff, 0x8080, 0xffff};
void GStatic::FormatText()
{
	int copy_width=width;
	int copy_height=height;

	FormatText2();
	RecalcLayout();

	if (source_width)
	{
		if (IsHScrollBar())
		{
			CMagicRect rect;
			GetClientRect(&rect);
			int vsource_width=source_width;
			source_width=rect.Width();
			int vwidth=width;
			width+=source_width-vsource_width;
			FormatText2();
			source_width=vsource_width;
			width=vwidth;
			RecalcLayout();
		}
	}
	else
	{
		if (/*source_height &&*/ IsVScrollBar())
		{
			CMagicRect rect1, rect2;
			GetClientRect(&rect1);
			GetInnerRect(&rect2);
			int rx=rect2.Width()-rect1.Width();
			width+=rx;
			RecalcLayout();
		}
	}
	
	AlignText();

	if (copy_width!=width || copy_height!=height)
	{
		RecalcCovering();

		if (copy_width>width || copy_height>height)
		{
			int apx, apy;
			parent->GetAbsoluteWindowPosition(apx, apy);

			int ax, ay;
			GetAbsoluteWindowPosition(ax, ay);

			CMagicRect rect;
			rect.left=ax-apx;
			rect.top=ay-apy;
			rect.right=rect.left+copy_width;
			rect.bottom=rect.top+copy_height;

			parent->InvalidateRect(rect);
		}
	}
}


void GStatic::FormatText2()
{
	ClearText();

	STATIC_TEXT line;
	line.x=0;
	line.y=0;

	unsigned int color=text_color;
	if (color==COLOR_KEY_EMPTY)
		color=theme->GetOwner()->GetTextColorDefault();

	line.color=color;
	line.font=theme->GetTextFont(this);

	int len=text.GetLength();
	CMagicString str;

	bool new_line=true;

	for (int i=0;i<len;i++)
	{
		char s=text[i];
		if (s=='<')
		{
			//  
			FormatString(&line, new_line);
			CMagicString tags;
			for (int j=i+1;j<len;j++)
			{
				s=text[j];
				if (s=='>')
				{
					i=j;
					break;
				}
				tags+=s;
			}

			if (!tags.IsEmpty())
			{
				//   
				CText format;
				int flen=tags.GetLength();
				format.Create(flen);
				char* data=format.GetData();
				for (int j=0;j<flen;j++)
					data[j]=tags[j];

				while (true)
				{
					CMagicString param;
					if (format.ReadWordC(&param))
					{
						param=param.toLower();
						if (param=="default")
						{
							line.color=color;
							line.font=theme->GetTextFont(this);
							continue;
						}
						else
						{
							if (param=="/")
							{
								//  
								if (format.ReadWordC(&param))
								{
									if (param=="url")
									{
										line.link="";
										continue;
									}
								}
								else
									break;
							}
						}
					}
					else
						break;

					CMagicString equal;
					if (format.ReadWordC(&equal))
					{
						if (equal!='=')
							break;
					}
					else
						break;

					CMagicString value;
					if (format.ReadWordC(&value))
					{
						DIGIT digit;
						if (digit.FromString(&value))
						{
							if (param=="font")
							{
								if (digit.IsInteger())
									line.font=digit.value1;
								else
									break;
							}
							else
							{
								if (param=="color")
								{
									if (digit.IsInteger())
										line.color=digit.value1;
									else
									{
										if (digit.type==DIGIT_STRING)
										{
											CMagicString str_color=digit.str;
											str_color=str_color.toLower();
											bool is=false;
											for (int k=0;k<16;k++)
											{
												if (str_color==color_text_name[k])
												{
													line.color=color_text_value[k];
													is=true;
													break;
												}
											}

											if (!is)
											{
												//      
												break;
											}
										}
									}
								}
								else
								{
									if (param=="url")
									{
										has_links=true;
										if (digit.str)
											line.link=digit.str;
										else
											line.link="No link";
									}
								}
							}
						}
						else
							break;
					}
					else
						break;
				}
			}
		}
		else
		{
			if (s=='\n')
			{
				//  
				if (line.text.IsEmpty() && line.link.IsEmpty())
					line.text=' ';

				FormatString(&line, new_line);
				new_line=true;
				if (line.x)
				{
					line.x=0;
				}
			}
			else
				line.text+=s;
		}
	}

	FormatString(&line, new_line);

	//     Y
	text_width=text_height=0;
	if (k_text>0)
	{
		int start=0;
		int y=0;
		text_width=text_height=0;

		for (int i=0;i<k_text;i++)
		{
			int end=-1;
			if ((!m_text[i]->x) && start!=i)
			{
				end=i;
				i--;
			}
			else
			{
				if (i==k_text-1)
					end=i+1;
			}

			if (end!=-1)
			{
				//   
				//    
				int max_he=0;
				for (int j=start;j<end;j++)
				{
					int fwi, fhe;
					game->device->GetTextSize(m_text[j]->text.c_str(), m_text[j]->font, fwi, fhe);

					if (max_he<fhe)
						max_he=fhe;

					fwi+=m_text[j]->x;

					if (text_width<fwi)
						text_width=fwi;
				}

				for (int j=start;j<end;j++)
				{
					int fhe=game->device->GetFontex(m_text[j]->font)->data.height;
					m_text[j]->y=text_height+max_he-fhe;
				}

				text_height+=max_he;
				start=end;
			}
		}
	}

	CMagicRect client;
	GetClientRect(&client);
	int rx=width-client.Width();
	int ry=height-client.Height();

	if (source_width)
		width=source_width;
	else
		width=text_width;

	width+=rx;

	if (source_height)
	height=source_height;
	else
	height=text_height;

	height+=ry;
}

//        
void GStatic::AlignText()
{
	if (align!=ALGN_LEFT)
	{
		CMagicRect client;
		GetClientRect(&client);
		int max_width=client.Width();
		if (k_text)
		{
			int first=0;
			int count=1;
			for (int i=1;i<k_text+1;i++)
			{
				if (i==k_text || (!m_text[i]->x))
				{
					//   
					//       

					int line_wi=0;
					for (int j=0;j<count;j++)
					{
						int fwi, fhe;
						int index=j+first;
						game->device->GetTextSize(m_text[index]->text.c_str(), m_text[index]->font, fwi, fhe);
						line_wi+=fwi;
					}

					int nx=max_width-line_wi;
					if (align==ALGN_CENTER)
						nx=nx/2;

					for (int j=0;j<count;j++)
					{
						int index=j+first;
						m_text[index]->x+=nx;
					}
					
					first=i;
					count=1;
				}
				else
					count++;
			}
		}
	}
}

void GStatic::ClientOnRender()
{
	theme->RenderClient(this);
	OnRender();
}

//   
int GStatic::GetCurrentLink()
{
	if (has_links)
		return current_link;
	return -1;
}

// ------------------------------------------------------------------------------------

GPicture::GPicture() : GComponent()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_PICTURE);
	index=-1;
}

bool GPicture::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id)
{
	if (GComponent::Create(parent, style, x, y, width, height, id))
	{
		CMagicRect rect;
		GetClientRect(&rect);
		CMagicRect gap;
		theme->GetClientGap(&gap);
		list.Create(rect.Width()-gap.left-gap.right, rect.Height()-gap.top-gap.bottom);
		return true;
	}
	return false;
}

//    
void GPicture::AddImage(GSurface* surface)
{
	list.AddImage(surface);
	if (list.GetImageCount()==1)
		index=0;
}

//     
int GPicture::GetCount()
{
	return list.GetImageCount();
}

//  
void GPicture::SetSel(int index)
{
	if (this->index!=index)
	{
		this->index=index;
		Invalidate();
	}
}

//   
GSurface* GPicture::GetImage()
{
	GImage* image=list.GetImage(index);
	if (image)
		return image->GetSurface();
	return NULL;
}

void GPicture::ClientOnRender()
{
	theme->RenderClient(this);
	OnRender();
}

//   
GSinglePicture::GSinglePicture()
{
	surface=NULL;
}

GSinglePicture::~GSinglePicture()
{
	if (surface)
	{
		if (game)
		{
			GDevice* device=game->device;
			if (device)
			{
				device->DeleteSurface(surface);
			}
		}
		surface=NULL;
	}
}

bool GSinglePicture::Create(GComponent* parent, unsigned int style, int x, int y, GSurface* surface, unsigned int id)
{
	int wi, he;
	surface->GetSize(wi, he);

	if (GComponent::Create(parent, style, x, y, wi, he, id))
	{
		this->surface=surface;
		return true;
	}
	return false;
}

void GSinglePicture::ClientOnRender()
{
	GSurface* backbuffer=game->GetBackbuffer();
	int x, y;
	GetAbsoluteWindowPosition(x, y);
	backbuffer->Blt(surface, x, y);
}

//  ,       
bool GSinglePicture::IsOver(int x, int y)
{
	int sx, sy;
	GetAbsoluteWindowPosition(sx, sy);
	int xx=sx-x;
	int yy=sy-y;
	return surface->IsOver(xx, yy);
}

// ------------------------------------------------------------------------------------

GDiscretBox::GDiscretBox() : GPicture()
{
	gap_discret.left=2;
	gap_discret.top=0;
	gap_discret.right=2;
	gap_discret.bottom=2;

	sel=over=-1;
}

GDiscretBox::~GDiscretBox()
{
}

bool GDiscretBox::Create(GComponent* parent, unsigned int style, int x, int y, int count, GSize element_size, GSize element_between, unsigned int id)
{
	//    
	this->count=count;
	this->element_size=element_size;
	this->element_between=element_between;

	int border_size=theme->GetBorderSize(this);
	CMagicRect gap;
	theme->GetClientGap(&gap);
	int wi=count*element_size.cx+(count-1)*element_between.cx+border_size*2;//+gap.left+gap.right;
	int he=count*element_size.cy+(count-1)*element_between.cy+border_size*2;//+gap.top+gap.bottom;
	wi+=gap_discret.left+gap_discret.right;
	he+=gap_discret.top+gap_discret.bottom;

	GSurface* image=game->device->AddSurface();
	image->Create(wi, he);
	int len=wi*he;
	for (int i=0;i<len;i++)
		image->MaskTransparentPixel(i);

	unsigned int color=0;//theme->GetSelectedTextColor(this);
	image->DrawHorizontalLine(0, he-1, wi, color);

	wi+=gap.left+gap.right;
	he+=gap.top+gap.bottom;

	if (GPicture::Create(parent, style, x, y, wi, he, id))
	{
		AddImage(image);
		RefreshImage();
		game->device->DeleteSurface(image);
		return true;
	}
	game->device->DeleteSurface(image);
	return false;
}

void GDiscretBox::SetSel(int index)
{
	if (sel!=index)
	{
		if (index>=0 && index<count)
		{
			sel=index;
			RefreshImage();
		}
	}
}

//  
void GDiscretBox::RefreshImage()
{
	GSurface* image=list.GetImage(0)->GetSurface();

	unsigned int color=theme->GetTextColor(this);
	for (int i=0;i<count;i++)
	{
		CMagicRect rect;
		GetDiscretRect(i, &rect);

		unsigned int c=color;
		if (i==sel)
			c=theme->GetSelectedColor(this);
		else
			if (i==over)
				c=theme->GetSelectedTextColor(this);
				
		int y=rect.top;
		for (int j=0;j<i+1;j++)
		{
			image->FillRect(rect.left, y, rect.Width(), element_size.cy, c);
			y+=element_size.cy+element_between.cy;
		}
	}
}

//   
void GDiscretBox::GetDiscretRect(int index, CMagicRect* rect)
{
	if (index>=0 && index<count)
	{
		rect->left=gap_discret.left+index*(element_size.cx+element_between.cx);
		rect->right=rect->left+element_size.cx;
		rect->top=(count-(index+1))*(element_size.cy+element_between.cy);
		rect->bottom=rect->top+(index+1)*element_size.cy+index*element_between.cy;
	}
	else
	{
		rect->left=-1;
		rect->top=-1;
		rect->right=-1;
		rect->bottom=-1;
	}
}

//     
int GDiscretBox::DetectColumn(int x, int y)
{
	CMagicRect gap;
	theme->GetClientGap(&gap);
	x-=gap.left;
	//y-=gap.top;

	for (int i=0;i<count;i++)
	{
		CMagicRect rect;
		GetDiscretRect(i, &rect);
		if (x>=rect.left && x<rect.right)
			return i;
	}

	return -1;
}

void GDiscretBox::ClientOnLButtonDown(unsigned int nFlags, int x, int y)
{
	int column=DetectColumn(x, y);
	if (column!=sel && column!=-1)
	{
		sel=column;
		RefreshImage();
		OnChangeSelected(column);
		Invalidate();
	}
	OnLButtonDown(nFlags, x, y);
}

void GDiscretBox::ClientOnMouseMove(unsigned int nFlags, int x, int y)
{
	int column=DetectColumn(x, y);
	if (nFlags & CONTROL_LEFT)
	{
		if (column!=sel && column!=-1)
		{
			sel=column;
			over=-1;
			RefreshImage();
			OnChangeSelected(column);
			Invalidate();
		}
	}
	else
	{
		if (column!=over)
		{
			over=column;
			RefreshImage();
			Invalidate();
		}
	}
	OnMouseMove(nFlags, x, y);
}

void GDiscretBox::ClientOnMouseLeave(unsigned int nFlags)
{
	if (over!=-1)
	{
		over=-1;
		RefreshImage();
		Invalidate();
	}
	OnMouseLeave(nFlags);
}

// ------------------------------------------------------------------------------------

GDialog::GDialog() : GComponent()
{
	theme=game->GetTheme()->GetThemeComponent(THEME_DIALOG);
	background=NULL;
	is_background_ptr=false;
}

//  
bool GDialog::Create(GComponent* parent, unsigned int style, int x, int y, int width, int height, unsigned int id)
{
	if (GComponent::Create(parent, style, x, y, width, height, id))
	{
		OnInitDlg();
		return true;
	}
	return false;
}

//     
void GDialog::BeginModal()
{
	if (!IsCreated())
	{
		Create(NULL, GS_VISIBLE | GS_BORDER | GS_CAPTION, 0, 0, 100, 100, 0);
	}
	else
		OnInitDlg();

	GComponent::BeginModal();

	AlignToCenter();
}

//      
GButton* GDialog::GetEnterButton()
{
	GComponent* focus=game->GetFocusComponent();
	if ((!focus) || (!focus->IsEdit()) || (!(focus->GetStyle() & GSES_OWNRETURN)))
	{
		int i;

		for (i=0;i<k_child;i++)
		{
			GComponent* c=m_child[i];
			if (c->IsButton())
			{
				unsigned int  style=c->GetStyle();
				if (style & GSBS_OK)
				{
					//     OK
					if ((!(style & GS_DISABLED)) && (style & GS_VISIBLE))
						return (GButton*)c;
					else
					{
						for (int j=i+1;j<k_child;j++)
						{
							GComponent* c=m_child[j];
							if (c->IsButton())
							{
								unsigned int  style=c->GetStyle();
								if (style & GSBS_OK)
								{
									//    ,   GSBS_OK
									if ((!(style & GS_DISABLED)) && (style & GS_VISIBLE))
										return (GButton*)c;
								}
							}
						}
						//      ,    
						return NULL;
					}
				}
			}
		}

		for (i=0;i<k_child;i++)
		{
			GComponent* c=m_child[i];
			if (c->IsButton() && c->IsVisibled())
			{
				if (c->IsEnabled())
					return ((GButton*)m_child[i]);
				break;
			}
		}
	}
	return NULL;
}

//      
GButton* GDialog::GetEscapeButton()
{
	GComponent* focus=game->GetFocusComponent();
	if ((!focus) || (!focus->IsEdit()) || (!(focus->GetStyle() & GSES_OWNRETURN)))
	{
		int i;

		for (i=0;i<k_child;i++)
		{
			GComponent* c=m_child[i];
			if (c->IsButton())
			{
				unsigned int  style=c->GetStyle();
				if (style & GSBS_CANCEL)
				{
					//     GSBS_CANCEL
					if ((!(style & GS_DISABLED)) && (style & GS_VISIBLE))
						return (GButton*)c;
					else
					{
						for (int j=i+1;j<k_child;j++)
						{
							GComponent* c=m_child[j];
							if (c->IsButton())
							{
								unsigned int  style=c->GetStyle();
								if (style & GSBS_CANCEL)
								{
									//    ,   GSBS_CANCEL
									if ((!(style & GS_DISABLED)) && (style & GS_VISIBLE))
										return (GButton*)c;
								}
							}
						}
						//      ,    
						return NULL;
					}
				}
			}
		}

		int k=0;
		for (i=0;i<k_child;i++)
		{
			GComponent* c=m_child[i];
			if (c->IsButton() && c->IsVisibled())
			{
				k++;
				if (k==2)
				{
					if (c->IsEnabled())
						return ((GButton*)c);
					break;
				}
			}
		}
	}
	return NULL;
}

void GDialog::SetBackground(const char* file)
{
	if (background)
	{
		if (!is_background_ptr)
			game->device->DeleteSurface(background);
		background=NULL;
	}
	is_background_ptr=false;

	if (file)
	{
		FILE_DATA fd;
		if (game->platform->LoadFile(file, &fd))
		{
			background=game->device->AddSurface();
			if (!background->Load(fd.data, fd.length))
			{
				game->device->DeleteSurface(background);
				background=NULL;
			}
		}
	}

	Invalidate();
}

void GDialog::SetBackground(GSurface* surface)
{
	if (background)
	{
		if (!is_background_ptr)
			game->device->DeleteSurface(background);
		background=NULL;
	}
	is_background_ptr=false;
	background=surface;

	Invalidate();
}

//      (  ,   ,    )
void GDialog::SetBackgroundPtr(GSurface* surface)
{
	if (background)
	{
		if (!is_background_ptr)
			game->device->DeleteSurface(background);
		background=NULL;
	}
	is_background_ptr=true;
	background=surface;

	Invalidate();
}


bool GDialog::WinOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (nChar==KEY_RETURN)
	{
		GComponent* enter=GetEnterButton();
		if (enter)
		{
			enter->Fix(true);
			return false;
		}
	}
	else
	{
		if (nChar==KEY_ESCAPE)
		{
			GComponent* escape=GetEscapeButton();
			if (escape)
			{
				escape->Fix(true);
				return false;
			}
		}
	}
	return true;
}

bool GDialog::WinOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (nChar==KEY_RETURN)
	{
		GButton* enter=GetEnterButton();
		if (enter)
		{
			enter->Fix(false);
			//enter->ClientOnBtnClk();
			enter->ClientOnLButtonClick(nFlags, -1, -1);
			return false;
		}
	}
	else
	{
		if (nChar==KEY_ESCAPE)
		{
			GButton* escape=GetEscapeButton();
			if (escape)
			{
				escape->Fix(false);
				//escape->ClientOnBtnClk();
				escape->ClientOnLButtonClick(nFlags, -1, -1);
				return false;
			}
		}
	}
	return true;
}

void GDialog::OnEnter()
{
	if (game->GetModalComponent()==this)
		EndModal();
}

void GDialog::OnCancel()
{
	if (game->GetModalComponent()==this)
		EndModal();
}

void GDialog::OnInitDlg()
{
	for (int i=0;i<k_child;i++)
	{
		DeleteChild(m_child[i]);
		i--;
	}
}

// ------------------------------------------------------------------------------------

// ,  MessageBox
void GMessageBox::Alert(const char* title, const char* message, unsigned int style)
{
	if (!IsCreated())
		GDialog::Create(NULL, GS_BORDER | GS_CAPTION, 0, 0, 100, 100, 0xfffffe);
	else
	{
		for (int i=0;i<k_child;i++)
		{
			DeleteChild(m_child[i]);
			i--;
		}
	}

	SetWindowText(title);
	int max_nessage_width=400;	//        
	int left_right_gap=32;		//    
	int top_bottom_gap=22;		//    
	int button_gap=20;			//     

	int button_width, button_height=20;
	theme->GetOwner()->GetButtonSizeForMessageBox(button_width, button_height);
	int between_buttons=30;		//   

	CMagicRect inner;
	theme->GetInnerGap(this, &inner);
	CMagicRect client;
	theme->GetClientGap(&client);

	GStatic* st=new GStatic;
	st->Create(this, GS_VISIBLE | GS_NOBGND, 0, 0, max_nessage_width, 0, 0, message);

	//    MessageBox-   
	int mwi, mhe;
	st->GetSize(mwi, mhe);

	int k_button=0;
	if (style & GSMB_OK)
		k_button=1;
	if (style & GSMB_CANCEL)
		k_button++;

	if (!k_button)
		k_button=1;

	int button_width_full=button_width*k_button+between_buttons*(k_button-1);
	int min_button_size=button_width_full+(between_buttons*3)*2;
	if (mwi<min_button_size)
		mwi=min_button_size;

	int border_size=theme->GetBorderSize(this);
	int caption_height=theme->GetCaptionHeight(this);

	int client_wi=mwi+left_right_gap*2;
	width=client.left+client.right+inner.left+inner.right+client_wi;
	int client_he=top_bottom_gap*2+button_height+top_bottom_gap+mhe;
	height=client.top+client.bottom+inner.top+inner.bottom+client_he;// border_size*2+caption_height+gap.top+gap.bottom+client_he+button_height;

	//     
	GDevice* device=game->device;
	int stwi=0;
	int k_str=st->GetStringCount();
	for (int i=0;i<k_str;i++)
	{
		STATIC_TEXT* t=st->GetString(i);
		int twi, the;
		device->GetTextSize(t->text, t->font, twi, the);
		twi+=t->x;
		if (stwi<twi)
			stwi=twi;
	}

	int v, sthe;
	st->GetSize(v, sthe);
	st->SetSize(stwi, sthe);
	CMagicRect rect;
	st->GetClientRect(&rect);
	int rwi=stwi-rect.Width();
	if (rwi)
		st->SetSize(stwi+rwi, sthe);

	int stx=(width-stwi)/2;//left_right_gap;
	int sty=top_bottom_gap;
	st->SetPosition(stx, sty);

	int button_x=(client_wi-button_width_full)/2;
	int button_y=sty+mhe+button_gap;
	
	CMagicString button_name[]={"OK", "Cancel"};

	for (int i=0;i<k_button;i++)
	{
		GButton* b=new GButton;
		b->Create(this, GS_VISIBLE | GS_BORDER, button_x, button_y, button_width, button_height, 100+i, theme->GetOwner()->Translate(button_name[i]));
		button_x+=button_width+between_buttons;
	}

	//   
	int swi, she;
	game->device->GetSize(swi, she);
	x=(swi-width)/2;
	y=(she-height)/2;

	GComponent::BeginModal();

	Show(true);
}

//    
void GMessageBox::Invalidate()
{
	//        
	GSurface* backbuffer=game->GetBackbuffer();
	int wi, he;
	backbuffer->GetSize(wi, he);
	int sx=(wi-width)/2;
	int sy=(he-height)/2;
	int ax, ay;
	GetAbsoluteWindowPosition(ax, ay);
	if (ax!=sx || ay!=sy)
	{
		x+=sx-ax;
		y+=sy-ay;
	}

	GDialog::Invalidate();
}

// ------------------------------------------------------------------------------------

GScene::GScene() : GDialog()
{
	style=0;//GS_VISIBLE;// | GS_DISABLED;

	game->device->GetSize(width, height);

	previous_scene=-1;

	scene_period=0;

	is_covering=true;
	k_component=0;
}

GScene::~GScene()
{
}

void GScene::OnUpdate()
{
	if (scene_period)
	{
		unsigned int tick=game->platform->GetTick();
		if (tick>=scene_tick+scene_period)
		{
			scene_tick=tick;
			OnPeriod();
		}
	}
}

//    
void GScene::HideChildren()
{
	for (int i=0;i<k_child;i++)
	{
		m_child[i]->Show(false);
	}
}

void GScene::SetScenePeriod(unsigned int tick_period)
{
	scene_period=tick_period;
	scene_tick=game->platform->GetTick();
}

//     ,    
void GScene::RefreshCovering()
{
	if (is_covering)
	{
		is_covering=false;

		k_component=0;
		
		m_component[k_component]=this;
		k_component++;

		DetectCovering(k_component, m_component);
		GComponent* modal_component=game->GetModalComponent();
		if (modal_component)
		{
			//       ,    
			m_component[k_component]=modal_component;
			k_component++;
			modal_component->DetectCovering(k_component, m_component);
		}

		for (int i=0;i<k_component;i++)
		{
			GComponent* c=m_component[i];
			CMagicRect r1;
			c->GetAbsoluteWindowPosition(r1.left, r1.top);
			int wi, he;
			c->GetSize(wi, he);
			r1.right=r1.left+wi;
			r1.bottom=r1.top+he;

			c->ClearCoveringComponents();

			for (int j=i+1;j<k_component;j++)
			{
				GComponent* cc=m_component[j];
				CMagicRect r2;
				cc->GetAbsoluteWindowPosition(r2.left, r2.top);
				int wi2, he2;
				cc->GetSize(wi2, he2);
				r2.right=r2.left+wi2;
				r2.bottom=r2.top+he2;

				CMagicRect r;
				if (r.Intersect(r1, r2))
					c->AddCoveringComponent(cc);
			}
		}
	}
}

//   
void GScene::RenderAll()
{
	RefreshCovering();

	int i;
	//  ,    
	for (i=0;i<k_component;i++)
	{
		GComponent* component=m_component[i];
		if (component->IsInvalidated())
			component->InvalidateCoveringComponents();
	}

	//   
	for (i=0;i<k_component;i++)
	{
		GComponent* component=m_component[i];
		component->Draw();
	}
}

// ------------------------------------------------------------------------------------

GGame::GGame(GUID_GAME guid, const char* name, const char* version, bool fullscreen_mode)
{
	GComponent::game=this;
	GDevice::game=this;
	GNet::game=this;

	this->guid=guid;
	game_name=name;
	game_version=version;
	game_digital_version.FromString(version);

	this->fullscreen_mode=fullscreen_mode;

	user="";

	theme=NULL;

	backbuffer=NULL;

	platform=NULL;
	device=NULL;
	net=NULL;
	net_downloader=NULL;
	sound=NULL;

	cursor_x=cursor_y=100;
	fcursor_x=fcursor_y=100.f;
	cursor_factor=1.f;

	cursor=NULL;
	is_cursor=true;

	//max_invalidated=k_invalidated=0;
	//m_invalidated=NULL;

	k_scene=0;
	m_scene=NULL;
	t_scene=-1;

	modal=NULL;
	focus=NULL;
	pressed=NULL;
	key_down=NULL;
	key_down_count=0;
	ClearControl();

	cursor_default=NULL;

	#ifndef _DEBUG
	tick_mouse_capture=0;
	#endif
}

GGame::~GGame(void)
{
	KillMainTimer();

	while (modal)
	{
		modal->EndModal();
	}

	StopNet();
	StopNetDownloader();

	GComponent::game=NULL;
	GDevice::game=NULL;

	if (platform)
	{
		delete platform;
		platform=NULL;
	}

	if (sound)
	{
		delete sound;
		sound=NULL;
	}

	for (int i=0;i<k_scene;i++)
		delete m_scene[i];

	if (m_scene)
	{
		delete []m_scene;
		m_scene=NULL;
	}

	/*
	if (m_invalidated)
	{
		delete []m_invalidated;
		m_invalidated=NULL;
	}
	max_invalidated=k_invalidated=0;
	*/

	k_scene=0;
	t_scene=-1;

	if (device)
	{
		delete device;
		device=NULL;
	}

	if (theme)
	{
		delete theme;
		theme=NULL;
	}
}

//  
void GGame::ClearControl()
{
	//if (focus)
	//	focus->SysOnKillFocus();
	//captured=focus=over=NULL;
	captured=over=NULL;
	control=0;
	for (int i=0;i<256;i++)
		keys[i]=false;

	ResetRepeatedKey();
}

//  ,    
bool GGame::CreateGame(void* init, int view_width, int view_height)
{
	platform=NewPlatform(this, game_name.c_str());
	platform->Create();

	device=NewDevice(init);
	device->Create(view_width, view_height);
	sound=NewSound();

	VIDEO_MODE vm;
	GetStartedVideoMode(&vm);

	if (view_width<=0)
		view_width=vm.width;
	if (view_height<=0)
		view_height=vm.height;

	backbuffer=device->AddSurface();
	backbuffer->Create(view_width, view_height);

	FILE_DATA fd;
	GSurface* cur=device->AddSurface();
	CMagicString path_cursor=platform->GetPathToGame();
	path_cursor+="cursor.png";
	platform->LoadFile(path_cursor, &fd);
	if (cur->Load(fd.data, fd.length))
		cursor_default=device->AddPicture(cur, 0, 0);


	#ifndef MAX_FPS
	SetMainTimer(15);
	#endif

	unsigned int tick=platform->GetTick();
	#ifndef _DEBUG
	tick_mouse_capture=tick;
	#endif
	SetRndIndex(tick);

	/*
	#ifdef _DEBUG
	//      
	//license_type=0;
		for (int i=0;i<16;i++)
		license+=(char)GetRndInt('0', '9');
	#endif
	*/

	platform->DetectFirstLaunch();

	LoadLicense();
	if (!GetLicense())
	{
		//  
		//   
		for (int i=0;i<16;i++)
			license+=(char)GetRndInt(' ', 255);
		SaveLicense(license);
		license="";
	}

	return true;
}

// 
const CMagicString* GGame::GetLicense()
{
	if (license.IsEmpty())
		return NULL;
	return &license;
}

//   (Steam/)
int GGame::GetLicenseType()
{
	#ifdef STEAM
	return 1;
	#else
	return 0;
	#endif
}

CMagicString license_file="fpsl.ctl";
unsigned char license_xor=0x4a;
//  
void GGame::SaveLicense(const char* license)
{
	CMagicString path=platform->GetPathToConfig();
	path+=license_file;

	CMagicStreamFile file;
	if (file.Open(path, MAGIC_STREAM_ADD))
	{
		CMagicString password=license;
		int len=password.GetLength();
		for (int i=0;i<len;i++)
			password[i]=password[i] ^ license_xor;
		//    8 
		int str0=0;
		file<<str0;
		file<<str0;
		file<<password;
	}

	this->license=license;
}

//  
void GGame::LoadLicense()
{
	#ifdef STEAM

	license="STEAM";
	if (InitSteam())
	{
		CSteamID user_id=SteamUser()->GetSteamID();
		int size=sizeof(user_id);
		while(license.GetLength()<size)
			license+=' ';
		char* v=(char*)license.c_str();
		memcpy(v, &user_id, size);
		CloseSteam();
	}

	#else

	license="";
	CMagicString path=platform->GetPathToConfig();
	path+=license_file;

	FILE_DATA fd;
	if (platform->LoadFile(path, &fd))
	{
		int pos=-1;
		if (fd.length>=7)
		{
			for (int i=fd.length-1;i>=7;i--)
			{
				if (!fd.data[i])
				{
					bool is=true;
					for (int j=1;j<8;j++)
					{
						if (fd.data[i-j])
						{
							is=false;
							break;
						}
					}
					if (is)
					{
						pos=i+1;
						break;
					}
				}
			}
		}

		if (pos!=-1)
		{
			//  
			rd1.Start(&(fd.data[pos]));
			rd1>>license;
			int len=license.GetLength();
			for (int i=0;i<len;i++)
				license[i]=license[i] ^ license_xor;

			if (!VerifyLicense(GetLicenseType(), license))
				license="";
		}
	}
	#endif
}

//     
GComponent* GGame::DetectComponentByCursor(int& x, int& y, int& place)
{
	x=cursor_x;
	y=cursor_y;

	if (captured)
	{
		place=captured_place;
		return captured->CorrectPositionClientForCaptured(x, y);
	}

	if (modal)
	{
		place=COMPONENT_PLACE_WIN;

		GComponent* component=modal->CorrectPositionClientForModal(x, y);
		
		int vx, vy;
		component->GetAbsoluteWindowPosition(vx, vy);
		int rx=x-vx;
		int ry=y-vy;
		int cx, cy;
		component->GetPosition(cx, cy);
		x=cx+rx;
		y=cy+ry;

		GComponent* child_modal=component->DetectComponentByCursorRecurse(x, y, place);
		if (child_modal && child_modal!=component)
			return child_modal;

		//x=rx;
		//y=ry;
		
		return component;
	}
	else
	{
		GScene* scene=GetScene();
		if (scene)
			return scene->DetectComponentByCursorRecurse(x, y, place);
	}

	return NULL;
}

//   
GScene* GGame::GetScene()
{
	if (t_scene!=-1)
		return m_scene[t_scene];
	return NULL;
}

//   
GScene* GGame::GetScene(int index)
{
	if (index>=0 && index<k_scene)
		return m_scene[index];
	return NULL;
}


// / 
void GGame::ShowCursor(bool show)
{
	if (is_cursor!=show)
	{
		is_cursor=show;
		if (is_cursor)
			SysOnMouseMove(control);
	}
}


//     
GImage* GGame::GetCursor()
{
	if (is_cursor)
	{
		if (!cursor)
			cursor=GetCursorDefault();
		return cursor;
	}
	return NULL;
}

//   
void GGame::SetCursorPosition(int x, int y)
{
	if (cursor_x!=x || cursor_y!=y)
	{
		int old_x=cursor_x;
		int old_y=cursor_y;

		cursor_x=x;
		cursor_y=y;

		fcursor_x=(float)cursor_x;
		fcursor_y=(float)cursor_y;

		int wi, he;
		device->GetSize(wi, he);

		if (cursor_x<0)
		{
			cursor_x=0;
			fcursor_x=0.f;
		}
		else
			if (cursor_x>=wi)
			{
				cursor_x=wi-1;
				fcursor_x=(float)cursor_x;
			}

			if (cursor_y<0)
			{
				cursor_y=0;
				fcursor_y=0.f;
			}
			else
				if (cursor_y>=he)
				{
					cursor_y=he-1;
					fcursor_y=(float)cursor_y;
				}

		if (old_x!=cursor_x || old_y!=cursor_y)
			SysOnMouseMove(control);
	}
}

//    
bool GGame::IsKeyHeld(unsigned int key)
{
	if (key<1 || key>=KEY_MAX)
		return false;

	switch (key)
	{
	case KEY_LALT:
	case KEY_RALT:
		return (control & CONTROL_ALT)>0;

	case KEY_LCONTROL:
	case KEY_RCONTROL:
		return (control & CONTROL_CTRL)>0;

	case KEY_LSHIFT:
	case KEY_RSHIFT:
		return (control & CONTROL_SHIFT)>0;

	case KEY_LMOUSE:
		return (control & CONTROL_LEFT)>0;

	case KEY_RMOUSE:
		return (control & CONTROL_RIGHT)>0;

	case KEY_MMOUSE:
		return (control & CONTROL_MIDDLE)>0;
		break;

	default:
		return keys[key];
	}
}

//  
void GGame::PlatformMoveCursor(int sx, int sy)
{
	float fsx, fsy;
	if (device->IsWindowed())
	{
		fsx=(float)sx;
		fsy=(float)sy;
	}
	else
	{
		fsx=((float)sx)*cursor_factor;
		fsy=((float)sy)*cursor_factor;
	}
	
	fcursor_x+=fsx;
	fcursor_y+=fsy;

	int old_x=cursor_x;
	int old_y=cursor_y;

	cursor_x=(int)fcursor_x;
	cursor_y=(int)fcursor_y;

	int wi, he;
	device->GetSize(wi, he);

	if (cursor_x<0)
	{
		cursor_x=0;
		fcursor_x=0.f;
	}
	else
		if (cursor_x>=wi)
		{
			cursor_x=wi-1;
			fcursor_x=(float)cursor_x;
		}

	if (cursor_y<0)
	{
		cursor_y=0;
		fcursor_y=0.f;
	}
	else
		if (cursor_y>=he)
		{
			cursor_y=he-1;
			fcursor_y=(float)cursor_y;
		}

	if (old_x!=cursor_x || old_y!=cursor_y)
		SysOnMouseMove(control);
}

//   ,  
void GGame::PlatformKeyDown(int code, unsigned int nRepCnt)
{
	if (code<1 || code>=KEY_MAX)
		return;

	if (!keys[code])
	{
		keys[code]=true;
		unsigned int control_keys=control & (CONTROL_ALT | CONTROL_CTRL | CONTROL_SHIFT);
		//    
		switch (code)
		{
		case KEY_LMOUSE:
			if (!(control & (CONTROL_LEFT | CONTROL_RIGHT | CONTROL_MIDDLE)))
			{
				control=control | CONTROL_LEFT;
				SysOnLButtonDown(control_keys);
			}
			break;

		case KEY_RMOUSE:
			if (!(control & (CONTROL_LEFT | CONTROL_RIGHT | CONTROL_MIDDLE)))
			{
				control=control | CONTROL_RIGHT;
				SysOnRButtonDown(control_keys);
			}
			break;

		case KEY_MMOUSE:
			if (!(control & (CONTROL_LEFT | CONTROL_RIGHT | CONTROL_MIDDLE)))
			{
				control=control | CONTROL_MIDDLE;
				SysOnMButtonDown(control_keys);
			}
			break;

		case KEY_LALT:
		case KEY_RALT:
			control=control | CONTROL_ALT;
			break;

		case KEY_LSHIFT:
		case KEY_RSHIFT:
			control=control | CONTROL_SHIFT;
			break;

		case KEY_LCONTROL:
		case KEY_RCONTROL:
			control=control | CONTROL_CTRL;
			break;

		case KEY_SYSRQ:
			//  PrintScreen
			{
				keys[code]=false;
				SavePrintScreen();
			}
			break;

		default:
			if (code==KEY_TAB && (control & CONTROL_ALT))
			{
				//  ALT+TAB
				ToIconic();
			}
			else
			{
				//    
				key_down_count++;
				SysOnKeyDown(code, nRepCnt, control);
				SetRepeatedKey(code);
			}

			break;
		}
	}
}

//   ,  
void GGame::PlatformKeyUp(int code)
{
	if (code<1 || code>=KEY_MAX)
		return;

	if (keys[code])
	{
		keys[code]=false;
		unsigned int control_keys=control & (CONTROL_ALT | CONTROL_CTRL | CONTROL_SHIFT);
		//    
		switch (code)
		{
		case KEY_LMOUSE:
			control=control & (~CONTROL_LEFT);
			ResetRepeatedKey();
			SysOnLButtonUp(control_keys);
			break;

		case KEY_RMOUSE:
			control=control & (~CONTROL_RIGHT);
			ResetRepeatedKey();
			SysOnRButtonUp(control_keys);
			break;

		case KEY_MMOUSE:
			control=control & (~CONTROL_MIDDLE);
			ResetRepeatedKey();
			SysOnMButtonUp(control_keys);
			break;

		case KEY_LALT:
		case KEY_RALT:
			control=control & (~CONTROL_ALT);
			break;

		case KEY_LSHIFT:
		case KEY_RSHIFT:
			control=control & (~CONTROL_SHIFT);
			break;

		case KEY_LCONTROL:
		case KEY_RCONTROL:
			control=control & (~CONTROL_CTRL);
			break;

		default:
			//    
			key_down_count--;
			ResetRepeatedKey();
			SysOnKeyUp(code, 0, control);
			break;
		}
	}
}

// 
void GGame::PlatformTimer()
{
	SysOnUpdate();
	if (IsActivated())
	{
		//    
		unsigned int tick=platform->GetTick();
		#ifndef _DEBUG
		unsigned int dtime=tick-tick_mouse_capture;
		if (dtime>1000*30)
		{
			tick_mouse_capture=tick;
			if ((!device->IsWindowed()) && platform)
			{
				//    ,        
				platform->CaptureMouse();
			}
		}
		#endif

		if (focus)
		{
			focus->SysOnFocusTimer(tick);
		}

		RepeateKey();

		SysOnRender();
	}
	else
	{
		//   
		if (!device->IsWindowed())
		{
			if (!IsIconic())
			{
				//   
				//  
				ToIconic();
			}
		}
	}
}

//  ,    
bool GGame::IsActivated()
{
	if (platform)
		return platform->IsActivated();
	return false;
}

//  ,   
bool GGame::IsIconic()
{
	if (platform)
		return platform->IsIconic();
	return false;
}

//    
void GGame::ToIconic()
{
	if (platform)
		platform->ToIconic();
}

//    
void GGame::ToMaximize()
{
	if (platform)
		platform->ToMaximize();
}

//   
void GGame::GetStartedVideoMode(VIDEO_MODE* vm)
{
	device->GetStartedMode(vm);
}

//   
bool GGame::SetScene(int index)
{
	if (index>=0 && index<k_scene)
	{
		if (t_scene!=index)
		{
			GComponent* modal=GetModalComponent();
			if (modal)
				modal->EndModal();

			if (t_scene!=-1)
			{
				m_scene[t_scene]->Show(false);
				m_scene[t_scene]->OnFinish(index);
			}

			ClearControl();
			PlatformMoveCursor(0, 0);

			m_scene[index]->SetPreviousScene(t_scene);
			t_scene=index;

			focus=m_scene[index];
			m_scene[index]->ShowChildrenByDefault();
			//k_invalidated=0;
			m_scene[index]->Show(true);
			m_scene[index]->OnStart();
			m_scene[index]->Invalidate();
			SysOnMouseMove(control);
		}
		return true;
	}
	return false;
}

//  
void GGame::AddScene(GScene* scene)
{
	if (k_scene)
	{
		GScene** vm_scene=new GScene*[k_scene+1];

		for (int i=0;i<k_scene;i++)
			vm_scene[i]=m_scene[i];

		delete []m_scene;
		m_scene=vm_scene;
	}
	else
	{
		m_scene=new GScene*[1];
	}

	m_scene[k_scene]=scene;
	k_scene++;

	if (t_scene==-1)
	{
		SetScene(0);
		focus=scene;
		scene->Show(true);
		scene->Invalidate();
	}
}

//  
int GGame::CreateFontex(const char* name)
{
	int index=-1;
	FILE_DATA fd1, fd2;
	CMagicString path;
	CMagicString texture_name=name;
	texture_name+=".tga";
	if (!platform->LoadFile(texture_name, &fd1))
	{
		path=GetPathToFont();
		texture_name=path+texture_name;
		if (!platform->LoadFile(texture_name, &fd1))
			return -1;
	}

	CMagicString description_name=path+name;
	description_name+=".txt";
	if (platform->LoadFile(description_name, &fd2))
		index=device->AddFontex(fd1.data, fd1.length, fd2.data, fd2.length);
	return index;
}

/*
//      
void GGame::AddInvalidate(GComponent* component)
{
	if (k_invalidated==max_invalidated)
	{
		if (max_invalidated)
		{
			GComponent** vm_invalidated=new GComponent*[max_invalidated+1];

			for (int i=0;i<k_invalidated;i++)
				vm_invalidated[i]=m_invalidated[i];

			delete []m_invalidated;
			m_invalidated=vm_invalidated;
		}
		else
		{
			m_invalidated=new GComponent*[1];
		}
		max_invalidated++;
	}

	m_invalidated[k_invalidated]=component;
	k_invalidated++;
}

//     
void GGame::DeleteInvalidate(GComponent* component)
{
	for (int i=0;i<k_invalidated;i++)
	{
		if (m_invalidated[i]==component)
		{
			for (int j=i+1;j<k_invalidated;j++)
			{
				m_invalidated[j-1]=m_invalidated[j];
			}
			k_invalidated--;
			break;
		}
	}
}
*/

//     
void GGame::SetMainTimer(unsigned int period)
{
	KillMainTimer();
	main_timer.period=period;
	platform->SetTimer(&main_timer);
}

//   
void GGame::KillMainTimer()
{
	if (IsMainTimer())
	{
		platform->KillTimer(&main_timer);
		main_timer.Clear();
	}
}

//     
bool GGame::IsMainTimer()
{
	return (main_timer.timer!=0);
}

//   
unsigned int GGame::GetMainTimerPeriod()
{
	unsigned int period;
	if (IsMainTimer())
		period=main_timer.period;
	else
		period=0;
	return period;

}

//  
void GGame::SysOnRender()
{
	bool is=IsActivated();
	if (is)
	{
		GScene* scene=GetScene();
		if (scene)
			scene->RenderAll();


		GComponent* cur_component=captured;
		int cur_place=captured_place;
		if (!cur_component)
		{
			cur_component=over;
			cur_place=over_place;
		}
		if (cur_component)
		{
			int x, y;
			cur_component->GetAbsoluteWindowPosition(x, y);
			x=cursor_x-x;
			y=cursor_y-y;

			switch (cur_place)
			{
			case COMPONENT_PLACE_WIN:
				cursor=cur_component->OnSetCursorWIN(control, x, y);
				break;
			case COMPONENT_PLACE_INNER:
				{
					int sx, sy;
					cur_component->GetInnerOffset(sx, sy);
					x-=sx;
					y-=sy;
					cursor=cur_component->OnSetCursorINNER(control, x, y);
				}
				break;
			case COMPONENT_PLACE_CLIENT:
				{
					int sx, sy;
					cur_component->GetInnerOffset(sx, sy);
					x-=sx;
					y-=sy;
					cur_component->GetClientOffset(sx, sy);
					x-=sx;
					y-=sy;
					cursor=cur_component->OnSetCursor(control, x, y);
				}
				break;
			}
		}
	}
}

//  
void GGame::OnChangeVideoMode(int width, int height)
{
	if (backbuffer)
		backbuffer->Create(width, height);

	for (int i=0;i<k_scene;i++)
	{
		GScene* scene=m_scene[i];
		scene->SetSize(width, height);
	}

	GScene* scene=GetScene();
	if (scene)
		scene->Invalidate();
}

//  
void GGame::SetRepeatedKey(unsigned int rkey)
{
	if (repeated_key==0xffffffff && repeated_char==0xffffffff)
	{
		if (rkey!=KEY_LSHIFT && rkey!=KEY_RSHIFT && rkey!=KEY_LCONTROL && rkey!=KEY_RCONTROL && rkey!=KEY_LALT && rkey!=KEY_RALT)
		{
			repeated_key=rkey;
			unsigned int tick=platform->GetTick();
			repeated_tick=tick+350;
		}
	}
}

void GGame::SetRepeatedChar(unsigned int rchar)
{
	if (repeated_char==0xffffffff)
	{
		ResetRepeatedKey();
		repeated_key=rchar;
		unsigned int tick=platform->GetTick();
		repeated_tick=tick+350;
	}
}

void GGame::ResetRepeatedKey()
{
	repeated_key=repeated_char=repeated_tick=0xffffffff;
	repeated_count=0;
}

void GGame::RepeateKey()
{
	if (repeated_key!=0xffffffff || repeated_char!=0xffffffff)
	{
		unsigned int tick=platform->GetTick();
		if (repeated_tick<tick)
		{
			repeated_tick=tick+75;
			repeated_count++;
			if (repeated_key!=0xffffffff)
			{
				if (keys[repeated_key])
				{
					keys[repeated_key]=false;
					unsigned int vcontrol= control & (~(CONTROL_LEFT | CONTROL_RIGHT | CONTROL_MIDDLE));
					PlatformKeyDown(repeated_key, repeated_count);
					control=vcontrol;
					keys[repeated_key]=true;
				}
				else
					ResetRepeatedKey();
			}
			else
			{
				SysOnChar(repeated_char, repeated_count, control);
			}
		}
	}
}

//  
CMagicString GGame::SavePrintScreen()
{
	const char* vpath=platform->GetPathToProfile();
	if (!vpath)
		vpath=platform->GetPathToTemp();
	if (vpath)
	{
		CMagicString path=vpath;
		for (int i=0;true;i++)
		{ 
			//    
			CMagicString filename;
			filename.IntToString(i);
			int dlina=8-filename.GetLength();
			CMagicString name="";
			for (int j=0;j<dlina;j++)
				name+='0';
			name+=filename;
			name+=".tga";
			CMagicString fn=path;
			fn=fn+name;
			FILE* f=platform->OpenFile(fn, "rb");
			if (f)
				platform->CloseFile(f);
			else
			{
				//   
				GSurface* backbuffer=GetBackbuffer();
				backbuffer->SaveTGA(fn);
				return fn;
			}
		}
	}
	return "";
}

//   
bool GGame::StartNet(const char* ip)
{
	StopNet();

	hello_server_reply="";

	//  
	net=NewNet();

	CMagicString ip_addr;
	if (ip)
	{
		ip_addr=ip;
		if (!net->IsServer())
		{
			GNetClient* client=(GNetClient*)net;
			int pos=ip_addr.FindChar(':');
			if (pos!=-1)
			{
				//     
				ip_addr=ip_addr.Left(pos);
			}
			ip_addr=client->URLtoIP(ip_addr);
			if (ip_addr.IsEmpty())
				ip_addr=ip;
			else
			{
				//     
				ip_addr+=&(ip[pos]);
			}
		}
	}

	bool is;
	if (ip_addr.IsEmpty())
	{
		is=((GNetClient*)net)->Create(guid, NULL);
		if (is)
			is=((GNetClient*)net)->SetStady(NET_STADY_ENUM_SESSION, NULL);
	}
	else
	{
		is=((GNetClient*)net)->Create(guid, ip_addr);
		if (is)
			is=((GNetClient*)net)->SetStady(NET_STADY_ENUM_SESSION, (void*)(ip_addr.c_str()));
	}

	if (!is)
		StopNet();

	return is;
}

//   
void GGame::StopNet()
{
	if (net)
	{
		net->Destroy();
		delete net;
		net=NULL;
	}
}

//  ,     
bool GGame::IsMigratingHost()
{
	if (net && (!net->IsServer()))
	{
		NET_STADY stady=((GNetClient*)net)->GetStady();
		if (stady==NET_STADY_MIGRATION_HOST)
			return true;
	}
	return false;
}

//    
bool GGame::DownloadFile(const char* url_file)
{
	if (!net_downloader)
		net_downloader=NewDownloader();

	return net_downloader->DownloadFile(url_file);
}

//    
void GGame::StopNetDownloader()
{
	if (net_downloader)
	{
		net_downloader->Destroy();
		delete net_downloader;
		net_downloader=NULL;
	}
}


//  ,     
void GGame::Play()
{
	if (platform)
		platform->Play();
}

// ,       
void GGame::Quit()
{
	if (CanQuit())
	{
		if (platform)
			platform->Quit();
	}
}

void GGame::SysOnActivate(unsigned int nState, bool bMinimized)
{
	if (nState==WA_INACTIVE)
	{
		//  
		#ifndef _DEBUG
		if ((!device->IsWindowed()) && platform)
			platform->ReleaseMouse();
		#endif

		//if (!bMinimized)
		{
			ClearControl();
			device->Inactivate();
			ToIconic();
		}
	}
	else
	{
		//  
		device->Activate();
		PlatformMoveCursor(0, 0);
		GScene* scene=GetScene();
		if (scene)
			scene->Invalidate();

		#ifndef _DEBUG
		if (platform)
			tick_mouse_capture=platform->GetTick();
		#endif
	}

	OnActivate(nState, bMinimized);
}

//  
void GGame::SysOnUpdate()
{
	if (net && (!net->IsServer()))
	{
		//   
		GNetClient* client=(GNetClient*)net;
		client->IncreaseTakt();
	}

	UpdateNet();

	if (net_downloader)
	{
		//       
		net_downloader->UpdateNet();
		net_downloader->OnUpdate();
	}

	GScene* scene=GetScene();
	if (scene)
		scene->OnUpdate();
	GComponent* modal=GetModalComponent();
	if (modal)
		modal->OnUpdateModal();

	UpdateNet();
}

//    
void GGame::UpdateNet()
{
	if (net)
	{
		//      
		bool is=net->UpdateNet();

		if (!net->IsServer())
		{
			GNetClient* client=(GNetClient*)net;
			GNetSocket* socket=client->GetMainSocket();
			if (socket)
			{
				NET_BUFFER* buffer=socket->GetReceivingBuffer();
				if (buffer->GetLength())
				{
					//     
					if (socket->ReceiveMessages() && is)
					{
						if (net)
							net->UpdateNet();
					}
				}
			}

			if (is)
			{
				if (net)
					net->OnUpdate();
			}
			else
			{
				//  
				NET_DISCONNECT disconnect_state=NET_DISCONNECT_LOST;
				if (!net->IsServer())
				{
					GNetClient* client=(GNetClient*)net;
					disconnect_state=client->GetDisconnectionState();
				}
				StopNet();
				OnLostConnection(disconnect_state);
			}
		}
	}
}

void GGame::SysOnLButtonDown(unsigned int nFlags)
{
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		captured=component;
		captured_place=place;
		over=NULL;
		pressed=component;
		pressed_place=place;

		if (focus!=component)
		{
			if (focus)
				focus->SysOnKillFocus();
			focus=component;
			focus->SysOnSetFocus();
		}

		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnLButtonDown(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnLButtonDown(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnLButtonDown(nFlags, x, y);
			break;
		}
	}
	else
	{
		if (focus)
			focus->SysOnKillFocus();
	}
}

void GGame::SysOnLButtonUp(unsigned int nFlags)
{
	int x, y, place;
	GComponent* vcap=captured;
	int vplace=captured_place;
	captured=pressed;
	captured_place=pressed_place;
	GComponent* vpressed=pressed;
	pressed=NULL;
	pressed_place=COMPONENT_PLACE_WIN;
	GComponent* component=DetectComponentByCursor(x, y, place);
	captured_place=vplace;
	captured=vcap;
	if (component)
	{
		captured=NULL;
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnLButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnLButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnLButtonUp(nFlags, x, y);
			break;
		}

		if (vpressed)
		{
			int place_new;
			GComponent* component_new=DetectComponentByCursor(x, y, place_new);
			if (component==component_new)
			{
				if (place==place_new)
				{
					switch (place)
					{
					case COMPONENT_PLACE_WIN:
						component->WinOnLButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_INNER:
						component->InnerOnLButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_CLIENT:
						component->ClientOnLButtonClick(nFlags, x, y);
						break;
					}
				}
			}
		}

		SysOnMouseMove(nFlags);
	}
}

void GGame::SysOnLButtonDblClk()
{
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		unsigned int control_keys=control & (CONTROL_ALT | CONTROL_CTRL | CONTROL_SHIFT);
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnLButtonDblClk(control_keys, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnLButtonDblClk(control_keys, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnLButtonDblClk(control_keys, x, y);
			break;
		}
	}
}

void GGame::SysOnRButtonDown(unsigned int nFlags)
{
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		captured=component;
		captured_place=place;
		over=NULL;
		pressed=component;
		pressed_place=place;

		if (focus!=component)
		{
			if (focus)
				focus->SysOnKillFocus();
			focus=component;
			focus->SysOnSetFocus();
		}

		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnRButtonDown(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnRButtonDown(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnRButtonDown(nFlags, x, y);
			break;
		}
	}
	else
	{
		if (focus)
			focus->SysOnKillFocus();
	}
}

void GGame::SysOnRButtonUp(unsigned int nFlags)
{
	int x, y, place;
	GComponent* vcap=captured;
	int vplace=captured_place;
	captured=pressed;
	captured_place=pressed_place;
	GComponent* vpressed=pressed;
	pressed=NULL;
	pressed_place=COMPONENT_PLACE_WIN;
	GComponent* component=DetectComponentByCursor(x, y, place);
	captured_place=vplace;
	captured=vcap;
	if (component)
	{
		captured=NULL;
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnRButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnRButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnRButtonUp(nFlags, x, y);
			break;
		}

		if (vpressed)
		{
			int place_new;
			GComponent* component_new=DetectComponentByCursor(x, y, place_new);
			if (component==component_new)
			{
				if (place==place_new)
				{
					switch (place)
					{
					case COMPONENT_PLACE_WIN:
						component->WinOnRButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_INNER:
						component->InnerOnRButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_CLIENT:
						component->ClientOnRButtonClick(nFlags, x, y);
						break;
					}
				}
			}
		}

		SysOnMouseMove(nFlags);
	}
}

void GGame::SysOnRButtonDblClk()
{
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		unsigned int control_keys=control & (CONTROL_ALT | CONTROL_CTRL | CONTROL_SHIFT);
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnRButtonDblClk(control_keys, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnRButtonDblClk(control_keys, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnRButtonDblClk(control_keys, x, y);
			break;
		}
	}
}

void GGame::SysOnMButtonDown(unsigned int nFlags)
{
	int x, y, place;
	GComponent* vcap=captured;
	int vplace=captured_place;
	captured=pressed;
	captured_place=pressed_place;
	pressed=NULL;
	pressed_place=COMPONENT_PLACE_WIN;
	GComponent* component=DetectComponentByCursor(x, y, place);
	captured_place=vplace;
	captured=vcap;
	if (component)
	{
		captured=component;
		captured_place=place;
		over=NULL;
		pressed=component;
		pressed_place=place;

		if (focus!=component)
		{
			if (focus)
				focus->SysOnKillFocus();
			focus=component;
			focus->SysOnSetFocus();
		}

		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnMButtonDown(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnMButtonDown(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnMButtonDown(nFlags, x, y);
			break;
		}
	}
	else
	{
		if (focus)
			focus->SysOnKillFocus();
	}
}

void GGame::SysOnMButtonUp(unsigned int nFlags)
{
	int x, y, place;
	GComponent* vcap=captured;
	int vplace=captured_place;
	captured=pressed;
	captured_place=pressed_place;
	GComponent* vpressed=pressed;
	pressed=NULL;
	pressed_place=COMPONENT_PLACE_WIN;
	GComponent* component=DetectComponentByCursor(x, y, place);
	captured_place=vplace;
	captured=vcap;
	if (component)
	{
		captured=NULL;
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnMButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnMButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnMButtonUp(nFlags, x, y);
			break;
		}

		if (vpressed)
		{
			int place_new;
			GComponent* component_new=DetectComponentByCursor(x, y, place_new);
			if (component==component_new)
			{
				if (place==place_new)
				{
					switch (place)
					{
					case COMPONENT_PLACE_WIN:
						component->WinOnMButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_INNER:
						component->InnerOnMButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_CLIENT:
						component->ClientOnMButtonClick(nFlags, x, y);
						break;
					}
				}
			}
		}

		SysOnMouseMove(nFlags);
	}


	/*
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		captured=NULL;
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnMButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnMButtonUp(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnMButtonUp(nFlags, x, y);
			break;
		}

		if (vpressed)
		{
			int place_new;
			GComponent* component_new=DetectComponentByCursor(x, y, place_new);
			if (component==component_new)
			{
				if (place==place_new)
				{
					switch (place)
					{
					case COMPONENT_PLACE_WIN:
						component->WinOnMButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_INNER:
						component->InnerOnMButtonClick(nFlags, x, y);
						break;
					case COMPONENT_PLACE_CLIENT:
						component->ClientOnMButtonClick(nFlags, x, y);
						break;
					}
				}
			}
		}

		SysOnMouseMove(nFlags);
	}
	*/
}

void GGame::SysOnMButtonDblClk()
{
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		unsigned int control_keys=control & (CONTROL_ALT | CONTROL_CTRL | CONTROL_SHIFT);
		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnMButtonDblClk(control_keys, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnMButtonDblClk(control_keys, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnMButtonDblClk(control_keys, x, y);
			break;
		}
	}
}

void GGame::SysOnMouseWheel(short zDelta)
{
	if (focus)
	{
		if (focus->WinOnMouseWheel(zDelta))
			if (focus->InnerOnMouseWheel(zDelta))
				focus->ClientOnMouseWheel(zDelta);
	}
}

void GGame::SysOnMouseMove(unsigned int nFlags)
{
	int x, y, place;
	GComponent* component=DetectComponentByCursor(x, y, place);
	if (component)
	{
		if ((!captured))
		{
			if (over!=component)
			{
				if (over)
				{
					over->GlobalOnMouseLeave(nFlags);
					switch (over_place)
					{
					case COMPONENT_PLACE_WIN:
						over->WinOnMouseLeave(nFlags);
						break;
					case COMPONENT_PLACE_INNER:
						over->InnerOnMouseLeave(nFlags);
						break;
					case COMPONENT_PLACE_CLIENT:
						over->ClientOnMouseLeave(nFlags);
						break;
					}
				}

				component->GlobalOnMouseEnter(nFlags, x, y);

				over=component;
				over_place=place;
				switch (place)
				{
				case COMPONENT_PLACE_WIN:
					component->WinOnMouseEnter(nFlags, x, y);
					break;
				case COMPONENT_PLACE_INNER:
					component->InnerOnMouseEnter(nFlags, x, y);
					break;
				case COMPONENT_PLACE_CLIENT:
					component->ClientOnMouseEnter(nFlags, x, y);
					break;
				}
			}
			else
			{
				if (over_place!=place)
				{
					if (over)
					{
						switch (over_place)
						{
							case COMPONENT_PLACE_WIN:
								over->WinOnMouseLeave(nFlags);
								break;
							case COMPONENT_PLACE_INNER:
								over->InnerOnMouseLeave(nFlags);
								break;
							case COMPONENT_PLACE_CLIENT:
								over->ClientOnMouseLeave(nFlags);
								break;
						}
					}

					over_place=place;

					switch (place)
					{
					case COMPONENT_PLACE_WIN:
						component->WinOnMouseEnter(nFlags, x, y);
						break;
					case COMPONENT_PLACE_INNER:
						component->InnerOnMouseEnter(nFlags, x, y);
						break;
					case COMPONENT_PLACE_CLIENT:
						component->ClientOnMouseEnter(nFlags, x, y);
						break;
					}
				}
			}
		}

		switch (place)
		{
		case COMPONENT_PLACE_WIN:
			component->WinOnMouseMove(nFlags, x, y);
			break;
		case COMPONENT_PLACE_INNER:
			component->InnerOnMouseMove(nFlags, x, y);
			break;
		case COMPONENT_PLACE_CLIENT:
			component->ClientOnMouseMove(nFlags, x, y);
			break;
		}
	}
}

void GGame::SysOnKeyDown(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	GComponent* component=GetFocusComponent();
	if (component)
	{
		if (platform->IsCopyKeyPressed(nChar))
			component->SysOnCopy();
		else
		{
			if (platform->IsCutKeyPressed(nChar))
				component->SysOnCut();
			else
				if (platform->IsPastKeyPressed(nChar))
					component->SysOnPast();
				else
				{
					key_down=NULL;
					bool is_client=true;
					GComponent* next=component;
					do
					{
						key_down=next;
						if ((!next->WinOnKeyDown(nChar, nRepCnt, nFlags)) || (!next->InnerOnKeyDown(nChar, nRepCnt, nFlags)))
						{
							is_client=false;
							break;
						}
						key_down=NULL;

						if (next->IsDialog())
							break;

						next=next->GetParent();
					}
					while (next);

					if (is_client)
					{
						key_down=component;
						component->ClientOnKeyDown(nChar, nRepCnt, nFlags);
					}
				}
		}
	}
}

void GGame::SysOnKeyUp(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	GComponent* component=key_down;
	if (component)
	{
		if (!(platform->IsCopyKeyPressed(nChar) || platform->IsCutKeyPressed(nChar) || platform->IsCutKeyPressed(nChar)))
		{
			if (component->WinOnKeyUp(nChar, nRepCnt, nFlags))
				if (component->InnerOnKeyUp(nChar, nRepCnt, nFlags))
					component->ClientOnKeyUp(nChar, nRepCnt, nFlags);

			if (key_down_count<=0)
			{
				key_down=NULL;
				key_down_count=0;
			}
		}
	}
}

void GGame::SysOnChar(unsigned int nChar, unsigned int nRepCnt, unsigned int nFlags)
{
	if (nChar>=0x20 && nChar<=0xff)
	{
		GComponent* component=GetFocusComponent();
		if (component)
		{
			if (component->WinOnChar(nChar, nRepCnt, control))
				if (component->InnerOnChar(nChar, nRepCnt, control))
					component->ClientOnChar(nChar, nRepCnt, control);
			SetRepeatedChar(nChar);
		}
	}
}

//        
int GGame::GetIndexOfConnectedPlayer()
{
	NET_SESSION_INFO si(net);
	if (GetSessionInfoStruct(&si))
	{
		for (int i=0;i<si.k_player;i++)
		{
			if (si.m_player[i]->type==PLAYER_OPENED)
				return i;
		}
	}
	return -1;
}

//       
bool GGame::IsNetVersionsCompatible(DIGITAL_VERSION* ver1, DIGITAL_VERSION* ver2)
{
	return (ver1->ver1==ver2->ver1 && ver1->ver2==ver2->ver2);
}

//        
//     ,      
void GGame::OnStatisticInfo(int max_license_type, NET_STATISTICS* statistics)
{
	if (statistics)
	{
		delete []statistics;
		statistics=NULL;
	}
}

//  
//  : 0 -  , 1 -   
int GGame::VerifyLicense(int license_type, const char* license)
{
	if (license_type==0)
		return 1;
	return 0;
}

//   
void GGame::SetUserName(const char* user_name)
{
	CMagicString v=user_name;
	if (user!=v)
	{
		user=v;
		profile_path="";
		saved_game_path="";
	}
}

// eng: Returns path to folder with users
// rus:      
const char* GGame::GetPathToUsers()
{
	if (users_path.IsEmpty())
	{
		users_path=platform->GetPathToTemp();
		char slash=users_path[users_path.GetLength()-1];
		users_path+="Users";
		users_path+=slash;
	}
	return users_path.c_str();
}

// eng: Returns path to folder with user profile
// rus:       
const char* GGame::GetPathToProfile()
{
	if (user.IsEmpty())
		return NULL;

	if (profile_path.IsEmpty())
	{
		profile_path=GetPathToUsers();
		char slash=profile_path[profile_path.GetLength()-1];
		profile_path+=user;
		profile_path+=slash;
	}

	return profile_path.c_str();
}

// eng: Returns path to folder with saved games
// rus:       
const char* GGame::GetPathToSavedGame()
{
	if (saved_game_path.IsEmpty())
	{
		saved_game_path=GetPathToProfile();
		char slash=saved_game_path[saved_game_path.GetLength()-1];
		saved_game_path+="save";
		platform->CreateFolder(saved_game_path);
		saved_game_path+=slash;
	}
	return saved_game_path.c_str();
}

// eng: Returns path to folder with music
// rus:      
const char* GGame::GetPathToMusic()
{
	if (music_path.IsEmpty())
	{
		music_path=platform->GetPathToGame();
		char slash=music_path[music_path.GetLength()-1];
		music_path+="Music";
		music_path+=slash;
	}
	return music_path.c_str();
}

// eng: Returns path to folder with sounds
// rus:      
const char* GGame::GetPathToSound()
{
	if (sound_path.IsEmpty())
	{
		sound_path=platform->GetPathToGame();
		char slash=sound_path[sound_path.GetLength()-1];
		sound_path+="Sound";
		sound_path+=slash;
	}
	return sound_path.c_str();
}

// eng: Returns path to folder with voices
// rus:      
const char* GGame::GetPathToVoice()
{
	if (voice_path.IsEmpty())
	{
		voice_path=platform->GetPathToGame();
		char slash=voice_path[voice_path.GetLength()-1];
		voice_path+="Voice";
		voice_path+=slash;
	}
	return voice_path.c_str();
}

// eng: Returns path to folder with maps
// rus:      
const char* GGame::GetPathToMaps()
{
	if (maps_path.IsEmpty())
	{
		maps_path=platform->GetPathToGame();
		char slash=maps_path[maps_path.GetLength()-1];
		maps_path+="Maps";
		maps_path+=slash;
	}
	return maps_path.c_str();
}

// eng: Returns path to folder with downloaded maps
// rus:       
const char* GGame::GetPathToDownloadedMaps()
{
	if (downloaded_maps_path.IsEmpty())
	{
		downloaded_maps_path=GetPathToConfig();
		char slash=downloaded_maps_path[downloaded_maps_path.GetLength()-1];
		downloaded_maps_path+="DownloadedMaps";
		platform->CreateFolder(downloaded_maps_path);
		downloaded_maps_path+=slash;
	}
	return downloaded_maps_path.c_str();
}

// eng: Returns path to folder with configuration
// rus:      
const char* GGame::GetPathToConfig()
{
	if (config_path.IsEmpty())
	{
		config_path=platform->GetPathToTemp();
	}
	return config_path.c_str();
}

// eng: Returns path to folder with initialization data
// rus:       
const char* GGame::GetPathToInit()
{
	if (init_path.IsEmpty())
	{
		init_path=platform->GetPathToGame();
		char slash=init_path[init_path.GetLength()-1];
		init_path+="Init";
		init_path+=slash;
	}
	return init_path.c_str();
}

// eng: Returns path to folder with objects
// rus:      
const char* GGame::GetPathToObject()
{
	if (object_path.IsEmpty())
	{
		object_path=platform->GetPathToGame();
		object_path+="Object";
		object_path+=platform->GetSlash();
	}
	return object_path;
}

// eng: Returns path to folder with images
// rus:      
const char* GGame::GetPathToImage()
{
	if (image_path.IsEmpty())
	{
		image_path=platform->GetPathToGame();
		image_path+="Image";
		image_path+=platform->GetSlash();
	}
	return image_path;
}

// eng: Returns path to folder with fonts
// rus:      
const char* GGame::GetPathToFont()
{
	if (font_path.IsEmpty())
	{
		font_path=platform->GetPathToGame();
		font_path+="Font";
		font_path+=platform->GetSlash();
	}
	return font_path;
}


// eng: Returns path to folder with autosave
// rus:      autosave
const char* GGame::GetPathToAutosave()
{
	if (autosave_path.IsEmpty())
	{
		autosave_path=platform->GetPathToTemp();
		char slash=autosave_path[autosave_path.GetLength()-1];
		autosave_path+="autosave";
		platform->CreatePath(autosave_path);
		autosave_path+=slash;
	}
	return autosave_path.c_str();
}

